// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.IO;

namespace Px2Pg
{
	public class PxSqlFixer
	{
		public static int numeroTabla = 1;
		
		public static void Convertir(string archivoSql, string archivoSalida)
		{
			// Verifico que el archivo a convertir exista
			if (!File.Exists(archivoSql)) {
				throw new Exception("El archivo sql especificado no existe (" +
				                    archivoSql +
				                    ")");
			}
			
			if (archivoSql.Equals(archivoSalida))
				throw new Exception("Los archivos de entrada y salida no pueden coincidir.");
			
			//FileInfo fiOriginal = new FileInfo(archivoSql);
			FileInfo fiNuevo = new FileInfo(archivoSalida);
			
			StreamReader lector =
				new StreamReader(archivoSql,
				                 Encoding.GetEncoding("ISO-8859-1"));
			
			StreamWriter escritor = fiNuevo.AppendText();
			
			// Esto iría afuera en un while por cada archivo
			//int numeroTabla = 1;
			
			string lineaOriginal = "";
			string nuevaLinea = "";
			string nuevoNombreTabla = "";
			
			string[] segmentos = new string[] {};
			
			while(!lector.EndOfStream) {
				lineaOriginal = lector.ReadLine();
				
				/* La linea está formada por [INSTRUCCION] "[TABLA]" ([VALORES]);
				 * Si usamos Split con " como delimitador nos deja en la posición
				 * 1 del array que devuelve el nombre de la tabla. Por lo que
				 * podemos armar la nueva linea concatenando la posición 0 y 2 del
				 * array y el nuevo nombre de la tabla*/
				
				segmentos = lineaOriginal.Split(new char[] {'\"'});
				
				/* Si la linea contiene "CREATE TABLE", buscamos el original de
				 * nombre de la tabla en ella */
				if (lineaOriginal.Contains("CREATE TABLE")) {
					
					// Al nombre de la tabla le agrego "-stratos" y un numero.
					nuevoNombreTabla = segmentos[1] + "-stratos" + numeroTabla;
					
					numeroTabla++;
					
					// Armamos la nueva linea y la escribimos en el archivo
					nuevaLinea = segmentos[0] + "\"" + nuevoNombreTabla + "\"" +
						segmentos[2];
				}
				else if (lineaOriginal.Contains("INSERT")) {
					nuevaLinea = lineaOriginal.Replace("\"" + segmentos[1] + "\"",
					                                   "\"" + nuevoNombreTabla + "\"");
					
					nuevaLinea = nuevaLinea.Replace(@"\\", @"/");
				}
				else if (lineaOriginal.EndsWith("BLOB,"))
					nuevaLinea = lineaOriginal.Replace("BLOB,","VARCHAR,");
				else if (lineaOriginal.EndsWith(" BLOB"))
					nuevaLinea = lineaOriginal.Replace(" BLOB"," VARCHAR");
				else
					nuevaLinea = lineaOriginal;
				
				// Escribimos la nueva linea en el nuevo archivo
				escritor.WriteLine(nuevaLinea);
				escritor.Flush();
			}
			
			lector.Close();
			escritor.Close();
		}
	}
}
