// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using Stratos.Common.Excepciones;

namespace ConsultasWeb
{
	public partial class Default : System.Web.UI.Page
	{
		protected void btnBuscar_Click (object sender, System.EventArgs e)
		{
			ControladorProductosPrx controlador = (ControladorProductosPrx) Application["controladorProductos"];
			ProductoDto[] productos = controlador.BuscarPorDescripcion(this.txtBusqueda.Text);
			
			this.rptListaProductos.DataSource = productos;
			this.rptListaProductos.DataBind();
		}
		
		protected void MostrarMensaje(string mensaje)
		{
			this.lblMensaje.Text = mensaje;
		}
		
		protected void MostrarMensaje(Exception ex)
		{
			this.MostrarMensaje(ex.Message + "\n" + ex.StackTrace);
		}
		
		protected override void OnError (EventArgs e)
		{
			// At this point we have information about the error
			HttpContext ctx = HttpContext.Current;
			
			Exception exception = ctx.Server.GetLastError ();
			
			string mensaje = exception.Message;
			
			if (exception is StratosException)
				mensaje = (exception as StratosException).mensaje;
			
			string errorInfo = 
				"<h2>Error<h2>" + mensaje;
			
			ctx.Response.Write (errorInfo);
			
			Console.WriteLine(exception.ToString());
			
			// --------------------------------------------------
			// To let the page finish running we clear the error
			// --------------------------------------------------
			ctx.Server.ClearError ();
			
			base.OnError (e);
		}
	}
}

