// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate;
using NHibernate.Criterion;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace Migrador
{
	public class MapeoCreditos : IMapeoTabla
	{
		static MapeoCreditos instancia;
		
		public MapeoCreditos ()
		{
		}
		
		#region IMapeoTabla implementation
		public void RealizarLimpieza (System.Data.IDbConnection con)
		{
		}
		
		
		public string MensajeAntesDeComenzar {
			get {
				return "";
			}
		}
		
		
		public string MensajeDespuesDeTerminar {
			get {
				return "";
			}
		}
		
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Créditos";
			}
		}
		
		
		public string Tabla {
			get {
				return "viejo.\"Pencre.db-stratos5\"";
			}
		}
		
		public string ConsultaSql {
			get {
				return
					"select * from " + this.Tabla;
			}
		}
		
		public void Transformar (System.Data.IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			while (reader.Read()) {
				Console.WriteLine("Procesando Crédito...");
				using (SessionScope sessionScope = new SessionScope()) {
					this.ProcesarFila(reader);
				}
				Console.WriteLine("Crédito procesado...");
				Console.WriteLine();
			}
		}
		
		private void ProcesarFila(System.Data.IDataReader reader)
		{
			DocumentoCredito documentoCredito = this.LeerRecibo(reader);
			
			if (documentoCredito == null) {
				Console.WriteLine("   DocumentoCredito devuelto es nulo");
				return;
			}
			
			try {
				documentoCredito.Create();
				Console.WriteLine("DocumentoCredito pasado correctamente");
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine("Error al procesar: " + this.ObtenerMensajesExcepcion(ex));
			}
		}
		
		public string ObtenerMensajesExcepcion(ActiveRecordValidationException excepcion)
		{
			return ObtenerMensajesValidacion(excepcion.ValidationErrorMessages);
		}
		
		private string ObtenerMensajesValidacion(string[] mensajesError)
		{
			StringBuilder mensaje = new StringBuilder();
			
			foreach (string errStr in mensajesError) {
				mensaje.Append(" - ");
				mensaje.Append(errStr);
				mensaje.Append("\n");
			}
			
			return mensaje.ToString();
		}
		#endregion
		
		public static MapeoCreditos Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoCreditos();
				
				return instancia;
			}
		}
		
		public DocumentoCredito LeerRecibo(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			DocumentoCredito docCredito = new DocumentoCredito();
			
			long codigoCliente = long.Parse(dg.GetString("Cod_cli").Trim());
			Cliente cliente = Cliente.FindOne(Restrictions.Eq("Codigo", codigoCliente));
			
			if (cliente != null) {
				Console.WriteLine("  Cliente: {0} - {1}", cliente.Codigo, cliente.RazonSocial);
			}
			else
				Console.WriteLine("  Cliente nulo !!");
			
			docCredito.Cliente = cliente;
			
			string tipDoc = dg.GetString("Tip_doc").Trim().ToUpper();
			
			// Si es una nota de crédito
			if (tipDoc.StartsWith("NC")) {
				Console.WriteLine("   Es NotaCredito");
				
				docCredito.TipoDocumento = TipoDocumento.NotaCredito;
				
				string tipoFac = dg.GetString("Serie").Trim().ToUpper();
				FacturaTipoVenta tipoFactura;
				if (tipoFac == "A")
					tipoFactura = FacturaTipoVenta.FacturaA;
				else
					tipoFactura = FacturaTipoVenta.FacturaB;
				
				string codigoFac = dg.GetString("Cod_suc");
				int codigoFactura = Int32.Parse(codigoFac);
				
				long numeroFactura = Convert.ToInt64(dg.GetDouble("Num_doc").Value);
				
				Console.WriteLine("   Nota crédito: {0} - {1}", codigoFactura, numeroFactura);
				
				// Busco la factura
				NotaCredito notaCredito =
					ActiveRecordBase<NotaCredito>.FindOne(Restrictions.Eq("Tipo", tipoFactura),
					                Restrictions.Eq("Codigo", codigoFactura),
					                Restrictions.Eq("Numero", numeroFactura));
				
				if (notaCredito == null) {
					Console.WriteLine("   No se encontró la nota de crédito {0} - {1} - {2}",
					                  tipoFactura, codigoFactura, numeroFactura);
					
					if (codigoFactura == 4) {
						Console.WriteLine("   Buscando con código 40...");
						notaCredito =
							ActiveRecordBase<NotaCredito>.FindOne(Restrictions.Eq("Tipo", tipoFactura),
							                Restrictions.Eq("Codigo", 40),
							                Restrictions.Eq("Numero", numeroFactura));
					}
					
					if (notaCredito == null) {
						Console.WriteLine("     Tampoco se encontró con código 40");
						return null;
					}
				}
				
				docCredito.CodigoNotaCredito = notaCredito.Codigo;
				docCredito.NumeroNotaCredito = notaCredito.Numero;
				docCredito.IdDocumento = notaCredito.Id;
				
				docCredito.ImportePendiente = dg.GetDecimal("Imp_pen").Value;
				docCredito.ImporteTotal = dg.GetDecimal("Imp_cuo").Value;
			}
			
			// Si es un recibo
			else if (tipDoc == "RE") {
				Console.WriteLine("   Es Recibo");
				
				docCredito.TipoDocumento = TipoDocumento.Recibo;
				
				long numeroRecibo = Convert.ToInt64(dg.GetDouble("Num_doc").Value);
				
				Console.WriteLine("   Número recibo {0}", numeroRecibo);
				
				// Busco la factura
				Recibo recibo =
					Recibo.FindOne(Restrictions.Eq("Numero", numeroRecibo));
				
				if (recibo == null) {
					Console.WriteLine("   No se encontró el recibo");
					
					return null;
				}
				
				docCredito.IdDocumento = recibo.Id;
				
				docCredito.ImportePendiente = dg.GetDecimal("Imp_pen").Value;
				docCredito.ImporteTotal = dg.GetDecimal("Imp_cuo").Value;
			}
			
			else {
				Console.WriteLine("Tipo de documento desconocido: {0}", tipDoc);
				
				docCredito = null;
			}
			
			return docCredito;
		}
	}
}
