// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate;
using NHibernate.Criterion;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace Migrador
{
	public class MapeoDebitos : IMapeoTabla
	{
		static MapeoDebitos instancia;
		
		public MapeoDebitos ()
		{
		}
		
		#region IMapeoTabla implementation
		public void RealizarLimpieza (System.Data.IDbConnection con)
		{
		}
		
		
		public string MensajeAntesDeComenzar {
			get {
				return "";
			}
		}
		
		
		public string MensajeDespuesDeTerminar {
			get {
				return "";
			}
		}
		
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Débitos";
			}
		}
		
		
		public string Tabla {
			get {
				return "viejo.\"Pendeb.db-stratos6\"";
			}
		}
		
		public string ConsultaSql {
			get {
				return
					"select * from " + this.Tabla;
			}
		}
		
		public void Transformar (System.Data.IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			while (reader.Read()) {
				Console.WriteLine("Procesando Débito...");
				using (SessionScope sessionScope = new SessionScope()) {
					this.ProcesarFila(reader);
				}
				
				Console.WriteLine("Débito procesado...");
				Console.WriteLine();
			}
		}
		
		private void ProcesarFila(System.Data.IDataReader reader)
		{
			DocumentoDebito documentoDebito = this.LeerRecibo(reader);
			
			if (documentoDebito == null) {
				Console.WriteLine("    Documento débito nulo");
				return;
			}
			
			try {
				documentoDebito.Create();
				Console.WriteLine("DocumentoDebito pasado correctamente");
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine("Error al procesar: " + this.ObtenerMensajesExcepcion(ex));
			}
		}
		
		public string ObtenerMensajesExcepcion(ActiveRecordValidationException excepcion)
		{
			return ObtenerMensajesValidacion(excepcion.ValidationErrorMessages);
		}
		
		private string ObtenerMensajesValidacion(string[] mensajesError)
		{
			StringBuilder mensaje = new StringBuilder();
			
			foreach (string errStr in mensajesError) {
				mensaje.Append(" - ");
				mensaje.Append(errStr);
				mensaje.Append("\n");
			}
			
			return mensaje.ToString();
		}
		#endregion
		
		public static MapeoDebitos Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoDebitos();
				
				return instancia;
			}
		}
		
		public DocumentoDebito LeerRecibo(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			DocumentoDebito docDebito = new DocumentoDebito();
			
			long codigoCliente = long.Parse(dg.GetString("Cod_cli").Trim());
			Cliente cliente = Cliente.FindOne(Restrictions.Eq("Codigo", codigoCliente));
			
			docDebito.Cliente = cliente;
			
			string tipDoc = dg.GetString("Tip_doc").Trim().ToUpper();
			
			// Si es una nota de crédito
			if (tipDoc == "FA" || tipDoc == "FA1") {
				Console.WriteLine("   Es Factura");
				
				docDebito.TipoDocumento = TipoDocumento.Factura;
				
				string tipoFac = dg.GetString("Serie").Trim().ToUpper();
				FacturaTipoVenta tipoFactura;
				if (tipoFac == "A")
					tipoFactura = FacturaTipoVenta.FacturaA;
				else
					tipoFactura = FacturaTipoVenta.FacturaB;
				
				string codigoFac = dg.GetString("Cod_suc");
				int codigoFactura = Int32.Parse(codigoFac);
				
				long numeroFactura = Convert.ToInt64(dg.GetDouble("Num_doc").Value);
				
				Console.WriteLine("   Factura: {0} - {1} - {2}", tipoFactura, codigoFactura, numeroFactura);
				
				// Busco la factura
				Factura factura =
					Factura.FindOne(Restrictions.Eq("Tipo", tipoFactura),
					                Restrictions.Eq("Codigo", codigoFactura),
					                Restrictions.Eq("Numero", numeroFactura));
				
				Factura factura40 =
					Factura.FindOne(Restrictions.Eq("Tipo", tipoFactura),
					                Restrictions.Eq("Codigo", 40),
					                Restrictions.Eq("Numero", numeroFactura));
				
				if (factura == null) {
					Console.WriteLine("   Buscando con código 40");
					
					factura = factura40;
					
					if (factura == null) {
						Console.WriteLine("    factura nula, saliendo!");
						return null;
					}
				}
				
				DocumentoDebito docDeb =
					DocumentoDebito.FindOne(Restrictions.Eq("TipoDocumento", TipoDocumento.Factura),
					                        Restrictions.Eq("IdDocumento", factura.Id));
				
				DocumentoDebito docDeb40 =
					DocumentoDebito.FindOne(Restrictions.Eq("TipoDocumento", TipoDocumento.Factura),
					                        Restrictions.Eq("IdDocumento", factura40 != null ? factura40.Id : -1));
				
				if (docDeb != null) {
					if (factura40 != null) {
						Console.WriteLine("    Cambiando a factura con código 40");
						factura = factura40;
					}
					
					Console.WriteLine("   Ya existe un documento débito con esa factura");
					return null;
				}
				
				docDebito.CodigoFactura = factura.Codigo;
				docDebito.NumeroFactura = factura.Numero;
				
				docDebito.IdDocumento = factura.Id;
				docDebito.ImportePendiente = dg.GetDecimal("Imp_pen").Value;
				docDebito.ImporteTotal = dg.GetDecimal("Imp_cuo").Value;
				docDebito.FechaVencimiento = dg.GetDateTime("Fec_ven").Value;
			}
			
			return docDebito;
		}
	}
}
