// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate.Criterion;

using Stratos.Common;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using StratosServer.Clientes;
using StratosServer.Productos;

namespace Migrador
{
	public class MapeoFacturas : IMapeoTabla
	{
		private static MapeoFacturas instancia;
		
		private Rubro rubroComercio;
		private Rubro rubroIndustria;
		
		private int codigoProductoNumeracionAutomatica;
		private ControladorClientesI controladorClientes;
		
		private MapeoFacturas()
		{
			this.controladorClientes = new ControladorClientesI();
			
//			Console.WriteLine("Creando Rubro automático");
			this.rubroComercio = Rubro.FindOne(Restrictions.Eq("Nombre", "COMERCIO"));
			this.rubroIndustria = Rubro.FindOne(Restrictions.Eq("Nombre", "INDUSTRIA"));
			
			this.codigoProductoNumeracionAutomatica = 100;
		}
		
		public string MensajeAntesDeComenzar {
			get {
				return "";//"- La clase cliente tiene que tener en su clave primaria PrimaryKeyType.Native";
			}
		}
		
		public string MensajeDespuesDeTerminar {
			get {
				return String.Empty;
			}
		}
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Facturas";
			}
		}
		
		public string TablaItems {
			get { return "viejo.\"detFac.DB-stratos25\""; }
		}
		
		public string Tabla {
			get {
				// La tabla de los items parece ser Hisdet.DB-stratos28
				return "viejo.\"resttemp.DB-stratos20\"";
			}
		}
		
		public string ConsultaSql {
			get {
				// FIXME: Se levantan unicamente las facturas con un cliente con código
				// GUARDA: Debería haber una etiqueta GUARDA para avisar cosas de las que
				// hay que tener cuidado :)
				// GUARDA: No se qué diferencia hay entre FA y FA1 -> son tandas distintas, de distintas fechas
				return "select * from " + this.Tabla +
					//" where trim(both \"Cod_Cli\") <> ''" +
					" where " +
					"   \"Num_Doc\" is not null " +
//					"	and \"Fec_Doc\" <= '15-11-2008' " +
					"	and \"Imp_Bru\" > 0.00 " +
//					"   and (\"Num_Doc\" = 7898) " +
//					"	and (\"Tip_Doc\" = 'NC' or \"Tip_Doc\" = 'NC1')" +
					" order by \"Fec_Doc\" asc, \"Cod_Suc\"::numeric asc, \"Num_Doc\" asc";
			}
		}
		
		public void Transformar(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			Factura.ModoMigracionFacturas();
			NotaCredito.ModoMigracionFacturas();
			
			while (reader.Read()) {
				using (SessionScope sessionScope = new SessionScope()) {
					this.ProcesarFila(reader, dg, sessionScope);
				}
			}
			
			Factura.ModoNormal();
			NotaCredito.ModoNormal();
		}
		
		private void ProcesarFila(IDataReader reader, DataGetter dg, SessionScope sessionScope)
		{
			Factura factura;
			
//			Factura.ModoNormal();
			
			Console.Write("  Creando objeto ");
			
			// Veo si se trata de una Factura o una Nota de crédito
			string tipoDocumento = dg.GetString("Tip_Doc");
			if (tipoDocumento == "FA" || tipoDocumento == "FA1") {
				factura = new Factura();
				Console.WriteLine(" Factura");
			}
			else if (tipoDocumento == "NC" || tipoDocumento == "NC1") {
				factura = new NotaCredito();
				Console.WriteLine(" NotaCredito");
			}
			else {
				Console.WriteLine("Documento no soportado: " + tipoDocumento);
				Console.WriteLine();
				Console.WriteLine();
				return;
			}
			
			Console.WriteLine("  Tipo de documento: {0}", factura.GetType().FullName);
			
			factura.Valida = true;
			
			// Tipo factura
			string tipoFactura = dg.GetString("Serie");
			Console.WriteLine("  Tipo de Factura: {0}", tipoFactura);
			
			if (tipoFactura == "A")
				factura.Tipo = FacturaTipoVenta.FacturaA;
			else if (tipoFactura == "B")
				factura.Tipo = FacturaTipoVenta.FacturaB;
			else
				Console.WriteLine("WARNING: Factura con tipo desconocido: '{0}'", tipoFactura);
			
			// Fecha
			Console.WriteLine("  Fecha de Factura");
			try {
				factura.Fecha = dg.GetDateTime("Fec_Doc").Value;
				Console.WriteLine("    " + factura.Fecha.ToShortDateString());
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: la fecha de la factura es nula: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Código, Número, Anulada y Remitos
			int codigoFacturaVerdadero = Convert.ToInt32(dg.GetString("Cod_suc"));
			factura.Codigo = codigoFacturaVerdadero;
			
			// El tipo "FA1" parece ser un conjunto de facturas viejas. Cambio el código porque se repite
			if (tipoDocumento == "FA1" || tipoDocumento == "NC1")
				factura.Codigo = codigoFacturaVerdadero * 10;
			
			// Si el código de factura no es el 4, entonces es una factura manual
			if (factura.Codigo != 4)
				factura.FacturaManual = true;
			
			Console.WriteLine("  Codigo de Factura: {0}", factura.Codigo);
			
			factura.Numero = Convert.ToInt64(dg.GetDouble("Num_Doc"));
			Console.WriteLine("  Número de Factura: {0}", factura.Numero);
			
			factura.Anulada = dg.GetString("Estado") == "A";
			factura.Remitos = dg.GetString("Num_Rem");
			
			// Cliente
			Console.WriteLine("  Cliente de Factura");
//			string codigoCliente = dg.GetString("Cod_Cli");
			Cliente clienteLeido = MapeoClientes.LeerClienteDesdeDataReader(reader);
			Cliente cliente = null;
			
			// Busco por código
			if (clienteLeido.Codigo > 0 && Cliente.Exists(Restrictions.Eq("Codigo",clienteLeido.Codigo))) {
				Console.WriteLine("    El cliente tiene código {0} y existe en la base de datos", clienteLeido.Codigo);
				
				try {
					cliente = Cliente.Find(clienteLeido.Codigo);
				}
				catch (System.Exception ex) {
					Console.WriteLine("     ERROR al buscar cliente por código: " + ex.ToString());
					
//					transactionScope.VoteRollBack();
					return;
				}
			}
			// Busco por cuit
			else if (clienteLeido.Cuit.CuitValido && Cliente.Exists(Restrictions.Eq("Cuit.ValorNumerico", clienteLeido.Cuit.ValorNumerico))) {
				Console.WriteLine("    El cliente tiene un cuit válido {0} y existe en la base de datos.", clienteLeido.Cuit.ValorFormateado);
				
				try {
					cliente = Cliente.FindOne(Restrictions.Eq("Cuit.ValorNumerico", clienteLeido.Cuit.ValorNumerico));
				}
				catch (System.Exception ex) {
					Console.WriteLine("     ERROR al buscar cliente con cuit: " + ex.ToString());
					
//					transactionScope.VoteRollBack();
					return;
				}
			}
			else {
				Console.WriteLine("    Sin código y cuit válido.");
				
				// Busco por razón social (búsqueda exacta)
				if (clienteLeido.RazonSocial != null && clienteLeido.RazonSocial != String.Empty) {
					// Busco por razón social
					Console.WriteLine("      Busco por razón social.");
					
					ClienteDto[] clientesRazonSocial = null;
					
					// Si es un consumidor final, entonces realizo una consulta (menos exacto, pero con más
					// probabilidades de encontrar algo). Sino realizo una búsqueda exacta por razón social.
					if (clienteLeido.TipoResponsable == TipoResponsable.ConsumidorFinal)
						clientesRazonSocial = this.controladorClientes.ConsultaPorRazonSocial(clienteLeido.RazonSocial);
					else
						clientesRazonSocial = this.controladorClientes.BusquedaExactaPorRazonSocial(clienteLeido.RazonSocial);
					
					if (clientesRazonSocial.Length == 0)
						Console.WriteLine("         Tampoco se encontró por razón social");
					else {
						if (clientesRazonSocial.Length == 1)
							Console.WriteLine("         Se encontró exactamente un cliente por razón social");
						else
							Console.WriteLine("         Se encontró más de un cliente por razón social. Tomo el primero.");
						
						cliente = Cliente.Find(clientesRazonSocial[0].codigo);
						
						this.AgregarObservaciones(factura, "Cliente buscado por razón social");
					}
				}
				else {
					Console.WriteLine("    Se carga el cliente leído");
					cliente = clienteLeido;
				}
			}
			
			factura.Cliente = cliente;
			
			// Condición de venta
			Console.WriteLine("  Condición de venta de Factura");
			string condicionVenta = dg.GetString("Nom_Cuo").Trim().ToUpper();
			
			if (condicionVenta == String.Empty ||
			    	condicionVenta.Contains("TADO"))
				factura.CondicionVenta = new CondicionVentaFacturaGenerica(CondicionPago.Contado, 0);
			else
				factura.CondicionVenta = new CondicionVentaFacturaGenerica(CondicionPago.CuentaCorriente, 0);
			
			// Dejo una nota sobre la condición de venta en las observaciones, porque a veces ponen
			// en este campo la cantidad de cuotas
			if (condicionVenta.StartsWith("Ent"))
				this.AgregarObservaciones(factura, "CONDICION VENTA: " + condicionVenta);
			
			// Porcentaje de bonificación
			Console.WriteLine("  Porcentaje bonificación de Factura");
			factura.PorcentajeBonificacion = Convert.ToDecimal(dg.GetDouble("Por_Bon"));
			
			// Items
			Console.WriteLine("  Items de Factura");
			try {
				long orden = Convert.ToInt64(dg.GetDouble("Orden"));
				factura.Items = this.BuscarItemsFactura(factura, orden);
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: algún valor numérico de los items es nulo: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			catch (ActiveRecordValidationException) {
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Importes
			Console.WriteLine("  Importes de Factura");
			try {
				factura.ImporteBruto = dg.GetDecimal("Imp_Bru").Value;
				factura.ImporteBonificacion = dg.GetDecimal("Imp_Bon").Value;
				factura.ImporteNeto = dg.GetDecimal("Imp_Net").Value;
				
				if (factura.Tipo == FacturaTipoVenta.FacturaA)
					factura.ImporteIva = dg.GetDecimal("Iva_Ins").Value;
				else
					factura.ImporteIva = 0.00m;
				
				factura.ImporteTotal = dg.GetDecimal("Tot_Gen").Value;
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: algún importe no tiene valor, es nulo: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Observaciones
			Console.WriteLine("  Observaciones de Factura");
			string observacionesFactura = this.LimpiarObservaciones(dg.GetString("Obser"));
			
			if (observacionesFactura.Trim() != String.Empty)
				this.AgregarObservaciones(factura, "OBSERVACIONES: " + observacionesFactura);
			
			#region Validaciones
			// Comienzo la validación y guardado
			Console.WriteLine("###");
			Console.WriteLine("  Validando factura");
			
			// Si hay una falla por la fecha, entonces es muy probable que el código esté 
			// desincronizado. Intento arreglarlo ingresando una factura inválida que lo arregle
			if (!factura.IsValid() &&
			    	(this.PropiedadFallida("Fecha", factura) || this.PropiedadFallida("Numero", factura))) {
				
				Console.WriteLine("    Sincronizando número");
				try {
					using (SessionScope sessionScope2 = new SessionScope(FlushAction.Auto)) {
						this.IngresarFacturaInvalida(factura);
					};
				}
				catch(Exception ex) {
					Console.WriteLine("    ERROR al sincronizar: " + ObtenerMensajesExcepcion(ex));
					
//					transactionScope.VoteRollBack();
					return;
				}
			}
			
			// Si la factura es inválida en algún importe, los recalculo
			if (!factura.IsValid()) {
				var propiedadesImporteInvalidas =
					from prop in factura.PropertiesValidationErrorMessages.Keys.Cast<PropertyInfo>()
					where prop.Name.StartsWith("Importe")
					select 1;
				
				if (propiedadesImporteInvalidas.Count() > 0)
					this.CalcularImportes(factura);
			}
			
			// Verifico si la factura es inválida
			if (!factura.IsValid()) {
				Console.WriteLine("    Factura INVALIDA");
				
				// Itero por cada propiedad inválida y agrego una nota sobre la misma
				foreach (PropertyInfo pi in factura.PropertiesValidationErrorMessages.Keys) {
					Console.WriteLine("    Propiedad inválida: '" + pi.Name + "' ({0})", pi.GetValue(factura,null));
					Console.WriteLine("      Mensaje: " + ((ArrayList)factura.PropertiesValidationErrorMessages[pi])[0]);
				}
				
				//throw new Exception("Factura inválida");
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Si llego a este punto, entonces la factura muy probablemente sea válida.
			// Entonces doy de alta el cliente asociado si se trata de una factura A y
			// éste todavía no se guardó.
//			if (factura.Tipo == FacturaTipoVenta.FacturaA &&
//			    	factura.Cliente.Codigo == 0) {
			if (factura.Cliente.Codigo == 0) {
				Console.WriteLine("  Creando cliente asociado a factura");
				
//				using (TransactionScope ts = new TransactionScope(TransactionMode.Inherits, OnDispose.Commit)) {
					try {
						Cliente.ModoMigracion();
						
						if (cliente.TipoResponsable == TipoResponsable.ConsumidorFinal)
							cliente.Cuit = null;
					
						cliente.AltaAutomatica = true;
						cliente.Create();
					
						factura.Cliente = cliente;
					
						Cliente.ModoNormal();
					
//						factura.Cliente.AltaAutomatica = true;
//						factura.Cliente.Create();
					}
					catch (ActiveRecordValidationException ex) {
						Console.WriteLine("      ERROR al guardar el cliente, cliente inválido.");
						Console.WriteLine("        Propiedades inválidas:");
						
						// Itero por cada propiedad inválida y agrego una nota sobre la misma
						foreach (PropertyInfo pi in factura.Cliente.PropertiesValidationErrorMessages.Keys) {
							Console.WriteLine("          Propiedad inválida: '" + pi.Name + "'");
							Console.WriteLine("          Mensaje: " + ((ArrayList)factura.Cliente.PropertiesValidationErrorMessages[pi])[0]);
						}
					}
					catch (System.Exception ex) {
						Console.WriteLine("      ERROR al guardar el cliente: " + ex.ToString());
						
//						ts.VoteRollBack();
//						transactionScope.VoteRollBack();
						return;
					}
//				}
			}
			#endregion
			
//				try {
			
			Console.WriteLine("  Guardando Factura");
			
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine("    ERROR Validación en Factura: " + ObtenerMensajesExcepcion(ex));
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			catch (Exception ex) {
				Console.WriteLine("    ERROR desconocido en Factura: " + ObtenerMensajesExcepcion(ex));
				Console.WriteLine();
				Console.WriteLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			Console.WriteLine();
			Console.WriteLine("  ¡Factura convertido correctamente!");
			
			Console.WriteLine();
			Console.WriteLine();
		}
		
		private void IngresarFacturaInvalida(Factura facturaPorIngresar)
		{
			Factura facturaInvalida = new Factura();
			
			facturaInvalida.Fecha = facturaPorIngresar.Fecha;
			
			facturaInvalida.Tipo = facturaPorIngresar.Tipo;
			facturaInvalida.Codigo = facturaPorIngresar.Codigo;
			facturaInvalida.Numero = facturaPorIngresar.Numero - 1;
			
			// Asigno un cliente inválido a la factura inválida, según el tipo de ésta
			ControladorClientesI controladorClientes = new ControladorClientesI();
			if (facturaPorIngresar.Tipo == FacturaTipoVenta.FacturaA)
				facturaInvalida.Cliente =
					controladorClientes.ObtenerClienteInvalido(TipoResponsable.ResponsableInscripto);
			else
				facturaInvalida.Cliente =
					controladorClientes.ObtenerClienteInvalido(TipoResponsable.ConsumidorFinal);
//			transactionScope.Flush();
			
			// Asigno un item de producto
			ControladorProductosI controladorProductos = new ControladorProductosI();
			ItemFactura itemFactura = new ItemFactura(controladorProductos.ObtenerProductoInvalido(), 1);
			itemFactura.PrecioUnitario = 1.00m;
			itemFactura.Importe = 1.00m;
			facturaInvalida.Items.Add(itemFactura);
			
			// La nueva factura será inválida y estará anulada
			facturaInvalida.Anulada = true;
			facturaInvalida.Valida = false;
			
			// Asigno los valores de los importes según corresponda
			if (facturaPorIngresar.Tipo == FacturaTipoVenta.FacturaA) {
				facturaInvalida.ImporteBruto = 1.00m;
				facturaInvalida.ImporteBonificacion = 0.00m;
				facturaInvalida.ImporteNeto = 1.00m;
				facturaInvalida.ImporteIva = 0.21m;
				facturaInvalida.ImporteTotal = 1.21m;
			}
			else {
				// Si es una Factura B, el Item debe tener el precio unitario más el IVA
				itemFactura.PrecioUnitario = itemFactura.ProductoImporteVenta;
				itemFactura.Importe = itemFactura.PrecioUnitario;
				
				facturaInvalida.ImporteBruto = 1.21m;
				facturaInvalida.ImporteBonificacion = 0.00m;
				facturaInvalida.ImporteNeto = 1.21m;
				facturaInvalida.ImporteIva = 0.00m;
				facturaInvalida.ImporteTotal = 1.21m;
			}
			
			try {
				facturaInvalida.CreateAndFlush();
//				transactionScope.Flush();
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine(this.ObtenerMensajesExcepcion(ex));
			}
		}
		
		private string ObtenerMensajesExcepcion(Exception excepcion)
		{
			StringBuilder mensaje = new StringBuilder();
			System.Exception exTemp = excepcion;
			int i = 1;
			
			// Proceso el mensaje de la excepción y todas sus excepciones internas
			do {
				if (exTemp.Message != null && !exTemp.Message.Equals(String.Empty)) {
					mensaje.Append(i.ToString() + ": " + exTemp.ToString());
					mensaje.Append("\nStacktrace:\n" + exTemp.StackTrace);
					
					mensaje.Append("---\n\n");
				}
				
				exTemp = exTemp.InnerException;
				i++;
			}
			while (exTemp != null);
			
			return mensaje.ToString();
		}
		
		public string ObtenerMensajesExcepcion(ActiveRecordValidationException excepcion)
		{
			return ObtenerMensajesValidacion(excepcion.ValidationErrorMessages);
		}
		
		private string ObtenerMensajesValidacion(string[] mensajesError)
		{
			StringBuilder mensaje = new StringBuilder();
			
			foreach (string errStr in mensajesError) {
				mensaje.Append(" - ");
				mensaje.Append(errStr);
				mensaje.Append("\n");
			}
			
			return mensaje.ToString();
		}
		
		private bool PropiedadFallida<T>(string nombrePropiedad,
		                                 ActiveRecordValidationBase<T> objeto)
			where T : class
		{
			bool propiedadFallida = false;
			
			foreach (PropertyInfo pi in objeto.PropertiesValidationErrorMessages.Keys) {
				if (pi.Name.Equals(nombrePropiedad)) {
					propiedadFallida = true;
					break;
				}
			}
			
			return propiedadFallida;
		}
		
		private string LimpiarObservaciones(string observacion)
		{
			// No guardo las observaciones
			return String.Empty;
			
			// Dejo implementado este código que limpia las observaciones
			
//			if (observacion == null || observacion.Trim() == String.Empty)
//				return String.Empty;
//			
//			char[] chars = observacion.ToCharArray();
//			
//			// Si el string consiste en caracteres raros, que no son ni letras ni números,
//			// los saco.
//			if (chars.All(unChar => !Char.IsLetter(unChar) && !Char.IsNumber(unChar)))
//				return String.Empty;
//			
//			return observacion.Trim();
		}
		
		private void AgregarObservaciones(Factura factura, string observacion)
		{
			if (observacion.Trim() != String.Empty)
				factura.Observaciones += observacion + "\n";
		}
		
//		private List<ItemFactura> BuscarItemsFactura(Factura factura,
//		                                             string tipoDocumento,
//		                                             int codigoFacturaVerdadero)
//		{
		private List<ItemFactura> BuscarItemsFactura(Factura factura,
		                                             long orden)
		{
			// Creo un nuevo comando para obtener los items de la factura
			Console.WriteLine("    Creando nuevo comando");
			Npgsql.NpgsqlCommand comandoItems =
				MainClass.ComandoNuevo();
			
			// Voy armando la consulta Sql
			Console.WriteLine("    Construyendo consulta SQL");
			StringBuilder consultaSql = new StringBuilder();
			
			// Para prefacturas
			consultaSql.Append("select * from " + this.TablaItems);
			consultaSql.Append(" where \"Orden\" = " + orden);
			
			// Para facturas históricas
//			consultaSql.Append("select * from " + this.TablaItems);
//			consultaSql.Append(" where (\"Tip_Doc\" = '" + tipoDocumento + "') ");
//			
//			// Tipo Factura
//			if (factura.Tipo == FacturaTipoVenta.FacturaA)
//				consultaSql.Append(" and \"Serie\" = 'A' ");
//			else
//				consultaSql.Append(" and \"Serie\" = 'B' ");
//			
//			// Código y número
//			consultaSql.Append(" and \"Cod_suc\" = '" + codigoFacturaVerdadero.ToString("D4") + "' ");
//			consultaSql.Append(" and \"Num_Doc\" = " + factura.Numero.ToString() + " ");
			
			// Seteo la consulta
			comandoItems.CommandText = consultaSql.ToString();
			
			// Ejecuto el comando
			Console.WriteLine("    Ejecutando ExecuteReader");
			IDataReader reader;
			reader = comandoItems.ExecuteReader();
			
			DataGetter dg = new DataGetter(reader);
			
			// Leo los items
			ItemFactura itemFactura;
			List<ItemFactura> listaItems = new List<ItemFactura>();
			
			int ordenItems = 0;
			int? ordenActual;
			
			while (reader.Read()) {
				itemFactura = new ItemFactura();
				
				ordenActual = dg.GetInteger("Item");
				
				if (ordenActual.HasValue)
					itemFactura.Orden = ordenActual.Value;
				else
					itemFactura.Orden = ordenItems;
				
				try {
					itemFactura.Cantidad = Convert.ToDecimal(dg.GetDouble("Can_Pro").Value);
					itemFactura.PrecioUnitario = dg.GetDecimal("Pre_Cal").Value;
					itemFactura.Importe = dg.GetDecimal("Tot_Pro").Value;
				}
				catch (System.Exception ex) {
					Console.WriteLine("      ERROR con algún importe de los items: " + ex.ToString());
					Console.WriteLine("         Se obvia el item en la factura y se continúa");
					
					continue;
				}
				
				Console.WriteLine("    Item: " + itemFactura.Orden);
				
				// Producto del item
				string codigoProducto = dg.GetString("Cod_Pro");
				string descripcionProducto = dg.GetString("Nom_Pro");
				
				if (codigoProducto != null) codigoProducto = codigoProducto.Trim();
				if (descripcionProducto != null) descripcionProducto = descripcionProducto.Trim();
				
				// Busco el producto por código
				Producto productoItem = Producto.FindOne(Restrictions.Eq("Codigo", codigoProducto));
				
				// Si no lo encuentro por código, lo busco por descripción
				if (productoItem == null)
					productoItem = Producto.FindOne(Restrictions.Eq("Descripcion", descripcionProducto));
				
				// Si el producto no existe, entonces lo doy de alta en la base de datos
				// como un producto abstracto y con un código especial si no tiene asignado uno
				if (productoItem == null) {
					
					Console.WriteLine("          Producto nulo, creando nuevo");
					Console.WriteLine("            Código: {0}", codigoProducto);
					Console.WriteLine("            Descripción: {0}", descripcionProducto);
					
					// Si el código del producto es nulo, creo uno automático
					if (codigoProducto.Trim() == String.Empty)
						codigoProducto = "AUTO" + this.codigoProductoNumeracionAutomatica++;
					
					// Creo el producto
					using (SessionScope s = new SessionScope()) {
						productoItem = new Producto();
						
						productoItem.Codigo = codigoProducto;
						productoItem.Descripcion = descripcionProducto;
						productoItem.ActivarGestionStock = false;
						productoItem.ImporteSinIva = itemFactura.PrecioUnitario;
						productoItem.PorcentajeIva = 21.0m;
						productoItem.PrecioCosto = 0.01m;
						productoItem.ProductoAbstracto = false;
						productoItem.Eliminado = true;
						
						if (codigoProducto.ToUpper().Trim() == "ARTI") {
							productoItem.Rubro = Rubro.Find(this.rubroIndustria.Codigo);
							productoItem.Descripcion = "ART. IND: ";
							productoItem.ProductoAbstracto = true;
						}
						else
							productoItem.Rubro = Rubro.Find(this.rubroComercio.Codigo);
						
						productoItem.Unidad = UnidadProducto.Unidad;
						productoItem.Notas = "Producto creado automáticamente en la importación de Facturas";
						
						try {
							productoItem.Create();
						}
						catch(ActiveRecordValidationException ex) {
							Console.WriteLine("    ERROR al crear el producto: " + ex.Message);
							
							Console.WriteLine("      Codigo producto: {0}", productoItem.Codigo);
							Console.WriteLine("      Descripción producto: {0}", productoItem.Descripcion);
							
							// Itero por cada propiedad inválida y agrego una nota sobre la misma
							foreach (PropertyInfo pi in productoItem.PropertiesValidationErrorMessages.Keys) {
								Console.WriteLine("      Propiedad inválida: '" + pi.Name + "'");
								Console.WriteLine("        Mensaje: " + ((ArrayList)productoItem.PropertiesValidationErrorMessages[pi])[0]);
							}
							
							reader.Close();
							reader.Dispose();
							throw;
						}
					}
				}
				
				// Asigno el producto
				itemFactura.AsignarProducto(Producto.Find(productoItem.Id));
				
				// Asigno la descripción del item
				itemFactura.Descripcion = productoItem.Descripcion + descripcionProducto;
				
				// Asigno a la información del producto almacenada en el item el precio unitario
				// del mismo en los datos del producto, ya que ese es el precio que se utilizó.
				// Si luego se hace una nota de crédito sobre una factura migrada, y no se realiza
				// esta operación, puede que salte la validación de que el precio unitario del item
				// no es igual al precio del producto (con o sin) IVA.
				if (factura.Tipo == FacturaTipoVenta.FacturaA) {
					itemFactura.ProductoImporteSinIva = itemFactura.PrecioUnitario;
					itemFactura.ProductoImporteVenta = StratosUtil.RoundDecimal(itemFactura.ProductoImporteSinIva * 1.21m, 2);
				}
				else {
					itemFactura.ProductoImporteVenta = itemFactura.PrecioUnitario;
					itemFactura.ProductoImporteSinIva = StratosUtil.RoundDecimal(itemFactura.ProductoImporteVenta / 1.21m, 2);
				}
				
				// Asigno la descripción del producto
				itemFactura.Descripcion = descripcionProducto;
				
				// TODO: Habría que calcular el porcentaje de bonificación de acuerdo a la desviación
				// que tiene el precio. Tener en cuenta que los precios de los produtos pueden haber cambiado
				// La columna "Pre_Ven" parece interesante para realizar estos cálculos.
				//itemFactura.PorcentajeBonificacion
				
				if (!itemFactura.IsValid()) {
					Console.WriteLine("    El item es inválido. Propiedades inválidas:");
					
					// Itero por cada propiedad inválida y agrego una nota sobre la misma
					foreach (PropertyInfo pi in itemFactura.PropertiesValidationErrorMessages.Keys) {
						Console.WriteLine("      Propiedad inválida: '" + pi.Name + "'");
						Console.WriteLine("        Mensaje: " + ((ArrayList)itemFactura.PropertiesValidationErrorMessages[pi])[0]);
					}
					
					// Corrijo el importe si está mal calculado
					if (this.PropiedadFallida("Importe", itemFactura)) {
						Console.Write("      Corrigiendo importe del item, modificando la bonificación del mismo.");
						
						decimal importeCorrecto = StratosUtil.RoundDecimal(itemFactura.Cantidad * itemFactura.PrecioUnitario, 2);
						decimal importeErroneo = itemFactura.Importe;
						
						itemFactura.PorcentajeBonificacion = ((importeCorrecto / importeErroneo) - 1) * 100;
						
						this.AgregarObservaciones(factura, "Importe items corregido con porcentaje bonificación");
					}
				}
				
				listaItems.Add(itemFactura);
				
				ordenItems++;
			}
			
			// Hay que cerrar el DataReader
			reader.Close();
			reader.Dispose();
			
//			return listaItems.ToArray();
			return listaItems;
		}
		
		public void CalcularImportes(Factura factura)
		{
			decimal sumaItems =
				factura.Items.Sum(item => item.Importe);
			
			// Calculo los importes de la factura
			factura.ImporteBruto = StratosUtil.RoundDecimal(sumaItems, 2);
			factura.ImporteBonificacion = StratosUtil.RoundDecimal(factura.ImporteBruto * (factura.PorcentajeBonificacion / 100m), 2);
			factura.ImporteNeto = factura.ImporteBruto - factura.ImporteBonificacion;
			
			if (factura.Tipo == FacturaTipoVenta.FacturaA)
				factura.ImporteIva = StratosUtil.RoundDecimal((factura.ImporteNeto * 0.21m), 2);
			else
				factura.ImporteIva = 0.00m;
			
			factura.ImporteTotal = (factura.ImporteIva + factura.ImporteNeto);
		}
		
		
		public void RealizarLimpieza(IDbConnection con)
		{
//			IDbCommand comando = con.CreateCommand();
//			
//			comando.CommandText =
//				"update " + this.Tabla + " " +
//				"set \"Num_Cuit\" = NULL " +
//				"where trim(both \"Num_Cuit\") = '';"
//				+
//				"update " + this.Tabla + " " +
//				"set \"Cod_Pcia\" = NULL " +
//				"where trim(both \"Cod_Pcia\") = '';"
//				+
//				"update " + this.Tabla + " " +
//				"set \"Num_Cuit\" = regexp_replace(\"Num_Cuit\",'[^0-9]', '', 'g');";
//			
//			comando.ExecuteNonQuery();
		}
		
		public static MapeoFacturas Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoFacturas();
				
				return instancia;
			}
		}
	}
}
