// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate.Criterion;

using Stratos.Common;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using StratosServer.Productos;
using StratosServer.Proveedores;

namespace Migrador
{
	public class MapeoFacturasCompras : IMapeoTabla
	{
		private static MapeoFacturasCompras instancia;
		
		private Rubro rubroComercio;
//		private Rubro rubroIndustria;
		
		private int codigoProductoNumeracionAutomatica;
		
		private ControladorProveedoresI controladorProveedores;
		
		private MapeoFacturasCompras()
		{
			this.controladorProveedores = new ControladorProveedoresI();
//			this.controladorClientes = new ControladorClientesI();
			
//			Console.WriteLine("Creando Rubro automático");
			this.rubroComercio = Rubro.FindOne(Restrictions.Eq("Nombre", "COMERCIO"));
//			this.rubroIndustria = Rubro.FindOne(Restrictions.Eq("Nombre", "INDUSTRIA"));
			
			this.codigoProductoNumeracionAutomatica = 100;
		}
		
		public string MensajeAntesDeComenzar {
			get {
				return "";//"- La clase cliente tiene que tener en su clave primaria PrimaryKeyType.Native";
			}
		}
		
		public string MensajeDespuesDeTerminar {
			get {
				return String.Empty;
			}
		}
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Facturas de compras";
			}
		}
		
		public string TablaItems {
			get { return "viejo.\"Hdetcom.db-stratos12\""; }
		}
		
		public string Tabla {
			get {
				// La tabla de los items parece ser Hisdet.DB-stratos28
				return "viejo.\"resttemp.DB-stratos11\"";
			}
		}
		
		public string ConsultaSql {
			get {
				// FIXME: Se levantan unicamente las facturas con un cliente con código
				// GUARDA: Debería haber una etiqueta GUARDA para avisar cosas de las que
				// hay que tener cuidado :)
				// GUARDA: No se qué diferencia hay entre FA y FA1 -> son tandas distintas, de distintas fechas
				return "select * from " + this.Tabla +// +
					//" where trim(both \"Cod_Cli\") <> ''" +
					" where " +
					"   \"Nom_cli\" is not null " +
//					"	and \"Fec_Doc\" <= '15-11-2008' " +
					"	and \"Tot_gen\" > 0.00 " +
//					"   where \"Num_doc\" = 1482 " +
//					"	and (\"Tip_Doc\" = 'NC' or \"Tip_Doc\" = 'NC1')" +
					" order by \"Orden\" asc";
			}
		}
		
		public void Transformar(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
//			Factura.ModoMigracionFacturas();
//			NotaCredito.ModoMigracionFacturas();
			FacturaCompra.ModoMigracion();
			
			while (reader.Read()) {
				using (SessionScope sessionScope = new SessionScope()) {
					this.ProcesarFila(reader, dg, sessionScope);
				}
			}
			
			FacturaCompra.ModoNormal();
//			Factura.ModoNormal();
//			NotaCredito.ModoNormal();
		}
		
		private void ProcesarFila(IDataReader reader, DataGetter dg, SessionScope sessionScope)
		{
			FacturaCompra factura;
			
//			Factura.ModoNormal();
			
			Console.Write("  Creando objeto ");
			factura = new FacturaCompra();
			
			// Tipo factura
			string tipoFactura = dg.GetString("Serie");
			Console.WriteLine("  Tipo de Factura: {0}", tipoFactura);
			
			if (tipoFactura == "A")
				factura.Tipo = FacturaTipoCompra.FacturaA;
			else if (tipoFactura == "C")
				factura.Tipo = FacturaTipoCompra.FacturaC;
			else
				Console.WriteLine("WARNING: Factura con tipo desconocido: '{0}'", tipoFactura);
			
			// Fecha
			Console.WriteLine("  Fecha de Factura");
			try {
				factura.Fecha = dg.GetDateTime("Fec_Doc").Value;
				factura.FechaImputacion = dg.GetDateTime("Fec_imp").Value;
				Console.WriteLine("    Fecha: " + factura.Fecha.ToShortDateString());
				Console.WriteLine("    Fecha imputación: " + factura.FechaImputacion.ToShortDateString());
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: la fecha de la factura es nula: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
				Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Código, Número, Anulada y Remitos
			int codigoFacturaVerdadero = Convert.ToInt32(dg.GetString("Suc_doc"));
			factura.Codigo = codigoFacturaVerdadero;
			
			Console.WriteLine("  Codigo de Factura: {0}", factura.Codigo);
			
			factura.Numero = Convert.ToInt64(dg.GetDouble("Num_doc"));
			Console.WriteLine("  Número de Factura: {0}", factura.Numero);
			
			// Proveedor
			Console.WriteLine("  Proveedor de Factura");
			Proveedor proveedorLeido = MapeoProveedores.LeerProveedorDesdeDataReader(reader, "Cod_cli", "Nom_cli", "Tip_iva", "Num_cuit");
			Proveedor proveedor = null;
			
			// Busco por código
			if (proveedorLeido.Codigo > 0 && Proveedor.Exists(Restrictions.Eq("Codigo",proveedorLeido.Codigo))) {
				Console.WriteLine("    El proveedor tiene código {0} y existe en la base de datos", proveedorLeido.Codigo);
				
				try {
					proveedor = Proveedor.Find(proveedorLeido.Codigo);
				}
				catch (System.Exception ex) {
					Console.WriteLine("     ERROR al buscar proveedor por código: " + ex.ToString());
					
					Console.ReadLine();
					
//					transactionScope.VoteRollBack();
					return;
				}
			}
			// Busco por cuit
			else if (proveedorLeido.Cuit.CuitValido && Proveedor.Exists(Restrictions.Eq("Cuit.ValorNumerico", proveedorLeido.Cuit.ValorNumerico))) {
				Console.WriteLine("    El proveedor tiene un cuit válido {0} y existe en la base de datos.", proveedorLeido.Cuit.ValorFormateado);
				
				try {
					proveedor = Proveedor.FindOne(Restrictions.Eq("Cuit.ValorNumerico", proveedorLeido.Cuit.ValorNumerico));
				}
				catch (System.Exception ex) {
					Console.WriteLine("     ERROR al buscar proveedor con cuit: " + ex.ToString());
					
					Console.ReadLine();
					
//					transactionScope.VoteRollBack();
					return;
				}
			}
			else {
				Console.WriteLine("    Sin código y cuit válido.");
				
				// Busco por razón social (búsqueda exacta)
				if (proveedorLeido.RazonSocial != null && proveedorLeido.RazonSocial != String.Empty) {
					// Busco por razón social
					Console.WriteLine("      Busco por razón social.");
					
					ProveedorDto[] proveedoresRazonSocial = null;
					
					// Si es un consumidor final, entonces realizo una consulta (menos exacto, pero con más
					// probabilidades de encontrar algo). Sino realizo una búsqueda exacta por razón social.
					proveedoresRazonSocial = this.controladorProveedores.ConsultaPorRazonSocial(proveedorLeido.RazonSocial);
					
					if (proveedoresRazonSocial.Length == 0) {
						Console.WriteLine("         Tampoco se encontró por razón social");
						proveedor = proveedorLeido;
					}
					else {
						if (proveedoresRazonSocial.Length == 1)
							Console.WriteLine("         Se encontró exactamente un proveedor por razón social");
						else
							Console.WriteLine("         Se encontró más de un proveedor por razón social. Tomo el primero.");
						
						proveedor = Proveedor.Find(proveedoresRazonSocial[0].codigo);
						
						this.AgregarObservaciones(factura, "Proveedor buscado por razón social");
					}
				}
				else {
					Console.WriteLine("    Se carga el proveedor leído");
					proveedor = proveedorLeido;
				}
			}
			
			factura.Proveedor = proveedor;
			
			// Items
			Console.WriteLine("  Items de FacturaCompra");
			try {
				long orden = Convert.ToInt64(dg.GetDouble("Orden"));
				factura.Items = this.BuscarItemsFactura(factura, orden);
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: algún valor numérico de los items es nulo: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
				Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			catch (ActiveRecordValidationException) {
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Importes
			Console.WriteLine("  Importes de Factura");
			try {
				decimal? importeGravado = dg.GetDecimal("Tot_gra");
				decimal? importeExento = dg.GetDecimal("Tot_exe");
				decimal? importeIva = dg.GetDecimal("Tot_iva");
				decimal? percepcionIva = dg.GetDecimal("Tot_per");
				decimal? percepcionIB = dg.GetDecimal("Tot_ret");
				decimal? impuestoCombustible = dg.GetDecimal("Tot_imp_comb");
				decimal? impuestoTotal = dg.GetDecimal("Tot_gen");
				
				if (importeGravado.HasValue)
					factura.ImporteGravado = importeGravado.Value;
				
				if (importeExento.HasValue)
					factura.ImporteExento = importeExento.Value;
				
				if (importeIva.HasValue)
					factura.ImporteIva = importeIva.Value;
				
				if (percepcionIva.HasValue)
					factura.PercepcionIva = percepcionIva.Value;
				
				if (percepcionIB.HasValue)
					factura.PercepcionIB = percepcionIB.Value;
				
				if (impuestoCombustible.HasValue)
					factura.ImpuestoCombustible = impuestoCombustible.Value;
				
				if (impuestoTotal.HasValue)
					factura.ImporteTotal = impuestoTotal.Value;
			}
			catch (System.InvalidOperationException ex) {
				Console.WriteLine("    ERROR: algún importe no tiene valor, es nulo: " + ex.Message);
				Console.WriteLine();
				Console.WriteLine();
				
				Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Observaciones
			Console.WriteLine("  Observaciones de Factura");
			factura.Observaciones = String.Empty;
			
			#region Validaciones
			// Comienzo la validación y guardado
			Console.WriteLine("###");
			Console.WriteLine("  Validando factura");
			
//			// Si la factura es inválida en algún importe, los recalculo
//			if (!factura.IsValid()) {
//				var propiedadesImporteInvalidas =
//					from prop in factura.PropertiesValidationErrorMessages.Keys.Cast<PropertyInfo>()
//					where prop.Name.StartsWith("Importe")
//					select 1;
//				
//				if (propiedadesImporteInvalidas.Count() > 0)
//					this.CalcularImportes(factura);
//			}
			
			if (factura.Proveedor.Codigo == 0 || !Proveedor.Exists(Restrictions.Eq("Codigo", factura.Proveedor.Codigo))) {
				Console.WriteLine("   El proveedor asignado no existe. Asigno solo los datos mínimos del mismo a la factura");
				
				factura.ProveedorRazonSocial = proveedor.RazonSocial;
				factura.ProveedorTipoResponsable = proveedor.TipoResponsable;
				factura.ProveedorCuit = proveedor.Cuit.ValorNumerico.Value;
				
				factura.Proveedor = null;
			}
			
			// Verifico si la factura es inválida
			if (!factura.IsValid()) {
				Console.WriteLine("    Factura INVALIDA");
				
				// Itero por cada propiedad inválida y agrego una nota sobre la misma
				foreach (PropertyInfo pi in factura.PropertiesValidationErrorMessages.Keys) {
					Console.WriteLine("    Propiedad inválida: '" + pi.Name + "' ({0})", pi.GetValue(factura,null));
					Console.WriteLine("      Mensaje: " + ((ArrayList)factura.PropertiesValidationErrorMessages[pi])[0]);
				}
				
				//throw new Exception("Factura inválida");
				Console.WriteLine();
				Console.WriteLine();
				
				IDictionary propiedadesInválidas = factura.PropertiesValidationErrorMessages;
				
				if (propiedadesInválidas.Count > 1)
					Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			// Si llego a este punto, entonces la factura muy probablemente sea válida.
			// Entonces doy de alta el cliente asociado si se trata de una factura A y
			// éste todavía no se guardó.
//			if (factura.Tipo == FacturaTipoVenta.FacturaA &&
//			    	factura.Cliente.Codigo == 0) {
//			if (factura.Cliente.Codigo == 0) {
//				Console.WriteLine("  Creando cliente asociado a factura");
//				
////				using (TransactionScope ts = new TransactionScope(TransactionMode.Inherits, OnDispose.Commit)) {
//					try {
//						Cliente.ModoMigracion();
//						
//						if (cliente.TipoResponsable == TipoResponsable.ConsumidorFinal)
//							cliente.Cuit = null;
//					
//						cliente.AltaAutomatica = true;
//						cliente.Create();
//					
//						factura.Cliente = cliente;
//					
//						Cliente.ModoNormal();
//					
////						factura.Cliente.AltaAutomatica = true;
////						factura.Cliente.Create();
//					}
//					catch (ActiveRecordValidationException ex) {
//						Console.WriteLine("      ERROR al guardar el cliente, cliente inválido.");
//						Console.WriteLine("        Propiedades inválidas:");
//						
//						// Itero por cada propiedad inválida y agrego una nota sobre la misma
//						foreach (PropertyInfo pi in factura.Cliente.PropertiesValidationErrorMessages.Keys) {
//							Console.WriteLine("          Propiedad inválida: '" + pi.Name + "'");
//							Console.WriteLine("          Mensaje: " + ((ArrayList)factura.Cliente.PropertiesValidationErrorMessages[pi])[0]);
//						}
//					}
//					catch (System.Exception ex) {
//						Console.WriteLine("      ERROR al guardar el cliente: " + ex.ToString());
//						
////						ts.VoteRollBack();
////						transactionScope.VoteRollBack();
//						return;
//					}
////				}
//			}
			#endregion
			
//				try {
			
			
			Console.WriteLine("  Guardando Factura");
			
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine("    ERROR Validación en Factura: " + ObtenerMensajesExcepcion(ex));
				Console.WriteLine();
				Console.WriteLine();
				
				Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			catch (Exception ex) {
				Console.WriteLine("    ERROR desconocido en Factura: " + ObtenerMensajesExcepcion(ex));
				Console.WriteLine();
				Console.WriteLine();
				
				Console.ReadLine();
				
//				transactionScope.VoteRollBack();
				return;
			}
			
			Console.WriteLine();
			Console.WriteLine("  ¡Factura convertido correctamente!");
			
			Console.WriteLine();
			Console.WriteLine();
		}
		
		private string ObtenerMensajesExcepcion(Exception excepcion)
		{
			StringBuilder mensaje = new StringBuilder();
			System.Exception exTemp = excepcion;
			int i = 1;
			
			// Proceso el mensaje de la excepción y todas sus excepciones internas
			do {
				if (exTemp.Message != null && !exTemp.Message.Equals(String.Empty)) {
					mensaje.Append(i.ToString() + ": " + exTemp.ToString());
					mensaje.Append("\nStacktrace:\n" + exTemp.StackTrace);
					
					mensaje.Append("---\n\n");
				}
				
				exTemp = exTemp.InnerException;
				i++;
			}
			while (exTemp != null);
			
			return mensaje.ToString();
		}
		
		public string ObtenerMensajesExcepcion(ActiveRecordValidationException excepcion)
		{
			return ObtenerMensajesValidacion(excepcion.ValidationErrorMessages);
		}
		
		private string ObtenerMensajesValidacion(string[] mensajesError)
		{
			StringBuilder mensaje = new StringBuilder();
			
			foreach (string errStr in mensajesError) {
				mensaje.Append(" - ");
				mensaje.Append(errStr);
				mensaje.Append("\n");
			}
			
			return mensaje.ToString();
		}
		
		private bool PropiedadFallida<T>(string nombrePropiedad,
		                                 ActiveRecordValidationBase<T> objeto)
			where T : class
		{
			bool propiedadFallida = false;
			
			foreach (PropertyInfo pi in objeto.PropertiesValidationErrorMessages.Keys) {
				if (pi.Name.Equals(nombrePropiedad)) {
					propiedadFallida = true;
					break;
				}
			}
			
			return propiedadFallida;
		}
		
		private string LimpiarObservaciones(string observacion)
		{
			// No guardo las observaciones
			return String.Empty;
			
			// Dejo implementado este código que limpia las observaciones
			
//			if (observacion == null || observacion.Trim() == String.Empty)
//				return String.Empty;
//			
//			char[] chars = observacion.ToCharArray();
//			
//			// Si el string consiste en caracteres raros, que no son ni letras ni números,
//			// los saco.
//			if (chars.All(unChar => !Char.IsLetter(unChar) && !Char.IsNumber(unChar)))
//				return String.Empty;
//			
//			return observacion.Trim();
		}
		
		private void AgregarObservaciones(FacturaCompra factura, string observacion)
		{
			if (observacion.Trim() != String.Empty)
				factura.Observaciones += observacion + "\n";
		}
		
		private List<ItemFacturaCompra> BuscarItemsFactura(FacturaCompra factura,
		                                             long orden)
		{
			// Creo un nuevo comando para obtener los items de la factura
			Console.WriteLine("    Creando nuevo comando");
			Npgsql.NpgsqlCommand comandoItems =
				MainClass.ComandoNuevo();
			
			// Voy armando la consulta Sql
			Console.WriteLine("    Construyendo consulta SQL");
			StringBuilder consultaSql = new StringBuilder();
			
			// Para prefacturas
			consultaSql.Append("select * from " + this.TablaItems);
			consultaSql.Append(" where \"Orden\" = " + orden);
			consultaSql.Append(" and \"Pre_Com\" > 0");
			
			
			// Para facturas históricas
//			consultaSql.Append("select * from " + this.TablaItems);
//			consultaSql.Append(" where (\"Tip_Doc\" = '" + tipoDocumento + "') ");
//			
//			// Tipo Factura
//			if (factura.Tipo == FacturaTipoVenta.FacturaA)
//				consultaSql.Append(" and \"Serie\" = 'A' ");
//			else
//				consultaSql.Append(" and \"Serie\" = 'B' ");
//			
//			// Código y número
//			consultaSql.Append(" and \"Cod_suc\" = '" + codigoFacturaVerdadero.ToString("D4") + "' ");
//			consultaSql.Append(" and \"Num_Doc\" = " + factura.Numero.ToString() + " ");
			
			// Seteo la consulta
			comandoItems.CommandText = consultaSql.ToString();
			
			// Ejecuto el comando
			Console.WriteLine("    Ejecutando ExecuteReader");
			IDataReader reader;
			reader = comandoItems.ExecuteReader();
			
			DataGetter dg = new DataGetter(reader);
			
			// Leo los items
			ItemFacturaCompra itemFactura;
			List<ItemFacturaCompra> listaItems = new List<ItemFacturaCompra>();
			
			int ordenItems = 0;
			int? ordenActual;
			
			while (reader.Read()) {
				itemFactura = new ItemFacturaCompra();
				
				ordenActual = dg.GetInteger("Item");
				
				if (ordenActual.HasValue)
					itemFactura.Orden = ordenActual.Value;
				else
					itemFactura.Orden = ordenItems;
				
				try {
					double? cantidad = dg.GetDouble("Can_Pro");
					decimal? precioUnitario = dg.GetDecimal("Pre_Com");
					
					if (cantidad.HasValue)
						itemFactura.Cantidad = Convert.ToDecimal(cantidad.Value);
					else
						itemFactura.Cantidad = 1.00m;
					
					if (precioUnitario.HasValue)
						itemFactura.PrecioUnitario = precioUnitario.Value;
					else
						itemFactura.PrecioUnitario = 1.00m;
					
					itemFactura.ProductoUnidad = this.ObtenerUnidadProducto(dg.GetString("Uni_Med"));
				}
				catch (System.Exception ex) {
					Console.WriteLine("      ERROR con algún importe de los items: " + ex.ToString());
					Console.WriteLine("         Se obvia el item en la factura y se continúa");
					
					Console.ReadLine();
					
					continue;
				}
				
				Console.WriteLine("    Item: " + itemFactura.Orden);
				
				// Producto del item
				string codigoProducto = dg.GetString("Cod_Pro");
				string descripcionProducto = dg.GetString("Nom_Pro");
				
				if (codigoProducto != null) codigoProducto = codigoProducto.Trim();
				if (descripcionProducto != null) descripcionProducto = descripcionProducto.Trim();
				
				// Busco el producto por código
				Producto productoItem = Producto.FindOne(Restrictions.Eq("Codigo", codigoProducto));
				
				// Si no lo encuentro por código, lo busco por descripción
				if (productoItem == null)
					productoItem = Producto.FindOne(Restrictions.Eq("Descripcion", descripcionProducto));
				
				// Si el producto no existe, entonces lo doy de alta en la base de datos
				// como un producto abstracto y con un código especial si no tiene asignado uno
				if (productoItem == null) {
					
					Console.WriteLine("          Producto nulo, creando nuevo");
					Console.WriteLine("            Código: {0}", codigoProducto);
					Console.WriteLine("            Descripción: {0}", descripcionProducto);
					
					Console.ReadLine();
					
					// Si el código del producto es nulo, creo uno automático
					if (codigoProducto.Trim() == String.Empty)
						codigoProducto = "AUTO" + this.codigoProductoNumeracionAutomatica++;
					
					// Creo el producto
					using (SessionScope s = new SessionScope()) {
						productoItem = new Producto();
						
						productoItem.Codigo = codigoProducto;
						productoItem.Descripcion = descripcionProducto;
						productoItem.ActivarGestionStock = false;
						productoItem.PrecioCosto = itemFactura.PrecioUnitario;
						productoItem.ImporteSinIva = productoItem.PrecioCosto + 1.0m;
						productoItem.PorcentajeIva = 21.0m;
						productoItem.ProductoAbstracto = false;
						productoItem.Eliminado = true;
						
						productoItem.Rubro = Rubro.Find(this.rubroComercio.Codigo);
						
						productoItem.Unidad = itemFactura.ProductoUnidad;
						productoItem.Notas = "Producto creado automáticamente en la importación de Facturas de compras";
						
						try {
							productoItem.Create();
						}
						catch(ActiveRecordValidationException ex) {
							Console.WriteLine("    ERROR al crear el producto: " + ex.Message);
							
							Console.WriteLine("      Codigo producto: {0}", productoItem.Codigo);
							Console.WriteLine("      Descripción producto: {0}", productoItem.Descripcion);
							
							// Itero por cada propiedad inválida y agrego una nota sobre la misma
							foreach (PropertyInfo pi in productoItem.PropertiesValidationErrorMessages.Keys) {
								Console.WriteLine("      Propiedad inválida: '" + pi.Name + "'");
								Console.WriteLine("        Mensaje: " + ((ArrayList)productoItem.PropertiesValidationErrorMessages[pi])[0]);
							}
							
							reader.Close();
							reader.Dispose();
							throw;
						}
					}
				}
				
				// Asigno el producto
				itemFactura.AsignarProducto(Producto.Find(productoItem.Id));
				
				// Asigno la descripción del producto
				itemFactura.Descripcion = descripcionProducto;
				
				if (!itemFactura.IsValid()) {
					Console.WriteLine("    El item es inválido. Propiedades inválidas:");
					
					// Itero por cada propiedad inválida y agrego una nota sobre la misma
					foreach (PropertyInfo pi in itemFactura.PropertiesValidationErrorMessages.Keys) {
						Console.WriteLine("      Propiedad inválida: '" + pi.Name + "'");
						Console.WriteLine("        Mensaje: " + ((ArrayList)itemFactura.PropertiesValidationErrorMessages[pi])[0]);
					}
				}
				
				listaItems.Add(itemFactura);
				
				ordenItems++;
			}
			
			// Hay que cerrar el DataReader
			reader.Close();
			reader.Dispose();
			
//			return listaItems.ToArray();
			return listaItems;
		}

		private UnidadProducto ObtenerUnidadProducto (string par1)
		{
			if (String.IsNullOrEmpty(par1))
				return UnidadProducto.Unidad;
			
			if (par1.ToLower().Equals("kil"))
				return UnidadProducto.Kilogramo;
			else if (par1.ToLower().Equals("mts"))
				return UnidadProducto.Metro;
			else if (par1.ToLower().Equals("uni"))
				return UnidadProducto.Unidad;
			else if (par1.ToLower().Equals("m2"))
				return UnidadProducto.MetroCuadrado;
			else if (par1.ToLower().Equals("lts"))
				return UnidadProducto.Litro;
			else
				return UnidadProducto.Unidad;
		}

		
//		public void CalcularImportes(Factura factura)
//		{
//			decimal sumaItems =
//				factura.Items.Sum(item => item.Importe);
//			
//			// Calculo los importes de la factura
//			factura.ImporteBruto = StratosUtil.RoundDecimal(sumaItems, 2);
//			factura.ImporteBonificacion = StratosUtil.RoundDecimal(factura.ImporteBruto * (factura.PorcentajeBonificacion / 100m), 2);
//			factura.ImporteNeto = factura.ImporteBruto - factura.ImporteBonificacion;
//			
//			if (factura.Tipo == FacturaTipoVenta.FacturaA)
//				factura.ImporteIva = StratosUtil.RoundDecimal((factura.ImporteNeto * 0.21m), 2);
//			else
//				factura.ImporteIva = 0.00m;
//			
//			factura.ImporteTotal = (factura.ImporteIva + factura.ImporteNeto);
//		}
		
		
		public void RealizarLimpieza(IDbConnection con)
		{
//			IDbCommand comando = con.CreateCommand();
//			
//			comando.CommandText =
//				"update " + this.Tabla + " " +
//				"set \"Num_Cuit\" = NULL " +
//				"where trim(both \"Num_Cuit\") = '';"
//				+
//				"update " + this.Tabla + " " +
//				"set \"Cod_Pcia\" = NULL " +
//				"where trim(both \"Cod_Pcia\") = '';"
//				+
//				"update " + this.Tabla + " " +
//				"set \"Num_Cuit\" = regexp_replace(\"Num_Cuit\",'[^0-9]', '', 'g');";
//			
//			comando.ExecuteNonQuery();
		}
		
		public static MapeoFacturasCompras Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoFacturasCompras();
				
				return instancia;
			}
		}
	}
}
