// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate;
using NHibernate.Criterion;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace Migrador
{
	public class MapeoProductos : IMapeoTabla
	{
		
		private static MapeoProductos instancia;
		private static int codigoRepeticionDescripcionProducto;
		
		private static Rubro rubroComercio;
		private static Rubro rubroIndustria;
		
		static MapeoProductos()
		{
			codigoRepeticionDescripcionProducto = 1;
		}
		
		private MapeoProductos()
		{
		}
		
		#region IMapeoTabla implementation
		public string MensajeAntesDeComenzar {
			get {
				return String.Empty;
			}
		}
		
		public string MensajeDespuesDeTerminar {
			get {
				return String.Empty;
			}
		}
		
		public string ConsultaSql {
			get {
				return "select * " +
					"from " + this.Tabla;
			}
		}
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Productos";
			}
		}
		
		public void RealizarLimpieza (System.Data.IDbConnection con)
		{
			IDbCommand comando = con.CreateCommand();
			
			comando.CommandText =
				"delete from " + this.Tabla + " " +
				"where \"Pre_Ven\" is null or \"Pre_Ven\" = 0;";
			
			comando.ExecuteNonQuery();
		}
		
		public string Tabla {
			get {
				return "viejo.\"resttemp.DB-stratos16\"";
			}
		}
		
		public static Producto LeerProductoDesdeDataReader(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			string temp;
			UnidadProducto unidad;
			
			Producto producto = new Producto();
			
			producto.ActivarGestionStock = false;
			
			producto.Codigo = dg.GetString("Cod_Pro");
			producto.Descripcion = dg.GetString("Nom_Pro");
			
			// Unidad del producto
			temp = dg.GetString("Uni_Med");
			if (temp.Equals("Kil"))
				unidad = UnidadProducto.Kilogramo;
			else if (temp.Equals("Lit"))
				unidad = UnidadProducto.Litro;
			else if (temp.Equals("M2"))
				unidad = UnidadProducto.MetroCuadrado;
			else if (temp.Equals("Mts"))
				unidad = UnidadProducto.Metro;
			else if (temp.Equals("Ton"))
				unidad = UnidadProducto.Tonelada;
			else
				unidad = UnidadProducto.Unidad;
			
			producto.Unidad = unidad;
			
			// Rubro
			Rubro rubro = null;
			temp = dg.GetString("Cod_Rub").Trim();
			
			if (temp.ToLower() == "c")
				rubro = Rubro.Find(rubroComercio.Codigo);
			else if (temp.ToLower() == "i")
				rubro = Rubro.Find(rubroIndustria.Codigo);
			
			producto.Rubro = rubro;
			
			// Subrubro: para buscar por código, sumo dos, ya que en el sistema viejo
			// los rubros y subrubros están en tablas separadas, y el primer subrubro
			// comienza con 1. En cambio en Stratos el primer rubro empieza con 1, y
			// el primer subrubro con 3 (están todos en la misma tabla y hay 2 rubros).
			temp = dg.GetString("Cod_Sub").Trim();
			Rubro subRubro;
			
			if (temp == null || temp.Equals(""))
				subRubro = Rubro.FindAllByProperty("Nombre", "VARIOS")[0];
			else
				subRubro = Rubro.FindAllByProperty("Codigo", Convert.ToInt64(temp) + 2)[0];
			
			producto.SubRubro = subRubro;
			
			producto.ImporteSinIva = dg.GetDecimal("Pre_Ven").Value;
			producto.PrecioCosto = 0.01m;
			producto.Notas = "El precio de costo debe ser corregido\n";
			
			Console.WriteLine("Procesando producto: " + producto.Descripcion);
			
			// Verifico si el producto es inválido
			if (!producto.IsValid()) {
				Console.WriteLine("  Corrijiendo producto...");
				
				// Itero por cada propiedad inválida y agrego una nota sobre la misma
				foreach (PropertyInfo pi in producto.PropertiesValidationErrorMessages.Keys) {
					
					Console.WriteLine("    Propiedad inválida: '" + pi.Name + "'");
					
					if (pi.Name == "Descripcion") {
						producto.Descripcion += " (Producto repetido " +
							codigoRepeticionDescripcionProducto++ + ")";
					}
					
					else {
						producto.Notas += pi.Name + " inválido (" +
							pi.GetValue(producto, null) + ");\n";
						pi.SetValue(producto, null, null);
					}
				}
				
			}
			
			return producto;
		}
		
		public void Transformar (System.Data.IDataReader reader)
		{
//			DataGetter dg = new DataGetter(reader);
//			
			Producto producto;
			
			rubroComercio = Rubro.FindOne(Restrictions.Eq("Nombre", "COMERCIO"));
			rubroIndustria = Rubro.FindOne(Restrictions.Eq("Nombre", "INDUSTRIA"));
			
//			string temp;
//			UnidadProducto unidad;
//			Rubro rubro;

			while (reader.Read()) {
				using (SessionScope sessionScope = new SessionScope()) {
					producto = LeerProductoDesdeDataReader(reader);
					
					try {
						producto.Create();
						Console.WriteLine("  ¡Producto convertido correctamente!");
					}
					catch(ActiveRecordValidationException ex) {
						Console.WriteLine("  No se pudo corregir. Propiedades inválidas:");
						
						// Itero por cada propiedad inválida y la imprimo en consola
						for (int i=0; i<ex.FailedProperties.Length; i++)
							Console.WriteLine("    " + ex.FailedProperties[i].Name +
							                  " (" + ex.ValidationErrorMessages[i] + ")");
						
						Console.WriteLine();
						Console.WriteLine("Presione una tecla para continuar...");
						Console.ReadLine();
					}
					
					Console.WriteLine();
				}
			}
		}
		#endregion
				
		public static MapeoProductos Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoProductos();
				
				return instancia;
			}
		}
	}
}
