// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;
using NHibernate;
using NHibernate.Criterion;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace Migrador
{
	public class MapeoRecibos : IMapeoTabla
	{
		static MapeoRecibos instancia;
		
		public MapeoRecibos ()
		{
		}
		
		#region IMapeoTabla implementation
		public void RealizarLimpieza (System.Data.IDbConnection con)
		{
		}
		
		
		public string MensajeAntesDeComenzar {
			get {
				return "Que la clase Recibo no asigne en BeforeSave el siguiente número";
			}
		}
		
		
		public string MensajeDespuesDeTerminar {
			get {
				return "Volver a poner en la clase Recibo en el método BeforeSave la asignación automática de último número";
			}
		}
		
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Recibos";
			}
		}
		
		
		public string Tabla {
			get {
				return "viejo.\"resttemp.DB-stratos27\"";
			}
		}
		
		public string TablaItems {
			get { return "viejo.\"valores.DB-stratos28\""; }
		}
		
		public string ConsultaSql {
			get {
				return
					"select * from " + this.Tabla + " " +
					"where \"Monto_total\" is not null and \"Monto_total\" > 0 " +
					"	and \"Numero_documento\" is not null";
			}
		}
		
		public void Transformar (System.Data.IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			while (reader.Read()) {
				Console.WriteLine("Procesando Recibo...");
				using (SessionScope sessionScope = new SessionScope()) {
					this.ProcesarFila(reader);
				}
				Console.WriteLine("Recibo procesado...");
				Console.WriteLine();
			}
		}
		
		private void ProcesarFila(System.Data.IDataReader reader)
		{
			Recibo recibo = this.LeerRecibo(reader);
			
			try {
				Console.WriteLine("  Fecha: {0}", recibo.Fecha);
				recibo.Create();
				Console.WriteLine("Recibo pasado correctamente");
			}
			catch (ActiveRecordValidationException ex) {
				Console.WriteLine("Error al procesar recibo: " + this.ObtenerMensajesExcepcion(ex));
			}
		}
		
		public string ObtenerMensajesExcepcion(ActiveRecordValidationException excepcion)
		{
			return ObtenerMensajesValidacion(excepcion.ValidationErrorMessages);
		}
		
		private string ObtenerMensajesValidacion(string[] mensajesError)
		{
			StringBuilder mensaje = new StringBuilder();
			
			foreach (string errStr in mensajesError) {
				mensaje.Append(" - ");
				mensaje.Append(errStr);
				mensaje.Append("\n");
			}
			
			return mensaje.ToString();
		}
		#endregion
		
		public static MapeoRecibos Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoRecibos();
				
				return instancia;
			}
		}
		
		public Recibo LeerRecibo(IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			Recibo recibo = new Recibo();
			
			long codigoCliente = long.Parse(dg.GetString("Cod_cli").Trim());
			Cliente cliente = Cliente.FindOne(Restrictions.Eq("Codigo", codigoCliente));
			
			recibo.Cliente = cliente;
			
			recibo.Concepto = dg.GetString("Concepto");
			if (recibo.Concepto.Trim() == String.Empty)
				recibo.Concepto = "PAGO";
			
			recibo.Fecha = dg.GetDateTime("Fecha_documento").Value;
			recibo.Importe = dg.GetDecimal("Monto_total").Value;
			recibo.Numero = Convert.ToInt64(dg.GetDouble("Numero_documento").Value);
			
			long reciboId = Convert.ToInt64(dg.GetDouble("#_id").Value);
			
			// Items:
			recibo.Items = BuscarItemsRecibo(recibo, reciboId);
			
			return recibo;
		}
		
		protected List<ItemRecibo> BuscarItemsRecibo(Recibo recibo,
		                                             long reciboId)
		{
			List<ItemRecibo> itemsRecibo = new List<ItemRecibo>();
			
			// Creo un nuevo comando para obtener los items de la factura
			Console.WriteLine("    Creando nuevo comando");
			Npgsql.NpgsqlCommand comandoItems =
				MainClass.ComandoNuevo();
			
			// Voy armando la consulta Sql
			Console.WriteLine("    Construyendo consulta SQL");
			
			string consultaItems =
				"select * from " + this.TablaItems +
					" where \"Rec_#_id\" = " + reciboId.ToString() + " " +
					"and \"Cod_val\" is not null and \"Cod_val\" <> ''";
			
			// Seteo la consulta
			comandoItems.CommandText = consultaItems;
			
			// Ejecuto el comando
			Console.WriteLine("    Ejecutando ExecuteReader");
			IDataReader reader;
			reader = comandoItems.ExecuteReader();
			
			DataGetter dg = new DataGetter(reader);
			
			ItemRecibo itemRecibo;
			
			while (reader.Read()) {
				itemRecibo = new ItemRecibo();
				
				itemRecibo.Orden = dg.GetInteger("Num_ite").Value;
				
				string medioPago = dg.GetString("Cod_val").ToUpper();
				if (medioPago == "EF") {
					itemRecibo.MedioPago = MedioPago.Efectivo;
					itemRecibo.Importe = dg.GetDecimal("Imp_val").Value;
				}
				else if (medioPago == "CH") {
					itemRecibo.MedioPago = MedioPago.Cheque;
					
					Cheque cheque = new Cheque();
					
					cheque.Banco = dg.GetString("Ban_tar");
					if (cheque.Banco.Trim() == String.Empty)
						cheque.Banco = "BANCO NO ASIGNADO";
					
					cheque.Sucursal = dg.GetString("Nom_fil");
					if (cheque.Sucursal.Trim() == String.Empty)
						cheque.Sucursal = "SUCURSAL NO ASIGNADA";
					
					string numeroCheque = dg.GetString("Num_val").Trim();
					if (Stratos.Common.StratosUtil.IsDouble(numeroCheque))
						cheque.Numero = double.Parse(numeroCheque);
					else
						cheque.Numero = 1;
					
					if (dg.GetDateTime("Fec_val").HasValue)
						cheque.FechaEmision = dg.GetDateTime("Fec_val").Value;
					else
						cheque.FechaEmision = DateTime.Today;
					
					cheque.FechaVencimiento = cheque.FechaEmision;
					cheque.Importe = dg.GetDecimal("Imp_val").Value;
					
					itemRecibo.Cheque = cheque;
					itemRecibo.Importe = cheque.Importe;
				}
				
				itemsRecibo.Add(itemRecibo);
			}
			
			return itemsRecibo;
		}
	}
}
