// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Reflection;
using System.Data;

using Castle.ActiveRecord;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace Migrador
{
	public class MapeoSubrubros : IMapeoTabla
	{
		
		private static MapeoSubrubros instancia;
		
		private MapeoSubrubros()
		{
		}
		
		#region IMapeoTabla implementation
		public string MensajeAntesDeComenzar {
			get {
				return String.Empty;
			}
		}
		
		public string MensajeDespuesDeTerminar {
			get {
				return String.Empty;
			}
		}
		
		public string ConsultaSql {
			get {
				return "select * from " + this.Tabla +
					" order by \"Cod_sub\" asc";
			}
		}
		
		public string DescripcionMapeo {
			get {
				return "Tabla de Subrubros";
			}
		}
		
		public void RealizarLimpieza (System.Data.IDbConnection con)
		{
			
		}
		
		public string Tabla {
			get {
				return "viejo.\"Tsubrub.db-stratos19\"";
			}
		}
		
		public void Transformar (System.Data.IDataReader reader)
		{
			DataGetter dg = new DataGetter(reader);
			
			// Codigos de subrubros que son de industria
			int[] codigosIndustria = new int[] {1, 3, 4, 24};
			
			Rubro rubro;
			int codigo;
			
			while (reader.Read()) {
				using (SessionScope sessionScope = new SessionScope()) {
					rubro = new Rubro();
					
					// Le sumo dos al código para que los dos primeros queden para los Rubros
					// INDUSTRIA y COMERCIO
					codigo = Convert.ToInt32(dg.GetString("Cod_sub"));
					rubro.Nombre = dg.GetString("Nom_sub");
					
					// Si el código del subrubro pertenece al rubro industria...
					if (codigosIndustria.Any(x => x == codigo))
						rubro.RubroPadre = Rubro.FindAllByProperty("Nombre", "INDUSTRIA")[0];
					else
						rubro.RubroPadre = Rubro.FindAllByProperty("Nombre", "COMERCIO")[0];
					
					try {
						rubro.Create();
						Console.WriteLine("Subrubro convertido: " + rubro.Nombre);
					}
					catch(ActiveRecordValidationException ex) {
						Console.WriteLine("Subrubro falló: " + rubro.Nombre);
					}
				}
			}
		}
		#endregion
		
		public static MapeoSubrubros Instancia {
			get {
				if (instancia == null)
					instancia = new MapeoSubrubros();
				
				return instancia;
			}
		}
	}
}