#ifndef _CONTROLADOR_BASE
#define _CONTROLADOR_BASE

#include <Excepciones/StratosException.ice>
#include <QueryObjects/QueryObject.ice>
#include <Controladores/ConsultaPaginable.ice>

module StratosServer {
	module Controladores {
		
		sequence<Object> ObjetosSeq;
		
		interface ControladorBase {
			
			/**
			 * Permite enviar una consulta en HQL al servidor con un objeto ConsultaPaginable
			 * opcional. Si este es nulo o vacio se obvia.
			 **/
			ObjetosSeq BuscarHQL (string consultaHQL, ConsultaPaginable consPaginable)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite enviar un QueryObject al servidor con un objeto ConsultaPaginable
			 * opcional. Si este es nulo o vacio se obvia.
			 **/
			ObjetosSeq BuscarQueryObjects(::Stratos::QueryObjects::QueryObject queryObject,
					ConsultaPaginable consPaginable)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Dada una instancia de algún objeto DTO, permite refrescar su datos con
			 * la última versión que hay en la base de datos.
			 **/
			Object Refrescar(Object objeto)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Guarda el objeto dado en la base de datos, y devuelve la versión
			 * Dto actualizada.
			 **/
			Object Guardar(Object objeto)
				throws ::Stratos::Common::Excepciones::StratosException;
				
			/**
			 * Modifica el objeto dado en la base de datos, y devuelve la versión
			 * Dto actualizada.
			 **/
			Object Modificar(Object objeto)
				throws ::Stratos::Common::Excepciones::StratosException;
		};
	
	};
};

#endif