#ifndef _CONTROLADOR_FACTURACION
#define _CONTROLADOR_FACTURACION

#include <Controladores/ControladorBase.ice>
#include <Controladores/ConsultaPaginable.ice>
#include <Excepciones/StratosException.ice>
#include <Documentos/Facturas/FacturaDto.ice>
#include <Documentos/Facturas/PreFacturaDto.ice>
#include <Documentos/CodigoFacturaDto.ice>

module StratosServer {
	module Controladores {
		
		sequence<::Stratos::ModeloDto::FacturaDto> FacturaSeq;
		sequence<::Stratos::ModeloDto::PreFacturaDto> PreFacturaSeq;
		sequence<::Stratos::ModeloDto::CodigoFacturaDto> CodigoFacturaSeq;
		
		interface ControladorFacturacion extends ControladorBase {
		
			/**
			 * Permite buscar PreFacturas con HQL y un objeto ConsultaPaginable.
			 **/
			PreFacturaSeq BuscarPreFacturaHQL (string consultaHQL, ConsultaPaginable consPaginable)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite buscar Facturas con HQL y un objeto ConsultaPaginable.
			 **/
			FacturaSeq BuscarFacturaHQL (string consultaHQL, ConsultaPaginable consPaginable)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Premite obtener todas las PreFacturas que una terminal es capaz de imprimir.
			 **/
			PreFacturaSeq ObtenerPreFacturasEncoladasParaImprimir(string identificadorTerminal,
					ConsultaPaginable consPaginable)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Almacena una PreFactura (que puede ser nueva o una ya existente, en cuyo caso la
			 * modifica) y la envía a la terminal impresora correspondiente, si es
			 * que hay una conectada. El valor de retorno depende de esto último: si se envió la
			 * PreFactura a una terminal impresora, se devuelve true. Si hay algún error se setea
			 * la variable de salida 'mensajeError'.
			 **/
			bool GuardarPreFactura(::Stratos::ModeloDto::PreFacturaDto preFacturaDto,
					out string mensajeError)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Guarda los cambios realizados en una factura manual ya ingresada.
			 **/
			::Stratos::ModeloDto::FacturaDto GuardarFacturaManual(::Stratos::ModeloDto::FacturaDto facturaManual)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Esta operación la ejecuta el Servicio de Facturacion en la Terminal cuando el usuario
			 * decide imprimir una PreFactura (y que por lo tanto pasará a ser Factura). Realiza algunas
			 * validaciones sobre la misma y retorna el siguiente número de factura por imprimir.
			 **/
			void PrepararPreFacturaParaImprimir(::Stratos::ModeloDto::PreFacturaDto preFacturaDto,
					out long numeroFactura)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Hace que la PreFactura se elimine y pase a ser una Factura. Devuelve
			 * la nueva Factura.
			 **/
			::Stratos::ModeloDto::FacturaDto ImpresionPreFacturaFinalizadaCorrectamente(::Stratos::ModeloDto::PreFacturaDto preFacturaDto,
					long numeroFactura)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite ingresar una factura manual (que no necesita ser impresa).
			 **/
			 ::Stratos::ModeloDto::FacturaDto RegistrarFacturaManual(::Stratos::ModeloDto::FacturaDto facturaDto)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite eliminar realmente una Factura de la base de datos. Se debe utilizar sólamente
			 * en casos puntuales, como por ejemplo cuando se está sincronizando los números de factura.
			 **/
			void EliminarFactura(::Stratos::ModeloDto::FacturaDto facturaDto)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite eliminar realmente una Factura de la base de datos. Se debe utilizar sólamente
			 * en casos puntuales, como por ejemplo cuando se está sincronizando los números de factura.
			 **/
			void EliminarPreFactura(::Stratos::ModeloDto::PreFacturaDto preFacturaDto)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite anular una Factura.
			 **/
			void AnularFactura(::Stratos::ModeloDto::FacturaDto facturaDto,
							   bool crearPreFactura,
							   out ::Stratos::ModeloDto::FacturaDto facturaDtoNuevaVersion)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite desanular una Factura anulada.
			 **/
			void DesanularFactura(::Stratos::ModeloDto::FacturaDto facturaDto,
								  out ::Stratos::ModeloDto::FacturaDto facturaDtoNuevaVersion)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Ingresa una Factura inválida. Esto se utiliza únicamente para sincronizar
			 * los números de Factura en el sistema.
			 **/
			::Stratos::ModeloDto::FacturaDto
				IngresarFacturaDeSincronizacion(::Stratos::ModeloDto::FacturaTipoVenta facturaTipo,
					int codigo, long numero)
				throws ::Stratos::Common::Excepciones::StratosException;
				
			/**
			 * Obtiene todos los codigos de Facturas.
			 **/
			CodigoFacturaSeq ObtenerCodigosDeFactura()
				throws ::Stratos::Common::Excepciones::StratosException;
		};
	
	};
};

#endif