#ifndef _CONTROLADOR_CLIENTES
#define _CONTROLADOR_CLIENTES

#include <Controladores/ControladoresPersonas/ControladorPersonas.ice>
#include <Controladores/ControladorBase.ice>
#include <Documentos/DocumentoGenericoDto.ice>
#include <Excepciones/StratosException.ice>
#include <Personas/Clientes/ClienteDto.ice>
#include <Personas/Clientes/CuentaCorriente/DocumentoCreditoDto.ice>
#include <Personas/Clientes/CuentaCorriente/DocumentoDebitoDto.ice>
#include <Personas/TipoResponsable.ice>
#include <QueryObjects/ClienteDtoQuery.ice>

module StratosServer {
	module Controladores {
	
		sequence<::Stratos::ModeloDto::ClienteDto> ClienteSeq;
		sequence<::Stratos::ModeloDto::TipoResponsable> TipoResponsableSeq;
		sequence<::Stratos::ModeloDto::DocumentoGenericoDto> DocumentoGenericoSeq;
		sequence<::Stratos::ModeloDto::DocumentoCreditoDto> DocumentosCreditoSeq;
		sequence<::Stratos::ModeloDto::DocumentoDebitoDto> DocumentosDebitoSeq;
		sequence<long> IdsFacturasSeq;
		
		interface ControladorClientes extends ControladorPersonas {
			
			ClienteSeq ObtenerClienteConsumidorFinalGenerico()
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Marca como eliminado el cliente pasado como argumento. Un cliente nunca se
			 * elimina fisicamente de la base de datos, utilizar este metodo para realizar
			 * la 'eliminación' de un cliente. Un cliente marcado como eliminado no se
			 * devuelve en las busquedas.
			 **/
			void MarcarComoEliminado (::Stratos::ModeloDto::ClienteDto clienteAEliminar)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			DocumentosDebitoSeq ObtenerDocumentosDebito(long idCliente)
				throws ::Stratos::Common::Excepciones::StratosException;
				
			DocumentosCreditoSeq ObtenerDocumentosCredito(long idCliente)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			void Imputar(DocumentosCreditoSeq documentosCredito, DocumentosDebitoSeq documentosDebito)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Obtiene todos los movimientos de un cliente dado, en un rango de fechas
			 * especificado. Se debe especificar el id (código) de un cliente y el rango
			 * de fechas.
			 * El parámetro out 'resultadoRecortado' indica si el resultado ha sido
			 * recortado debido a que la consulta retorna demasiadas filas.
			 **/
			DocumentoGenericoSeq ObtenerMovimientos(long idCliente, long fechaDesdeTicks,
					long fechaHastaTicks, out bool resultadoRecortado)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite pasar un conjunto de facturas a la cuenta corriente de los
			 * clientes correspondientes.
			 **/
			void PasarFacturasACuentaCorriente(IdsFacturasSeq idsFacturas)
				throws ::Stratos::Common::Excepciones::StratosException;
			
			/**
			 * Permite pasar un cliente marcado como eliminado a no eliminado
			 **/
			::Stratos::ModeloDto::ClienteDto RecuperarClienteEliminado(long codigo)
				throws ::Stratos::Common::Excepciones::StratosException;			
		};
	
	};
};

#endif