#ifndef _QUERY_OBJECT
#define _QUERY_OBJECT

#include <Personas/Clientes/ClienteDto.ice>

module Stratos {
	module QueryObjects {
		
		enum TipoOperador {
			Igual,
			LikeMatchAnywhere
		};
		
		/**
		 * Permite definir un CriterioGenerico, que puede ser un CriterioSimple o un CriterioComplejo.
		 **/
		class CriterioGenerico {
		};
		
		/**
		 * Un CriterioSimple es aquel que permite especificar un tipo de operador (mayor, igual,
		 * menor, etc), una propiedad de una clase de negocio en la que se aplicará, y un valor
		 * y su tipo. Por ejemplo "Fecha > 10/09/2009"
		 **/
		class CriterioSimple extends CriterioGenerico {
			/**
			 * Un operador puede ser: =, <, >, Like, etc.
			 **/
			TipoOperador operador;
			
			/**
			 * La propiedad de una clase de Negocio.
			 **/
			string propiedad;
			
			/**
			 * El tipo de valor (se coloca el nombre del tipo, por ejemplo TipoResponsable) y
			 * el valor propiamente dicho.
			 **/
			string tipoValor;
			
			string valor;
		};
		
		["clr:generic:List"]
		sequence<CriterioGenerico> CriterioGenericoSeq;
		
		/**
		 * Un CriterioComplejo puede tener varios CriteriosSimple o CriterioComplejo. Por ejemplo
		 * un CriterioComplejo puede ser un CriterioAnd o CriterioOr. Todos los criterios
		 * pertenecientes a CriterioAnd deberán cumplirse para que se acepte un objeto en la consulta,
		 * y sólo alguno de ellos para un CriterioOr.
		 **/
		class CriterioComplejo extends CriterioGenerico {
			CriterioGenericoSeq criterios;
		};
		
		class CriterioAnd extends CriterioComplejo {
		};
		
		class CriterioOr extends CriterioComplejo {
		};
		
		["clr:generic:List"]
		sequence<CriterioComplejo> CriterioComplejoSeq;
		
		/**
		 * Esta es la clase principal, de la cual derivan otras según el tipo de objeto
		 * que se quiere consultar a la base de datos.
		 **/
		class QueryObject {
			CriterioComplejoSeq listaCriteriosComplejos;
		};
		
	};
};

#endif