// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using AutoMapper;
using Castle.ActiveRecord;
using NHibernate;

using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos
{
	public class ColeccionMapper<TItemDto,TItem>
		where TItemDto : DocumentoIdDto
		where TItem : IItemGenerico, new()
	{
		#region Campos
		protected IList<TItemDto> coleccionDto;
		protected IList<TItem> coleccion;
		#endregion
		
		#region Constructores
		public ColeccionMapper(IList<TItemDto> coleccionDto, IList<TItem> coleccion)
		{
			this.coleccionDto = coleccionDto;
			this.coleccion = coleccion;
		}
		#endregion
		
		#region Métodos
		public void Mapear()
		{
			this.coleccion.Clear();
			
			// Me aseguro que la colección Dto no sea nula antes de iterar
			if (this.coleccionDto != null) {
				TItem item;
				
				foreach(TItemDto itemDto in this.coleccionDto) {
					item = ActiveRecordBase<TItem>.TryFind(itemDto.id);
					
					if (item == null)
						item = new TItem();
					
					item = this.MapearItemInterno(itemDto, item);
					
					this.coleccion.Add(item);
				}
			}
		}
		
		protected virtual TItem MapearItemInterno(TItemDto itemDto, TItem item)
		{
			return Mapper.Map<TItemDto,TItem>(itemDto, item);
		}
		#endregion
	}
}
