// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Castle.ActiveRecord;
using Castle.Components.Validator;

namespace Stratos.ModeloObjetos
{
	[ActiveRecord]
	public class Cheque : ActiveRecordValidationBase<Cheque>, IActiveRecordValidationBase
	{
		#region Campos
		protected long id;
		
		protected string banco;
		protected string sucursal;
		protected double numero;
		protected DateTime fechaEmision;
		protected DateTime fechaVencimiento;
		protected string observacion;
		protected decimal importe;
		#endregion
		
		#region Constructores
		public Cheque ()
		{
		}
		#endregion
		
		#region Propiedades
		[PrimaryKey(typeof(TableHiLoGeneratorWithMultipleColumns))]
		public long Id {
			get { return this.id; }
			set { this.id = value; }
		}
		
		[Property(NotNull=true)]
		[ValidateNonEmpty("El cheque debe tener el banco asignado")]
		public string Banco {
			get { return this.banco; }
			set { this.banco = value; }
		}
		
		[Property(NotNull=true)]
		[ValidateNonEmpty("El cheque debe tener la sucursal del banco asignada")]
		public string Sucursal {
			get { return this.sucursal; }
			set { this.sucursal = value; }
		}
		
		[Property(NotNull=true)]
		public double Numero {
			get { return this.numero; }
			set { this.numero = value; }
		}
		
		[Property(NotNull=true)]
		public DateTime FechaEmision {
			get { return this.fechaEmision; }
			set { this.fechaEmision = value; }
		}
		
		[Property(NotNull=true)]
		public DateTime FechaVencimiento {
			get { return this.fechaVencimiento; }
			set { this.fechaVencimiento = value; }
		}
		
		[Property(NotNull=false)]
		public string Observacion {
			get { return this.observacion; }
			set { this.observacion = value; }
		}
		
		[Property(NotNull=true)]
		public decimal Importe {
			get { return this.importe; }
			set { this.importe = value; }
		}
		#endregion
		
		#region Métodos de validación
		[ValidateSelf]
		public void ValidarNumero(ErrorSummary errorSummary)
		{
			if (this.numero <= 0.00)
				errorSummary.RegisterErrorMessage("Numero",
				                                  "El número del cheque no puede ser menor o igual a cero");
		}
		
		[ValidateSelf]
		public void ValidarFechas(ErrorSummary errorSummary)
		{
			// Valido la fecha de emisión
			if (this.fechaEmision == DateTime.MinValue || this.fechaEmision == DateTime.MaxValue)
				errorSummary.RegisterErrorMessage("FechaEmision",
				                                  "La fecha de emisión del cheque debe ser asignada");
			
			// Valido la fecha de vencimiento
			if (this.fechaVencimiento == DateTime.MinValue || this.fechaVencimiento == DateTime.MaxValue)
				errorSummary.RegisterErrorMessage("FechaVencimiento",
				                                  "La fecha de vencimiento del cheque debe ser asignada");
			
			// La fecha de vencimiento no puede ser menor a la fecha de emisión
			if (this.fechaVencimiento < this.fechaEmision)
				errorSummary.RegisterErrorMessage("FechaVencimiento",
				                                  "La fecha de vencimiento no puede ser anterior " +
				                                  "a la fecha de emisión");
		}
		
		[ValidateSelf]
		public void ValidarImporte(ErrorSummary errorSummary)
		{
			if (this.importe <= 0.00m)
				errorSummary.RegisterErrorMessage("Importe",
				                                  "El importe del cheque no puede ser menor o igual a cero");
		}
		#endregion
	}
}
