// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Castle.ActiveRecord;
using Castle.Components.Validator;

using Stratos.Common;

namespace Stratos.ModeloObjetos
{
	public abstract class DocumentoCompraventa<T> : DocumentoGenerico<T>, IDocumentoCompraventa
		where T : class
	{
		#region Constructores
		public DocumentoCompraventa () : base()
		{
		}
		#endregion
		
		#region Propiedades
		public abstract IList ItemsGenericos {
			get; set;
		}
		
		public abstract int LimiteItems {
			get;
		}
		
		/// <value>
		/// Esta propiedad debe ser implementada por las subclases y devolver el
		/// importe final del documento. Por ejemplo, para una Factura el ImporteFinal
		/// será el valor de ImporteTotal. Para un presupuesto será ImporteNeto. Es el
		/// monto total del documento.
		/// </value>
		public abstract decimal ImporteFinal {
			get;
		}
		#endregion
		
		#region Métodos validadores
		/// <summary>
		/// Valida que la cantidad de items en el DocumentoGenerico sea correcta.
		/// </summary>
		[ValidateSelf]
		public virtual void ValidarCantidadItems(ErrorSummary errorSummary)
		{
			// Verifico que la cantidad de items no sea cero
			/* FIXME: Ojo con usar RegisterErrorMessage con una propiedad que no está mapeada.
			 * En este caso estoy suponiendo que todas las subclases de DocumentoGenerico tienen
			 * una propiedad Items. Esto suena raro, pero no lo pude solucionar. */
			if (this.ItemsGenericos.Count == 0)
				errorSummary.RegisterErrorMessage("Items",
				                                  "El documento debe poseer por lo menos un item de producto");
			
			// Verifico que la cantidad de items no sea mayor al limite establecido
			if (this.ItemsGenericos.Count > this.LimiteItems)
				errorSummary.RegisterErrorMessage("Items",
				                                  "El documento no puede tener mas de " +
				                                  this.LimiteItems +
				                                  " items de producto.");
		}
		#endregion
	}
}
