// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Castle.ActiveRecord;
using Castle.Components.Validator;

using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos
{
	/// <summary>
	/// Documento genérico que pose datos de la fecha, el cliente, los items de
	/// producto y los importes. Cualquier documento que tenga en común estos datos
	/// puede heredar de esta clase. Ejemplos de subclases son Factura, PreFactura
	/// y Presupuesto.
	/// No genera una tabla en la base de datos.
	/// </summary>
	public abstract class DocumentoGenerico<T> : ActiveRecordValidationBase<T>, IDocumentoGenerico, IActiveRecordValidationBase
		where T : class
	{
		#region Campos
		private long id;
		private DateTime fecha;
		
		private int version;
		
		private IPersona persona;
		
		private string observaciones;
		#endregion
		
		#region Constructores
		public DocumentoGenerico()
		{
		}
		#endregion
		
		#region Propiedades
		/// <summary>
		/// Esta propiedad representa la clave primaria de la entidad. Si la misma es otra,
		/// entonces sería conveniente sobreescribirla devolviendo dicho valor.
		/// </summary>
		public virtual long Id {
			get { return this.id; }
			set { this.id = value; }
		}
		
		public virtual DateTime Fecha {
			get { return this.fecha; }
			set { this.fecha = value; }
		}
		
		[Version("Version")]
		public virtual int Version {
			get { return this.version; }
			set { this.version = value; }
		}
		
		[Property(NotNull=false)]
		[ValidateLength(0, 300,
		                "Las observaciones no pueden tener mas de 300 caracteres")]
		public virtual string Observaciones {
			get { return this.observaciones; }
			set { this.observaciones = value; }
		}
		
		public virtual IPersona Persona {
			get { return this.persona; }
			set { this.persona = value; }
		}
		#endregion
	}
}
