// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using NHibernate.Criterion;
using Castle.ActiveRecord;
using Castle.Components.Validator;

using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos
{
	[ActiveRecord(DiscriminatorValue="1")]
	public class NotaCredito : Factura, INotaCredito
	{
		#region Campos
		private Factura facturaAsociada;
		#endregion
		
		#region Constructores
		public NotaCredito () : base()
		{
		}
		
		public NotaCredito (Factura facturaAsociada) : this()
		{
			this.facturaAsociada = facturaAsociada;
			
			// La nota de crédito comparte muchos datos de la factura:
			this.Cliente = this.facturaAsociada.Cliente;
			this.Codigo = this.facturaAsociada.Codigo;
			this.CondicionVenta = new CondicionVentaFacturaGenerica(CondicionPago.Contado, 0);
			this.FacturaManual = this.facturaAsociada.FacturaManual;
			this.Tipo = this.facturaAsociada.Tipo;
		}
		#endregion
		
		#region Propiedades
		[BelongsTo("facturaAsociadaId",Lazy=FetchWhen.Immediate,NotNull=false,Cascade=CascadeEnum.None)]
		public Factura FacturaAsociada {
			get { return this.facturaAsociada; }
			set { this.facturaAsociada = value; }
		}
		
		public override string TipoAbreviado {
			get { return "NC"; }
		}
		#endregion
		
		#region Métodos de validación
		[ValidateSelf]
		public void ValidarNotaCredito(ErrorSummary errorSummary)
		{
			// Solo valido si estoy en modo normal. Si estoy en un proceso de migración
			// salteo las validaciones por ahora.
			if (migrandoDatos)
				return;
			
			NotaCreditoValidador.ValidarNotaCredito(errorSummary, this);
		}
		#endregion
		
		#region Eventos internos
		protected override void OnFacturaAnulada ()
		{
			DocumentoCredito.EliminarCredito(this);
		}
		
		protected override void OnFacturaDesanulada ()
		{
			DocumentoCredito.CrearCredito(this);
		}
		
		protected override void OnFacturaCreada ()
		{
			DocumentoCredito.CrearCredito(this);
		}
		
		protected override void OnFacturaEliminada ()
		{
			DocumentoCredito.EliminarCredito(this);
		}
		#endregion
		
		#region Métodos ActiveRecord
		public new static void DeleteAll()
		{
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
		}
		
		public new static NotaCredito[] FindAll()
		{
			return ActiveRecordValidationBase<NotaCredito>.FindAll();
		}
		
		public new static NotaCredito[] FindAll(params ICriterion[] criteria)
		{
			return ActiveRecordValidationBase<NotaCredito>.FindAll(criteria);
		}
		
		public static NotaCredito Find(long id)
		{
			return ActiveRecordValidationBase<NotaCredito>.Find(id);
		}
		#endregion
	}
}
