// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using NHibernate.Criterion;
using Castle.ActiveRecord;
using Castle.Components.Validator;

using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos
{
	[ActiveRecord(DiscriminatorValue="1")]
	public class PreNotaCredito : PreFactura, INotaCredito
	{
		#region Campos
		private Factura facturaAsociada;
		#endregion
		
		#region Constructores
		public PreNotaCredito () : base()
		{
		}
		
		public PreNotaCredito (Factura facturaAsociada) : this()
		{
			this.facturaAsociada = facturaAsociada;
			
			// La nota de crédito comparte muchos datos de la factura:
			this.Cliente = this.facturaAsociada.Cliente;
			this.CondicionVenta = new CondicionVentaFacturaGenerica(CondicionPago.Contado, 0);
			this.Tipo = this.facturaAsociada.Tipo;
		}
		#endregion
		
		#region Propiedades
		[BelongsTo("facturaAsociadaId",NotNull=false,Cascade=CascadeEnum.None)]
		public Factura FacturaAsociada {
			get { return this.facturaAsociada; }
			set { this.facturaAsociada = value; }
		}
		#endregion
		
		#region Métodos de validación
		[ValidateSelf]
		public void ValidarNotaCredito(ErrorSummary errorSummary)
		{
			NotaCreditoValidador.ValidarPreNotaCredito(errorSummary, this);
		}
		#endregion
		
		#region Métodos ActiveRecord
		public new static void DeleteAll()
		{
			ActiveRecordValidationBase<PreNotaCredito>.DeleteAll();
		}
		
		public new static PreNotaCredito[] FindAll()
		{
			return ActiveRecordValidationBase<PreNotaCredito>.FindAll();
		}
		
		public static PreNotaCredito Find(long id)
		{
			return ActiveRecordValidationBase<PreNotaCredito>.Find(id);
		}
		
		public static PreNotaCredito TryFind(long id)
		{
			return ActiveRecordValidationBase<PreNotaCredito>.TryFind(id);
		}
		#endregion
	}
}
