// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Castle.ActiveRecord;
using Castle.Components.Validator;

using Stratos.Common;
using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos
{
	[ActiveRecord]
	public class ItemRecibo : ItemGenerico<ItemRecibo>
	{
		#region Campos
		protected MedioPago medioPago;
		
		// Para el caso de que sea un cheque
		protected Cheque cheque;
		
		protected decimal importe;
		#endregion
		
		#region Constructores
		public ItemRecibo()
		{
			// Por defecto el medio de pago es Efectivo
			this.medioPago = MedioPago.Efectivo;
			this.importe = 0.00m;
		}
		
		/// <summary>
		/// Constructor para MedioPago Efectivo.
		/// </summary>
		/// <param name="importe">
		/// A <see cref="System.Decimal"/>
		/// </param>
		public ItemRecibo(decimal importe) : this()
		{
			// Por defecto el medio de pago es Efectivo
			this.MedioPago = MedioPago.Efectivo;
			this.Importe = importe;
		}
		
		/// <summary>
		/// Constructor para MedioPago Cheque. El importe del item es igual al importe del cheque.
		/// </summary>
		/// <param name="cheque">
		/// A <see cref="Cheque"/>
		/// </param>
		public ItemRecibo(Cheque cheque) : this()
		{
			this.MedioPago = MedioPago.Cheque;
			this.Cheque = cheque;
			this.Importe = this.cheque.Importe;
		}
		#endregion
		
		#region Propiedades
		[PrimaryKey(typeof(TableHiLoGeneratorWithMultipleColumns))]
		public override long Id {
			get { return base.Id; }
			set { base.Id = value; }
		}
		
		[Property(NotNull=true)]
		public MedioPago MedioPago {
			get { return this.medioPago; }
			set { this.medioPago = value; }
		}
		
		[BelongsTo("chequeId",NotNull=false,Cascade=CascadeEnum.SaveUpdate)]
		public Cheque Cheque {
			get { return this.cheque; }
			set { this.cheque = value; }
		}
		
		[Property(NotNull=true)]
		public decimal Importe {
			get { return this.importe; }
			set { this.importe = StratosUtil.RoundDecimal(value, 2); }
		}
		#endregion
		
		#region Métodos de validación
		#endregion
	}
}
