// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

using Castle.Components.Validator;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Framework;
using Castle.ActiveRecord.Queries;
using NHibernate.Criterion;

using Stratos.ModeloDto;
using Stratos.ModeloObjetos.MetaCampos;

namespace Stratos.ModeloObjetos
{
	[ActiveRecord]
	public class Cliente : Persona<Cliente>, IExtensibleConMetaCampos<MetaCampoCliente>
	{
		#region Campos
		private bool valido;
		
		private IList<MetaCampoCliente> metaCampos;
		
		private decimal porcentajeBonificacion;
		#endregion
    			
		#region Constructores
		public Cliente() : base ()
		{
			this.valido = true;
			this.metaCampos = new List<MetaCampoCliente>();
		}
		#endregion
		
		#region Propiedades
		/* La siguiente línea comentada es utilizada para la migración de los clientes desde
		 * el sistema viejo.*/
//		[PrimaryKey(PrimaryKeyType.Assigned, UnsavedValue="0")]
		[PrimaryKey(PrimaryKeyType.Native, SequenceName="cliente_sequence", UnsavedValue="0")]
		public override long Codigo {
			get { return base.Codigo; }
			set { base.Codigo = value; }
		}
		
		[Property(NotNull=true)]
    	public bool Valido {
    		get { return this.valido; }
    		set { this.valido = value; }
    	}

		[Property(NotNull=false)]
		[ValidateRange(RangeValidationType.Decimal, 0, 50,
		               "El porcentaje de bonificación es inválida. No puede ser superior al 50%")]
    	public decimal PorcentajeBonificacion {
    		get { return this.porcentajeBonificacion; }
    		set { this.porcentajeBonificacion = value; }
    	}
		
		public override bool InstanciaValida {
			get { return this.Valido; }
		}
		
		[HasMany(typeof(MetaCampoCliente),
		         Table="MetaCampoCliente",
		         ColumnKey="clienteId",
		         Inverse=true,
		         Cascade=ManyRelationCascadeEnum.AllDeleteOrphan)]
		public IList<MetaCampoCliente> MetaCampos {
			get { return this.metaCampos; }
			set { this.metaCampos = value; }
		}
		
		public void SetearMetaCampo(string clave, string valor)
		{
			MetaCampoCliente metaCampo = new MetaCampoCliente()
			{
				Clave = clave,
				Valor = valor,
				Cliente = this
			};
			
			int indice = this.metaCampos.IndexOf(metaCampo);
			
			if (indice < 0) {
				this.metaCampos.Add(metaCampo);
				return;
			}
			
			this.metaCampos[indice].Valor = valor;
		}
		
		public string ObtenerMetaCampo(string clave)
		{
			MetaCampoCliente metaCampo = new MetaCampoCliente()
			{
				Clave = clave
			};
			
			int indice = this.metaCampos.IndexOf(metaCampo);
			
			if (indice < 0)
				return null;
			
			return this.metaCampos[indice].Valor;
		}
		
		public void EliminarMetaCampo(string clave)
		{
			MetaCampoCliente metaCampo = new MetaCampoCliente()
			{
				Clave = clave
			};
			
			int indice = this.metaCampos.IndexOf(metaCampo);
			
			if (indice < 0)
				return;
			
			metaCampo = this.metaCampos[indice];
			this.metaCampos.RemoveAt(indice);
		}
		#endregion
	}
}
