// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using NHibernate.Type;
using NHibernate.Dialect;

namespace Stratos.ModeloObjetos
{
	public class TableHiLoGeneratorWithMultipleColumns : NHibernate.Id.TableHiLoGenerator
	{
		static HashSet<string> tables = new HashSet<string>();
		public override void Configure(IType type, IDictionary<string, string> parms, Dialect dialect)
		{
			string table;
			if (parms.ContainsKey("target_table"))
			{
				table = parms["target_table"];
				tables.Add(table);
				parms["column"] = string.Format("{0}_{1}", DefaultColumnName, table);
			}
			base.Configure(type, parms, dialect);
		}
		
		public override string[] SqlCreateStrings(Dialect dialect)
		{
			string createTableTemplate = "create table " + DefaultTableName + "({0})";
			
			string insertInitialValuesTemplate = "insert into " + DefaultTableName + "({0})" + " values ( {1} )";
			
			StringBuilder createTables = new StringBuilder();
			StringBuilder columns = new StringBuilder();
			StringBuilder inserts = new StringBuilder();
			StringBuilder initialInsert = new StringBuilder();
			StringBuilder insertsValues = new StringBuilder();
			foreach (string table in tables)
			{
				columns.AppendFormat("{0}_{1} {2},", DefaultColumnName, table, dialect.GetTypeName(columnSqlType));
				inserts.AppendFormat("{0}_{1},", DefaultColumnName, table);
				insertsValues.Append("1, ");
			}
			columns.Remove(columns.Length - 1, 1);
			inserts.Remove(inserts.Length - 1, 1);
			createTables.AppendFormat(createTableTemplate, columns);
			insertsValues.Remove(insertsValues.Length - 2, 2);
			initialInsert.AppendFormat(insertInitialValuesTemplate, inserts, insertsValues);
			
			return new[] { createTables.ToString(), initialInsert.ToString() };
		}
	}
}
