// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using Gtk;

using Stratos.Common.Excepciones;
using StratosClient.Core.Gui;
using StratosClient.Ventas;
using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace StratosClient.Clientes.CuentaCorriente.Avanzado
{
	public class AccionadorFacturas : IAccionadorSobreElemento
	{
		#region Campos
		private FacturaDto[] facturas;
		private TreeIter[] treeIters;
		private TreeModel modelo;
		
		private Gtk.Action pasarCuentaCorrienteAction;
		#endregion
		
		#region Constructores
		public AccionadorFacturas()
		{
			this.pasarCuentaCorrienteAction =
				new Gtk.Action("pasarcuentacorriente",
				               "Pasar a cuenta corriente",
				               "Pasa una factura a cuenta corriente",
				               "gtk-copy");
			
			this.pasarCuentaCorrienteAction.Activated += this.OnPasarCuentaCorrienteActivated;
		}
		#endregion
		
		#region Acciones
		protected virtual void OnPasarCuentaCorrienteActivated(object sender, EventArgs e)
		{
			// Primero pregunto si el usuario está seguro de la operación
			ResponseType respuesta =
				SMessageDialog
					.MensajePregunta("¿Seguro que desea pasar las facturas seleccionadas a " +
						"la cuenta corriente del cliente?");
			
			if (respuesta != ResponseType.Yes)
				return;
			
			// Obtengo el controlador de clientes
			ControladorClientesPrx controladorClientes =
				((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"));
			
			// Selecciono los ids de las facturas seleccionadas
			var idsFacturas =
				from facturaSeleccionada in this.facturas
				select facturaSeleccionada.id;
			
			// Paso las facturas a cuenta corriente
			try {
				controladorClientes.PasarFacturasACuentaCorriente(idsFacturas.ToArray());
				
				SMessageDialog
					.MensajeInformacion("Las facturas seleccionadas fueron pasadas a la cuenta " +
						"corriente del cliente correspondiente a cada una");
			}
			catch (ParametrosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
			}
		}
		#endregion
		
		#region IAccionadorSobreElemento implementation
		public void ActualizarAcciones (TreeIter[] iters, DocumentoGenericoDto[] documentos, TreeModel modelo)
		{
			this.treeIters = iters;
			this.facturas = documentos.Cast<FacturaDto>().ToArray();
			this.modelo = modelo;
			
			// Busco las facturas anuladas seleccionadas (si es que las hay)
			var facturasAnuladas =
				from factura in this.facturas
				where factura.anulada
				select factura;
			
			// Activo la acción de pasar a cuenta corriente únicamente si
			// hay factura seleccionadas y si ninguna está anulada.
			this.pasarCuentaCorrienteAction.Sensitive =
				this.facturas.Length > 0 && facturasAnuladas.Count() == 0;
			
			// Si ya no esta desactivada, activo la accion solo si ninguna
			// de los documentos seleccionados es una Nota de Credito.
			if (this.pasarCuentaCorrienteAction.Sensitive)
			{
				var notasDeCredito =
					from nc in this.facturas
					where (nc is NotaCreditoDto)
					select nc;
				
				this.pasarCuentaCorrienteAction.Sensitive =
					this.facturas.Length > 0 && notasDeCredito.Count() == 0;
			}
		}
		
		
		public Gtk.Action[] Acciones {
			get { return new Gtk.Action[] { this.pasarCuentaCorrienteAction }; }
		}
		#endregion
	}
}
