// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Widgets.Level2;

namespace StratosClient.Clientes.CuentaCorriente
{
	public partial class CuentaCorrienteVista : Gtk.Bin, ICuentaCorrienteVista
	{
		#region Campos
		protected TvDocumentosCredito tvDocumentosCredito;
		protected TvDocumentosDebito tvDocumentosDebito;
		
		protected CuentaCorrientePresenter cuentaCorrientePresenter;
		#endregion
		
		#region Constructores
		public CuentaCorrienteVista ()
		{
			this.Build ();
			
			// Documentos de crédito
			this.tvDocumentosCredito = new TvDocumentosCredito();
			this.tvDocumentosCredito.tvcSeleccionado.Visible = false;
			this.swRecibosExistentes.Add(this.tvDocumentosCredito);
			this.tvDocumentosCredito.ShowAll();
			
			// Documentos de débito
			this.tvDocumentosDebito = new TvDocumentosDebito();
			this.tvDocumentosDebito.tvcSeleccionado.Visible = false;
			this.swFacturasAsociadas.Add(this.tvDocumentosDebito);
			this.tvDocumentosDebito.ShowAll();
			
			/* Me suscribo a los eventos de conexión y desconexión de la terminal.
			 * Si hay una conexión nueva, tengo que actualizar los controladores (proxies) pasados
			 * al Presenter. */
			MainIce.ConexionEstablecida += this.CargarControladores;
			
			this.cuentaCorrientePresenter = new CuentaCorrientePresenter(this, null);
			
			// Cargo los controladores en el Presenter
			this.CargarControladores();
		}
		#endregion
		
		#region IDeudasVista - Implementacion
		#region Valores
		public bool EsPosibleActualizar {
			set { this.btnActualizar.Sensitive = value; }
		}
		
		public bool EsPosibleImprimir {
			set { this.btnImprimir.Sensitive = value; }
		}
		
		public double ImporteFavor {
			get { return this.txtImporteFavor.ImporteDouble; }
			set { this.txtImporteFavor.ImporteDouble = value; }
		}
		
		public double ImporteContra {
			get { return this.txtImporteContra.ImporteDouble; }
			set { this.txtImporteContra.ImporteDouble = value; }
		}
		
		public double SaldoTotal {
			get { return this.txtSaldoFinal.ImporteDouble; }
			set {
				this.txtSaldoFinal.ImporteDouble = value;
				
				Gdk.Color colorFondo;
				
				// Si el saldo es negativo, coloco un color de fondo adecuado
				if (value < 0.00) {
					colorFondo = Gdk.Color.Zero;
					Gdk.Color.Parse("#f43c3c", ref colorFondo);
					this.txtSaldoFinal.ModifyBase(StateType.Normal, colorFondo);
				}
				// Si es positivo...
				else if (value > 0.00) {
					colorFondo = Gdk.Color.Zero;
					Gdk.Color.Parse("#66FF33", ref colorFondo);
					this.txtSaldoFinal.ModifyBase(StateType.Normal, colorFondo);
				}
				// Si es cero lo pongo en su color normal
				else
					this.txtSaldoFinal.ModifyBase(StateType.Normal);
			}
		}
		#endregion
		
		#region Componentes gráficos
		public IRazonSocialVista<ClienteDto> RazonSocialVista {
			get { return this.razonSocial; }
		}
		
		public IDocumentoCreditoVista DocumentosCredito {
			get { return this.tvDocumentosCredito; }
		}
		
		public IDocumentoDebitoVista DocumentosDebito {
			get { return this.tvDocumentosDebito; }
		}
		#endregion
		#endregion
		
		#region Eventos
		protected virtual void OnActualizarDatosClicked (object sender, System.EventArgs e)
		{
			this.cuentaCorrientePresenter.ActualizarDatos();
		}
		
		protected virtual void OnImprimirClicked (object sender, System.EventArgs e)
		{
			this.cuentaCorrientePresenter.Imprimir();
		}
		#endregion
		
		#region Otros métodos
		public void CargarControladores()
		{
			// Obtengo los controladores necesarios para inyectarlos en el Presenter
			ControladorClientesPrx controladorClientes = (ControladorClientesPrx)
				MainIce.GetControlador("ControladorClientes");
			
			this.cuentaCorrientePresenter.ControladorClientes = controladorClientes;
		}
		#endregion
	}
}
