// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Widgets.Level2;
using StratosClient.Widgets.Level1;

namespace StratosClient.Clientes.Movimientos
{
	public partial class MovimientosVista : Gtk.Bin, IMovimientosVista
	{
		#region Campos
		private TvMovimientos tvMovimientos;
		private MovimientosPresenter movimientosPresenter;
		#endregion
		
		#region Constructores
		public MovimientosVista ()
		{
			this.Build ();
			
			this.tvMovimientos = new TvMovimientos();
			this.swMovimientos.Add(this.tvMovimientos);
			this.tvMovimientos.ShowAll();
			
			/* Me suscribo a los eventos de conexión y desconexión de la terminal.
			 * Si hay una conexión nueva, tengo que actualizar los controladores (proxies) pasados
			 * al Presenter. */
			MainIce.ConexionEstablecida += this.CargarControladores;
			
			this.movimientosPresenter = new MovimientosPresenter(this, null);
			
			// Cargo los controladores en el Presenter
			this.CargarControladores();
		}
		#endregion
		
		#region IMovimientosVista - Implementación
		#region Notificaciones
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		#endregion
		
		#region Valores
		public bool EsPosibleImprimir {
			set { this.btnImprimir.Sensitive = value; }
		}
		#endregion
		
		#region Componentes
		public IRazonSocialVista<ClienteDto> RazonSocialVista {
			get { return this.razonSocial; }
		}
		
		public IDateComboVista FechaDesde {
			get { return this.dcFechaDesde; }
		}
		
		public IDateComboVista FechaHasta {
			get { return this.dcFechaHasta; }
		}
		
		public IListaElementosVista<DocumentoGenericoDto> ListaMovimientosVista {
			get { return this.tvMovimientos; }
		}
		#endregion
		#endregion
		
		#region Eventos
		protected virtual void OnCargarClicked (object sender, System.EventArgs e)
		{
			this.movimientosPresenter.CargarMovimientos();
		}
		
		protected virtual void OnImprimirClicked (object sender, System.EventArgs e)
		{
			this.movimientosPresenter.ImprimirMovimientos();
		}
		#endregion
		
		#region Otros métodos
		public void CargarControladores()
		{
			// Obtengo los controladores necesarios para inyectarlos en el Presenter
			ControladorClientesPrx controladorClientes = (ControladorClientesPrx)
				MainIce.GetControlador("ControladorClientes");
			
			this.movimientosPresenter.ControladorClientes = controladorClientes;
		}
		#endregion
	}
}
