// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;
using StratosClient.Core.Gui;
using StratosServer.Controladores;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level1;
using Stratos.QueryObjects;
using System.Collections.Generic;

namespace StratosClient.Clientes
{
	public partial class SolapaAgregarModificar : Gtk.Bin, IVisualizadorNavegable
	{
		#region Campos
		private static SolapaAgregarModificar instancia;
		
		private ComboBoxEnumeracion<TipoResponsable> cmbTipoResponsable;
		private ComboBoxEnumeracion<CondicionPago> cmbCondVentaPorDefecto;
		
		private AdministradorEstadoObjeto administradorEstado;
		private ClienteDto clienteCargado;
		#endregion
		
		#region Constructores
		public SolapaAgregarModificar()
		{
			this.Build();
			
			instancia = this;
			
			#region Combo TipoResponsable
			this.cmbTipoResponsable = new ComboBoxEnumeracion<TipoResponsable>();
			this.cmbTipoResponsable.Renderizador = delegate (TipoResponsable tipoResponsable) {
				return StratosUtil.SplitCamelString(tipoResponsable.ToString());
			};
			this.hbTipoIva.Add(this.cmbTipoResponsable);
			#endregion
			
			#region Combo Condicion de venta por defecto
			this.cmbCondVentaPorDefecto = new ComboBoxEnumeracion<CondicionPago>();
			this.cmbCondVentaPorDefecto.Renderizador = delegate (CondicionPago condPago) {
				return StratosUtil.SplitCamelString(condPago.ToString());
			};
			this.hbCondVentaPorDefecto.Add(this.cmbCondVentaPorDefecto);
			#endregion
			
			// Desactivo el textbox que muestra el código
			this.txtCodigo.Sensitive = false;
			
			#region Suscripción a eventos
			this.cmbTipoResponsable.Changed += OnCmbTipoResponsableChanged;
			#endregion
			
			#region Administrador del estado
			// Creo el objeto administrador del estado
			this.administradorEstado =
				new AdministradorEstadoObjeto(estado => this.lblEstadoCliente.Markup = estado,
				                              x => this.btnGuardar.Sensitive = x);
			
			// Desactivo el botón 'Verificar existencia' del cuit cuando el estado es 'Cargado' o 'Guardado'.
			this.administradorEstado.EstadoCambiado += delegate(EstadoObjeto estado) {
				if (estado == EstadoObjeto.Cargado || estado == EstadoObjeto.Guardado)
					this.btnRecuperarEliminado.Sensitive = false;
				else
					this.btnRecuperarEliminado.Sensitive = true;
			};
			
			this.txtRazonSocial.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbTipoResponsable.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbCondVentaPorDefecto.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.rtxtCuit.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.domicilio.Cambiado += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtTelefono.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.rtxtEmail.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtBonificacion.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtObservaciones.Buffer.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			#endregion
			
			#region NavegadorElementos
			// Seteo el proveedor de la instancia que implementa
			// la interface IListaNavegable
			this.navegadorElementos.ProveedorListaNavegable = delegate {
				return SolapaClientes.Instancia;
			};
			
			// Seteo el cargador
			this.navegadorElementos.Cargador = delegate(object o) {
				this.CargarCliente(o);
				
				// Si despues de cargar el estado es 'Cargado', retorno true
				if (this.administradorEstado.EstaCargado)
					return true;
				
				return false;
			};
			#endregion
			
			this.VaciarFormulario();
		}
		#endregion
		
		#region Propiedades
		public static SolapaAgregarModificar InstanciaActual {
			get { return instancia; }
		}
		
		public AdministradorEstadoObjeto AdministradorEstado {
			get { return this.administradorEstado; }
		}
		
		public bool ClienteEnModificacion {
			get {
				if (this.clienteCargado != null)
					return true;
				
				return false;
			}
		}
		
		public INavegadorElementosVista NavegadorElementos {
			get { return this.navegadorElementos; }
		}
		#endregion
		
		#region Eventos
		protected virtual void OnBtnGuardarClicked(object sender, System.EventArgs e)
		{
			/* Valido los datos ingresados en el formulario. Si se lanza alguna execepción
			 * es porque hay campos inválidos. Muestro los mensajes y retorno */
			try {
				this.ValidarDatosIngresados();
			}
			catch(DatosIngresadosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.Message);
				return;
			}
			
			// Si no hay un cliente cargado, creo uno nuevo
			if (this.clienteCargado == null)
				this.clienteCargado = new ClienteDto();
			
			// Cargo el objeto con los datos ingresado en el formulario
			this.LeerDatosIngresados(ref this.clienteCargado);
			
			ClienteDto clientePersistido;
			
			// Obtengo el controlador de clientes
			ControladorClientesPrx controladorClientes = 
				((ControladorClientesPrx)MainIce.GetControlador("ControladorClientes"));
			
			// Si el cliente es nuevo, llamo al metodo Guardar
			if (this.clienteCargado.codigo <= 0) {
				try {
					clientePersistido = (ClienteDto)
						controladorClientes.Guardar(this.clienteCargado);
					
					// Cargo el cliente guardado en la interfaz gráfica y cambio el estado a guardado
					this.CargarCliente(clientePersistido, true);
					
					// Desabilito el navegador de elementos, ya que este es un producto
					// nuevo y no tiene nada que ver con los posiblemente cargados en la lista
					this.navegadorElementos.Sensitive = false;
					
					this.administradorEstado
						.CambiarEstadoObjeto(EstadoObjeto.Guardado);
					
					// Informo al usuario
					SMessageDialog
						.MensajeInformacion("El cliente se ha ingresado correctamente al " +
						                    "sistema. El código asignado a este cliente es " +
						                    clientePersistido.codigo.ToString() + ".");
				}
				catch (ObjetoInvalidoException ex) {
				  	SMessageDialog.MensajeError("El cliente tiene datos inválidos:\n" + ex.mensaje);
					return;
				}
			}
			
			// Si el cliente ya existe, es porque está en modificación. Llamo al metodo 'Modificar'.
			else {
				try {
					clientePersistido = (ClienteDto)
						controladorClientes.Modificar(this.clienteCargado);
					
					// Cargo el cliente guardado en la interfaz gráfica y cambio el estado a guardado
					this.CargarCliente(clientePersistido, true);
					
					this.administradorEstado
						.CambiarEstadoObjeto(EstadoObjeto.Guardado);
				}
				catch (ObjetoInvalidoException ex) {
				  	SMessageDialog
						.MensajeError("El cliente tiene datos inválidos:\n" +
						              ex.mensaje);
					return;
				}
				catch (ObjetoVersionIncorrectaException ex) {
				  	SMessageDialog
						.MensajeError("<b>El cliente que intenta modificar fue " +
							"modificado ya por otra terminal</b>.\nVuelva a cargar el " +
							"cliente actual y realice nuevamente las modificaciones");
					return;
				}
				catch (ObjetoInexistenteException ex) {
				  	SMessageDialog
						.MensajeError("El cliente que intenta modificar ya no existe " +
							"en la base de datos.");
					return;
				}
			}
		}
		
		protected virtual void OnBtnNuevoClicked(object sender, System.EventArgs e)
		{
			// Si hay modificaciones hechas, pregunto al usuario si está seguro de perderlas
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.VaciarFormulario();
		}
		
		/// <summary>
		/// Este método se ejecuta cuando se dispara el evento Changed del ComboBox TipoResponsable
		/// y se encarga de activar o desactivar el campo que permite ingresar el cuit. Si el
		/// elemento seleccionado es 'Consumidor final' o 'Exento' el campo cuit se desactiva.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnCmbTipoResponsableChanged(object sender, System.EventArgs e)
		{
			if (this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.ConsumidorFinal)
				this.rtxtCuit.Sensitive = false;
			else
				this.rtxtCuit.Sensitive = true;
		}
		
		protected virtual void OnBtnRecuperarEliminadoClicked (object sender, System.EventArgs e)
		{
			if (!rtxtCuit.TextoValido || string.IsNullOrEmpty(rtxtCuit.Text))
				return;
			
			// Obtengo el controlador de clientes
			ControladorClientesPrx controladorClientes = 
				((ControladorClientesPrx)MainIce.GetControlador("ControladorClientes"));
			
			CriterioSimple cs = new CriterioSimple();
			cs.operador = TipoOperador.Igual;
			cs.propiedad = "Cuit.ValorNumerico";
			cs.valor = rtxtCuit.Text.Trim().Replace("-", "") + "L";
			
			CriterioOr criterioCuit = new CriterioOr();
			criterioCuit.criterios = new List<CriterioGenerico>();
			criterioCuit.criterios.Add(cs);
			
			ClienteDtoQuery consulta = new ClienteDtoQuery();
			consulta.listaCriteriosComplejos = new List<CriterioComplejo>();
			consulta.listaCriteriosComplejos.Add(criterioCuit);
			
			ConsultaPaginable cp = new ConsultaPaginable();
			cp.pagina = 0;
			cp.tamanioPagina = 10;
			cp.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Cuit.ValorNumerico",
					sentidoOrd = SentidoOrdenacion.Asc
				}
			};
			
			Ice.Object[] resultado = controladorClientes.BuscarQueryObjects(consulta, cp);
			
			// No existe ningun cliente que concuerde con la busqueda
			if (resultado == null || resultado.Length == 0)
			{
				SMessageDialog.MensajeInformacion("El Cuit ingresado no corresponde a ningun cliente existente.");
				return;
			}
			
			ClienteDto cliente = (ClienteDto)resultado[0];
			
			Gtk.ResponseType respuesta;
			
			// Existe el cliente y no está eliminado, pregunto si desea cargarlo
			if (!cliente.eliminado)
			{
				respuesta = SMessageDialog.MensajePregunta("El Cuit ingresado corresponde al\ncliente <b>" + 
				                               cliente.razonSocial + "</b>.\n" +
				                               "¿Desea cargarlo?");
				
				if (respuesta == ResponseType.Yes)
					this.CargarClienteInterno(cliente);
				
				return;
			}
			
			// Existe el cliente pero está eliminado, pregunto si quiere recuperar el cliente
			respuesta = SMessageDialog.MensajePregunta("El Cuit ingresado corresponde al cliente <b>" + 
				                               cliente.razonSocial + "</b>.\nEste cliente actualmente se " +
				                               "encuentra <b>eliminado</b>. ¿Desea recuperarlo?");
			
			// Si la respuesta es 'Si', recupero el cliente
			if (respuesta == ResponseType.Yes)
			{
				ClienteDto clienteRecuperado = controladorClientes.RecuperarClienteEliminado(cliente.codigo);
				this.CargarClienteInterno(clienteRecuperado);
				SMessageDialog.MensajeInformacion("El cliente ha sido recuperado exitosamente");
			}
		}
		
		#endregion
		
		#region Métodos Públicos
		#region Carga de cliente
		public void CargarCliente(object objeto)
		{
			ClienteDto cliente = (ClienteDto) objeto;
			this.CargarCliente(cliente);
		}
		
		public void CargarCliente(ClienteDto cliente, bool evitarPreguntaPorModificaciones)
		{
			if (!evitarPreguntaPorModificaciones &&
			    	(this.administradorEstado.EstaModificado &&
			     		!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones))
				return;
			
			this.CargarClienteInterno(cliente);
		}
		
		public void CargarCliente(ClienteDto cliente)
		{
			// Primero me fijo si no hay modificaciones sin guardar
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.CargarClienteInterno(cliente);
		}
		
		protected void CargarClienteInterno(ClienteDto cliente)
		{
			this.VaciarFormulario();
			
			// Seteo la instancia de cliente que se está modificando
			this.clienteCargado = cliente;
			
			// Lleno los campos
			
			// Código
			this.txtCodigo.Text = cliente.codigo.ToString();
			
			// Razón social
			this.txtRazonSocial.Text = cliente.razonSocial;
			
			// Tipo responsable
			this.cmbTipoResponsable.ObjetoActivo = cliente.tipoRes;
			
			// Cuit
			this.rtxtCuit.Text = cliente.cuit == null ? "" : cliente.cuit;
			
			// Domicilio
			if (cliente.domicilio == null)
				cliente.domicilio = new DomicilioDto();
			
			this.domicilio.Domicilio = cliente.domicilio;
			
			// Contacto
			this.txtTelefono.Text = cliente.telefono;
			this.rtxtEmail.Text = cliente.correoElectronico;
			
			// Otros datos
			this.txtBonificacion.Text = cliente.porcentajeBonificacion.ToString("N2");
			
			// Condicion de venta por defecto
			if ((cliente.condicionPagoPorDefecto != null) && (cliente.condicionPagoPorDefecto.Length > 0))
				this.cmbCondVentaPorDefecto.ObjetoActivo = cliente.condicionPagoPorDefecto[0];
			
			// Observaciones
			this.txtObservaciones.Buffer.Text = cliente.notas;
			
			// Activo el navegador de elementos
			this.navegadorElementos.Sensitive = true;
			
			// Cambio el estado de la interfaz gráfica
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado);
		}
		#endregion
		#endregion
		
		#region Métodos Privados
		/// <summary>
		/// Crea un objeto ClienteDto con los datos ingresados en el formulario
		/// </summary>
		/// <returns>
		/// A <see cref="ClienteDto"/>
		/// </returns>
		protected void LeerDatosIngresados(ref ClienteDto cliente)
		{
			// Razon social y tipo de responsable
			cliente.razonSocial = this.txtRazonSocial.Text.ToUpper();
			cliente.tipoRes = this.cmbTipoResponsable.ObjetoActivo;

			// Si es Consumidor final o Exento, no corresponde un cuit
			if (cliente.tipoRes != TipoResponsable.ConsumidorFinal)
				cliente.cuit = this.rtxtCuit.Text;
			else
				cliente.cuit = "";
			
			// Datos del domicilio
			cliente.domicilio = this.domicilio.Domicilio;
			
			// Datos de contacto
			cliente.telefono = this.txtTelefono.Text;
			cliente.correoElectronico = this.rtxtEmail.Text;
			
			// Si no es vacía, parseo la bonificación
			if (!this.txtBonificacion.Text.Equals(string.Empty))
				cliente.porcentajeBonificacion =
					StratosUtil.ParseDouble(this.txtBonificacion.Text, 2);
			else
				cliente.porcentajeBonificacion = 0.0d;
			
			/* Condición de venta por defecto:
			 * solo cargo si el valor es distinto a 'Contado'. */
			if (this.cmbCondVentaPorDefecto.ObjetoActivo != CondicionPago.Contado)
				cliente.condicionPagoPorDefecto = new CondicionPago[] {
					this.cmbCondVentaPorDefecto.ObjetoActivo
				};
			else
				cliente.condicionPagoPorDefecto = null;
			
			// Observaciones
			cliente.notas = this.txtObservaciones.Buffer.Text;
		}
		
		protected void ValidarDatosIngresados()
		{
			// Razón social
			if (this.txtRazonSocial.Text.Equals(string.Empty))
			    throw new DatosIngresadosInvalidosException("Razón social invalida. No puede ser vacía.");
			
			// Tipo de responsable
			if (this.cmbTipoResponsable.TextoActivo.Equals(""))
				throw new DatosIngresadosInvalidosException("Tipo IVA no seleccionado. " +
							"Debe seleccionar uno.");
			
			// CUIT
			if (!this.rtxtCuit.TextoValido)
				throw new DatosIngresadosInvalidosException(this.rtxtCuit.MensajeError);
			
			// Domicilio
			string mensajeErrorDomicilio = String.Empty;
			
			if (!this.domicilio.DatosValidos(out mensajeErrorDomicilio))
				throw new DatosIngresadosInvalidosException(mensajeErrorDomicilio);
			
			// E-mail
			if (!this.rtxtEmail.TextoValido)
				throw new DatosIngresadosInvalidosException(this.rtxtEmail.MensajeError);
			
			// Telefono
			if (!this.txtTelefono.TextoValido)
				throw new DatosIngresadosInvalidosException(this.txtTelefono.MensajeError);
			
			// Bonificación
			if (!this.txtBonificacion.TextoValido)
				throw new DatosIngresadosInvalidosException(this.txtBonificacion.MensajeError);
		}
		
		/// <summary>
		/// Vacía el formulario que permite Ingresar/Modificar los
		/// datos de un cliente.
		/// </summary>
		protected void VaciarFormulario()
		{
			this.txtCodigo.Text = "";
			
			this.txtRazonSocial.Text = "";
			
			this.cmbTipoResponsable.Resetear();
			this.rtxtCuit.Text = "";
			
			this.domicilio.Vaciar();
			
			this.txtTelefono.Text = "";
			this.rtxtEmail.Text = "";
			this.txtBonificacion.Text = "0";
			
			this.txtObservaciones.Buffer.Text = "";
			this.cmbCondVentaPorDefecto.Resetear();
			
			// Desactivo el navegador de elementos
			this.navegadorElementos.Sensitive = false;
			
			// Reseteo el cliente cargado
			this.clienteCargado = null;
			
			// El estado de la interfaz es NUEVO
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo);
		}
		#endregion
		
	}
}
