// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections.Generic;

using Gtk;
using Mono.Addins;

using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;
using Stratos.QueryObjects;
using StratosClient.Core.Gui;
using StratosServer.Controladores;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level2;

namespace StratosClient.Clientes
{
	public partial class SolapaClientes : Gtk.Bin, IListaNavegable
	{
		#region Campos
		private static SolapaClientes instancia;
		
		private const int TAMAÑO_PAGINA = 30;
		
		private SolapaAgregarModificar solapaModificar;
		
		private TvClientes tvClientes;
		
		private ClienteDtoQuery consultaActual;
		
		private TreeIter iterCargado;
		private int paginaIterCargado;
		
		private ServicioNavegadorElementos servicioNavegador;
		#endregion
		
		#region Constructores
		public SolapaClientes()
		{
			this.Build();
			
			#region Lista de clientes
			this.tvClientes = new TvClientes();
			this.scrolledwindow1.Add(this.tvClientes);
			this.tvClientes.ShowAll();
			#endregion
			
			#region Navegador de páginas
			this.navegadorPaginas.TamañoPagina = TAMAÑO_PAGINA;
			this.navegadorPaginas.ColeccionElementos = this.tvClientes;
			this.navegadorPaginas.CargadorDatos = this.CargadorClientes;
			this.navegadorPaginas.CriterioOrdenacion =
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				};
			#endregion
			
			#region Solapa Agregar-Modificar
			// Creo una instancia de "Agregar-Modificar"
			this.solapaModificar = new SolapaAgregarModificar();
			
			// Agrego la pestaña "Agregar-Modificar"
			Gtk.Label etiqueta = new Label("Agregar-Modificar");
			this.nbSolapaClientes.AppendPage(this.solapaModificar, etiqueta);
			
			// Hago que el navegador de elementos de la solapa Agregar-Modificar
			// se desactive cuando se vacía la lista.
			this.ListaElementosVaciada += delegate {
				this.solapaModificar.NavegadorElementos.Sensitive = false;
			};
			#endregion
			
			#region Suscripción a eventos
			this.rtxtCodigoBusqueda.Activated += OnBtnBuscarClicked;
			this.rtxtCuitBusqueda.Activated += this.OnBtnBuscarClicked;
			this.txtRazonSocialBusqueda.Activated += OnBtnBuscarClicked;
			
		 	this.tvClientes.Selection.Changed += OnTvClientesSelectionChanged;
			this.tvClientes.RowActivated += OnBtnModificarClicked;
			
			// Evento para disparar el menú desplegable
			this.tvClientes.ButtonReleaseEvent += this.OnButtonReleaseEvent;		
			#endregion
			
			#region Criterios de búsqueda
			this.chkCodigo.Toggled += delegate {
				this.rtxtCodigoBusqueda.Sensitive = this.chkCodigo.Active;
			};
			this.chkCodigo.Active = true;
			
			this.chkRazonSocial.Toggled += delegate {
				this.txtRazonSocialBusqueda.Sensitive = this.chkRazonSocial.Active;
			};
			this.chkRazonSocial.Active = true;
			
			this.chkCuit.Toggled += delegate {
				this.rtxtCuitBusqueda.Sensitive = this.chkCuit.Active;
			};
			this.chkCuit.Active = true;
			#endregion
			
			// Creo el servicio de navegacion de elementos
			this.servicioNavegador = new ServicioNavegadorElementos();
			
			// Suscribo el evento 'ListaElementosVaciada' de IListaNavegable al
			// evento 'ListaVaciada' de treeview
			this.tvClientes.ListaVaciada += delegate {
				if (this.ListaElementosVaciada != null)
					this.ListaElementosVaciada();
			};
			
			this.consultaActual = null;
			
			this.ConstruirSolapas();
			
			this.VaciarFormularioBusqueda();
			
			instancia = this;
		}
		#endregion
		
		#region Propiedades
		public static SolapaClientes Instancia {
			get { return instancia; }
		}
		#endregion
		
		#region Eventos
		protected virtual void OnBtnBuscarClicked (object sender, System.EventArgs e)
		{	
			// Chequeo que sea válido el texto ingresado, si no lo es lo notifico
			if (this.chkCodigo.Active && !this.rtxtCodigoBusqueda.TextoValido) {
				SMessageDialog.MensajeError(this.rtxtCodigoBusqueda.MensajeError);
				return;
			}
			
			if (this.chkCuit.Active && !this.rtxtCuitBusqueda.TextoValido) {
				SMessageDialog.MensajeError(this.rtxtCuitBusqueda.MensajeError);
				return;
			}
			
			this.consultaActual = this.CrearQueryObject();
			
			// Creo una consulta paginable
			ConsultaPaginable cp = new ConsultaPaginable();
			cp.pagina = 0;
			cp.tamanioPagina = TAMAÑO_PAGINA;
			cp.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				}
			};
			
			ClienteDto[] clientesEncontrados = this.EjecutarConsulta(cp);
			
			/* Si no hay clientes que cumplan con los criterios de busqueda,
			 * notifico al usuario y retorno */
			if (clientesEncontrados.Length <= 0) {
				SMessageDialog.MensajeAdvertencia("No hay clientes que cumplan con los criterios " +
				               			"de busqueda.\nReformule la busqueda e intente nuevamente.");
				return;
			}
					
			// Activo el Navegador de páginas
			this.navegadorPaginas.Sensitive = true;
			
			// Vacío el treeview
			this.tvClientes.Vaciar();
					
			// Cargo los clientes encontrados en el treeview
			this.tvClientes.AgregarElementosPrimeraPagina(clientesEncontrados);
		}

		/// <summary>
		/// Este metodo se ejecuta cuando se dispara el evento Clicked del botón 'Limpiar' del
		/// formulario de busqueda. Limpia los campos de busqueda y los resultados de busquedas
		/// anteriores.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnLimpiarClicked(object sender, System.EventArgs e)
		{
			this.VaciarFormularioBusqueda();
			
			// Desactivo los botones 'Modificar' y 'Eliminar'
			this.btnModificar.Sensitive = false;
			this.btnEliminar.Sensitive = false;
					
			// Activo los campos de busqueda y el botón 'Buscar'
			this.rtxtCodigoBusqueda.Sensitive = true;
			this.txtRazonSocialBusqueda.Sensitive = true;
			this.btnBuscar.Sensitive = true;
					
			// Desactivo el navegador de páginas
			this.navegadorPaginas.Sensitive = false;
		}
		
		[GLib.ConnectBefore]
		protected void OnButtonReleaseEvent(object o, ButtonReleaseEventArgs args)
		{
			// Solo activo el menú si es el botón derecho el presionado y el treeview no está vacío
			if (args.Event.Button == 3)
				this.MostrarMenuEmergente();
		}		
		
		protected virtual void OnBtnModificarClicked(object sender, System.EventArgs e)
		{
			TreeIter[] iters;
			TreeIter iterSeleccionado;
			
			// Chequeo que el cliente seleccionado no sea nulo
			if (this.tvClientes.ObtenerSeleccionados(out iters).Length == 0)
				return;
			
			iterSeleccionado = iters[0];
			
			// Obtengo el objeto refrescado
			ClienteDto clienteRefrescado = this.ObtenerClienteRefrescado(iterSeleccionado);
			
			// Actualizo el iter cargado
			this.iterCargado = iterSeleccionado;
			this.paginaIterCargado = this.navegadorPaginas.PaginaActual;
			
			// Actualizo el treeview con el cliente refrescado
			this.tvClientes.ActualizarObjetoSeleccionado(clienteRefrescado);
			
			// Paso a la pestaña 'Agregar/Modificar'
			this.nbSolapaClientes.CurrentPage = 1;
			
			// Cargo el cliente en el formulario de modificación
			this.solapaModificar.CargarCliente(clienteRefrescado);
		}
		
		protected virtual void OnMiModificarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnModificarClicked(sender, e);
		}

		protected virtual void OnBtnEliminarClicked(object sender, System.EventArgs e)
		{
			// Chequeo que el cliente no sea nulo
			if (this.tvClientes.ClientesSeleccionados.Length == 0)
				return;
			
			// Muestro el diálogo de confirmación
			ResponseType respuesta =
				SMessageDialog.MensajePregunta("¿Está seguro de eliminar el cliente " +
				                               this.tvClientes.ClientesSeleccionados[0].razonSocial +
				                               " del sistema?");
			
			// Si la respuesta es 'Si' borro el cliente seleccionado
			if (respuesta == ResponseType.Yes) {
				
				// Le aviso al servidor que marque el cliente como eliminado
				((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"))
					.MarcarComoEliminado(this.tvClientes.ClientesSeleccionados[0]);
				
				// Borro el item seleccionado del treeview
				this.tvClientes.EliminarObjetosSeleccionados();
			}
		}
		
		protected virtual void OnMiEliminarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnEliminarClicked(sender, e);
		}
		
		/// <summary>
		/// Este metodo se ejecuta cuando se dispara el evento Selection.Changed del TvClientes.
		/// Si se ha seleccionado una fila, se activan los botones 'Modificar' y 'Eliminar'.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnTvClientesSelectionChanged(object o, System.EventArgs e)
		{
			// Chequeo que el cliente seleccionado no sea nulo
			if (this.tvClientes.ObtenerSeleccionados().Length == 0)
				return;
			
			// Activo los botones 'Modificar' y 'Eliminar'
			this.btnModificar.Sensitive = true;
			this.btnEliminar.Sensitive = true;
		}
		#endregion
		
		#region Otros métodos
		protected void ConstruirSolapas()
		{
//			foreach(Gtk.Widget w in this.nbSolapasVentas.AllChildren)
//				nbSolapasVentas.Remove(w);
			
			var listaOrdenada =
				from tabNode in AddinManager.GetExtensionNodes("/StratosClient/Clientes/Tabs").Cast<ClientesTabNode>()
				orderby tabNode.Posicion ascending
				select tabNode;
			
			/* Agrego las solapas y almaceno el número en el diccionario, para luego poder obtener
			 * el número de solapa según el componente que almacena. */
			foreach (ClientesTabNode node in listaOrdenada)
				this.nbSolapaClientes.AppendPage(node.GetWidget(), node.Etiqueta);
			
			// Coloco la primer solapa como solapa por defecto
			this.nbSolapaClientes.CurrentPage = 0;
			
			nbSolapaClientes.ShowAll();
		}
		
		protected ClienteDto[] EjecutarConsulta(ConsultaPaginable cp)
		{
			// Realizo la búsqueda al servidor
			Ice.Object[] resultadoQueryObjects =
			((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"))
				.BuscarQueryObjects(this.consultaActual, cp);
			
			ClienteDto[] clientesEncontrados = resultadoQueryObjects.Cast<ClienteDto>().ToArray();
			
			return clientesEncontrados;
		}
		
		protected object[] CargadorClientes (ConsultaPaginable cp)
		{
			return this.EjecutarConsulta(cp);	
		}
				
		protected ClienteDtoQuery CrearQueryObject()
		{
			ClienteDtoQuery consulta = new ClienteDtoQuery();
			consulta.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioSimple temp;
			
			/* Si el campo código no es vacío, busco por código,
			 * ya que es una busqueda mas precisa */
			if (this.chkCodigo.Active && !this.rtxtCodigoBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioOr criterioCodigo = new CriterioOr();
				criterioCodigo.criterios = new List<CriterioGenerico>();
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "Codigo";
				temp.valor = this.rtxtCodigoBusqueda.Text.Trim() + "L";
				
				criterioCodigo.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioCodigo);
			}
			
			// La búsqueda por cuit también es precisa, así que si está activada
			// busco por CUIT
			if (this.chkCuit.Active && !this.rtxtCuitBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioOr criterioCuit = new CriterioOr();
				criterioCuit.criterios = new List<CriterioGenerico>();
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "Cuit.ValorNumerico";
				temp.valor = this.rtxtCuitBusqueda.Text.Trim().Replace("-", "") + "L";
				
				criterioCuit.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioCuit);
			}
			
			// Si no hay nada ingresado en el campo Código, consulto por Razon Social
			if (this.chkRazonSocial.Active && !this.txtRazonSocialBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioAnd criterioRazonSocial = new CriterioAnd();
				criterioRazonSocial.criterios = new List<CriterioGenerico>();
				
				foreach (string palabra in this.txtRazonSocialBusqueda.Text.Split(' ')) {
					temp = new CriterioSimple();
					temp.operador = TipoOperador.LikeMatchAnywhere;
					temp.propiedad = "RazonSocial";
					temp.valor = palabra.ToUpper();
					
					criterioRazonSocial.criterios.Add(temp);
				}
				
				consulta.listaCriteriosComplejos.Add(criterioRazonSocial);
			}
					
			// Agrego un criterio para que no me devuelva los clientes marcados como 'Eliminados'
			CriterioAnd criterioEliminado = new CriterioAnd();
			criterioEliminado.criterios = new List<CriterioGenerico>();
			
			temp = new CriterioSimple();
			temp.operador = TipoOperador.Igual;
			temp.propiedad = "Eliminado";
			temp.valor = bool.FalseString;
			
			criterioEliminado.criterios.Add(temp);
			consulta.listaCriteriosComplejos.Add(criterioEliminado);
		
			return consulta;
		}
		
		protected void MostrarMenuEmergente()
		{
			Menu menu = new Menu();
			ImageMenuItem menuItem;
			
			menuItem = new ImageMenuItem("Modificar");
			menuItem.Image = Image.NewFromIconName("gtk-edit", IconSize.Menu);
			menuItem.Activated += this.OnMiModificarActivated;
			menu.Append(menuItem);
			
			menuItem = new ImageMenuItem("Eliminar");
			menuItem.Image = Image.NewFromIconName("gtk-remove", IconSize.Menu);
			menuItem.Activated += this.OnMiEliminarActivated;
			menu.Append(menuItem);
			
			
			menu.Popup();
			menu.ShowAll();
		}
		
		/// <summary>
		/// Vacía el formulario de busqueda: los campos de filtrado y los
		/// resultados de la busqueda anterior.
		/// </summary>
		protected void VaciarFormularioBusqueda()
		{
			// Limpio los campos de filtrado
			this.rtxtCodigoBusqueda.Text = "";
			this.txtRazonSocialBusqueda.Text = "";
			
			this.tvClientes.Vaciar();
		}
		
		private ClienteDto ObtenerClienteRefrescado (TreeIter iter)
		{
			Ice.Object resultado = ((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"))
				.Refrescar((ClienteDto)this.tvClientes.Model.GetValue(iter, 0));
			
			return (ClienteDto) resultado;
		}
		#endregion
		
		#region IListaNavegable implementation
		public event System.Action ListaElementosVaciada;
		
		public object ObtenerObjetoRefrescado (TreeIter iter)
		{
			return this.ObtenerClienteRefrescado(iter);
		}
		
		public ServicioNavegadorElementos ServicioNavegador {
			get {
				this.servicioNavegador.ListaNavegable = this;
				return this.servicioNavegador;
			}
		}
		
		public TreeIter IterCargado {
			get { return this.iterCargado; }
			set { this.iterCargado = value; }
		}
		
		public int PaginaIterCargado {
			get { return this.paginaIterCargado; }
			set { this.paginaIterCargado = value; }
		}
		
		public ColeccionElementos ListaElementos {
			get { return this.tvClientes; }
		}

		public NavegadorPaginas NavegadorDePaginas {
			get { return this.navegadorPaginas; }
		}
		#endregion
	}
}
