// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

using Gtk;

using StratosClient.Core.Gui;
using StratosServer.Controladores;
using Stratos.ModeloDto;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level1;
using StratosClient.Widgets.Level2;

namespace StratosClient.Compras
{
	public partial class BusquedasComprasVista : Gtk.Bin, IBusquedasComprasVista
	{
		#region Campos
		protected static BusquedasComprasVista instancia;
		
		protected TvFacturasCompra tvFacturasCompra;
		
		protected BusquedasComprasPresenter presenter;
		
		protected ServicioNavegadorElementos servicioNavegador;
		
		protected TreeIter iterCargado;
		protected int paginaIterCargado;
		#endregion

		public BusquedasComprasVista ()
		{
			this.Build ();
			
			// Obtengo el controlador de compras
			ControladorComprasPrx controladorCompras = (ControladorComprasPrx)
				MainIce.GetControlador("ControladorCompras");
			
			// Creo el presentador
			this.presenter = new BusquedasComprasPresenter(this, controladorCompras);
			
			#region Campos de busqueda
			this.chkProveedor.Toggled += delegate {
				this.txtRazonSocial.Sensitive = this.chkProveedor.Active;
			};
			this.chkProveedor.Active = false;
			
			this.chkNumero.Toggled += delegate {
				this.txtNumero.Sensitive = this.chkNumero.Active;
			};
			this.chkNumero.Active = false;
			
			this.dcFechaMinima.Fecha = DateTime.Today.Date;
			this.chkFechaMinima.Toggled += delegate {
				this.dcFechaMinima.Sensitive = this.chkFechaMinima.Active;
			};
			this.chkFechaMinima.Active = false;
			
			this.dcFechaMaxima.Fecha = DateTime.Today.Date;
			this.chkFechaMaxima.Toggled += delegate {
				this.dcFechaMaxima.Sensitive = this.chkFechaMaxima.Active;
			};
			this.chkFechaMaxima.Active = false;
			#endregion
			
			#region Lista de facturas de compra
			// Creo y agrego el treeview al formulario
			this.tvFacturasCompra = new TvFacturasCompra();
			this.alignment5.Add(this.tvFacturasCompra);
			
			// Me suscribo al evento RowActivated que se lanza cuando se hace
			// doble click en una fila del trewview.
			this.tvFacturasCompra.RowActivated += delegate {
				this.OnBtnModificarClicked(null, null);
			};
			
			// Evento para disparar el menú desplegable sobre la lista de facturas
			this.tvFacturasCompra.ButtonReleaseEvent += this.OnButtonReleaseEvent;
			#endregion
			
			#region Navegador de páginas
			// Seteo las propiedades del navegador de páginas
			this.navegadorPaginas.TamañoPagina = this.presenter.TamañoPagina;
			this.navegadorPaginas.ColeccionElementos = this.tvFacturasCompra;
			this.navegadorPaginas.CargadorDatos = this.presenter.CargadorFacturas;
			this.navegadorPaginas.CriterioOrdenacion = this.presenter.CriterioDeOrdenacion;
			#endregion
			
			this.tvFacturasCompra.ListaVaciada += delegate {
				if (this.ListaElementosVaciada != null)
					this.ListaElementosVaciada();
			};
			
			// Creo el servicio de navegación de elementos
			this.servicioNavegador = new ServicioNavegadorElementos();
			
			instancia = this;
		}
		
		#region Propiedades
		public FacturasProveedoresPresenter SolapaImputacionPresenter {
			set { this.presenter.SolapaFacturasProveedores = value; }
		}
		
		public static BusquedasComprasVista Instancia {
			get { return instancia; }
		}
		#endregion
		
		#region IBusquedasComprasVista implementation
		
		#region Camponentes gráficos
		public IDateComboVista DateComboFechaInicialVista {
			get { return this.dcFechaMinima; }
		}
		
		public IDateComboVista DateComboFechaFinalVista {
			get { return this.dcFechaMaxima; }
		}
		
		public IListaElementosVista<FacturaCompraDto> ListaFacturasComprasVista {
			get { return this.tvFacturasCompra; }
		}
		
		public INavegadorPaginasVista NavegadorPaginasVista {
			get { return this.navegadorPaginas; }
		}
		#endregion
		
		#region Valores
		public bool BusquedaPorRazonSocialActiva {
			get { return this.chkProveedor.Active; }
		}
		
		public bool BusquedaPorNumeroActiva {
			get { return this.chkNumero.Active; }
		}
		
		public bool BusquedaPorFechaInicialActiva {
			get { return this.chkFechaMinima.Active; }
		}
		
		public bool BusquedaPorFechaFinalActiva {
			get { return this.chkFechaMaxima.Active;}
		}
		
		public string RazonSocial {
			get { return this.txtRazonSocial.Text; }
		}
		
		public string NumeroFactura {
			get { return this.txtNumero.Text; }
		}
		#endregion
		
		#region Métodos
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		
		public ResponseType RealizarPregunta (string texto)
		{
			return SMessageDialog.MensajePregunta(texto);
		}

		#endregion
		#endregion
				
		#region Eventos
		protected virtual void OnCriteriosBusquedaKeyRelease (object o, Gtk.KeyReleaseEventArgs args)
		{
			if (args.Event.Key == Gdk.Key.Return || args.Event.Key == Gdk.Key.KP_Enter)
				this.OnBtnBuscarClicked(null, null);
		}
		
		/// <summary>
		/// Este metodo se ejecuta cuando se dispara el evento 'Clicked'
		/// del botón 'Buscar'.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnBuscarClicked (object sender, System.EventArgs e)
		{
			// LLamo al método 'Buscar' del presenter
			this.presenter.Buscar();
		}
		
		/// <summary>
		/// Este método se ejecuta cuando se dispara el evento 'Clicked'
		/// del botón 'Modificar'.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnModificarClicked (object sender, System.EventArgs e)
		{
			TreeIter[] iters;
			
			// Si no hay elementos seleccionados, retorno.
			if (this.tvFacturasCompra.ObtenerSeleccionados(out iters).Length == 0)
				return;
			
			// Seteo el primero de los iters seleccionados como el iter cargado actualmente
			this.iterCargado = iters[0];
			
			// Guardo la página del iter cargado
			this.paginaIterCargado = this.navegadorPaginas.PaginaActual;
			
			// Llamo al método 'CargarFacturaSeleccionada' del presenter.
			this.presenter.CargarFacturaSeleccionada();
		}
		
		/// <summary>
		/// Método que se ejecuta cuando se dispara el evento 'Clicked' del
		/// botón Eliminar.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnEliminarClicked (object sender, System.EventArgs e)
		{
			// Llamo al método 'EliminarFacturasSeleccionadas' del presenter
			this.presenter.EliminarFacturasSeleccionadas();
		}
		
		[GLib.ConnectBefore]
		protected void OnButtonReleaseEvent(object o, ButtonReleaseEventArgs args)
		{
			// Solo activo el menú si es el botón derecho el presionado y el treeview no está vacío
			if (args.Event.Button == 3)
				this.MostrarMenuEmergente();
		}
		#endregion
		
		#region Métodos privados
		
		/// <summary>
		/// Muestra el menú emergente en la lista de facturas de compras
		/// </summary>
		protected void MostrarMenuEmergente()
		{
			// Creo el menú contextual
			Menu menu = new Menu();
			ImageMenuItem menuItem;
			
			// Creo el menu item 'Modificar', me suscribo al evento Activated y lo agrego al menú
			menuItem = new ImageMenuItem("Modificar");
			menuItem.Image = Image.NewFromIconName("gtk-edit", IconSize.Menu);
			menuItem.Activated += delegate { this.presenter.CargarFacturaSeleccionada(); };
			menu.Append(menuItem);
			
			// Creo el menu item 'Eliminar', me suscribo al evento Activated y lo agrego al menú
			menuItem = new ImageMenuItem("Eliminar");
			menuItem.Image = Image.NewFromIconName("gtk-remove", IconSize.Menu);
			menuItem.Activated += delegate { this.presenter.EliminarFacturasSeleccionadas(); };
			menu.Append(menuItem);

			// Muestro el menú
			menu.Popup();
			menu.ShowAll();
		}
		#endregion
		
		#region IListaNavegable implementation
		public Gtk.TreeIter IterCargado {
			get { return this.iterCargado; }
			set { this.iterCargado = value; }
		}
		
		public ColeccionElementos ListaElementos {
			get { return this.tvFacturasCompra; }
		}
		
		public event System.Action ListaElementosVaciada;
		
		public NavegadorPaginas NavegadorDePaginas {
			get { return this.navegadorPaginas; }
		}
		
		public object ObtenerObjetoRefrescado (Gtk.TreeIter iter)
		{
			FacturaCompraDto facturaSeleccionada = 
				(FacturaCompraDto)this.tvFacturasCompra.Model.GetValue(iter, 0);
			
			return this.presenter.ObtenerObjetoRefrescado(facturaSeleccionada);
		}
		
		public int PaginaIterCargado { 
			get { return this.paginaIterCargado; }
			set { this.paginaIterCargado = value; }
		}
		
		public ServicioNavegadorElementos ServicioNavegador {
			get {
				this.servicioNavegador.ListaNavegable = this;
				return this.servicioNavegador;
			}
		}
		#endregion
	}
}
