// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;
using Stratos.Common;
using StratosServer.Controladores;
using StratosClient.Core.Gui;
using StratosClient.Widgets.Level1;
using StratosClient.Widgets.Level2;
using StratosClient.Widgets.Level3;

namespace StratosClient.Compras
{
	public partial class FacturasProveedoresVista : Gtk.Bin, IFacturasProveedoresVista
	{
		protected ComboBoxEnumeracion<FacturaTipoCompra> cmbFacturaTipo;
		protected ComboBoxEnumeracion<TipoResponsable> cmbTipoResponsable;
		
		protected ListaItemsCompras tvItemsProducto;
		private ItemsInteractivo itemsProducto;
		
		protected FacturasProveedoresPresenter presenter;
		
		public FacturasProveedoresVista ()
		{
			this.Build ();
					
			#region Comboboxes
			// Solo creo y agrego los comboboxes a la interfaz, el resto de
			// la inicialización la realiza el presenter.
			
			// Combobox Tipo Factura
			this.cmbFacturaTipo = new ComboBoxEnumeracion<FacturaTipoCompra>();
			this.cmbFacturaTipo.Renderizador = delegate (FacturaTipoCompra facturaTipo) {
				return StratosUtil.SplitCamelString(facturaTipo.ToString());
			};
			this.hbTipo.Add(this.cmbFacturaTipo);
			
			// Combobox Tipo de responsable
			this.cmbTipoResponsable = new ComboBoxEnumeracion<TipoResponsable>();
			this.cmbTipoResponsable.Renderizador = delegate (TipoResponsable tipoResponsable) {
				return StratosUtil.SplitCamelString(tipoResponsable.ToString());
			};
			this.hbTipoIva.Add(this.cmbTipoResponsable);
			
			// Me suscribo al evento Changed del combobox tipo responsable y cambio
			// al tipo de factura que corresponde a cada tipo de responsabilidad del
			// proveedor.
			this.cmbTipoResponsable.Changed += delegate {
				if (this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.ResponsableInscripto)
					this.cmbFacturaTipo.ObjetoActivo = FacturaTipoCompra.FacturaA;
				
				else if (this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.Monotributo)
					this.cmbFacturaTipo.ObjetoActivo = FacturaTipoCompra.FacturaC;
			};
			#endregion
			
			this.txtCodigo.Xalign = 1.0f;
			this.txtNumero.Xalign = 1.0f;
			
			#region Lista de items
			this.tvItemsProducto = new ListaItemsCompras();
			this.itemsProducto = new ItemsInteractivo(this.tvItemsProducto);
			this.hbItems.PackStart(this.itemsProducto, true, true, 0);
			#endregion
			
			// Obtengo el controlador de compras
			ControladorComprasPrx controladorCompras = (ControladorComprasPrx)
				MainIce.GetControlador("ControladorCompras");
			
			// Creo el presenter
			this.presenter = new FacturasProveedoresPresenter(this, controladorCompras);
			
			#region Suscripción a eventos de cambio
			// Fechas
			this.dcFechaImputacion.FechaCambiada += delegate { this.presenter.DatosModificados(false); };
			this.dcFechaFactura.FechaCambiada += delegate { this.presenter.DatosModificados(false); };
			
			// Campos de la cabecera de la factura
			this.cmbFacturaTipo.Changed += delegate { this.presenter.DatosModificados(false); };
			this.txtCodigo.Changed += delegate { this.presenter.DatosModificados(false); };
			this.txtNumero.Changed += delegate { this.presenter.DatosModificados(false); };
			
			// Campos del proveedor
			this.razonSocial.RazonSocialEntry.Changed += delegate { this.presenter.DatosModificados(false); };
			this.cmbTipoResponsable.Changed += delegate { this.presenter.DatosModificados(false); };
			this.rtxtCuit.Changed += delegate { this.presenter.DatosModificados(false); };
			this.domicilio.Cambiado += delegate { this.presenter.DatosModificados(false); };
			
			// Lista de items
			this.tvItemsProducto.ItemsCambiados += delegate { this.presenter.DatosModificados(false); };
			
			// Campos de importes
			this.txtGravado.Changed += delegate { this.presenter.DatosModificados(true); };
			this.txtExento.Changed += delegate { this.presenter.DatosModificados(true); };
			this.txtImporteIva.Changed += delegate { this.presenter.DatosModificados(true); };
			this.txtPercepcionIva.Changed += delegate { this.presenter.DatosModificados(true); };
			this.txtPercepcionIB.Changed += delegate { this.presenter.DatosModificados(true); };
			this.txtImpCombustible.Changed += delegate { this.presenter.DatosModificados(true); };
			
			// Observaciones
			this.txtObservaciones.Buffer.Changed += delegate { this.presenter.DatosModificados(false); };
			#endregion
			
			this.navegadorElementos.Sensitive = false;
			
			// Evento de persona cambiada
			this.razonSocial.PersonaCargada += delegate {
				this.tvItemsProducto.GrabFocus();
			};
		}
		
		#region Propiedades
		public FacturasProveedoresPresenter Presenter {
			get { return this.presenter; }
		}
		
		public bool SoloLectura {
			set {
				this.dcFechaImputacion.Sensitive = !value;
				
				this.dcFechaFactura.Sensitive = !value;
				CustomWidgetsUtils.WidgetEditable(this.cmbFacturaTipo, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtCodigo, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtNumero, !value);
				
				this.razonSocial.BotonLimpiar.Sensitive = !value;
				CustomWidgetsUtils.WidgetEditable(this.rtxtCuit, !value);
				
				this.itemsProducto.Sensitive = !value;
				
				CustomWidgetsUtils.WidgetEditable(this.txtGravado, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtExento, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtImporteIva, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtPercepcionIB, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtPercepcionIva, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtImpCombustible, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtImporteTotal, !value);
				
				CustomWidgetsUtils.WidgetEditable(this.txtObservaciones, !value);
			}
		}
		#endregion
		
		#region Implementacion de IImputacionCompraVista
		
		#region Componentes gráficos
		public IDateComboVista DateComboFechaImputacionVista {
			get { return this.dcFechaImputacion; }
		}
		
		public IDateComboVista DateComboFechaFacturaVista {
			get { return this.dcFechaFactura; }
		}
		
		public IRazonSocialVista<ProveedorDto> RazonSocialVista {
			get { return this.razonSocial; }
		}
		
		public IDomicilioEntry DomicilioEntryVista {
			get { return this.domicilio; }
		}
		
		public IComboBoxGenericoVista<FacturaTipoCompra> ComboBoxTipoFacturaVista {
			get { return this.cmbFacturaTipo; }
		}
		
		public IComboBoxGenericoVista<TipoResponsable> ComboBoxTipoResponsableVista {
			get { return this.cmbTipoResponsable; }
		}
		
		public IRegExEntryVista RegExEntryCuitVista {
			get { return this.rtxtCuit; }
		}
		
		public IListaInteractivaVista ListaItems {
			get { return this.tvItemsProducto; }
		}
		
		public INavegadorElementosVista NavegadorElementos {
			get { return this.navegadorElementos; }
		}
		
		public IImporteRegExEntry ImporteExentoVista {
			get { return this.txtExento; }
		}
		
		public IImporteRegExEntry ImporteGravadoVista {
			get { return this.txtGravado;}
		}
		
		public IImporteRegExEntry ImpCombustibleVista {
			get { return this.txtImpCombustible; }
		}
		
		public IImporteRegExEntry ImporteIvaVista {
			get { return this.txtImporteIva; }
		}
		
		public IImporteRegExEntry ImporteTotalVista {
			get { return this.txtImporteTotal; }
		}
		
		public IImporteRegExEntry PercepcionIBVista {
			get { return this.txtPercepcionIB; }
		}
		
		public IImporteRegExEntry PercepcionIvaVista {
			get { return this.txtPercepcionIva; }
		}
		
		public IRegExEntryVista RegExEntryCodigoVista {
			get { return this.txtCodigo; }
		}
		
		public IRegExEntryVista RegExEntryNumeroVista {
			get { return this.txtNumero; }
		}
		
		public IEntryRazonSocial RazonSocialProveedor {
			get { return this.razonSocial.RazonSocialEntry; }
		}
		#endregion
		
		#region Valores
		public string EstadoFactura {
			set { this.lblEstadoFactura.Markup = value; }
		}
		
		public string CuitVista {
			get { return this.rtxtCuit.Text; }
			set { this.rtxtCuit.Text = value; }
		}
		
		public bool EsPosibleGuardarFactura {
			set { this.btnGuardar.Sensitive = value; }
		}
		
		public string ObservacionesVista {
			get { return this.txtObservaciones.Buffer.Text; }
			set { this.txtObservaciones.Buffer.Text = value; }
		}
		#endregion
		
		#endregion
		
		#region Eventos
		protected virtual void OnBtnNuevoClicked (object sender, System.EventArgs e)
		{
			this.presenter.NuevaFactura();
		}
		
		protected virtual void OnBtnGuardarClicked (object sender, System.EventArgs e)
		{
			this.presenter.Guardar();
		}
		#endregion
		
		#region Implementacion de INotificador
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		#endregion		
	}
}
