// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;
using StratosServer.Controladores;
using StratosClient.Core.Gui;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level1;
using StratosClient.Widgets.Level2;

namespace StratosClient.Productos
{
	public partial class SolapaAgregarModificar : Gtk.Bin, IVisualizadorNavegable
	{
		#region Campos
		private static SolapaAgregarModificar instancia;
		
		private ComboBoxGenerico<RubroDto> cmbRubro;
		private ComboBoxGenerico<RubroDto> cmbSubRubro;
		private ComboBoxEnumeracion<UnidadProducto> cmbUnidadProducto;
		
		private AdministradorEstadoObjeto administradorEstado;
		ProductoDto productoCargado;
		
		private ControladorProductosPrx controladorProductos;
		#endregion
	
		public SolapaAgregarModificar()
		{
			this.Build();
			
			instancia = this;
			
			#region Combo Unidad
			this.cmbUnidadProducto = new ComboBoxEnumeracion<UnidadProducto>();
			this.cmbUnidadProducto.Renderizador = delegate (UnidadProducto unidadProducto) {
				return StratosUtil.SplitCamelString(unidadProducto.ToString());
			};
			this.hbUnidad.PackEnd(this.cmbUnidadProducto, true, true, 0);
			#endregion
			
			#region Combo Sub-rubro
			this.cmbSubRubro = new ComboBoxGenerico<RubroDto>(new RubroDtoEqualityComparer());
			this.cmbSubRubro.Renderizador = delegate (RubroDto unSubRubro) {
				return unSubRubro.nombre;
			};
			this.hbSubRubro.PackEnd(this.cmbSubRubro, true, true, 0);
			#endregion
			
			#region Combo Rubro
			this.cmbRubro = new ComboBoxGenerico<RubroDto>(new RubroDtoEqualityComparer());
			this.cmbRubro.Renderizador = delegate (RubroDto unRubro) {
				return unRubro.nombre;
			};
			this.hbRubro.PackEnd(this.cmbRubro, true, true, 0);
			#endregion
			
			#region Suscripción a eventos
			this.chkEspecificarGanancia.Toggled += OnChkEspecificarGananciaToggled;
			this.cmbRubro.Changed += OnCmbRubroChanged;
			
			// Eventos que requieren recalcular importes
			this.txtPrecioCosto.Changed += CalcularPrecioVenta;
			this.spnGanancia.Changed += CalcularPrecioVenta;
			this.txtPrecioVenta.Changed += CalcularPrecioFinal;
			this.txtIva.Changed += CalcularPrecioFinal;
			
			// Botón 'Verificar disponibilidad'
			this.btnCodigoDisponible.Clicked += this.OnBtnCodigoDisponibleClicked;
			
			// Botón 'Nuevo'
			this.btnNuevo.Clicked += this.OnBtnNuevoClicked;
			
			// Botón Guardar
			this.btnGuardar.Clicked += OnBtnGuardarClicked;
			
			// Entry 'Codigo'
			this.txtCodigo.Changed += this.OnTxtCodigoChanged;
			#endregion
			
			#region Administrador del estado del producto
			// Creo el objeto administrador del estado del producto
			this.administradorEstado =
				new AdministradorEstadoObjeto(estado => this.lblEstadoProducto.Markup = estado,
				                              x => this.btnGuardar.Sensitive = x);
			
			this.txtCodigo.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtDescripcion.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbRubro.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbSubRubro.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.chkProductoAbstracto.Toggled += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbFabricante.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbUnidadProducto.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
//			this.chkControlStock.Toggled += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtPrecioCosto.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtPrecioVenta.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtIva.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtPrecioFinal.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtObservaciones.Buffer.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			// Me suscribo al evento 'EstadoCambiado'
			this.administradorEstado.EstadoCambiado += this.OnEstadoProductoCambiado;
			#endregion
			
			#region NavegadorElementos
			// Seteo el proveedor de la instancia que implementa
			// la interface IListaNavegable
			this.navegadorElementos.ProveedorListaNavegable = delegate {
				return SolapaProductos.Instancia;
			};
			
			// Seteo el cargador
			this.navegadorElementos.Cargador = delegate(object o) {
				this.CargarProducto(o);
				
				// Si despues de cargar el estado es 'Cargado', retorno true
				if (this.administradorEstado.EstaCargado)
					return true;
				
				return false;
			};
			#endregion
			
			// Cargo el controlador de productos para luego poder cargar los rubros.
			this.controladorProductos =
					(ControladorProductosPrx) MainIce.GetControlador("ControladorProductos");
			
			this.InicializarComboBoxRubro();
			
			// Además me suscribo al evento de ConexionEstablecida para reinicializar
			// el ComboBox de rubros cuando el cliente se conecta
			MainIce.ConexionEstablecida += delegate {
				// Seteo el controlador
				this.controladorProductos =
					(ControladorProductosPrx) MainIce.GetControlador("ControladorProductos");
				
				this.InicializarComboBoxRubro();
			};
			
			// Desactivo los campos que por ahora no tienen funcionalidad
			this.fraProveedores.Sensitive = false;
			this.hbFabricante.Sensitive = false;
			
			this.productoCargado = null;
			this.VaciarFormulario();
		}
		
		#region Propiedades
		public static SolapaAgregarModificar InstanciaActual {
			get { return instancia; }
		}
		
		public AdministradorEstadoObjeto AdministradorEstado {
			get { return this.administradorEstado; }
		}
		
		public bool ProductoEnModificacion {
			get {
				if (this.productoCargado != null)
					return true;
				
				return false;
			}
		}
		#endregion
		
		#region IVisualizadorNavegable implementation
		public INavegadorElementosVista NavegadorElementos {
			get { return this.navegadorElementos; }
		}
		#endregion

		#region Eventos comunes
		
		protected virtual void OnCmbRubroChanged (object sender, System.EventArgs e)
		{
			this.InicializarComboBoxSubRubro(this.cmbRubro.ObjetoActivo);
		}
		
		#region Eventos que requieren recalcular los precios
		protected virtual void OnChkEspecificarGananciaToggled (object sender, System.EventArgs e)
		{
			if (this.chkEspecificarGanancia.Active)
			{
				this.spnGanancia.Sensitive = true;
				CustomWidgetsUtils.WidgetEditable(this.txtPrecioVenta, false);
				
				this.CalcularPrecioVenta(sender, e);
			}
			
			else {
				this.spnGanancia.Sensitive = false;
				CustomWidgetsUtils.WidgetEditable(this.txtPrecioVenta, true);
			}
		}
		
		protected virtual void CalcularPrecioVenta(object sender, System.EventArgs e)
		{
			if (!chkEspecificarGanancia.Active)
				return;
			
			if (!this.txtPrecioCosto.TextoValido ||
			    	String.IsNullOrEmpty(this.txtPrecioCosto.Text))
				return;
			
			/* Calculo el importe de venta a partir del precio de costo y el porcentaje de ganancia
			 * especificado. El 'Precio final' se va a calcular solo, ya que al cambiar 
			 * 'Precio de Venta' emite el evento Changed cuando es asignado.*/
			this.txtPrecioVenta.Importe =
				this.txtPrecioCosto.Importe * (1 + ((decimal)this.spnGanancia.Value/100));
			
			// Hago que el Widget no sea editable (FIXME: hay que volverlo a setearlo, no debería ser así)
			CustomWidgetsUtils.WidgetEditable(this.txtPrecioVenta, false);
		}
		
		protected virtual void CalcularPrecioFinal(object sender, System.EventArgs e)
		{
			// Necesito que los valores sean válidos...
			if (!this.txtPrecioVenta.TextoValido ||
			    	!this.txtIva.TextoValido)
				return;
			
			// También que los valores en los Entries no sean nulos
			if (String.IsNullOrEmpty(this.txtPrecioVenta.Text) ||
			    	String.IsNullOrEmpty(this.txtIva.Text))
				return;
			
			// Calculo el precio final a partir del precio de venta y porcentaje de IVA.
			this.txtPrecioFinal.Importe =
				this.txtPrecioVenta.Importe * (1 + (this.txtIva.Porcentaje/100.0m));
			
			// Hago que el Widget no sea editable (FIXME: hay que volverlo a setearlo, no debería ser así)
			CustomWidgetsUtils.WidgetEditable(this.txtPrecioFinal, false);
		}
		#endregion
		
		protected virtual void OnTxtCodigoChanged (object sender, System.EventArgs e)
		{
			// Si no hay nada ingresado, desactivo los botones 
			// 'Comprobar disponibilidad' y 'Recuperar producto eliminado'
			if (this.txtCodigo.Text.Equals(String.Empty))
			    this.btnRecuperarProducto.Sensitive = this.btnCodigoDisponible.Sensitive = false;
			
			// Si hay algo escrito los activo
			else
				this.btnRecuperarProducto.Sensitive = this.btnCodigoDisponible.Sensitive = true;
		}
		
		/// <summary>
		/// Evento que se lanza cuando se hace click en el botón 'Verificar disponibilidad'.
		/// Basicamente, verifica si el código ingresado por el usuario corresponde o no a
		/// otro producto ya dado de alta en la base de datos.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnCodigoDisponibleClicked (object sender, System.EventArgs e)
		{
			bool productoEliminado = false;
			
			// Pregunto al controlador si el código ingresado está disponible,
			// Si está disponible, se lo notifico al usuario.
			if (this.controladorProductos.
			    CodigoDisponible(this.txtCodigo.Text.Trim(), out productoEliminado)) {
				
				SMessageDialog.MensajeInformacion("El código que ingresado está disponible.");
				return;
			}
			
			// Si el código no está disponible, veo si el producto está eliminado
			if (!productoEliminado) {
				// Si no está eliminado, notifico al usuario que el código ya está ocupado
				SMessageDialog.MensajeAdvertencia("Ya existe un producto con ese código ingresado" +
				                                  " en el sistema.");
				return;
			}
			
			// Si llegué hasta acá es porque el código no está disponible,
			// pero pertenece a un producto que está marcado como eliminado
			SMessageDialog.MensajeAdvertencia("Ya existe un producto con ese código, " +
				"pero está marcado como eliminado. Puede recuperarlo con el botón " +
				"<b>Recuperar producto eliminado</b>.");
		}
		
		protected virtual void OnBtnRecuperarProductoClicked (object sender, System.EventArgs e)
		{
			ProductoDto productoRecuperdado;
			
			// Trato de recuperar el producto
			try {
				productoRecuperdado = this.controladorProductos.
					RecuperarProductoEliminado(this.txtCodigo.Text.Trim());
			}
			catch (ObjetoInexistenteException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			catch (ObjetoInvalidoException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			catch (StratosException ex) {
				// Si hubo un error, muestro el mensaje de la excepción y retorno
				SMessageDialog.MensajeError(ex);
				return;
			}
			
			// Si no hubo errores, notifico al usuario y cargo el producto en el formulario
			if (productoRecuperdado != null)
				SMessageDialog.MensajeInformacion("El producto ha sido recuperado exitosamente");
			
			ProductoDto productoCargadoTemp = this.productoCargado;
			
			// Cargo el producto en el formulario
			this.CargarProductoInterno(productoRecuperdado);
			
			// Si había un producto cargado antes de recuperarlo
			if (productoCargadoTemp != null) {
				this.LeerDatosIngresados(ref productoCargadoTemp);
				this.productoCargado = productoCargadoTemp;
				
				this.navegadorElementos.Sensitive = true;
			}
		}
		
		protected virtual void OnBtnGuardarClicked (object sender, System.EventArgs e)
		{
			/* Valido los datos ingresados en el formulario. Si se lanza alguna execepción
			 * es porque hay campos inválidos. Muestro los mensajes y retorno */
			try {
				this.ValidarDatosIngresados();
			}
			catch(DatosIngresadosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.Message);
				return;
			}
			
			// Si no hay un producto cargado, creo uno nuevo
			if (this.productoCargado == null)
				this.productoCargado = new ProductoDto();
			
			/* Creo otra instancia del producto (posiblemente por modificar), y no utilizo
			 * this.productoCargado, ya que si el mismo es inválido, el método LeerDatosIngresados
			 * lo modificaría, y se cambiaría en el listado de productos. Esto puede ser confuso.
			 * Por lo tanto, únicamente modifico this.productoCargado una vez que la modificación
			 * del producto haya tenido éxito.
			 * En caso de que se trate de la alta de un producto, no habría problemas y la lógica
			 * no cambiaría. */
			ProductoDto productoNuevo = new ProductoDto();
			this.LeerDatosIngresados(ref productoNuevo);
			
			productoNuevo.id = this.productoCargado.id;
			productoNuevo.version = this.productoCargado.version;
			
			ProductoDto productoPersistido;
			
			// Si el producto es nuevo, llamo al metodo AgregarProducto
			if (productoNuevo.id <= 0) {
				try {
					productoPersistido = (ProductoDto)
						this.controladorProductos.Guardar(productoNuevo);
					
					// Cargo el producto persistido en la interfaz gráfica
					this.CargarProducto(productoPersistido, true);
					
					// Desabilito el navegador de elementos, ya que este es un producto
					// nuevo y no tiene nada que ver con los posiblemente cargados en la lista
					this.navegadorElementos.Sensitive = false;
					
					this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
					
					SMessageDialog
						.MensajeInformacion("El producto se ha ingresado correctamente al " +
						                    "sistema.");
				}
				catch (ObjetoInvalidoException ex) {
				  	SMessageDialog.MensajeError("El producto tiene datos inválidos:\n" + ex.mensaje);
					return;
				}
			}
			
			// Si el producto ya existe, es porque está en modificación. Llamo al metodo 'Modificar'.
			else {
				try {
					// Veo si fué modificado el código del producto. Si es así, pregunto para confirmar
					// la voluntad del usuario de realizar dicho cambio.
					if (this.productoCargado.codigo != productoNuevo.codigo) {
						Gtk.ResponseType respuesta = SMessageDialog.MensajePregunta("El código" +
							" del producto ha sido modificado. ¿Está seguro que quiere realizar" +
							" este cambio?");
						
						// Si la respuesta es 'No', vuelvo a poner el código viejo del producto y retorno,
						// sin alterara los demas campos, para que no se pierdan los demás cambios
						if (respuesta != Gtk.ResponseType.Yes) {
							this.txtCodigo.Text = this.productoCargado.codigo;
							return;
						}
					}
					
					productoPersistido = (ProductoDto)
						controladorProductos.Modificar(productoNuevo);
					
					/* En este punto la modificación fue exitosa.
					 * Realizo la lectura de los datos ingresados en este punto para actualizar la
					 * instancia this.productoCargado, y de esta forma que se actualice el producto
					 * en la lista. */
					this.LeerDatosIngresados(ref this.productoCargado);
					
					// Cargo el producto persistido en la interfaz gráfica
					this.CargarProducto(productoPersistido, true);
					
					this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
				}
				catch (ObjetoInvalidoException ex) {
				  	SMessageDialog.MensajeError("El producto tiene datos inválidos:\n" + ex.mensaje);
					return;
				}
				catch (ObjetoVersionIncorrectaException ex) {
				  	SMessageDialog
						.MensajeError("<b>El producto que intenta modificar fue " +
							"modificado ya por otra terminal</b>.\nVuelva a cargar el " +
							"producto actual y realice nuevamente las modificaciones");
					return;
				}
				catch (ObjetoInexistenteException ex) {
				  	SMessageDialog.MensajeError("El producto que intenta modificar ya no existe en la base de datos.");
					return;
				}
			}
		}
		
		protected virtual void OnBtnNuevoClicked (object sender, System.EventArgs e)
		{
			// Si hay modificaciones hechas, pregunto al usuario si está seguro de perderlas
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.VaciarFormulario();
		}
		
		protected void OnEstadoProductoCambiado(EstadoObjeto estado)
		{
			// Si el estado es 'Cargado'. FIXME: Solo manejo este estado por ahora
			if (estado == EstadoObjeto.Cargado) {
				
				// Si el producto está eliminado, activo el botón 'Recuperar' y
				// desactivo el resto de la interfaz, excepto el NavegadorElementos
				if (this.productoCargado.eliminado) {
					
					this.btnRecuperarProducto.Sensitive = true;
					
					this.btnCodigoDisponible.Sensitive = false;
					this.btnGuardar.Sensitive = false;
					this.btnNuevo.Sensitive = false;
					
					this.fraInformacionGeneral.Sensitive = false;
					this.fraPrecios.Sensitive = false;
					this.frmObservaciones.Sensitive = false;
				}
				
				// Si no está eliminado desactivo los botones 'Recuperar' y 
				// 'Verificar disponibilidad' y activo lo demas
				else {
					this.btnRecuperarProducto.Sensitive = false;
					this.btnCodigoDisponible.Sensitive = false;
					
					this.btnGuardar.Sensitive = true;
					this.btnNuevo.Sensitive = true;
					
					this.fraInformacionGeneral.Sensitive = true;
					this.fraPrecios.Sensitive = true;
					this.frmObservaciones.Sensitive = true;
				}
			}				
		}
		#endregion
		
		#region Métodos públicos
		#region Carga de productos
		public void CargarProducto (object objeto)
		{
			ProductoDto producto = (ProductoDto) objeto;
			this.CargarProducto(producto);
		}
		
		public void CargarProducto (ProductoDto producto, bool evitarPreguntaPorModificaciones)
		{
			if (!evitarPreguntaPorModificaciones &&
			    	(this.administradorEstado.EstaModificado &&
			     		!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones))
				return;
			
			this.CargarProductoInterno(producto);
			
			// Activo el navegador de elementos
			this.navegadorElementos.Sensitive = true;
		}
		
		public void CargarProducto (ProductoDto producto)
		{
			// Primero me fijo si no hay modificaciones sin guardar
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.CargarProductoInterno(producto);
			
			// Activo el navegador de elementos
			this.navegadorElementos.Sensitive = true;
		}
		
		protected void CargarProductoInterno (ProductoDto producto)
		{
			// Reseteo la interfaz
			this.VaciarFormulario();
			
			this.productoCargado = producto;
			
			// Seteo el código y la descripción
			this.txtCodigo.Text = producto.codigo;
			this.txtDescripcion.Text = producto.descripcion;
			
			// FIXME: Por ahora, y para facilitar el cambio de precios, dejamos que se pueda modificar el código
			// Primero, desactivo los Entrys de Código y Descripción, ya que estos campos no se pueden modificar
			//CustomWidgetsUtils.WidgetEditable(this.txtCodigo, false);
			//this.btnCodigoDisponible.Sensitive = false;
			
			// FIXME: Por ahora dejo que se puedan modificar la descripción de los productos.
			//CustomWidgetsUtils.WidgetEditable(this.txtDescripcion, false);
			
			// Rubro y Sub-rubro
			if (producto.rubro != null)
				this.cmbRubro.ObjetoActivo = producto.rubro;
			
			if (producto.subRubro != null)
				this.cmbSubRubro.ObjetoActivo = producto.subRubro;
			
			// Unidad
			this.cmbUnidadProducto.ObjetoActivo = producto.unidad;
			
			// Producto abstracto o genérico
			this.chkProductoAbstracto.Active = producto.productoAbstracto;
			
			// Activar gestión de stock
//			this.chkControlStock.Active = producto.activarGestionStock;
			
			// Precio
			this.txtPrecioCosto.Importe = (decimal)producto.precioCosto;
			this.txtPrecioVenta.Importe = (decimal)producto.importeSinIva;
			this.txtIva.Porcentaje = (decimal)producto.porcentajeIva;
			this.txtPrecioFinal.Importe = (decimal)producto.importeVenta;
			
			// Notas
			this.txtObservaciones.Buffer.Text = producto.notas;
				
			// Cambio el estado de la interfaz gráfica
			if (producto.eliminado)
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "ELIMINADO", "red");
			else
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado);
		}
		#endregion
		#endregion
		
		#region Otros métodos
		protected void InicializarComboBoxRubro()
		{
			// Verifico que haya conexión y que el controlador no sea nulo
			if (!MainIce.ConectadoAlServidor || this.controladorProductos == null)
				return;
			
			/* Cuando se trata de métodos de inicialización como éste, es importante captar cualquier excepción
			 * y tratarla de una forma parecida a como está aquí. Esto es así porque si se lanza alguna en
			 * este punto y es gestionada por algún evento como MainGui.GLibUnhandledException o
			 * MainGui.CurrentDomainUnhandledException la interfaz gráfica no se terminaría de inicializar. */
			
			// Obtengo todos los rubros
			Ice.Object[] resultado = null;
			try {
				resultado = this.controladorProductos.ObtenerRubros();
			}
			catch (System.Exception ex) {
				MainGui.GestionarException(ex);
				return;
			}
			
			RubroDto[] rubros = resultado.Cast<RubroDto>().ToArray();

			this.cmbRubro.Limpiar();
			this.cmbRubro.AgregarObjetos(rubros);
			this.cmbRubro.Resetear();
			
			// Inicializo el Combobox de Sub-rubros
			this.InicializarComboBoxSubRubro(this.cmbRubro.ObjetoActivo);
			
			// Vuelvo a setear el estado a Nuevo, ya que el cambiar el combo de
			// rubros provoca que la interfaz se coloque en "Modificado".
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo);
		}
		
		
		protected void InicializarComboBoxSubRubro (RubroDto rubroPadre)
		{
			/* Si no estoy conectado al servidor, no continúo con la inicializacion. Esto puede
			 * pasar al iniciarse el sistema. */
			if (!MainIce.ConectadoAlServidor || this.controladorProductos == null)
				return;
			
			/* Cuando se trata de métodos de inicialización como éste, es importante captar cualquier excepción
			 * y tratarla de una forma parecida a como está aquí. Esto es así porque si se lanza alguna en
			 * este punto y es gestionada por algún evento como MainGui.GLibUnhandledException o
			 * MainGui.CurrentDomainUnhandledException la interfaz gráfica no se terminaría de inicializar. */
			
			// Obtengo todos los sub-rubros del rubro seleccionado
			Ice.Object[] resultado = null;
			try {
				resultado = this.controladorProductos.ObtenerSubrubros(rubroPadre);
			}
			catch (System.Exception ex) {
				MainGui.GestionarException(ex);
				return;
			}
			
			// Limpio el contenido del anterior del combobox sub-rubro
			this.cmbSubRubro.Limpiar();
			
			RubroDto[] subRubrosEncontrados = resultado.Cast<RubroDto>().ToArray();
			
			// Cargo los sub-rubros encontrados en el combobox
			this.cmbSubRubro.AgregarObjetos(subRubrosEncontrados);
			
			this.cmbSubRubro.Resetear();
			
			// Vuelvo a setear el estado a Nuevo, ya que el cambiar el combo de
			// rubros provoca que la interfaz se coloque en "Modificado".
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo);
		}
		
		protected void LeerDatosIngresados(ref ProductoDto producto)
		{
			producto.codigo = this.txtCodigo.Text.Trim().ToUpper();
			producto.descripcion = this.txtDescripcion.Text.Trim();
			
			producto.unidad = this.cmbUnidadProducto.ObjetoActivo;
			
			// Si no hay sub-rubro seleccionado, seteo el rubro
			if (this.cmbSubRubro.TextoActivo == "")
				producto.subRubro = this.cmbRubro.ObjetoActivo;
			else
				producto.subRubro = this.cmbSubRubro.ObjetoActivo;
			
			// Precios
			producto.precioCosto = this.txtPrecioCosto.ImporteDouble;
			producto.importeSinIva = this.txtPrecioVenta.ImporteDouble;
			producto.importeVenta = this.txtPrecioFinal.ImporteDouble;
			producto.porcentajeIva = this.txtIva.PorcentajeDouble;
			
			// Observaciones
			producto.notas = this.txtObservaciones.Buffer.Text;
			
			// Otras propiedades
			producto.eliminado = false;
			producto.productoAbstracto = this.chkProductoAbstracto.Active;
		}
		
		protected void VaciarFormulario()
		{
			this.productoCargado = null;
			
			// Activo los campos de código y descripción
			CustomWidgetsUtils.WidgetEditable(this.txtCodigo, true);
			CustomWidgetsUtils.WidgetEditable(this.txtDescripcion, true);
			
			// Vacío el Entry de código y deshabilito el botón 'Verificar disponibilidad'
			this.txtCodigo.Text = "";
			this.btnCodigoDisponible.Sensitive = false;
			this.btnRecuperarProducto.Sensitive = false;
			
			this.txtDescripcion.Text = "";
			
			// Combo-boxes
			this.cmbUnidadProducto.Resetear();
			this.cmbRubro.Resetear();
			this.cmbSubRubro.Resetear();
			
			this.chkProductoAbstracto.Active = false;
			
			// Importes
			this.txtPrecioCosto.Resetear();
			this.txtIva.Porcentaje = 21.00m;
			this.txtPrecioVenta.Resetear();
			this.txtPrecioFinal.Resetear();
			
			CustomWidgetsUtils.WidgetEditable(this.txtPrecioFinal, false);
			
			// Desactivo el checkbox 'Especificar ganancia' y el Entry Ganancia.
			this.chkEspecificarGanancia.Active = false;
			this.spnGanancia.Sensitive = false;
			
			// Observaciones
			this.txtObservaciones.Buffer.Text = "";
			
			// Desactivo el navegador de elementos
			this.navegadorElementos.Sensitive = false;
			
			// El estado de la interfaz es NUEVO
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo);
		}
		
		protected void ValidarDatosIngresados()
		{
			// Chequeo que el código no sea vacío
			if (this.txtCodigo.Text.Trim().Equals(String.Empty))
				throw new DatosIngresadosInvalidosException("Código inválido. No puede ser vacío.");
			
			// Chequeo que la descripción no sea vacía
			if (this.txtDescripcion.Text.Trim().Equals(String.Empty))
				throw new DatosIngresadosInvalidosException("Descripción inválida. No puede ser vacía.");
			
			// Chequeo que la ganancia sea mayor a 0 %
			if (this.chkEspecificarGanancia.Active && this.spnGanancia.Value == 0.00d)
				throw new DatosIngresadosInvalidosException("La ganancia debe ser mayor a 0 %");
			
			// Chequeo que el precio de costo no sea 0.00
			if (String.IsNullOrEmpty(this.txtPrecioCosto.Text) ||
			    	!this.txtPrecioCosto.TextoValido ||
			    	this.txtPrecioCosto.Importe == 0.00m)
				throw new DatosIngresadosInvalidosException("El precio de costo debe ser mayor a cero.");
			
			// Porcentaje de IVA
			if (String.IsNullOrEmpty(this.txtIva.Text) ||
			    	!this.txtIva.TextoValido ||
			    	this.txtIva.Porcentaje == 0.00m)
				throw new DatosIngresadosInvalidosException("El porcentaje de IVA no es válido.");
			
			// Chequeo que el precio de venta se mayor que el precio de costo
			if (String.IsNullOrEmpty(this.txtPrecioVenta.Text) ||
			    	!this.txtPrecioVenta.TextoValido ||
			    	this.txtPrecioCosto.Importe >= this.txtPrecioVenta.Importe)
				throw new DatosIngresadosInvalidosException("El precio de venta debe ser mayor al " +
				                                            "precio de costo.");
		}
		#endregion
	}
}
