// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Text;
using System.Collections.Generic;

using Gtk;

using StratosClient.Common;
using Stratos.Common;
using Stratos.Common.Excepciones;
using Stratos.ModeloDto;
using StratosClient.Core.Gui;
using StratosServer.Controladores;
using StratosClient.Widgets.Level2;
using StratosClient.Widgets.Level1;

namespace StratosClient.Productos
{
	public partial class SolapaPrecios : Gtk.Bin
	{
		// TODO: Ver si este límite esta bien
		private const int TAMANIO_PAGINA = 500;
		
		private ListaPreciosProductos tvPreciosProductos;
		private ComboBoxGenerico<RubroDto> cmbRubro;
		private ComboBoxGenerico<RubroDto> cmbSubRubro;
		
		public SolapaPrecios()
		{
			this.Build();
			
			this.tvPreciosProductos = new ListaPreciosProductos();
			this.scrolledwindow.Add(this.tvPreciosProductos);
			
			#region Navegador de páginas
			this.navegadorPaginas.TamañoPagina = TAMANIO_PAGINA;
			this.navegadorPaginas.ColeccionElementos = this.tvPreciosProductos;
			this.navegadorPaginas.CargadorDatos = this.CargadorProductos;
			this.navegadorPaginas.CriterioOrdenacion =
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				};
			#endregion
			
			#region Combo Sub-rubro
			this.cmbSubRubro = new ComboBoxGenerico<RubroDto>(new RubroDtoEqualityComparer());
			this.cmbSubRubro.Renderizador = delegate (RubroDto unSubRubro) {
				return unSubRubro.nombre;
			};
			this.hbSubRubro.PackEnd(this.cmbSubRubro, true, true, 0);
			#endregion
			
			#region Combo Rubro
			this.cmbRubro = new ComboBoxGenerico<RubroDto>(new RubroDtoEqualityComparer());
			this.cmbRubro.Renderizador = delegate (RubroDto unRubro) {
				return unRubro.nombre;
			};
			this.hbRubro.PackEnd(this.cmbRubro, true, true, 0);
			
			// Si estoy conectado ya al servidor, inicializo el combobox de rubros.
			if (MainIce.ConectadoAlServidor)
				this.InicializarComboBoxRubro();
			
			// Además me suscribo al evento de ConexionEstablecida para reinicializar
			// el ComboBox de rubros cuando el cliente se conecta
			MainIce.ConexionEstablecida += delegate {
				this.InicializarComboBoxRubro();
			};
			
			this.cmbRubro.Changed += delegate(object sender, EventArgs e) {
				this.InicializarComboBoxSubRubro(this.cmbRubro.ObjetoActivo);
			};
			#endregion
			
			#region Checkboxes de filtrado
			this.chkCodigo.Toggled += delegate(object sender, EventArgs e) {
				this.hbCodigo.Sensitive = this.hbRangoCodigo.Sensitive = this.chkCodigo.Active;
				this.rbtnContieneCodigo.Active = this.chkCodigo.Active;
				// Si busco por código, entonces desactivo la busqueda por descripción,
				// Rubro y Sub-rubro
				this.hbDescripcion.Sensitive = this.hbRubroSubRubro.Sensitive = !this.chkCodigo.Active;
			};
			
			this.chkDescripcion.Toggled += delegate(object sender, EventArgs e) {
				this.txtDescripcion.Sensitive = this.chkDescripcion.Active;
			};
			
			this.chkRubro.Toggled += delegate(object sender, EventArgs e) {
				this.cmbRubro.Sensitive = this.chkRubro.Active;
				this.chkSubRubro.Sensitive = this.chkRubro.Active;
			};
			
			this.chkSubRubro.Toggled += delegate(object sender, EventArgs e) {
				this.cmbSubRubro.Sensitive = this.chkSubRubro.Active;
			};
			#endregion

			#region RadioButtons de código
			this.rbtnContieneCodigo.Clicked += delegate(object sender, EventArgs e) {
				this.txtCodigo.Sensitive = true;
				this.txtCodigoDesde.Sensitive = this.txtCodigoHasta.Sensitive = false;
				this.txtCodigoDesde.Text = this.txtCodigoHasta.Text = "";
			};
			
			this.rbtnRangoCodigo.Clicked += delegate(object sender, EventArgs e) {
				this.txtCodigoDesde.Sensitive = this.txtCodigoHasta.Sensitive = true;
				this.txtCodigo.Sensitive = false;
				this.txtCodigo.Text = "";
			};
			#endregion
			
			#region Suscripción a eventos
			this.btnBuscar.Clicked += this.OnBtnBuscarClicked;
			this.btnNuevaBusqueda.Clicked += this.OnBtnLimpiarClicked;
			this.btnAplicar.Clicked += this.OnBtnAplicarClicked;
			
			this.spbtnDiferencia.Changed += delegate {
				string valor = this.spbtnDiferencia.Text;
				
				this.btnAplicar.Sensitive =
					StratosUtil.IsDouble(valor) && StratosUtil.ParseDouble(valor, 2) > 0.0d;
			};
			
			this.txtDescripcion.Activated += this.OnBtnBuscarClicked;
			this.txtCodigo.Activated += this.OnBtnBuscarClicked;
			#endregion
			
			this.VaciarFormulario();
		}
		
		#region Eventos
		protected virtual void OnBtnBuscarClicked (object sender, System.EventArgs e)
		{
			// Si no hay ningun campo de filtrado activo, aviso y retorno
			if (!this.chkCodigo.Active && !this.chkDescripcion.Active &&
			    !this.chkRubro.Active) {
				SMessageDialog.MensajeAdvertencia("No existen criterios de busqueda definidos.");
				return;
			}
			
			// Si está activada la busqueda por rango, veo si el rango es correcto.
			// Si los códigos son iguales, o están invertidos en el orden, aviso y retorno.
			if (this.chkCodigo.Active && this.rbtnRangoCodigo.Active) {
				int resultado = string.CompareOrdinal(this.txtCodigoDesde.Text.Trim().ToUpper(),
				                                      this.txtCodigoHasta.Text.Trim().ToUpper());
				
				if (resultado == 0) {
					SMessageDialog.MensajeError("Los códigos en la busqueda por rangos no pueden" +
						" iguales.");
					return;
				}
				else if (resultado == 1) {
					SMessageDialog.MensajeError("Los códigos están invertidos respecto a su orden.");
					return;
				}
			}
			
			// Creo una consulta paginable
			ConsultaPaginable cp = new ConsultaPaginable();
			cp.pagina = 0;
			cp.tamanioPagina = TAMANIO_PAGINA;
			cp.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				}
			};
			
			ProductoDto[] productosEncontrados = this.EjecutarConsulta(cp);
			
			/* Si no hay productos que cumplan con los criterios de busqueda,
			 * notifico al usuario y retorno */
			if (productosEncontrados.Length <= 0) {
				SMessageDialog.MensajeAdvertencia("No hay productos que cumplan con los criterios " +
				               			"de busqueda.\nReformule la busqueda e intente nuevamente.");
				return;
			}
			
			// Activo el Navegador de páginas
			this.navegadorPaginas.Sensitive = true;
			
			// Vacío el treeview
			this.tvPreciosProductos.Vaciar();
					
			// Cargo los clientes encontrados en el treeview
			this.tvPreciosProductos.AgregarElementosPrimeraPagina(productosEncontrados);
			
			// Desactivo los campos de filtrado y el botón buscar
			this.vbFiltro.Sensitive = false;
			this.btnBuscar.Sensitive = false;
		}
		
		protected virtual void OnBtnLimpiarClicked(object sender, System.EventArgs e)
		{
			this.vbFiltro.Sensitive = true;
			this.btnBuscar.Sensitive = true;
			
			this.navegadorPaginas.Sensitive = false;
			
			this.VaciarFormulario();
		}
		
		protected virtual void OnBtnAplicarClicked(object sender, System.EventArgs e)
		{
			// Si no hay productos seleccionados para modificar, muestro una advertencia
			if (this.tvPreciosProductos.ElementosSeleccionados.Count == 0) {
				SMessageDialog
					.MensajeAdvertencia("<b>No ha seleccionado ningún producto para " +
					                    "modificar.</b>\n\n" +
					                    "Puede seleccionar todos los productos haciendo " +
					                    "click en la columna de selección, o elegir " +
					                    "cada producto individualmente.");
				return;
			}
			
			// Muestro el diálogo de confirmación antes de modificar los precios
			string pregunta =
				"<b>¿Está seguro que desea modificar los precios de los " +
				"productos seleccionados?</b>";
				
			ResponseType respuesta =
				SMessageDialog.MensajePregunta(pregunta);
			
			// Si la respuesta es No, salgo
			if (respuesta != ResponseType.Yes)
				return;
			
			// Obtengo los productos seleccionado del treeview
			long[] idsProductos = this.tvPreciosProductos.IdsProductosSeleccionados();
			
			// Si no hay productos seleccionados, retorno
			if (idsProductos.Length <= 0)
				return;
			
			// Armo el objeto AccionModificacionPrecio
			AccionModificacionPrecio amp = new AccionModificacionPrecio();
			
			// Seteo que acción tengo que llevar a cabo: aumentar o disminuir el precio
			amp.accion = this.cmbAumentarDisminuir.ActiveText.Equals("Aumentar") ?
				TipoAccion.Aumentar : TipoAccion.Disminuir;
			
			// Seteo la diferencia
			amp.cantidad = this.spbtnDiferencia.Value;
			
			// Seteo si la diferencia se tiene que aplicar de manera porcentual o fija
			amp.cambioTipo = this.cmbPorcientoFijo.ActiveText.Equals("% (porcentaje)") ?
				TipoCambio.Porcentual : TipoCambio.PrecioFijo;
			
			// Envio al servidor los cambios de precios
			try {
				((ControladorProductosPrx) MainIce.GetControlador("ControladorProductos"))
					.ModificarPrecios(idsProductos, amp);
			}
			catch (StratosException ex) {
				SMessageDialog.MensajeError(ex);
				return;
			}
			
			// Aplico los cambios en el modelo del treeview
			this.tvPreciosProductos.ActualizarProductosSeleccionados(amp);
			
			// Aviso que los cambios se realizaron exitosamente
			SMessageDialog.MensajeInformacion("Los precios se actualizaron exitosamente");
		}
		
		protected virtual void OnBtnImprimirClicked (object sender, System.EventArgs e)
		{
			// Verifico que haya productos buscados.
			List<ProductoDto> todosLosProductos =
				this.tvPreciosProductos.ElementosSeleccionados;
			
			if (todosLosProductos.Count == 0) {
				SMessageDialog.MensajeInformacion("Realice una búsqueda de productos y seleccione los que desea imprimir.");
				return;
			}
			
			// Deshabilito el botón de imprimir
			this.btnImprimir.Sensitive = false;
			
			while (Gtk.Application.EventsPending())
				Gtk.Application.RunIteration();
			
			List<ProductoWrapper> listaProductos =
				new List<ProductoWrapper>();
			
			// Paso todo a ProductoWrapper
			foreach (ProductoDto productoDto in todosLosProductos)
				listaProductos.Add(new ProductoWrapper(productoDto));
			
			// Realizo la impresión.
			ImpresionDocumento impresionBorrador =
				new ImpresionDocumento("Addins/Productos/templateListadoProductos.vm",
				                      "Addins/Productos/",
				                      "listadoProductos",
				                      listaProductos);
			
			impresionBorrador.MargenArriba = 10.0;
			impresionBorrador.MargenAbajo = 10.0;
			
			impresionBorrador.AgregarObjeto("fechaActual", DateTime.Today);
			
			impresionBorrador.ImpresionEjecutada += delegate {
				this.btnImprimir.Sensitive = true;
			};
			
			try {
				impresionBorrador.Imprimir();
			}
			catch {
				// Si hay algún problema, vuelvo a habilitar el botón para imprimir
				// y relanzo la excepción.
				this.btnImprimir.Sensitive = true;
				throw;
			}
		}
		#endregion
		
		#region Otros métodos
		protected ProductoDto[] EjecutarConsulta(ConsultaPaginable cp)
		{
			// Realizo la consulta en el servidor
			Ice.Object[] resultadoConsulta =
			((ControladorProductosPrx) MainIce.GetControlador("ControladorProductos"))
					.BuscarHQL(this.CrearConsultaHql(), cp);
			
			ProductoDto[] productosEncontrados = resultadoConsulta.Cast<ProductoDto>().ToArray();
			
			return productosEncontrados;
		}
		
		protected object[] CargadorProductos(ConsultaPaginable cp)
		{
			return this.EjecutarConsulta(cp);	
		}
		
		protected void InicializarComboBoxRubro()
		{
			/* Cuando se trata de métodos de inicialización como éste, es importante captar cualquier excepción
			 * y tratarla de una forma parecida a como está aquí. Esto es así porque si se lanza alguna en
			 * este punto y es gestionada por algún evento como MainGui.GLibUnhandledException o
			 * MainGui.CurrentDomainUnhandledException la interfaz gráfica no se terminaría de inicializar. */
			
			// Obtengo todos los rubros
			Ice.Object[] resultado = null;
			try {
				resultado = ((ControladorProductosPrx)MainIce.GetControlador("ControladorProductos"))
					.ObtenerRubros();
			}
			catch (System.Exception ex) {
				MainGui.GestionarException(ex);
				return;
			}
			
			RubroDto[] rubros = resultado.Cast<RubroDto>().ToArray();

			this.cmbRubro.Limpiar();
			this.cmbRubro.AgregarObjetos(rubros);
			this.cmbRubro.Resetear();
			
			// Inicializo el Combobox de Sub-rubros
			this.InicializarComboBoxSubRubro(this.cmbRubro.ObjetoActivo);
		}
		
		protected void InicializarComboBoxSubRubro (RubroDto rubroPadre)
		{
			/* Si no estoy conectado al servidor, no continúo con la inicializacion. Esto puede
			 * pasar al iniciarse el sistema. */
			if (!MainIce.ConectadoAlServidor)
				return;
			
			/* Cuando se trata de métodos de inicialización como éste, es importante captar cualquier excepción
			 * y tratarla de una forma parecida a como está aquí. Esto es así porque si se lanza alguna en
			 * este punto y es gestionada por algún evento como MainGui.GLibUnhandledException o
			 * MainGui.CurrentDomainUnhandledException la interfaz gráfica no se terminaría de inicializar. */
			
			// Obtengo todos los sub-rubros del rubro seleccionado
			Ice.Object[] resultado = null;
			try {
				resultado = ((ControladorProductosPrx)MainIce.GetControlador("ControladorProductos"))
					.ObtenerSubrubros(rubroPadre);
			}
			catch (System.Exception ex) {
				MainGui.GestionarException(ex);
				return;
			}
			
			// Limpio el contenido del anterior del combobox sub-rubro
			this.cmbSubRubro.Limpiar();
			
			RubroDto[] subRubrosEncontrados = resultado.Cast<RubroDto>().ToArray();
			
			// Cargo los sub-rubros encontrados en el combobox
			this.cmbSubRubro.AgregarObjetos(subRubrosEncontrados);
			
			this.cmbSubRubro.Resetear();
		}
		
		protected string CrearConsultaHql()
		{
			StringBuilder consulta = new StringBuilder();
			consulta.Append("from Producto p where ");
			
			// Si el campo código está activado...
			if (this.chkCodigo.Active) {
				
				// Si está activada la busqueda por parte del código y el campo no está vacío,
				// lo agrego a la busqueda.
				if (this.rbtnContieneCodigo.Active && !this.txtCodigo.Text.Trim().Equals(String.Empty))
					consulta.Append("p.Codigo like '%" + this.txtCodigo.Text.Trim().ToUpper() + "%' and ");
				
				// Si, por el contrario, está activada la busqueda por rango y estos no son vacíos
				else if (this.rbtnRangoCodigo.Active &&
				         !this.txtCodigoDesde.Text.Trim().Equals(String.Empty) &&
				         !this.txtCodigoHasta.Text.Trim().Equals(String.Empty))
					consulta.Append("p.Codigo between '" + this.txtCodigoDesde.Text.Trim().ToUpper()
					                + "' and '" + this.txtCodigoHasta.Text.Trim().ToUpper() + "' and");
			}
			
			// Si el campo código está desactivado, entonces veo si hay activado alguno
			// de los otros campos de filtrado
			else {
				// Si el campo Descripción está activado y no es vacío, agrego la descripción
				if (this.chkDescripcion.Active && !this.txtDescripcion.Text.Trim().Equals(string.Empty))
					consulta.Append("p.Descripcion like '%" + this.txtDescripcion.Text.Trim().ToUpper() + "%' and ");
				
				// Si el campo Sub-rubro está activado, lo agrego a la consulta antes que el rubro, ya que un sub-rubro
				// es mas específico y contiene ya al rubro padre
				if (this.chkSubRubro.Active)
					consulta.Append("p.SubRubro.Codigo = " + this.cmbSubRubro.ObjetoActivo.codigo + " and ");
				
				else if (this.chkRubro.Active)
					consulta.Append("p.SubRubro.RubroPadre.Codigo = " + this.cmbRubro.ObjetoActivo.codigo + " and ");
			}	
				
			// Agrego una condición para que no devuelva los productos eliminados
			consulta.Append("p.Eliminado = cast(0 as boolean)");
		
			return consulta.ToString();
		}
		
		protected void VaciarFormulario()
		{
			// Codigo
			this.chkCodigo.Active = true;
			this.rbtnContieneCodigo.Active = true;
			this.txtCodigo.Text = "";
			this.txtCodigoDesde.Sensitive = this.txtCodigoHasta.Sensitive = false;
			
			// Descripción
			this.chkDescripcion.Active = false;
			this.txtDescripcion.Sensitive = false;
			this.txtDescripcion.Text = "";
			
			// Rubro
			this.chkRubro.Active = false;
			this.cmbRubro.Sensitive = false;
			this.cmbRubro.Resetear();
			
			// Sub-rubro
			this.chkSubRubro.Active = false;
			this.cmbSubRubro.Sensitive = false;
			
			// Spinbutton diferencia
			this.spbtnDiferencia.Value = 0.0d;
			
			// TreeView
			this.tvPreciosProductos.Vaciar();
		}
		#endregion
	}
}
