// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections.Generic;

using Gtk;

using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.QueryObjects;
using StratosServer.Controladores;
using StratosClient.Core.Gui;
using Stratos.Common.Excepciones;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level2;

namespace StratosClient.Productos
{
	public partial class SolapaProductos : Gtk.Bin, IListaNavegable
	{
		private const int TAMAÑO_PAGINA = 250;
		
		private SolapaAgregarModificar solapaAgregarModificar;
		private SolapaPrecios solapaPrecios;
		
		private TvProductos tvProductos;
		
		private ProductoDtoQuery consultaActual;
		
		private TreeIter iterCargado;
		private int paginaIterCargado;
		
		private static SolapaProductos instancia;
		
		protected ServicioNavegadorElementos servicioNavegador;

		public SolapaProductos()
		{
			this.Build();
			
			#region Lista de productos
			this.tvProductos = new TvProductos();
			this.scrolledwindow1.Add(this.tvProductos);
			this.tvProductos.ShowAll();
			#endregion
			
			#region Navegador de páginas
			this.navegadorPaginas.TamañoPagina = TAMAÑO_PAGINA;
			this.navegadorPaginas.ColeccionElementos = this.tvProductos;
			this.navegadorPaginas.CargadorDatos = this.CargadorProductos;
			this.navegadorPaginas.CriterioOrdenacion =
				new CriterioOrdenacion {
					campoOrdenacion = "Descripcion",
					sentidoOrd = SentidoOrdenacion.Asc
				};
			#endregion
			
			#region Solapas
			this.solapaAgregarModificar = new SolapaAgregarModificar();
			Gtk.Label lblAgregarModificar = new Gtk.Label("Agregar-Modificar");
			this.nbProductos.AppendPage(this.solapaAgregarModificar, lblAgregarModificar);
			
			// Me suscribo al evento de IListaNavegable para que se desactive el navegador de
			// elementos cuando la lista se vacíe.
			this.ListaElementosVaciada += delegate {
				this.solapaAgregarModificar.NavegadorElementos.Sensitive = false;
			};
			
			this.solapaPrecios = new SolapaPrecios();
			Gtk.Label lblPrecios = new Gtk.Label("Precios");
			this.nbProductos.AppendPage(this.solapaPrecios, lblPrecios);
			#endregion
			
			#region Criterios de búsqueda
			this.chkCodigo.Toggled += delegate {
				this.txtCodigo.Sensitive = this.chkCodigo.Active;
			};
			this.chkCodigo.Active = true;
			
			this.chkDescripcion.Toggled += delegate {
				this.txtDescripcion.Sensitive = this.chkDescripcion.Active;
			};
			this.chkDescripcion.Active = true;
			
			this.chkIncluirEliminados.Active = false;
			#endregion
			
			#region Suscripción a eventos
			this.txtCodigo.Activated += OnBtnBuscarClicked;
			this.txtDescripcion.Activated += OnBtnBuscarClicked;
			
		 	this.tvProductos.Selection.Changed += OnTvProductosSelectionChanged;
			this.tvProductos.RowActivated += OnBtnModificarClicked;
			
			// Evento para disparar el menú desplegable
			this.tvProductos.ButtonReleaseEvent += this.OnButtonReleaseEvent;		
			#endregion
			
			this.servicioNavegador = new ServicioNavegadorElementos();
			
			// Suscribo el evento 'ListaElementosVaciada' de IListaNavegable al
			// evento 'ListaVaciada' de treeview
			this.tvProductos.ListaVaciada += delegate {
				if (this.ListaElementosVaciada != null)
					this.ListaElementosVaciada();
			};
			
			this.consultaActual = null;
			
			instancia = this;
		}
		
		#region Propiedades
		
		public static SolapaProductos Instancia {
			get { return instancia; }
		}
		#endregion
		
		#region IListaNavegable implementation
		public event System.Action ListaElementosVaciada;
		
		public object ObtenerObjetoRefrescado (TreeIter iter)
		{
			return this.ObtenerProductoRefrescado(iter);
		}

		public ServicioNavegadorElementos ServicioNavegador {
			get {
				this.servicioNavegador.ListaNavegable = this;
				return this.servicioNavegador;
			}
		}

		public TreeIter IterCargado {
			get { return this.iterCargado; }
			set { this.iterCargado = value; }
		}
		
		public int PaginaIterCargado {
			get { return this.paginaIterCargado; }
			set { this.paginaIterCargado = value; }
		}

		public ColeccionElementos ListaElementos {
			get { return this.tvProductos; }
		}

		public NavegadorPaginas NavegadorDePaginas {
			get { return this.navegadorPaginas; }
		}
		#endregion
		
		#region Eventos
		
		/// <summary>
		/// Este metodo se ejecuta cuando se dispara el evento Selection.Changed del TvProductos.
		/// Si se ha seleccionado una fila, se activan los botones 'Modificar' y 'Eliminar'.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnTvProductosSelectionChanged(object o, System.EventArgs e)
		{
			ProductoDto[] productosSeleccionados = this.tvProductos.ObtenerSeleccionadosGenericos();
			
			if (productosSeleccionados.Length == 0)
				return;
			
			// Obtengo el producto seleccionado
			ProductoDto productoSeleccionado = productosSeleccionados[0];
			
			// Chequeo que el producto seleccionado no sea nulo
			if (productoSeleccionado == null)
				return;
			
			// Según el estádo del producto, activo los botones correspondientes
			
			// Si está eliminado, activo el botón 'Recuperar' y viceversa.
			this.btnRecuperar.Sensitive = productoSeleccionado.eliminado;
			
			// Si está eliminado, desactivo los botones 'Modificar' y 'Eliminar' y viceversa.
			this.btnEliminar.Sensitive = btnModificar.Sensitive = !productoSeleccionado.eliminado;
		}
		
		protected virtual void OnBtnBuscarClicked (object sender, System.EventArgs e)
		{
			this.consultaActual = this.CrearQueryObject();
			
			// Creo una consulta paginable
			ConsultaPaginable cp = new ConsultaPaginable();
			cp.pagina = 0;
			cp.tamanioPagina = TAMAÑO_PAGINA;
			cp.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				}
			};
			
			ProductoDto[] productosEncontrados = this.EjecutarConsulta(cp);
			
			/* Si no hay productos que cumplan con los criterios de busqueda,
			 * notifico al usuario y retorno */
			if (productosEncontrados.Length <= 0) {
				SMessageDialog.MensajeAdvertencia("No hay productos que cumplan con los criterios " +
				               			"de busqueda.\nReformule la busqueda e intente nuevamente.");
				return;
			}
			
			// Activo el Navegador de páginas
			this.navegadorPaginas.Sensitive = true;
			
			// Vacío el treeview
			this.tvProductos.Vaciar();
					
			// Cargo los clientes encontrados en el treeview
			this.tvProductos.AgregarElementosPrimeraPagina(productosEncontrados);
		}	

		protected virtual void OnBtnLimpiarClicked (object sender, System.EventArgs e)
		{
			this.VaciarFormularioBusqueda();
			
			// Desactivo los botones 'Modificar' y 'Eliminar'
			this.btnModificar.Sensitive = false;
			this.btnEliminar.Sensitive = false;
					
			// Activo los campos de busqueda y el botón 'Buscar'
			this.txtCodigo.Sensitive = true;
			this.txtDescripcion.Sensitive = true;
			this.btnBuscar.Sensitive = true;
					
			// Desactivo el navegador de páginas
			this.navegadorPaginas.Sensitive = false;
		}
				
		[GLib.ConnectBefore]
		protected void OnButtonReleaseEvent(object o, ButtonReleaseEventArgs args)
		{
			// Solo activo el menú si es el botón derecho el presionado y el treeview no está vacío
			if (args.Event.Button == 3)
				this.MostrarMenuEmergente();
		}

		protected virtual void OnBtnModificarClicked (object sender, System.EventArgs e)
		{
			TreeIter[] iters;
			TreeIter iterSeleccionado;
			
			// Chequeo que el producto seleccionado no sea nulo
			if (this.tvProductos.ObtenerSeleccionados(out iters).Length == 0)
				return;
			
			iterSeleccionado = iters[0];
			
			// Obtengo el objeto refrescado
			ProductoDto productoRefrescado = this.ObtenerProductoRefrescado(iterSeleccionado);
			
			// Actualizo el iter cargado
			this.iterCargado = iterSeleccionado;
			this.paginaIterCargado = this.navegadorPaginas.PaginaActual;
			
			// Actualizo el treeview con el producto refrescado
			this.tvProductos.ActualizarObjetoSeleccionado(productoRefrescado);
			
			// Paso a la pestaña 'Agregar/Modificar'
			this.nbProductos.CurrentPage = 1;
			
			// Cargo el producto en el formulario de modificación
			this.solapaAgregarModificar.CargarProducto(productoRefrescado);
		}

		protected virtual void OnMiModificarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnModificarClicked(sender, e);
		}

		protected virtual void OnBtnEliminarClicked(object sender, System.EventArgs e)
		{
			// Chequeo que el item seleccionado no sea nulo
			if (this.tvProductos.ObtenerSeleccionadosGenericos().Length == 0)
				return;
			
			// Muestro el diálogo de confirmación
			string pregunta =
				"<b>¿Está seguro que desea marcar el producto '" +
				this.tvProductos.ObtenerSeleccionadosGenericos()[0].descripcion +
				"' como eliminado?</b>\n\n" +
				"Si responde afirmativamente, el producto se marcará como eliminado y no " +
				"aparecerá en los listados, pero no se eliminará realmente del sistema.";
				
			ResponseType respuesta =
				SMessageDialog.MensajePregunta(pregunta);
			
			// Si la respuesta es 'Si' borro el producto seleccionado
			if (respuesta == ResponseType.Yes) {
				
				// Le aviso al servidor que marque el cliente como eliminado
				((ControladorProductosPrx) MainIce.GetControlador("ControladorProductos"))
					.MarcarComoEliminado(this.tvProductos.ObtenerSeleccionadosGenericos()[0]);
			}
			
			// Si el checkbox 'Incluir eliminados' no está activo, elimino el objeto
			// del treeview y retorno
			if (!this.chkIncluirEliminados.Active) {
				this.tvProductos.EliminarObjetosSeleccionados();
				return;
			}
			
			// Si se deben incluir los productos eliminados en el treeview,
			// actualizo el objeto producto
			ProductoDto productoEliminado = this.tvProductos.ObtenerSeleccionadosGenericos()[0];
			productoEliminado.eliminado = true;
			this.tvProductos.ActualizarObjetoSeleccionado(productoEliminado);
		}
			
		protected virtual void OnMiEliminarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnEliminarClicked(sender, e);
		}
		
		protected virtual void OnBtnRecuperarClicked (object sender, System.EventArgs e)
		{
			ProductoDto[] productosSeleccionados = this.tvProductos.ObtenerSeleccionadosGenericos();
			
			if (productosSeleccionados.Length == 0)
				return;
			
			ProductoDto productoSeleccionado = productosSeleccionados[0];
			
			// Chequeo que el item seleccionado no sea nulo
			if (productoSeleccionado == null)
				return;
			
			// Chequeo que el producto esté marcado como eliminado
			if (!productoSeleccionado.eliminado) {
				return;
			}
			
			// Pregunto al usuario si está seguro de recuperar el producto
			ResponseType respuesta = SMessageDialog.MensajePregunta("¿Está seguro que desea " +
				"recuperar el producto seleccionado?");
			
			// Si la respuesta no es 'Si', retorno
			if (respuesta != ResponseType.Yes)
				return;
			
			ProductoDto productoRecuperado;
			
			// Trato de recuperar el producto
			try {
				productoRecuperado =
					((ControladorProductosPrx)MainIce.GetControlador("ControladorProductos"))
						.RecuperarProductoEliminado(productoSeleccionado.codigo);
			}
			catch (ObjetoInexistenteException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			catch (ObjetoInvalidoException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			catch (StratosException ex) {
				SMessageDialog.MensajeError(ex);
				return;
			}
			
			// Si no hubo errores, notifico al usuario y lo actualizo en la lista
			if (productoRecuperado != null)
				SMessageDialog.MensajeInformacion("El producto ha sido recuperado exitosamente");
			
			// Cargo el producto en el formulario
			this.tvProductos.ActualizarObjetoSeleccionado(productoRecuperado);
		}
		
		protected virtual void OnMiRecuperarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnRecuperarClicked(sender, e);
		}
		
		#endregion
			
		#region Otros métodos
		protected ProductoDto ObtenerProductoRefrescado(TreeIter iter)
		{
			Ice.Object resultado = ((ControladorProductosPrx) MainIce.GetControlador("ControladorProductos"))
				.Refrescar((ProductoDto)this.tvProductos.Model.GetValue(iter, 0));
			
			return (ProductoDto) resultado;
		}
		
		protected ProductoDto[] EjecutarConsulta(ConsultaPaginable cp)
		{
			// Realizo la búsqueda al servidor
			Ice.Object[] resultadoQueryObjects =
			((ControladorProductosPrx) MainIce.GetControlador("ControladorProductos"))
				.BuscarQueryObjects(this.consultaActual, cp);
			
			ProductoDto[] clientesEncontrados = resultadoQueryObjects.Cast<ProductoDto>().ToArray();
			
			return clientesEncontrados;
		}
		
		protected object[] CargadorProductos(ConsultaPaginable cp)
		{
			return this.EjecutarConsulta(cp);	
		}
				
		protected ProductoDtoQuery CrearQueryObject()
		{
			ProductoDtoQuery consulta = new ProductoDtoQuery();
			consulta.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioSimple temp;
			
			/* Si el campo código no es vacío, busco por código,
			 * ya que es una busqueda mas precisa */
			if (this.chkCodigo.Active && !this.txtCodigo.Text.Trim().Equals(string.Empty)) {
				
				CriterioOr criterioCodigo = new CriterioOr();
				criterioCodigo.criterios = new List<CriterioGenerico>();
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.LikeMatchAnywhere;
				temp.propiedad = "Codigo";
				temp.valor = this.txtCodigo.Text.Trim().ToUpper();
				
				criterioCodigo.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioCodigo);
			}
			
			// Si no hay nada ingresado en el campo Código, consulto por Descripcion
			if (this.chkDescripcion.Active && !this.txtDescripcion.Text.Trim().Equals(string.Empty)) {
				
				CriterioAnd criterioDescripcion = new CriterioAnd();
				criterioDescripcion.criterios = new List<CriterioGenerico>();
				
				foreach (string palabra in this.txtDescripcion.Text.Split(' ')) {
					temp = new CriterioSimple();
					temp.operador = TipoOperador.LikeMatchAnywhere;
					temp.propiedad = "Descripcion";
					temp.valor = palabra.ToUpper();
					
					criterioDescripcion.criterios.Add(temp);
				}
				
				consulta.listaCriteriosComplejos.Add(criterioDescripcion);
			}
			
			// Si no está activado el checkbox 'Incluir producto eliminados'
			// agrego un criterio para que la propiedad 'Eliminado' sea 'false'.
			if (!chkIncluirEliminados.Active) {
				CriterioAnd criterioEliminado = new CriterioAnd();
				criterioEliminado.criterios = new List<CriterioGenerico>();
			
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "Eliminado";
				temp.valor = bool.FalseString;
			
				criterioEliminado.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioEliminado);
			}
		
			return consulta;
		}
				
		protected void VaciarFormularioBusqueda()
		{
			this.txtCodigo.Text = "";
			this.txtDescripcion.Text = "";
			
			// Desactivo los botones 'Modificar', 'Eliminar' y 'Recuperar'
			this.btnModificar.Sensitive = false;
			this.btnEliminar.Sensitive = false;
			this.btnRecuperar.Sensitive = false;
			
			this.tvProductos.Vaciar();
		}
				
				
		protected void MostrarMenuEmergente()
		{
			Menu menu = new Menu();
			ImageMenuItem menuItem;
			
			ProductoDto productoSeleccionado = this.tvProductos.ObtenerSeleccionadosGenericos()[0];
			
			if (productoSeleccionado == null)
				return;
			
			// Según el estádo del producto, muestro los items del menú emergente
			// Si el producto está eliminado, solo agrego la opción "Recuperar"
			if (productoSeleccionado.eliminado) {
				menuItem = new ImageMenuItem("Recuperar");
				menuItem.Image = Image.NewFromIconName("gtk-undelete", Gtk.IconSize.Menu);
				menuItem.Activated += this.OnMiRecuperarActivated;
				menu.Append(menuItem);
			}
			// Si el producto no está eliminado, agrego las opciones "Modificar" y "Eliminar"
			else {
				menuItem = new ImageMenuItem("Modificar");
				menuItem.Image = Image.NewFromIconName("gtk-edit", IconSize.Menu);
				menuItem.Activated += this.OnMiModificarActivated;
				menu.Append(menuItem);
				
				menuItem = new ImageMenuItem("Eliminar");
				menuItem.Image = Image.NewFromIconName("gtk-remove", IconSize.Menu);
				menuItem.Activated += this.OnMiEliminarActivated;
				menu.Append(menuItem);
			}
			
			menu.Popup();
			menu.ShowAll();
		}
		#endregion
	}
}
