// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections.Generic;

using Gtk;

using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;
using Stratos.QueryObjects;
using StratosClient.Core.Gui;
using StratosServer.Controladores;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level2;

namespace StratosClient.Proveedores
{
	public partial class SolapaProveedores : Gtk.Bin, IListaNavegable
	{
		private static SolapaProveedores instancia;
		
		private const int TAMAÑO_PAGINA = 20;
		
		private SolapaAgregarModificar solapaModificar;
		
		private TvPersonas<ProveedorDto> tvProveedores;
		
		private ProveedorDtoQuery consultaActual;
		
		private TreeIter iterCargado;
		private int paginaIterCargado;
		
		private ServicioNavegadorElementos servicioNavegador;
		
		public SolapaProveedores()
		{
			this.Build();
			#region Lista de proveedores
			this.tvProveedores = new TvPersonas<ProveedorDto>();
			this.scrolledwindow1.Add(this.tvProveedores);
			this.tvProveedores.ShowAll();
			#endregion
			
			#region Navegador de páginas
			this.navegadorPaginas.TamañoPagina = TAMAÑO_PAGINA;
			this.navegadorPaginas.ColeccionElementos = this.tvProveedores;
			this.navegadorPaginas.CargadorDatos = this.CargadorProveedores;
			this.navegadorPaginas.CriterioOrdenacion =
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				};
			#endregion
			
			#region Solapa Agregar-Modificar
			// Creo una instancia de "Agregar-Modificar"
			this.solapaModificar = new SolapaAgregarModificar();
			
			// Agrego la pestaña "Agregar-Modificar"
			Gtk.Label etiqueta = new Label("Agregar-Modificar");
			this.nbSolapaProveedores.AppendPage(this.solapaModificar, etiqueta);
			
			// Hago que el navegador de elementos de la solapa Agregar-Modificar
			// se desactive cuando se vacía la lista.
			this.ListaElementosVaciada += delegate {
				this.solapaModificar.NavegadorElementos.Sensitive = false;
			};
			#endregion
			
			#region Suscripción a eventos
			// Me suscribo al evento 'Activated' de ambos campos de busqueda
			// para que se inicie la busqueda cuando presiono 'Enter'
			this.rtxtCodigoBusqueda.Activated += OnBtnBuscarClicked;
			this.rtxtCuitBusqueda.Activated += this.OnBtnBuscarClicked;
			this.txtRazonSocialBusqueda.Activated += OnBtnBuscarClicked;
			
			// Evento de cambio de seleccion de la lista
		 	this.tvProveedores.Selection.Changed += OnTvProveedoresSelectionChanged;
			
			// Evento que se dispara cuando presiono 'Enter' o hago doble-click
			// sobre un elemento de la lista.
			this.tvProveedores.RowActivated += OnBtnModificarClicked;
			
			// Evento para disparar el menú desplegable
			this.tvProveedores.ButtonReleaseEvent += this.OnButtonReleaseEvent;		
			#endregion
			
			#region Criterios de búsqueda
			this.chkCodigo.Toggled += delegate {
				this.rtxtCodigoBusqueda.Sensitive = this.chkCodigo.Active;
			};
			this.chkCodigo.Active = true;
			
			this.chkRazonSocial.Toggled += delegate {
				this.txtRazonSocialBusqueda.Sensitive = this.chkRazonSocial.Active;
			};
			this.chkRazonSocial.Active = true;
			
			this.chkCuit.Toggled += delegate {
				this.rtxtCuitBusqueda.Sensitive = this.chkCuit.Active;
			};
			this.chkCuit.Active = true;
			#endregion
			
			// Creo el servicio de navegacion de elementos
			this.servicioNavegador = new ServicioNavegadorElementos();
			
			// Suscribo el evento 'ListaElementosVaciada' de IListaNavegable al
			// evento 'ListaVaciada' de treeview
			this.tvProveedores.ListaVaciada += delegate {
				if (this.ListaElementosVaciada != null)
					this.ListaElementosVaciada();
			};
			
			// Desactivo el botón eliminar
			this.btnEliminar.Sensitive = false;
			
			// Seteo en null la consulta actual
			this.consultaActual = null;
			
			// Vacío el formulario
			this.VaciarFormularioBusqueda();
			
			// Seteo la variable instancia
			instancia = this;
		}
		
		#region Propiedades
		public static SolapaProveedores Instancia {
			get { return instancia; }
		}
		#endregion
		
		#region Eventos
				
		protected virtual void OnBtnBuscarClicked (object sender, System.EventArgs e)
		{	
			// Chequeo que sea válido el texto ingresado, si no lo es lo notifico
			if (this.chkCodigo.Active && !this.rtxtCodigoBusqueda.TextoValido) {
				SMessageDialog.MensajeError(this.rtxtCodigoBusqueda.MensajeError);
				return;
			}
			
			if (this.chkCuit.Active && !this.rtxtCuitBusqueda.TextoValido) {
				SMessageDialog.MensajeError(this.rtxtCuitBusqueda.MensajeError);
				return;
			}
			
			this.consultaActual = this.CrearQueryObject();
			
			// Creo una consulta paginable
			ConsultaPaginable cp = new ConsultaPaginable();
			cp.pagina = 0;
			cp.tamanioPagina = TAMAÑO_PAGINA;
			cp.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Asc
				}
			};
			
			ProveedorDto[] proveedoresEncontrados = this.EjecutarConsulta(cp);
			
			/* Si no hay proveedores que cumplan con los criterios de busqueda,
			 * notifico al usuario y retorno */
			if (proveedoresEncontrados.Length <= 0) {
				SMessageDialog.MensajeAdvertencia("<b>No hay proveedores que cumplan con los criterios " +
				               			"de busqueda.</b>\nReformule la busqueda e intente nuevamente.");
				return;
			}
					
			// Activo el Navegador de páginas
			this.navegadorPaginas.Sensitive = true;
			
			// Vacío el treeview
			this.tvProveedores.Vaciar();
			
			// Lanzo el evento ListaElementosVaciada
			if (this.ListaElementosVaciada != null)
				this.ListaElementosVaciada();
					
			// Cargo los proveedores encontrados en el treeview
			this.tvProveedores.AgregarElementosPrimeraPagina(proveedoresEncontrados);
		}
		
		[GLib.ConnectBefore]
		protected void OnButtonReleaseEvent(object o, ButtonReleaseEventArgs args)
		{
			// Solo activo el menú si es el botón derecho el presionado y el treeview no está vacío
			if (args.Event.Button == 3)
				this.MostrarMenuEmergente();
		}		
		
		protected virtual void OnBtnModificarClicked(object sender, System.EventArgs e)
		{
			TreeIter[] iters;
			TreeIter iterSeleccionado;
			
			// Chequeo que el proveedor seleccionado no sea nulo
			if (this.tvProveedores.ObtenerSeleccionados(out iters).Length == 0)
				return;
			
			iterSeleccionado = iters[0];
			
			// Obtengo el objeto refrescado
			ProveedorDto proveedorRefrescado = this.ObtenerProveedorRefrescado(iterSeleccionado);
			
			// Actualizo el iter cargado
			this.iterCargado = iterSeleccionado;
			this.paginaIterCargado = this.navegadorPaginas.PaginaActual;
			
			// Actualizo el treeview con el proveedor refrescado
			this.tvProveedores.ActualizarObjetoSeleccionado(proveedorRefrescado);
			
			// Paso a la pestaña 'Agregar/Modificar'
			this.nbSolapaProveedores.CurrentPage = 1;
			
			// Cargo el proveedor en el formulario de modificación
			this.solapaModificar.CargarProveedor(proveedorRefrescado);
		}
		
		protected virtual void OnMiModificarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnModificarClicked(sender, e);
		}

		protected virtual void OnBtnEliminarClicked(object sender, System.EventArgs e)
		{
//			// Chequeo que el proveedor no sea nulo
//			if (this.tvProveedores.PersonasSeleccionadas.Length == 0)
//				return;
//			
//			// Muestro el diálogo de confirmación
//			ResponseType respuesta =
//				SMessageDialog.MensajePregunta("¿Está seguro de eliminar el proveedor " +
//				                               this.tvProveedores.PersonasSeleccionadas[0].razonSocial +
//				                               " del sistema?");
//			
//			// Si la respuesta es 'Si' borro el proveedor seleccionado
//			if (respuesta == ResponseType.Yes) {
//				
//				// Le aviso al servidor que marque el proveedor como eliminado
//				((ControladorProveedoresPrx) MainIce.GetControlador("ControladorProveedores"))
//					.MarcarComoEliminado(this.tvProveedores.PersonaSeleccionada);
//				
//				// Borro el item seleccionado del treeview
//				this.tvProveedores.EliminarObjetoSeleccionado();
//			}
		}
		
		protected virtual void OnMiEliminarActivated(object sender, System.EventArgs e)
		{
			this.OnBtnEliminarClicked(sender, e);
		}
		
		/// <summary>
		/// Este metodo se ejecuta cuando se dispara el evento Selection.Changed del TvProveedores.
		/// Si se ha seleccionado una fila, se activan los botones 'Modificar' y 'Eliminar'.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnTvProveedoresSelectionChanged(object o, System.EventArgs e)
		{
			// Chequeo que el proveedor seleccionado no sea nulo
			if (this.tvProveedores.ObtenerSeleccionados().Length == 0)
				return;
			
			// Activo los botones 'Modificar' y 'Eliminar'
			this.btnModificar.Sensitive = true;
//			this.btnEliminar.Sensitive = true;
		}
		#endregion
		
		#region Otros métodos
		protected ProveedorDto[] EjecutarConsulta(ConsultaPaginable cp)
		{
			// Realizo la búsqueda al servidor
			Ice.Object[] resultadoQueryObjects =
			((ControladorProveedoresPrx) MainIce.GetControlador("ControladorProveedores"))
				.BuscarQueryObjects(this.consultaActual, cp);
			
			ProveedorDto[] proveedoresEncontrados = resultadoQueryObjects.Cast<ProveedorDto>().ToArray();
			
			return proveedoresEncontrados;
		}
		
		protected object[] CargadorProveedores (ConsultaPaginable cp)
		{
			return this.EjecutarConsulta(cp);	
		}
		
		/// <summary>
		/// Crea el objeto QueryObject con los criterio de busquedas
		/// actualmente ingresados en la interfaz.
		/// </summary>
		/// <returns>
		/// A <see cref="ProveedorDtoQuery"/>
		/// </returns>
		protected ProveedorDtoQuery CrearQueryObject()
		{
			ProveedorDtoQuery consulta = new ProveedorDtoQuery();
			consulta.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioSimple temp;
			
			/* Si el campo código no es vacío, busco por código,
			 * ya que es una busqueda mas precisa */
			if (this.chkCodigo.Active && !this.rtxtCodigoBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioOr criterioCodigo = new CriterioOr();
				criterioCodigo.criterios = new List<CriterioGenerico>();
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "Codigo";
				temp.valor = this.rtxtCodigoBusqueda.Text.Trim() + "L";
				
				criterioCodigo.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioCodigo);
			}
			
			// La búsqueda por cuit también es precisa, así que si está activada
			// busco por CUIT
			if (this.chkCuit.Active && !this.rtxtCuitBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioOr criterioCuit = new CriterioOr();
				criterioCuit.criterios = new List<CriterioGenerico>();
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "Cuit.ValorNumerico";
				temp.valor = this.rtxtCuitBusqueda.Text.Trim().Replace("-", "") + "L";
				
				criterioCuit.criterios.Add(temp);
				consulta.listaCriteriosComplejos.Add(criterioCuit);
			}
			
			// Si no hay nada ingresado en el campo Código, consulto por Razon Social
			else if (this.chkRazonSocial.Active && !this.txtRazonSocialBusqueda.Text.Trim().Equals(string.Empty)) {
				
				CriterioAnd criterioRazonSocial = new CriterioAnd();
				criterioRazonSocial.criterios = new List<CriterioGenerico>();
				
				foreach (string palabra in this.txtRazonSocialBusqueda.Text.Split(' ')) {
					temp = new CriterioSimple();
					temp.operador = TipoOperador.LikeMatchAnywhere;
					temp.propiedad = "RazonSocial";
					temp.valor = palabra.ToUpper();
					
					criterioRazonSocial.criterios.Add(temp);
				}
				
				consulta.listaCriteriosComplejos.Add(criterioRazonSocial);
			}
					
			// Agrego un criterio para que no me devuelva los proveedores marcados como 'Eliminados'
			CriterioAnd criterioEliminado = new CriterioAnd();
			criterioEliminado.criterios = new List<CriterioGenerico>();
			
			temp = new CriterioSimple();
			temp.operador = TipoOperador.Igual;
			temp.propiedad = "Eliminado";
			temp.valor = bool.FalseString;
			
			criterioEliminado.criterios.Add(temp);
			consulta.listaCriteriosComplejos.Add(criterioEliminado);
		
			return consulta;
		}
		
		/// <summary>
		/// Muestra el menú emergente en la lista de facturas de compras
		/// </summary>
		protected void MostrarMenuEmergente()
		{
			Menu menu = new Menu();
			ImageMenuItem menuItem;
			
			menuItem = new ImageMenuItem("Modificar");
			menuItem.Image = Image.NewFromIconName("gtk-edit", IconSize.Menu);
			menuItem.Activated += this.OnMiModificarActivated;
			menu.Append(menuItem);
			
//			menuItem = new ImageMenuItem("Eliminar");
//			menuItem.Image = Image.NewFromIconName("gtk-remove", IconSize.Menu);
//			menuItem.Activated += this.OnMiEliminarActivated;
//			menu.Append(menuItem);
			
			// Muestro el menú
			menu.Popup();
			menu.ShowAll();
		}
		
		/// <summary>
		/// Vacía el formulario de busqueda: los campos de filtrado y los
		/// resultados de la busqueda anterior.
		/// </summary>
		protected void VaciarFormularioBusqueda()
		{
			// Limpio los campos de filtrado
			this.rtxtCodigoBusqueda.Text = "";
			this.txtRazonSocialBusqueda.Text = "";
			
			// Vacío la lista de proveedores
			this.tvProveedores.Vaciar();
			
			// Si hay suscripciones, lanzo el evento ListaElementosVaciada
			if (this.ListaElementosVaciada != null)
				this.ListaElementosVaciada();
		}
		
		/// <summary>
		/// Obtiene el Proveedor refrescado del proveedor correspondiente
		/// al iter de la lista de proveedores pasado como parametro.
		/// </summary>
		/// <param name="iter">
		/// A <see cref="TreeIter"/>
		/// </param>
		/// <returns>
		/// A <see cref="ProveedorDto"/>
		/// </returns>
		private ProveedorDto ObtenerProveedorRefrescado (TreeIter iter)
		{
			// LLamo al metodo 'Refrescar' del controladora de proveedores.
			Ice.Object resultado = ((ControladorProveedoresPrx) MainIce.GetControlador("ControladorProveedores"))
				.Refrescar((ProveedorDto)this.tvProveedores.Model.GetValue(iter, 0));
			
			// Retorno el proveedor refrescado
			return (ProveedorDto) resultado;
		}
		#endregion
		
		#region IListaNavegable implementation
		public event System.Action ListaElementosVaciada;
		
		public object ObtenerObjetoRefrescado (TreeIter iter)
		{
			return this.ObtenerProveedorRefrescado(iter);
		}
		
		public ServicioNavegadorElementos ServicioNavegador {
			get {
				this.servicioNavegador.ListaNavegable = this;
				return this.servicioNavegador;
			}
		}
		
		public TreeIter IterCargado {
			get { return this.iterCargado; }
			set { this.iterCargado = value; }
		}
		
		public int PaginaIterCargado {
			get { return this.paginaIterCargado; }
			set { this.paginaIterCargado = value; }
		}
		
		public ColeccionElementos ListaElementos {
			get { return this.tvProveedores; }
		}

		public NavegadorPaginas NavegadorDePaginas {
			get { return this.navegadorPaginas; }
		}
		#endregion
	}
}
