// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;

using Gtk;

using StratosClient.Reportes;
using StratosClient.Core.Gui;
using StratosClient.Common;
using Stratos.Common;
using StratosServer.Controladores;
using Stratos.Common.Excepciones;

namespace StratosClient.Reportes
{
	/// <summary>
	/// Clase que centraliza las funcionalidades para generar en el lado
	/// del cliente los reportes de IVA.
	/// </summary>
	public abstract class ReporteIva : ReporteGenerico
	{
		#region Campos
		protected DateTime fechaDesde;
		protected DateTime fechaHasta;
		#endregion
		
		#region Constructores
		public ReporteIva() : base()
		{
		}
		#endregion
		
		#region ReporteGenerico implementation
		protected override Gtk.Dialog DialogoReporte {
			get {
				// Creo un diálogo para permitir al usuario seleccionar el rango de fechas
				DialogoReporteIva dialogoReporteIva =
					new DialogoReporteIva("Generar " + this.NombreReporte,
					                      MainWindow.Instancia);
				
				dialogoReporteIva.Title = "Generar " + this.NombreReporte;
				
				return dialogoReporteIva;
			}
		}
		
		protected override void LeerDatos (Dialog dialogoReporte)
		{
			DialogoReporteIva dialogoReporteIva =
				(DialogoReporteIva) dialogoReporte;
			
			this.fechaDesde = dialogoReporteIva.FechaDesde;
			this.fechaHasta = dialogoReporteIva.FechaHasta;
		}
		
		protected override void ConfigurarImpresionDocumento (ImpresionDocumento impresionDocumento)
		{
			/* Cuando el usuario escoge imprimir (presiona el botón imprimir) ejecuto
			 * un método especial que pregunta si desea pasar las facturas a históricas. */
			impresionDocumento.AccionImprimirElegida += this.PreguntarParaMarcarFacturasComoHistoricas;
		}
		#endregion
		
		#region Métodos
		private void PreguntarParaMarcarFacturasComoHistoricas()
		{
			Gtk.Application.Invoke(delegate {
				// Pregunto al usuario si quiere marcar las facturas incluídas en el
				// Reporte de Iva como históricas.
				ResponseType respuestaFacturasHistoricas =
					SMessageDialog
						.MensajePregunta("<b>¿Desea pasar las facturas de este " + this.NombreReporte + " a históricas?</b>\n\n" +
						                 "Las facturas históricas no pueden anularse ni eliminarse.",
						                 ResponseType.Yes);
				
				if (respuestaFacturasHistoricas != ResponseType.Yes)
					return;
				
				// Marco las facturas como históricas
				try {
					this.MarcarFacturasComoHistoricas();
					
					SMessageDialog
						.MensajeInformacion("Las facturas desde '" + this.fechaDesde.ToShortDateString() +
						                    "' hasta '" + this.fechaHasta.ToShortDateString() + "' fueron " +
						                    "pasadas como históricas.");
				}
				catch (StratosException excepcion) {
					SMessageDialog.MensajeError("Hubo un error al pasar las facturas a históricas", excepcion);
				}
			});
		}
		
		/// <summary>
		/// Marca las facturas como históricas, según los datos leídos de las fechas.
		/// </summary>
		/// <param name="fechaInicio">
		/// A <see cref="DateTime"/>
		/// </param>
		/// <param name="fechaFinal">
		/// A <see cref="DateTime"/>
		/// </param>
		protected abstract void MarcarFacturasComoHistoricas();
		
		/// <summary>
		/// Devuelve el nombre del reporte.
		/// </summary>
		protected abstract string NombreReporte {
			get;
		}
		#endregion
	}
}
