// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using Stratos.Servicios;
using StratosClient.ServicioFacturacion.Configuracion;

namespace StratosClient.ServicioFacturacion
{
	public class CargaDatosTerminalListener :  ICargaDatosTerminal
	{

		#region ICargaDatosTerminal implementation
		public void CargarDatosTerminal ()
		{
			CargarDatos();
		}
		#endregion
		
		/// <summary>
		/// Setea los datos de impresión de la estructura DatosTerminal.
		/// </summary>
		public static void CargarDatos()
		{
			DatosTerminal datosTerminal = MainIce.DatosTerminal;
			datosTerminal.datosTermImpresora = new DatosTerminalImpresora();
			
			List<TipoDocumento> listaTiposDocumento = new List<TipoDocumento>();
			
			// Cargo todos los tipos de documentos que se pueden imprimir en esta terminal
			foreach(Impresora impresora in ServicioFacturacionElement.Instancia.Impresoras) {
				foreach(Documento doc in impresora.TiposDocumento) {
					if (!listaTiposDocumento.Contains(doc.Tipo))
						listaTiposDocumento.Add(doc.Tipo);
				}
			}
			
			datosTerminal.datosTermImpresora.tiposDocumento = listaTiposDocumento.ToArray();
			
			/* Cargamos el proxy del servicio de facturación solamente si hay algun tipo
			 * de documento soportado para imprimir */
			if (datosTerminal.datosTermImpresora.tiposDocumento.Length > 0) {
				Dictionary<string, Ice.ObjectPrx> proxies =
					MainIce.ProxiesServiciosServidores;
				
				List<ServicioFacturacionPrx> servFacturacionList =
					new List<ServicioFacturacionPrx>();
				
				servFacturacionList.Add(ServicioFacturacionPrxHelper
				                        .uncheckedCast(proxies["ServicioFacturacion"]));
				
				datosTerminal.datosTermImpresora.servFacturacion = servFacturacionList;
			}
		}
	}
}
