// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Configuration;

using Stratos.ModeloDto;

namespace StratosClient.ServicioFacturacion.Configuracion
{
	/// <summary>
	/// Colección de elementos Documento.
	/// </summary>
	[ConfigurationCollection(typeof(Documento),
	                         CollectionType = ConfigurationElementCollectionType.BasicMap,
	                         AddItemName = "documento")]
	public class DocumentosCollection : ConfigurationElementCollection
	{
		public override ConfigurationElementCollectionType CollectionType
		{
			get {
				return ConfigurationElementCollectionType.BasicMap;
			}
		}
		
		protected override ConfigurationElement CreateNewElement()
		{
			return new Documento();
		}
		
		protected override Object GetElementKey(ConfigurationElement element)
		{
			return ((Documento)element).Tipo;
		}
		
		protected override string ElementName {
			get { return "documento"; }
		}
		
		public int IndexOf(Impresora impresora)
		{
			return BaseIndexOf(impresora);
		}
		
		public Documento this[int index]
		{
			get { return (Documento)BaseGet(index); }
			set {
				if (BaseGet(index) != null)
				    BaseRemoveAt(index);
				
				BaseAdd(index, value);
			}
		}
		
		public new Documento this[TipoDocumento name]
		{
			get { return (Documento)BaseGet(name); }
		}
		
		public void Add(Documento documento)
		{
			BaseAdd(documento);
		}
		
		protected override void BaseAdd(ConfigurationElement element)
		{
			base.BaseAdd(element, false);
		}
		
		public void Remove(Documento documento)
		{
			BaseRemove(documento.Tipo);
		}
	}
}
