// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing.Printing;

namespace StratosClient.ServicioFacturacion.Configuracion
{
	/// <summary>
	/// Colección de elementos tipo Impresora.
	/// </summary>
	[ConfigurationCollection(typeof(Impresora),
	                         CollectionType = ConfigurationElementCollectionType.BasicMap,
	                         AddItemName = "impresora")]
	public class ImpresorasCollection : ConfigurationElementCollection
	{
		public override ConfigurationElementCollectionType CollectionType
		{
			get {
				return ConfigurationElementCollectionType.BasicMap;
			}
		}
		
		protected override ConfigurationElement CreateNewElement()
		{
			return new Impresora();
		}
		
		
		protected override ConfigurationElement CreateNewElement(string elementName)
		{
			return new Impresora(elementName);
		}
		
		
		protected override Object GetElementKey(ConfigurationElement element)
		{
			return ((Impresora)element).Nombre;
		}
		
		protected override string ElementName {
			get { return "impresora"; }
		}
		
		public int IndexOf(Impresora impresora)
		{
			return BaseIndexOf(impresora);
		}
		
		public Impresora this[int index]
		{
			get { return (Impresora)BaseGet(index); }
			set {
				if (BaseGet(index) != null)
				    BaseRemoveAt(index);
				
				BaseAdd(index, value);
			}
		}
		
		public new Impresora this[string name]
		{
			get { return (Impresora)BaseGet(name); }
		}
		
		public void Add(Impresora impresora)
		{
			BaseAdd(impresora);
		}
		
		protected override void BaseAdd(ConfigurationElement element)
		{
			base.BaseAdd(element, false);
		}
		
		public void Remove(Impresora impresora)
		{
			BaseRemove(impresora.Nombre);
		}
		
		protected override void PostDeserialize ()
		{
			base.PostDeserialize ();
			
			// Elimino las impresoras que no existen en el sistema
			Impresora[] impresorasActuales = this.Cast<Impresora>().ToArray();
			IEnumerable<string> impresorasExistentes = PrinterSettings.InstalledPrinters.Cast<string>();
			
			foreach (Impresora impresora in impresorasActuales)
				if (!impresorasExistentes.Contains(impresora.Nombre))
					this.Remove(impresora);
		}
	}
}
