// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;

using StratosClient.Core.Gui.Configuracion;
using Stratos.ModeloDto;

namespace StratosClient.ServicioFacturacion.Configuracion
{
	/// <summary>
	/// Elemento principal de configuración del módulo Servicio de Facturación.
	/// Posee propiedades para indicar la configuración de las impresoras.
	/// </summary>
	public class ServicioFacturacionElement : ConfigurationElement
	{
		public static ServicioFacturacionElement Instancia {
			get {
				return (ServicioFacturacionElement) ConfiguracionStratos
					.GetConfigurationElement(NombreSeccion);
			}
		}
		
		#region Propiedades
		public static string NombreSeccion {
			get { return "ServicioFacturacion"; }
		}
		
		[ConfigurationProperty("impresionAutomatica",
		                       IsRequired = false,
		                       DefaultValue = false)]
		public bool ImpresionAutomatica {
			get { return (bool) base["impresionAutomatica"]; }
			set { base["impresionAutomatica"] = value; }
		}
		
		[ConfigurationProperty("Impresoras",
		                       IsRequired = false,
		                       IsDefaultCollection = true)]
		public ImpresorasCollection Impresoras {
			get { return (ImpresorasCollection) base["Impresoras"]; }
		}
		#endregion
		
		#region Métodos
		/// <summary>
		/// Según el tipo de documento dado como parámetro, busca todas las
		/// impresoras que pueden imprimirlo y devuelve una lista de las
		/// mismas.
		/// </summary>
		/// <param name="tipoDocumento">
		/// A <see cref="TipoDocumento"/>
		/// </param>
		/// <returns>
		/// A <see cref="List"/>
		/// </returns>
		public List<Impresora> BuscarImpresora(TipoDocumento tipoDocumento)
		{
			List<Impresora> listaImpresoras = new List<Impresora>();
			
			foreach(Impresora impresora in this.Impresoras)
				foreach(Documento doc in impresora.TiposDocumento)
					if (doc.Tipo == tipoDocumento)
						listaImpresoras.Add(impresora);
			
			return listaImpresoras;
		}
		#endregion
	}
}
