// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;
using Pango;
using Cairo;
using log4net;

using Stratos.Common;
using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosClient.ServicioFacturacion.Configuracion;

namespace StratosClient.ServicioFacturacion
{
	[Obsolete("Ahora usamos WebKit, pero dejo esta clase por las dudas")]
	public class ImpresionCairo
	{
		#region Campos
		private static readonly ILog logger =
			LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod()
			                     .DeclaringType);
		
		private PrintOperation printOperation;
		
		private PreFacturaDto factura;
		private long numeroFactura;
		private Documento configuracionDocumento;
		#endregion
		
		#region Constructores
		public ImpresionCairo(PreFacturaDto factura,
		                      long numeroFactura,
		                      Documento configuracionDocumento)
		{
			this.factura = factura;
			this.numeroFactura = numeroFactura;
			this.configuracionDocumento = configuracionDocumento;
		}
		#endregion
		
		#region Propiedades
		public PrintOperation PrintOperation {
			get { return this.printOperation; }
		}
		#endregion
		
		#region Métodos comunes
		public void EjecutarImpresion(string nombreImpresora,
		                              DoneHandler impresionFinalizadaHandler)
		{
			this.EjecutarImpresion(nombreImpresora, impresionFinalizadaHandler, null);
		}
		
		public void EjecutarImpresion(string nombreImpresora,
		                              DoneHandler impresionFinalizadaHandler,
		                              EventHandler estadoImpresionCambiado)
		{
			logger.Debug("Ejecutando impresión");
			logger.Debug("Creando objeto PrintOperation");
			
			this.printOperation = new PrintOperation();
			
			this.printOperation.AllowAsync = false;
			
			if (impresionFinalizadaHandler != null)
				this.printOperation.Done += impresionFinalizadaHandler;
			
			if (estadoImpresionCambiado != null)
				this.printOperation.StatusChanged += estadoImpresionCambiado;
			
			this.printOperation.JobName =
				this.factura.tipo.ToString() + " - " +
					(new DateTime(this.factura.fechaTicks)).ToShortDateString() + " - " +
					this.factura.cliente.razonSocial;
			
			// PageSetup
			PageSetup pageSetup = new PageSetup();
			
			PaperSize paperSize = new PaperSize("Custom", "Custom",
			                                    223, 202, Unit.Mm);
			
			pageSetup.PaperSize = paperSize;
			pageSetup.Orientation = PageOrientation.Portrait;
			
			this.printOperation.DefaultPageSetup = pageSetup;
			
			// PrintSettings
			this.printOperation.PrintSettings = new PrintSettings();
			this.printOperation.PrintSettings.Printer = nombreImpresora;
			this.printOperation.PrintSettings.UseColor = false;
			
			this.printOperation.UseFullPage = true;
			this.printOperation.Unit = Unit.Mm;
			
			this.printOperation.DrawPage += this.DrawPage;
			this.printOperation.BeginPrint += this.BeginPrint;
			
			// Ejecuto la operación PrintOperation
			Gtk.Application.Invoke(delegate {
				logger.Debug("Ejecutando PrintOperation.Run");
				
				this.printOperation.Run(PrintOperationAction.Print, MainGui.MainWindow);
				//this.printOperation.Dispose();
				
				logger.Debug("PrintOperation.Run finalizado");
			});
		}
		#endregion
		
		#region Métodos GtkPrint
		private void BeginPrint(object o, Gtk.BeginPrintArgs args)
		{
			logger.Debug("BeginPrint iniciado");
			
			PrintOperation printOperation = (PrintOperation) o;
			
			printOperation.NPages = 1;
			
			logger.Debug("BeginPrint finalizado");
		}
		
		private void DrawPage(object o, DrawPageArgs args)
		{
			logger.Debug("DrawPage iniciado");
			
			Pango.Layout pangoLayout =
				args.Context.CreatePangoLayout();
			
			Cairo.Context cairoContext =
				args.Context.CairoContext;
			
			/* Traslado el origen según la configuración del documento */
			double leftOffset = this.configuracionDocumento.LeftOffset;
			double topOffset = this.configuracionDocumento.TopOffset;
			
			cairoContext.Translate(leftOffset, topOffset);

			// Color negro
			cairoContext.Color = new Cairo.Color(0, 0, 0);

			// Fuente por defecto
			FontDescription fontDescription =
				FontDescription.FromString("Arial 8");
			//fontDescription.Style = Pango.Style.Normal;
			pangoLayout.FontDescription = fontDescription;

			#region Datos de la factura
			// Número de factura
			PointD pNroFactura = new PointD(165, 26);
			cairoContext.MoveTo(165,15);//(pNroFactura);
			pangoLayout.SetText(this.numeroFactura.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Fecha
			PointD pFecha = new PointD(163, 29);
			cairoContext.MoveTo(pFecha);
			DateTime fechaFactura = new DateTime(this.factura.fechaTicks);
			pangoLayout.SetText(fechaFactura.ToShortDateString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Comprobante (factura)
			PointD pComprobante = new PointD(163, 34);
			cairoContext.MoveTo(pComprobante);
			// Me fijo si no estoy imprimiendo una nota de crédito
			if (this.factura is PreNotaCreditoDto)
				pangoLayout.SetText("NOTA DE CRÉDITO");
			else
				pangoLayout.SetText("FACTURA");
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);
			#endregion

			#region Datos del cliente
			// Razón social
			PointD pRazonSocial = new PointD(50, 60);
			cairoContext.MoveTo(pRazonSocial);
			pangoLayout.SetText(this.factura.cliente.razonSocial.ToUpper());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Código (Nro de cuenta)
			PointD pNroCuenta = new PointD(157, 60);
			cairoContext.MoveTo(pNroCuenta);
			pangoLayout.SetText(this.factura.cliente.codigo.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Domicilio
			// FIXME: Hay que revisar que no se pase de largo
			PointD pDomicilio = new PointD(43, 68);
			cairoContext.MoveTo(pDomicilio);
			pangoLayout.SetText(this.factura.cliente.domicilio.direccion.ToUpper() +
			                    " - " + this.factura.cliente.domicilio.localidad.ToUpper() +
			                    " - " + this.factura.cliente.domicilio.provincia.ToString().ToUpper());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Condición de venta
			PointD pCondVenta = new PointD(160, 64);
			cairoContext.MoveTo(pCondVenta);
			pangoLayout.SetText(this.factura.condVenta.condPago.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);
			
			// FIXME: Faltaría el vencimiento por acá

			// IVA
			PointD pIva = new PointD(35, 77);
			cairoContext.MoveTo(pIva);
			string tipoReposable =
				StratosUtil
					.SplitCamelString(this.factura.cliente.tipoRes.ToString()).ToUpper();
			pangoLayout.SetText(tipoReposable);
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// CUIT
			PointD pCuit = new PointD(98, 77);
			cairoContext.MoveTo(pCuit);
			pangoLayout.SetText(this.factura.cliente.cuit);
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);
			#endregion

			#region Items de la factura
			PointD pItem = new PointD(28, 89);
			
			foreach(ItemProductoDto item in this.factura.items) {
				// Código del producto
				cairoContext.MoveTo(pItem);
				pangoLayout.SetText(item.productoCodigo.ToString());
				Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

				// Cantidad
				cairoContext.MoveTo(53, pItem.Y);
				pangoLayout.SetText(item.cantidad.ToString());
				Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

				// Descripción del producto
				cairoContext.MoveTo(73, pItem.Y);
				pangoLayout.SetText(item.descripcion);
				Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

				// Precio unitario
				cairoContext.MoveTo(175, pItem.Y);
				pangoLayout.SetText(item.precioUnitario.ToString());
				Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

				// Importe
				cairoContext.MoveTo(197, pItem.Y);
				pangoLayout.SetText(item.importe.ToString());
				Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

				pItem = new PointD(pItem.X, pItem.Y + 3);
			}
			#endregion

			#region Importes totales
			// Importe en letras
			PointD pImporteLetras = new PointD(60, 165);
			cairoContext.MoveTo(pImporteLetras);
			NumeroLetras nl = new NumeroLetras();
			pangoLayout.SetText("SON PESOS: " +
			                    nl.enletras(this.factura.importeTotal.ToString()));
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Importe bruto (Subtotal)
			PointD pImporteBruto = new PointD(30, 182);
			cairoContext.MoveTo(pImporteBruto);
			pangoLayout.SetText(this.factura.importeBruto.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Importe bonificación (Bonif)
			PointD pImporteBonificacion = new PointD(63, 182);
			cairoContext.MoveTo(pImporteBonificacion);
			pangoLayout.SetText(this.factura.importeBonificacion.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Importe neto (importe bruto - bonificación)
			PointD pImporteNeto = new PointD(95, 182);
			cairoContext.MoveTo(pImporteNeto);
			pangoLayout.SetText(this.factura.importeNeto.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Importe IVA inscripto (IVA insc)
			PointD pImporteIvaInscripto = new PointD(127, 182);
			cairoContext.MoveTo(pImporteIvaInscripto);
			pangoLayout.SetText(this.factura.importeIva.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);

			// Importe total
			PointD pImporteTotal = new PointD(193, 182);
			cairoContext.MoveTo(pImporteTotal);
			pangoLayout.SetText(this.factura.importeTotal.ToString());
			Pango.CairoHelper.ShowLayout(cairoContext, pangoLayout);
			#endregion

			pangoLayout.Dispose();
			fontDescription.Dispose();
			((IDisposable) cairoContext.Target).Dispose();
			((IDisposable) cairoContext).Dispose();

			logger.Debug("DrawPage finalizado");
		}
		#endregion
	}
}
