// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using StratosClient.Common;
using StratosClient.ServicioFacturacion.Configuracion;
using Stratos.ModeloDto;
using StratosClient.Core.Gui.ImpresionBorrador;

namespace StratosClient.ServicioFacturacion
{
	public class ImpresionWebKit
	{
		#region Campos
		private FacturaDto factura;
		
		private readonly string DIRECTORIO_ADDIN;
		#endregion
		
		#region Constructores
		public ImpresionWebKit (PreFacturaDto preFactura, long numeroFactura)
		{
			DIRECTORIO_ADDIN =
				"Addins" + System.IO.Path.DirectorySeparatorChar +
				"ServicioFacturacion" + System.IO.Path.DirectorySeparatorChar;
			
			/* Creo un objeto FacturaDto o NotaCreditoDto (según corresponda)
			 * a partir de la PreFacturaDto y del número dado */
			if (preFactura is PreNotaCreditoDto) {
				NotaCreditoDto notaCredito = new NotaCreditoDto();
				notaCredito.facturaAsociada = (preFactura as PreNotaCreditoDto).facturaAsociada;
				
				this.factura = notaCredito;
			}
			else
				this.factura = new FacturaDto();
			
			this.factura.anulada = false;
			this.factura.cliente = preFactura.cliente;
			
			// FIXME: El código no debería estar hardcodeado
			this.factura.codigo = 4;
			
			this.factura.condVenta = preFactura.condVenta;
			this.factura.facturaManual = false;
			this.factura.fechaTicks = preFactura.fechaTicks;
			
			// Importes
			this.factura.importeBonificacion = preFactura.importeBonificacion;
			this.factura.importeBruto = preFactura.importeBruto;
			this.factura.importeFinal = preFactura.importeFinal;
			this.factura.importeIva = preFactura.importeIva;
			this.factura.importeNeto = preFactura.importeNeto;
			this.factura.importeTotal = preFactura.importeTotal;
			
			// Otros datos
			this.factura.items = preFactura.items;
			this.factura.numero = numeroFactura;
			this.factura.observaciones = preFactura.observaciones;
			this.factura.porcentajeBonificacion = preFactura.porcentajeBonificacion;
			this.factura.remitos = preFactura.remitos;
			this.factura.tipo = preFactura.tipo;
			this.factura.valida = true;
		}
		#endregion
		
		#region Métodos
		public void EjecutarImpresion (string nombreImpresora, System.Action<PrintStatus> impresionFinalizada)
		{
			// Obtengo la sección de configuración de ServicioFacturacion
			ServicioFacturacionElement section = ServicioFacturacionElement.Instancia;
			
			// Obtengo de la configuración de la impresora para las facturas
			// los datos de la página
			Impresora impresora = section.Impresoras[nombreImpresora];
			
			// Obtengo de la impresora el nombre del archivo de template a usar
			// para la misma
			string archivoTemplate =
				DIRECTORIO_ADDIN + impresora.ImpresionFacturas.Template;
			
			// Creo el objeto de impresión
			ImpresionFactura impresionFactura =
				new ImpresionFactura(archivoTemplate,
				                     DIRECTORIO_ADDIN,
				                     "factura",
				                     this.factura,
				                     nombreImpresora);
			
			// Si se especificó un método para ejecutarse al finalizar la impresión:
			if (impresionFinalizada != null) {
				impresionFactura.ImpresionFinalizada += delegate (PrintStatus printStatus) {
					impresionFinalizada(printStatus);
				};
			}
			
			// Imprimo la factura
			try {
				impresionFactura.Imprimir();
			}
			catch {
				// Si hay algún problema, ejecuto el delegate de impresión finalizada
				impresionFinalizada(PrintStatus.FinishedAborted);
				
				throw;
			}
		}
		#endregion
	}
}
