// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Drawing.Printing;

using Gtk;

using CustomWidgets;
using StratosClient.Core.Gui;
using StratosClient.Core.Gui.Configuracion;
using StratosClient.ServicioFacturacion.Configuracion;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using Stratos.QueryObjects;
using Stratos.Common.Excepciones;

namespace StratosClient.ServicioFacturacion
{
	public partial class SolapaServicioImpresion : Gtk.Bin
	{
		private const int TAMAÑO_PAGINA = 9;
		
		private TvPreFacturas documentosPorImprimir;
		
		private bool cargandoDatosImpresora = false;
		private bool cargandoDatosGenerales = false;
		
		private Impresora impresoraSeleccionada;
		
		private long numeroFactura;
		
		public SolapaServicioImpresion()
		{
			this.Build();
			
			#region Lista de documentos por imprimir
			this.documentosPorImprimir = new TvPreFacturas();
			this.documentosPorImprimir.WidthRequest = 600;
			this.documentosPorImprimir.HeightRequest = 200;
			this.scrolledwindow1.Add(this.documentosPorImprimir);
			this.documentosPorImprimir.ShowAll();
			#endregion
			
			#region Navegador de páginas
//			this.navegadorPaginas.TamañoPagina = TAMAÑO_PAGINA;
//			this.navegadorPaginas.ColeccionElementos = this.documentosPorImprimir;
//			this.navegadorPaginas.CargadorDatos =
//				this.CargadorFacturasPorImprimir;
			#endregion
			
			// Me suscribo al evento ServicioFacturacionI.PedidoImpresion
			ServicioFacturacionI.PedidoImpresion += this.PedidoImpresionRealizado;
			
			#region TreeView de documentos enviados a imprimir
			this.documentosPorImprimir.tvcCliente.Expand = true;
			this.documentosPorImprimir.Selection.Changed += this.DocumentoSeleccionado;
			#endregion
			
			#region Setup solapa "configuración general"
			this.cargandoDatosGenerales = true;
			
			this.chkImpresionAutomatica.Active = ServicioFacturacionElement
				.Instancia.ImpresionAutomatica;
			
			this.cargandoDatosGenerales = false;
			#endregion
			
			#region Setup solapa "configuración impresoras"
			// ComboBox de impresoras disponibles en el sistema
			ListStore modeloImpresoras = new ListStore(typeof(string));
			
			// Cargo el modelo con las impresoras disponibles
			foreach (string impresora in PrinterSettings.InstalledPrinters)
				modeloImpresoras.AppendValues(impresora);
			
			this.cmbImpresoras.Model = modeloImpresoras;
			
			// Creo el CellRenderer para el ComboBox
			CellRendererText renderer = new CellRendererText();
			this.cmbImpresoras.PackStart(renderer, true);
			
			this.cmbImpresoras.SetCellDataFunc(renderer, this.RenderPrinter);
			#endregion
		}
		
		private object[] CargadorFacturasPorImprimir(ConsultaPaginable consultaPaginable)
		{
			// Pido al servidor todas las prefacturas que esta terminal puede imprimir
			PreFacturaDto[] resultado =
				((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
						.ObtenerPreFacturasEncoladasParaImprimir(MainIce.DatosTerminal.identificador,
					                                         consultaPaginable);
			
			return resultado;
		}
		
		#region Eventos de impresión
		private void PedidoImpresionRealizado(PreFacturaDto factura)
		{
			this.documentosPorImprimir.AgregarElementos(factura);
		}
		
		protected virtual void OnImprimirClicked (object sender, System.EventArgs e)
		{
			ResponseType respuestaImpresion =
				SMessageDialog.MensajePregunta("¿Seguro que desea imprimir la factura?");
			
			if (respuestaImpresion != ResponseType.Yes)
				return;
			
			// Escribo el estado actual y deshabilito parte de la interfaz gráfica
			this.lblEstadoActual.Text = "Imprimiendo factura";
			
			this.fraImpresionDocumentos.Sensitive = false;
			this.fraConfiguracion.Sensitive = false;
			
			while (Gtk.Application.EventsPending())
				Gtk.Application.RunIteration();
			
			ControladorFacturacionPrx controladorFacturacion =
				((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"));
			
			// Obtengo la PreFacturaDto que se ha seleccionado para imprimir
			PreFacturaDto factura = (PreFacturaDto)
				this.documentosPorImprimir.ObtenerSeleccionados()[0];
			
			// Refresco la instancia de la factura
			factura = (PreFacturaDto) controladorFacturacion.Refrescar(factura);
			
			// Obtengo el tipo de documento para buscar la impresora configurada
			TipoDocumento tipoDocumento = factura.tipo == FacturaTipoVenta.FacturaA ?
				TipoDocumento.FacturaTipoA :TipoDocumento.FacturaTipoB;
			
			// Busco una impresora que soporte la impresión del tipo de documento seleccionado
			List<Impresora> impresoras = ServicioFacturacionElement
				.Instancia.BuscarImpresora(tipoDocumento);
			
			if (impresoras.Count == 0) {
				SMessageDialog.MensajeError("No hay impresoras configuradas para imprimir el tipo de " +
				                            "documento seleccionado. Si cambió la configuración, " +
				                            "presione el botón 'Refrescar'.");
				
				this.lblEstadoActual.Text = "En espera";
				this.fraImpresionDocumentos.Sensitive = true;
				this.fraConfiguracion.Sensitive = true;
				
				return;
			}
			
			/* Me comunico con el servidor para conocer cuál es el siguiente número de factura
			 * del mismo tipo y código. Si hay un error (por ejemplo, la PreFactura es inválida)
			 * muestro el mensaje y salgo. */
			try {
				controladorFacturacion
						.PrepararPreFacturaParaImprimir(factura, out this.numeroFactura);
			}
			catch (ImpresionFacturaException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				
				this.lblEstadoActual.Text = "En espera";
				this.fraImpresionDocumentos.Sensitive = true;
				this.fraConfiguracion.Sensitive = true;
				
				return;
			}
			catch {
				this.lblEstadoActual.Text = "En espera";
				this.fraImpresionDocumentos.Sensitive = true;
				this.fraConfiguracion.Sensitive = true;
				
				throw;
			}
			
			// Realizo la impresión de la PreFactura seleccionada
			
			// Primero muestro el diálogo de resultado de impresión
			ResultadoImpresion resultadoImpresionDialog =
					new ResultadoImpresion(this.RespuestaImpresion);
			
			Gtk.Application.Invoke(delegate {
				resultadoImpresionDialog.Run();
				resultadoImpresionDialog.Destroy();
			});
			
			Impresora configImpresora = impresoras[0];
			Documento configDocumento = configImpresora.BuscarDocumento(tipoDocumento);
			
			ImpresionWebKit objetoImpresion =
				new ImpresionWebKit(factura, this.numeroFactura);
			
			try {
				objetoImpresion
					.EjecutarImpresion(configImpresora.Nombre,
					                   delegate (PrintStatus printStatus) {
									   		resultadoImpresionDialog.HabilitarAcciones(printStatus);
									   });
			}
			catch (NVelocity.Exception.ParseErrorException ex) {
				Gtk.Application.Invoke(delegate {
					// Cierro el diálogo de impresión
					resultadoImpresionDialog.Destroy();
					
					// Muestro el mensaje de error
					SMessageDialog
						.MensajeError("Hubo un error al imprimir la factura. Asegúrese que el " +
						              "archivo de template sea válido", ex);
				});
			}
			finally {
				Gtk.Application.Invoke(delegate {
					this.lblEstadoActual.Text = "En espera";
					this.fraImpresionDocumentos.Sensitive = true;
					this.fraConfiguracion.Sensitive = true;
				});
			}
		}
		
		protected void RespuestaImpresion(bool impresionCorrecta)
		{
			if (impresionCorrecta) {
				/* Si el usuario indicó que la Factura se imprimió correctamente, entonces
				 * le aviso al servidor y quito la PreFactura impresa de la lista. */
				
				TreeIter[] iters;
				TreeIter iterSeleccionado;
				
				PreFacturaDto factura = (PreFacturaDto)
					this.documentosPorImprimir.ObtenerSeleccionados(out iters)[0];
				
				iterSeleccionado = iters[0];
				
				try {
					((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
							.ImpresionPreFacturaFinalizadaCorrectamente(factura, this.numeroFactura);
				}
				catch (ImpresionFacturaException ex) {
					SMessageDialog.MensajeError(ex.mensaje);
					
					this.lblEstadoActual.Text = "En espera";
					this.fraImpresionDocumentos.Sensitive = true;
					this.fraConfiguracion.Sensitive = true;
					
					return;
				}
				catch {
					this.lblEstadoActual.Text = "En espera";
					this.fraImpresionDocumentos.Sensitive = true;
					this.fraConfiguracion.Sensitive = true;
					
					throw;
				}
				
				this.documentosPorImprimir.BorrarItems(iterSeleccionado);
			}
			
			// Escribo el nuevo estado actual y vuelvo a habilitar parte de la interfaz gráfica
			this.lblEstadoActual.Text = "En espera";
			
			this.fraImpresionDocumentos.Sensitive = true;
			this.fraConfiguracion.Sensitive = true;
		}
		
		protected void DocumentoSeleccionado(object sender, EventArgs e)
		{
			TreeIter iter;
			
			// FIXME: Este es un fix temporal para poder imprimir, pero
			// este addin ya no va más, asi que...
			PreFacturaDto[] preFacturas = this.documentosPorImprimir.ObtenerSeleccionadosGenericos();
			
			if (preFacturas.Length == 1) {
				this.btnImprimirSeleccionado.Sensitive = true;
				this.btnYaImpresa.Sensitive = true;
			}
			else {
				this.btnImprimirSeleccionado.Sensitive = false;
				this.btnYaImpresa.Sensitive = false;
			}
		}
		
		protected virtual void OnRefrescarClicked (object sender, System.EventArgs e)
		{
			// Quito todos los documentos de la lista
			this.documentosPorImprimir.Vaciar();
			
			// Creo el objeto ConsultaPaginable
			ConsultaPaginable consultaPaginable = new ConsultaPaginable {
				pagina = 0,
				tamanioPagina = TAMAÑO_PAGINA,
				criteriosOrdenacion = new List<CriterioOrdenacion> {
					new CriterioOrdenacion {
						campoOrdenacion = "Fecha",
						sentidoOrd = SentidoOrdenacion.Desc
					}
				}
			};
			
			// Pido al servidor todas las prefacturas que esta terminal puede imprimir
			PreFacturaDto[] resultado = null;
			try {
				resultado =
					((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
							.BuscarPreFacturaHQL("from PreFactura d left join fetch d.Cliente order by d.Fecha desc",
						                     null);
			}
			catch (ConexionException ex) {
				SMessageDialog
					.MensajeError("La terminal no figura como conectada en el " +
					              "servidor.", ex);
				return;
			}
			
			/* Si no se obtuvieron resultados, le aviso al usuario. */
			if (resultado.Length == 0) {
				SMessageDialog
					.MensajeAdvertencia("No hay PreFacturas enviadas a imprimir para esta terminal.");
				return;
			}
			
			this.documentosPorImprimir.AgregarElementosPrimeraPagina(resultado);
		}
		#endregion
		
		#region Configuración general
		protected virtual void OnImpresionAutomaticaToggled (object sender, System.EventArgs e)
		{
			if (this.cargandoDatosGenerales)
				return;
			
			ServicioFacturacionElement.Instancia.ImpresionAutomatica =
				this.chkImpresionAutomatica.Active;
		}
		#endregion
		
		#region Configuración de impresoras
		private void RenderPrinter(CellLayout layout,
		                           CellRenderer renderer,
		                           TreeModel model,
		                           TreeIter iter)
		{
			string impresora = (string) model.GetValue(iter, 0);
			(renderer as CellRendererText).Text = impresora;
		}
		
		private void CambiarActivacionTiposDocumento(bool nuevoValor)
		{
			this.chkFacturaA.Active = nuevoValor;
			this.chkFacturaB.Active = nuevoValor;
			this.chkPresupuesto.Active = nuevoValor;
		}
		
		protected virtual void OnImpresorasChanged (object sender, System.EventArgs e)
		{
			this.cargandoDatosImpresora = true;
			
			// Activo el botón para activar la impresión en la impresora seleccionada
			this.chkActivarImpresora.Sensitive = true;
			
			// Obtengo el objeto Gtk.Printer del modelo del ComboBox
			TreeIter iter;
			this.cmbImpresoras.GetActiveIter(out iter);
			
			string nombreImpresora = (string) this.cmbImpresoras.Model.GetValue(iter, 0);
			
			// Obtengo la sección de configuración de ServicioFacturacion
			ServicioFacturacionElement section = ServicioFacturacionElement.Instancia;
			
			// Si la impresora seleccionada está en la configuración, entonces está activa
			Impresora impresora = section.Impresoras[nombreImpresora];
			if (impresora == null) {
				this.chkActivarImpresora.Active = false;
				
				this.CambiarActivacionTiposDocumento(false);
			}
			else {
				this.impresoraSeleccionada = impresora;
				
				this.chkActivarImpresora.Active = true;
				
				this.chkFacturaA.Active =
					impresora.TiposDocumento[TipoDocumento.FacturaTipoA] != null;
				this.chkFacturaB.Active =
					impresora.TiposDocumento[TipoDocumento.FacturaTipoB] != null;
				this.chkPresupuesto.Active =
					impresora.TiposDocumento[TipoDocumento.Presupuesto] != null;
			}
			
			this.cargandoDatosImpresora = false;
		}
		
		protected virtual void OnBtnYaImpresaClicked (object sender, System.EventArgs e)
		{
			ResponseType respuesta =
				SMessageDialog.MensajePregunta("¿Esta seguro que la PreFactura seleccionada ya fue impresa?");
			
			if (respuesta != ResponseType.Yes)
				return;
			
			ControladorFacturacionPrx controladorFacturacion =
				((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"));
			
			// Obtengo la PreFacturaDto que se ha seleccionado para imprimir
			PreFacturaDto factura = (PreFacturaDto)
				this.documentosPorImprimir.ObtenerSeleccionados()[0];
			
			// Refresco la instancia de la factura
			factura = (PreFacturaDto) controladorFacturacion.Refrescar(factura);
			
			/* Primero obtengo el siguiente número de factura. */
			try {
				controladorFacturacion
						.PrepararPreFacturaParaImprimir(factura, out this.numeroFactura);
			}
			catch (ImpresionFacturaException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				
				this.lblEstadoActual.Text = "En espera";
				this.fraImpresionDocumentos.Sensitive = true;
				this.fraConfiguracion.Sensitive = true;
				
				return;
			}
			catch {
				this.lblEstadoActual.Text = "En espera";
				this.fraImpresionDocumentos.Sensitive = true;
				this.fraConfiguracion.Sensitive = true;
				
				throw;
			}
			
			this.RespuestaImpresion(true);
		}
		
		protected virtual void OnActivarImpresoraToggle (object sender, System.EventArgs e)
		{
			this.tbDocumentos.Sensitive = this.chkActivarImpresora.Active;
			
			/* Si el evento Toggle se disparó por estar cargándose los datos (por ejemplo
			 * desde el método OnImpresorasChanged), no sigo avanzando. */
			if (this.cargandoDatosImpresora)
				return;
			
			// Desactivo todos los tipos de documento. No importa si la impresora se activó
			// o desactivó
			this.CambiarActivacionTiposDocumento(false);
			
			// Ahora procedo a cambiar la configuración:
			
			// Obtengo el objeto Gtk.Printer del modelo del ComboBox
			TreeIter iter;
			this.cmbImpresoras.GetActiveIter(out iter);
			
			string nombreImpresora = (string) this.cmbImpresoras.Model.GetValue(iter, 0);
			
			/* Obtengo la sección de configuración y la impresora seleccionada del mismo
			 * (si es que está en la configuración) */
			ServicioFacturacionElement section = ServicioFacturacionElement.Instancia;
			
			Impresora impresora = section.Impresoras[nombreImpresora];
			
			/* Si la impresora está en la configuración y ha sido desactivada, la quito.
			 * Si no está y ha sido activada, la agrego. Luego guardo la configuración. */
			if (impresora != null && !this.chkActivarImpresora.Active) {
				section.Impresoras.Remove(impresora);
			}
			else if (impresora == null && this.chkActivarImpresora.Active) {
				this.impresoraSeleccionada = new Impresora(nombreImpresora);
				section.Impresoras.Add(this.impresoraSeleccionada);
			}
		}
		
		#region Tipos de documento
		protected virtual void OnFacturaAToggled (object sender, System.EventArgs e)
		{
			this.DocumentoModificado(TipoDocumento.FacturaTipoA, this.chkFacturaA);
		}

		protected virtual void OnFacturaBToggled (object sender, System.EventArgs e)
		{
			this.DocumentoModificado(TipoDocumento.FacturaTipoB, this.chkFacturaB);
		}

		protected virtual void OnPresupuestoToggled (object sender, System.EventArgs e)
		{
			this.DocumentoModificado(TipoDocumento.Presupuesto, this.chkPresupuesto);
		}
		
		private void DocumentoModificado (TipoDocumento tipoDocumento, CheckButton checkButton)
		{
			/* Si el evento Toggle se disparó por estar cargándose los datos (por ejemplo
			 * desde el método OnImpresorasChanged), no sigo avanzando. */
			if (this.cargandoDatosImpresora)
				return;
			
			Documento documento = this.impresoraSeleccionada.TiposDocumento[tipoDocumento];
			
			/* Si el documento está en la configuración de la impresora y ha sido desactivado,
			 * lo quito. Si no está y ha sido activado, lo agrego. */
			if (documento != null && !checkButton.Active)
				this.impresoraSeleccionada.TiposDocumento.Remove(documento);
			else if (documento == null && checkButton.Active)
				this.impresoraSeleccionada.TiposDocumento.Add(new Documento(tipoDocumento));
		}
		#endregion
		#endregion
	}
}
