// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Gtk;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using Stratos.QueryObjects;
using Stratos.Common;
using Stratos.Common.Excepciones;
using StratosClient.Core.Gui;
using StratosClient.Widgets.Level1;
using StratosClient.Widgets.Level2;

namespace StratosClient.Ventas.Facturacion.Avanzado
{
	public partial class Sincronizador : Gtk.Dialog
	{
		private const int TAMAÑO_PAGINA = 7;
		
		private TvFacturas facturas;
		private ComboBoxEnumeracion<FacturaTipoVenta> cmbFacturaTipo;
		
		private int codigoActual;
		private long numeroSiguienteActual;
		private FacturaTipoVenta tipoFacturaActual;
		
		public Sincronizador()
			: base("Sincronizador de números de factura", MainGui.MainWindow,
			       DialogFlags.Modal)
		{
			this.Build();
			
			#region Lista de facturas
			this.facturas = new TvFacturas();
			this.facturas.Selection.Mode = SelectionMode.Browse;
			this.facturas.tvcObservaciones.Visible = false;
			this.scrolledwindow1.Add(this.facturas);
			this.facturas.ShowAll();
			#endregion
			
			#region Combo Factura Tipo
			this.cmbFacturaTipo = new ComboBoxEnumeracion<FacturaTipoVenta>();
			this.cmbFacturaTipo.Renderizador = delegate (FacturaTipoVenta facturaTipo) {
				return StratosUtil.SplitCamelString(facturaTipo.ToString());
			};
			this.cmbFacturaTipo.Resetear();
			this.cmbFacturaTipo.ShowAll();
			this.hbTipoFactura.Add(this.cmbFacturaTipo);
			#endregion
			
			#region TreeView Facturas
			this.facturas.Selection.Changed += this.SeleccionCambiada;
			#endregion
			
			#region Eventos
			this.txtCodigo.Changed += this.CodigoCambiado;
			this.txtCodigo.Activated += this.OnConsultarClicked;
			this.cmbFacturaTipo.Changed += this.FacturaTipoCambiada;
			this.facturas.ModeloCambiado += this.ModeloCambiado;
			#endregion
		}
		
		#region Eventos
		protected virtual void CodigoCambiado (object sender, System.EventArgs e)
		{
			this.facturas.Vaciar();
		}
		
		protected virtual void FacturaTipoCambiada (object sender, System.EventArgs e)
		{
			this.facturas.Vaciar();
		}
		
		protected virtual void ModeloCambiado()
		{
			if (this.facturas.Model.IterNChildren() == 0) {
				this.lblSiguienteNumero.Markup = "<b>-</b>";
				this.btnIngresar.Sensitive = false;
				this.btnEliminar.Sensitive = false;
			}
			else {
				// Calculo el siguiente número de la factura y lo coloco en el label
				this.lblSiguienteNumero.Markup =
							"<b>" + this.CalcularSiguienteNumero().ToString() + "</b>";
				
				this.btnIngresar.Sensitive = true;
				
				/* Si el modelo del TreeView de facturas ha cambiado y tiene nuevos elementos,
				 * entonces deselecciono a todos ellos. */
				this.facturas.Selection.UnselectAll();
				
				// Seteo el código, número y tipo de factura actual
				this.codigoActual = Int32.Parse(this.txtCodigo.Text);
				this.numeroSiguienteActual = this.CalcularSiguienteNumero();
				this.tipoFacturaActual = this.cmbFacturaTipo.ObjetoActivo;
			}
		}
		
		protected virtual void SeleccionCambiada (object sender, System.EventArgs e)
		{
			// Obtengo el TreeIter del primer elemento y del elemento seleccionado
			TreeIter iterPrimero;
			TreeIter[] itersSeleccionados;
			
			this.facturas.Model.GetIterFirst(out iterPrimero);
			FacturaDto[] facturas = this.facturas.ObtenerSeleccionadosGenericos(out itersSeleccionados);
			
			if (facturas.Length != 1 || !itersSeleccionados[0].Equals(iterPrimero))
				this.btnEliminar.Sensitive = false;
			else if (facturas.Length == 1 && itersSeleccionados[0].Equals(iterPrimero)) {
				/* Si ambos iters son iguales entonces el usuario seleccionó el primero, y por
				 * lo tanto le permito eliminar esa factura. */
				this.btnEliminar.Sensitive = true;
			}
		}
		
		private FacturaDto[] RealizarConsulta()
		{
			return this.RealizarConsulta(null);
		}
		
		private FacturaDto[] RealizarConsulta(ConsultaPaginable consultaPaginableOpcional)
		{
			// Preparo la consulta al servidor
			FacturaDtoQuery consultaFacturas = new FacturaDtoQuery();
			consultaFacturas.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioAnd conjuncion = new CriterioAnd();
			conjuncion.criterios = new List<CriterioGenerico>();
			
			// Criterio de código
			CriterioSimple criterioCodigo = new CriterioSimple();
			criterioCodigo.operador = TipoOperador.Igual;
			criterioCodigo.propiedad = "Codigo";
			criterioCodigo.valor = this.txtCodigo.Text;
			
			conjuncion.criterios.Add(criterioCodigo);
			
			// Criterio de tipo de Factura
			CriterioSimple criterioTipoFactura = new CriterioSimple();
			criterioTipoFactura.operador = TipoOperador.Igual;
			criterioTipoFactura.propiedad = "Tipo";
			criterioTipoFactura.tipoValor = "FacturaTipoVenta";
			criterioTipoFactura.valor = this.cmbFacturaTipo.ObjetoActivo.ToString();
			
			conjuncion.criterios.Add(criterioTipoFactura);
			
			// Agrego el criterio AND a la consulta
			consultaFacturas.listaCriteriosComplejos.Add(conjuncion);
			
			// Creo la Consulta Paginable
			ConsultaPaginable consultaPaginable = new ConsultaPaginable();
			consultaPaginable.pagina =
				consultaPaginableOpcional == null ? 0 : consultaPaginableOpcional.pagina;
			consultaPaginable.tamanioPagina =
				consultaPaginableOpcional == null ? TAMAÑO_PAGINA : consultaPaginableOpcional.tamanioPagina;
			consultaPaginable.criteriosOrdenacion = new List<CriterioOrdenacion> {
				new CriterioOrdenacion {
					campoOrdenacion = "Fecha",
					sentidoOrd = SentidoOrdenacion.Desc
				},
				new CriterioOrdenacion {
					campoOrdenacion = "Codigo",
					sentidoOrd = SentidoOrdenacion.Desc
				},
				new CriterioOrdenacion {
					campoOrdenacion = "Numero",
					sentidoOrd = SentidoOrdenacion.Desc
				}
			};
			
			// Realizo la consulta
			Ice.Object[] resultadoQueryObjects =
				((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
					.BuscarQueryObjects(consultaFacturas, consultaPaginable);
			
			FacturaDto[] facturasRecibidas =
				resultadoQueryObjects.Cast<FacturaDto>().ToArray<FacturaDto>();
			
			return facturasRecibidas;
		}
		
		protected virtual void OnConsultarClicked (object sender, System.EventArgs e)
		{
			// Valido los datos
			if (!this.txtCodigo.TextoValido) {
				SMessageDialog.MensajeError(this.txtCodigo.MensajeError, this);
				return;
			}
			
			FacturaDto[] facturasRecibidas = this.RealizarConsulta();
			
			// Agrego el resultado al TreeView
			this.facturas.AgregarElementosPrimeraPagina(facturasRecibidas);
		}
		
		protected virtual void OnIngresarVaciaClicked (object sender, System.EventArgs e)
		{
			// Pido el siguiente número para sincronizar. Por defecto es el siguiente,
			// pero el usuario puede cambiarlo.
			DialogoEntrada dialogEntrada =
				new DialogoEntrada(this, this.numeroSiguienteActual.ToString());
			
			ResponseType respuesta = (ResponseType) dialogEntrada.Run();
			
			if (respuesta != ResponseType.Ok) {
				dialogEntrada.Destroy();
				return;
			}
			
			// Tomo el valor ingresado y luego destruyo el diálogo.
			string valorIngresado = dialogEntrada.Valor;
			long numeroSiguienteIngresado = 0L;
			
			dialogEntrada.Destroy();
			
			// Verifico que el valor ingresado sea numérico.
			if (StratosUtil.IsLong(valorIngresado))
				numeroSiguienteIngresado = long.Parse(valorIngresado);
			else {
				SMessageDialog.MensajeError("El valor ingresado no es numérico");
				return;
			}
			
			// Ingreso en el servidor una factura inválida con el número correspondiente
			FacturaDto nuevaFacturaInvalida = null;
			try {
				nuevaFacturaInvalida = 
					((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
						.IngresarFacturaDeSincronizacion(this.tipoFacturaActual, this.codigoActual,
									                     numeroSiguienteIngresado);
			}
			catch (ParametrosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			
			// Agrego la nueva factura inválida a la lista
			this.facturas.InsertarElemento(nuevaFacturaInvalida, 0);
		}
		
		protected virtual void OnEliminarClicked (object sender, System.EventArgs e)
		{
			// Primero le preguntamos al usuario si está seguro de realizar la operación
			ResponseType respuesta =
							SMessageDialog.MensajePregunta(this,
						                   "Esta operación eliminará la Factura de la base " +
						                   "de datos. ¿Está seguro que quiere continuar?");
			
			if (respuesta != ResponseType.Yes)
				return;
			
			// Si se respondió afirmativamente, continuamos...
			
			TreeIter[] iters;
			TreeIter iter;
			
			FacturaDto facturaSeleccionada = (FacturaDto) this.facturas.ObtenerSeleccionados(out iters)[0];
			
			iter = iters[0];
			
			// Realizo la operación de eliminación.
			try {
				((ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion"))
					.EliminarFactura(facturaSeleccionada);
			}
			catch (FacturaImposibleDeEliminarException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
				return;
			}
			
			// Quito el item eliminado de la lista y recalculo el siguiente número
			this.facturas.BorrarItems(iter);
		}
		#endregion
		
		#region Métodos
		private long CalcularSiguienteNumero()
		{
			TreeIter iterPrimero;
			this.facturas.Model.GetIterFirst(out iterPrimero);
			
			FacturaDto primerFactura =
				(FacturaDto) this.facturas.Model.GetValue(iterPrimero, 0);
			
			if (primerFactura == null)
				return -1L;
			
			return (primerFactura.numero + 1);
		}

		#endregion
	}
}
