// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Configuration;
using System.Reflection;

using Stratos.ModeloDto;

namespace StratosClient.Ventas.Facturacion.ImpresionFacturas.Configuracion
{
	/// <summary>
	/// Representa un elemento Documento en el archivo de configuración. Se utiliza
	/// para indicar que una determinada impresora soporta la impresión de ese
	/// documento, por ejemplo una Factura A o un Presupuesto.
	/// </summary>
	public class Documento : ConfigurationElement
	{
		public Documento()
		{
		}
		
		public Documento(TipoDocumento tipoDocumento)
		{
			this.Tipo = tipoDocumento;
		}
		
		[ConfigurationProperty("tipo",
		                       IsRequired = true,
		                       IsKey = true,
		                       DefaultValue = TipoDocumento.SinTipo)]
		public TipoDocumento Tipo {
			get { return (TipoDocumento)this["tipo"]; }
			set { this["tipo"] = value; }
		}
		
		[ConfigurationProperty("numeroCopias",
		                       IsRequired = false,
		                       IsKey = false,
		                       DefaultValue = 1)]
		public int NumeroCopias {
			get { return (int)this["numeroCopias"]; }
			set { this["numeroCopias"] = value; }
		}
		
		[ConfigurationProperty("topOffset",
		                       IsRequired = false,
		                       IsKey = false,
		                       DefaultValue = 0.0)]
		public double TopOffset {
			get { return (double)this["topOffset"]; }
			set { this["topOffset"] = value; }
		}
		
		[ConfigurationProperty("leftOffset",
		                       IsRequired = false,
		                       IsKey = false,
		                       DefaultValue = 0.0)]
		public double LeftOffset {
			get { return (double)this["leftOffset"]; }
			set { this["leftOffset"] = value; }
		}
	}
}
