// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Configuration;

using Stratos.ModeloDto;

namespace StratosClient.Ventas.Facturacion.ImpresionFacturas.Configuracion
{
	/// <summary>
	/// Representa un elemento Impresora indicando las capacidades de impresión de la misma.
	/// </summary>
	public class Impresora : ConfigurationElement
	{
		public Impresora()
		{
		}
		
		public Impresora(string nombre)
		{
			this.Nombre = nombre;
		}
		
		/// <value>
		/// Nombre de la impresora
		/// </value>
		[ConfigurationProperty("nombre", IsRequired = true, IsKey = true)]
		public string Nombre {
			get { return (string)base["nombre"]; }
			set { base["nombre"] = value; }
		}
		
		/// <value>
		/// Colección de los tipos de documentos que la impresora soporta para imprimir. 
		/// </value>
		[ConfigurationProperty("tiposDocumento", IsDefaultCollection = true, IsRequired = true)]
		public DocumentosCollection TiposDocumento {
			get { return (DocumentosCollection)base["tiposDocumento"]; }
		}
		
		[ConfigurationProperty("impresionFacturas",
		                       IsRequired = false)]
		public ImpresionFacturas ImpresionFacturas {
			get { return (ImpresionFacturas) base["impresionFacturas"]; }
			set { base["impresionFacturas"] = value; }
		}
		
		/// <summary>
		/// Dado un objeto TipoDocumento, devuelve el objeto Documento
		/// asociado.
		/// </summary>
		/// <param name="tipoDocumento">
		/// A <see cref="TipoDocumento"/>
		/// </param>
		/// <returns>
		/// A <see cref="Documento"/>
		/// </returns>
		public Documento BuscarDocumento(TipoDocumento tipoDocumento)
		{
			Documento documento =
				(from doc in this.TiposDocumento.Cast<Documento>()
				 where doc.Tipo == tipoDocumento
				 select doc).First();
			
			return documento;
		}
	}
}
