// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Drawing.Printing;

using Gtk;

using StratosClient.Ventas.Facturacion.ImpresionFacturas.Configuracion;
using Stratos.ModeloDto;

namespace StratosClient.Ventas.Facturacion.ImpresionFacturas
{
	public partial class ConfiguracionImpresion : Gtk.Bin
	{
		#region Campos
		private bool cargandoDatosImpresora = false;
		private Impresora impresoraSeleccionada;
		#endregion
		
		#region Constructores
		public ConfiguracionImpresion ()
		{
			this.Build ();
			
			#region Setup solapa "configuración impresoras"
			// ComboBox de impresoras disponibles en el sistema
			ListStore modeloImpresoras = new ListStore(typeof(string));
			this.cmbImpresoras.Model = modeloImpresoras;
			
			// Cargo el modelo con las impresoras disponibles
			this.CargarImpresoras();
			
			// Creo el CellRenderer para el ComboBox
			CellRendererText renderer = new CellRendererText();
			this.cmbImpresoras.PackStart(renderer, true);
			
			this.cmbImpresoras.SetCellDataFunc(renderer, this.RenderPrinter);
			#endregion
		}
		#endregion
		
		#region Eventos
		#region Tipos de documentos para imprimir
		protected virtual void OnFacturaAToggled (object sender, System.EventArgs e)
		{
			this.DocumentoModificado(TipoDocumento.FacturaTipoA, this.chkFacturaA);
		}

		protected virtual void OnFacturaBToggled (object sender, System.EventArgs e)
		{
			this.DocumentoModificado(TipoDocumento.FacturaTipoB, this.chkFacturaB);
		}
		
		private void DocumentoModificado (TipoDocumento tipoDocumento, CheckButton checkButton)
		{
			/* Si el evento Toggle se disparó por estar cargándose los datos (por ejemplo
			 * desde el método OnImpresorasChanged), no sigo avanzando. */
			if (this.cargandoDatosImpresora)
				return;
			
			Documento documento = this.impresoraSeleccionada.TiposDocumento[tipoDocumento];
			
			/* Si el documento está en la configuración de la impresora y ha sido desactivado,
			 * lo quito. Si no está y ha sido activado, lo agrego. */
			if (documento != null && !checkButton.Active)
				this.impresoraSeleccionada.TiposDocumento.Remove(documento);
			else if (documento == null && checkButton.Active)
				this.impresoraSeleccionada.TiposDocumento.Add(new Documento(tipoDocumento));
		}
		#endregion
		
		protected virtual void OnActivarImpresoraToggle (object sender, System.EventArgs e)
		{
			this.tbDocumentos.Sensitive = this.chkActivarImpresora.Active;
			
			/* Si el evento Toggle se disparó por estar cargándose los datos (por ejemplo
			 * desde el método OnImpresorasChanged), no sigo avanzando. */
			if (this.cargandoDatosImpresora)
				return;
			
			// Desactivo todos los tipos de documento. No importa si la impresora se activó
			// o desactivó
			this.CambiarActivacionTiposDocumento(false);
			
			// Ahora procedo a cambiar la configuración:
			
			// Obtengo el objeto Gtk.Printer del modelo del ComboBox
			TreeIter iter;
			this.cmbImpresoras.GetActiveIter(out iter);
			
			string nombreImpresora = (string) this.cmbImpresoras.Model.GetValue(iter, 0);
			
			/* Obtengo la sección de configuración y la impresora seleccionada del mismo
			 * (si es que está en la configuración) */
			ImpresionFacturasElement section = ImpresionFacturasElement.Instancia;
			
			Impresora impresora = section.Impresoras[nombreImpresora];
			
			/* Si la impresora está en la configuración y ha sido desactivada, la quito.
			 * Si no está y ha sido activada, la agrego. Luego guardo la configuración. */
			if (impresora != null && !this.chkActivarImpresora.Active) {
				section.Impresoras.Remove(impresora);
			}
			else if (impresora == null && this.chkActivarImpresora.Active) {
				this.impresoraSeleccionada = new Impresora(nombreImpresora);
				section.Impresoras.Add(this.impresoraSeleccionada);
			}
		}
		
		protected virtual void OnImpresorasChanged (object sender, System.EventArgs e)
		{
			// Obtengo el objeto Gtk.Printer del modelo del ComboBox
			TreeIter iter;
			if (!this.cmbImpresoras.GetActiveIter(out iter))
				return;
			
			this.cargandoDatosImpresora = true;
			
			// Activo el botón para activar la impresión en la impresora seleccionada
			this.chkActivarImpresora.Sensitive = true;
			
			string nombreImpresora = (string) this.cmbImpresoras.Model.GetValue(iter, 0);
			
			// Obtengo la sección de configuración de ServicioFacturacion
			ImpresionFacturasElement section = ImpresionFacturasElement.Instancia;
			
			// Si la impresora seleccionada está en la configuración, entonces está activa
			Impresora impresora = section.Impresoras[nombreImpresora];
			if (impresora == null) {
				this.chkActivarImpresora.Active = false;
				
				this.CambiarActivacionTiposDocumento(false);
			}
			else {
				this.impresoraSeleccionada = impresora;
				
				this.chkActivarImpresora.Active = true;
				
				this.chkFacturaA.Active =
					impresora.TiposDocumento[TipoDocumento.FacturaTipoA] != null;
				this.chkFacturaB.Active =
					impresora.TiposDocumento[TipoDocumento.FacturaTipoB] != null;
			}
			
			this.cargandoDatosImpresora = false;
		}
		
		protected virtual void OnBtnRecargarClicked (object sender, System.EventArgs e)
		{
			this.CargarImpresoras();
		}
		#endregion
		
		#region Métodos
		private void CargarImpresoras()
		{
			// Obtengo el modelo del combo box de impresoras
			ListStore modelo = this.cmbImpresoras.Model as ListStore;
			
			if (modelo == null)
				return;
			
			// Quito todas y agrego las impresoras actuales del sistema
			modelo.Clear();
			
			foreach (string impresora in PrinterSettings.InstalledPrinters)
				modelo.AppendValues(impresora);
			
			// Deshabilito la activación ya que en este punto no habrá impresoras
			// seleccionadas
			this.chkActivarImpresora.Sensitive = false;
			this.chkActivarImpresora.Active = false;
		}
		
		private void CambiarActivacionTiposDocumento(bool nuevoValor)
		{
			this.chkFacturaA.Active = nuevoValor;
			this.chkFacturaB.Active = nuevoValor;
		}
		#endregion
		
		#region Métodos de renderizado
		private void RenderPrinter(CellLayout layout,
		                           CellRenderer renderer,
		                           TreeModel model,
		                           TreeIter iter)
		{
			string impresora = (string) model.GetValue(iter, 0);
			(renderer as CellRendererText).Text = impresora;
		}
		#endregion
	}
}
