// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using Gtk;

using Stratos.ModeloDto;

namespace StratosClient.Ventas.Facturacion.ImpresionFacturas
{
	/// <summary>
	/// Acciones de impresión para una factura en el listado de Facturas / PreFacturas
	/// </summary>
	public class AccionadorListadoPreFactura : AccionadorPreFactura, IAccionadorSobreElemento
	{
		#region Campos
		private PreFacturaDto[] preFacturas;
		private TreeIter[] treeIters;
		private TreeModel modelo;
		
		private Gtk.Action imprimirAction;
		private Gtk.Action yaImpresaAction;
		#endregion
		
		#region Constructores
		public AccionadorListadoPreFactura()
		{
			this.imprimirAction =
				new Gtk.Action("imprimir",
				               "Imprimir",
				               "Imprime una PreFactura",
				               "gtk-print");
			
			this.imprimirAction.Activated += this.OnImprimirActivated;
			
			this.yaImpresaAction =
				new Gtk.Action("yaimpresa",
				               "Ya impresa",
				               "Marca una PreFactura como ya impresa, y sin enviarla a imprimir crea la Factura correspondiente",
				               "gtk-floppy");
			
			this.yaImpresaAction.Activated += this.OnYaImpresaActivated;
		}
		#endregion
		
		#region Propiedades
		protected override FacturaGenericaDto FacturaSeleccionada {
			get { return this.preFacturas[0]; }
		}
		#endregion
		
		#region IAccionadorSobreElemento implementation
		public Gtk.Action[] Acciones {
			get { return new [] { this.imprimirAction, this.yaImpresaAction }; }
		}
		
		public void ActualizarAcciones (TreeIter[] iters, DocumentoGenericoDto[] documentos, TreeModel modelo)
		{
			this.treeIters = iters;
			this.preFacturas = documentos.Cast<PreFacturaDto>().ToArray();
			this.modelo = modelo;
			
			this.imprimirAction.Sensitive = this.preFacturas.Length == 1;
			this.yaImpresaAction.Sensitive = this.preFacturas.Length == 1;
		}
		#endregion
		
		protected override PreFacturaDto ObtenerPreFactura()
		{
			PreFacturaDto factura = base.ObtenerPreFactura();
			
			this.modelo.SetValue(this.treeIters[0], 0, factura);
			this.modelo.EmitRowChanged(this.modelo.GetPath(this.treeIters[0]), this.treeIters[0]);
			
			return factura;
		}
	}
}
