// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using StratosClient.Core.Gui;
using StratosClient.Ventas.Facturacion;
using Stratos.ModeloDto;
using StratosClient.Widgets.Level3;

namespace StratosClient.Ventas.Facturacion.ImpresionFacturas
{
	/// <summary>
	/// Acciones para una factura cuando está cargada en el visualizador.
	/// </summary>
	public class AccionadorVisualizadorPreFactura : AccionadorPreFactura, IAccionadorSobreFactura
	{
		#region Campos
		private FacturaGenericaDto factura;
		
		private Gtk.Action imprimirAction;
		
		private IVisualizadorFactura visualizadorFactura;
		#endregion
		
		#region Constructores
		public AccionadorVisualizadorPreFactura()
		{
			this.imprimirAction =
				new Gtk.Action("imprimir",
				               "Imprimir",
				               "Imprime una PreFactura",
				               "gtk-print");
			
			this.imprimirAction.Activated += this.OnImprimirActivated;
		}
		#endregion
		
		#region AccionadorPreFactura implementation
		protected override FacturaGenericaDto FacturaSeleccionada {
			get { return this.factura; }
		}
		
		protected override FacturaDto PreFacturaImpresa (long numeroFactura)
		{
			// Antes de llamar a PreFacturaImpresa, registro el estado anterior
			// del Navegador de elementos.
			IVisualizadorNavegable visualizadorNavegable = null;
			bool estadoNavegadorElementos = false;
			
			// FIXME: Acá no tendría que haber noción de activar el navegador de elementos indicando
			// que el siguiente es el actual :S
			if (this.visualizadorFactura is IVisualizadorNavegable) {
				visualizadorNavegable =
					(this.visualizadorFactura as IVisualizadorNavegable);
				
				estadoNavegadorElementos = visualizadorNavegable.NavegadorElementos.Sensitive;
			}
			
			// Llamo al método PreFacturaImpresa de la clase base
			FacturaDto facturaImpresa =
				base.PreFacturaImpresa (numeroFactura);
			
			// FIXME: Este accionador de impresión dentro del visualizador no debería
			// tener que encargarse de cargar la factura. Se podría disparar un evento
			// global.
			this.visualizadorFactura.CargarFactura(facturaImpresa);
			
			if (visualizadorNavegable != null) {
				// Restauro el Navegador de Elementos a su estado de activación anterior,
				// ya que cargar la factura cambia el estado del mismo.
				visualizadorNavegable.NavegadorElementos.Sensitive = estadoNavegadorElementos;
				
				// Si el navegador de elementos estaba activado, entonces le indico
				// que el siguiente elemento es el actual;
				visualizadorNavegable.NavegadorElementos.SiguienteEsActual = estadoNavegadorElementos;
			}
			
			return facturaImpresa;
		}
		#endregion
		
		#region IAccionadorSobreElemento implementation
		public Gtk.Action[] Acciones {
			get { return new [] { this.imprimirAction }; }
		}
		
		public FacturaGenericaDto Factura {
			set { this.factura = value; }
		}
		
		public IVisualizadorFactura VisualizadorFactura {
			set {
				this.visualizadorFactura = value;
				
				/* Me suscribo al evento de estado cambiado del visualizador de factura
				 * La impresión es posible solo si el documento cargado es una PreFactura y el
				 * estado es Cargado o Guardado (hay que guardar las modificaciones hechas antes
				 * de intentar imprimir). */
				this.visualizadorFactura.AdministradorEstado.EstadoCambiado += delegate(EstadoObjeto obj) {
					this.imprimirAction.Sensitive =
						this.factura != null && this.factura is PreFacturaDto &&
							(obj == EstadoObjeto.Cargado || obj == EstadoObjeto.Guardado);
				};
			}
		}
		
		public int Orden {
			get { return 0; }
		}
		#endregion
	}
}
