// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;
using StratosClient.Core.Gui;
using StratosClient.Core.Gui.ImpresionBorrador;
using StratosClient.Common;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Acción que permite imprimir una factura en modo borrador.
	/// </summary>
	public class AccionImprimirBorrador : IAccionadorSobreFactura
	{
		#region Campos
		private readonly string DIRECTORIO_ADDIN;
		private readonly string ARCHIVO_TEMPLATE;
		
		private FacturaGenericaDto factura;
		
		private Gtk.Action imprimirBorradorAction;
		
		private IVisualizadorFactura visualizadorFactura;
		#endregion
		
		#region Constructores
		public AccionImprimirBorrador()
		{
			DIRECTORIO_ADDIN =
				"Addins" + System.IO.Path.DirectorySeparatorChar +
				"Ventas.Facturacion" + System.IO.Path.DirectorySeparatorChar;
			
			ARCHIVO_TEMPLATE =
				DIRECTORIO_ADDIN + "templateFactura.vm";
			
			// ImprimirBorradorAction
			this.imprimirBorradorAction =
				new Gtk.Action("imprimirborrador",
				               "Imprimir (borrador)",
				               "Imprime una factura como borrador",
				               "gtk-print");
			
			this.imprimirBorradorAction.Activated += this.OnImprimirBorradorActivated;
		}
		#endregion
		
		#region IAccionadorSobreElemento implementation
		public Gtk.Action[] Acciones {
			get { return new [] { this.imprimirBorradorAction }; }
		}
		
		public FacturaGenericaDto Factura {
			set {
				this.factura = value;
			}
		}
		
		public IVisualizadorFactura VisualizadorFactura {
			set {
				this.visualizadorFactura = value;
			}
		}
		
		public int Orden {
			get { return 100; }
		}
		#endregion
		
		protected void OnImprimirBorradorActivated(object o, System.EventArgs args)
		{
			// Solo permito la impresión en modo borrador si el documento no está
			// ni modificado ni es nuevo.
			if (this.visualizadorFactura.AdministradorEstado.EstaModificado ||
			    	this.visualizadorFactura.AdministradorEstado.EsNuevo) {
				SMessageDialog.MensajeAdvertencia("Guarde la factura antes de imprimir en modo borrador");
				return;
			}
			
			// Ejecuto la impresión
			this.imprimirBorradorAction.Sensitive = false;
			
			ImpresionDocumento impresionBorrador =
				new ImpresionDocumento(ARCHIVO_TEMPLATE,
					                  DIRECTORIO_ADDIN,
				                      "factura",
				                      new FacturaWrapper(this.factura));
			
			impresionBorrador.ImpresionEjecutada += delegate {
				this.imprimirBorradorAction.Sensitive = true;
			};
			
			impresionBorrador.Imprimir();
		}
	}
}
