// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using StratosClient.Core.Gui;
using StratosClient.Ventas;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Widgets.Level3;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Se encarga de cargar objetos de tipo NotaCredito y PreNotaCredito
	/// en el visualizador correspondiente.
	/// </summary>
	public class CargadorNotasCredito : CargadorElemento
	{
		#region ICargador implementation
		public override ControladorBasePrx ControladorRemoto {
			get { return (ControladorBasePrx) MainIce.GetControlador("ControladorFacturacion"); }
		}
		
		public override bool PermitirCarga(DocumentoGenericoDto documento)
		{
			return true;
		}
		
		public override IVisualizadorNavegable VisualizadorNavegable {
			get { return SolapaNotaCredito.InstanciaActual; }
		}
		
		public override void Cargar(DocumentoGenericoDto documento)
		{
			PreNotaCreditoDto preNotaCredito = documento as PreNotaCreditoDto;
			NotaCreditoDto notaCredito = documento as NotaCreditoDto;
			
			// Si el documento por cargar es una PreFacturaDto...
			if (preNotaCredito != null)
				SolapaNotaCredito.InstanciaActual.CargarFactura(preNotaCredito);
			
			// Si el documento por cargar es una FacturaDto
			else if (notaCredito != null)
				SolapaNotaCredito.InstanciaActual.CargarFactura(notaCredito);
		}
		#endregion
	}
}
