// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Core.Gui;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Verifica al salir del cliente las tareas pendientes que puede haber
	/// para los visualizadores de facturas y notas de crédito.
	/// </summary>
	public class VerificadorTareasPendientes : IVerificador
	{
		#region Campos
		private List<string> tareasPendientes;
		#endregion
		
		#region Constructores
		public VerificadorTareasPendientes()
		{
			this.tareasPendientes = new List<string>();
		}
		#endregion
		
		#region IVerificacion implementation
		public bool HayTareaPendiente {
			get {
				this.tareasPendientes.Clear();
				
				if (SolapaFacturacion.InstanciaActual.AdministradorEstado.EstaModificado)
					this.tareasPendientes.Add("Una pre factura con modificaciones no fue guardada aún.");
				
				if (SolapaNotaCredito.InstanciaActual.AdministradorEstado.EstaModificado)
					this.tareasPendientes.Add("Una pre nota de crédito con modificaciones no fue guardada aún.");
				
				return this.tareasPendientes.Count > 0;
			}
		}
		
		public string Seccion {
			get { return "Facturación"; }
		}
		
		public string[] Mensajes {
			get { return this.tareasPendientes.ToArray(); }
		}
		#endregion
	}
}
