// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Ventas;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Core.Gui;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Se encarga de realizar la consulta de Facturas en el listado de documentos.
	/// </summary>
	public class ConsultaFacturas : IConsulta
	{
		public Type TipoDevuelto {
			get { return typeof(FacturaDto); }
		}
		
		public DocumentoGenericoDto[] RealizarConsulta(Dictionary<string,object> criteriosBusqueda,
		                                               ConsultaPaginable consultaPaginable)
		{
			// Obtengo el controlador para realizar la consulta
			ControladorFacturacionPrx controladorFacturacion =
				(ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion");
			
			// Primero genero los criterios HQL generales (razon social, fechas, etc) que se
			// encuentran en la interfaz de StratosClient.Ventas.
			// Para este caso especifico cuál es la propiedad que tiene la razón social. En el
			// caso de las facturas, es la propiedad con el valor fijo: ClienteRazonSocial.
			List<string> criterios =
				VentasUtils.GenerarCriteriosHQL(criteriosBusqueda,
				                                "ClienteRazonSocial");
			
			// Luego genero los criterios HQL específicos de Factura (código y número)
			if (criteriosBusqueda.ContainsKey("CodigoFactura"))
				criterios.Add("d.Codigo = " + criteriosBusqueda["CodigoFactura"]);
			
			if (criteriosBusqueda.ContainsKey("NumeroFactura"))
				criterios.Add("d.Numero = " + criteriosBusqueda["NumeroFactura"]);
			
			if (criteriosBusqueda.ContainsKey("SoloNotaCredito") &&
			    	((bool)criteriosBusqueda["SoloNotaCredito"]))
				criterios.Add("d.class = NotaCredito");
			
			criterios.Add("d.Valida = cast(1,boolean)");
			
			/* Realizo la consulta y retorno los resultados. Además de generar la cláusula where,
			 * ordeno los resultados por fecha, código y número de factura. */
			return controladorFacturacion
				.BuscarFacturaHQL(@"from Factura d
										left join fetch d.Cliente " +
				                  StratosClientUtils.GenerarClausulaWhere(criterios) +
				                  " order by d.Fecha desc, d.Codigo desc, d.Numero desc",
				                  consultaPaginable);
		}
	}
}
