// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Ventas;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Core.Gui;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Se encarga de realizar la consulta de PreFacturas en el listado de documentos.
	/// </summary>
	public class ConsultaPreFacturas : IConsulta
	{
		public Type TipoDevuelto {
			get { return typeof(PreFacturaDto); }
		}
		
		public DocumentoGenericoDto[] RealizarConsulta(Dictionary<string,object> criteriosBusqueda,
		                                               ConsultaPaginable consultaPaginable)
		{
			// Obtengo el controlador y realizo la consulta
			ControladorFacturacionPrx controladorFacturacion =
				(ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion");
			
			// Genero los criterios generales
			List<string> criterios = VentasUtils.GenerarCriteriosHQL(criteriosBusqueda);
			
			return controladorFacturacion
				.BuscarPreFacturaHQL(@"from PreFactura d
											left join fetch d.Cliente " +
				                     StratosClientUtils.GenerarClausulaWhere(criterios),
				                     consultaPaginable);
		}
	}
}
