// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

namespace StratosClient.Ventas.Facturacion
{
	public partial class CriteriosFactura : Gtk.Bin, ICriteriosPersonalizados
	{
		public CriteriosFactura()
		{
			this.Build();
			
			this.chkCodigoFactura.Toggled += delegate {
				this.txtCodigoFactura.Sensitive = this.chkCodigoFactura.Active;
			};
			this.chkCodigoFactura.Active = false;
			
			this.chkNumeroFactura.Toggled += delegate {
				this.txtNumeroFactura.Sensitive = this.chkNumeroFactura.Active;
			};
			this.chkNumeroFactura.Active = false;
			
			this.chkSoloNotaCredito.Active = false;
		}
		
		/// <summary>
		/// Cuando el usuario precionó el botón Buscar, StratosClient.Ventas valida los
		/// criterios personalizados de factura ejecutando este método.
		/// </summary>
		/// <param name="mensajeError">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		public bool DatosValidos(out string mensajeError)
		{
			// Valido el código de la factura
			if (!this.txtCodigoFactura.TextoValido) {
				mensajeError = this.txtCodigoFactura.MensajeError;
				return false;
			}
			
			// Valido el número de la factura
			if (!this.txtNumeroFactura.TextoValido) {
				mensajeError = this.txtNumeroFactura.MensajeError;
				return false;
			}
			
			mensajeError = String.Empty;
			return true;
		}
		
		/// <summary>
		/// Se encarga de armar un diccionario con los valores de los criterios de búsqueda.
		/// Por ejemplo, en la clave "CodigoFactura" coloca el valor que ha establecido el usuario
		/// para buscar por código. De esta manera luego se puede armar la consulta desde este
		/// diccionario, sin pensar en las validaciones de los mismos (si el valor está en
		/// el diccionario, es porque hay que buscar por ese criterio).
		/// </summary>
		public Dictionary<string,object> CriteriosPersonalizados {
			get {
				Dictionary<string,object> criterios = new Dictionary<string, object>();
				
				// Agrego el código y número únicamente si el checkbox correspondiente está
				// activo y si el texto no es vacío
				
				if (this.chkCodigoFactura.Active && this.txtCodigoFactura.Text != String.Empty)
					criterios.Add("CodigoFactura", this.txtCodigoFactura.Text);
				
				if (this.chkNumeroFactura.Active && this.txtNumeroFactura.Text != String.Empty)
					criterios.Add("NumeroFactura", this.txtNumeroFactura.Text);
				
				criterios.Add("SoloNotaCredito", this.chkSoloNotaCredito.Active);
				
				return criterios;
			}
		}
	}
}
