// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace StratosClient.Ventas.Facturacion
{
	/// <summary>
	/// Métodos de utilidad de facturación.
	/// </summary>
	public static class FacturacionUtil
	{
		/// <summary>
		/// Crea una PreNotaCredito a partir de una Factura existente que estará
		/// asociada a la nota de crédito retornada. Permite pasar una PreNotaCredito
		/// existente para modificarla (de esta forma se copia el id).
		/// </summary>
		/// <param name="notaCredito">
		/// A <see cref="PreNotaCreditoDto"/>
		/// </param>
		/// <param name="facturaAsociada">
		/// A <see cref="FacturaDto"/>
		/// </param>
		/// <returns>
		/// A <see cref="PreNotaCreditoDto"/>
		/// </returns>
		public static PreNotaCreditoDto PreNotaCreditoDesdeFactura(PreNotaCreditoDto notaCredito, FacturaDto facturaAsociada)
		{
			// Creo una nueva PreNotaCreditoDto si la pasada es nula
			if (notaCredito == null)
				notaCredito = new PreNotaCreditoDto();
			
			notaCredito.facturaAsociada = facturaAsociada;
			
			notaCredito.cliente = facturaAsociada.cliente;
			
			/* Agrego los items de la factura asociada a la nota de crédito. De esta manera
			 * facilito la creación al usuario. Creo nuevos items, ya que si copio directamente
			 * los items de la factura asociada quedarían con el mismo id y puede haber problemas
			 * al persistir, ya que si bien son los mismos items, a nivel de base de datos deben
			 * ser distintos. */
			notaCredito.items = new List<ItemGenericoDto>();
			ItemProductoDto itemNuevo;
			
			foreach (ItemProductoDto item in facturaAsociada.items) {
				itemNuevo = new ItemProductoDto();
				itemNuevo.precioUnitario = item.precioUnitario;
				itemNuevo.descripcion = item.descripcion;
				itemNuevo.cantidad = item.cantidad;
				itemNuevo.importe = item.importe;
				itemNuevo.orden = item.orden;
				itemNuevo.porcentajeBonificacion = item.porcentajeBonificacion;
				
				itemNuevo.productoId = item.productoId;
				itemNuevo.productoCodigo = item.productoCodigo;
				itemNuevo.productoDescripcion = item.productoDescripcion;
				itemNuevo.productoAbstracto = item.productoAbstracto;
				itemNuevo.productoImporteSinIva = item.productoImporteSinIva;
				itemNuevo.productoImporteVenta = item.productoImporteVenta;
				itemNuevo.productoUnidad = item.productoUnidad;
				
				notaCredito.items.Add(itemNuevo);
			}
			
			notaCredito.condVenta = new CondicionVentaDto(CondicionPago.Contado, 0);
			notaCredito.fechaTicks = DateTime.Today.Date.Ticks;
			notaCredito.tipo = facturaAsociada.tipo;
			notaCredito.observaciones = String.Empty;
			notaCredito.remitos = String.Empty;
			
			return notaCredito;
		}
	}
}
