// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.
﻿
using System;
using System.Linq;
using System.Collections.Generic;

using Ice;
using Gtk;
using Mono.Addins;
using log4net;

using StratosClient.Common;
using StratosServer.Controladores;
using Stratos.QueryObjects;
using Stratos.ModeloDto;
using StratosClient.Core.Gui;
using StratosClient.Core.Gui.ImpresionBorrador;
using Stratos.Common;
using Stratos.Common.Excepciones;
using StratosClient.Widgets.Level2;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level1;

namespace StratosClient.Ventas.Facturacion
{
	public partial class SolapaFacturacion : Gtk.Bin, IVisualizadorNavegable, IVisualizadorFactura
	{
		private static readonly ILog logger =
			LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod()
			                     .DeclaringType);
		
		private static SolapaFacturacion instanciaActual;
		
		private ItemsInteractivo itemsProducto;
		
		/// <summary>
		/// El valor de esta referencia es el mismo que this.itemsProducto.TreeView, pero
		/// está casteado para evitar tantas conversiones de tipo.
		/// </summary>
		private ListaItemsVenta tvItemsProducto;
		
		protected ComboBoxEnumeracion<TipoResponsable> cmbTipoResponsable;
		protected ComboBoxEnumeracion<CondicionPago> cmbCondicionPago;
		protected ComboBoxEnumeracion<FacturaTipoVenta> cmbTipoFactura;
		protected ComboBoxGenerico<CodigoFacturaDto> cmbCodigoFactura;
		protected CodigoFacturaDto[] codigosDeFactura;
		
		protected CheckButton chkClienteConsumidorFinal;
		
		protected Menu menuOtrasAcciones;
		
		protected AdministradorEstadoObjeto administradorEstado;
		protected FacturaGenericaDto facturaCargada;
		
		protected IAccionadorSobreFactura[] accionadoresFactura;
		
		#region Constructores
		public SolapaFacturacion()
		{
			if (this.GetType() == typeof(SolapaFacturacion))
				instanciaActual = this;
			
			this.Build();
			
			// Agrego la lista de items
			this.tvItemsProducto = new ListaItemsVenta();
			this.itemsProducto = new ItemsInteractivo(this.tvItemsProducto);
			this.hbItems.PackStart(this.itemsProducto, true, true, 0);
			
			this.dcFecha.Fecha = DateTime.Today;
			
			this.chkFacturaManual.Active = false;
			this.rtxtNroFactura.Sensitive = false;
			
			// Los items de producto
			this.tvItemsProducto.FacturaTipo = FacturaTipoVenta.FacturaA;
			
			#region ComboBoxes
			this.cmbTipoResponsable = new ComboBoxEnumeracion<TipoResponsable>();
			this.cmbTipoResponsable.Renderizador = delegate (TipoResponsable tipoResponsable) {
				return StratosUtil.SplitCamelString(tipoResponsable.ToString());
			};
			this.hbTipoIva.Add(this.cmbTipoResponsable);
			
			this.cmbCondicionPago = new ComboBoxEnumeracion<CondicionPago>();
			this.cmbCondicionPago.Renderizador = delegate (CondicionPago condicionPago) {
				return StratosUtil.SplitCamelString(condicionPago.ToString());
			};
			this.cmbCondicionPago.Resetear();
			this.hbAtributos.PackStart(this.cmbCondicionPago);
			this.hbAtributos.ReorderChild(this.cmbCondicionPago, 3);
			
			this.txtVence.Sensitive = false;
			
			// Combo tipo factura
			this.cmbTipoFactura = new ComboBoxEnumeracion<FacturaTipoVenta>();
			this.cmbTipoFactura.Renderizador = delegate (FacturaTipoVenta facturaTipo) {
				return StratosUtil.SplitCamelString(facturaTipo.ToString());
			};
			this.cmbTipoFactura.Changed += delegate {
				// No cambio el combobox de tiporesponsable si hay un cliente actualmente
				// cargado
				if (this.razonSocial.Cliente != null)
					return;
				
				if (this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaA) {
					// Desactivo el botón de cliente genérico
					this.tbtnConsumidorFinalGenerico.Active = false;
					this.tbtnConsumidorFinalGenerico.Sensitive = false;
				}
				else
					// Activo el botón de cliente genérico
					this.tbtnConsumidorFinalGenerico.Sensitive = true;
			};
			this.cmbTipoFactura.Resetear();
			this.hbFacturaTipo.Add(this.cmbTipoFactura);
			
			// Combo código factura
			this.cmbCodigoFactura = new ComboBoxGenerico<CodigoFacturaDto>(new CodigoFacturaDtoEqualityComparer());
			this.cmbCodigoFactura.Renderizador = delegate (CodigoFacturaDto codigoFactura) {
				return codigoFactura.codigo.ToString("D4");
			};
			
			this.hbCodigoNumeroFactura.PackStart(this.cmbCodigoFactura);
			this.hbCodigoNumeroFactura.ReorderChild(this.cmbCodigoFactura, 0);
			
			this.CargarCodigosFactura(chkFacturaManual.Active);
			
			// Me suscribo al evento de ConexionEstablecida para reinicializar
			// el ComboBox de los Codigos de Factura cuando el cliente se conecta
			MainIce.ConexionEstablecida += delegate {
				this.CargarCodigosFactura(chkFacturaManual.Active);
			};
			
			#endregion
			
			#region Administrador del estado del presupuesto
			// Creo el objeto administrador del estado del presupuesto
			this.administradorEstado =
				new AdministradorEstadoObjeto(estado => this.lblEstadoFactura.Markup = estado,
				                              x => this.nbGuardarIngresar.Sensitive = x);
//				                              x => this.btnGuardar.Sensitive = x && !this.btnIngresar.Sensitive);
			
			this.dcFecha.FechaCambiada += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.cmbCodigoFactura.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.rtxtNroFactura.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.razonSocial.RazonSocialEntry.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.razonSocial.CodigoPersonaEntry.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.cmbTipoResponsable.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtDireccion.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtLocalidad.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.rtxtCuit.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.rtxtCodigoPostal.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtProvincia.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtBonificacion.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtNroRemito.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.cmbCondicionPago.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			this.txtVence.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.itemsProducto.TreeView.ItemsCambiados += delegate { this.administradorEstado.DatosHanCambiado(); };
			
			this.txtObservaciones.Buffer.Changed += delegate { this.administradorEstado.DatosHanCambiado(); };
			#endregion
			
			#region Suscripción a eventos básicos
			this.razonSocial.PersonaCargada += this.OnClienteCargado;
			this.razonSocial.PersonaQuitada += this.OnClienteQuitado;
			this.cmbCondicionPago.Changed += this.OnCmbCondicionPagoChanged;
			this.cmbTipoResponsable.Changed += OnCmbTipoResponsableChanged;
			#endregion
			
			#region Eventos que requieren recalcular importes
			this.txtBonificacion.TextoValidoIngresado += CalcularImportes;
			this.itemsProducto.TreeView.ItemsCambiados += CalcularImportes;
			this.cmbTipoFactura.Changed += delegate {
				this.tvItemsProducto.FacturaTipo =
					this.cmbTipoFactura.ObjetoActivo;
			};
			#endregion
			
			#region Navegador de elementos
			this.navegadorElementos.Sensitive = false;
			
			// Seteo el proveedor de la instancia que implementa
			// la interface IListaNavegable
			this.navegadorElementos.ProveedorListaNavegable = delegate {
				return SolapaVentas.Instancia;
			};
			
			// Seteo el cargador
			this.navegadorElementos.Cargador = delegate(object o) {
				this.CargarElementoNavegable((FacturaGenericaDto) o);
				
				// Si despues de cargar el estado es 'Cargado', retorno true
				if (this.administradorEstado.EstaCargado)
					return true;
				
				return false;
			};
			#endregion
			
			this.ConstruirAcciones();
			
			foreach (IAccionadorSobreFactura accionador in this.accionadoresFactura)
				accionador.VisualizadorFactura = this;
			
			this.VaciarFormulario();
		}
		#endregion
		
		#region Propiedades
		public static SolapaFacturacion InstanciaActual {
			get { return instanciaActual; }
		}
		
		public AdministradorEstadoObjeto AdministradorEstado {
			get { return this.administradorEstado; }
		}
		
		public FacturaGenericaDto FacturaGenerica {
			get { return this.facturaCargada; }
		}
		
		public virtual string LaElNombreDocumento {
			get { return "La factura"; }
		}
		
		public virtual string UnUnaNombreDocumento {
			get { return "Una factura"; }
		}
		
		public virtual string LaElNombrePreDocumento {
			get { return "La pre factura"; }
		}
		
		public virtual string UnUnaNombrePreDocumento {
			get { return "Una pre factura"; }
		}
		
		protected virtual bool EsSolapaNotaCredito {
			get { return false; }
		}
		
		protected ToggleButton ChkFacturaManual {
			get { return this.chkFacturaManual; }
		}
		
		protected HBox HbFecha {
			get { return this.hbFecha; }
		}
		
		protected Label LblEstadoFactura {
			get { return this.lblEstadoFactura; }
		}
		
		protected VBox VbFacturacion {
			get { return this.vbFacturacion; }
		}
		
		protected HBox HbBotonera {
			get { return this.hbBotonera; }
		}
		
		protected Button BtnGuardar {
			get { return this.btnGuardar; }
		}
		
		protected Button BtnNueva {
			get { return this.btnNueva; }
		}
		
		protected Label LblDatosCliente {
			get { return this.lblDatosCliente; }
		}
		
		protected Table TbDatosCliente {
			get { return this.tbDatosCliente; }
		}
		
		protected HBox HbClienteRazonSocial {
			get { return this.hbox2; }
		}
		
		protected RazonSocialClienteVista RazonSocial {
			get { return this.razonSocial; }
		}
		
		protected ToggleButton TbtnConsumidorFinalGenerico {
			get { return this.tbtnConsumidorFinalGenerico; }
		}
		
		protected HBox HbDomicilio1 {
			get { return this.hbDomicilio1; }
		}
		
		protected HBox HbDomicilio2 {
			get { return this.hbDomicilio2; }
		}
		
		protected Frame FraAtributosFactura {
			get { return this.fraAtributosFactura; }
		}
		
		protected HBox HbBonificacion {
			get { return this.hbBonificacion; }
		}
		#endregion
		
		#region IVisualizadorNavegable implementation
		public INavegadorElementosVista NavegadorElementos {
			get { return this.navegadorElementos; }
		}
		#endregion
		
		#region Eventos comunes
		/// <summary>
		/// Recibe como argumento una factura genérica y carga los datos del formulario en ella.
		/// Permite especificar un cliente opcional (en el caso de crearse una PreFactura, puede
		/// que por la sugerencia de clientes se necesite utilizar esta función).
		/// </summary>
		/// <param name="facturaGenerica">
		/// A <see cref="FacturaGenericaDto"/>
		/// </param>
		protected virtual void LeerDatosFacturaGenerica(ref FacturaGenericaDto facturaGenerica,
		                                                ClienteDto cliente)
		{
			// Si la Factura genérica que estoy leyendo es una FacturaDto, entonces la cargo
			// como válida, ya que de otra forma el campo 'valida' queda 'false'. Ya que de otra
			// manera, cuando se ingresa una factura manual, esta se ingresaría como inválida.
			FacturaDto facturaDto = facturaGenerica as FacturaDto;
			if (facturaDto != null)
				facturaDto.valida = true;
			
			facturaGenerica.cliente = cliente;
			facturaGenerica.fechaTicks = this.dcFecha.Fecha.Ticks;
			facturaGenerica.tipo = cmbTipoFactura.ObjetoActivo;
			facturaGenerica.codigo = cmbCodigoFactura.ObjetoActivo.codigo;
			
			// Importes
			facturaGenerica.porcentajeBonificacion =
				this.txtBonificacion.Text.Trim() != String.Empty ? 
					StratosUtil.ParseDouble(this.txtBonificacion.Text, 2) : 0.00;
			facturaGenerica.importeBonificacion =
				this.txtImporteBonificacion.ImporteDouble;
			facturaGenerica.importeBruto = this.txtImporteBruto.ImporteDouble;
			facturaGenerica.importeIva = this.txtImporteIva.ImporteDouble;
			facturaGenerica.importeNeto = this.txtImporteNeto.ImporteDouble;
			facturaGenerica.importeTotal = this.txtImporteTotal.ImporteDouble;
			facturaGenerica.importeFinal = facturaGenerica.importeTotal;
			
			// Condición de venta
			facturaGenerica.condVenta.condPago =
				(CondicionPago)this.cmbCondicionPago.ObjetoActivo;
			
			if (facturaGenerica.condVenta.condPago != CondicionPago.Contado &&
			    	StratosUtil.IsInt(this.txtVence.Text))
				facturaGenerica.condVenta.plazo = int.Parse(this.txtVence.Text);
			
			// Items
			facturaGenerica.items = this.tvItemsProducto.Items.Cast<ItemGenericoDto>().ToList();
			
			// Remitos y observaciones
			facturaGenerica.remitos = this.txtNroRemito.Text;
			facturaGenerica.observaciones = this.txtObservaciones.Buffer.Text;
		}
		
		/// <summary>
		/// Recibe como argumento una factura genérica y carga los datos del formulario en ella.
		/// Lee el cliente actualmente cargado en el widget RazonSocial.
		/// </summary>
		/// <param name="facturaGenerica">
		/// A <see cref="FacturaGenericaDto"/>
		/// </param>
		protected virtual void LeerDatosFacturaGenerica(ref FacturaGenericaDto facturaGenerica)
		{
			ClienteDto cliente;
			
			// Obtengo el cliente cargado, ya sea desde la BD o ingresado manualmente
			if (this.razonSocial.Cliente != null)
				cliente = this.razonSocial.Cliente;
			else
				cliente = this.ObtenerClienteIngresado();
			
			this.LeerDatosFacturaGenerica(ref facturaGenerica, cliente);
		}
		
		protected virtual void OnCmbTipoResponsableChanged(object sender, System.EventArgs e)
		{
			// Si el tipo responsable es 'Responsable Inscripto' o 'Responsable no inscripto'
			// seteo el tipo de factura en A
			if (this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.ResponsableInscripto ||
			    this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.ResponsableNoInscripto)
				
				this.cmbTipoFactura.ObjetoActivo = FacturaTipoVenta.FacturaA;
			
			// Sino, seteo el tipo de factura en B
			else 
				this.cmbTipoFactura.ObjetoActivo = FacturaTipoVenta.FacturaB;
			
			// Ahora, si es Consumidor final, desactivo el campo cuit, sino lo activo
			if (this.cmbTipoResponsable.ObjetoActivo == TipoResponsable.ConsumidorFinal)
				this.rtxtCuit.Sensitive = false;
			else
				this.rtxtCuit.Sensitive = true;
		}
		
		protected virtual ClienteDto SugerirClientes()
		{
			ClienteDto cliente = null;
			
			// Si hay un cliente cargado, lo retorno
			if (this.razonSocial.Cliente != null)
				return this.razonSocial.Cliente;
			
			// Si no hay un cliente cargado, pero tampoco se escribió algo en la razón social
			// devuelvo null
			if (this.razonSocial.RazonSocialEntry.Text.Trim() == String.Empty)
				return null;
			
			/* Si no hay un cliente cargado, entonces intento sugerir posibles clientes 
			 * que ya existen en la base de datos. */
			ClienteDtoQuery consultaCliente = new ClienteDtoQuery();
			consultaCliente.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			// Consulta por Razón Social
			CriterioAnd criterioRazonSocial = new CriterioAnd();
			criterioRazonSocial.criterios = new List<CriterioGenerico>();
			
			CriterioSimple temp;
			
			foreach (string palabra in this.razonSocial.RazonSocialEntry.Text.Split(' ')) {
				temp = new CriterioSimple();
				temp.operador = TipoOperador.LikeMatchAnywhere;
				temp.propiedad = "RazonSocial";
				temp.valor = palabra;
				
				criterioRazonSocial.criterios.Add(temp);
			}
			
			consultaCliente.listaCriteriosComplejos.Add(criterioRazonSocial);
			
			// Consulta por Tipo Responsable
			CriterioOr criterioTipoResponsable = new CriterioOr();
			criterioTipoResponsable.criterios = new List<CriterioGenerico>();
			
			if (this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaA) {
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "TipoResponsable";
				temp.tipoValor = "TipoResponsable";
				temp.valor = TipoResponsable.ResponsableInscripto.ToString();
				
				criterioTipoResponsable.criterios.Add(temp);
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "TipoResponsable";
				temp.tipoValor = "TipoResponsable";
				temp.valor = TipoResponsable.ResponsableNoInscripto.ToString();
				
				criterioTipoResponsable.criterios.Add(temp);
			}
			else {
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "TipoResponsable";
				temp.tipoValor = "TipoResponsable";
				temp.valor = TipoResponsable.ConsumidorFinal.ToString();
				
				criterioTipoResponsable.criterios.Add(temp);
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "TipoResponsable";
				temp.tipoValor = "TipoResponsable";
				temp.valor = TipoResponsable.Exento.ToString();
				
				criterioTipoResponsable.criterios.Add(temp);
				
				temp = new CriterioSimple();
				temp.operador = TipoOperador.Igual;
				temp.propiedad = "TipoResponsable";
				temp.tipoValor = "TipoResponsable";
				temp.valor = TipoResponsable.Monotributo.ToString();
				
				criterioTipoResponsable.criterios.Add(temp);
			}
			
			consultaCliente.listaCriteriosComplejos.Add(criterioTipoResponsable);
			
			// Realizo la búsqueda al servidor
			Ice.Object[] resultadoQueryObjects =
			((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"))
				.BuscarQueryObjects(consultaCliente, null);
			
			ClienteDto[] clientesCandidatos = resultadoQueryObjects.Cast<ClienteDto>().ToArray();
			
			// Si hay sugerencias, entonces se las muestro al usuario...
			if (clientesCandidatos.Length > 0) {
				// Creo el diálogo de sugerencias
				SugerenciaClientes sugerenciaClientesDialog =
					new SugerenciaClientes(MainGui.MainWindow, clientesCandidatos);
				
				ResponseType respuesta = (ResponseType) sugerenciaClientesDialog.Run();
				
				// De acuerdo a la respuesta del usuario en el diálogo, realizo una acción:
				
				/* Si el usuario escogió cancelar, no continúo y vuelvo a la edición 
				 * de la factura. */
				if (respuesta == ResponseType.Cancel) {
					sugerenciaClientesDialog.Destroy();
					
					return null;
				}
				
				/* Si la respuesta es Apply, entonces corresponde al botón de "Cargar cliente",
				 * lo que carga el cliente en la factura que se está editando pero no continúa
				 * con la impresión. */
				else if (respuesta == ResponseType.Apply) {
					this.razonSocial
						.CargarCliente(sugerenciaClientesDialog.TreeViewClientes
						               .ObtenerSeleccionadosGenericos()[0]);
					
					sugerenciaClientesDialog.Destroy();
					
					return null;
				}
				
				/* Si la respuesta del diálogo es Accept, entonces se ignora la lista de
				 * clientes y se continúa con la impresión de la factura. */
				else if (respuesta == ResponseType.Accept) {
					cliente = this.ObtenerClienteIngresado();
					
					sugerenciaClientesDialog.Destroy();
				}
			}
			
			// ... sino hay sugerencias, cargo el cliente ingresado (éste se dará
			// de alta en el servidor automáticamente)
			else
				cliente = this.ObtenerClienteIngresado();
			
			return cliente;
		}
		
		protected virtual FacturaGenericaDto CrearPreDocumentoNuevo()
		{
			return new PreFacturaDto();
		}
		
		protected virtual FacturaGenericaDto CrearDocumentoNuevo()
		{
			return new FacturaDto();
		}
		
		protected virtual void GuardarClicked (object sender, System.EventArgs e)
		{
			// Luego valido los datos cargados en la factura
			try {
				this.ValidarDatosIngresados();
			}
			catch(DatosIngresadosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.Message);
				return;
			}
			
			#region Sugerencia de clientes para la factura
			/* Primero verifico si el cliente ingresado ha sido cargado desde el
			 * servidor, o se trata de un cliente no ingresado en la base
			 * de datos. En este último caso, realizamos una búsqueda para sugerir posibles
			 * clientes ya ingresados. */
			
			ClienteDto cliente = this.SugerirClientes();
			
			if (cliente == null)
				return;
			#endregion
			
			#region Lógica para guardar la PreFactura
			try {
				ControladorFacturacionPrx cf = 
					((ControladorFacturacionPrx)MainIce.GetControlador("ControladorFacturacion"));
				
				// Si no hay una factura guardada, entonces creo una nueva
				if (this.facturaCargada == null)
					this.facturaCargada = this.CrearPreDocumentoNuevo();
				
				this.LeerDatosFacturaGenerica(ref this.facturaCargada, cliente);
				
				// Primero veo si se trata de una factura manual por guardar
				FacturaDto factura = this.facturaCargada as FacturaDto;
				
				if (factura != null && factura.facturaManual) {
					factura = (FacturaDto) cf.GuardarFacturaManual(factura);
					
					// Cargo en la interfaz gráfica la factura manual guardada
					this.CargarFacturaManualExistenteIngresada(factura);
					
					this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
				}
				
				// Veo si se trata de una PreFactura y tengo que Guardar...
				else if (this.facturaCargada.id <= 0) {
					PreFacturaDto preFacturaGuardada = (PreFacturaDto)
						cf.Guardar((PreFacturaDto) this.facturaCargada);
					
					// Cargo en la interfaz gráfica la PreFactura recientemente guardada.
					this.CargarPreFacturaNuevaGuardada(preFacturaGuardada);
					
					this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
					
					SMessageDialog
						.MensajeInformacion(this.LaElNombrePreDocumento + " se ha creado correctamente.");
				}
				
				// ... o Modificar la factura existente
				else {
					PreFacturaDto preFacturaGuardada = (PreFacturaDto)
						cf.Modificar((PreFacturaDto) this.facturaCargada);
					
					// Cargo en la interfaz gráfica la PreFactura recientemente guardada.
					this.CargarPreFacturaExistenteGuardada(preFacturaGuardada);
					
					this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
				}
			}
			catch (ObjetoInvalidoException ex) {
			  	SMessageDialog.MensajeError(this.LaElNombreDocumento + " tiene datos inválidos:\n" +
				                            ex.mensaje);
			}
			catch (ObjetoVersionIncorrectaException ex) {
			  	SMessageDialog
					.MensajeError("<b>" + this.LaElNombreDocumento + " que intenta modificar fue " +
						"modificada ya por otra terminal</b>.\nVuelva a cargar " +
						this.LaElNombreDocumento + " actual y realice nuevamente las modificaciones");
				return;
			}
			catch (ObjetoInexistenteException ex) {
			  	SMessageDialog
					.MensajeError(this.LaElNombreDocumento + " que se quiere modificar no existe " +
					              "en la base de datos.", ex);
			}
			#endregion
		}
		
		protected virtual void OnBtnIngresarClicked (object sender, System.EventArgs e)
		{
			try {
				this.ValidarDatosIngresados();
			}
			catch(DatosIngresadosInvalidosException ex) {
				SMessageDialog.MensajeError(ex.Message);
				return;
			}
			
			try {
				ControladorFacturacionPrx cf = 
					((ControladorFacturacionPrx)MainIce.GetControlador("ControladorFacturacion"));
				
				FacturaDto factura = (FacturaDto) this.CrearDocumentoNuevo();
				FacturaGenericaDto facturaGenerica = factura;
				
				// Leo todos los datos del formulario y los cargo en el objeto factura
				this.LeerDatosFacturaGenerica(ref facturaGenerica);
				
				// Código y número de la factura
				factura.codigo = cmbCodigoFactura.ObjetoActivo.codigo;
				factura.numero = int.Parse(rtxtNroFactura.Text);
				
				// Registro la factura manual
				FacturaDto facturaManualIngresada = cf.RegistrarFacturaManual(factura);
				
				// Cargo en la interfaz gráfica la Factura manual ingresada
				this.CargarFacturaNuevaGuardada(facturaManualIngresada);
				
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Guardado);
				
				SMessageDialog
					.MensajeInformacion(this.LaElNombreDocumento +
					                    " manual se ha ingresado correctamente.");
			}
			catch (StratosException ex) {
			  	SMessageDialog.MensajeError(ex.mensaje);
				return;
			}	
		}
		
		/// <summary>
		/// Limpia el formulario.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtnLimpiarClicked (object sender, System.EventArgs e)
		{
			// Si hay modificaciones hechas, pregunto al usuario si está seguro de perderlas
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.VaciarFormulario();
			
			// Desactivo el navegador de elementos
			this.navegadorElementos.Sensitive = false;
			
			// Coloco el foco en el código de la razón social
			this.razonSocial.CodigoPersonaEntry.GrabFocus();
		}
		
		/// <summary>
		/// Evento disparado cuando se presiona una tecla. Incluir dentro de este metodo
		/// las teclas que se utilizan para interactuar con la interfaz.
		/// Las siguientes teclas tiene comportamiento:
		/// -Delete: borra el item seleccionado.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.KeyPressEventArgs"/>
		/// </param>
		protected virtual void OnKeyPressed (object o, Gtk.KeyPressEventArgs args)
		{
			/* Si se presiona la Delete, borro el item seleccionado */
			if (args.Event.Key == Gdk.Key.Delete)
				this.itemsProducto.TreeView.QuitarItemsSeleccionados();				
		}
		
		/// <summary>
		/// Método que se ejecuta cuando cambia el Checkbox "Factura manual".
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnChkFacturaManualToggled (object sender, System.EventArgs e)
		{
			/* Si la factura es manual, activo el Entry del número de la factura
			 * y el boton 'Ingresar' y desactivo el boton Imprimir */
			if (this.chkFacturaManual.Active) {
				this.rtxtNroFactura.Sensitive = true;
				
				// Activo la página del notebook con el botón de Ingresar
				this.nbGuardarIngresar.CurrentPage = 1;
			}
			else {
				this.rtxtNroFactura.Text = "";
				this.rtxtNroFactura.Sensitive = false;
				
				// Activo la página del notebook con el botón de Guardar
				this.nbGuardarIngresar.CurrentPage = 0;
			}
			
			// Cargo los códigos de factura según el estado del checkbox
			this.CargarCodigosFactura(this.chkFacturaManual.Active);
		}
		
		protected virtual void OnCmbCondicionPagoChanged(object o, EventArgs args)
		{
			if (this.cmbCondicionPago.ObjetoActivo != CondicionPago.Contado) {
				this.txtVence.Text = "0";
				this.txtVence.Sensitive = true;
			}
			else {
				this.txtVence.Text = "";
				this.txtVence.Sensitive = false;
			}
		}
		
		protected virtual void OnConsumidorFinalGenericoToggled (object sender, System.EventArgs e)
		{
			bool activo = this.tbtnConsumidorFinalGenerico.Active;
			
			// Si el botón se activó, cargo un cliente consumidor final genérico
			if (activo) {
				ClienteDto[] cliente =
				((ControladorClientesPrx) MainIce.GetControlador("ControladorClientes"))
					.ObtenerClienteConsumidorFinalGenerico();
				
				this.razonSocial.CargarCliente(cliente[0]);
				
				this.cmbCondicionPago.ObjetoActivo = CondicionPago.Contado;
			}
			else
				this.razonSocial.Vaciar();
			
			// Habilito o deshabilito los controles del clientes según corresponda
			this.DatosClienteEditables(!this.tbtnConsumidorFinalGenerico.Active);
		}
		#endregion
		
		#region Carga de facturas/prefacturas
		protected void CargarElementoNavegable(FacturaGenericaDto facturaGenerica)
		{
			// Casteo a varios tipos para saber qué clase de objeto estoy cargando
			PreFacturaDto preFacturaDto = facturaGenerica as PreFacturaDto;
			PreNotaCreditoDto preNotaCredito = facturaGenerica as PreNotaCreditoDto;
			
			FacturaDto facturaDto = facturaGenerica as FacturaDto;
			NotaCreditoDto notaCreditoDto = facturaGenerica as NotaCreditoDto;
			
			// Ejecuto el visualizador que corresponda
			if (preNotaCredito != null)
				SolapaVentas.Instancia.EjecutarVisualizador(preNotaCredito, true);
			
			else if (preFacturaDto != null)
				SolapaVentas.Instancia.EjecutarVisualizador(preFacturaDto, true);
			
			else if (notaCreditoDto != null)
				SolapaVentas.Instancia.EjecutarVisualizador(notaCreditoDto, true);
			
			else
				SolapaVentas.Instancia.EjecutarVisualizador(facturaDto, true);
		}
		
		/// <summary>
		/// Este método es utilizado por el Navegador de Elementos
		/// </summary>
		/// <param name="objeto">
		/// A <see cref="System.Object"/>
		/// </param>
		public void CargarFacturaGenerica(object objeto)
		{
			if (objeto is FacturaDto) {
				FacturaDto factura = (FacturaDto) objeto;
				this.CargarFactura(factura);
			}
			else {
				PreFacturaDto prefactura = (PreFacturaDto) objeto;
				this.CargarFactura(prefactura);
			}
		}
		
		/// <summary>
		/// Carga una FacturaGenerica (Factura o PreFactura).
		/// </summary>
		/// <param name="facturaGenerica">
		/// A <see cref="FacturaGenericaDto"/>
		/// </param>
		protected void CargarFacturaGenerica(FacturaGenericaDto facturaGenerica, bool esFacturaManual, bool habilitarCheckFacturaManual = false)
		{
			// Antes de cargar la factura, vacio el formulario
			this.VaciarFormulario();
			
			this.facturaCargada = facturaGenerica;
			
			// Checkbox de factura manual
			this.chkFacturaManual.Active = esFacturaManual;
			
			// Código de factura
			this.cmbCodigoFactura.ObjetoActivo = new CodigoFacturaDto(facturaGenerica.codigo, "", false, false);
			
			// Desactivo los widgets que no corresponden cuando se carga una factura
			this.chkFacturaManual.Sensitive = habilitarCheckFacturaManual;
			this.nbGuardarIngresar.CurrentPage = 0;
			
			// Datos generales
			this.dcFecha.Fecha = new DateTime(facturaGenerica.fechaTicks);
			this.cmbTipoFactura.ObjetoActivo = facturaGenerica.tipo;
			
			if (facturaGenerica.cliente != null) {
				/* Reviso si el cliente está persistido. Esto es necesario porque la factura
				 * se puede estar cargando desde un Presupuesto (Presupussto -> Factura) */
				if (facturaGenerica.cliente.codigo > 0)
					this.razonSocial.CargarCliente(facturaGenerica.cliente);
				else {
					this.razonSocial.RazonSocialEntry.Text =
						facturaGenerica.cliente.razonSocial;
				}
			}
			
			this.txtNroRemito.Text = facturaGenerica.remitos;
			this.cmbCondicionPago.ObjetoActivo = facturaGenerica.condVenta.condPago;
			this.txtVence.Text = facturaGenerica.condVenta.plazo.ToString();
			this.txtBonificacion.Text = facturaGenerica.porcentajeBonificacion.ToString();
			
			// Items
			this.itemsProducto.Vaciar(false);
			if (facturaGenerica.items != null)
				this.itemsProducto.AgregarItems(facturaGenerica.items.ToArray());
			else
				this.itemsProducto.AgregarItems();
			
			// Otros datos
			this.txtObservaciones.Buffer.Text = facturaGenerica.observaciones;
			
			// Importes
//			this.txtImporteBruto.Importe = (decimal) facturaGenerica.importeBruto;
//			this.txtImporteBonificacion.Importe = (decimal) facturaGenerica.importeBonificacion;
//			this.txtImporteNeto.Importe = (decimal) facturaGenerica.importeNeto;
//			this.txtImporteIva.Importe = (decimal) facturaGenerica.importeIva;
//			// FIXME: Esto no está bien:
//			this.txtImporteIvaNoInscripto.Importe = 0.0m;
//			this.txtImporteTotal.Importe = (decimal) facturaGenerica.importeTotal;
			
			this.CalcularImportes();
			
			// Aviso a cada accionador que se ha cargado una nueva factura
			foreach (IAccionadorSobreFactura accionador in this.accionadoresFactura)
				accionador.Factura = this.facturaCargada;
		}
		
		#region Carga de PreFacturas
		/// <summary>
		/// Se utiliza en el método de guardado. Carga una PreFactura recientemente
		/// creada (que no existía).
		/// </summary>
		/// <param name="preFacturaGuardada">
		/// A <see cref="PreFacturaDto"/>
		/// </param>
		public void CargarPreFacturaNuevaGuardada(PreFacturaDto preFacturaGuardada)
		{
			this.CargarFactura(preFacturaGuardada, true);
			
			// Desactivo el navegador de elementos
			this.navegadorElementos.Sensitive = false;
		}
		
		/// <summary>
		/// Se utiliza en el método de guardado. Carga una PreFactura reciemente
		/// guardada (pero que ya existía, o sea, que fue modificada).
		/// </summary>
		/// <param name="preFacturaGuardada">
		/// A <see cref="PreFacturaDto"/>
		/// </param>
		public void CargarPreFacturaExistenteGuardada(PreFacturaDto preFacturaGuardada)
		{
			this.CargarFactura(preFacturaGuardada, true);
		}
		
		/// <summary>
		/// Carga una PreFactura en el formulario. Brinda la posibilidad de especificar si se
		/// debe hacer la pregunta por las modificaciones o no.
		/// </summary>
		/// <param name="presupuesto">
		/// A <see cref="PresupuestoDto"/>
		/// </param>
		/// <param name="evitarPreguntaPorModificaciones">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public void CargarFactura(PreFacturaDto preFactura, bool evitarPreguntaPorModificaciones)
		{
			if (!evitarPreguntaPorModificaciones &&
			    	(this.administradorEstado.EstaModificado &&
			     		!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones))
				return;
			
			this.CargarPreFacturaInterno(preFactura, true);
		}
		
		/// <summary>
		/// Carga una PreFactura en el formulario.
		/// </summary>
		/// <param name="preFactura">
		/// A <see cref="PreFacturaDto"/>
		/// </param>
		public void CargarFactura(PreFacturaDto preFactura)
		{
			// Primero me fijo si no hay modificaciones sin guardar
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			// Primero cargo todos los datos comunes a una factura genérica
			this.CargarPreFacturaInterno(preFactura, false);
			
			// Si la prefactura cargada no tiene un id asignado, es porque es una prefactura
			// nueva que fue carga desde un presupuesto.
			if (preFactura.id <= 0) {
				this.tvItemsProducto.FacturaTipo = preFactura.tipo;
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Modificado, "MODIFICADO");
				this.navegadorElementos.Sensitive = false;
			}
		}
		
		/// <summary>
		/// Método interno. Carga una PreFactura en el formulario.
		/// </summary>
		/// <param name="preFactura">
		/// A <see cref="PreFacturaDto"/>
		/// </param>
		protected void CargarPreFacturaInterno(PreFacturaDto preFactura, bool esPreNotaCredito)
		{
			/* Si es una nota de credito no chequeo la validez del codigo,
			 * ya que se carga desde la factura asociada. */
			if (!esPreNotaCredito)
			{
				// Si el codigo de factura no es correcto, no cargo la factura
				if (!CodigoFacturaCorrecto(preFactura.codigo, false))
				{
					// Vacio el formulario antes de salir.
					this.VaciarFormulario();
					return;
				}
				
			}
			
			// Primero cargo todos los datos comunes a una factura genérica
			this.CargarFacturaGenerica(preFactura, false, esPreNotaCredito);
			
			// Estado de la factura
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "PRE-FACTURA CARGADA");
			
			// Carga extra
			this.CargarFacturaExtra(preFactura);
		}
		
		#endregion
		
		#region Carga de Facturas
		/// <summary>
		/// Carga una Factura manual ingresada en el formulario.
		/// </summary>
		/// <param name="facturaManualIngresada">
		/// A <see cref="FacturaDto"/>
		/// </param>
		public void CargarFacturaNuevaGuardada(FacturaDto facturaManualIngresada)
		{
			this.CargarFactura(facturaManualIngresada, true);
		}
		
		public void CargarFacturaManualExistenteIngresada(FacturaDto facturaManualIngresada)
		{
			this.CargarFactura(facturaManualIngresada, true);
		}
		
		/// <summary>
		/// Carga una Factura en el formulario. Permite indicar si preguntar al usuario o
		/// no por los cambios sin guardar en el formulario (se evita la pregunta para ciertos
		/// casos, como cuando se carga la factura recientemente guardada).
		/// </summary>
		/// <param name="factura">
		/// A <see cref="FacturaDto"/>
		/// </param>
		/// <param name="evitarPreguntaPorModificaciones">
		/// A <see cref="System.Boolean"/>
		/// </param>
		protected void CargarFactura(FacturaDto factura, bool evitarPreguntaPorModificaciones)
		{
			if (!evitarPreguntaPorModificaciones &&
			    	(this.administradorEstado.EstaModificado &&
			     		!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones))
				return;
			
			this.CargarFacturaInterno(factura);
		}
		
		/// <summary>
		/// Carga una Factura en el formulario. Si hay modificaciones hechas en el formulario
		/// sin guardar, pregunta al usuario antes de la carga.
		/// </summary>
		/// <param name="factura">
		/// A <see cref="FacturaDto"/>
		/// </param>
		public void CargarFactura(FacturaDto factura)
		{
			// Primero me fijo si no hay modificaciones sin guardar
			if (this.administradorEstado.EstaModificado &&
			    	!this.administradorEstado.UsuarioEstaSeguroDePerderModificaciones)
				return;
			
			this.CargarFacturaInterno(factura);
		}
		
		protected void CargarFacturaInterno(FacturaDto factura)
		{			
			// Si el codigo de factura no es correcto, no cargo la factura
			if (!CodigoFacturaCorrecto(factura.codigo, factura.facturaManual))
			{
				// Vacio el formulario
				this.VaciarFormulario();
				return;
			}

			// Cargo todos los datos comunes a una factura genérica
			this.CargarFacturaGenerica(factura, factura.facturaManual);
			
			this.rtxtNroFactura.Text = factura.numero.ToString();
			this.rtxtNroFactura.Sensitive = false;
			
			// Desactivo la interfaz gráfica según corresponda
			// Si se trata de una factura manual, activo la interfaz para que se pueda guardar
			// los cambios
			
			this.vbCabeceraFactura.Sensitive = factura.facturaManual && !factura.historica;
			
			// Muestro el botón de guardar, y lo activo o no dependiendo de la factura.
			this.nbGuardarIngresar.CurrentPage = 0;
			this.nbGuardarIngresar.Sensitive = factura.facturaManual && !factura.historica;
			
			this.razonSocial.BotonLimpiar.Sensitive = factura.facturaManual && !factura.historica;
			this.tbtnConsumidorFinalGenerico.Sensitive = false;
			
			CustomWidgetsUtils.WidgetEditable(this.txtNroRemito, factura.facturaManual && !factura.historica);
			CustomWidgetsUtils.WidgetEditable(this.cmbCondicionPago, factura.facturaManual && !factura.historica);
			CustomWidgetsUtils.WidgetEditable(this.txtVence, factura.facturaManual && !factura.historica);
			CustomWidgetsUtils.WidgetEditable(this.txtBonificacion, factura.facturaManual && !factura.historica);
			
			this.itemsProducto.Sensitive = factura.facturaManual && !factura.historica;
			
			CustomWidgetsUtils.WidgetEditable(this.txtObservaciones, factura.facturaManual && !factura.historica);
			
			// Estado de la factura
			if (factura.anulada)
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "FACTURA ANULADA", "red");
			else if (factura.facturaManual)
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "FACTURA MANUAL CARGADA");
			else
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "FACTURA CARGADA");
			
			// Ejecuto el método de carga extra (subclases)
			this.CargarFacturaExtra(factura);
		}

		#endregion
		
		#region Métodos de carga extra (para que lo implementen las subclases)
		protected virtual void CargarFacturaExtra(FacturaDto factura)
		{
		}
		
		protected virtual void CargarFacturaExtra(PreFacturaDto preFactura)
		{
		}
		#endregion
		#endregion
		
		#region Otros métodos
		/// <summary>
		/// Carga todas las acciones correspondientes a una Factura/PreFactura, como Imprimir,
		/// Imprimir borrador, etc.
		/// </summary>
		protected void ConstruirAcciones()
		{
			logger.Debug("Cargando acciones para factura");
			
			// Busco las acciones sobre las facturas suscriptas en el extension point
			this.accionadoresFactura =
				(from accionador in
					AddinManager.GetExtensionObjects("/StratosClient/Ventas/Facturacion/Acciones", false)
						.Cast<IAccionadorSobreFactura>()
				 orderby accionador.Orden
				 select accionador).ToArray();
			
			if (this.accionadoresFactura.Count() == 0) {
				logger.Debug("No hay accionadores registrados para las facturas");
				return;
			}
			
			// A la primer acción la coloco en un botón principal
			Gtk.Action primerAccion = this.accionadoresFactura.ElementAt(0).Acciones[0];
			
			Gtk.Button botonPrincipal = new Gtk.Button();
			botonPrincipal.Label = primerAccion.Label;
			botonPrincipal.Image = new Image(primerAccion.StockId, IconSize.Button);
			botonPrincipal.TooltipText = primerAccion.Tooltip;
			
			// Conecto el botón a la Gtk.Action, así se actualizan los estados (por ejemplo Sensitive)
			// cuando se actualiza el Gtk.Action
			primerAccion.ConnectProxy(botonPrincipal);
			botonPrincipal.ShowAll();
			
			this.hbOtrasAcciones.PackStart(botonPrincipal, true, true, 0);
			this.hbOtrasAcciones.ReorderChild(botonPrincipal, 0);
			
			// Agrego las acciones restantes del primer accionador
			this.menuOtrasAcciones = new Menu();
			
			Gtk.Action accionTemp;
			for (int i=1; i<this.accionadoresFactura.ElementAt(0).Acciones.Length; i++) {
				accionTemp = this.accionadoresFactura.ElementAt(0).Acciones[i];
				
				this.menuOtrasAcciones.Append(accionTemp.CreateMenuItem());
			}
			
			// Ahora continuo con los demás accionadores para Factura
			IAccionadorSobreFactura accionadorFactura;
			
			for (int i=1; i<this.accionadoresFactura.Count(); i++) {
				accionadorFactura = this.accionadoresFactura.ElementAt(i);
				
				// Itero por cada Gtk.Action que ofrece el accionador y las
				// agrego al menú de otras acciones
				foreach (Gtk.Action accion in accionadorFactura.Acciones)
					this.menuOtrasAcciones.Append(accion.CreateMenuItem());
				
				if (i != this.accionadoresFactura.Count() - 1)
					this.menuOtrasAcciones.Append(new SeparatorMenuItem());
			}
			
			this.menuOtrasAcciones.ShowAll();
		}
		
		protected void OnClienteQuitado()
		{
			// Cargo un cliente vacío para limpiar los campos
			ClienteDto clienteVacio = new ClienteDto();
			clienteVacio.codigo = -1;
			clienteVacio.domicilio = new DomicilioDto();
			
			this.CargarDatosCliente(clienteVacio);
			
			// Habilito los widgets
			this.DatosClienteEditables(true);
			
			this.tbtnConsumidorFinalGenerico.Active = false;
			this.tbtnConsumidorFinalGenerico.Sensitive =
				this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaB;
			
			// Según el tipo de factura, coloco el tipo de responsable por defecto
			if (this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaA)
				this.cmbTipoResponsable.ObjetoActivo = TipoResponsable.ResponsableInscripto;
			else
				this.cmbTipoResponsable.ObjetoActivo = TipoResponsable.ConsumidorFinal;
			
			// Reseteo el combo de tipo de venta, ya que puede cambiar cuando se carga un cliente
			this.cmbCondicionPago.Resetear();
		}
		
		protected void OnClienteCargado(ClienteDto cliente)
		{
			/* Si el cliente cargado es nulo o vacio, entonces se ha presionado el botón
			 * para limpiar el cliente cargado. Lo limpiamos bien limpito pero sin
			 * cobrarle nada :P */
			
			/* Si se ha limpiado el cliente cargado, entonces todos los campos son editables
			 * nuevamente. Si se ha cargado un cliente, entonces no podemos editar los datos
			 * del mismo en la factura. */
			
			// Cargo los datos del cliente
			this.CargarDatosCliente(cliente);
			
			// Marco los widgets de edición de datos del cliente como no editables
			this.DatosClienteEditables(false);
			
			// Una vez cargado todo, coloco el foco en el Entry de Remitos para
			// acelerar la creación de los documentos
			this.txtNroRemito.GrabFocus();
		}
		
		/// <summary>
		/// Marca todos los widgets de entrada de datos del cliente como editables o no,
		/// de acuerdo al parámetro recibido.
		/// </summary>
		/// <param name="editable">
		/// A <see cref="System.Boolean"/>
		/// </param>
		private void DatosClienteEditables(bool editable)
		{
			// Habilito/deshabilito los Entry relacionados con el cliente, según corresponda
			CustomWidgetsUtils.WidgetEditable(this.razonSocial.CodigoPersonaEntry, editable);
			CustomWidgetsUtils.WidgetEditable(this.razonSocial.RazonSocialEntry, editable);
			
			CustomWidgetsUtils.WidgetEditable(this.txtDireccion, editable);
			CustomWidgetsUtils.WidgetEditable(this.txtLocalidad, editable);
			CustomWidgetsUtils.WidgetEditable(this.rtxtCodigoPostal, editable);
			CustomWidgetsUtils.WidgetEditable(this.txtProvincia, editable);
			CustomWidgetsUtils.WidgetEditable(this.cmbTipoResponsable, editable);
			CustomWidgetsUtils.WidgetEditable(this.rtxtCuit, editable);
		}
		
		/// <summary>
		/// Permite cargar los datos restantes del cliente en la interfaz gráfica, a partir
		/// del evento ClienteCargado del widget RazonSocial.
		/// </summary>
		/// <param name="cliente">
		/// A <see cref="ClienteDto"/>
		/// </param>
		protected void CargarDatosCliente(ClienteDto cliente)
		{
			/* El código y la razón social ya están cargados en el widget RazonSocial.
			 * Aquí cargo los datos restantes. */
			
			if (cliente.domicilio == null)
				cliente.domicilio = new DomicilioDto();
			
			this.txtDireccion.Text =
				cliente.domicilio.direccion == null ? "" : cliente.domicilio.direccion;
			this.txtLocalidad.Text =
				cliente.domicilio.localidad == null ? "" : cliente.domicilio.localidad;
			this.rtxtCodigoPostal.Text =
				cliente.domicilio.codigoPostal == null ? "" : cliente.domicilio.codigoPostal;
			
			if (cliente.domicilio.direccion == null &&
			    cliente.domicilio.localidad == null &&
			    cliente.domicilio.codigoPostal == null) {
				
				this.txtProvincia.Text = String.Empty;
			}
			else {
				this.txtProvincia.Text = cliente.domicilio.provincia.ToString();
				this.cmbTipoResponsable.ObjetoActivo = cliente.tipoRes;
			}
			
			this.rtxtCuit.Text = cliente.cuit == null ? "" : cliente.cuit;
			this.txtBonificacion.Text = cliente.porcentajeBonificacion.ToString();
			
			// Cargo la condición de venta por defecto
			if ((cliente.condicionPagoPorDefecto != null) && (cliente.condicionPagoPorDefecto.Length > 0))
				this.cmbCondicionPago.ObjetoActivo = cliente.condicionPagoPorDefecto[0];
		}
		
		protected virtual void CalcularImportes()
		{
			decimal importeBruto;
			decimal importeBonificacion;
			decimal importeNeto;
			decimal importeIva = 0.00m;

			importeBruto =
				StratosUtil.RoundDecimal(this.tvItemsProducto.SumaImportesItemes(), 2);
			this.txtImporteBruto.Importe = importeBruto;
			
			importeBonificacion = this.CalcularImporteBonificacion(importeBruto);
			this.txtImporteBonificacion.Importe = importeBonificacion;
			
			importeNeto = importeBruto - importeBonificacion;
			this.txtImporteNeto.Importe = importeNeto;

			/* TODO: Ver como se calcula el IVA total de una factura, por ahora deja 21 %
			 * Si es una factura A calculo importe del IVA, sino lo dejo en 0,00 como
			 * está inicializado */
			if ((FacturaTipoVenta)this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaA)
				importeIva = StratosUtil.RoundDecimal(importeNeto * 0.21m, 2);
			
			this.txtImporteIva.Importe = importeIva;
			
			// FIXME: Esto está mal:
			this.txtImporteIvaNoInscripto.Importe = 0.00m;

			this.txtImporteTotal.Importe = importeIva + importeNeto;
		}
		
		/// <summary>
		/// Calcula el importe de bonificación leyendo el porcentaje de bonificación ingresado por
		/// el usuario.
		/// </summary>
		/// <param name="importeBruto">
		/// A <see cref="System.Decimal"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Decimal"/>
		/// </returns>
		protected virtual decimal CalcularImporteBonificacion(decimal importeBruto)
		{
			decimal porcentajeBonificacion = 0.00m;
			
			if (!this.txtBonificacion.Text.Equals("") && this.txtBonificacion.TextoValido)
				porcentajeBonificacion =
					StratosUtil.ParseDecimal(this.txtBonificacion.Text, 2);
			
			return this.CalcularImporteBonificacion(importeBruto, porcentajeBonificacion);
		}
		
		/// <summary>
		/// Calcula el importe de bonificación de acuerdo a un porcentaje dado como
		/// parámetro.
		/// </summary>
		/// <param name="importeBruto">
		/// A <see cref="System.Decimal"/>
		/// </param>
		/// <param name="porcentajeBonificacion">
		/// A <see cref="System.Decimal"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Decimal"/>
		/// </returns>
		protected virtual decimal CalcularImporteBonificacion(decimal importeBruto,
		                                                      decimal porcentajeBonificacion)
		{
			decimal importeBonificacion =
				StratosUtil.RoundDecimal(importeBruto * (porcentajeBonificacion / 100.0m), 2);
			
			return importeBonificacion;
		}

		/// <summary>
		/// Vacía los datos ingresado en el formulario
		/// </summary>
		protected virtual void VaciarFormulario()
		{
			this.facturaCargada = null;
			
			#region Activo los widgets correspondientes
			this.vbCabeceraFactura.Sensitive = true;
			this.chkFacturaManual.Sensitive = true;
			this.cmbCodigoFactura.Sensitive = true;
			
			// Dependiendo del valor de factura manual, muestro el botón de
			// guardar o ingresar, y siempre activo el notebook con estos botones
			this.nbGuardarIngresar.CurrentPage = this.chkFacturaManual.Active ? 1 : 0;
			this.nbGuardarIngresar.Sensitive = true;
			
			this.btnNueva.Sensitive = true;
			
			this.dcFecha.Fecha = DateTime.Today;
			this.cmbCodigoFactura.Resetear();
			this.rtxtNroFactura.Text = String.Empty;
			this.chkFacturaManual.Active = false;
			
			this.razonSocial.BotonLimpiar.Sensitive = true;
			this.tbtnConsumidorFinalGenerico.Sensitive =
				this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaB;
			
			CustomWidgetsUtils.WidgetEditable(this.txtNroRemito, true);
			CustomWidgetsUtils.WidgetEditable(this.cmbCondicionPago, true);
			CustomWidgetsUtils.WidgetEditable(this.txtVence, true);
			CustomWidgetsUtils.WidgetEditable(this.txtBonificacion, true);
			
			this.itemsProducto.Sensitive = true;
			
			CustomWidgetsUtils.WidgetEditable(this.txtObservaciones, true);
			#endregion
			
			#region Cargo los datos por defecto
			// Seteo la condición de pago como "Contado"
			this.cmbCondicionPago.Resetear();
			
			this.txtNroRemito.Text = "";
			this.txtObservaciones.Buffer.Text = "";
			
			// Limpio datos del cliente
			this.tbtnConsumidorFinalGenerico.Active = false;
			this.razonSocial.Vaciar();
			
			// Limpio los importes
			this.txtImporteBonificacion.Limpiar();
			this.txtImporteBruto.Limpiar();
			this.txtImporteIva.Limpiar();
			this.txtImporteNeto.Limpiar();
			this.txtImporteTotal.Limpiar();
			this.txtImporteIvaNoInscripto.Limpiar();
			
			// Borro todos los items
			this.itemsProducto.Vaciar();
			
			// Cargo el nuevo estado de la factura
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo, "PRE-FACTURA NUEVA");
			#endregion
		}

		protected virtual void CambiarBoton(Gtk.Button boton, string label, Gtk.Image image)
		{
			// Creo el contenedor interno para el botón, que tendrá la imagen y el label
			Gtk.Alignment w13 = new Gtk.Alignment(0.5F, 0.5F, 0F, 0F);
			
			Gtk.HBox w14 = new Gtk.HBox();
			w14.Spacing = 2;
			
			// Agrego la imagen
			w14.Add(image);
			
			// Creo y agrego el label
			Gtk.Label w17 = new Gtk.Label();
			w17.LabelProp = label;
			w17.UseUnderline = true;
			
			w14.Add(w17);
			w13.Add(w14);
			
			boton.Remove(boton.Child);
			boton.Add(w13);
			boton.ShowAll();
		}
		
		/// <summary>
		/// Carga los distintos códigos que puede tener una factura según esta sea
		/// manual o generada por el sistema.
		/// </summary>
		/// <param name="esFacturaManual">
		/// A <see cref="System.Boolean"/>
		/// </param>
		protected virtual void CargarCodigosFactura(bool esFacturaManual)
		{
			if (this.codigosDeFactura == null)
			{
				ControladorFacturacionPrx cf =
						(ControladorFacturacionPrx) MainIce.GetControlador("ControladorFacturacion");
				
				// Verifico que haya conexión y que el controlador no sea nulo
				if (!MainIce.ConectadoAlServidor || cf == null)
					return;
				
				// Obtengo todos los codigos de factura
				Ice.Object[] resultado = null;
				try {
					resultado = cf.ObtenerCodigosDeFactura();
				}
				catch (System.Exception ex) {
					MainGui.GestionarException(ex);
					return;
				}
				
				this.codigosDeFactura = resultado.Cast<CodigoFacturaDto>().ToArray();
			}
			
			// Limpio el combo
			this.cmbCodigoFactura.Limpiar();
			
			foreach (CodigoFacturaDto c in this.codigosDeFactura)
			{
				if ((c.facturamanual == esFacturaManual) && 
				    ((c.solonotacredito && EsSolapaNotaCredito) || !c.solonotacredito))
				{
					this.cmbCodigoFactura.AgregarObjeto(c);
				}
			}
			
			// Seteo el primer elemento del nuevo modelo
			this.cmbCodigoFactura.Resetear();
			
			// Vuelvo a setear el estado a Nuevo, ya que el cambiar el combo de
			// codigos provoca que la interfaz se coloque en "Modificado".
			if (this.administradorEstado != null)
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo);
		}
		
		#region Validaciones
		protected virtual void ValidarCliente()
		{
			/* Si la factura es de tipo A, chequeo que tenga un cliente asignado y que este
			 * sea Responsable Inscripto o Responsable No Inscripto */
			if (this.cmbTipoFactura.ObjetoActivo == FacturaTipoVenta.FacturaA) {
				
				if (this.razonSocial.Cliente == null)
					throw new DatosIngresadosInvalidosException("<b>No hay un cliente existente en la base de datos ingresado.</b>\n" +
				          	this.UnUnaNombreDocumento + " de tipo A debe tener asignado un cliente dado de alta en el sistema.");
				
				else if (this.cmbTipoResponsable.ObjetoActivo != TipoResponsable.ResponsableInscripto &&
				         this.cmbTipoResponsable.ObjetoActivo != TipoResponsable.ResponsableNoInscripto)
					throw new DatosIngresadosInvalidosException(this.UnUnaNombreDocumento +" de tipo A solo " +
				          "puede tener asignado un cliente del tipo Responsable Incripto o " +
					      "Responsable No Inscripto");
			}
			
			/* Si la factura es de tipo B, chequeo que tenga un cliente asignado y si no lo tiene aviso
			 * que voy a asumir "Cliente de Contado", y si tiene un cliente asingnado que este sea
			 * Consumidor Final, Exento o Monotributo. */
			else {
				
				if (!this.ClienteIngresado())
					throw new DatosIngresadosInvalidosException("<b>No hay un cliente ingresado.</b>\n" +
				          "Puede presionar el botón <b>Cliente genérico</b> para cargar un cliente " +
					      "de tipo Consumidor Final en condición de pago Contado");
				
				else if (this.cmbTipoResponsable.ObjetoActivo != TipoResponsable.ConsumidorFinal &&
				         this.cmbTipoResponsable.ObjetoActivo != TipoResponsable.Exento &&
				         this.cmbTipoResponsable.ObjetoActivo != TipoResponsable.Monotributo)
					throw new DatosIngresadosInvalidosException(this.UnUnaNombreDocumento + " de tipo B solo " +
				          "puede tener asignado un cliente del tipo Consumidor Final, Exento o " +
					      "Monotributo.");
			}
		}
		
		/// <summary>
		/// Verifica los datos ingresados en SolapaFacturación y lanza una excepción del tipo
		/// DatosIngresadosInvalidosException en caso de haber errores.
		/// </summary>
		protected virtual void ValidarDatosIngresados()
		{
			// No valido los datos ingresados si una Factura está cargada (se supone que es válida).
			if (this.facturaCargada is FacturaDto && this.facturaCargada != null)
				return;
			
			// Valido el cliente según el tipo de factura
			this.ValidarCliente();
			
			// Chequeo que haya items agregados al treeview
			if (this.itemsProducto.TreeView.Vacio)
				throw new DatosIngresadosInvalidosException("No hay items ingresados. " + 
				          "Debe ingresar al menos un item.");
			
			// Chequeo que todos los items ingresado sean válidos
			if (!this.itemsProducto.TreeView.ItemsValidos())
				throw new DatosIngresadosInvalidosException("Uno o mas items son inválidos. " + 
				          "Verifique que todos los items ingresados tengan un producto cargado y la cantidad mayor a cero.");
				
			// Chequeo el estado de los RegExEntry
			if (!this.txtBonificacion.TextoValido)
				throw new DatosIngresadosInvalidosException(this.txtBonificacion.MensajeError);
			
			if (!this.rtxtCuit.TextoValido)
				throw new DatosIngresadosInvalidosException(this.rtxtCuit.MensajeError);
			
			if (!this.rtxtCodigoPostal.TextoValido)
				throw new DatosIngresadosInvalidosException(this.rtxtCodigoPostal.MensajeError);
			
			if (this.chkFacturaManual.Active && !this.rtxtNroFactura.TextoValido)
				throw new DatosIngresadosInvalidosException(this.rtxtNroFactura.MensajeError);
			
			if (this.cmbCondicionPago.ObjetoActivo != CondicionPago.Contado &&
			    	!this.txtVence.TextoValido)
				throw new DatosIngresadosInvalidosException(this.txtVence.MensajeError);
		}
		
		/// <summary>
		/// Chequea si el codigo de factura que es pasado como parametro
		/// es correcto. Si no lo es, muestra un mensaje de error.
		/// </summary>
		/// <returns>
		/// 'true' si el codigo de factura es correcto y 'false' si no lo es.
		/// </returns>
		/// <param name='codigo'>
		/// Codigo que se quiere chequear.
		/// </param>
		/// <param name='facturaManual'>
		/// Indica si la factura a la que corresponde el codigo es manual.
		/// </param>
		protected bool CodigoFacturaCorrecto(int codigo, bool facturaManual)
		{	
			foreach(CodigoFacturaDto cod in this.codigosDeFactura)
			{
				if (cod.codigo == codigo &&
				    cod.facturamanual == facturaManual)
					return true;
			}
			
			SMessageDialog.MensajeError("El código de la Factura " + codigo.ToString("D4") + 
				                           " que intenta visualizar no está configurado en el sistema.\n" +
				                           "Agregue el código en el archivo de configuración.");
			
			return false;
		}
		#endregion
		
		/// <summary>
		/// Indica si hay actualmente un cliente ingresado, tanto cargado en una búsqueda al
		/// servidor como uno nuevo con los datos mínimos correspondientes.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		private bool ClienteIngresado()
		{
			if (this.razonSocial.Cliente != null ||
			    	this.razonSocial.RazonSocialEntry.Text.Trim() != String.Empty)
				return true;
			
			return false;
		}
		
		/// <summary>
		/// Obtiene el cliente ingresado por el usuario. Lo que se hace es armar un objeto
		/// ClienteDto con los valores que el usuario colocó en el formulario.
		/// Si se quiere obtener un ClienteDto válido (que tenga por lo menos la razón social)
		/// se debe realizar la verificación antes de llamar a éste método.
		/// </summary>
		/// <returns>
		/// A <see cref="ClienteDto"/>
		/// </returns>
		private ClienteDto ObtenerClienteIngresado()
		{
			ClienteDto clienteIngresado = new ClienteDto();
			clienteIngresado.codigo = -1;
			clienteIngresado.domicilio = new DomicilioDto();
			
			// Cargo los datos desde el formulario
			clienteIngresado.razonSocial = this.razonSocial.RazonSocialEntry.Text;
			clienteIngresado.domicilio.direccion = this.txtDireccion.Text;
			clienteIngresado.domicilio.localidad = this.txtLocalidad.Text;
			clienteIngresado.domicilio.codigoPostal = this.rtxtCodigoPostal.Text;
			clienteIngresado.domicilio.provincia =
				this.txtProvincia.Text == String.Empty ?
					ProvinciaDto.NoEspecificada : this.txtProvincia.ProvinciaSeteada;
			clienteIngresado.tipoRes = this.cmbTipoResponsable.ObjetoActivo;
			clienteIngresado.cuit = this.rtxtCuit.Text;
			clienteIngresado.porcentajeBonificacion = StratosUtil.ParseDouble(this.txtBonificacion.Text, 2);
			
			return clienteIngresado;
		}

		#endregion

		protected virtual void OnOtrasAccionesClicked (object sender, System.EventArgs e)
		{
			this.menuOtrasAcciones.Popup();
		}
	}
}
