// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;
using StratosClient.Core.Gui;
using StratosClient.Widgets.Level2;

namespace StratosClient.Ventas.Facturacion
{
	public class SolapaNotaCredito : SolapaFacturacion
	{
		#region Campos
		private static SolapaNotaCredito instanciaActual;
		
		private DatosFactura datosFactura;
		#endregion
		
		#region Constructores
		public SolapaNotaCredito () : base()
		{
			instanciaActual = this;
			
			// Modifico la interfaz gráfica de SolapaFacturación para adaptarla a una Nota de Crédito
			
			this.cmbTipoFactura.Renderizador = delegate (FacturaTipoVenta facturaTipo) {
				if (facturaTipo == FacturaTipoVenta.FacturaA)
					return "Nota de crédito A";
				else
					return "Nota de crédito B";
			};
			
			this.ChkFacturaManual.Label = "Nota de crédito manual";
			
			this.LblDatosCliente.Markup = "<b>Datos de la factura asociada</b>";
			
			this.TbDatosCliente.Remove(this.HbDomicilio1);
			this.TbDatosCliente.Remove(this.HbDomicilio2);
			this.TbDatosCliente.Remove(this.HbBonificacion);
			this.TbDatosCliente.Resize(2, 2);
			
			this.TbDatosCliente.Remove(this.HbClienteRazonSocial);
			this.TbDatosCliente.Attach(this.HbClienteRazonSocial, 0, 1, 1, 2);
			
			// Agrego los datos de la factura
			
			this.datosFactura = new DatosFactura();
			this.datosFactura.FacturaQuitada += this.OnFacturaQuitada;
			this.datosFactura.FacturaCargada += this.OnFacturaCargada;
			this.datosFactura.BotonBuscarFacturaPresionado += this.CrearNotaCreditoAPartirDeFactura;
			this.TbDatosCliente.Attach(this.datosFactura, 0, 1, 0, 1);
			
			// Quito los botones para limpiar el cliente y cliente genérico.
			// Hago que el widget de razón social sea de sólo lectura
			this.RazonSocial.SoloLectura = true;
			this.HbClienteRazonSocial.Remove(this.TbtnConsumidorFinalGenerico);
			
			this.VbFacturacion.Remove(this.FraAtributosFactura);
			
			/* Me suscribo al evento Changed del ComboBox que indica el tipo de factura.
			 * Cuando esto sucede, le indico al objeto DatosFactura el nuevo tipo de NotaCredito,
			 * para que pueda realizar la búsqueda de la factura correctamente. */
			this.datosFactura.ComboBoxFacturaTipoVenta.Changed += delegate {
				this.cmbTipoFactura.ObjetoActivo = this.datosFactura.ComboBoxFacturaTipoVenta.ObjetoActivo;
			};
		}
		#endregion
		
		#region Propiedades
		public static new SolapaNotaCredito InstanciaActual {
			get { return instanciaActual; }
		}
		
		public override string LaElNombreDocumento {
			get { return "La nota de crédito"; }
		}
		
		public override string UnUnaNombreDocumento {
			get { return "Una nota de crédito"; }
		}
		
		public override string LaElNombrePreDocumento {
			get { return "La pre nota de crédito"; }
		}
		
		public override string UnUnaNombrePreDocumento {
			get { return "Una pre nota de crédito"; }
		}
		
		protected override bool EsSolapaNotaCredito {
			get { return true; }
		}
		#endregion
		
		#region Eventos
		protected void OnFacturaQuitada()
		{
			this.RazonSocial.Vaciar();
			this.RazonSocial.SoloLectura = true;
		}
		
		protected void OnFacturaCargada(FacturaDto facturaCargada)
		{
			// Cargo el cliente de la factura cargada
			this.RazonSocial.CargarCliente(facturaCargada.cliente);
			
			/* Si se trata de una factura asociada que es manual, activo el checkbox de
			 * nota de crédito manual y asigno el código correspondiente, a menos que 
			 * haya una prefactura ya cargada, con lo cual no piso los datos que ya fueron cargados en esos cam*/
			if (facturaCargada.facturaManual && base.facturaCargada == null) {
				this.ChkFacturaManual.Active = true;
				this.cmbCodigoFactura.ObjetoActivo = new CodigoFacturaDto(facturaCargada.codigo, "", false, false);
			}
			
			this.CalcularImportes();
		}
		
		/// <summary>
		/// Dada una FacturaDto, crea una PreNotaCredito asociada y la carga.
		/// </summary>
		/// <param name="facturaCargada">
		/// A <see cref="FacturaDto"/>
		/// </param>
		protected void CrearNotaCreditoAPartirDeFactura(FacturaDto facturaCargada)
		{
			// Si hay una PreNotaCredito cargada, entonces copio su Id y se lo asigno
			long idNotaCreditoCargada = 0;
			bool hayUnaNotaCreditoCargada = false;
			
			PreNotaCreditoDto preNotaCreditoCargada = this.facturaCargada as PreNotaCreditoDto;
			
			if (preNotaCreditoCargada != null) {
				idNotaCreditoCargada = preNotaCreditoCargada.id;
				hayUnaNotaCreditoCargada = true;
			}
			
			// Creo la nota de crédito (o modifico la existente)
			PreNotaCreditoDto preNotaCredito =
				FacturacionUtil.PreNotaCreditoDesdeFactura(preNotaCreditoCargada, facturaCargada);
			
			// Cargo la nota de crédito
			this.CargarFactura(preNotaCredito, true);
			
			// Si había una nota de crédito cargada antes de cargar la nueva, entonces
			// marco el documento actual como modificado
			if (hayUnaNotaCreditoCargada)
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Modificado);
		}
		#endregion
		
		#region Métodos
		protected override ClienteDto SugerirClientes ()
		{
			return this.RazonSocial.Cliente;
		}
		
		protected override FacturaGenericaDto CrearPreDocumentoNuevo ()
		{
			return new PreNotaCreditoDto();
		}
		
		protected override FacturaGenericaDto CrearDocumentoNuevo ()
		{
			return new NotaCreditoDto();
		}
		
		protected override void LeerDatosFacturaGenerica (ref FacturaGenericaDto facturaGenerica,
		                                                  ClienteDto cliente)
		{
			// Leo los datos de la nota de crédito
			base.LeerDatosFacturaGenerica (ref facturaGenerica, cliente);
			
			// Además de los datos comunes con una factura genérica, cargo la factura asociada
			PreNotaCreditoDto preNotaCreditoDto = facturaGenerica as PreNotaCreditoDto;
			NotaCreditoDto notaCreditoDto = facturaGenerica as NotaCreditoDto;
			
			// El importe de bonificación para la nota de crédito es igual al de la factura asociada
			facturaGenerica.porcentajeBonificacion = this.datosFactura.Factura.porcentajeBonificacion;
			
			if (preNotaCreditoDto != null)
				preNotaCreditoDto.facturaAsociada = this.datosFactura.Factura;
			else
				notaCreditoDto.facturaAsociada = this.datosFactura.Factura;
		}
		
		protected override void ValidarCliente ()
		{
			// No valido el cliente para una nota de crédito
			return;
		}
		
		/// <summary>
		/// Valida los datos específicos de una nota de crédito
		/// </summary>
		protected override void ValidarDatosIngresados ()
		{
			base.ValidarDatosIngresados();
			
			// No valido los datos ingresados si hay una NotaCredito cargada
			if (this.facturaCargada is NotaCreditoDto && this.facturaCargada != null)
				return;
			
			if (this.datosFactura != null && this.datosFactura.Factura == null)
				throw new DatosIngresadosInvalidosException("No se ha especificado una factura. " + 
				          "Una nota de crédito debe estar asociada a una factura.");
		}
		
		protected override decimal CalcularImporteBonificacion (decimal importeBruto)
		{
			/* Para una nota de crédito utilizo el porcentaje de bonificación de la
			 * factura asociada. */
			decimal porcentajeBonificacion = 0.00m;
			
			if (this.datosFactura != null && this.datosFactura.Factura != null)
				porcentajeBonificacion = (decimal)this.datosFactura.Factura.porcentajeBonificacion;
			
			return base.CalcularImporteBonificacion(importeBruto, porcentajeBonificacion);
		}
		
		protected override void VaciarFormulario ()
		{
			base.VaciarFormulario ();
			
			/* Como el constructor de SolapaFacturacion llama a VaciarFormulario y datosFactura 
			 * se crea en el constructor de SolapaNotaCredito, verifico que no sea nulo antes
			 * de vaciarlo. */
			if (this.datosFactura != null) {
				this.datosFactura.SoloLectura = false;
				this.datosFactura.Vaciar();
			}
			
			this.RazonSocial.SoloLectura = true;
			
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Nuevo, "PRE-NOTA CREDITO NUEVA");
		}
		
		#region Métodos de carga extra
		protected override void CargarFacturaExtra (FacturaDto factura)
		{
			NotaCreditoDto notaCredito = (NotaCreditoDto) factura;
			
			// Debido a los datos de migración, tengo que fijarme si la nota de crédito
			// tiene una factura asociada antes de cargarla.
			if (notaCredito.facturaAsociada != null)
				this.datosFactura.CargarFactura(notaCredito.facturaAsociada);
			
			this.datosFactura.SoloLectura = true;
			
			this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "NOTA CREDITO CARGADA");
		}
		
		protected override void CargarFacturaExtra (PreFacturaDto preFactura)
		{
			PreNotaCreditoDto preNotaCredito = (PreNotaCreditoDto) preFactura;
			
			this.datosFactura.CargarFactura(preNotaCredito.facturaAsociada);
			this.datosFactura.SoloLectura = false;
			
			// De acuerdo a si la nota de crédito es nueva o no escribo el estado de la misma
			if (preNotaCredito.id <= 0) {
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Modificado);
				this.NavegadorElementos.Sensitive = false;
			}
			else {
				this.administradorEstado.CambiarEstadoObjeto(EstadoObjeto.Cargado, "PRE-NOTA CREDITO CARGADA");
				this.NavegadorElementos.Sensitive = true;
			}
		}
		#endregion
		#endregion
	}
}
