// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using Gtk;

using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using Stratos.Common.Excepciones;
using StratosServer.Controladores;

namespace StratosClient.Ventas.Presupuestos
{
	public class AccionadorMenuPresupuesto : IAccionadorSobreElemento
	{
		#region Campos
		private PresupuestoDto[] presupuestos;
		private TreeIter[] treeIters;
		private TreeModel modelo;
		
		private Gtk.Action modificarAction;
		private Gtk.Action borrarAction;
		#endregion
		
		#region Constructores
		public AccionadorMenuPresupuesto()
		{
			this.modificarAction =
				new Gtk.Action("modificar",
				               "Modificar",
				               "Carga un Presupuesto para modificar",
				               "gtk-edit");
			
			this.modificarAction.Activated += this.OnModificarActivated;
			
			this.borrarAction =
				new Gtk.Action("borrar",
				               "Borrar",
				               "Borra un Presupuesto",
				               "gtk-remove");
			
			this.borrarAction.Activated += this.OnBorrarActivated;
		}
		#endregion
		
		#region IAccionadorMenu implementation
		public Gtk.Action[] Acciones {
			get {
				return new [] {this.modificarAction, this.borrarAction};
			}
		}
		
		public void ActualizarAcciones (TreeIter[] iters, DocumentoGenericoDto[] documentos, TreeModel modelo)
		{
			this.treeIters = iters;
			this.presupuestos = documentos.Cast<PresupuestoDto>().ToArray();
			this.modelo = modelo;
			
			this.modificarAction.Sensitive = this.presupuestos.Length == 1;
			this.borrarAction.Sensitive = this.presupuestos.Length > 0;
		}
		#endregion
		
		#region Acciones de los menúes
		private void OnModificarActivated(object sender, EventArgs e)
		{
			SolapaVentas.Instancia.EjecutarVisualizador(this.treeIters[0]);
		}
		
		private void OnBorrarActivated(object sender, EventArgs e)
		{
			// Primero pregunto al usuario si está seguro con la eliminación
			ResponseType respuesta =
				SMessageDialog.MensajePregunta("¿Está seguro que desea eliminar el Presupuesto?");
			
			if (respuesta != ResponseType.Yes)
				return;
			
			// Envio el pedido de eliminación al servidor
			try {
				((ControladorPresupuestosPrx) MainIce.GetControlador("ControladorPresupuestos"))
					.EliminarPresupuesto(this.presupuestos[0]);
				
				// Quito la PreFactura eliminada del TreeView de Listados
				ListStore modeloListado =
					(ListStore) SolapaVentas.Instancia.TreeViewListado.Model;
				
				modeloListado.Remove(ref this.treeIters[0]);
				
				SMessageDialog.MensajeInformacion("Presupuesto eliminado correctamente");
			}
			catch (EliminarPreFacturaException ex) {
				SMessageDialog.MensajeError(ex.mensaje);
			}
		}
		#endregion
	}
}
