// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Ventas;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Core.Gui;

namespace StratosClient.Ventas.Presupuestos
{
	/// <summary>
	/// Se encarga de realizar la consulta de Presupuestos en el listado de documentos.
	/// </summary>
	public class ConsultaPresupuestos : IConsulta
	{
		public Type TipoDevuelto {
			get { return typeof(PresupuestoDto); }
		}
		
		public DocumentoGenericoDto[] RealizarConsulta(Dictionary<string,object> criteriosBusqueda,
		                                               ConsultaPaginable consultaPaginable)
		{
			// Pido el controlador y realizo la consulta según los criterios de búsqueda
			ControladorPresupuestosPrx controlador =
				(ControladorPresupuestosPrx) MainIce.GetControlador("ControladorPresupuestos");
			
			// Genero los criterios generales. Para los presupuestos necesito especificar
			// cuál es la propiedad de la razón social.
			List<string> criterios =
				VentasUtils.GenerarCriteriosHQL(criteriosBusqueda,
				                                "RazonSocial");
			
			// Luego genero los criterios HQL específicos de Factura (código y número)
			if (criteriosBusqueda.ContainsKey("NumeroPresupuesto"))
				criterios.Add("d.Id = " + criteriosBusqueda["NumeroPresupuesto"]);
			
			Ice.Object[] resultados = controlador
				.BuscarHQL(@"from Presupuesto d
								left join fetch d.Cliente " +
				           StratosClientUtils.GenerarClausulaWhere(criterios) +
				           " order by d.Fecha desc, d.Id desc",
				           consultaPaginable);
			
			return resultados.Cast<DocumentoGenericoDto>().ToArray();
		}
	}
}
