// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using Stratos.Common;
using StratosServer.Controladores;
using StratosClient.Widgets.Level2;
using StratosClient.Widgets.Level3;
using StratosClient.Widgets.Level1;

namespace StratosClient.Ventas.Recibos
{
	/// <summary>
	/// Implementación concreta de la vista para dar de alta recibos.
	/// IReciboVista
	/// </summary>
	public partial class ReciboVista : Gtk.Bin, IReciboVista
	{
		#region Campos
		private ItemsInteractivo itemsRecibo;
		
		/// <summary>
		/// El valor de esta referencia es el mismo que this.itemsRecibo.TreeView, pero
		/// está casteado para evitar tantas conversiones de tipo.
		/// </summary>
		private ListaItemsRecibo tvItemsRecibo;
		
		private ReciboPresenter reciboPresenter;
		#endregion
		
		#region Constructores
		public ReciboVista ()
		{
			this.Build ();
			
			// Agrego la lista de items
			this.tvItemsRecibo = new ListaItemsRecibo();
			this.itemsRecibo = new ItemsInteractivo(this.tvItemsRecibo);
			this.hbItems.PackStart(this.itemsRecibo, true, true, 0);
			
			/* Me suscribo a los eventos de conexión y desconexión de la terminal.
			 * Si hay una conexión nueva, tengo que actualizar los controladores (proxies) pasados
			 * al Presenter. */
			MainIce.ConexionEstablecida += this.CargarControladores;
			
			// Creo el Presenter
			this.reciboPresenter = new ReciboPresenter(this, null, null);
			
			#region Notificación de cambios en la interfaz gráfica
			this.dcFecha.FechaCambiada += delegate { this.reciboPresenter.DatosModificados(); };
			
			this.razonSocial.CodigoPersonaEntry.Changed += delegate { this.reciboPresenter.DatosModificados(); };
			this.razonSocial.RazonSocialEntry.Changed += delegate { this.reciboPresenter.DatosModificados(); };
			
			this.txtConcepto.Changed += delegate { this.reciboPresenter.DatosModificados(); };
			
			this.itemsRecibo.TreeView.ItemsCambiados += delegate { this.reciboPresenter.DatosModificados(); };
			#endregion
			
			// Cargo los controladores en el Presenter
			this.CargarControladores();
		}
		#endregion
		
		#region Otros métodos
		public void CargarControladores()
		{
			// Obtengo los controladores necesarios para inyectarlos en el Presenter
			ControladorRecibosPrx controladorRecibos = (ControladorRecibosPrx)
				MainIce.GetControlador("ControladorRecibos");
			ControladorClientesPrx controladorClientes = (ControladorClientesPrx)
				MainIce.GetControlador("ControladorClientes");
			
			this.reciboPresenter.ControladorClientes = controladorClientes;
			this.reciboPresenter.ControladorRecibos = controladorRecibos;
		}
		#endregion
		
		#region IVisualizadorNavegable implementation
		public INavegadorElementosVista NavegadorElementos {
			get { return this.navegadorElementos; }
		}
		#endregion
		
		#region IReciboVista implementation
		#region Valores
		public string Estado {
			set { this.lblEstado.Markup = value; }
		}
		
		public string Concepto {
			get { return this.txtConcepto.Text; }
			set { this.txtConcepto.Text = value; }
		}
		
		public double Importe {
			get { return this.txtImporte.ImporteDouble; }
			set { this.txtImporte.ImporteDouble = value; }
		}
		
		public bool EsPosibleModificarConcepto {
			set { CustomWidgetsUtils.WidgetEditable(this.txtConcepto, value); }
		}
		
		public bool EsPosibleModificarFecha {
			set { this.vbCabecera.Sensitive = value; }
		}
		
		public bool EsPosibleModificarItemsRecibo {
			set { this.itemsRecibo.Sensitive = value; }
		}
		
		public bool EsPosibleGuardarRecibo {
			set { this.btnGuardar.Sensitive = value; }
		}
		
		public bool EsPosibleImprimirRecibo {
			set { this.btnImprimir.Sensitive = value; }
		}
		#endregion
		
		#region Componentes gráficos
		public IDateComboVista DateComboVista {
			get { return this.dcFecha; }
		}
		
		public IRazonSocialVista<ClienteDto> RazonSocialVista {
			get { return this.razonSocial; }
		}
		
		public IListaItemsReciboVista ListaItemsReciboVista {
			get { return this.tvItemsRecibo; }
		}
		#endregion
		
		#region Métodos
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		
		public void DarElFoco_CodigoRazonSocial ()
		{
			this.razonSocial.CodigoPersonaEntry.GrabFocus();
		}
		#endregion
		#endregion
		
		#region Eventos
		protected virtual void OnNuevoClicked (object sender, System.EventArgs e)
		{
			this.reciboPresenter.NuevoRecibo();
		}
		
		protected virtual void OnImprimirClicked (object sender, System.EventArgs e)
		{
			this.reciboPresenter.ImprimirRecibo();
		}
		
		protected virtual void OnGuardarClicked(object sender, System.EventArgs e)
		{
			this.reciboPresenter.GuardarRecibo();
		}
		#endregion
	}
}
