// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using CustomWidgets;
using StratosClient.Core.Gui;

namespace StratosClient.Ventas.Recibos
{
	public partial class ImputacionesVista : Gtk.Bin, IImputacionesVista
	{
		#region Campos
		protected TvDocumentosCredito tvDocumentosCredito;
		protected TvDocumentosDebito tvDocumentosDebito;
		
		protected ImputacionesPresenter imputacionesPresenter;
		#endregion
		
		#region Constructores
		public ImputacionesVista ()
		{
			this.Build ();
			
			// Documentos de crédito
			this.tvDocumentosCredito = new TvDocumentosCredito();
			this.swRecibosExistentes.Add(this.tvDocumentosCredito);
			this.tvDocumentosCredito.ShowAll();
			
			// Documentos de débito
			this.tvDocumentosDebito = new TvDocumentosDebito();
			this.tvDocumentosDebito
				.MensajeSeleccionNoPermitida =
						"No se puede seleccionar la factura para imputar al recibo. " +
						"El importe del recibo no alcanza a cubrirla.";
			this.swFacturasAsociadas.Add(this.tvDocumentosDebito);
			this.tvDocumentosDebito.ShowAll();
			
			/* Me suscribo a los eventos de conexión y desconexión de la terminal.
			 * Si hay una conexión nueva, tengo que actualizar los controladores (proxies) pasados
			 * al Presenter. */
			MainIce.ConexionEstablecida += this.CargarControladores;
			
			this.imputacionesPresenter = new ImputacionesPresenter(this, null);
			
			// Cargo los controladores en el Presenter
			this.CargarControladores();
		}
		#endregion
		
		#region IImputacionesVista - Implementación
		#region Notificaciones
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		#endregion
		
		public bool EsPosibleActualizarRecibosYFacturas {
			set { this.btnActualizarRecibosFacturas.Sensitive = value; }
		}
		
		public IRazonSocialVista<ClienteDto> RazonSocialVista {
			get { return this.razonSocial; }
		}
		
		public IDocumentoCreditoVista DocumentosCredito {
			get { return this.tvDocumentosCredito; }
		}
		
		public IDocumentoDebitoVista DocumentosDebito {
			get { return this.tvDocumentosDebito; }
		}
		#endregion
		
		#region Eventos
		protected virtual void OnActualizarClicked (object sender, System.EventArgs e)
		{
			this.imputacionesPresenter.ActualizarDocumentos();
		}
		
		protected virtual void OnImputarClicked (object sender, System.EventArgs e)
		{
			this.imputacionesPresenter.Imputar();
		}
		#endregion
		
		#region Otros métodos
		public void CargarControladores()
		{
			// Obtengo los controladores necesarios para inyectarlos en el Presenter
			ControladorClientesPrx controladorClientes = (ControladorClientesPrx)
				MainIce.GetControlador("ControladorClientes");
			
			this.imputacionesPresenter.ControladorClientes = controladorClientes;
		}
		#endregion
	}
}
