// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

namespace StratosClient.Ventas
{
	/// <summary>
	/// Todos los addins que quieran registrar una interfaz gráfica con criterio personalizados
	/// tienen que hacer que la misma implemente esta interfaz.
	/// </summary>
	public interface ICriteriosPersonalizados
	{
		/// <summary>
		/// Se dispara para validar los datos de búsqueda. El addin debe devolver true o false
		/// de acuerdo a la validez de los mismos, y el mensaje de error que se quiera mostrar
		/// al usuario.
		/// </summary>
		/// <param name="mensajeError">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		bool DatosValidos(out string mensajeError);
		
		//// <value>
		/// Esta propiedad debe devolver un diccionario con los criterios de búsqueda que
		/// tenga la interfaz de criterios personalizados. Por ejemplo: "CodigoFactura", "0003".
		/// Luego se utilizan para generar las consultas al servidor.
		/// </value>
		Dictionary<string, object> CriteriosPersonalizados {
			get;
		}
	}
}
