// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Widgets.Level3;

namespace StratosClient.Ventas
{
	/// <summary>
	/// Interfaz que deben implementar todos los métodos que permitan cargar documentos desde
	/// el listado de DocumentoGenericoDto.
	/// </summary>
	public abstract class CargadorElemento
	{
		/// <summary>
		/// Propiedad que devuelve el ControladorBasePrx para poder refrescar el objeto antes
		/// de cargarlo.
		/// </summary>
		public abstract ControladorBasePrx ControladorRemoto {
			get;
		}
		
		public abstract IVisualizadorNavegable VisualizadorNavegable {
			get;
		}
		
		/// <summary>
		/// Este método es ejecutado antes de realizar la carga, para preguntar si realmente se
		/// debe realizar. Por ejemplo, si una FacturaDto es inválida, entonces no se visualiza.
		/// </summary>
		/// <param name="documento">
		/// A <see cref="DocumentoGenericoDto"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		public abstract bool PermitirCarga(DocumentoGenericoDto documento);
		
		public abstract void Cargar(DocumentoGenericoDto documento);
		
		/// <summary>
		/// Realiza la carga del documento, pero además permite especificar si
		/// se debe activar o no el navegador de elementos.
		/// </summary>
		/// <param name="documento">
		/// A <see cref="DocumentoGenericoDto"/>
		/// </param>
		/// <param name="activarNavegadorElementos">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public void Cargar(DocumentoGenericoDto documento, bool activarNavegadorElementos)
		{
			this.Cargar(documento);
			
			if (activarNavegadorElementos)
				this.VisualizadorNavegable.NavegadorElementos.Sensitive = true;
		}
	}
}
