// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;

namespace StratosClient.Ventas
{
	/// <summary>
	/// Especifica la interface de un objeto que devuelve acciones para ser
	/// ejecutadas en ciertos tipos de documentos.
	/// </summary>
	public interface IAccionadorSobreElemento
	{
		/// <summary>
		/// Las acciones que el usuario puede ejecutar sobre el tipo de documento.
		/// </summary>
		Gtk.Action[] Acciones {
			get;
		}
		
		/// <summary>
		/// Se encarga de actualizar los Gtk.Action (por ej, seteando su propiedad Sensitive)
		/// cuando el usuario realiza alguna acción como seleccionar o deseleccionar algún
		/// documento.
		/// </summary>
		/// <param name="iters">
		/// A <see cref="Gtk.TreeIter[]"/>
		/// </param>
		/// <param name="documento">
		/// A <see cref="DocumentoGenericoDto[]"/>
		/// </param>
		/// <param name="modelo">
		/// A <see cref="Gtk.TreeModel"/>
		/// </param>
		void ActualizarAcciones(Gtk.TreeIter[] iters, DocumentoGenericoDto[] documento, Gtk.TreeModel modelo);
	}
}
