// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using StratosServer.Controladores;
using Stratos.ModeloDto;

namespace StratosClient.Ventas
{
	/// <summary>
	/// Esta clase debe ser implementada por los addins que quieran definir consultas a
	/// realizar en el listado de documentos de STratosClient.Ventas.
	/// </summary>
	public interface IConsulta
	{
		/// <summary>
		/// Este método realiza la consulta propiamente dicha. Recibe un diccionario de criterios
		/// de búsqueda establecidos por el usuario (la clave es el nombre del criterio, por
		/// ejemplo "RazonSocial", y el valor es lo que estableció el usuario).
		/// Además recibe también una ConsultaPaginable que debe respetar.
		/// Devuelve un array con los objetos resultado
		/// </summary>
		/// <param name="criteriosBusqueda">
		/// A <see cref="Dictionary"/>
		/// </param>
		/// <param name="consultaPaginable">
		/// A <see cref="ConsultaPaginable"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Object"/>
		/// </returns>
		DocumentoGenericoDto[] RealizarConsulta(Dictionary<string,object> criteriosBusqueda,
		                                        ConsultaPaginable consultaPaginable);
		
		/// <summary>
		/// Esta propiedad debe devolver el tipo de documento que trae
		/// la consulta. Por ej: FacturaDto.
		/// </summary>
		System.Type TipoDevuelto {
			get;
		}
	}
}
