// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using StratosClient.Core.Gui;

namespace StratosClient.Ventas
{
	/// <summary>
	/// Método estáticos de utilidad y uso común para los addins de ventas.
	/// </summary>
	public static class VentasUtils
	{
		/// <summary>
		/// Dado unos criterios de búsqueda especificados por el usuario, este método se encarga
		/// de generar las condiciones where para una consulta HQL. Soporta la Razón social,
		/// fecha mínima y fecha máxima. Esta sobrecarga permite especificar cuál es el nombre
		/// de la propiedad para buscar por razón social.
		/// </summary>
		/// <param name="criteriosBusqueda">
		/// A <see cref="Dictionary<System.String, System.Object>"/>
		/// </param>
		/// <param name="propiedadRazonSocial">
		/// A <see cref="System.String"/>
		/// </param>
		/// <param name="criteriosPorIgnorar">
		/// A <see cref="System.String[]"/>
		/// </param>
		/// <returns>
		/// A <see cref="List<System.String>"/>
		/// </returns>
		public static List<string> GenerarCriteriosHQL(Dictionary<string,object> criteriosBusqueda,
		                                               string propiedadRazonSocial)
		{
			List<string> criterios = new List<string>();
			
			// Agrego los criterios de búsqueda sólo si estos fuerons especificados
			
			// Criterio de razón social
			if (criteriosBusqueda.ContainsKey("RazonSocial")) {
				criterios
					.AddRange(StratosClientUtils
					          	.GenerarCriteriosCadenaEnHQL("d." + propiedadRazonSocial,
					                                       criteriosBusqueda["RazonSocial"].ToString()));
			}
			
			// Criterio de fecha mínima
			if (criteriosBusqueda.ContainsKey("FechaMinima")) {
				DateTime fechaMinima = (DateTime) criteriosBusqueda["FechaMinima"];
				
				criterios.Add("d.Fecha >= '" +
				              fechaMinima.ToShortDateString() + "'");
			}
			
			// Criterio de fecha máxima
			if (criteriosBusqueda.ContainsKey("FechaMaxima")) {
				DateTime fechaMaxima = (DateTime) criteriosBusqueda["FechaMaxima"];
				
				criterios.Add("d.Fecha <= '" +
				              fechaMaxima.ToShortDateString() + "'");
			}
			
			return criterios;
		}
		
		/// <summary>
		/// Dado unos criterios de búsqueda especificados por el usuario, este método se encarga
		/// de generar las condiciones where para una consulta HQL. Soporta la Razón social,
		/// fecha mínima y fecha máxima.
		/// </summary>
		/// <param name="criteriosBusqueda">
		/// A <see cref="Dictionary"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public static List<string> GenerarCriteriosHQL(Dictionary<string,object> criteriosBusqueda)
		{
			return GenerarCriteriosHQL(criteriosBusqueda,
			                           "Cliente.RazonSocial");
		}
	}
}
