// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

namespace CustomWidgets
{
	/// <summary>
	/// Clase estática con métodos de utilidad para widgets.
	/// </summary>
	public static class CustomWidgetsUtils
	{
		#region WidgetEditable
		/// <summary>
		/// Permite habilitar o deshabilitar un Gtk.Entry para su edición.
		/// </summary>
		/// <param name="entry">
		/// A <see cref="Gtk.Entry"/>
		/// </param>
		/// <param name="editable">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public static void WidgetEditable(Gtk.Entry entry, bool editable)
		{
			// Activo/desactivo la edición del entry según corresponda
			entry.IsEditable = editable;
			
			// Cambio el color de fondo del entry según corresponda
			Gdk.Color color = Gdk.Color.Zero;
			
			if (!editable) {
				Gdk.Color.Parse("lightgray", ref color);
				entry.ModifyBase(StateType.Normal, color);
			}
			else
				entry.ModifyBase(StateType.Normal);
		}
		
		/// <summary>
		/// Permite habilitar o deshabilitar un Gtk.TreeView para su edición.
		/// </summary>
		/// <param name="textView">
		/// A <see cref="Gtk.TextView"/>
		/// </param>
		/// <param name="editable">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public static void WidgetEditable(Gtk.TextView textView, bool editable)
		{
			// Activo/desactivo la edición del entry según corresponda
			textView.Editable = editable;
			
			// Cambio el color de fondo del entry según corresponda
			Gdk.Color color = Gdk.Color.Zero;
			
			if (!editable) {
				Gdk.Color.Parse("lightgray", ref color);
				textView.ModifyBase(StateType.Normal, color);
			}
			else
				textView.ModifyBase(StateType.Normal);
		}
		
		/// <summary>
		/// Permite habilitar o deshabilitar un Gtk.ComboBox para su edición.
		/// </summary>
		/// <param name="combo">
		/// A <see cref="Gtk.ComboBox"/>
		/// </param>
		/// <param name="editable">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public static void WidgetEditable(Gtk.ComboBox combo, bool editable)
		{
			// Activo/desactivo el combo según corresponda
			combo.Sensitive = editable;
		}
		
		/// <summary>
		/// Permite habilitar o deshabilitar un Gtk.ToggleButton para su edición.
		/// </summary>
		/// <param name="button">
		/// A <see cref="Gtk.ToggleButton"/>
		/// </param>
		/// <param name="editable">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public static void WidgetEditable(Gtk.ToggleButton button, bool editable)
		{
			// Activo/desactivo el botón según corresponda
			button.Sensitive = editable;
		}
		#endregion
	}
}
