// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Globalization;

namespace CustomWidgets
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class DateCombo : Gtk.Bin, IDateComboVista
	{
		#region Campos
		public event System.Action<DateTime> FechaCambiada;
		
		private bool actualizarEntry;
		
		private DateTime currentDate;
		
		Gdk.Color colorError;
		#endregion
		
		#region Constructores
		public DateCombo()
		{
			this.Build();
			
			this.txtTexto.Xalign = 0.50f;
			Gdk.Color.Parse("#f43c3c", ref colorError);
			
			this.actualizarEntry = true;
			
			/* Cada vez que el Entry pierde el foco, asigno el valor de la fecha en el mismo.
			 * No lo hago antes porque sino molesta al modificar la fecha directamente en el
			 * Entry. */
			this.txtTexto.FocusOutEvent += delegate {
				this.actualizarEntry = true;
				this.ActualizarEntryFecha();
			};
			
			this.txtTexto.FocusInEvent += delegate {
				this.actualizarEntry = false;
			};
		}
		#endregion
		
		#region Propiedades
		public DateTime Fecha {
			get {
				DateTime fecha;
				
				if (this.FechaValida(out fecha))
					return fecha;
				
				throw new Exception("La fecha de la factura es inválida");
			}
			
			set {
				DateTime valorAnterior = this.currentDate;
				this.currentDate = value;
				
				if (this.actualizarEntry)
					this.ActualizarEntryFecha();
				
				// Sólo disparo el evento si han cambiado los valores
				if (!this.currentDate.Equals(valorAnterior))
					this.OnFechaCambiada(this.currentDate);
			}
		}
		
		public bool Editable {
			set { this.Sensitive = value; }
		}
		#endregion

		#region Eventos
		protected void OnFechaCambiada(DateTime fecha)
		{
			if (this.FechaCambiada != null)
				this.FechaCambiada(fecha);
		}
		
		protected virtual void OnClicked (object sender, System.EventArgs e)
		{
			int x,y;
			this.ParentWindow.GetOrigin(out x, out y);
			x += this.Allocation.Left;
			y += this.Allocation.Top + this.Allocation.Height;
			
			DateTime fecha = DateTime.Today;
			if (this.FechaValida())
				fecha = this.Fecha;
			
			DateDialog.ShowMe(x, y, fecha, this.OnPopupDateChanged);
		}
		
		private void OnPopupDateChanged(object sender, DateEventArgs args)
		{
			this.Fecha = args.Date;
		}
		
		protected virtual void OnTextoChanged (object sender, System.EventArgs e)
		{
			if (this.FechaValida()) {
				// Obtengo la nueva fecha
				DateTime fechaValida;
				this.FechaValida(out fechaValida);
				
				// Cambio el color de fondo
				this.txtTexto.ModifyBase(Gtk.StateType.Normal);
				
				// Asigno la nueva fecha
				this.Fecha = fechaValida;
			}
			else
				this.txtTexto.ModifyBase(Gtk.StateType.Normal, this.colorError);
			
			//if( DateChanged != null ) DateChanged ( this, e );
		}
		#endregion
		
		#region Métodos
		protected void ActualizarEntryFecha()
		{
			this.txtTexto.Text = this.currentDate.ToShortDateString();
		}
		
		public bool FechaValida() {
			DateTime temp;
			return this.FechaValida(out temp);
		}
		
		public bool FechaValida(out DateTime fecha) {
			bool fechaCorrecta = true;
			
			try {
				IFormatProvider culture = CultureInfo.CurrentCulture;
				fecha = DateTime.Parse(this.txtTexto.Text, culture);
			}
			catch {
				fechaCorrecta = false;
				fecha = DateTime.MinValue;
			}
			
			return fechaCorrecta;
		}
		#endregion
	}
}
