// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

namespace CustomWidgets
{
	
	public class DateEventArgs : EventArgs
	{
		private DateTime date;
		
		public DateEventArgs( DateTime date )
		{
			this.date = date;
		}
		
		public DateTime Date 
		{
			get { return date; }
		}
	}
	
	public partial class DateDialog : Gtk.Window
	{
		
		public delegate void DateEventHandler(object sender, DateEventArgs args);
		public event DateEventHandler OnChange = null;
		
		public DateDialog(int x, int y, DateTime currentDate, DateEventHandler handler)
			: base(Gtk.WindowType.Popup)
		{
			this.Move(x, y);
			this.Build();
			this.OnChange = handler;
			
//			if (currentDate.Equals(DateTime.MinValue))
//				this.calCalendario.Date = DateTime.Today;
//			else
			this.calCalendario.Date = currentDate;
			
			GrabUtil.GrabWindow(this);
		}
		
		public DateTime Fecha
		{
			get
			{
				return this.calCalendario.Date;
//				DateTime d = CCalendar.Date;
//				return new DateTime( d.Year, d.Month, d.Day, (int)TxtHour.Value, (int)TxtMin.Value, (int)TxtSec.Value );
			}
		}
		
		public static void ShowMe(int x, int y, DateTime currenDate, DateEventHandler handler)
		{
			new DateDialog(x, y, currenDate, handler);
		}
		
		protected override bool OnExposeEvent (Gdk.EventExpose args)
		{
			base.OnExposeEvent (args);
			
			int winWidth, winHeight;
			this.GetSize ( out winWidth, out winHeight );
			this.GdkWindow.DrawRectangle ( this.Style.ForegroundGC ( Gtk.StateType.Insensitive ), false, 0, 0, winWidth - 1, winHeight - 1 );
			
			return false;
		}

		protected virtual void OnButtonPressEvent (object o, Gtk.ButtonPressEventArgs args)
		{
			Close();
		}
		
		private void Close()
		{
			GrabUtil.RemoveGrab( this );
			this.Destroy();
		}

		protected virtual void OnHoyButtonClicked (object sender, System.EventArgs e)
		{
			this.calCalendario.Date = DateTime.Today;
		}

		protected virtual void OnCalendarDaySelected (object sender, System.EventArgs e)
		{
			if (this.OnChange != null)
				OnChange(this, new DateEventArgs(this.Fecha));
		}

		protected virtual void OnCalendarDaySelectedDoubleClick (object sender, System.EventArgs e)
		{
			if (this.OnChange != null)
				OnChange(this, new DateEventArgs(this.Fecha));
			
			this.Close();
		}

		protected virtual void OnCalendarButtonPressEvent (object o, Gtk.ButtonPressEventArgs args)
		{
			args.RetVal = true;
		}
	}
}
