// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.Common;

namespace CustomWidgets
{
	[System.ComponentModel.ToolboxItem(true)]
	public class ImporteRegExEntry : RegExEntry, IImporteRegExEntry
	{
		#region Campos
		/// <summary>
		/// El nombre del importe se utiliza para mostrar más semántica en los
		/// mensajes al usuario.
		/// </summary>
		private string nombreImporte;
		#endregion
		
		#region Constructores
		public ImporteRegExEntry() : base()
		{
			this.Importe = 0.00m;
			
			this.Xalign = 1;
			this.WidthChars = 12;
			
			/* No hace falta especificar una expresión regular para validar
			 * el Entry, ya que esta clase sobreescribte la propiedad TextoValido. */
//			this.ExpresionRegular = @"^($[ ]*)?[0-9]+[.,]?[0-9]{0,}$";
			this.MensajeError = "El formato del importe es incorrecto";
		}
		#endregion
		
		#region Propiedades
		public decimal Importe {
			get {
				if (!this.TextoValido)
					throw new Exception("'" + this.Text + "' no es un importe válido");
				
				return StratosUtil.ParseDecimal(this.Text, 2);
			}
			
			set {
				decimal importe = StratosUtil.RoundDecimal(value, 2);
				this.Text = StratosUtil.DecimalToString(importe);
			}
		}
		
		public double ImporteDouble {
			get { return (double)this.Importe; }
			set { this.Importe = (decimal) value; }
		}
		
		public string NombreImporte {
			get { return this.nombreImporte; }
			set {
				if (String.IsNullOrEmpty(value))
					return;
				
				this.nombreImporte = value;
				
				this.MensajeError =
					String.Format("El importe '{0}' tiene un valor incorrecto",
					              this.nombreImporte);
			}
		}
		
		public override bool TextoValido {
			get { return Stratos.Common.StratosUtil.IsDecimal(this.Text); }
		}
		#endregion
		
		#region Métodos
		public void Resetear()
		{
			this.Importe = 0.00m;
		}
		
		protected override bool OnFocusOutEvent (Gdk.EventFocus evnt)
		{
			// Al perder el foco, vuelvo a colocar el valor formateado del
			// importe, ya que el usuario puede poner coma o punto como separador
			// de decimales. Por defecto, siempre _muestro_ en el formato con
			// punto como separador de decimales
			if (this.TextoValido)
				this.Text = StratosUtil.DecimalToString(this.Importe);
			
			return base.OnFocusOutEvent (evnt);
		}
		#endregion
	}
}
