// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.Common;

namespace CustomWidgets
{
	/// <summary>
	/// Clase para el ingreso de valores porcentuales, como bonificaciones.
	/// </summary>
	[System.ComponentModel.ToolboxItem(true)]
	public class PorcentajeEntry : RegExEntry
	{
		private string mensajeErrorBreve;
		
		public PorcentajeEntry () : base()
		{
			this.Xalign = 1.0f;
		}
		
		public override bool TextoValido {
			get {
				// Un porcentaje es válido si el entry está vacío o si
				// es un Decimal lo ingresado.
				return this.Text.Trim() == String.Empty ||
					Stratos.Common.StratosUtil.IsDecimal(this.Text);
			}
		}
		
		public override string MensajeError {
			get { return this.MensajeErrorBreve + ". Solo puede contener números."; }
		}
		
		/// <summary>
		/// El mensaje de error breve simplemente indica qué es inválido.
		/// No da razones de por qué es inválido. Eso se agrega automáticamente.
		/// </summary>
		public string MensajeErrorBreve {
			get { return this.mensajeErrorBreve; }
			set { this.mensajeErrorBreve = value; }
		}
		
		public decimal Porcentaje {
			get {
				if (!this.TextoValido)
					throw new Exception("'" + this.Text + "' no es un porcentaje válido");
				
				return StratosUtil.ParseDecimal(this.Text, 2);
			}
			
			set { this.Text = StratosUtil.DecimalToString(value); }
		}
		
		public double PorcentajeDouble {
			get { return (double) this.Porcentaje; }
		}
	}
}
