// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

namespace CustomWidgets
{
	public class ServicioNavegadorElementos
	{
		#region Campos
		protected IListaNavegable listaNavegable;
		protected TreePath pathActual;
		protected TreePath pathProvisorio;
		protected int paginaIterActual;
		#endregion
		
		public ServicioNavegadorElementos ()
		{
		}
		
		#region Propiedades
		public IListaNavegable ListaNavegable {
			set {
				this.listaNavegable = value;
				this.pathActual = this.listaNavegable.ListaElementos.Model
					.GetPath(this.listaNavegable.IterCargado);
				this.paginaIterActual = this.listaNavegable.PaginaIterCargado;
			}
		}
		
		public object ElementoActual {
			get {
				TreeIter iterActual;
				
				this.listaNavegable.ListaElementos.Model.GetIter(out iterActual, this.pathProvisorio);
				
				return this.ObtenerObjetoYActualizarPathProvisorio(iterActual);
			}
		}
		
		public TreePath PathActual {
			set {
				this.pathActual = value;
				this.pathProvisorio = this.pathActual.Copy();
			}
		}
		#endregion
		
		#region Métodos públicos
		public object ElementoSiguiente()
		{
			TreeIter iterSiguiente;
			
			// Obtengo el iter siguiente al actual
			if (!this.IterSiguiente(out iterSiguiente))
				return null;
			
			return this.ObtenerObjetoYActualizarPathProvisorio(iterSiguiente);
		}
		
		public object ElementoAnterior()
		{
			TreeIter iterAnterior;
			
			// Obtengo el iter anterior al actual
			if (!this.IterAnterior(out iterAnterior))
				return null;
			
			return this.ObtenerObjetoYActualizarPathProvisorio(iterAnterior);
		}
		
		public void ElementoCargadoCorrectamente()
		{
			// Actualizo el iter actual y la pagina
			this.pathActual = this.pathProvisorio.Copy();
			this.paginaIterActual = this.listaNavegable.NavegadorDePaginas.PaginaActual;
			
			// Actualizo el iter y la seleccion de la lista de elementos
			TreeIter iterTemp = new TreeIter();
			this.listaNavegable.ListaElementos.Model.GetIter(out iterTemp, this.pathActual);
			this.listaNavegable.IterCargado = iterTemp;
			this.listaNavegable.PaginaIterCargado = this.paginaIterActual;
			
			// Quito las selecciones anteriores y selecciono el iter actual nomas
			this.listaNavegable.ListaElementos.Selection.UnselectAll();
			this.listaNavegable.ListaElementos.Selection.SelectIter(iterTemp);
		}
		
		public void ElementoNoCargado()
		{
			if (this.paginaIterActual == this.listaNavegable.NavegadorDePaginas.PaginaActual) {
				this.pathProvisorio = this.pathActual.Copy();
				return;
			}
			
			// Si para consultar el próximo elemento, se cargó la página anterior a la del
			// iterActual y entonces hago que se cargue la siguiente
			else if (this.paginaIterActual > this.listaNavegable.NavegadorDePaginas.PaginaActual)
				this.listaNavegable.NavegadorDePaginas.CargarPaginaSiguiente();
			
			// Si para consultar el proximo elemento, se cargó la página posterio a la del
			// iterActual y entonces hago que se cargue la siguiente
			else
				this.listaNavegable.NavegadorDePaginas.CargarPaginaAnterior();
				
			this.pathProvisorio = this.pathActual.Copy();
		}
		#endregion
		
		#region Métodos privados
		protected object ObtenerObjetoYActualizarPathProvisorio(TreeIter iter)
		{
			// Si el objeto asociado al iter actual en la lista es null, retorno null
			// Esto hay que chequearlo por un error en el navegador de paginas, que activa
			// la siguiente página cuando la anterior está llena
			if (this.listaNavegable.ListaElementos.Model.GetValue(iter, 0) == null)
				return null;
			
			// Actualizo el path provisorio
			this.pathProvisorio = this.listaNavegable.ListaElementos.Model.GetPath(iter);			
			
			// Retorno el documento correspondiente refrescado (o sea, completito)
			return this.listaNavegable.ObtenerObjetoRefrescado(iter);
		}
		
		protected bool IterAnterior(out TreeIter iterAnterior)
		{
			iterAnterior = new TreeIter();
			
			if (this.pathActual != null)
				this.pathProvisorio = this.pathActual.Copy();
				
			this.listaNavegable.ListaElementos.Model.GetIter(out iterAnterior, this.pathProvisorio);
			
			// Obtengo el path asociado al iter
			TreePath path = this.listaNavegable.ListaElementos.Model.GetPath(iterAnterior);
			
			// Entra en este if cuando el navegador activa la ultima página, pero esta
			// se encuentra vacía. El código que está adentro sirve para salvar esta
			// falla ocacional. Es mas facil incluir este código que arreglar el navegador de paginas
			if ((path == null) && (this.listaNavegable.NavegadorDePaginas.UltimaPagina)) {
				// Voy a la pagina anterior
				this.listaNavegable.NavegadorDePaginas.CargarPaginaAnterior();
				
				// Obtengo el iter del primer elemento
				TreeIter iterTemp;
				this.listaNavegable.ListaElementos.Model.GetIterFirst(out iterTemp);
				
				// Me desplazo la cantidad de elementos de la lista - 1 para tomar el ultimo
				((ListStore)this.listaNavegable.ListaElementos.Model).
					IterNthChild(out iterTemp, (this.listaNavegable.ListaElementos.Model.IterNChildren() - 1));
				
				path = this.listaNavegable.ListaElementos.Model.GetPath(iterTemp);
			}
			
			// En una situación normal, este es el código que se ejecuta.
			else {
				// Obtengo el path anterior
				bool quedanElementos = path.Prev();
				
				// Si no quedan elementos
				if (!quedanElementos) {
					
					// Si es la primer página
					if (this.listaNavegable.NavegadorDePaginas.PrimeraPagina)
						return false;
					
					// Si no es la primer página voy al ultimo de la página anterior
					this.listaNavegable.NavegadorDePaginas.CargarPaginaAnterior();
					
					// Obtengo el iter del primer elemento
					TreeIter iterTemp;
					this.listaNavegable.ListaElementos.Model.GetIterFirst(out iterTemp);
					
					// Me desplazo la cantidad de elementos de la lista - 1 para tomar el ultimo
					((ListStore)this.listaNavegable.ListaElementos.Model).
						IterNthChild(out iterTemp, (this.listaNavegable.ListaElementos.Model.IterNChildren() - 1));
					
					path = this.listaNavegable.ListaElementos.Model.GetPath(iterTemp);
				}
			}
			
			// Obtengo de nuevo el iter asociado al path
			this.listaNavegable.ListaElementos.Model.GetIter(out iterAnterior, path);

			return true;
		}
		
		protected bool IterSiguiente(out TreeIter iterSiguiente)
		{
			iterSiguiente = new TreeIter();
			
			if (this.pathActual != null)
				this.pathProvisorio = this.pathActual.Copy();
			
			this.listaNavegable.ListaElementos.Model.GetIter(out iterSiguiente, this.pathProvisorio);
			
			// Pido el siguiente iter
			bool quedanElementos = this.listaNavegable.ListaElementos.Model.IterNext(ref iterSiguiente);
			
			// Si no quedan elementos
			if (!quedanElementos) {
				
				// Si es la ultima página, retorno null
				if (this.listaNavegable.NavegadorDePaginas.UltimaPagina)
					return false;
				
				// Si no es la ultima página, me muevo a la página siguiente
				// y obtengo el primer iter de la lista
				else {
					this.listaNavegable.NavegadorDePaginas.CargarPaginaSiguiente();
					this.listaNavegable.ListaElementos.Model.GetIterFirst(out iterSiguiente);
				}
			}
			
			return true;
		}
		#endregion
	}
}
