// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Ice;
using Gtk;

using Stratos.Common;
using Stratos.ModeloDto;
using StratosClient.Core.Gui;
using StratosServer.Controladores;

namespace CustomWidgets
{	
	[System.ComponentModel.ToolboxItem(true)]
	public partial class RazonSocialClienteVista : Gtk.Bin, IRazonSocialVista<ClienteDto>
	{
		#region Campos
		protected EntryRazonSocial<ClienteDto> txtRazonSocial;
		
		protected RazonSocialPresenter<ClienteDto> razonSocialPresenter;
		#endregion
		
		#region Constructores
		public RazonSocialClienteVista()
		{
			this.Build();
			
			// Creo y agrego el EntryRazonSocial al widget
			this.txtRazonSocial = new EntryRazonSocial<ClienteDto>();
			this.hbEntryRazonSocial.Add(this.txtRazonSocial);
			
			// Suscripción a eventos
			this.rtxtCodigo.Activated += OnCodigoClienteActivated;
			this.txtRazonSocial.Activated += OnRazonSocialActivated;
			this.txtRazonSocial.Completion.MatchSelected += OnRazonSocialMatchSelected;
			
			// Preparo el presenter
			this.razonSocialPresenter = new RazonSocialClientePresenter(this);
		}
		#endregion
		
		#region IRazonSocialVista implementation
		public IRegExEntryVista EntryCodigoPersona {
			get { return this.rtxtCodigo; }
		}
		
		public IEntryRazonSocial EntryRazonSocialPersona {
			get { return this.txtRazonSocial; }
		}
		
		public ClienteDto Persona {
			get { return this.Cliente; }
			set { this.razonSocialPresenter.CargarPersona(value); }
		}
		
		/// <summary>
		/// Esta propiedad al ser verdadera marca el RazonEntry como de solo lectura,
		/// no dejando que sea posible la edición del cliente, ni siquiera el limpiarlo.
		/// </summary>
		public bool SoloLectura {
			set {
				// Desactivo el botón de limpiar
				this.btnLimpiar.Sensitive = !value;
				
				// Desactivo los dos Entry, el de código del cliente y razón social
				this.rtxtCodigo.IsEditable = !value;
				this.txtRazonSocial.IsEditable = !value;
			}
		}
		
		/// <summary>
		/// Esta propiedad marca a los campos de solo lectura, pero deja al botón
		/// quitar activado, para que se pueda limpiar el cliente
		/// </summary>
		public bool Editable {
			set {
				CustomWidgetsUtils.WidgetEditable(this.rtxtCodigo, value);
				CustomWidgetsUtils.WidgetEditable(this.txtRazonSocial, value);
			}
		}
		
		#region INotificador
		public void NotificarInformacion (string mensaje)
		{
			SMessageDialog.MensajeInformacion(mensaje);
		}
		
		public void NotificarAdvertencia (string mensaje)
		{
			SMessageDialog.MensajeAdvertencia(mensaje);
		}
		
		public void NotificarError (string mensaje)
		{
			SMessageDialog.MensajeError(mensaje);
		}
		
		public void NotificarError (string mensaje, System.Exception exception)
		{
			SMessageDialog.MensajeError(mensaje, exception);
		}
		#endregion
		#endregion
		
		#region Propiedades
		public ClienteDto Cliente {
			get { return this.razonSocialPresenter.Persona; }
			set { this.razonSocialPresenter.CargarPersona(value); }
		}
		
		public Button BotonLimpiar {
			get { return this.btnLimpiar; }
		}
		
		public event System.Action<ClienteDto> PersonaCargada {
			add { this.razonSocialPresenter.PersonaCargada += value; }
			remove { this.razonSocialPresenter.PersonaCargada -= value; }
		}
		
		public event System.Action PersonaQuitada {
			add { this.razonSocialPresenter.PersonaQuitada += value; }
			remove { this.razonSocialPresenter.PersonaQuitada -= value; }
		}
		
		public EntryRazonSocial<ClienteDto> RazonSocialEntry {
			get { return this.txtRazonSocial; }
		}
		
		public RegExEntry CodigoPersonaEntry {
			get { return this.rtxtCodigo; }
		}
		#endregion
		
		#region Eventos
		protected virtual void OnCodigoClienteActivated (object sender, System.EventArgs e)
		{
			this.razonSocialPresenter.CodigoPersonaActivado();
		}
		
		protected virtual void OnRazonSocialActivated (object sender, System.EventArgs e)
		{
			this.razonSocialPresenter.RazonSocialActivado();
		}
		
		[GLib.ConnectBefore]
		protected void OnRazonSocialMatchSelected(object o, MatchSelectedArgs args)
		{
			ClienteDto cliente = (ClienteDto) args.Model.GetValue(args.Iter, 0);
			this.razonSocialPresenter.CargarPersona(cliente);
			
			// Vacío el modelo, así queda listo para una nueva búsqueda
			(this.txtRazonSocial.Completion.Model as ListStore).Clear();
			
			args.RetVal = true;
		}
		
		protected virtual void OnLimpiarClicked (object sender, System.EventArgs e)
		{
			this.razonSocialPresenter.Vaciar();
		}
		#endregion
		
		#region Otros metodos
		public void CargarCliente(ClienteDto cliente)
		{
			this.razonSocialPresenter.CargarPersona(cliente);
		}
		
		public void Vaciar()
		{
			this.razonSocialPresenter.Vaciar();
		}
		#endregion
	}
}
