// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;

using Gtk;

namespace CustomWidgets
{
	/// <summary>
	/// TreeView base para las demás clases de listas.
	/// </summary>
	public abstract class ColeccionElementos : Gtk.TreeView, IColeccionElementosVista
	{
		public event System.Action PrimeraPaginaCargada;
		public event System.Action ListaVaciada;
		
		#region Propiedades
		public int CantidadElementos {
			get { return this.Model.IterNChildren(); }
		}
		
		public virtual bool SoloLectura {
			set {
				if (value)
					this.Selection.Mode = Gtk.SelectionMode.None;
				else
					this.Selection.Mode = Gtk.SelectionMode.Browse;
			}
		}
		#endregion
		
		#region Métodos
		public void AgregarElementosPrimeraPagina(params object[] elementos)
		{
			this.Vaciar();
			this.AgregarElementos(elementos);
			
			if (this.PrimeraPaginaCargada != null)
				this.PrimeraPaginaCargada();
		}
		
		protected bool HabilitarSetCursor()
		{
			Gdk.Event ev = Gtk.Application.CurrentEvent;
			
			if (ev == null)
				return true;
			
			// Verifico si el evento fue KeyPress y si se presionó la tecla return
			else if (ev.Type == Gdk.EventType.KeyPress) {
				Gdk.EventKey evKey = (Gdk.EventKey)ev;
				
				if (evKey.Key == Gdk.Key.Return || evKey.Key == Gdk.Key.KP_Enter)
					return true;
			}
			
			return false;
		}
		
		internal void ColocarCeldaEnModoEdicion(TreePath path, TreeViewColumn columna)
		{
			if (!this.HabilitarSetCursor())
				return;
			
			this.GrabFocus();
			this.SetCursor(path, columna, true);
		}
		
		public virtual void Vaciar()
		{
			((ListStore)this.Model).Clear();
			
			if (this.ListaVaciada != null)
				this.ListaVaciada();
		}
		#endregion
		
		#region Métodos abstractos
		public abstract void InsertarElemento(object elemento, int posicion);
		public abstract void AgregarElementos(params object[] elementos);
		
		public abstract void BorrarItems(params TreeIter[] iters);
		
		public abstract object[] ObtenerSeleccionados(out TreeIter[] itersSeleccionados);
		public abstract object[] ObtenerSeleccionados();
		public abstract IList ObtenerTodos();
		#endregion
	}
}
