// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.Common;
using Stratos.ModeloDto;

namespace CustomWidgets
{
	public class BonificacionCellRenderer : CellRendererInteractivoBase<ItemProductoDto>
	{
		public BonificacionCellRenderer(ListaItemsProducto treeView) : base(treeView)
		{
			this.Xalign = 1.0f;
		}
		
		protected override bool ValidarDatos (ItemProductoDto item, string nuevoValor)
		{
			double valorNumerico = 0;
		
			// Chequeo que la cantidad sea numérica (entera o decimal)
			if (StratosUtil.IsDouble(nuevoValor))
				valorNumerico = StratosUtil.ParseDouble(nuevoValor, 2);
			else
				return false;
			
			// La bonificación no puede ser superior al +/- 50%
			if (valorNumerico < -50.0 || valorNumerico > 50.0)
				return false;
			
			return true;
		}
		
		protected override Gtk.TreeViewColumn ModificarItem (ItemProductoDto item,
		                                       string nuevoValor)
		{
			/* Si el valor nuevo es igual al anterior, retorno false. Para eso
			 * primero lo paso a double y lo comparo con el valor actual */
			double nuevoValorDouble = StratosUtil.ParseDouble(nuevoValor, 2);
			
			item.porcentajeBonificacion = nuevoValorDouble;
			
			return null;
		}
		
		protected override void EdicionIniciada (Gtk.Entry entry, string path, ItemProductoDto item)
		{
			entry.Text = StratosUtil.DoubleToString(item.porcentajeBonificacion);
		}
		
		protected override bool ComenzarEdicion()
		{
			return true;
		}
	}
}
