// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;
using Stratos.Common;

namespace CustomWidgets
{
	public class CantidadCellRenderer : CellRendererInteractivoBase<ItemProductoDto>
	{
		public CantidadCellRenderer(ListaItemsProducto treeView) : base(treeView)
		{
			this.Xalign = 1.0f;
		}
		
		protected override bool ValidarDatos (ItemProductoDto item, string nuevoValor)
		{
			double valorNumerico = 0;
		
			// Chequeo que la cantidad sea numérica (entera o decimal)
			if (StratosUtil.IsDouble(nuevoValor))
				valorNumerico = StratosUtil.ParseDouble(nuevoValor, 2);
			else
				return false;
			
			// Esa cantidad numérica debe ser mayor a cero
			if (valorNumerico <= 0)
				return false;
			
			return true;
		}
		
		protected override TreeViewColumn ModificarItem (ItemProductoDto item,
		                                       string nuevoValor)
		{
			item.cantidad = StratosUtil.ParseDouble(nuevoValor, 2);
			
			return null;
		}
		
		protected override void EdicionIniciada (Gtk.Entry entry, string path, ItemProductoDto item)
		{
			entry.Text = StratosUtil.DoubleToString(item.cantidad);
		}
		
		protected override bool ComenzarEdicion()
		{
			return true;
		}
	}
}
