// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.Common;
using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace CustomWidgets
{
	public class CodigoCellRenderer : CellRendererMenuDesplegable
	{
		public CodigoCellRenderer(ListaItemsProducto treeView) : base(treeView)
		{
		}
		
		protected override bool ValidarDatos (ItemProductoDto item, string nuevoValor)
		{
			// Un código nulo es inválido
			/* FIXME: Si todos los código tuviesen un formato común, podríamos realizar
			 * más verificaciones */
			if (nuevoValor.Trim().Equals(String.Empty))
				return false;
			
			/* Verifico que el código del producto sea único en todos los items */
			if (this.tvItemsProducto.ProductoEstaCargado(nuevoValor,
			                                             item.descripcion)) {
				SMessageDialog.MensajeAdvertencia("El producto ya está cargado en los items");
				
				this.treeView
					.ColocarCeldaEnModoEdicion(this.tvItemsProducto.GetSelectedPath(),
					                           this.tvItemsProducto.tvcCodigoProducto);
				
				return false;
			}
			
			return true;
		}
		
		protected override Gtk.TreeViewColumn ModificarItem (ItemProductoDto item,
		                                                     string nuevoValor)
		{
//			if (item.producto.productoAbstracto) {
//				
//				SMessageDialog
//					.MensajeAdvertencia("Cuando se trata de un producto genérico, " +
//					                    "solamente se puede agregar texto a la descripción existente.");
//				
//				return this.tvItemsProducto.tvcDescripcionProducto;
//			}
			
//			item.productoCodigo = nuevoValor;
			
			return null;
		}
		
		protected override bool ComenzarEdicion()
		{
			return true;
		}
		
		#region CellRendererMenuDesplegable - Implementación
		protected override TreeViewColumn Columna {
			get { return this.tvItemsProducto.tvcCodigoProducto; }
		}
		
		protected override int TamañoMinimoConsulta {
			get { return 2; }
		}
		
		protected override bool NoRealizarConsulta (string consultaEscrita,
		                                            ItemProductoDto item,
		                                            KeyPressEventArgs args)
		{
			return false;
		}
		
		protected override ProductoDto[] RealizarBusqueda (string consulta)
		{
			// Realizo la búsqueda de productos por descripción
			ProductoDto[] productos =
				((ControladorProductosPrx)MainIce.GetControlador("ControladorProductos"))
					.BuscarPorCodigo(consulta);
			
			return productos;
		}
		
		protected override bool MostrarMenuConUnicoResultado (ProductoDto producto, string consulta)
		{
			/* Cuando busco por código y tengo un único resultado, y el código del producto
			 * es _exactamente_ igual a lo que estaba buscando, entonces no muestro el
			 * menú desplegable y cargo el producto */
			if (producto.codigo.ToLower() != consulta.ToLower())
				return true;
			
			// Antes de cargar el producto, me aseguro que haya sido cargado en otro item
			if (this.tvItemsProducto.ProductoEstaCargado(consulta,
			                                             producto.descripcion))
				return true;
			
			this.tvItemsProducto.CargarProducto(producto);
			
			return false;
		}
		#endregion
	}
}
