// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using Gtk;

using Stratos.Common;
using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace CustomWidgets
{
	/// <summary>
	/// Clase que permite crear una lista de items para facturas de compras.
	/// </summary>
	public class ListaItemsCompras : ListaItemsProducto
	{
		#region Constructores
		public ListaItemsCompras ()
		{
			// Quito la columna de bonificación
			this.tvcBonificacion.Visible = false;
			
			CellRendererText crtColumnaRender;
			
			this.tvcPrecioUnitarioProducto.Clear();
			crtColumnaRender = new PrecioUnitarioItemCompraCellRenderer(this);
			this.tvcPrecioUnitarioProducto.PackStart(crtColumnaRender, true);
			this.tvcPrecioUnitarioProducto.SetCellDataFunc(crtColumnaRender, this.RenderPrecioUnitario);
			
			this.tvcUnidad.Clear();
			crtColumnaRender = new UnidadCellRenderer(this);
			this.tvcUnidad.PackStart(crtColumnaRender, true);
			this.tvcUnidad.SetCellDataFunc(crtColumnaRender, this.RenderUnidad);
			
			// Escondo la columna de importe
			this.tvcImporte.Visible = false;
			
			this.LimiteItems = 50;
		}
		#endregion
		
		#region Métodos
		protected override bool PermitirCargaProducto (ProductoDto producto, out string mensajeCargaProductoNoPermitida)
		{
			mensajeCargaProductoNoPermitida = String.Empty;
			return true;
		}
		
		protected override void CargarDatosExtraProducto(ProductoDto producto, ItemProductoDto item)
		{
		}
		
		protected override TreeViewColumn SiguienteColumnaAEditar (TreeViewColumn columna)
		{
			ItemProductoDto itemProductoActual = this.ObtenerItemSeleccionadoGenerico();
			
			#region Acciones para productos abstractos
			if (!itemProductoActual.productoAbstracto)
				return null;
			
			/* Coloco en modo de edición la columna de descripción si este valor no ha
			 * sido cambiado del original. */
			if ((columna == this.tvcCodigoProducto || columna == this.tvcDescripcionProducto) &&
			    	itemProductoActual.descripcion == itemProductoActual.productoDescripcion)
				return this.tvcDescripcionProducto;
			
			/* Si estoy en la columna de código o descripción y la misma ha sido cambiada
			 * ya, entonces coloco el foco en la columna de precio unitario. */
			if ((columna == this.tvcCodigoProducto || columna == this.tvcDescripcionProducto))
				return this.tvcPrecioUnitarioProducto;
			#endregion
			
			return null;
		}
		
		protected override bool ItemValidoGenerico(ItemProductoDto item, out TreeViewColumn columnaInvalida)
		{
			// Verifico si el código del producto es correcto
			if (item.productoCodigo == null ||
			    item.productoCodigo.Trim().Equals(String.Empty)) {
				
				columnaInvalida = this.tvcCodigoProducto;
				
				return false;
			}
			
			// Verifico si la descripción del producto es correcta
			if (item.descripcion == null ||
			    item.descripcion.Trim().Equals(String.Empty)) {
				
				columnaInvalida = this.tvcDescripcionProducto;
				
				return false;
			}
			
			if (item.precioUnitario == 0.00) {
				
				//this.tvcPrecioUnitarioProducto.CellRenderers[0]
				columnaInvalida = this.tvcPrecioUnitarioProducto;
				
				return false;
			}
			
			// Verifico si la cantidad es correcta
			if (item.cantidad <= 0.00) {
				
				columnaInvalida = this.tvcCantidad;
				
				return false;
			}
			
			columnaInvalida = null;
			
			return true;
		}
		#endregion
	}
}
