// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Reflection;

using Gtk;

using Stratos.ModeloDto;
using StratosClient.Core.Gui;

namespace CustomWidgets
{
	/// <summary>
	/// Lista de items interactivos de recibos.
	/// </summary>
	public class ListaItemsRecibo : ListaInteractiva<ItemReciboDto>, IListaItemsReciboVista
	{
		#region Campos
		public TreeViewColumn tvcMedioPago;
		public TreeViewColumn tvcChequeBanco;
		public TreeViewColumn tvcChequeSucursal;
		public TreeViewColumn tvcChequeNumero;
		public TreeViewColumn tvcChequeFechaEmision;
		public TreeViewColumn tvcChequeFechaVencimiento;
		public TreeViewColumn tvcChequeObservacion;
		public TreeViewColumn tvcImporte;
		#endregion
		
		#region Constructores
		public ListaItemsRecibo () : base()
		{
			this.CanFocus = true;
			
			this.limiteItems = 15;
			
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Medio";
			MedioPagoCellRenderer crtColumnaMedioPago = new MedioPagoCellRenderer(this);
			tvcColumna.PackStart(crtColumnaMedioPago, true);
			
//			crtColumnaMedioPago.Editable = true;
//			crtColumnaMedioPago.HasEntry = false;
//			crtColumnaMedioPago.TextColumn = 0;
			
//			CellRendererCombo crtColumnaComboRender = new CellRendererCombo();
			tvcColumna.SetCellDataFunc(crtColumnaMedioPago, this.RenderMedioPago);
			this.AppendColumn(tvcColumna);
			this.tvcMedioPago = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Expand = true;
			tvcColumna.Title = "Banco";
			crtColumnaRender = new ChequeBancoCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeBanco);
			this.AppendColumn(tvcColumna);
			this.tvcChequeBanco = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Expand = true;
			tvcColumna.Title = "Sucursal";
			crtColumnaRender = new ChequeSucursalCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeSucursal);
			this.AppendColumn(tvcColumna);
			this.tvcChequeSucursal = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Número";
			crtColumnaRender = new ChequeNumeroCellRenderer(this);
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeNumero);
			this.AppendColumn(tvcColumna);
			this.tvcChequeNumero = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Fecha emisión";
			crtColumnaRender = new ChequeFechaEmisionCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeFechaEmision);
			this.AppendColumn(tvcColumna);
			this.tvcChequeFechaEmision = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Fecha vencimiento";
			crtColumnaRender = new ChequeFechaVencimientoCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeFechaVencimiento);
			this.AppendColumn(tvcColumna);
			this.tvcChequeFechaVencimiento = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Expand = true;
			tvcColumna.Title = "Observación";
			crtColumnaRender = new ChequeObservacionCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderChequeObservacion);
			this.AppendColumn(tvcColumna);
			this.tvcChequeObservacion = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe";
			crtColumnaRender = new ImporteItemReciboCellRenderer(this);
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporte);
			this.AppendColumn(tvcColumna);
			this.tvcImporte = tvcColumna;
		}
		#endregion
		
		#region Funciones de renderizado del TreeView
		private void RenderMedioPago(TreeViewColumn column,
		                             CellRenderer cell,
		                             TreeModel model,
		                             TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = item != null ?
				Stratos.Common.StratosUtil.SplitCamelString(item.medioPag.ToString()) :
				String.Empty;
			
			// Si se trata de un item vacío y es el último ítem, entonces
			// no muestro el medio de pago
			TreeIter ultimoIter = this.ObtenerUltimoIter();
			
			if (this.ItemVacioGenerico(item) &&
			    	iter.Equals(ultimoIter) &&
			    	!this.EstaFilaEnModoEdicion(iter))
				valor = String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeBanco(TreeViewColumn column,
		                               CellRenderer cell,
		                               TreeModel model,
		                               TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = (item != null && item.cheque != null) ? item.cheque.banco : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeSucursal(TreeViewColumn column,
		                                  CellRenderer cell,
		                                  TreeModel model,
		                                  TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = (item != null && item.cheque != null) ? item.cheque.sucursal : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeNumero(TreeViewColumn column,
		                                CellRenderer cell,
		                                TreeModel model,
		                                TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = (item != null && item.cheque != null) ? item.cheque.numero.ToString() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeFechaEmision(TreeViewColumn column,
		                                      CellRenderer cell,
		                                      TreeModel model,
		                                      TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			
			string valor = "-";
			if (item != null && item.cheque != null) {
				DateTime fecha = new DateTime(item.cheque.fechaEmisionTicks);
			
				if (fecha.Ticks > 0)
					valor = fecha.ToShortDateString();
			}
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeFechaVencimiento(TreeViewColumn column,
		                                          CellRenderer cell,
		                                          TreeModel model,
		                                          TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			
			string valor = "-";
			if (item != null && item.cheque != null) {
				DateTime fecha = new DateTime(item.cheque.fechaVencimientoTicks);
			
				if (fecha.Ticks > 0)
					valor = fecha.ToShortDateString();
			}
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderChequeObservacion(TreeViewColumn column,
		                                     CellRenderer cell,
		                                     TreeModel model,
		                                     TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = (item != null && item.cheque != null) ? item.cheque.observacion : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderImporte(TreeViewColumn column,
		                           CellRenderer cell,
		                           TreeModel model,
		                           TreeIter iter)
		{
			ItemReciboDto item = (ItemReciboDto) model.GetValue(iter, 0);
			string valor = (item != null) ? item.importe.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
		
		#region TreeViewInteractivo implementation
		protected override Gtk.TreeViewColumn ColumnaEdicionSobreItemVacio {
			get {
				return this.tvcMedioPago;
			}
		}
		
		internal override Gtk.TreeIter AgregarItemVacio()
		{
			ItemReciboDto nuevoItem = new ItemReciboDto();
			nuevoItem.cheque = new ChequeDto();
			nuevoItem.cheque.fechaEmisionTicks = 0;
			nuevoItem.cheque.fechaVencimientoTicks = 0;
			nuevoItem.importe = 0.00;
			
			ListStore modelo = (ListStore) this.Model;
			
			return modelo.AppendValues(nuevoItem);
		}
		
		internal override void ItemModificadoListener (ItemReciboDto item)
		{
			// No hago nada cuando se modifica un item
		}
		
		protected override bool ItemVacioGenericoInterno(ItemReciboDto item)
		{
			if (item.medioPag == MedioPago.Efectivo &&
			    	item.importe == 0.00)
				return true;
			
			return false;
		}
		
		protected override bool ItemValidoGenerico(ItemReciboDto item, out Gtk.TreeViewColumn columnaInvalida)
		{
			columnaInvalida = null;
			
			// Si el medio de pago es Efectivo y no hay importa ingresado
			if (item.medioPag == MedioPago.Efectivo &&
			    	item.importe == 0.00) {
				
				columnaInvalida = this.tvcImporte;
				
				return false;
			}
			
			// Si el medio de pago es Efectivo y hay importe, el item es válido
			if (item.medioPag == MedioPago.Efectivo)
				return true;
			
			// Continúo para el caso en que el medio de pago es Cheque
			
			// Si el medio de pago es un cheque y el campo 'cheque' es nulo
			if (item.cheque == null) {
				
				columnaInvalida = this.tvcChequeBanco;
				
				return false;
			}
			
			// Si el banco no fue ingresado
			if (item.cheque.banco == null ||
			    	item.cheque.banco.Trim() == String.Empty) {
				
				columnaInvalida = this.tvcChequeBanco;
				
				return false;
			}
			
			// Si la sucursal no fue ingresada
			if (item.cheque.sucursal == null ||
			    	item.cheque.sucursal.Trim() == String.Empty) {
				
				columnaInvalida = this.tvcChequeSucursal;
				
				return false;
			}
			
			// Si el número no fue ingresado
			if (item.cheque.numero == 0.00) {
				
				columnaInvalida = this.tvcChequeNumero;
				
				return false;
			}
			
			// Si la fecha de emisión no fue ingresada
			if (item.cheque.fechaEmisionTicks == 0) {
				
				columnaInvalida = this.tvcChequeFechaEmision;
				
				return false;
			}
			
			// Si la fecha de vencimiento no fue ingresada
			if (item.cheque.fechaVencimientoTicks == 0) {
				
				columnaInvalida = this.tvcChequeFechaVencimiento;
				
				return false;
			}
			
			// Si el importe no fue ingresado
			if (item.importe == 0.00) {
				
				columnaInvalida = this.tvcImporte;
				
				return false;
			}
			
			// En cualquier otro caso el item es válido
			return true;
		}
		
		/// <summary>
		/// Devuelve la suma los importes de los items del recibo.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Decimal"/>
		/// </returns>
		public double SumaImportesItemes()
		{
			double sumaItems = 0.00;
			
			foreach (ItemReciboDto item in
			         this.Model.TraverseColumn<ItemReciboDto>(0))
				sumaItems += item.importe;
			
			return sumaItems;
		}
		#endregion
	}
}
