// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Gtk;

using Stratos.Common;
using Stratos.ModeloDto;

namespace CustomWidgets
{
	public class TvDocumentosDebito : ListaSeleccionable<DocumentoDebitoDto>, IDocumentoDebitoVista
	{
		#region Campos
		public TreeViewColumn tvcTipoDocumento;
		public TreeViewColumn tvcCodigo;
		public TreeViewColumn tvcNumero;
		public TreeViewColumn tvcFecha;
		public TreeViewColumn tvcImporteTotal;
		public TreeViewColumn tvcImportePendiente;
		#endregion
		
		#region Constructores
		public TvDocumentosDebito ()
		{
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Tipo";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderTipoDocumento);
			this.AppendColumn(tvcColumna);
			this.tvcTipoDocumento = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Código";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCodigo);
			this.AppendColumn(tvcColumna);
			this.tvcCodigo = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Número";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderNumero);
			this.AppendColumn(tvcColumna);
			this.tvcNumero = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Fecha venc.";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderFechaVencimiento);
			this.AppendColumn(tvcColumna);
			this.tvcFecha = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe total";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporteTotal);
			this.AppendColumn(tvcColumna);
			this.tvcImporteTotal = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe pendiente";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImportePendiente);
			this.AppendColumn(tvcColumna);
			this.tvcImportePendiente = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = String.Empty;
			this.AppendColumn(tvcColumna);
			#endregion
		}
		#endregion
		
		#region Métodos de renderizado
		protected DocumentoDebitoDto ObtenerDocumento(TreeViewColumn column,
		                                              CellRenderer cell,
		                                              TreeModel model,
		                                              TreeIter iter)
		{
			ElementoSeleccionable<DocumentoDebitoDto> doc =
				(ElementoSeleccionable<DocumentoDebitoDto>) model.GetValue(iter, 0);
			
			return doc.Elemento;
		}
		
		protected void RenderTipoDocumento (TreeViewColumn column,
		                                    CellRenderer cell,
		                                    TreeModel model,
		                                    TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ?
				StratosUtil.SplitCamelString(doc.tipoDoc.ToString()) :
					String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderCodigo(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ? doc.codigoFactura.ToString("D4") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderNumero(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ? doc.numeroFactura.ToString() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderFechaVencimiento(TreeViewColumn column,
		                                    CellRenderer cell,
		                                    TreeModel model,
		                                    TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ?
				(new DateTime(doc.fechaVencimientoTicks)).ToShortDateString() :
				String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderImporteTotal(TreeViewColumn column,
		                                CellRenderer cell,
		                                TreeModel model,
		                                TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ? doc.importeTotal.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderImportePendiente(TreeViewColumn column,
		                                    CellRenderer cell,
		                                    TreeModel model,
		                                    TreeIter iter)
		{
			DocumentoDebitoDto doc =
				this.ObtenerDocumento(column, cell, model, iter);
			
			string valor = doc != null ? doc.importePendiente.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
		
		#region TreeViewSeleccionable implementation
		protected override bool ElementosIguales (DocumentoDebitoDto t1, DocumentoDebitoDto t2)
		{
			return (t1.id == t2.id);
		}
		#endregion
	}
}
