// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace CustomWidgets
{
	/// <summary>
	/// Clase base para mostrar Documentos de Venta.
	/// </summary>
	public class TvDocumentosVenta<T> : TvDocumentosGenericos<T>
		where T : DocumentoVentaDto
	{
		#region Campos
		public TreeViewColumn tvcCliente;
		public TreeViewColumn tvcImporteFinal;
		#endregion
		
		#region Constructores
		public TvDocumentosVenta () : base()
		{
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe total";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporteTotal);
			this.InsertColumn(tvcColumna, 2);
			this.tvcImporteFinal = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Cliente";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCliente);
			this.InsertColumn(tvcColumna, 2);
			this.tvcCliente = tvcColumna;
			#endregion
		}
		#endregion
		
		#region Métodos de renderizado
		protected virtual void RenderCliente(TreeViewColumn column,
		                                     CellRenderer cell,
		                                     TreeModel model,
		                                     TreeIter iter)
		{
			T doc = this.RenderBase(column, cell, model, iter);
			string valor = (doc != null && doc.cliente != null) ?
				doc.cliente.razonSocial : "-";
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected virtual void RenderImporteTotal(TreeViewColumn column,
		                                          CellRenderer cell,
		                                          TreeModel model,
		                                          TreeIter iter)
		{
			T doc = this.RenderBase(column, cell, model, iter);
			string valor = doc != null ? doc.importeFinal.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
	}
}
