// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace CustomWidgets
{
	public class TvMovimientos : TvDocumentosGenericos<DocumentoGenericoDto>
	{
		#region Campos
		public TreeViewColumn tvcCliente;
		public TreeViewColumn tvcReciboConcepto;
		public TreeViewColumn tvcImporteTotal;
		#endregion
		
		#region Constructores
		public TvMovimientos () : base()
		{
			#region Columnas
			this.tvcObservaciones.Visible = false;
			
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Cliente";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCliente);
			this.InsertColumn(tvcColumna, 2);
			this.tvcCliente = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Concepto recibo";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderReciboConcepto);
			this.AppendColumn(tvcColumna);
			this.tvcReciboConcepto = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe total";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporteTotal);
			this.AppendColumn(tvcColumna);
			this.tvcImporteTotal = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = String.Empty;
			this.AppendColumn(tvcColumna);
			#endregion
		}
		#endregion
		
		#region Métodos de renderizado
		protected virtual void RenderCliente(TreeViewColumn column,
		                                     CellRenderer cell,
		                                     TreeModel model,
		                                     TreeIter iter)
		{
			DocumentoGenericoDto doc = (DocumentoGenericoDto) model.GetValue(iter, 0);
			string valor = String.Empty;
			
			FacturaDto factura = doc as FacturaDto;
			ReciboDto recibo = doc as ReciboDto;
			
			if (factura != null)
				valor = factura.cliente.razonSocial;
			else if (recibo != null)
				valor = recibo.cliente.razonSocial;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected virtual void RenderReciboConcepto(TreeViewColumn column,
		                                            CellRenderer cell,
		                                            TreeModel model,
		                                            TreeIter iter)
		{
			ReciboDto recibo = model.GetValue(iter, 0) as ReciboDto;
			string valor = recibo != null ? recibo.concepto : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected virtual void RenderImporteTotal(TreeViewColumn column,
		                                          CellRenderer cell,
		                                          TreeModel model,
		                                          TreeIter iter)
		{
			DocumentoGenericoDto doc = (DocumentoGenericoDto) model.GetValue(iter, 0);
			string valor = String.Empty;
			
			FacturaDto factura = doc as FacturaDto;
			ReciboDto recibo = doc as ReciboDto;
			
			if (factura != null)
				valor = factura.importeTotal.ToString("N2");
			else if (recibo != null)
				valor = recibo.importe.ToString("N2");
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
	}
}
