// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace CustomWidgets
{
	/// <summary>
	/// Clase para mostrar listados de elementos tipo ReciboDto.
	/// </summary>
	public class TvRecibos : TvDocumentosGenericos<ReciboDto>
	{
		#region Campos
		public TreeViewColumn tvcCliente;
		public TreeViewColumn tvcConcepto;
		public TreeViewColumn tvcImporteTotal;
		public TreeViewColumn tvcImporteDisponible;
		#endregion
		
		#region Constructores
		public TvRecibos () : base()
		{
			this.tvcTipoDocumento.Visible = false;
			this.tvcObservaciones.Visible = false;
			
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Cliente";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCliente);
			this.AppendColumn(tvcColumna);
			this.tvcCliente = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Concepto";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderConcepto);
			this.AppendColumn(tvcColumna);
			this.tvcConcepto = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe total";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporteTotal);
			this.AppendColumn(tvcColumna);
			this.tvcImporteTotal = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = String.Empty;
			this.AppendColumn(tvcColumna);
			#endregion
		}
		#endregion
		
		#region Métodos de renderización
		protected virtual void RenderCliente(TreeViewColumn column,
		                                     CellRenderer cell,
		                                     TreeModel model,
		                                     TreeIter iter)
		{
			ReciboDto recibo = this.RenderBase(column, cell, model, iter);
			string valor = (recibo != null && recibo.cliente != null) ?
				recibo.cliente.razonSocial :
				String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected virtual void RenderConcepto(TreeViewColumn column,
		                                      CellRenderer cell,
		                                      TreeModel model,
		                                      TreeIter iter)
		{
			ReciboDto recibo = this.RenderBase(column, cell, model, iter);
			string valor = recibo != null ? recibo.concepto : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected virtual void RenderImporteTotal(TreeViewColumn column,
		                                          CellRenderer cell,
		                                          TreeModel model,
		                                          TreeIter iter)
		{
			ReciboDto recibo = this.RenderBase(column, cell, model, iter);
			string valor = recibo != null ? recibo.importe.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
	}
}
