// Copyright (C) 2009, 2010 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace CustomWidgets
{
	/// <summary>
	/// Lista simple de productos.
	/// </summary>
	public class TvProductos : ListaElementos<ProductoDto>
	{
		#region Campos
		public TreeViewColumn tvcCodigo;
		public TreeViewColumn tvcDescripcion;
		public TreeViewColumn tvcPrecioVenta;
		#endregion
		
		#region Constructores
		public TvProductos() : base()
		{
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Código";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCodigo);
			this.AppendColumn(tvcColumna);
			this.tvcCodigo = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Descripción";
			tvcColumna.Expand = true;
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderDescripcion);
			this.AppendColumn(tvcColumna);
			this.tvcDescripcion = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Precio Venta (con IVA) ";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderPrecioVenta);
			this.AppendColumn(tvcColumna);
			this.tvcPrecioVenta = tvcColumna;
			#endregion
		}
		#endregion
		
		#region Métodos de renderizado
		
		/// <summary>
		/// Este método realiza las operaciones de renderizado básicar para todas las
		/// columnas.
		/// </summary>
		/// <param name="column">
		/// A <see cref="TreeViewColumn"/>
		/// </param>
		/// <param name="cell">
		/// A <see cref="CellRenderer"/>
		/// </param>
		/// <param name="model">
		/// A <see cref="TreeModel"/>
		/// </param>
		/// <param name="iter">
		/// A <see cref="TreeIter"/>
		/// </param>
		/// <returns>
		/// A <see cref="ProductoDto"/>
		/// </returns>
		protected ProductoDto RenderBase(TreeViewColumn column,
		                                                   CellRenderer cell,
		                                                   TreeModel model,
		                                                   TreeIter iter)
		{
			ProductoDto producto = (ProductoDto) model.GetValue(iter, 0);
			if (producto == null) return null;
			
			// Si la ProductoDto está anulada, entonces cambio el texto y el fondo de la fila
			if (producto.eliminado) {
				CellRendererText cellText = cell as CellRendererText;
				
				cell.CellBackground = producto.eliminado ? "red" : "wheat";
				
				if (cellText != null)
					cellText.Strikethrough = true;
			}
			
			// Si el ProductoDto no está Eliminado, entonces vuelvo
			// a colocar todos los valores por defecto.
			else {
				CellRendererText cellText = cell as CellRendererText;
				
				cell.CellBackground = null;
				
				if (cellText != null)
					cellText.Strikethrough = false;
			}
			
			return producto;
		}		
		
		private void RenderCodigo(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			ProductoDto producto = this.RenderBase(column, cell, model, iter);
			string valor = producto != null ? producto.codigo.ToString() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}

		private void RenderDescripcion(TreeViewColumn column,
		                               CellRenderer cell,
		                               TreeModel model,
		                               TreeIter iter)
		{
			ProductoDto producto = this.RenderBase(column, cell, model, iter);
			string valor = producto != null ? producto.descripcion : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderPrecioCosto(TreeViewColumn column,
		                                   CellRenderer cell,
		                                   TreeModel model,
		                                   TreeIter iter)
		{
			ProductoDto producto = this.RenderBase(column, cell, model, iter);
			string valor = producto != null ? producto.precioCosto.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderPrecioVenta(TreeViewColumn column,
		                                   CellRenderer cell,
		                                   TreeModel model,
		                                   TreeIter iter)
		{
			ProductoDto producto = this.RenderBase(column, cell, model, iter);
			string valor = producto != null ? producto.importeVenta.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		#endregion
		
		#region Otros metodos
		protected override bool Iguales (ProductoDto t1, ProductoDto t2)
		{
			if (t1.id == t2.id)
				return true;
			return false;
		}
		#endregion
	}
}
