// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
//
// This file is part of Stratos.
//
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using Gtk;
using Mono.Addins;
using Stratos.Common;

namespace StratosClient.Core.Gui
{
	public class MenuItemNode : MenuBaseNode
	{
		[NodeAttribute]
		string icono;
		
		[NodeAttribute]
		string tipoComando;
		
		[NodeAttribute]
		bool necesitaConexion;
		
		static AccelGroup accelGroup = new AccelGroup ();
		
		public override Gtk.Bin GetWidget ()
		{
			MenuItem item = null;
			
			/* Creo un ImageMenuItem o un MenuItem de acuerdo a los datos que haya proporcionado el
			 * developer del addin. */
			if (icono != null && this.etiqueta != null) {
				item = new ImageMenuItem (this.icono, accelGroup);
				(item.Child as AccelLabel).LabelProp = this.etiqueta;
			}
			else if (icono != null)
				item = new ImageMenuItem (this.icono, accelGroup);
			else if (this.etiqueta != null)
				item = new MenuItem(this.etiqueta);
			
			item.Activated += OnClicked;
			
			/* Si el item necesita conexión con el servidor, entonces me suscribo
			 * a los eventos de conexión y desconexión para habilitarlo/deshabilitarlo
			 * cuando corresponda. */
			if (this.necesitaConexion) {
				MainIce.ConexionEstablecida += delegate {
					item.Sensitive = true;
				};
				
				MainIce.DesconexionRealizada += delegate {
					item.Sensitive = false;
				};
				
				item.Sensitive = MainIce.ConectadoAlServidor;
			}
			
			return item;
		}
		
		void OnClicked (object s, EventArgs a)
		{
			IComando cmd;
			
			try {
				cmd = (IComando) Addin.CreateInstance (tipoComando);
				
				// Al comando lo ejecuto dentro del thread de Gtk.
				Gtk.Application.Invoke(delegate {
					cmd.Ejecutar();
				});
			}
			catch (InvalidOperationException ex) {
				SMessageDialog.MensajeAdvertencia("No se ha definido una acción correcta para el menú.");
			}
		}
	}
}
