// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Mono.Addins;

using Stratos.Common;

namespace StratosClient.Core.Gui
{
	public class TabNode : WidgetNode
	{
		[NodeAttribute]
		protected string etiqueta;
		
		[NodeAttribute]
		protected string icono;
		
		[NodeAttribute]
		protected string componente;

		[NodeAttribute]
		protected int posicion;
		
		public string Componente {
			get { return this.componente; }
		}
		
		public string EtiquetaCadena {
			get { return this.etiqueta; }
		}
		
		public Gtk.Widget Etiqueta {
			get {
				Gtk.HBox hbox = new Gtk.HBox(false, 0);
				
				// Cargamos el ícono si éste fue establecido
				if (this.icono != null && !this.icono.Equals(String.Empty)) {
					Gtk.Image imagenIcono = null;
					
					/* Creo unas imágenes de referencia, del tamaño de un StockItem, para poder
					 * redimencionar las imagenes definidas por el developer del addin al tamaño
					 * adecuado. */
					Gdk.Pixbuf pixbufReferencia =
						MainWindow.Instancia.RenderIcon("gtk-new", Gtk.IconSize.Button, null);
					
					/* Vemos si 'icono' es un StockItem (gtk-new, gtk-connect, etc, iconos que vienen)
					 * incluidos con la distribución de Gtk+ */
					Gtk.StockItem temp;
					if (Gtk.StockManager.LookupItem(this.icono, out temp))
						imagenIcono = new Gtk.Image(this.icono, Gtk.IconSize.Button);
					else {
						/* En caso de ser una imágen definida por el developer, la carga (suponiendo
						 * que se encuentra incluida como recurso embebido en el assembly) y la
						 * redimensiono. */
						Gdk.Pixbuf pixbuf = new Gdk.Pixbuf(this.Addin.GetResource(this.icono));
						
						imagenIcono =
							new Gtk.Image(pixbuf.ScaleSimple(pixbufReferencia.Width,
							                                 pixbufReferencia.Height,
							                                 Gdk.InterpType.Bilinear));
					}
					
					// Si fue seteada, coloco la imagen en el hbox
					if (imagenIcono != null)
						hbox.PackStart(imagenIcono, false, false, 3);
				}
				
				// Ahora coloco el label y lo agrego al hbox
				Gtk.Label label = new Gtk.Label(this.etiqueta);
				label.WidthRequest = 90;
				
				// Justifico el texto del label y activo line wrap.
				label.Justify = Gtk.Justification.Center;
				label.LineWrap = true;
				label.LineWrapMode = Pango.WrapMode.Word;
				
				hbox.PackStart(label, true, true, 0);
				
				hbox.ShowAll();
				
				return hbox;
			}
		}
		
		public override Gtk.Bin GetWidget ()
		{
			Gtk.Bin widget = (Gtk.Bin) Addin.CreateInstance (this.componente);
			widget.ShowAll();
			return widget;
		}

		public int Posicion {
			get { return this.posicion; }
		}
	}
}
