// Copyright (C) 2009 Chris Howie
// Released under the MIT/X11 license

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using Gtk;

namespace StratosClient.Core.Gui
{
	public static class Extensiones
	{
		public static IEnumerable TraverseColumn(this TreeModel model, int column)
		{
			foreach (TreeIter i in model.Traverse()) {
				yield return model.GetValue(i, column);
			}
		}

		public static IEnumerable<T> TraverseColumn<T>(this TreeModel model, int column)
		{
			return model.TraverseColumn(column).Cast<T>();
		}

		public static IEnumerable<TreeIter> Traverse(this TreeModel model)
		{
			TreeIter current;
			if (!model.GetIterFirst(out current)) {
				yield break;
			}
		
			for (;;) {
				yield return current;
				
				TreeIter child;
				if (model.IterChildren(out child, current)) {
					current = child;
					continue;
				}
				
				while (!model.IterNext(ref current)) {
					if (!model.IterParent(out current, current)) {
						yield break;
					}
				}
			}
		}

	}
}
