// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Threading;

using Gtk;

namespace StratosClient.Core.Gui
{
	public partial class DialogoEspera : Gtk.Dialog
	{
		#region Campos
		private System.Action accion;
		private System.Exception excepcionArrojada;
		#endregion
		
		#region Constructores
		public DialogoEspera (System.Action accion)
			: base("Operación en curso", MainWindow.Instancia, DialogFlags.Modal)
		{
			this.Build ();
			
			this.ActionArea.Visible = false;
			this.HasSeparator = false;
			
			this.accion = accion;
		}
		#endregion
		
		#region Propiedades
		public System.Exception ExcepcionArrojada {
			get { return this.excepcionArrojada; }
		}
		#endregion
		
		#region Métodos
		public new int Run()
		{
			// Creo el thread de la acción y lo ejecuto
			Thread threadAccion = new Thread(delegate() {
				
				// La respusta por defecto es  Cancel. Si la ejecución falla
				// se devuelve este valor.
				ResponseType respuesta = ResponseType.Cancel;
				
				try {
					// Ejecuto la acción, y hago que el diálogo responda
					// ReponseType.Ok si se ejecutó correctamente
					
					this.accion();
					respuesta = ResponseType.Ok;
				}
				catch (System.Exception excepcion) {
					// Si hubo un error al ejecutar la acción, entonces
					// guardo la excepción y respondo con ResponseType.Cancel.
					this.excepcionArrojada = excepcion;
				}
				finally {
					Gtk.Application.Invoke(delegate {
						this.Respond(respuesta);
					});
				}
			});
			
			threadAccion.Start();
			
			GLib.Timeout.Add(250, delegate {
				this.barraProgreso.Pulse();
				
				return threadAccion.IsAlive;
			});
			
			// Muestro el diálogo y retorno
			return base.Run();
		}
		#endregion
	}
}
