// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Stratos.ModeloDto;

namespace StratosClient.Core.Gui.ImpresionBorrador
{
	public class CuentaCorrienteWrapper
	{
		#region Campos
		private ClienteWrapper clienteWrapper;
		
		private List<DocumentoCreditoWrapper> documentosCredito;
		private double importeFavor;
		
		private List<DocumentoDebitoWrapper> documentosDebito;
		private double importeContra;
		
		private double saldoTotal;
		#endregion
		
		#region Constructores
		public CuentaCorrienteWrapper (ClienteDto cliente,
		                               List<DocumentoCreditoDto> documentosCredito,
		                               double importeFavor,
		                               List<DocumentoDebitoDto> documentosDebito,
		                               double importeContra,
		                               double saldoTotal)
		{
			this.clienteWrapper = new ClienteWrapper(cliente);
			
			this.documentosCredito =
				(from docCred in documentosCredito
				 select new DocumentoCreditoWrapper(docCred))
					.ToList();
			this.importeFavor = importeFavor;
			
			this.documentosDebito =
				(from docDeb in documentosDebito
				 select new DocumentoDebitoWrapper(docDeb))
					.ToList();
			this.importeContra = importeContra;
			
			this.saldoTotal = saldoTotal;
		}
		#endregion
		
		#region Propiedades
		public ClienteWrapper Cliente {
			get { return this.clienteWrapper; }
		}
		
		public DateTime Fecha {
			get { return DateTime.Today; }
		}
		
		public double SaldoTotal {
			get { return saldoTotal; }
		}
		
		public List<DocumentoCreditoWrapper> DocumentosCredito {
			get { return this.documentosCredito; }
		}
		
		public double ImporteFavor {
			get { return importeFavor; }
		}
		
		public List<DocumentoDebitoWrapper> DocumentosDebito {
			get { return this.documentosDebito; }
		}
		
		public double ImporteContra {
			get { return importeContra; }
		}
		#endregion
	}
}
