// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Stratos.Common;
using Stratos.ModeloDto;

namespace StratosClient.Core.Gui.ImpresionBorrador
{
	/// <summary>
	/// El objetivo de esta clase es ser de wrapper para PresupuestoDto y así
	/// poder ser utilizada desde NVelocity, ya que éste no puede leer campos
	/// públicos, sino solo propiedades.
	/// </summary>
	public class FacturaWrapper : DocumentoGenericoWrapper
	{
		private FacturaGenericaDto factura;
		
		public FacturaWrapper(FacturaGenericaDto factura) : base (factura)
		{
			this.factura = factura;
		}
		
		/// <summary>
		/// Esta propiedad se utiliza al imprimir una factura.
		/// </summary>
		public string Comprobante {
			get {
				if (this.factura is NotaCreditoDto)
					return "NOTA CREDITO";
				
				return "FACTURA";
			}
		}
		
		public override string Tipo {
			get {
				if (this.factura.tipo == FacturaTipoVenta.FacturaA)
				    return "A";
				else
					return "B";
			}
		}
		
		public int Codigo {
			get {
				FacturaDto facturaDto = this.factura as FacturaDto;
				
				if (facturaDto == null)
					return 0;
				
				return facturaDto.codigo;
			}
		}
		
		public long Numero {
			get {
				FacturaDto facturaDto = this.factura as FacturaDto;
				
				if (facturaDto == null)
					return 0;
				
				return facturaDto.numero;
			}
		}
		
		public CondicionVentaWrapper CondicionVenta {
			get { return new CondicionVentaWrapper(this.factura.condVenta); }
		}
		
		/// <summary>
		/// Para el caso de que sea una nota de crédito.
		/// </summary>
		public string FacturaAsociada {
			get {
				NotaCreditoDto notaCreditoDto = this.factura as NotaCreditoDto;
				PreNotaCreditoDto preNotaCreditoDto = this.factura as PreNotaCreditoDto;
				
				/* Busco la factura asociada dependiendo del tipo de documento (NotaCredito 
				 * o PreNotaCredito. */
				FacturaDto facturaAsociada;
				
				if (notaCreditoDto != null)
					facturaAsociada = notaCreditoDto.facturaAsociada;
				else if (preNotaCreditoDto != null)
					facturaAsociada = preNotaCreditoDto.facturaAsociada;
				else
					return String.Empty;
				
				return facturaAsociada.codigo.ToString("D4") + " - " +
					facturaAsociada.numero.ToString("D8");
			}
		}
		
		public ClienteWrapper Cliente {
			get { return new ClienteWrapper(this.factura.cliente); }
		}
		
		public string Remitos {
			get { return this.factura.remitos; }
		}
		
		public double ImporteBruto {
			get { return this.factura.importeBruto; }
		}
		
		public double ImporteBonificacion {
			get { return this.factura.importeBonificacion; }
		}
		
		public double ImporteNeto {
			get { return this.factura.importeNeto; }
		}
		
		public double ImporteIva {
			get { return this.factura.importeIva; }
		}
		
		public double ImporteTotal {
			get { return this.factura.importeTotal; }
		}
		
		public List<ItemProductoWrapper> Items {
			get {
				List<ItemProductoWrapper> itemsProducto = new List<ItemProductoWrapper>();
				
				foreach (ItemProductoDto item in this.factura.items.OrderBy(it => it.orden))
					itemsProducto.Add(new ItemProductoWrapper(item));
				
				return itemsProducto;
			}
		}
	}
}
