// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Stratos.Common;
using Stratos.ModeloDto;

namespace StratosClient.Core.Gui.ImpresionBorrador
{
		/// <summary>
	/// El objetivo de esta clase es ser de wrapper para PresupuestoDto y así
	/// poder ser utilizada desde NVelocity, ya que éste no puede leer campos
	/// públicos, sino solo propiedades.
	/// </summary>
	public class PresupuestoWrapper
	{
		private PresupuestoDto presupuesto;
		
		public PresupuestoWrapper(PresupuestoDto presupuesto)
		{
			this.presupuesto = presupuesto;
		}
		
		public DateTime Fecha {
			get { return new DateTime(this.presupuesto.fechaTicks); }
		}
		
		public long Id {
			get { return this.presupuesto.id; }
		}
		
		public string RazonSocial {
			get { return this.presupuesto.razonSocial; }
		}
		
		public string Telefono {
			get { return this.presupuesto.telefono; }
		}
		
		public string CorreoElectronico {
			get { return this.presupuesto.correoElectronico; }
		}
		
		public string ClienteIva {
			get {
				return
					this.presupuesto.cliente == null ?
						StratosUtil.SplitCamelString(TipoResponsable.ConsumidorFinal.ToString())
				     	: StratosUtil.SplitCamelString(this.presupuesto.cliente.tipoRes.ToString());
			}
		}
		
		public string Observaciones {
			get {
				if (this.presupuesto.observaciones == null)
					return null;
				
				return this.presupuesto.observaciones.Replace(Environment.NewLine, "<br />");
			}
		}
		
		public double ImporteBruto {
			get { return this.presupuesto.importeBruto; }
		}
		
		public double ImporteBonificacion {
			get { return this.presupuesto.importeBonificacion; }
		}
		
		public double ImporteFinal {
			get { return this.presupuesto.importeFinal; }
		}
		
		public List<ItemProductoWrapper> Items {
			get {
				List<ItemProductoWrapper> itemsProducto = new List<ItemProductoWrapper>();
				
				foreach (ItemProductoDto item in this.presupuesto.items)
					itemsProducto.Add(new ItemProductoWrapper(item));
				
				return itemsProducto;
			}
		}
	}
}
