// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Stratos.Common;
using Stratos.ModeloDto;

namespace StratosClient.Core.Gui.ImpresionBorrador
{
	/// <summary>
	/// El objetivo de esta clase es ser de wrapper para PresupuestoDto y así
	/// poder ser utilizada desde NVelocity, ya que éste no puede leer campos
	/// públicos, sino solo propiedades.
	/// </summary>
	public class ReciboWrapper : DocumentoGenericoWrapper
	{
		private ReciboDto recibo;
		private ClienteWrapper clienteWrapper;
		
		public ReciboWrapper(ReciboDto recibo) : base(recibo)
		{
			this.recibo = recibo;
			this.clienteWrapper = new ClienteWrapper(this.recibo.cliente);
		}
		
		public string Concepto {
			get { return this.recibo.concepto; }
		}
		
		public ClienteWrapper Cliente {
			get { return this.clienteWrapper; }
		}
		
		public double Importe {
			get { return this.recibo.importe; }
		}
		
		public List<ItemReciboWrapper> Items {
			get {
				var lista =
					from item in this.recibo.items
					select new ItemReciboWrapper((ItemReciboDto) item);
				
				return lista.ToList();
			}
		}
	}
}
