// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Reflection;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace StratosClient.Core.Gui
{
	/// <summary>
	/// Clase con métodos útiles y de uso común para StratosClient.
	/// </summary>
	public static class StratosClientUtils
	{
		#region Consultas HQL
		/// <summary>
		/// Dado el nombre de una propiedad de búsqueda en HQL y una consulta, genera un criterio
		/// para una consulta HQL. Por ejemplo, se puede dar la propiedad "f.Cliente.RazonSocial"
		/// </summary>
		/// <param name="nombrePropiedad">
		/// A <see cref="System.String"/>
		/// </param>
		/// <param name="razonSocial">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public static List<string> GenerarCriteriosCadenaEnHQL(string nombrePropiedad, string consulta)
		{
			// Si la razón social es vacia, retorno un string vacio
			if (consulta.Trim() == String.Empty)
				return new List<string>();
			
			StringBuilder criterioRazonSocial = new StringBuilder();
			criterioRazonSocial.Append("(");
			
			// Paso la razón social a lower case
			string razonSocialLower = consulta.ToLower();
			
			// Separo el string en palabras
			string[] palabras = razonSocialLower.Split(' ');
			
			// Proceso la primer palabra
			criterioRazonSocial.Append("lower(" + nombrePropiedad + ")");
			criterioRazonSocial.Append(" like '%" + palabras[0] + "%'");
			
			for (int i=1; i<palabras.Length; i++) {
				criterioRazonSocial.Append(" and ");
				criterioRazonSocial.Append("lower(" + nombrePropiedad + ")");
				criterioRazonSocial.Append(" like '%" + palabras[i] + "%'");
			}
			
			criterioRazonSocial.Append(")");
			
			return new List<string> { criterioRazonSocial.ToString() };
		}
		
		/// <summary>
		/// Recibe una lista de criterios y arma la cláusula where de la consulta HQL.
		/// Por ejemplo, recibe { "f.Cliente = "VICENTIN", "f.Fecha > DateTime.Today" }
		/// y genera "where (f.Cliente = "VICENTIN) and (f.Fecha > DateTime.Today)"
		/// </summary>
		/// <param name="criterios">
		/// A <see cref="List"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public static string GenerarClausulaWhere(List<string> criterios)
		{
			StringBuilder clausulaWhere = new StringBuilder();
			
			// Ahora armo el criterio where completo
			if (criterios.Count > 0) {
				clausulaWhere.Append("where (" + criterios[0] + ")");
				
				for(int i=1; i<criterios.Count; i++)
					clausulaWhere.Append(" and (" + criterios[i] + ")");
			}
			
			return clausulaWhere.ToString();
		}
		
		/// <summary>
		/// Obtiene el directorio actual de la aplicación
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public static string DirectorioAplicacion()
		{
			return Path.GetDirectoryName(System.Reflection.Assembly
			                             .GetExecutingAssembly().Location);
		}
		#endregion
	}
}
