// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Configuration;

namespace StratosClient.Updater
{
	/// <summary>
	/// Clase encargada de gestionar funciones de configuración y guardar las claves
	/// del archivo de configuración.
	/// </summary>
	public static class Configuracion
	{
		private const string CLAVE_URL_ARCHIVO_CONFIGURACION_UPDATER = "urlArchivoConfiguracionUpdater";
		private const string CLAVE_URL_ARCHIVO_ACTUALIZACION = "urlArchivoActualizacion";
		private const string CLAVE_VERSION = "version";
		private const string CLAVE_ARCHIVOS_NO_ACTUALIZABLES = "archivosNoActualizables";

		public const string ASSEMBLY_VERSION = "StratosClient.Core.Gui.dll";
		public const string ARCHIVO_CONFIGURACION_UPDATER = "StratosClient.Updater.exe.config";
		public const string ASSEMBLY_PRINCIPAL_STRATOS_CLIENT = "StratosClient.Startup.exe";
		public static readonly string PATH_DIRECTORIO_TEMPORAL_STRATOS = Path.Combine(Path.GetTempPath(), "StratosClientUpdate");
		public static readonly string PATH_DIRECTORIO_BACKUP_STRATOS = Path.Combine(Path.GetTempPath(), "StratosClientBackup");
		public const string ARCHIVO_ACTUALIZACION = "StratosClient-bin-latest.zip";

		private static Configuration config;

		static Configuracion()
		{
			RecargarConfiguracion();
		}

		public static void CerrarConfiguracion()
		{
			config = null;
		}

		public static void RecargarConfiguracion()
		{
			config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
		}

		public static string UrlArchivoConfiguracionUpdater
		{
			get
			{
				if (config == null)
					throw new InvalidOperationException("Configuración no cargada");
				
				return config.AppSettings.Settings[CLAVE_URL_ARCHIVO_CONFIGURACION_UPDATER].Value;
			}
		}

		public static string UrlArchivoActualizacion
		{
			get
			{
				if (config == null)
					throw new InvalidOperationException("Configuración no cargada");
				
				return config.AppSettings.Settings[CLAVE_URL_ARCHIVO_ACTUALIZACION].Value;
			}
		}
		
		public static ICollection<string> ArchivosNoActualizables
		{
			get
			{
				if (config == null)
					throw new InvalidOperationException("Configuración no cargada");
				
				return config.AppSettings
					.Settings[CLAVE_ARCHIVOS_NO_ACTUALIZABLES]
					.Value.Split(',')
					.Select(archivo => archivo.Trim())
					.ToList();
			}
		}

		public static string VersionConfiguracion
		{
			get
			{
				if (config == null)
					throw new InvalidOperationException("Configuración no cargada");
				
				return config.AppSettings.Settings[CLAVE_VERSION].Value;
			}
		}
	}
}
