// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Net;

using Gtk;
using NLog;

using StratosClient.Updater;

namespace StratosClient.Updater.GUI
{
	public partial class MainWindow : Gtk.Window
	{
		private static readonly Logger logger = LogManager.GetCurrentClassLogger();
		
		private static MainWindow instancia;
		
		private ClientUpdater updater;
		
		private double[] progresoPorFases = new double[10];
		
		public MainWindow() : base(Gtk.WindowType.Toplevel)
		{
			Build();
			
			updater = ClientUpdater.Instancia;
			lblInfoActualizacion.Text =
				"Hay una nueva versión disponible: " +
					updater.VersionRepositorio.ToString(3);
			
			instancia = this;
		}
		
		public static MainWindow Instancia
		{
			get { return instancia; }
		}
	
		protected void OnDeleteEvent(object sender, DeleteEventArgs a)
		{
			logger.Info("El usuario cierra la ventana de actualización");
			
			SMessageDialog.MensajeInformacion("<b>No ha actualizado el sistema</b>.\n" +
				"La próxima vez que inicie StratosClient podrá intentarlo nuevamente");
			
			Application.Quit();
			a.RetVal = false;
		}
	
		protected virtual void OnActualizarClicked(object sender, System.EventArgs e)
		{
			// Arranco el proceso de actualización
			updater.ProgresoActualizacionHandler = GestionarProgresoActualizacion;
			updater.ComenzarActualizacion();
			
			this.btnActualizar.Sensitive = false;
			
			// Mantengo la GUI "responsive" mientras el updater trabaja.
			while (this.updater.Ocupado)
			{
				if (Gtk.Application.EventsPending())
					Gtk.Application.RunIteration();
			}
			
			// Cuando el updater termina, informo al usuario del éxito o fracaso.
			if (!updater.ActualizacionCorrecta)
				SMessageDialog.MensajeError("Hubo un error al actualizar Stratos. Se iniciará la versión anterior",
				                            updater.ExcepcionArrojada);
			else
				SMessageDialog.MensajeInformacion("Stratos ha sido actualizado correctamente a la versión " +
				                                  updater.VersionRepositorio);
			
			this.Destroy();
			Gtk.Application.Quit();
		}
		
		private void GestionarProgresoActualizacion(double progreso, int fase, string estado)
		{
			if (progreso == 0)
				return;
			
			// Actualizo el progreso de la fase
			progresoPorFases[fase] = progreso / 100.0f;
			
			double progresoTotal = progresoPorFases.Sum();
			if (progresoTotal > 1.0 || (1.0 - progresoTotal) < 0.01)
				progresoTotal = 1.0;
			
			Gtk.Application.Invoke(delegate
			{
				this.progressbar1.Text = estado;
				this.progressbar1.Fraction = progresoTotal;
			});
			
			while (Gtk.Application.EventsPending())
				Gtk.Application.RunIteration();
		}
	}
}
