// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text.RegularExpressions;

using Gtk;

namespace StratosClient.Widgets.Level1
{
	/// <summary>
	/// Clase que permite utilizar un Gtk.Entry con soporte de validación
	/// utilizando expresiones regulares.
	/// </summary>
	public abstract class BaseRegExEntry : Gtk.Entry, IRegExEntryVista
	{
		#region Campos
		private string tooltipOriginal;
		private string expresionRegular = "";
		private string mensajeError = "El valor del campo es inválido";
		
		public event System.Action TextoValidoIngresado;
		#endregion
		
		#region Propiedades
		public virtual bool TextoValido
		{
			get {
				Regex rx;
				
				try {
					rx = new Regex(this.ExpresionRegular);
				}
				catch {
					return false;
				}
				
				return rx.IsMatch(this.Text);
			}
		}
		
		public string TooltipOriginal {
			get { return this.tooltipOriginal; }
			set {
				this.tooltipOriginal = value;
				this.TooltipText = this.tooltipOriginal;
			}
		}
		
		public string ExpresionRegular {
			get { return this.expresionRegular; }
			set { this.expresionRegular = value; }
		}

		public virtual string MensajeError {
			get { return this.mensajeError; }
			set { this.mensajeError = value; }
		}
		
		public bool Sensible {
			set { CustomWidgetsUtils.WidgetEditable(this, value); }
		}
		#endregion
		
		#region Eventos
		protected override void OnChanged ()
		{
			base.OnChanged ();
			
			if (TextoValido) {
				this.SetearColorTextoValido();
				this.TooltipText = this.tooltipOriginal;
				
				// Evento
				this.OnTextoValidoIngresado();
			}
			else
				this.TooltipText = this.MensajeError;
		}
		
		protected override bool OnFocusOutEvent (Gdk.EventFocus evnt)
		{
			bool retval = base.OnFocusOutEvent(evnt);
			
			if (TextoValido)
				// Reseteo al color por defecto
				this.SetearColorTextoValido();
			else
				this.SetearColorTextoInvalido();
			
			return retval;
		}
		
		protected virtual void OnTextoValidoIngresado()
		{
			if (this.TextoValidoIngresado != null)
				this.TextoValidoIngresado();
		}
		#endregion
		
		#region Métodos
		protected void SetearColorTextoInvalido()
		{
			// Si el Entry no permite modifiaciones, no cambio el color de fondo
			if (!this.IsEditable)
				return;
			
			Gdk.Color color = Gdk.Color.Zero;
			Gdk.Color.Parse("#f43c3c", ref color);
			
			this.ModifyBase(StateType.Normal, color);
		}
		
		protected void SetearColorTextoValido()
		{
			// Si el Entry no permite modifiaciones, no cambio el color de fondo
			if (!this.IsEditable)
				return;
			
			this.ModifyBase(StateType.Normal);
		}
		#endregion
	}
}
