// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.Common;

namespace StratosClient.Widgets.Level1
{
	/// <summary>
	/// Clase que especializa un Gtk.Entry para mostrar importes
	/// </summary>
	[System.ComponentModel.ToolboxItem(true)]
	public class ImporteEntry : Gtk.Entry
	{	
		protected decimal importe;
		protected bool usarSignoPesos;
		protected bool mostrarImporteCero;
		
		public ImporteEntry()
		{
			this.Importe = 0.00m;
			this.usarSignoPesos = false;
			this.mostrarImporteCero = false;
			
			this.Xalign = 1;
			this.IsEditable = false;
			
			this.WidthChars = 12;
		}
		
		#region Propiedades
		/// <summary>
		/// Propiedad que al ser verdadera imprime el signo pesos al importe.
		/// </summary>
		public bool UsarSignoPesos {
			get { return this.usarSignoPesos; }
			set {
				this.usarSignoPesos = value;
				this.Importe = this.Importe;
			}
		}
		
		/// <summary>
		/// Indica si, frente a un importe igual a cero, se debe mostrar el valor
		/// cero o colocar un string vacío.
		/// </summary>
		public bool MostrarImporteCero {
			get { return this.mostrarImporteCero; }
			set {
				this.mostrarImporteCero = value;
				this.Importe = this.Importe;
			}
		}
		
		public decimal Importe {
			get { return this.importe; }
			set {
				this.importe = StratosUtil.RoundDecimal(value, 2);
				
				// Solo muestro el importe si es distinto a cero
				if (this.importe != 0.00m)
					this.Text = (this.usarSignoPesos ? "$ " : "") + this.importe.ToString("N2");
				else
					this.Text = this.mostrarImporteCero ? this.importe.ToString("N2") : String.Empty;
			}
		}
		
		public double ImporteDouble {
			get { return (double)this.Importe; }
			set { this.Importe = (decimal) value; }
		}
		#endregion
		
		#region Métodos públicos
		/// <summary>
		/// Vuelve a cero el importe
		/// </summary>
		public void Limpiar()
		{
			this.Importe = 0.00m;
		}
		#endregion
	}
}
