// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.Common;
using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;
using StratosClient.Widgets.Level1;

namespace StratosClient.Widgets.Level2
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class DatosFactura : Gtk.Bin
	{
		#region Campos
		public event System.Action<FacturaDto> FacturaCargada;
		public event System.Action FacturaQuitada;
		
		public event System.Action<FacturaDto> BotonBuscarFacturaPresionado;
		
		private FacturaDto facturaCargada;
//		private FacturaTipoVenta facturaTipo;
		
		protected ComboBoxEnumeracion<FacturaTipoVenta> cmbTipoFactura;
		#endregion
		
		#region Constructores
		public DatosFactura ()
		{
			this.Build ();
			
			this.txtCodigoFactura.Xalign = 0.5f;
			this.txtCodigoFactura.Activated += delegate {
				this.btnBuscarFactura.Click();
			};
			
			this.txtNumeroFactura.Xalign = 0.5f;
			this.txtNumeroFactura.Activated += delegate {
				this.btnBuscarFactura.Click();
			};
			
			this.txtImporteFactura.Xalign = 0.5f;
			this.txtImporteFactura.UsarSignoPesos = true;
			CustomWidgetsUtils.WidgetEditable(this.txtImporteFactura, false);
			
			// Por defecto vamos a buscar un tipo de Factura A
//			this.facturaTipo = FacturaTipoVenta.FacturaA;
			
			// Combo de tipo de factura
			this.cmbTipoFactura = new ComboBoxEnumeracion<FacturaTipoVenta>();
			this.cmbTipoFactura.Renderizador = delegate (FacturaTipoVenta facturaTipo) {
				return StratosUtil.SplitCamelString(facturaTipo.ToString());
			};
			this.cmbTipoFactura.Resetear();
			this.hbDatosBusqueda.PackStart(this.cmbTipoFactura, false, false, 0);
			this.hbDatosBusqueda.ReorderChild(this.cmbTipoFactura, 0);
			
			this.OnBtnLimpiar(null, null);
		}
		#endregion
		
		#region Propiedades
		public FacturaDto Factura {
			get {
				if (this.facturaCargada != null)
					return this.facturaCargada;
				
				return new FacturaDto();
			}
		}
		
		public IComboBoxGenericoVista<FacturaTipoVenta> ComboBoxFacturaTipoVenta {
			get { return this.cmbTipoFactura; }
		}
		
		public bool SoloLectura {
			set {
				CustomWidgetsUtils.WidgetEditable(this.cmbTipoFactura, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtCodigoFactura, !value);
				CustomWidgetsUtils.WidgetEditable(this.txtNumeroFactura, !value);
				
				this.btnBuscarFactura.Sensitive = !value;
				this.btnLimpiar.Sensitive = !value;
			}
		}
		#endregion
		
		#region Eventos
		protected virtual void OnBtnLimpiar (object sender, System.EventArgs e)
		{
			this.Vaciar();
		}
		
		protected virtual void OnBuscarFactura (object sender, System.EventArgs e)
		{
			// Verifico que el código y número de factura no sean vacías
			if (this.txtCodigoFactura.Text.Trim() == String.Empty ||
			    	this.txtNumeroFactura.Text.Trim() == String.Empty) {
				SMessageDialog.MensajeError("Falta especificar el código y número de factura");
				return;
			}
			
			// Verifico que el código sea válido
			if (!this.txtCodigoFactura.TextoValido) {
				SMessageDialog.MensajeError(this.txtCodigoFactura.MensajeError);
				return;
			}
			
			// Verifico que el número sea válido
			if (!this.txtNumeroFactura.TextoValido) {
				SMessageDialog.MensajeError(this.txtNumeroFactura.MensajeError);
				return;
			}
			
			// Realizo la búsqueda de la factura indicada
			ControladorFacturacionPrx cf =
				((ControladorFacturacionPrx)MainIce.GetControlador("ControladorFacturacion"));
			
			string consulta =
				@"from Factura f
					left join fetch f.Items
				where f.class = Factura
					and f.Valida = cast(true, boolean)
					and f.Codigo = " + this.txtCodigoFactura.Text + @"
					and f.Numero = " + this.txtNumeroFactura.Text + @"
					and f.Tipo = " + (int)this.cmbTipoFactura.ObjetoActivo;
			
			FacturaDto[] resultado = cf.BuscarFacturaHQL(consulta, null);
			
			// Verifico si hubo resultados
			if (resultado.Length == 0) {
				SMessageDialog
					.MensajeAdvertencia("No existe una factura del tipo, código y número especificados");
				return;
			}
			
			// Verifico si la factura recibida no está anulada
			if (resultado[0].anulada) {
				SMessageDialog
					.MensajeAdvertencia("La factura indicada es una factura anulada");
				return;
			}
			
			// Cargo la factura
			this.CargarFactura(resultado[0]);
			
			if (this.BotonBuscarFacturaPresionado != null)
				this.BotonBuscarFacturaPresionado(resultado[0]);
		}
		#endregion
		
		#region Métodos
		public void CargarFactura(FacturaDto factura)
		{
			this.CargarFactura(factura, true);
		}
		
		public void CargarFactura(FacturaDto factura, bool dispararEventoFacturaCargada)
		{
			this.facturaCargada = factura;
			
			this.cmbTipoFactura.ObjetoActivo = factura.tipo;
			this.txtCodigoFactura.Text = factura.codigo.ToString("D4");
			this.txtNumeroFactura.Text = factura.numero.ToString("D8");
			this.txtImporteFactura.Importe = (decimal)factura.importeTotal;
			
			CustomWidgetsUtils.WidgetEditable(this.cmbTipoFactura, false);
			CustomWidgetsUtils.WidgetEditable(this.txtCodigoFactura, false);
			CustomWidgetsUtils.WidgetEditable(this.txtNumeroFactura, false);
			
			this.btnBuscarFactura.Sensitive = false;
			
			if (dispararEventoFacturaCargada && this.FacturaCargada != null)
				this.FacturaCargada(factura);
		}
		
		public void Vaciar()
		{
			this.facturaCargada = null;
			
			this.cmbTipoFactura.ObjetoActivo = FacturaTipoVenta.FacturaA;
			this.txtCodigoFactura.Text = String.Empty;
			this.txtNumeroFactura.Text = String.Empty;
			this.txtImporteFactura.Text = String.Empty;
			
			CustomWidgetsUtils.WidgetEditable(this.cmbTipoFactura, true);
			CustomWidgetsUtils.WidgetEditable(this.txtCodigoFactura, true);
			CustomWidgetsUtils.WidgetEditable(this.txtNumeroFactura, true);
			
			this.SoloLectura = false;
			
			if (this.FacturaQuitada != null)
				this.FacturaQuitada();
		}
		#endregion
	}
}
