// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloDto;
using StratosClient.Widgets.Level1;

namespace StratosClient.Widgets.Level2
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class DomicilioEntry : Gtk.Bin, IDomicilioEntry
	{
		public event Action Cambiado;
		
		public DomicilioEntry ()
		{
			this.Build ();
			
			#region Suscripción a eventos de cambios
			this.txtDireccion.Changed += delegate { this.OnDomicilioCambiado(); };
			this.txtLocalidad.Changed += delegate { this.OnDomicilioCambiado(); };
			this.txtProvincia.Changed += delegate { this.OnDomicilioCambiado(); };
			this.rtxtCodigoPostal.Changed += delegate { this.OnDomicilioCambiado(); };
			#endregion
		}

		#region Propiedades
		
		// HACK: Se podría ver si se cambió algún campo y en caso de no haberse
		// cambiado se podría devolver el objeto que fue seteado sin crear uno
		// nuevo.
		public DomicilioDto Domicilio {
			get { return this.LeerDomicilio(); }			
			set { this.CargarDomicilio(value); }
		}
		
		public bool Editable {
			set {
				CustomWidgetsUtils.WidgetEditable(this.txtDireccion, value);
				CustomWidgetsUtils.WidgetEditable(this.txtLocalidad, value);
				CustomWidgetsUtils.WidgetEditable(this.rtxtCodigoPostal, value);
				CustomWidgetsUtils.WidgetEditable(this.txtProvincia, value);
			}
		}
		
		#endregion
		
		#region Métodos públicos
		
		/// <summary>
		/// Vacía el formulario
		/// </summary>
		public void Vaciar()
		{
			// creo un nuevo objeto domicilio vacío
			DomicilioDto domicilioVacio = new DomicilioDto();
			
			// Cargo el domicilio vacío
			this.CargarDomicilio(domicilioVacio);
		}
		
		/// <summary>
		/// Valida los datos ingresado en los campos correspondientes al domicilio.
		/// Devuelve 'true' si todos son válidos y 'false' en caso contrario.
		/// Si pasa esto ultimo, un mensaje de error se setea en el parametro out.
		/// </summary>
		/// <param name="mensajeError">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		public bool DatosValidos(out string mensajeError)
		{
			mensajeError = String.Empty;
			
			if (this.txtLocalidad.Text.Equals(string.Empty)) {
				mensajeError = "Localidad inválida. La localidad no puede ser vacía.";
				return false;
			}
			
			if (this.txtProvincia.Text.Equals(string.Empty)) {
				mensajeError = "Provincia inválida. La Provincia no puede ser vacía.";
				return false;
			}
			
			return true;
		}
		#endregion
		
		#region Métodos privados
		
		/// <summary>
		/// Lee el domicilio ingresado en el formulario y retorna un
		/// objeto DomicilioDto con dichos datos.
		/// </summary>
		/// <returns>
		/// A <see cref="DomicilioDto"/>
		/// </returns>
		protected DomicilioDto LeerDomicilio()
		{
			// Creo un domicilio vacío
				DomicilioDto domicilio = new DomicilioDto();
				
			// Agrego los datos del formulario
			domicilio.direccion = this.txtDireccion.Text.Trim();
			domicilio.localidad = this.txtLocalidad.Text.Trim();
			domicilio.codigoPostal = this.rtxtCodigoPostal.Text.Trim();
			domicilio.provincia = this.txtProvincia.Text == String.Empty ?
				ProvinciaDto.NoEspecificada : this.txtProvincia.ProvinciaSeteada;
			
			// Retorno el domicilio
				return domicilio;
		}
		
		/// <summary>
		/// Carga en el formulario los datos del domicilio pasado
		/// como parámetro.
		/// </summary>
		/// <param name="domicilio">
		/// A <see cref="DomicilioDto"/>
		/// </param>
		protected void CargarDomicilio(DomicilioDto domicilio)
		{
			if (domicilio == null)
				domicilio = new DomicilioDto();
			
			this.txtDireccion.Text =
				domicilio.direccion == null ? "" : domicilio.direccion;
			this.txtLocalidad.Text =
				domicilio.localidad == null ? "" : domicilio.localidad;
			this.rtxtCodigoPostal.Text =
				domicilio.codigoPostal == null ? "" : domicilio.codigoPostal;
			
			if (domicilio.direccion == null &&
			    domicilio.localidad == null &&
			    domicilio.codigoPostal == null)
				
				this.txtProvincia.Text = String.Empty;
			
			else
				this.txtProvincia.Text = domicilio.provincia.ToString();
		}
		
		
		protected void OnDomicilioCambiado()
		{
			if (this.Cambiado != null)
				this.Cambiado();
		}
		#endregion
	}
}
