// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Reflection;

using Gtk;

using Stratos.ModeloDto;

namespace StratosClient.Widgets.Level2
{
	public abstract class ComboCellRendererBase<T,TItem> : CellRendererCombo
		where TItem : ItemGenericoDto
	{
		#region Campos
		protected ListStore modelo;
		protected ComboBox comboBox;
		protected ListaInteractiva<TItem> treeView;
		#endregion
		
		#region Constructores
		public ComboCellRendererBase (ListaInteractiva<TItem> treeView) : base()
		{
			this.treeView = treeView;
			this.Editable = true;
			this.TextColumn = 0;
			
			this.modelo = new ListStore(typeof(string));
			T obj;
			
			// Por cada valor de la enumeración, lo agrego al modelo del
			// CellRendererCombo
			foreach (FieldInfo fi in typeof(T).GetFields(BindingFlags.Public |
			                                             BindingFlags.Static)) {
				
				// Agrego el valor al modelo
				obj = (T)fi.GetRawConstantValue();
				this.modelo.AppendValues(obj.ToString());
			}
			
			// Asigno el nuevo modelo con los valores seteados
			this.Model = modelo;
		}
		#endregion
		
		#region Propiedades
//		public T ObjetoActivo {
//			get {
//				TreeIter iter;
//	        	this.comboBox.GetActiveIter(out iter);
//				T obj = (T) this.comboBox.Model.GetValue(iter, 0);
//				
//				return obj;
//			}
//		}
		#endregion
		
		public override CellEditable StartEditing (Gdk.Event evnt, Gtk.Widget widget, string path, Gdk.Rectangle background_area, Gdk.Rectangle cell_area, Gtk.CellRendererState flags)
		{
			CellEditable cellEditable;
			
			// Pregunto al CellRenderer si está permitido comenzar la edición.
			if (this.ComenzarEdicion())
				cellEditable =
					base.StartEditing(evnt, widget, path, background_area, cell_area, flags);
			else
				cellEditable = null;
			
			return cellEditable;
		}
		
		protected override void OnEdited (string path, string new_text)
		{
			if (new_text == null) {
				base.OnEdited(path, new_text);
				return;
			}
			
			// Obtengo el TreePath y el TreeIter de la fila modificada
			TreePath treePath = new TreePath(path);
			
			TreeIter iter;
			treeView.Model.GetIter(out iter, treePath);
			
			/* Obtengo el objeto TItem de la fila modificada. */
			TItem item = (TItem) treeView.Model.GetValue(iter, 0);
			
			// Modifico realmente el item y lo notifico al TreeView padre
			this.ModificarItem(item, new_text);
			
			// Después de la edición le doy el foco al TreeView
			this.treeView.GrabFocus();
			
			this.treeView.ItemModificado(iter);
			
			base.OnEdited(path, new_text);
		}
		
		protected override void OnEditingCanceled ()
		{
			base.OnEditingCanceled();
			
			/* Al cancelarse la edición, le doy el foco al TreeView. */
			this.treeView.GrabFocus();
		}
		
		/// <summary>
		/// Este método se encarga de renderizar las opciones del ComboBox.
		/// </summary>
		/// <param name="layout">
		/// A <see cref="CellLayout"/>
		/// </param>
		/// <param name="renderer">
		/// A <see cref="CellRenderer"/>
		/// </param>
		/// <param name="model">
		/// A <see cref="TreeModel"/>
		/// </param>
		/// <param name="iter">
		/// A <see cref="TreeIter"/>
		/// </param>
		protected void RenderObjetoComboBox(CellLayout layout,
		                                    CellRenderer renderer,
		                                    TreeModel model,
		                                    TreeIter iter)
		{
			T obj = (T) model.GetValue(iter, 0);
			string valor = obj != null ? obj.ToString() : String.Empty;
			
			(renderer as CellRendererText).Text = valor;
		}
		
		/// <summary>
		/// Esta función realiza efectivamente las modificaciones en el Item de tipo T.
		/// Devuelve true o false para indicar si efectivamente estas modificaciones se llevaron
		/// a cabo. Esto es importante por cuestiones de performance para los casos en que una
		/// modificación al valor anterior requiere una petición al servidor.
		/// </summary>
		/// <param name="item">
		/// A <see cref="ItemDocumentoGenericoDto"/>
		/// </param>
		/// <param name="nuevoValor">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		protected abstract void ModificarItem(TItem item, string nuevoValor);
		
		protected abstract bool ComenzarEdicion();
	}
}
