// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Linq;

using Gtk;

using Stratos.ModeloDto;

namespace StratosClient.Widgets.Level2
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class ItemsInteractivo : Gtk.Bin
	{
		#region Campos
		private ListaInteractivaBase tvItems;
		#endregion
		
		#region Constructores
		public ItemsInteractivo(ListaInteractivaBase treeView)
		{
			this.Build();
			
			// Creo el objeto TvItemsProducto
			this.tvItems = treeView;
			this.gswItems.Add(this.tvItems);
			this.tvItems.ShowAll();
			
			this.Sensitive = true;
			
			// Suscripcion a eventos
			this.tvItems.ItemsCambiados += this.OnItemsCambiados;
		}
		#endregion
		
		#region Propiedades
		public ListaInteractivaBase TreeView {
			get { return this.tvItems; }
		}
		
		public new bool Sensitive {
			get { return this.Sensitive; }
			set {
				if (value)
					this.tvItems.Selection.Mode = SelectionMode.Browse;
				else
					this.tvItems.Selection.Mode = SelectionMode.None;
				
				this.hbBotonera.Sensitive = value;
			}
		}
		#endregion
		
		#region Manejadores de eventos comunes
		protected virtual void OnBtnQuitarItemClicked (object sender, System.EventArgs e)
		{
			this.tvItems.QuitarItemsSeleccionados();
		}
		#endregion
		
		#region Manejadores de eventos del TreeView
		protected virtual void OnItemsCambiados()
		{
			// Si ya alcancé el límite de items, cambio el color del label.
			string color = "black";
			
			if (this.tvItems.CantidadItems >= this.tvItems.LimiteItems)
				color = "red";
			
			// Armo el mensaje de estado y lo asigno al label
			StringBuilder estado = new StringBuilder();
			estado.AppendFormat("<b>Cantidad items:</b> {0} - <span foreground=\"{1}\">Límite items: {2}</span>",
			                    this.tvItems.CantidadItems,
			                    color,
			                    this.tvItems.LimiteItems);
			
			this.lblEstadoItems.Markup = estado.ToString();
		}
		#endregion
		
		#region Métodos públicos
		public void Vaciar()
		{
			this.Vaciar(true);
		}
		
		/// <summary>
		/// Vacia el TreeView pero da la posibilidad de indicar si se debe agregar el item vacío al
		/// final o no.
		/// </summary>
		/// <param name="agregarItemVacio">
		/// A <see cref="System.Boolean"/>
		/// </param>
		public void Vaciar(bool agregarItemVacio)
		{
			this.tvItems.Vaciar(agregarItemVacio);
		}
		
		public void AgregarItems(params ItemGenericoDto[] items)
		{
			this.tvItems.AgregarItems(items);
		}
		#endregion
	}
}
