// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

using Gtk;

using Stratos.Common;
using StratosClient.Core.Gui;
using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace StratosClient.Widgets.Level2
{
	public abstract class ListaItemsProducto : ListaInteractiva<ItemProductoDto>
	{
		#region Campos
		public TreeViewColumn tvcCodigoProducto;
		public TreeViewColumn tvcDescripcionProducto;
		public TreeViewColumn tvcObservaciones;
		public TreeViewColumn tvcPrecioUnitarioProducto;
		public TreeViewColumn tvcCantidad;
		public TreeViewColumn tvcUnidad;
		public TreeViewColumn tvcBonificacion;
		public TreeViewColumn tvcImporte;
		#endregion
		
		#region Constructores
		public ListaItemsProducto() : base()
		{
			this.HasTooltip = true;
			
			this.QueryTooltip += this.GestionarTooltip;
			
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Código";
			crtColumnaRender = new CodigoCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCodigoProducto);
			this.AppendColumn(tvcColumna);
			this.tvcCodigoProducto = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Descripción";
			crtColumnaRender = new DescripcionProductoCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.Expand = true;
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderDescripcionProducto);
			this.AppendColumn(tvcColumna);
			this.tvcDescripcionProducto = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Obs.";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.FontDesc.Weight = Pango.Weight.Bold;
			crtColumnaRender.Xalign = 0.5f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderNotasProducto);
			this.AppendColumn(tvcColumna);
			this.tvcObservaciones = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Precio unitario";
			crtColumnaRender = new PrecioUnitarioCellRenderer(this);
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderPrecioUnitario);
			this.AppendColumn(tvcColumna);
			this.tvcPrecioUnitarioProducto = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Bonif. (%)";
			crtColumnaRender = new BonificacionCellRenderer(this);
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderBonificacion);
			this.AppendColumn(tvcColumna);
			this.tvcBonificacion = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Cantidad";
			crtColumnaRender = new CantidadCellRenderer(this);
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCantidad);
			this.AppendColumn(tvcColumna);
			this.tvcCantidad = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Unidad";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderUnidad);
			this.AppendColumn(tvcColumna);
			this.tvcUnidad = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe ($)";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporte);
			this.AppendColumn(tvcColumna);
			this.tvcImporte = tvcColumna;
			#endregion
		}
		#endregion
		
		#region Funciones de renderizado del TreeView
		protected ItemProductoDto RenderProductoAbstracto(TreeViewColumn column,
		                                                CellRenderer cell,
		                                                TreeModel model,
		                                                TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			if (item == null) return null;
			
			CellRendererText cellText = (CellRendererText) cell;
			
			if (item.productoId > 0 && item.productoAbstracto)
				cellText.Underline = Pango.Underline.Single;
			else
				cellText.Underline = Pango.Underline.None;
			
			return item;
		}
		
		protected void RenderCodigoProducto(TreeViewColumn column,
		                                  CellRenderer cell,
		                                  TreeModel model,
		                                  TreeIter iter)
		{
			ItemProductoDto item =
				this.RenderProductoAbstracto(column, cell, model, iter);
			
			string valor = item != null ? item.productoCodigo : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}

		protected void RenderDescripcionProducto(TreeViewColumn column,
		                                       CellRenderer cell,
		                                       TreeModel model,
		                                       TreeIter iter)
		{
			ItemProductoDto item =
				this.RenderProductoAbstracto(column, cell, model, iter);
			
			string valor = item != null ? item.descripcion.Trim() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected void RenderNotasProducto(TreeViewColumn column,
		                                   CellRenderer cell,
		                                   TreeModel model,
		                                   TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			
			string notas = (item != null && item.productoNotas != null) ?
				item.productoNotas : String.Empty;
			
			CellRendererText cellText = cell as CellRendererText;
			
			if (notas.Trim() == String.Empty) {
				cellText.Background = null;
				cellText.Text = String.Empty;
			}
			else {
				cellText.Background = "lightgreen";
				cellText.Text = "P";
			}
		}
		
		protected void RenderPrecioUnitario(TreeViewColumn column,
		                                  CellRenderer cell,
		                                  TreeModel model,
		                                  TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			string valor = item != null ? item.precioUnitario.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected void RenderCantidad(TreeViewColumn column,
		                            CellRenderer cell,
		                            TreeModel model,
		                            TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			string valor = item != null ? item.cantidad.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected void RenderBonificacion(TreeViewColumn column,
		                            	CellRenderer cell,
		                            	TreeModel model,
		                            	TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			string valor = item != null ? item.porcentajeBonificacion.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected void RenderUnidad(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			string valor = item != null ? StratosUtil.SplitCamelString(item.productoUnidad.ToString()) : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}

		protected void RenderImporte(TreeViewColumn column,
		                           CellRenderer cell,
		                           TreeModel model,
		                           TreeIter iter)
		{
			ItemProductoDto item = (ItemProductoDto) model.GetValue(iter, 0);
			string valor = item != null ? item.importe.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
		
		#region Eventos
		private bool ObtenerDatosEventoTooltip(object o, QueryTooltipArgs args,
		                                       out TreeView treeView,
		                                       out TreePath path, out TreeIter iter,
		                                       out TreeViewColumn columna)
		{
			iter = TreeIter.Zero;
			
			treeView = (TreeView) o;
			
			// Obtengo el TreePath en la posición donde está el mouse
			treeView.GetPathAtPos(args.X, args.Y, out path, out columna);
			
			if (path == null)
				return false;
			
			if (!path.Prev())
				return false;
			
			// Si el path no es nulo y se pudo obtener, obtento el TreIter
			// a partir del mismo.
			if (path == null || !treeView.Model.GetIter(out iter, path))
				return false;
			
			return true;
		}
		
		private void GestionarTooltip(object o, QueryTooltipArgs args)
		{
			TreeView treeView;
			TreePath path;
			TreeIter iter;
			TreeViewColumn columna;
			bool tooltipProductoGenericoAsignado = false;
			bool tooltipNotasProductoAsignado = false;
			
			if (!this.ObtenerDatosEventoTooltip(o, args, out treeView, out path, out iter, out columna)) {
				args.RetVal = false;
				return;
			}
			
			ItemProductoDto item = (ItemProductoDto) treeView.Model.GetValue(iter, 0);
			
			// Si el producto no es nulo y es abstracto, muestro el tooltip correspondiente.
			if (item.productoId > 0 && item.productoAbstracto &&
			    	(columna == this.tvcCodigoProducto || columna == this.tvcDescripcionProducto)) {
				
				args.Tooltip.Markup = "Este es un <b>producto genérico</b>";
				
				treeView.SetTooltipCell(args.Tooltip, path, columna, null);
				
				tooltipProductoGenericoAsignado = true;
			}
			
			// Muestro un tooltip en la columna de observaciones si el producto
			// tiene notas.
			if (item.productoId > 0 && item.productoNotas != null &&
			    item.productoNotas.Trim() != String.Empty && columna == this.tvcObservaciones) {
				
				args.Tooltip.Markup = "<b>Observaciones del producto:</b>\n" + item.productoNotas;
				
				treeView.SetTooltipCell(args.Tooltip, path, columna, null);
				
				tooltipNotasProductoAsignado = true;
			}
			
			args.RetVal = tooltipNotasProductoAsignado || tooltipProductoGenericoAsignado;
		}
		#endregion
		
		#region Otros métodos
		#region Carga producto
		protected abstract bool PermitirCargaProducto(ProductoDto producto, out string mensajeCargaProductoNoPermitida);
		
		/// <summary>
		/// Carga un ProductoDto en el item actual. Envia el tipo de factura por defecto
		/// para calcular los importes.
		/// </summary>
		/// <param name="producto">
		/// A <see cref="ProductoDto"/>
		/// </param>
		internal void CargarProducto(ProductoDto producto)
		{
			string mensajeCargaProductoNoPermitida;
			
			if (!this.PermitirCargaProducto(producto, out mensajeCargaProductoNoPermitida)) {
				SMessageDialog.MensajeAdvertencia(mensajeCargaProductoNoPermitida);
				return;
			}
			
			// Obtengo el item de producto
			TreeIter iter;
			this.Selection.GetSelected(out iter);
			
			ItemProductoDto item = (ItemProductoDto) this.Model.GetValue(iter, 0);
			
			// Cargo los datos básicos del producto en el item
			item.productoId = producto.id;
			item.productoCodigo = producto.codigo;
			item.descripcion = producto.descripcion;
			item.productoNotas = producto.notas;
			
			if (producto.productoAbstracto)
				item.productoDescripcion = producto.descripcion;
			
			item.productoAbstracto = producto.productoAbstracto;
			item.productoImporteSinIva = producto.importeSinIva;
			item.productoImporteVenta = producto.importeVenta;
			item.productoUnidad = producto.unidad;
			item.productoRubroId = producto.rubro != null ? producto.rubro.codigo : 0;
			
			// Llamo al método CargarDatosExtraProducto, por si alguna subclase
			// implementó el método para cargar datos extra en el item
			this.CargarDatosExtraProducto(producto, item);
			
			// Una vez realizado los cambios, llamo a ItemModificado
			this.ItemModificado(iter);
		}
		
		protected abstract void CargarDatosExtraProducto(ProductoDto producto, ItemProductoDto item);
		#endregion
		
		/// <summary>
		/// Retorna la suma de los importes de los items
		/// </summary>
		/// <returns>
		/// A <see cref="System.Int32"/>
		/// </returns>
		public decimal SumaImportesItemes()
		{
			decimal sumaItems = 0.00m;
			
			foreach (ItemProductoDto item in
			         this.Model.TraverseColumn<ItemProductoDto>(0))
				sumaItems += Convert.ToDecimal(item.importe);
			
			return sumaItems;
		}
		
		/// <summary>
		/// Devuelve true si el producto indicado en los parámetros, ya sea por su código o
		/// por su descripción, ya está cargado en otro item de la lista.
		/// </summary>
		/// <param name="itemActual">
		/// A <see cref="ItemProductoDto"/>
		/// </param>
		/// <param name="codigoProducto">
		/// A <see cref="System.String"/>
		/// </param>
		/// <param name="descripcionProducto">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		public bool ProductoEstaCargado(string codigoProducto,
		                                string descripcionProducto)
		{
			// Paso la descripción a minúsculas
			string codigoProductoLower = codigoProducto.ToLower();
			string descripcionProductoLower = descripcionProducto.ToLower();
			
			ItemProductoDto itemActual = (ItemProductoDto) this.ObtenerItemSeleccionado();
			
			// Realizo la búsqueda
			int itemsCoincidentes = this.Model.TraverseColumn<ItemProductoDto>(0)
				.Count(item => !item.Equals(itemActual) && !item.productoAbstracto &&
				       (item.productoCodigo.ToLower().Equals(codigoProductoLower) ||
				        item.descripcion.ToLower().Equals(descripcionProductoLower)));
			
			return itemsCoincidentes > 0;
		}
		#endregion
		
		#region ListaInterativa - Implementación
		protected override TreeViewColumn ColumnaEdicionSobreItemVacio {
			get { return this.tvcCodigoProducto; }
		}
		
		protected override bool ItemVacioGenericoInterno(ItemProductoDto item)
		{
			if (item.cantidad == 0.0 &&
			    	String.IsNullOrEmpty(item.productoCodigo) &&
			    	String.IsNullOrEmpty(item.descripcion))
				return true;
			
			return false;
		}
		
		internal override TreeIter AgregarItemVacio()
		{
			ItemProductoDto nuevoItem = new ItemProductoDto();
			nuevoItem.descripcion = String.Empty;
			nuevoItem.productoCodigo = String.Empty;
			nuevoItem.productoDescripcion = String.Empty;
			nuevoItem.productoAbstracto = false;
			nuevoItem.productoId = 0;
			nuevoItem.productoImporteSinIva = 0.00;
			nuevoItem.productoImporteVenta = 0.00;
			nuevoItem.productoUnidad = UnidadProducto.Unidad;
			
			ListStore modelo = (ListStore) this.Model;
			
			return modelo.AppendValues(nuevoItem);
		}
		
		internal override void ItemModificadoListener(ItemProductoDto item)
		{
			// Recalculo el importe del item
			double nuevoPrecioUnitario =
				item.precioUnitario -
					(item.precioUnitario * (item.porcentajeBonificacion / 100.0));
			
			double nuevoImporte =
				nuevoPrecioUnitario * item.cantidad;
			
			item.importe = StratosUtil.RoundDouble(nuevoImporte, 2);
		}
		#endregion
	}
}
