// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.Common;
using Stratos.ModeloDto;

namespace StratosClient.Widgets.Level2
{
	/// <summary>
	/// Lista simple de clientes.
	/// </summary>
	[System.ComponentModel.ToolboxItem(true)]
	public class TvClientes : ListaElementos<ClienteDto>
	{
		#region Campos
		public TreeViewColumn tvcCodigo;
		public TreeViewColumn tvcRazonSocial;
		public TreeViewColumn tvcTipoResponsable;
		public TreeViewColumn tvcCuit;
		
		#endregion
		
		public TvClientes() : base()
		{
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Código";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCodigo);
			this.AppendColumn(tvcColumna);
			this.tvcCodigo = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Razón social";
			tvcColumna.Expand = true;
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderRazonSocial);
			this.AppendColumn(tvcColumna);
			this.tvcRazonSocial = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Tipo Responsable";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderTipoResponsable);
			this.AppendColumn(tvcColumna);
			this.tvcTipoResponsable = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "CUIT";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCuit);
			this.AppendColumn(tvcColumna);
			this.tvcCuit = tvcColumna;
			#endregion
		}
		
		#region Propiedades
		public ClienteDto[] ClientesSeleccionados {
			get { return this.ObtenerSeleccionadosGenericos(); }
		}
		
		public bool Vacio {
			get {
				if (this.Model.IterNChildren() > 0)
					return false;
				
				return true;
			}
		}
		#endregion
		
		#region Métodos de renderizado
		private void RenderCodigo(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			ClienteDto cliente = (ClienteDto) model.GetValue(iter, 0);
			string valor = cliente != null ? cliente.codigo.ToString() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}

		private void RenderRazonSocial(TreeViewColumn column,
		                               CellRenderer cell,
		                               TreeModel model,
		                               TreeIter iter)
		{
			ClienteDto cliente = (ClienteDto) model.GetValue(iter, 0);
			string valor = cliente != null ? cliente.razonSocial : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderTipoResponsable(TreeViewColumn column,
		                                   CellRenderer cell,
		                                   TreeModel model,
		                                   TreeIter iter)
		{
			ClienteDto cliente = (ClienteDto) model.GetValue(iter, 0);
			string valor = cliente != null ?
				StratosUtil.SplitCamelString(cliente.tipoRes.ToString()) :
				String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderCuit(TreeViewColumn column,
		                        CellRenderer cell,
		                        TreeModel model,
		                        TreeIter iter)
		{
			ClienteDto cliente = (ClienteDto) model.GetValue(iter, 0);
			string valor = cliente != null ? cliente.cuit : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
		
		#region Otros métodos	
		protected override bool Iguales (ClienteDto c1, ClienteDto c2)
		{
			return (c1.codigo == c2.codigo);
		}
		#endregion
	}
}
