// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace StratosClient.Widgets.Level2
{
	public class TvFacturasCompra : TvDocumentosGenericos<FacturaCompraDto>
	{
		#region Campos
		public TreeViewColumn tvcFechaImputacion;
		public TreeViewColumn tvcCodigo;
		public TreeViewColumn tvcNumero;
		public TreeViewColumn tvcRazonSocial;
		public TreeViewColumn tvcImporteTotal;
		#endregion
		
		public TvFacturasCompra ()
		{
			#region Columnas
			TreeViewColumn tvcColumna;
			CellRendererText crtColumnaRender;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Imputación";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderFechaImputacion);
			this.InsertColumn(tvcColumna, 1);
			this.tvcFechaImputacion = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Código";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderCodigo);
			this.InsertColumn(tvcColumna, 3);
			this.tvcNumero = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Número";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderNumero);
			this.InsertColumn(tvcColumna, 4);
			this.tvcNumero = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Proveedor";
			crtColumnaRender = new CellRendererText();
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderRazonSocial);
			this.InsertColumn(tvcColumna, 5);
			this.tvcRazonSocial = tvcColumna;
			
			tvcColumna = new TreeViewColumn();
			tvcColumna.Title = "Importe total";
			crtColumnaRender = new CellRendererText();
			crtColumnaRender.Xalign = 1.0f;
			tvcColumna.PackStart(crtColumnaRender, true);
			tvcColumna.SetCellDataFunc(crtColumnaRender, this.RenderImporteTotal);
			this.InsertColumn(tvcColumna, 6);
			this.tvcImporteTotal = tvcColumna;
			
			this.tvcObservaciones.Expand = true;
			#endregion
		}
		
		#region Métodos de renderizado
		
		/// <summary>
		/// Este método realiza operaciones de renderizado básicas para todas las columnas.
		/// Por ejemplo, si una FacturaDto es histórica, cambia el color de fondo de la fila.
		/// </summary>
		/// <param name="column">
		/// A <see cref="TreeViewColumn"/>
		/// </param>
		/// <param name="cell">
		/// A <see cref="CellRenderer"/>
		/// </param>
		/// <param name="model">
		/// A <see cref="TreeModel"/>
		/// </param>
		/// <param name="iter">
		/// A <see cref="TreeIter"/>
		/// </param>
		/// <returns>
		/// A <see cref="DocumentoGenericoDto"/>
		/// </returns>
		protected override FacturaCompraDto RenderBase(TreeViewColumn column,
		                                                   CellRenderer cell,
		                                                   TreeModel model,
		                                                   TreeIter iter)
		{
			FacturaCompraDto factura = (FacturaCompraDto) model.GetValue(iter, 0);
			
			if (factura == null) return null;
			
			// Si la factura es histórica...
			if (factura.historica) {
				CellRendererText cellText = cell as CellRendererText;
				
				cell.CellBackground = "lightgray";
			}
			
			// Si la factura no es histórica, vuelvo a poner
			// todos los valores por defecto
			else {
				CellRendererText cellText = cell as CellRendererText;
				
				cell.CellBackground = null;
				
				if (cellText != null)
					cellText.Strikethrough = false;
			}
			
			return factura;
		}
		
		private void RenderFechaImputacion(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			FacturaCompraDto facturaCompra = this.RenderBase(column, cell, model, iter);

			if (facturaCompra != null) {
				DateTime fechaImputacion = new DateTime(facturaCompra.fechaImputacionTicks);
				(cell as CellRendererText).Text = fechaImputacion.ToShortDateString();
			}				
			else
				(cell as CellRendererText).Text = String.Empty;
		}
		
		private void RenderCodigo(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			FacturaCompraDto facturaCompra = this.RenderBase(column, cell, model, iter);
			
			string valor = facturaCompra != null ? facturaCompra.codigo.ToString("D4") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderNumero(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			FacturaCompraDto facturaCompra = this.RenderBase(column, cell, model, iter);
			
			string valor = facturaCompra != null ? facturaCompra.numero.ToString("D8") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderRazonSocial(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			FacturaCompraDto facturaCompra = this.RenderBase(column, cell, model, iter);
			
			string valor = (facturaCompra != null && facturaCompra.proveedor != null)
				? facturaCompra.proveedor.razonSocial.ToString() : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		
		private void RenderImporteTotal(TreeViewColumn column,
		                          CellRenderer cell,
		                          TreeModel model,
		                          TreeIter iter)
		{
			FacturaCompraDto facturaCompra = this.RenderBase(column, cell, model, iter);
			
			string valor = facturaCompra != null ? facturaCompra.importeTotal.ToString("N2") : String.Empty;
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
		
		#region Otros metodos
		protected override bool Iguales (FacturaCompraDto t1, FacturaCompraDto t2)
		{
			if (t1.id == t2.id)
				return true;
			return false;
		}
		#endregion
	}
}
