// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Gtk;

using Stratos.ModeloDto;

namespace StratosClient.Widgets.Level2
{
	/// <summary>
	/// Clase para mostrar listados de elementos tipo PreFacturaDto.
	/// </summary>
	public class TvPreFacturas : TvDocumentosVenta<PreFacturaDto>
	{
		#region Constructores
		public TvPreFacturas() : base()
		{
		}
		#endregion
		
		#region Métodos renderizadores
		protected override void RenderTipoDocumento (TreeViewColumn column,
		                                             CellRenderer cell,
		                                             TreeModel model,
		                                             TreeIter iter)
		{
			PreFacturaDto factura = this.RenderBase(column, cell, model, iter);
			string valor = factura != null ? "Pre Factura" : String.Empty;
			
			// Me fijo si no se trata de una NotaCredito
			if (factura is PreNotaCreditoDto)
				valor = "Pre Nota de crédito";
			
			// Me fijo en el tipo de Factura
			if (factura != null) {
				if (factura.tipo == FacturaTipoVenta.FacturaA)
					valor += " A";
				else
					valor += " B";
			}
			
			(cell as CellRendererText).Text = valor;
		}
		
		protected override void RenderObservaciones (TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
		{
			PreFacturaDto preFactura = this.RenderBase(column, cell, model, iter);
			
			// Si no se trata de una PreNotaCredito, entonces salgo
			PreNotaCreditoDto preNotaCredito = preFactura as PreNotaCreditoDto;
			if (preNotaCredito == null) {
				base.RenderObservaciones(column, cell, model, iter);
				return;
			}
			
			// Si es una NotaCredito, entonces pongo en las observaciones la factura asociada
			FacturaDto facturaAsociada = preNotaCredito.facturaAsociada;
			string valor = String.Empty;
			
			// Puede que la nota de crédito no tenga una factura asociada, por
			// las facturas de migración del sistema viejo.
			if (facturaAsociada != null)
				valor = "Factura asociada: " + facturaAsociada.codigo.ToString("D4") + "-" +
					facturaAsociada.numero.ToString();
			
			(cell as CellRendererText).Text = valor;
		}
		#endregion
	}
}
