// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;

using StratosClient.Core.Gui;

namespace StratosClient.Widgets.Level3
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class NavegadorElementos : Gtk.Bin, INavegadorElementosVista
	{
		#region Campos
		protected System.Func<object, bool> cargador;
		protected System.Func<IListaNavegable> proveedorListaNavegable;
		
		/// <summary>
		/// Este campo indica que al pedirse el siguiente elemento significa que
		/// se quiere el actual. Por ej: cuando se imprime una PreFactura esta se
		/// elimina y se pasa a Factura. El TreePath actual sería ahora el siguiente.
		/// Por lo tanto si se pide el siguiente TreePath se pasará al siguiente
		/// del siguiente.
		/// </summary>
		protected bool siguienteEsActual;
		#endregion
		
		public NavegadorElementos()
		{
			this.Build();
			
			this.siguienteEsActual = false;
			
			// Seteo la imagen del botón 'Siguiente' a la derecha,
			// por defecto tiene la imágen a la izquierda.
			this.btnSiguiente.ImagePosition = Gtk.PositionType.Left;
			
			this.ShowAll();
		}
		
		#region Propiedades
		public System.Func<object, bool> Cargador {
			set { this.cargador = value; }
		}
		
		public System.Func<IListaNavegable> ProveedorListaNavegable {
			get { return this.proveedorListaNavegable; }
			set { this.proveedorListaNavegable = value; }
		}
		
		public bool SiguienteEsActual {
			set { this.siguienteEsActual = value; }
		}
		#endregion
		
		#region Eventos
		protected virtual void OnBtnAnteriorClicked (object sender, System.EventArgs e)
		{
			// Obtengo el servicio de navegación de la lista navegable
			ServicioNavegadorElementos servicio =
				((IListaNavegable)this.proveedorListaNavegable()).ServicioNavegador;
		
			// Obtengo el siguiente objeto de la lista
			object objetoAnterior = servicio.ElementoAnterior();
			
			// Si no hay mas elementos antes de este, aviso al usuario y retorno
			if (objetoAnterior == null) {
				SMessageDialog.MensajeAdvertencia("Esta visualizando el primer objeto de la lista");
				return;
			}
			
			// Ejecuto el cargador y segun se haya cargado o no efectivamente el objeto
			// en el visualizador, se lo notifico al servicio de navegacion de la lista navegable
			if(this.cargador(objetoAnterior))
				servicio.ElementoCargadoCorrectamente();
			else
				servicio.ElementoNoCargado();
			
			// Siempre dejo el navegador activado
			this.Sensitive = true;
		}
		
		protected virtual void OnBtnSiguienteClicked (object sender, System.EventArgs e)
		{
			// Obtengo el servicio de navegación de la lista navegable
			ServicioNavegadorElementos servicio =
				((IListaNavegable)this.proveedorListaNavegable()).ServicioNavegador;
			
			// Si no se ha seteado siguienteEsActual, obtengo el siguiente elemento
			// de la lista.
			object objetoSiguiente = null;
			
			if (this.siguienteEsActual)
				objetoSiguiente = servicio.ElementoActual;
			else
				objetoSiguiente = servicio.ElementoSiguiente();
			
			// Si no hay mas elementos despues de este, aviso al usuario y retorno
			if (objetoSiguiente == null) {
				SMessageDialog.MensajeAdvertencia("Esta visualizando el ultimo objeto de la lista");
				return;
			}
			
			// Ejecuto el cargador y segun se haya cargado o no efectivamente el objeto
			// en el visualizador, se lo notifico al servicio de navegacion de la lista navegable
			if(this.cargador(objetoSiguiente)) {
				servicio.ElementoCargadoCorrectamente();
				this.siguienteEsActual = false;
			}
			else
				servicio.ElementoNoCargado();
			
			// Siempre dejo el navegador activado
			this.Sensitive = true;
		}
		#endregion		
	}
}
