// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using Ice;
using log4net;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Queries;

using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Controladores;
using StratosServer.Core;
using Stratos.Common.Excepciones;

namespace StratosServer.Compras
{
	public class ControladorComprasI : ControladorComprasDisp_
	{
		public override Ice.Object[] BuscarHQL (string consulta, ConsultaPaginable consPaginable, Current current__)
		{
			FacturaCompraDto[] resultados =
				StratosServerUtil
					.BuscarHQLGenerico<FacturaCompraDto,FacturaCompra>(consulta, consPaginable);
			
			return resultados;
		}

		public override Ice.Object[] BuscarQueryObjects (Stratos.QueryObjects.QueryObject queryObject, ConsultaPaginable consPaginable, Current current__)
		{
			throw new System.NotImplementedException();
		}
		
		public override Ice.Object Refrescar (Ice.Object objeto, Current current__)
		{
			FacturaCompraDto facturaCompraDto = objeto as FacturaCompraDto;
			
			return StratosServerUtil
				.Refrescar<FacturaCompra,FacturaCompraDto>(facturaCompraDto, facturaCompraDto.id);
		}
		
		public override Ice.Object Guardar (Ice.Object facturaCompra, Current current__)
		{
			// Utilizo una transacción para realizar todas las operaciones en la base de datos
			using (TransactionScope transactionScope = new TransactionScope(TransactionMode.Inherits)) {
				
				FacturaCompraDto facturaCompraDto =
					StratosServerUtil.TipoCorrecto<FacturaCompraDto>(facturaCompra);
				
				// Chequeo que la versión sea correcta
				StratosServerUtil
					.VersionCorrecta<FacturaCompra>(facturaCompraDto.id,
					                                facturaCompraDto.version,
					                                transactionScope);
				
				// Convierto el dto en un objeto persistente
				FacturaCompra facturaCompraNueva =
					ConvertidorDto.FromDto<FacturaCompraDto,FacturaCompra>(facturaCompraDto);
				
				if (facturaCompraDto.id <= 0) {
					StratosServerUtil.CrearObjeto<FacturaCompra>(facturaCompraNueva, transactionScope);
				}
				else {
					StratosServerUtil.ActualizarObjeto<FacturaCompra>(facturaCompraNueva, transactionScope);
				}
				
				// Retorno una versión Dto actualizada.
				return ConvertidorDto.ToDto(facturaCompraNueva);
			}
		}
		
		public override Ice.Object Modificar (Ice.Object objeto, Current current__)
		{
			/* FIXME: En este controlador el método Guardar crea nuevos objetos o los
			 * modifica. Hace ambas operaciones. Estaría bueno que estén separadas. */
			return this.Guardar(objeto, current__);
		}
		
		public override void EliminarFacturas (long[] ids, Current current__)
		{
			// Si la lista de ids está vacía, retorno.
			if (ids.Length <= 0)
				return;
			
			using (TransactionScope transactionScope = new TransactionScope(TransactionMode.Inherits)) {
				
				// Creo una consulta HQL para obtener todas las facturas cuyo id esté dentro
				// de la lista de ids pasada como parámetro
				string consultaHql = "from FacturaCompra f where f.id in (:ids)";
				
				SimpleQuery<FacturaCompra> consulta = new SimpleQuery<FacturaCompra>(consultaHql);
				consulta.SetParameterList("ids", ids);
				
				// Realizo la consulta
				FacturaCompra[] facturasEncontradas = consulta.Execute();
				
				// De las facturas encontradas, selecciono los ids de
				// las facturas que no son históricas
				var idsFacturasNoHistoricas =
					from f in facturasEncontradas
						where f.Historica == false
						select f.Id;
				
				// Borro las facturas
				FacturaCompra.DeleteAll(idsFacturasNoHistoricas);
			}
		}
	}
}
