// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;

using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using Stratos.Common;

namespace StratosServer.Reportes
{
	public class FacturaIvaComprasWrapper
	{
		protected FacturaCompra facturaAsociada;
			
		public FacturaIvaComprasWrapper(FacturaCompra factura)
		{
			this.facturaAsociada = factura;
		}
		
		#region Propiedades
		#region Importes
		public decimal ImporteGravado {
			get { return this.facturaAsociada.ImporteGravado; }
		}
		
		public decimal ImporteExento {
			get { return this.facturaAsociada.ImporteExento; }
		}
		
		public decimal ImporteIva {
			get { return this.facturaAsociada.ImporteIva; }
		}
		
		public decimal ImportePercIB {
			get { return this.facturaAsociada.PercepcionIB; }
		}
		
		public decimal ImportePercIva {
			get { return this.facturaAsociada.PercepcionIva; }
		}
		
		public decimal ImporteImpComb {
			get { return this.facturaAsociada.ImpuestoCombustible; }
		}
		
		public decimal ImporteTotal {
			get { return this.facturaAsociada.ImporteTotal; }
		}
		#endregion
		
		#region Propiedades utilizadas para el reporte		
		public string Fecha {
			get { return this.facturaAsociada.Fecha.ToShortDateString(); }
		}
		
		/// <summary>
		/// Devuelve un string con un resumen del documento. El formato del resumen
		/// es, por ejemplo: "FA A  0004  00007634", para una Factura A cuyo código
		/// es 0004 y su número es 7634. 
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public string ResumenFactura {
			get {
				StringBuilder resumen = new StringBuilder();
				
				// Tipo de documento
				resumen.Append(this.facturaAsociada.TipoAbreviado);
				
				// Tipo de Factura
				if (this.facturaAsociada.Tipo == FacturaTipoCompra.FacturaA)
					resumen.Append(" A  ");
				else
					resumen.Append(" C  ");
				
				// Código en formato 'XXXX'
				resumen.Append(this.facturaAsociada.Codigo.ToString("D4") + "  ");
				
				// Número en formato 'XXXXXXXX'
				resumen.Append(this.facturaAsociada.Numero.ToString("D8"));
				
				return resumen.ToString();
			}
		}
		
		public string RazonSocial {
			get {
				return this.facturaAsociada.ProveedorRazonSocial;
			}
		}
		
		public string Iva {
			get {
				switch (this.facturaAsociada.ProveedorTipoResponsable) {
					case TipoResponsable.ResponsableInscripto:
						return "I";
					case TipoResponsable.ConsumidorFinal:
						return "C";
					case TipoResponsable.Exento:
						return "E";
					case TipoResponsable.Monotributo:
						return "M";
					case TipoResponsable.ResponsableNoInscripto:
						return "N";
					default:
						return " ";
				}
			}
		}
		
		public string Cuit {
			get {
				if (this.facturaAsociada.ProveedorCuit == 0.00)
					return "";
				
				if (this.facturaAsociada.ProveedorTipoResponsable == TipoResponsable.ConsumidorFinal)
					return "";
				
				CUIT cuit = new CUIT(this.facturaAsociada.ProveedorCuit);
				
				return cuit.ValorFormateado;					
			}
		}
		#endregion
		#endregion
	}
}
