// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using Stratos.Common;

namespace StratosServer.Reportes
{
	public class ResumenFacturaTipo
	{
		protected string tipoFactura;
		
		protected decimal rubroComercio;
		protected decimal rubroIndustria;
		protected decimal rubroNoDefinido;
				
		public ResumenFacturaTipo (string facturaTipo)
		{
			this.tipoFactura = facturaTipo;
			
			this.rubroComercio = 0.00m;
			this.rubroIndustria = 0.00m;
			this.rubroNoDefinido = 0.00m;
		}
		
		
		#region Propiedades
		
		public string TipoFactura {
			get { return tipoFactura; }
		}
		
		public decimal RubroComercio {
			get { return StratosUtil.RoundDecimal(this.rubroComercio, 2); }
		}
		
		public decimal RubroIndustria {
			get { return StratosUtil.RoundDecimal(this.rubroIndustria, 2); }
		}
		
		public decimal RubroNoDefinido {
			get { return StratosUtil.RoundDecimal(this.rubroNoDefinido, 2); }
		}
		
		public decimal Iva { 
			get { return StratosUtil.
				RoundDecimal((this.RubroComercio + this.RubroIndustria +
				              this.RubroNoDefinido) * 0.21m , 2); }
		}
		
		public decimal Total {
			get { return (this.RubroComercio + this.RubroIndustria +
				              this.RubroNoDefinido + this.Iva); }
		}
		
		#endregion
		
		public void Imputar(Factura factura)
		{	
			decimal factor = 1.00m;
			decimal importeItemActual;
			decimal bonificacion = factura.PorcentajeBonificacion;
		
			// Si es nota de credito, el importe tiene que restarse en vez de sumar
			if (factura is NotaCredito)
				factor = -1.00m;
			
			foreach (ItemFactura item in factura.Items) {

				importeItemActual = 0.00m;
				
				/* Para cada item, sumo el importe del producto al rubro correspondiente.
				 * Si es factura B le descuento el IVA */
				importeItemActual = (factura.Tipo == FacturaTipoVenta.FacturaA ?
				                     item.Importe : (item.Importe / 1.21m));
				
				// Le quito el porcentaje de bonificación
				importeItemActual -= (importeItemActual * (bonificacion / 100.0m));
				
				// Rubro COMERCIO
				if (item.ProductoRubroId == 1)
					this.rubroComercio += factor * importeItemActual;
				
				// Rubro INDUSTRIA
				else if (item.ProductoRubroId == 2)
					this.rubroIndustria += factor * importeItemActual;
				
				// Rubro NO DEFINIDO - FIXME: Creo que nunca se utilizaría este
				else
					this.rubroNoDefinido += factor * importeItemActual;
			}
		}
	}
}
