// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

using Stratos.Common;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace StratosServer.Reportes
{
	public class ResumenSucursal
	{
		protected string descripcion;
		
		protected ResumenFacturaTipo resumenFacturasA;
		protected ResumenFacturaTipo resumenFacturasB;
		
		public ResumenSucursal()
		{
			// Inicializo los items
			this.resumenFacturasA = new ResumenFacturaTipo("A");
			this.resumenFacturasB = new ResumenFacturaTipo("B");
		}
		
		#region Propiedades
		public string Descripcion {
			get { return this.descripcion; }
			set { this.descripcion = value; }
		}
		
		public ResumenFacturaTipo[] ResumenesFacturas {
			get {
				List<ResumenFacturaTipo> listaResumenes = new List<ResumenFacturaTipo>();
			
				listaResumenes.Add(this.resumenFacturasA);
				listaResumenes.Add(this.resumenFacturasB);
				
				return listaResumenes.ToArray();
			}
		}
				
		public decimal TotalComercio {
			get { return (this.resumenFacturasA.RubroComercio + this.resumenFacturasB.RubroComercio); }
		}
		
		public decimal TotalIndustria {
			get { return (this.resumenFacturasA.RubroIndustria + this.resumenFacturasB.RubroIndustria); }
		}
		
		public decimal TotalNoDefinido {
			get { return (this.resumenFacturasA.RubroNoDefinido + this.resumenFacturasB.RubroNoDefinido); }
		}
		
		public decimal TotalIva {
			get { return (this.resumenFacturasA.Iva + this.resumenFacturasB.Iva); }
		}
		
		public decimal TotalTotal {
			get { return (this.resumenFacturasA.Total + this.resumenFacturasB.Total); }
		}
		
		#endregion
		
		public void AgregarFactura(Factura factura) {
			
			// Si es una factura A
			if (factura.Tipo == FacturaTipoVenta.FacturaA)
				this.resumenFacturasA.Imputar(factura);
			
			// Si es una factura B
			else
				this.resumenFacturasB.Imputar(factura);
		}
	}
}
