// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Stratos.Servicios;
using StratosServer.Core;
using Stratos.ModeloDto;

namespace StratosServer.Ventas.Facturacion
{
	public class ConexionTerminalListener : ITerminalConectada, ITerminalDesconectada,
		IDatosTerminalActualizados
	{
		#region Campos
		private static Dictionary<string, DatosTerminal> terminalesImpresoras;
		#endregion
		
		#region Constructores
		public ConexionTerminalListener()
		{
		}
		#endregion
		
		#region Propiedades
		public static Dictionary<string, DatosTerminal> TerminalesImpresoras {
			get { return terminalesImpresoras; }
			set { terminalesImpresoras = value; }
		}
		#endregion
		
		#region Métodos
		public static void ResetearTerminalesConectadas()
		{
			terminalesImpresoras.Clear();
		}
		#endregion
		
		#region ITerminalConectada implementation
		public void TerminalConectada (string identificador, DatosTerminal datosTerminal)
		{
			if (datosTerminal.datosTermImpresora != null &&
			    	datosTerminal.datosTermImpresora.servFacturacion != null &&
			    	datosTerminal.datosTermImpresora.servFacturacion.Count > 0)
				terminalesImpresoras.Add(identificador, datosTerminal);
		}

		#endregion

		#region ITerminalDesconectada implementation
		public void TerminalDesconectada (string identificador, DatosTerminal datosTerminal)
		{
			if (terminalesImpresoras.ContainsKey(identificador))
				terminalesImpresoras.Remove(identificador);
		}

		#endregion

		#region IDatosTerminalActualizados implementation
		public void DatosTerminalActualizados (string identificador, DatosTerminal datosTerminal)
		{
			// Si la terminal ya existe, actualizo la estructura con un nuevo valor
			if (terminalesImpresoras.ContainsKey(identificador))
				terminalesImpresoras[identificador] = datosTerminal;
			
			/* Si la terminal no está en la lista de terminales impresoras, entonces es
			 * una terminal que al conectarse no tenía capacidades de impresión, y ahora sí.
			 * Por lo tanto delego el problema al método TerminalConectada */
			else
				this.TerminalConectada(identificador, datosTerminal);
		}
		#endregion
	}
}
