// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using log4net;
using Castle.ActiveRecord;
using NHibernate;
using NHibernate.Criterion;

using StratosServer.Core;
using Stratos.Common.Excepciones;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace StratosServer.Ventas.Recibos
{
	public class ControladorRecibosI : ControladorRecibosDisp_
	{
		#region ControladorRecibosDisp_ implementation
		#region Métodos de consulta
		public override Ice.Object[] BuscarHQL (string consultaHQL, ConsultaPaginable consPaginable, Ice.Current current__)
		{
			ReciboDto[] resultados =
				StratosServerUtil
					.BuscarHQLGenerico<ReciboDto,Recibo>(consultaHQL, consPaginable);
			
			return resultados;
		}
		
		public override Ice.Object[] BuscarQueryObjects (Stratos.QueryObjects.QueryObject queryObject, ConsultaPaginable consPaginable, Ice.Current current__)
		{
			throw new System.NotImplementedException();
		}
		
		public override Ice.Object Refrescar (Ice.Object objeto, Ice.Current current__)
		{
			ReciboDto reciboDto = objeto as ReciboDto;
			
			return StratosServerUtil
				.Refrescar<Recibo,ReciboDto>(reciboDto, reciboDto.id);
		}
		#endregion
		
		#region Métodos de guardado
		public override Ice.Object Guardar (Ice.Object recibo, Ice.Current current__)
		{
			// Utilizo una transacción para realizar todas las operaciones en la base de datos
			using (TransactionScope transactionScope = new TransactionScope(TransactionMode.Inherits)) {
				
				ReciboDto reciboDto =
					StratosServerUtil.TipoCorrecto<ReciboDto>(recibo);
				
				// Chequeo que la versión que se intenta eliminar sea la actual
				StratosServerUtil
					.VersionCorrecta<Recibo>(reciboDto.id,
					                         reciboDto.version,
					                         transactionScope);
				
				// Persisto el Presupuesto
				Recibo reciboNuevo = ConvertidorDto
						.FromDto<ReciboDto,Recibo>(reciboDto);
				
				if (reciboDto.id <= 0) {
					StratosServerUtil
						.CrearObjeto<Recibo>(reciboNuevo, transactionScope);
				}
				else {
					StratosServerUtil
						.ActualizarObjeto<Recibo>(reciboNuevo, transactionScope);
				}
				
				// Retorno una versión Dto actualizada.
				return ConvertidorDto.ToDto(reciboNuevo);
			}
		}
		
		public override Ice.Object Modificar (Ice.Object objeto, Ice.Current current__)
		{
			/* FIXME: En este controlador el método Guardar crea nuevos objetos o los
			 * modifica. Hace ambas operaciones. Estaría bueno que estén separadas. */
			return this.Guardar(objeto, current__);
		}
		#endregion
		#endregion
	}
}
