// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Stratos.ModeloObjetos;

using NHibernate;
using Castle.ActiveRecord;

namespace StratosServer.Core
{
	/// <summary>
	/// Consulta genérica personalizada para ejecutar con ActiveRecord
	/// una operación 'Update' en HQL. En el HQL se pueden incluir
	/// parametros, los cuales deben ser pasados al constructor
	/// en el diccionario con sus respectivos valores.
	/// </summary>
	public class CustomUpdateQuery<T> : ActiveRecordBaseQuery
		where T : ActiveRecordValidationBase<T>
	{
		#region Campos
		private string consultaHql;
		private Dictionary <string, object> parametros;
		private Dictionary <string, ICollection> listaParametros;
		#endregion

		#region Constructores
		public CustomUpdateQuery(string consultaHql)
				: base(typeof(T))
		{
			this.consultaHql = consultaHql;
			this.parametros = new Dictionary<string, object>();
			this.listaParametros = new Dictionary<string, ICollection>();
		}
		
		public void SetearParametros(string nombre, object valor)
		{
			this.parametros.Add(nombre, valor);
		}
		
		public void SetearListaDeParametros(string nombre, ICollection valores)
		{
			this.listaParametros.Add(nombre, valores);
		}
		#endregion
		
		#region ActiveRecordBaseQuery - Implementación
		protected override IQuery CreateQuery (ISession session)
		{
			IQuery consulta = session.CreateQuery(this.consultaHql);
			
			foreach (KeyValuePair<string, object> kvp in parametros)
				consulta.SetParameter(kvp.Key, kvp.Value);
			
			foreach (KeyValuePair<string, ICollection> kvp in listaParametros)
				consulta.SetParameterList(kvp.Key, kvp.Value);
			
			return consulta;
		}
		
		protected override object InternalExecute (ISession session)
		{
			IQuery consulta = this.CreateQuery(session);
			return consulta.ExecuteUpdate();
		}
		#endregion
	}
}
