// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;

using log4net;

using Castle.Core.Interceptor;

namespace StratosServer.Core.Interceptores
{
	public class LoggerInterceptor : IInterceptor
	{
		private readonly ILog logger;
		
		public LoggerInterceptor(System.Type tipo)
		{
			this.logger = LogManager.GetLogger(tipo);
		}
		
		public void Intercept (IInvocation invocation)
		{
			string nombreMetodoCompleto =
				String.Format("{0}", invocation.Method.Name);
			
			// Calculo los argumentos del método
			StringBuilder argumentos = new StringBuilder();
			
			foreach (var par in invocation.Method.GetParameters()) {
				argumentos.AppendFormat("{0} = '{1}'; ", par.Name, invocation.GetArgumentValue(par.Position));
			}
			
			this.logger.DebugFormat("Ejecutando método '{0}'. Parámetros: {1}", nombreMetodoCompleto, argumentos.ToString());
			
			try {
				invocation.Proceed();
				this.logger.DebugFormat("Ejecución finalizadada: '{0}'", nombreMetodoCompleto);
			}
			catch (System.Exception ex) {
				this.logger.ErrorFormat("Excepción en la ejecución: '{0}'. Detalles:\n{1}", invocation.Method.Name, ex.ToString());
				throw ex;
			}
		}
	}
}
