// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using System.Reflection;
using System.ServiceProcess;

using log4net;

using StratosServer.Core;

namespace StratosServer.Startup
{
	public class StratosServerServicio : ServiceBase
	{
		#region Campos
		private static ILog logger;

		private ServidorStratos servidorStratos;
		#endregion

		#region Constructores y métodos de inicio
		static StratosServerServicio()
		{
			log4net.Config.XmlConfigurator.Configure();
			logger = LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod()
			                              .DeclaringType);
		}

		public StratosServerServicio()
		{
			this.CanPauseAndContinue = false;
			this.ServiceName = "Stratos Server";
			this.EventLog.Log = "Application";
		}

		public static void Main(string[] args)
		{
			ServiceBase.Run(new StratosServerServicio());
		}
		
		/// <summary>
		/// Método de entrada para ejecutar el servidor.
		/// </summary>
		/// <param name="args">
		/// Argumentos de inicio.
		/// </param>
		/// <param name="esperarThreadDeComunicaciones">
		/// Indica si al iniciar el servidor se debe esperar el thread the Ice,
		/// bloqueando el actual. Se utiliza cuando el servidor se ejecuta por consola.
		/// </param>
		public void IniciarServicio(string[] args, bool esperarThreadDeComunicaciones)
		{
			logger.DebugFormat("Iniciando Stratos Server (OnStart) versión {0}",
			                   Assembly.GetExecutingAssembly().GetName().Version.ToString());
			
			logger.Debug("Creando instancia de ServidorStratos");
			this.servidorStratos = new ServidorStratos(esperarThreadDeComunicaciones);
			
			logger.Debug("Inicializando ServidorStratos");
			this.servidorStratos.Iniciar(args, Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location));
			
			// Si espero el thread de Ice, entonces al final ejecuto las operaciones
			// de cierre (ya que de otra manera solo se ejecutarían cuando el servidor
			// corre en modo de servicio y no de consola).
			if (esperarThreadDeComunicaciones)
				this.OnStop();
		}
		#endregion
		
		#region Eventos de ServiceBase
		protected override void OnStart (string[] args)
		{
			try
			{
				logger.Info("Iniciando servicio Stratos Server");
				this.IniciarServicio(args, false);
				logger.Info("Servicio Stratos Server iniciado");
			}
			catch (Exception ex)
			{
				logger.Error("Hubo un error al iniciar el servicio Stratos Server", ex);
				throw ex;
			}
		}
		
		protected override void OnStop ()
		{
			try
			{
				logger.Debug("Deteniendo servicio Stratos Server");
				this.servidorStratos.Detener();
				logger.Debug("Stratos Server se ha detenido correctamente");
			}
			catch (Exception ex)
			{
				logger.Error("Error al detener Stratos Server", ex);
			}
		}
		#endregion
	}
}
