// Copyright (C) 2009 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;

using Stratos.ModeloDto;
using StratosServer.Controladores;

namespace ConcurrencyTest
{
	class MainClass
	{
		protected static Ice.Communicator communicator;
		
		protected static ControladorFacturacionPrx ControladorFacturacion {
			get {
				Ice.ObjectPrx obj = communicator.stringToProxy("ControladorFacturacion -t:tcp -h localhost -p 10000 -t 10000");
				
				ControladorFacturacionPrx cf =
					ControladorFacturacionPrxHelper.checkedCast(obj);
				
				if (cf == null)
					throw new ApplicationException("No se pudo obtener el controlador de facturación.");
				
				return cf;
			}
		}
		
		protected static ControladorPresupuestosPrx ControladorPresupuestos {
			get {
				Ice.ObjectPrx obj = communicator.stringToProxy("ControladorPresupuestos -t:tcp -h localhost -p 10000 -t 10000");
				
				ControladorPresupuestosPrx cp =
					ControladorPresupuestosPrxHelper.checkedCast(obj);
				
				if (cp == null)
					throw new ApplicationException("No se pudo obtener el controlador de presupuestos.");
				
				return cp;
			}
		}
		
		protected static ControladorClientesPrx ControladorClientes {
			get {
				Ice.ObjectPrx obj = communicator.stringToProxy("ControladorClientes -t:tcp -h localhost -p 10000 -t 10000");
				
				ControladorClientesPrx cc =
					ControladorClientesPrxHelper.checkedCast(obj);
				
				if (cc == null)
					throw new ApplicationException("No se pudo obtener el controlador de clientes.");
				
				return cc;
			}
		}
		
		protected static ControladorProductosPrx ControladorProductos {
			get {
				Ice.ObjectPrx obj = communicator.stringToProxy("ControladorProductos -t:tcp -h localhost -p 10000 -t 10000");
				
				ControladorProductosPrx cp =
					ControladorProductosPrxHelper.checkedCast(obj);
				
				if (cp == null)
					throw new ApplicationException("No se pudo obtener el controlador de productos.");
				
				return cp;
			}
		}
		
		public static void Main (string[] args)
		{
			Console.WriteLine ("Iniciando prueba...");
			
			int status = 0;
			
			try {
				communicator = Ice.Util.initialize(ref args);
				
				
				PreFacturaDto preFactura;
				PresupuestoDto presupuesto;
				
				ItemProductoDto unItem;
				ProductoDto productoMosaico =
					MainClass.ControladorProductos.BuscarPorCodigo("M-000")[0];
				
				for (int i=0; i<1000; i++) {
					preFactura = new PreFacturaDto();
					
					preFactura.cliente = MainClass.ControladorClientes.ObtenerClienteConsumidorFinalGenerico()[0];
					preFactura.condVenta = new CondicionVentaDto(CondicionPago.Contado, 0);
					preFactura.fechaTicks = DateTime.Today.Ticks;
					preFactura.tipo = FacturaTipoVenta.FacturaB;
					
					preFactura.items = new List<ItemGenericoDto>();
					
					unItem = new ItemProductoDto();
					unItem.cantidad = 1;
					unItem.descripcion = productoMosaico.descripcion;
					unItem.orden = 1;
					unItem.precioUnitario = productoMosaico.importeVenta;
					unItem.productoId = productoMosaico.id;
					unItem.productoCodigo = productoMosaico.codigo;
					unItem.productoDescripcion = productoMosaico.descripcion;
					unItem.productoImporteSinIva = productoMosaico.importeSinIva;
					unItem.productoImporteVenta = productoMosaico.importeVenta;
					unItem.importe = Math.Round(unItem.cantidad * unItem.precioUnitario, 2);
					
					preFactura.items.Add(unItem);
					
					preFactura.importeBruto = unItem.importe;
					preFactura.importeBonificacion = 0.00;
					preFactura.importeNeto = preFactura.importeBruto - preFactura.importeBonificacion;
					preFactura.importeIva = 0.00;
					preFactura.importeTotal = preFactura.importeNeto + preFactura.importeIva;
					preFactura.importeFinal = preFactura.importeTotal;
					
					MainClass.ControladorFacturacion.Guardar(preFactura);
					
//					presupuesto = new PresupuestoDto();
//					
//					presupuesto.cliente = MainClass.ControladorClientes.ObtenerClienteConsumidorFinalGenerico()[0];
//					presupuesto.fechaTicks = DateTime.Today.Ticks;
//					
//					presupuesto.items = new List<ItemGenericoDto>();
//					
//					unItem = new ItemProductoDto();
//					unItem.cantidad = 1;
//					unItem.descripcion = productoMosaico.descripcion;
//					unItem.orden = 1;
//					unItem.precioUnitario = productoMosaico.importeVenta;
//					unItem.productoId = productoMosaico.id;
//					unItem.productoCodigo = productoMosaico.codigo;
//					unItem.productoDescripcion = productoMosaico.descripcion;
//					unItem.productoImporteSinIva = productoMosaico.importeSinIva;
//					unItem.productoImporteVenta = productoMosaico.importeVenta;
//					unItem.importe = Math.Round(unItem.cantidad * unItem.precioUnitario, 2);
//					
//					presupuesto.items.Add(unItem);
//					
//					presupuesto.importeBruto = unItem.importe;
//					presupuesto.importeBonificacion = 0.00;
//					presupuesto.importeNeto = presupuesto.importeBruto;
//					presupuesto.importeFinal = presupuesto.importeBruto;
//					
//					MainClass.ControladorPresupuestos.Guardar(presupuesto);
					
					Console.WriteLine(i.ToString());
				}
			}
			catch (System.Exception ex) {
				Console.Error.WriteLine(ex.ToString());
				status = 1;
			}
			
			if (communicator != null) {
				try {
					communicator.destroy();
				}
				catch (System.Exception ex) {
					Console.Error.WriteLine(ex.ToString());
					status = 1;
				}
			}
			
			Environment.Exit(status);
		}
	}
}
