// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Castle.ActiveRecord;
using Castle.Components.Validator;
using NUnit.Framework;

namespace Stratos.ModeloObjetos.Tests
{
	[TestFixture()]
	public class CUITTests
	{
		#region Campos
		private CUIT cuit;
		#endregion
		
		#region Métodos inicializadores
//		[SetUp]
//		public void Init()
//		{
//		}
		#endregion
		
		#region Cuits válidos
		#region Entrada numérica
		[Test()]
		public void CuitValido_ConstructorPorDefecto()
		{
			this.cuit = new CUIT();
			Assert.IsFalse(this.cuit.CuitValido, "Válido");
			Assert.IsNull(this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(String.Empty, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaNumerica_2()
		{
			this.cuit = new CUIT(30674393683);
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(30674393683, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("30-67439368-3", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaNumerica_3()
		{
			this.cuit = new CUIT(20119260397);
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(20119260397, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("20-11926039-7", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaNumerica_4()
		{
			this.cuit = new CUIT(30674346227);
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(30674346227, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("30-67434622-7", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_11_20()
		{
			this.cuit = new CUIT(20119260397);
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(20119260397, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("20-11926039-7", this.cuit.ValorFormateado, "Valor formateado");
		}
		#endregion
		
		#region Entrada en cadena
		[Test()]
		public void CuitValido_EntradaCadena_1()
		{
			this.cuit = new CUIT("30-67439368-3");
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(30674393683, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("30-67439368-3", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaCadena_2()
		{
			this.cuit = new CUIT("20-11926039-7");
			Assert.IsTrue(this.cuit.CuitValido,"Válido");
			Assert.AreEqual(20119260397, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("20-11926039-7", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaCadena_3()
		{
			this.cuit = new CUIT("30-67434622-7");
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(30674346227, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("30-67434622-7", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitValido_EntradaCadena_4()
		{
			this.cuit = new CUIT("20-07882123-0");
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(20078821230, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("20-07882123-0", this.cuit.ValorFormateado, "Valor formateado");
		}
		#endregion
		#endregion
		
		#region Cuits inválidos
		#region Entrada numérica
		[Test()]
		public void CuitInvalido_Cero()
		{
			this.cuit = new CUIT(0);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("0", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitInvalido_Diez()
		{
			this.cuit = new CUIT(10);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(10, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("10", this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_DiezDigitos()
		{
			this.cuit = new CUIT(3067439368);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_DoceDigitos()
		{
			this.cuit = new CUIT(306743936812);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_Negativo()
		{
			this.cuit = new CUIT(-30674346227);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_CabeceraInvalida_HashInvalido()
		{
			this.cuit = new CUIT(28340596820);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_CabeceraInvalida_HashValido()
		{
			this.cuit = new CUIT(21119260393);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		public void CuitInvalido_CabeceraValida_HashInvalido()
		{
			this.cuit = new CUIT(21119260398);
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(0, this.cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		#endregion
		
		#region Entrada en cadena
		[Test()]
		public void CuitInvalido_CadenaVacia()
		{
			this.cuit = new CUIT("");
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(String.Empty, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitInvalido_CadenaVaciaEspacios()
		{
			this.cuit = new CUIT("  ");
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitInvalido_Cadena1()
		{
			this.cuit = new CUIT("letras");
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitInvalido_Cadena2()
		{
			this.cuit = new CUIT("letras");
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			Assert.AreEqual(CUIT.CUIT_INVALIDO, this.cuit.ValorFormateado, "Valor formateado");
		}
		
		[Test()]
		public void CuitInvalido_Cadena3()
		{
			cuit = new CUIT("30-6743936-8");
			Assert.IsFalse(this.cuit.CuitValido, "Valido");
			
			// Aunque el cuit sea inválido, igual muestro el valor
			// Esto es necesario para los clientes migrados, que algunos tienen
			// su cuit inválido.
			Assert.AreEqual(3067439368, this.cuit.ValorNumerico, "Valor numérico");
			Assert.AreEqual("30-6743936-8", this.cuit.ValorFormateado, "Valor formateado");
		}
		#endregion
		#endregion
		
		#region Cuits ignorados (hay que revisar)
		[Test]
		[Ignore("Cuits ignorados (hay que revisarlos)")]
		public void CuitsIgnorados()
		{			
			this.cuit = new CUIT(2006300051);
			Assert.IsTrue(this.cuit.CuitValido, "Válido");
			Assert.AreEqual(2006300051, cuit.ValorNumerico, "Valor numerico");
			Assert.AreEqual("20-0630005-1", cuit.ValorFormateado, "Valor formateado");
		}
		#endregion
	}
}
