// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

using Castle.ActiveRecord;
using Castle.ActiveRecord.Framework;
using NUnit.Framework;
using AutoMapper;

using Stratos.Tests;
using Stratos.ModeloObjetos;
using Stratos.ModeloObjetos.MetaCampos;
using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos.Tests
{
	[TestFixture()]
	public class ConvertidorDtoTests : DocumentoGenericoTestBase
	{
		[Test()]
		public void MapeosValidos()
		{
			/* Accedo a alguna propiedad de ConvertidorDto para que se inicialicen los
			 * mapeos en el constructor de clase. */
			object obj = ConvertidorDto.MapperEngine;
			
			Mapper.AssertConfigurationIsValid();
		}
		
		#region Clientes
		[Test()]
		public void ToDtoCliente()
		{
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteConBonificacion);
			
			Assert.AreEqual(this.ClienteConBonificacion.Codigo, clienteDto.codigo, "Código");
			Assert.AreEqual(this.ClienteConBonificacion.RazonSocial, clienteDto.razonSocial, "Razón social");
			
			Assert.IsNotNull(clienteDto.domicilio);
			Assert.AreEqual(this.ClienteConBonificacion.Domicilio.Direccion, clienteDto.domicilio.direccion, "Domicilio - Dirección");
			Assert.AreEqual(this.ClienteConBonificacion.Domicilio.Localidad, clienteDto.domicilio.localidad, "Domicilio - Localidad");
			Assert.AreEqual(this.ClienteConBonificacion.Domicilio.Provincia, clienteDto.domicilio.provincia, "Domicilio - Provincia");
			Assert.AreEqual(this.ClienteConBonificacion.Domicilio.CodigoPostal, clienteDto.domicilio.codigoPostal, "Domicilio - Codigo postal");
			
			Assert.AreEqual(this.ClienteConBonificacion.Telefono, clienteDto.telefono, "Teléfono");
			Assert.AreEqual(this.ClienteConBonificacion.CorreoElectronico, clienteDto.correoElectronico, "Correo electrónico");
			Assert.AreEqual(this.ClienteConBonificacion.TipoResponsable, clienteDto.tipoRes, "Tipo responsable");
			Assert.AreEqual(this.ClienteConBonificacion.Cuit.ValorFormateado, clienteDto.cuit, "CUIT");
			Assert.AreEqual(this.ClienteConBonificacion.PorcentajeBonificacion, clienteDto.porcentajeBonificacion, "Bonificación");
			Assert.AreEqual(this.ClienteConBonificacion.Notas, clienteDto.notas, "Notas");
			
			Assert.AreEqual(this.ClienteConBonificacion.Eliminado, clienteDto.eliminado, "Eliminado");
			Assert.AreEqual(this.ClienteConBonificacion.Version, clienteDto.version, "Version");
			
			Assert.AreEqual(this.ClienteConBonificacion.ObtenerMetaCampo(MetaCamposKeys.ClienteCondicionVentaPorDefecto),
			                ((int)clienteDto.condicionPagoPorDefecto[0]).ToString(), "Condicion de pago por defecto");
		}
		
		[Test()]
		public void ToDtoClienteSinCondicionPagoPorDefecto()
		{
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteSinBonificacion);
			
			Assert.AreEqual(this.ClienteSinBonificacion.Codigo, clienteDto.codigo, "Código");
			Assert.AreEqual(this.ClienteSinBonificacion.RazonSocial, clienteDto.razonSocial, "Razón social");
			
			Assert.IsNull(clienteDto.condicionPagoPorDefecto, "Condicion de pago por defecto");
		}
		
		[Test()]
		public void FromDtoCliente()
		{
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteConBonificacion);
			
			Cliente cliente = ConvertidorDto.FromDto(clienteDto);
			
			Assert.AreEqual(clienteDto.codigo, cliente.Codigo, "Código");
			Assert.AreEqual(clienteDto.razonSocial, cliente.RazonSocial, "Razón social");
			Assert.IsTrue(cliente.Valido, "Valido");
			
			Assert.IsNotNull(cliente.Domicilio);
			Assert.AreEqual(clienteDto.domicilio.direccion, cliente.Domicilio.Direccion, "Domicilio - Dirección");
			Assert.AreEqual(clienteDto.domicilio.localidad, cliente.Domicilio.Localidad, "Domicilio - Localidad");
			Assert.AreEqual(clienteDto.domicilio.provincia, cliente.Domicilio.Provincia, "Domicilio - Provincia");
			Assert.AreEqual(clienteDto.domicilio.codigoPostal, cliente.Domicilio.CodigoPostal, "Domicilio - Codigo postal");
			
			Assert.AreEqual(clienteDto.telefono, cliente.Telefono, "Teléfono");
			Assert.AreEqual(clienteDto.correoElectronico, cliente.CorreoElectronico, "Correo electrónico");
			Assert.AreEqual(clienteDto.tipoRes, cliente.TipoResponsable, "Tipo responsable");
			
			Assert.IsNotNull(cliente.Cuit);
			Assert.AreEqual(clienteDto.cuit, cliente.Cuit.ValorFormateado, "CUIT");
			
			Assert.AreEqual(clienteDto.porcentajeBonificacion, cliente.PorcentajeBonificacion, "Bonificación");
			Assert.AreEqual(clienteDto.notas, cliente.Notas, "Notas");
			
			Assert.AreEqual(clienteDto.eliminado, cliente.Eliminado, "Eliminado");
			Assert.AreEqual(clienteDto.version, cliente.Version, "Versión");
			
			Assert.AreEqual(((int)clienteDto.condicionPagoPorDefecto[0]).ToString(),
			                cliente.ObtenerMetaCampo(MetaCamposKeys.ClienteCondicionVentaPorDefecto),
			                "Condicion de venta por defecto");
		}
		
		[Test()]
		public void FromDtoClienteSinCondicionPagoPorDefecto()
		{
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteSinBonificacion);

			Cliente cliente = ConvertidorDto.FromDto(clienteDto);
			
			Assert.AreEqual(clienteDto.codigo, cliente.Codigo, "Código");
			Assert.AreEqual(clienteDto.razonSocial, cliente.RazonSocial, "Razón social");
			
			Assert.AreEqual(0, cliente.MetaCampos.Count, "Cantidad de metacampos");
			Assert.IsNull(cliente.ObtenerMetaCampo(MetaCamposKeys.ClienteCondicionVentaPorDefecto),
			              "Metacampo Condición de venta por defecto");
		}
		
		[Test()]
		public void FromDtoClienteExistenteCondicionDePagoEliminada()
		{
			try {
				/* Convierto el cliente persistido a dto y elimino la 
				 * condición de pago por defecto. */
				ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteConBonificacion);
				clienteDto.condicionPagoPorDefecto = null;
	
				Cliente clienteConvertido = ConvertidorDto.FromDto(clienteDto);
				
				Assert.AreEqual(0, clienteConvertido.MetaCampos.Count, "Cantidad de metacampos");
				Assert.IsNull(clienteConvertido.ObtenerMetaCampo(MetaCamposKeys.ClienteCondicionVentaPorDefecto),
				              "Metacampo Condición de venta cliente converido");
				
				// Guardo el cliente y lo vuelvo a recuperar
				clienteConvertido.Save();
				base.FlushAndRecreateScope();
				
				Cliente[] clientes = Cliente.FindAll();
				Cliente clienteRecuperado = clientes[0];
				
				// Me aseguro que el cliente recuperado no tenga ningun metacampo seteado
				Assert.AreEqual(0, clienteRecuperado.MetaCampos.Count, "Cantidad de metacampos");
				Assert.IsNull(clienteRecuperado.ObtenerMetaCampo(MetaCamposKeys.ClienteCondicionVentaPorDefecto),
				              "Metacampo Condición de venta cliente converido");
			}
			catch (Exception ex) {
				Assert.Fail("Ocurrió una excepción: " + ex.Message);
			}
		}
		
		[Test()]
		public void FromDtoClienteConsumidorFinalSinCUIT()
		{
			/* Cambio el CUIT del cliente para simular uno que se crea desde la terminal,
			 * al cual se le asigna desde un string vacío. Al crear un objeto CUIT vacio,
			 * luego se traducirá a un string vacío al pasarlo a ClienteDto, y de esta
			 * forma simulamos el comportamiento de la terminal. */
			this.ClienteConBonificacion.TipoResponsable = TipoResponsable.ConsumidorFinal;
			this.ClienteConBonificacion.Cuit = new CUIT();
			
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteConBonificacion);
			
			Cliente cliente = ConvertidorDto.FromDto(clienteDto);
			
			Assert.AreEqual(clienteDto.codigo, cliente.Codigo, "Código");
			Assert.AreEqual(clienteDto.razonSocial, cliente.RazonSocial, "Razón social");
			
			Assert.IsNotNull(cliente.Domicilio);
			Assert.AreEqual(clienteDto.domicilio.direccion, cliente.Domicilio.Direccion, "Domicilio - Dirección");
			Assert.AreEqual(clienteDto.domicilio.localidad, cliente.Domicilio.Localidad, "Domicilio - Localidad");
			Assert.AreEqual(clienteDto.domicilio.provincia, cliente.Domicilio.Provincia, "Domicilio - Provincia");
			Assert.AreEqual(clienteDto.domicilio.codigoPostal, cliente.Domicilio.CodigoPostal, "Domicilio - Codigo postal");
			
			Assert.AreEqual(clienteDto.telefono, cliente.Telefono, "Teléfono");
			Assert.AreEqual(clienteDto.correoElectronico, cliente.CorreoElectronico, "Correo electrónico");
			Assert.AreEqual(clienteDto.tipoRes, cliente.TipoResponsable, "Tipo responsable");
			
			Assert.IsNotNull(cliente.Cuit);
			Assert.AreEqual(clienteDto.cuit, cliente.Cuit.ValorFormateado, "CUIT");
			
			Assert.AreEqual(clienteDto.porcentajeBonificacion, cliente.PorcentajeBonificacion, "Bonificación");
			Assert.AreEqual(clienteDto.notas, cliente.Notas, "Notas");
			
			Assert.AreEqual(clienteDto.eliminado, cliente.Eliminado, "Eliminado");
			Assert.AreEqual(clienteDto.version, cliente.Version, "Versión");
		}
		#endregion
		
		#region Proveedor
		[Test()]
		public void ToDtoProveedor()
		{
			ProveedorDto proveedorDto = ConvertidorDto.ToDto(this.ProveedorFerrum);
			
			Assert.AreEqual(this.ProveedorFerrum.Codigo, proveedorDto.codigo, "Código");
			Assert.AreEqual(this.ProveedorFerrum.RazonSocial, proveedorDto.razonSocial, "Razón social");
			
			Assert.IsNotNull(proveedorDto.domicilio);
			Assert.AreEqual(this.ProveedorFerrum.Domicilio.Direccion, proveedorDto.domicilio.direccion, "Domicilio - Dirección");
			Assert.AreEqual(this.ProveedorFerrum.Domicilio.Localidad, proveedorDto.domicilio.localidad, "Domicilio - Localidad");
			Assert.AreEqual(this.ProveedorFerrum.Domicilio.Provincia, proveedorDto.domicilio.provincia, "Domicilio - Provincia");
			Assert.AreEqual(this.ProveedorFerrum.Domicilio.CodigoPostal, proveedorDto.domicilio.codigoPostal, "Domicilio - Codigo postal");
			
			Assert.AreEqual(this.ProveedorFerrum.Telefono, proveedorDto.telefono, "Teléfono");
			Assert.AreEqual(this.ProveedorFerrum.CorreoElectronico, proveedorDto.correoElectronico, "Correo electrónico");
			Assert.AreEqual(this.ProveedorFerrum.TipoResponsable, proveedorDto.tipoRes, "Tipo responsable");
			Assert.AreEqual(this.ProveedorFerrum.Cuit.ValorFormateado, proveedorDto.cuit, "CUIT");
			Assert.AreEqual(this.ProveedorFerrum.Notas, proveedorDto.notas, "Notas");
			
			Assert.AreEqual(this.ProveedorFerrum.Eliminado, proveedorDto.eliminado, "Eliminado");
			Assert.AreEqual(this.ProveedorFerrum.Version, proveedorDto.version, "Version");
		}
		
		[Test()]
		public void FromDtoProveedor()
		{
			ProveedorDto proveedorDto = ConvertidorDto.ToDto(this.ProveedorFerrum);
			
			Proveedor proveedor = ConvertidorDto.FromDto(proveedorDto);
			
			Assert.AreEqual(proveedorDto.codigo, proveedor.Codigo, "Código");
			Assert.AreEqual(proveedorDto.razonSocial, proveedor.RazonSocial, "Razón social");
			
			Assert.IsNotNull(proveedor.Domicilio);
			Assert.AreEqual(proveedorDto.domicilio.direccion, proveedor.Domicilio.Direccion, "Domicilio - Dirección");
			Assert.AreEqual(proveedorDto.domicilio.localidad, proveedor.Domicilio.Localidad, "Domicilio - Localidad");
			Assert.AreEqual(proveedorDto.domicilio.provincia, proveedor.Domicilio.Provincia, "Domicilio - Provincia");
			Assert.AreEqual(proveedorDto.domicilio.codigoPostal, proveedor.Domicilio.CodigoPostal, "Domicilio - Codigo postal");
			
			Assert.AreEqual(proveedorDto.telefono, proveedor.Telefono, "Teléfono");
			Assert.AreEqual(proveedorDto.correoElectronico, proveedor.CorreoElectronico, "Correo electrónico");
			Assert.AreEqual(proveedorDto.tipoRes, proveedor.TipoResponsable, "Tipo responsable");
			
			Assert.IsNotNull(proveedor.Cuit);
			Assert.AreEqual(proveedorDto.cuit, proveedor.Cuit.ValorFormateado, "CUIT");
			
			Assert.AreEqual(proveedorDto.notas, proveedor.Notas, "Notas");
			
			Assert.AreEqual(proveedorDto.eliminado, proveedor.Eliminado, "Eliminado");
			Assert.AreEqual(proveedorDto.version, proveedor.Version, "Versión");
		}
		#endregion
		
		#region Producto
		[Test]
		public void ToDtoProducto()
		{
			Rubro rubroPadre = TestBase.CrearRubroMosaico();
			rubroPadre.Nombre = "Materiales construcción";
			rubroPadre.Descripcion = "Materiales para pisos";
			rubroPadre.Codigo = 65;
			rubroPadre.RubroPadre = null;
			rubroPadre.Create();
			
			this.ProductoMosaico.SubRubro.RubroPadre = rubroPadre;
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			ProductoDto productoDto = ConvertidorDto.ToDto(this.ProductoMosaico);
			
			// Verifico los datos del producto
			Assert.AreEqual(this.ProductoMosaico.Id, productoDto.id, "Id");
			Assert.AreEqual(this.ProductoMosaico.Codigo, productoDto.codigo, "Código");
			Assert.AreEqual(this.ProductoMosaico.Descripcion, productoDto.descripcion, "Descripción");
			Assert.AreEqual(this.ProductoMosaico.ImporteSinIva, productoDto.importeSinIva, "Importe sin IVA");
			Assert.AreEqual(this.ProductoMosaico.PorcentajeIva, productoDto.porcentajeIva, "Porcenetaje IVA");
			Assert.AreEqual(this.ProductoMosaico.PrecioCosto, productoDto.precioCosto, "Precio costo");
			Assert.AreEqual(this.ProductoMosaico.ImporteVenta, productoDto.importeVenta, "Importe venta");
			Assert.AreEqual(this.ProductoMosaico.Unidad, productoDto.unidad, "Unidad");
			Assert.AreEqual(this.ProductoMosaico.ActivarGestionStock, productoDto.activarGestionStock, "Activar Stock");
			Assert.AreEqual(this.ProductoMosaico.Notas, productoDto.notas, "Notas");
			
			// Verifico el subrubro
			Assert.IsNotNull(productoDto.subRubro, "Subrubro");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.Codigo, productoDto.subRubro.codigo, "Código subrubro");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.Nombre, productoDto.subRubro.nombre, "Nombre subrubro");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.Descripcion, productoDto.subRubro.descripcion, "Descripción subrubro");
			
			// Verifico el rubro padre del subrubro
			Assert.IsNotNull(productoDto.subRubro.rubroPadre, "Rubro padre");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.RubroPadre.Codigo, productoDto.subRubro.rubroPadre.codigo, "Código rubro padre");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.RubroPadre.Nombre, productoDto.subRubro.rubroPadre.nombre, "Nombre rubro padre");
			Assert.AreEqual(this.ProductoMosaico.SubRubro.RubroPadre.Descripcion, productoDto.subRubro.rubroPadre.descripcion, "Descripción rubro padre");
		}
		
		[Test]
		//[Ignore("Hay que ver bien cómo se pasa un objeto DTO a un objeto de negocios")]
		public void FromDtoProducto()
		{
			Rubro rubroPadre = TestBase.CrearRubroMosaico();
			rubroPadre.Nombre = "Materiales construcción";
			rubroPadre.Descripcion = "Materiales para pisos";
			rubroPadre.Codigo = 65;
			rubroPadre.RubroPadre = null;
			rubroPadre.Create();
			
			this.ProductoMosaico.SubRubro.RubroPadre = rubroPadre;
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			ProductoDto productoDto = ConvertidorDto.ToDto(this.ProductoMosaico);
			
			Producto producto = ConvertidorDto.FromDto(productoDto);
			
			// Verifico los datos del producto
			Assert.AreEqual(productoDto.codigo, producto.Codigo, "Código");
			Assert.AreEqual(productoDto.descripcion, producto.Descripcion, "Descripción");
			Assert.IsTrue(producto.Valido);
			
			Assert.AreEqual(productoDto.importeSinIva, producto.ImporteSinIva, "Importe sin IVA");
			Assert.AreEqual(productoDto.porcentajeIva, producto.PorcentajeIva, "Porcenetaje IVA");
			Assert.AreEqual(productoDto.precioCosto, producto.PrecioCosto, "Precio costo");
			Assert.AreEqual(productoDto.unidad, producto.Unidad, "Unidad");
			Assert.AreEqual(productoDto.activarGestionStock, producto.ActivarGestionStock, "Activar Stock");
			Assert.AreEqual(productoDto.notas, producto.Notas, "Notas");
			
			// Verifico el subrubro
			Assert.IsNotNull(producto.SubRubro, "Subrubro");
			Assert.AreEqual(productoDto.subRubro.codigo, producto.SubRubro.Codigo, "Código subrubro");
			Assert.AreEqual(productoDto.subRubro.nombre, producto.SubRubro.Nombre, "Nombre subrubro");
			Assert.AreEqual(productoDto.subRubro.descripcion, producto.SubRubro.Descripcion, "Descripción subrubro");
			
			// Verifico el rubro padre del subrubro
			Assert.IsNotNull(producto.SubRubro.RubroPadre, "Rubro padre");
			Assert.AreEqual(productoDto.subRubro.rubroPadre.codigo, producto.SubRubro.RubroPadre.Codigo, "Código rubro padre");
			Assert.AreEqual(productoDto.subRubro.rubroPadre.nombre, producto.SubRubro.RubroPadre.Nombre, "Nombre rubro padre");
			Assert.AreEqual(productoDto.subRubro.rubroPadre.descripcion, producto.SubRubro.RubroPadre.Descripcion, "Descripción rubro padre");
		}
		#endregion
		
		#region Presupuesto
		[Test]
		public void ToDtoPresupuesto()
		{
			Presupuesto presupuesto = TestBase.CrearPresupuesto();
			presupuesto.Cliente = this.ClienteSinBonificacion2;
			
			presupuesto.Items.Clear();
			presupuesto.Items.Add(TestBase.CrearItemPresupuesto(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesPresupuesto(presupuesto);
			
			// Agrego un item con un producto abstracto
			ItemPresupuesto itemProductoAbstracto =
				TestBase.CrearItemPresupuesto(this.ProductoAbstracto, 1);
			itemProductoAbstracto.Descripcion += " agregado";
			itemProductoAbstracto.PrecioUnitario = 1.00m;
			itemProductoAbstracto.Importe = 1.00m;
			
			presupuesto.Items.Add(itemProductoAbstracto);
			
			presupuesto.ImporteBruto += 1.00m;
			presupuesto.ImporteNeto += 1.00m;
			
			try {
				presupuesto.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			PresupuestoDto presupuestoDto = ConvertidorDto.ToDto(presupuesto);
			
			Assert.AreEqual(presupuesto.Id, presupuestoDto.id, "Id");
			Assert.AreEqual(presupuesto.Fecha.Ticks, presupuestoDto.fechaTicks, "Fecha");
			
			Assert.IsNotNull(presupuestoDto.cliente, "Cliente no nulo");
			Assert.AreEqual(presupuesto.Cliente.RazonSocial, presupuestoDto.cliente.razonSocial, "Cliente razón social");
			Assert.AreEqual(presupuesto.Cliente.RazonSocial, presupuestoDto.razonSocial, "Presupuesto razón social");
			Assert.AreEqual(presupuesto.Telefono, presupuestoDto.telefono, "Cliente teléfono");
			Assert.AreEqual(presupuesto.CorreoElectronico, presupuestoDto.correoElectronico, "Cliente mail");
			
			// FIXME: Hacer un test para los items aparte, no acá
			Assert.IsNotNull(presupuestoDto.items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] presupuestoDtoItems = presupuestoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(presupuesto.Items.Count, presupuestoDtoItems.Length, "Items count");
			Assert.AreEqual(presupuesto.Items[0].Id, presupuestoDtoItems[0].id, "Item id");
			Assert.AreEqual(presupuesto.Items[0].ProductoId, presupuestoDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(presupuesto.Items[0].ProductoCodigo, presupuestoDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(presupuesto.Items[0].ProductoAbstracto, presupuestoDtoItems[0].productoAbstracto, "Item producto abstracto");
			Assert.AreEqual(presupuesto.Items[0].ProductoDescripcion, presupuestoDtoItems[0].productoDescripcion, "Item producto descripcion");
			Assert.AreEqual(presupuesto.Items[0].ProductoImporteSinIva, presupuestoDtoItems[0].productoImporteSinIva, "Item producto importe sin iva");
			Assert.AreEqual(presupuesto.Items[0].ProductoImporteVenta, presupuestoDtoItems[0].productoImporteVenta, "Item producto importe venta");
			Assert.AreEqual(presupuesto.Items[0].ProductoPrecioCosto, presupuestoDtoItems[0].productoPrecioCosto, "Item producto precio costo");
			Assert.AreEqual(presupuesto.Items[0].ProductoUnidad, presupuestoDtoItems[0].productoUnidad, "Item producto unidad");
			Assert.AreEqual(presupuesto.Items[0].Descripcion, presupuestoDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(presupuesto.Items[0].PrecioUnitario, presupuestoDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(presupuesto.Items[0].PorcentajeBonificacion, presupuestoDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(presupuesto.Items[0].Cantidad, presupuestoDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(presupuesto.Items[0].Importe, presupuestoDtoItems[0].importe, "Item importe");
			
			Assert.AreEqual(this.ProductoAbstracto.Descripcion + " agregado", presupuestoDtoItems[1].descripcion, "Item 2 producto descripción");
				
			Assert.AreEqual(presupuesto.ImporteBruto, presupuestoDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(presupuesto.PorcentajeBonificacion, presupuestoDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(presupuesto.ImporteBonificacion, presupuestoDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(presupuesto.ImporteNeto, presupuestoDto.importeNeto, "Importe Neto");
			Assert.AreEqual(presupuesto.ImporteFinal, presupuestoDto.importeFinal, "Importe Final");
			Assert.AreEqual(presupuesto.Observaciones, presupuestoDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoPresupuesto()
		{
			Presupuesto presupuestoTemp = TestBase.CrearPresupuesto();
			presupuestoTemp.Cliente = this.ClienteSinBonificacion2;
			presupuestoTemp.Items.Clear();
			presupuestoTemp.Items.Add(TestBase.CrearItemPresupuesto(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesPresupuesto(presupuestoTemp);
			
			try {
				presupuestoTemp.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail("No se pudo crear el presupuesto: " + this.ObtenerMensaje(ex));
			}
			
			this.FlushAndRecreateScope();
			
			presupuestoTemp.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			this.MarcarComoNoPersistido(ref presupuestoTemp);
			PresupuestoDto presupuestoDto = ConvertidorDto.ToDto(presupuestoTemp);
			
			Presupuesto presupuesto = ConvertidorDto.FromDto<PresupuestoDto,Presupuesto>(presupuestoDto);
			
			Assert.AreEqual(presupuestoDto.id, presupuesto.Id, "Id");
			Assert.AreEqual(presupuestoDto.fechaTicks, presupuesto.Fecha.Ticks, "Fecha");
			
			Assert.IsNotNull(presupuestoDto.cliente, "Cliente no nulo");
			Assert.AreEqual(presupuestoDto.cliente.razonSocial, presupuesto.Cliente.RazonSocial, "Cliente razón social");
			
			Assert.AreEqual(presupuestoDto.id, presupuesto.Id, "Id");
			
			// FIXME: Hacer un test para los items aparte, no acá
			Assert.IsNotNull(presupuesto.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] presupuestoDtoItems = presupuestoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(presupuestoDtoItems.Length, presupuesto.Items.Count, "Items count");
			Assert.AreEqual(presupuestoDtoItems[0].productoId, presupuesto.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(presupuestoDtoItems[0].productoCodigo, presupuesto.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(presupuestoDtoItems[0].descripcion, presupuesto.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(presupuestoDtoItems[0].precioUnitario, presupuesto.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(presupuestoDtoItems[0].porcentajeBonificacion, presupuesto.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(presupuestoDtoItems[0].cantidad, presupuesto.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(presupuestoDtoItems[0].importe, presupuesto.Items[0].Importe, "Item importe");
				
			Assert.AreEqual(presupuestoDto.importeBruto, presupuesto.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(presupuestoDto.porcentajeBonificacion, presupuesto.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(presupuestoDto.importeBonificacion, presupuesto.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(presupuestoDto.importeNeto, presupuesto.ImporteNeto, "Importe Neto");
			Assert.AreEqual(presupuestoDto.observaciones, presupuesto.Observaciones, "Observaciones");
			Assert.AreEqual(presupuestoDto.importeFinal, presupuesto.ImporteFinal, "Importe Final");
		}
		
		[Test]
		public void FromDtoPresupuesto_PresupuestoExistente()
		{
			Presupuesto presupuestoTemp = TestBase.CrearPresupuesto();
			presupuestoTemp.Cliente = this.ClienteSinBonificacion2;
			presupuestoTemp.Items.Clear();
			presupuestoTemp.Items.Add(TestBase.CrearItemPresupuesto(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesPresupuesto(presupuestoTemp);
			
			try {
				presupuestoTemp.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail("No se pudo crear el presupuesto: " + this.ObtenerMensaje(ex));
			}
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			PresupuestoDto presupuestoDto = ConvertidorDto.ToDto(presupuestoTemp);
			
			Presupuesto presupuesto = ConvertidorDto.FromDto<PresupuestoDto,Presupuesto>(presupuestoDto);
			
			Assert.AreEqual(presupuestoDto.id, presupuesto.Id, "Id");
			Assert.AreEqual(presupuestoDto.fechaTicks, presupuesto.Fecha.Ticks, "Fecha");
			
			Assert.IsNotNull(presupuestoDto.cliente, "Cliente no nulo");
			Assert.AreEqual(presupuestoDto.cliente.razonSocial, presupuesto.Cliente.RazonSocial, "Cliente razón social");
			
			Assert.AreEqual(presupuestoDto.id, presupuesto.Id, "Id");
			
			// FIXME: Hacer un test para los items aparte, no acá
			Assert.IsNotNull(presupuesto.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] presupuestoDtoItems = presupuestoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(presupuestoDtoItems.Length, presupuesto.Items.Count, "Items count");
			Assert.AreEqual(presupuestoDtoItems[0].productoId, presupuesto.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(presupuestoDtoItems[0].productoCodigo, presupuesto.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(presupuestoDtoItems[0].descripcion, presupuesto.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(presupuestoDtoItems[0].precioUnitario, presupuesto.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(presupuestoDtoItems[0].porcentajeBonificacion, presupuesto.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(presupuestoDtoItems[0].cantidad, presupuesto.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(presupuestoDtoItems[0].importe, presupuesto.Items[0].Importe, "Item importe");
				
			Assert.AreEqual(presupuestoDto.importeBruto, presupuesto.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(presupuestoDto.porcentajeBonificacion, presupuesto.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(presupuestoDto.importeBonificacion, presupuesto.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(presupuestoDto.importeNeto, presupuesto.ImporteNeto, "Importe Neto");
			Assert.AreEqual(presupuestoDto.observaciones, presupuesto.Observaciones, "Observaciones");
			Assert.AreEqual(presupuestoDto.importeFinal, presupuesto.ImporteFinal, "Importe Final");
		}
		#endregion
		
		#region PreFactura
		[Test]
		public void ToDtoPreFactura()
		{
			PreFactura preFactura = TestBase.CrearPreFactura();
			preFactura.Cliente = this.ClienteSinBonificacion;
			
			preFactura.Items.Clear();
			preFactura.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			preFactura.Items[0].Orden = 1;
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preFactura);
			
			preFactura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la PreFactura a PreFacturaDto
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(preFactura);
			
			Assert.AreEqual(preFactura.Id, preFacturaDto.id, "Id");
			Assert.AreEqual(preFactura.Fecha.Ticks, preFacturaDto.fechaTicks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preFacturaDto.cliente, "Cliente no nulo");
			Assert.AreEqual(preFactura.Cliente.RazonSocial, preFacturaDto.cliente.razonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preFacturaDto.items, "Items nulo");
			Assert.AreEqual(preFactura.Items.Count, preFacturaDto.items.Count, "Items count");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preFacturaDtoItems = preFacturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preFactura.Items.Count, preFacturaDtoItems.Length, "Items count");
			Assert.AreEqual(preFactura.Items[0].Id, preFacturaDtoItems[0].id, "Item id");
			Assert.AreEqual(preFactura.Items[0].ProductoId, preFacturaDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(preFactura.Items[0].ProductoCodigo, preFacturaDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(preFactura.Items[0].Descripcion, preFacturaDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(preFactura.Items[0].PrecioUnitario, preFacturaDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(preFactura.Items[0].PorcentajeBonificacion, preFacturaDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(preFactura.Items[0].Cantidad, preFacturaDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(preFactura.Items[0].Importe, preFacturaDtoItems[0].importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preFactura.ImporteBruto, preFacturaDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(preFactura.PorcentajeBonificacion, preFacturaDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preFactura.ImporteBonificacion, preFacturaDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(preFactura.ImporteNeto, preFacturaDto.importeNeto, "Importe Neto");
			Assert.AreEqual(preFactura.ImporteFinal, preFacturaDto.importeFinal, "Importe Final");
			Assert.AreEqual(preFactura.ImporteTotal, preFacturaDto.importeTotal, "Importe Total");
			Assert.AreEqual(preFactura.ImporteIva, preFacturaDto.importeIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preFactura.CondicionVenta.Condicion, preFacturaDto.condVenta.condPago, "CondicionVenta - Condicion");
			Assert.AreEqual(preFactura.CondicionVenta.Plazo, preFacturaDto.condVenta.plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preFactura.Tipo, preFacturaDto.tipo, "Tipo de factura");
			Assert.AreEqual(preFactura.Remitos, preFacturaDto.remitos, "Remitos");
			Assert.AreEqual(preFactura.Observaciones, preFacturaDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void ToDtoPreFactura_ConItemConBonificacion()
		{
			PreFactura preFactura = TestBase.CrearPreFactura();
			preFactura.Cliente = this.ClienteSinBonificacion;
			
			preFactura.Items.Clear();
			
			// Agrego un item con bonificación
			ItemPreFactura itemConBonificacion =
				TestBase.CrearItemPreFactura(this.ProductoMosaico, 1);
			itemConBonificacion.PorcentajeBonificacion = 10.00m;
			
			preFactura.Items.Add(itemConBonificacion);
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preFactura);
			
			preFactura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la PreFactura a PreFacturaDto
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(preFactura);
			
			Assert.AreEqual(preFactura.Id, preFacturaDto.id, "Id");
			Assert.AreEqual(preFactura.Fecha.Ticks, preFacturaDto.fechaTicks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preFacturaDto.cliente, "Cliente no nulo");
			Assert.AreEqual(preFactura.Cliente.RazonSocial, preFacturaDto.cliente.razonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preFacturaDto.items, "Items nulo");
			Assert.AreEqual(preFactura.Items.Count, preFacturaDto.items.Count, "Items count");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preFacturaDtoItems = preFacturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preFactura.Items.Count, preFacturaDtoItems.Length, "Items count");
			Assert.AreEqual(preFactura.Items[0].Id, preFacturaDtoItems[0].id, "Item id");
			Assert.AreEqual(preFactura.Items[0].ProductoId, preFacturaDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(preFactura.Items[0].ProductoCodigo, preFacturaDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(preFactura.Items[0].Descripcion, preFacturaDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(preFactura.Items[0].PrecioUnitario, preFacturaDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(preFactura.Items[0].PorcentajeBonificacion, preFacturaDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(preFactura.Items[0].Cantidad, preFacturaDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(preFactura.Items[0].Importe, preFacturaDtoItems[0].importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preFactura.ImporteBruto, preFacturaDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(preFactura.PorcentajeBonificacion, preFacturaDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preFactura.ImporteBonificacion, preFacturaDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(preFactura.ImporteNeto, preFacturaDto.importeNeto, "Importe Neto");
			Assert.AreEqual(preFactura.ImporteFinal, preFacturaDto.importeFinal, "Importe Final");
			Assert.AreEqual(preFactura.ImporteTotal, preFacturaDto.importeTotal, "Importe Total");
			Assert.AreEqual(preFactura.ImporteIva, preFacturaDto.importeIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preFactura.CondicionVenta.Condicion, preFacturaDto.condVenta.condPago, "CondicionVenta - Condicion");
			Assert.AreEqual(preFactura.CondicionVenta.Plazo, preFacturaDto.condVenta.plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preFactura.Tipo, preFacturaDto.tipo, "Tipo de factura");
			Assert.AreEqual(preFactura.Remitos, preFacturaDto.remitos, "Remitos");
			Assert.AreEqual(preFactura.Observaciones, preFacturaDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoPreFactura()
		{
			PreFactura preFacturaTemp = TestBase.CrearPreFactura();
			preFacturaTemp.Cliente = this.ClienteSinBonificacion;
			preFacturaTemp.Items.Clear();
			preFacturaTemp.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			preFacturaTemp.Items[0].Orden = 1;
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTemp);
			preFacturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(preFacturaTemp);
			
			PreFactura preFactura = ConvertidorDto.FromDto<PreFacturaDto,PreFactura>(preFacturaDto);
			
			Assert.AreEqual(preFacturaDto.id, preFactura.Id, "Id");
			Assert.AreEqual(preFacturaDto.fechaTicks, preFactura.Fecha.Ticks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preFactura.Cliente, "Cliente no nulo");
			Assert.AreEqual(preFacturaDto.cliente.razonSocial, preFactura.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preFactura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preFacturaDtoItems = preFacturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preFacturaDtoItems.Length, preFactura.Items.Count, "Items count");
			Assert.AreEqual(preFacturaDtoItems[0].productoId, preFactura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preFacturaDtoItems[0].productoCodigo, preFactura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(preFacturaDtoItems[0].descripcion, preFactura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(preFacturaDtoItems[0].precioUnitario, preFactura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(preFacturaDtoItems[0].porcentajeBonificacion, preFactura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(preFacturaDtoItems[0].cantidad, preFactura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preFacturaDtoItems[0].importe, preFactura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preFacturaDto.importeBruto, preFactura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(preFacturaDto.porcentajeBonificacion, preFactura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preFacturaDto.importeBonificacion, preFactura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(preFacturaDto.importeNeto, preFactura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(preFacturaDto.importeFinal, preFactura.ImporteFinal, "Importe Final");
			Assert.AreEqual(preFacturaDto.importeTotal, preFactura.ImporteTotal, "Importe Total");
			Assert.AreEqual(preFacturaDto.importeIva, preFactura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preFacturaDto.condVenta.condPago, preFactura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(preFacturaDto.condVenta.plazo, preFactura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preFacturaDto.tipo, preFactura.Tipo, "Tipo de factura");
			Assert.AreEqual(preFacturaDto.remitos, preFactura.Remitos, "Remitos");
			Assert.AreEqual(preFacturaDto.observaciones, preFactura.Observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoPreFactura_ConItemConDescuento()
		{
			PreFactura preFacturaTemp = TestBase.CrearPreFactura();
			preFacturaTemp.Cliente = this.ClienteSinBonificacion;
			
			preFacturaTemp.Items.Clear();
			
			// Agrego un item con bonificación
			ItemPreFactura itemConBonificacion =
				TestBase.CrearItemPreFactura(this.ProductoMosaico, 1);
			itemConBonificacion.PorcentajeBonificacion = 10.00m;
			
			preFacturaTemp.Items.Add(itemConBonificacion);
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTemp);
			preFacturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la PreFactura a PreFacturaDto
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(preFacturaTemp);
			
			// Vuelvo a pasar la PreFacturaDto a PreFactura
			PreFactura preFactura = ConvertidorDto.FromDto<PreFacturaDto,PreFactura>(preFacturaDto);
			
			Assert.AreEqual(preFacturaDto.id, preFactura.Id, "Id");
			Assert.AreEqual(preFacturaDto.fechaTicks, preFactura.Fecha.Ticks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preFactura.Cliente, "Cliente no nulo");
			Assert.AreEqual(preFacturaDto.cliente.razonSocial, preFactura.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preFactura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preFacturaDtoItems = preFacturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preFacturaDtoItems.Length, preFactura.Items.Count, "Items count");
			Assert.AreEqual(preFacturaDtoItems[0].productoId, preFactura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preFacturaDtoItems[0].productoCodigo, preFactura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(preFacturaDtoItems[0].descripcion, preFactura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(preFacturaDtoItems[0].precioUnitario, preFactura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(preFacturaDtoItems[0].porcentajeBonificacion, preFactura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(preFacturaDtoItems[0].cantidad, preFactura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preFacturaDtoItems[0].importe, preFactura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preFacturaDto.importeBruto, preFactura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(preFacturaDto.porcentajeBonificacion, preFactura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preFacturaDto.importeBonificacion, preFactura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(preFacturaDto.importeNeto, preFactura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(preFacturaDto.importeFinal, preFactura.ImporteFinal, "Importe Final");
			Assert.AreEqual(preFacturaDto.importeTotal, preFactura.ImporteTotal, "Importe Total");
			Assert.AreEqual(preFacturaDto.importeIva, preFactura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preFacturaDto.condVenta.condPago, preFactura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(preFacturaDto.condVenta.plazo, preFactura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preFacturaDto.tipo, preFactura.Tipo, "Tipo de factura");
			Assert.AreEqual(preFacturaDto.remitos, preFactura.Remitos, "Remitos");
			Assert.AreEqual(preFacturaDto.observaciones, preFactura.Observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoPreFactura_PrecioProductoAsociadoCambia_PreFacturaNoCambia()
		{
			// Creo una PreFactura
			PreFactura preFacturaTemp = TestBase.CrearPreFactura();
			preFacturaTemp.Cliente = this.ClienteSinBonificacion;
			
			preFacturaTemp.Items.Clear();
			preFacturaTemp.Items.Add(TestBase.CrearItemPreFactura(this.ProductoMosaico, 1));
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTemp);
			preFacturaTemp.Create();
			
			this.FlushAndRecreateScope();
			
			preFacturaTemp = PreFactura.Find(preFacturaTemp.Id);
			ItemPreFactura itemProducto = preFacturaTemp.Items[0];
			
			// Paso la PreFactura a PreFacturaDto
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(preFacturaTemp);
			
			// Modifico el precio de un producto asociado
			Producto productoMosaico = this.ProductoMosaico;
			productoMosaico.ImporteSinIva += 10.0m;
			productoMosaico.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Vuelvo a pasar la PreFacturaDto a PreFactura
			// La PreFactura debería tener lo mismos precios
			PreFactura preFactura = ConvertidorDto.FromDto<PreFacturaDto,PreFactura>(preFacturaDto);
			
			// Items
			Assert.IsNotNull(preFactura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preFacturaDtoItems = preFacturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preFacturaDtoItems.Length, preFactura.Items.Count, "Items count");
			Assert.AreEqual(itemProducto.ProductoId, preFactura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(itemProducto.ProductoCodigo, preFactura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(itemProducto.ProductoDescripcion, preFactura.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(itemProducto.Descripcion, preFactura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(itemProducto.PrecioUnitario, preFactura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(itemProducto.ProductoImporteSinIva, preFactura.Items[0].ProductoImporteSinIva, "Item producto importe sin iva");
			Assert.AreEqual(itemProducto.ProductoImporteVenta, preFactura.Items[0].ProductoImporteVenta, "Item producto importe venta");
			Assert.AreEqual(itemProducto.PrecioUnitario, preFactura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(itemProducto.PorcentajeBonificacion, preFactura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(itemProducto.Cantidad, preFactura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(itemProducto.Importe, preFactura.Items[0].Importe, "Item importe");
		}
		#endregion
		
		#region PreNotaCredito
		[Test]
		public void ToDtoPreNotaCredito()
		{
			// Primero creo la factura que estará asociada con la nota de crédito
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo la nota de crédito
			PreNotaCredito preNotaCredito = TestBase.CrearPreNotaCredito();
			preNotaCredito.FacturaAsociada = Factura.Find(factura.Id);
			preNotaCredito.Cliente = this.ClienteSinBonificacion;
			
			preNotaCredito.Items.Clear();
			preNotaCredito.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			preNotaCredito.Items[0].Orden = 1;
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preNotaCredito);
			
			preNotaCredito.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la PreFactura a PreFacturaDto
			PreNotaCreditoDto preNotaCreditoDto =
				ConvertidorDto.ToDto<PreNotaCredito,PreNotaCreditoDto>(preNotaCredito);
			
			Assert.AreEqual(preNotaCredito.Id, preNotaCreditoDto.id, "Id");
			Assert.IsNotNull(preNotaCreditoDto.facturaAsociada, "Factura asociada no nula");
			Assert.AreEqual(factura.Id, preNotaCreditoDto.facturaAsociada.id, "Id factura asociada");
			Assert.AreEqual(preNotaCredito.Fecha.Ticks, preNotaCreditoDto.fechaTicks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preNotaCreditoDto.cliente, "Cliente no nulo");
			Assert.AreEqual(preNotaCredito.Cliente.RazonSocial, preNotaCreditoDto.cliente.razonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preNotaCreditoDto.items, "Items nulo");
			Assert.AreEqual(preNotaCredito.Items.Count, preNotaCreditoDto.items.Count, "Items count");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preNotaCreditoDtoItems = preNotaCreditoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preNotaCredito.Items.Count, preNotaCreditoDtoItems.Length, "Items count");
			Assert.AreEqual(preNotaCredito.Items[0].Id, preNotaCreditoDtoItems[0].id, "Item id");
			Assert.AreEqual(preNotaCredito.Items[0].ProductoId, preNotaCreditoDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(preNotaCredito.Items[0].ProductoCodigo, preNotaCreditoDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(preNotaCredito.Items[0].Descripcion, preNotaCreditoDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(preNotaCredito.Items[0].PrecioUnitario, preNotaCreditoDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(preNotaCredito.Items[0].PorcentajeBonificacion, preNotaCreditoDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(preNotaCredito.Items[0].Cantidad, preNotaCreditoDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(preNotaCredito.Items[0].Importe, preNotaCreditoDtoItems[0].importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preNotaCredito.ImporteBruto, preNotaCreditoDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(preNotaCredito.PorcentajeBonificacion, preNotaCreditoDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preNotaCredito.ImporteBonificacion, preNotaCreditoDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(preNotaCredito.ImporteNeto, preNotaCreditoDto.importeNeto, "Importe Neto");
			Assert.AreEqual(preNotaCredito.ImporteFinal, preNotaCreditoDto.importeFinal, "Importe Final");
			Assert.AreEqual(preNotaCredito.ImporteTotal, preNotaCreditoDto.importeTotal, "Importe Total");
			Assert.AreEqual(preNotaCredito.ImporteIva, preNotaCreditoDto.importeIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preNotaCredito.CondicionVenta.Condicion, preNotaCreditoDto.condVenta.condPago, "CondicionVenta - Condicion");
			Assert.AreEqual(preNotaCredito.CondicionVenta.Plazo, preNotaCreditoDto.condVenta.plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preNotaCredito.Tipo, preNotaCreditoDto.tipo, "Tipo de factura");
			Assert.AreEqual(preNotaCredito.Remitos, preNotaCreditoDto.remitos, "Remitos");
			Assert.AreEqual(preNotaCredito.Observaciones, preNotaCreditoDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoPreNotaCredito()
		{
			// Primero creo la factura que estará asociada con la nota de crédito
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo la nota de crédito
			PreNotaCredito preNotaCreditoTemp = TestBase.CrearPreNotaCredito();
			preNotaCreditoTemp.FacturaAsociada = Factura.Find(factura.Id);
			preNotaCreditoTemp.Cliente = this.ClienteSinBonificacion;
			
			preNotaCreditoTemp.Items.Clear();
			preNotaCreditoTemp.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			preNotaCreditoTemp.Items[0].Orden = 1;
			
			TestBase.ProcesarItemsYCalcularImportesFactura(preNotaCreditoTemp);
			
			preNotaCreditoTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Lo paso a Dto
			PreNotaCreditoDto preNotaCreditoDto =
				ConvertidorDto.ToDto<PreNotaCredito,PreNotaCreditoDto>(preNotaCreditoTemp);
			
			// Lo vuelvo a pasar a persistente
			PreNotaCredito preNotaCredito =
				ConvertidorDto.FromDto<PreNotaCreditoDto,PreNotaCredito>(preNotaCreditoDto);
			
			Assert.AreEqual(preNotaCreditoDto.id, preNotaCredito.Id, "Id");
			Assert.IsNotNull(preNotaCredito.FacturaAsociada, "Factura asociada no nula");
			Assert.AreEqual(preNotaCreditoDto.facturaAsociada.id, preNotaCredito.FacturaAsociada.Id, "Id factura asociada");
			Assert.AreEqual(preNotaCreditoDto.fechaTicks, preNotaCredito.Fecha.Ticks, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preNotaCredito.Cliente, "Cliente no nulo");
			Assert.AreEqual(preNotaCreditoDto.cliente.razonSocial, preNotaCredito.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(preNotaCredito.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] preNotaCreditoDtoItems = preNotaCreditoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(preNotaCreditoDtoItems.Length, preNotaCredito.Items.Count, "Items count");
			Assert.AreEqual(preNotaCreditoDtoItems[0].productoId, preNotaCredito.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preNotaCreditoDtoItems[0].productoCodigo, preNotaCredito.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(preNotaCreditoDtoItems[0].descripcion, preNotaCredito.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(preNotaCreditoDtoItems[0].precioUnitario, preNotaCredito.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(preNotaCreditoDtoItems[0].porcentajeBonificacion, preNotaCredito.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(preNotaCreditoDtoItems[0].cantidad, preNotaCredito.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preNotaCreditoDtoItems[0].importe, preNotaCredito.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(preNotaCreditoDto.importeBruto, preNotaCredito.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(preNotaCreditoDto.porcentajeBonificacion, preNotaCredito.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preNotaCreditoDto.importeBonificacion, preNotaCredito.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(preNotaCreditoDto.importeNeto, preNotaCredito.ImporteNeto, "Importe Neto");
			Assert.AreEqual(preNotaCreditoDto.importeFinal, preNotaCredito.ImporteFinal, "Importe Final");
			Assert.AreEqual(preNotaCreditoDto.importeTotal, preNotaCredito.ImporteTotal, "Importe Total");
			Assert.AreEqual(preNotaCreditoDto.importeIva, preNotaCredito.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preNotaCreditoDto.condVenta.condPago, preNotaCredito.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(preNotaCreditoDto.condVenta.plazo, preNotaCredito.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preNotaCreditoDto.tipo, preNotaCredito.Tipo, "Tipo de factura");
			Assert.AreEqual(preNotaCreditoDto.remitos, preNotaCredito.Remitos, "Remitos");
			Assert.AreEqual(preNotaCreditoDto.observaciones, preNotaCredito.Observaciones, "Observaciones");
		}
		#endregion
		
		#region Factura
		[Test]
		public void ToDtoFactura()
		{
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			NHibernate.NHibernateUtil.Initialize(factura.Items);
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(factura);
			
			Assert.AreEqual(factura.Id, facturaDto.id, "Id");
			Assert.AreEqual(factura.Fecha.Ticks, facturaDto.fechaTicks, "Fecha");
			Assert.AreEqual(factura.Codigo, facturaDto.codigo, "Codigo");
			Assert.AreEqual(factura.Numero, facturaDto.numero, "Numero");
			
			Assert.AreEqual(factura.Anulada, facturaDto.anulada, "Anulada");
			Assert.AreEqual(factura.Valida, facturaDto.valida, "Valida");
			Assert.AreEqual(factura.FacturaManual, facturaDto.facturaManual, "Factura manual");
			
			// Cliente
			Assert.IsNotNull(facturaDto.cliente, "Cliente no nulo");
			Assert.AreEqual(factura.Cliente.Codigo, facturaDto.cliente.codigo, "Cliente razón social");
			Assert.AreEqual(factura.ClienteRazonSocial, facturaDto.cliente.razonSocial, "Cliente razón social");
			Assert.AreEqual(factura.ClienteTipoResponsable, facturaDto.cliente.tipoRes, "Cliente tipo responsable");
			Assert.AreEqual(factura.ClienteCuit, Convert.ToInt64(facturaDto.cliente.cuit.Replace("-", "")), "Cliente cuit");
			
			// Items
			Assert.IsNotNull(facturaDto.items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(factura.Items.Count, facturaDtoItems.Length, "Items count");
			Assert.AreEqual(factura.Items[0].Id, facturaDtoItems[0].id, "Item id");
			Assert.AreEqual(factura.Items[0].ProductoId, facturaDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(factura.Items[0].ProductoCodigo, facturaDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(factura.Items[0].Descripcion, facturaDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(factura.Items[0].PrecioUnitario, facturaDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(factura.Items[0].PorcentajeBonificacion, facturaDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(factura.Items[0].Cantidad, facturaDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(factura.Items[0].Importe, facturaDtoItems[0].importe, "Item importe");
			
				
			// Importes
			Assert.AreEqual(factura.ImporteBruto, facturaDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(factura.PorcentajeBonificacion, facturaDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(factura.ImporteBonificacion, facturaDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(factura.ImporteNeto, facturaDto.importeNeto, "Importe Neto");
			Assert.AreEqual(factura.ImporteFinal, facturaDto.importeFinal, "Importe Final");
			Assert.AreEqual(factura.ImporteTotal, facturaDto.importeTotal, "Importe Total");
			Assert.AreEqual(factura.ImporteIva, facturaDto.importeIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(factura.CondicionVenta.Condicion, facturaDto.condVenta.condPago, "CondicionVenta - Condicion");
			Assert.AreEqual(factura.CondicionVenta.Plazo, facturaDto.condVenta.plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(factura.Tipo, facturaDto.tipo, "Tipo de factura");
			Assert.AreEqual(factura.Remitos, facturaDto.remitos, "Remitos");
			Assert.AreEqual(factura.Observaciones, facturaDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void ToDtoFactura_FacturaCreada_LuegoCambianDatosCliente_VersionDtoUsaDatosFijosDeCliente()
		{
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			
			// Copio los datos fijos de la factura
			string clienteRazonSocialFijo = factura.ClienteRazonSocial;
			TipoResponsable clienteTipoResponsableFijo = factura.ClienteTipoResponsable;
			long clienteCuitFijo = factura.ClienteCuit.Value;
			
			this.FlushAndRecreateScope();
			
			// Modifico los datos del cliente asociado a la factura.
			this.ClienteSinBonificacion.RazonSocial += "agregado";
			this.ClienteSinBonificacion.TipoResponsable = TipoResponsable.ResponsableNoInscripto;
			this.ClienteSinBonificacion.Cuit = new CUIT("20-31486846-4");
			this.ClienteSinBonificacion.Update();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la Factura a Dto
			factura = Factura.Find(factura.Id);
			NHibernate.NHibernateUtil.Initialize(factura.Items);
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(factura);
			
			Assert.AreEqual(factura.Id, facturaDto.id, "Id");
			Assert.AreEqual(factura.Fecha.Ticks, facturaDto.fechaTicks, "Fecha");
			Assert.AreEqual(factura.Codigo, facturaDto.codigo, "Codigo");
			Assert.AreEqual(factura.Numero, facturaDto.numero, "Numero");
			
			// Me aseguro que la versión Dto esté usando los datos fijos del cliente,
			// y no los datos variables (ya que el cliente cambió).
			Assert.IsNotNull(facturaDto.cliente, "Cliente no nulo");
			Assert.AreEqual(factura.Cliente.Codigo, facturaDto.cliente.codigo, "Cliente código");
			Assert.AreEqual(clienteRazonSocialFijo, facturaDto.cliente.razonSocial, "Cliente razón social");
			Assert.AreEqual(clienteTipoResponsableFijo, facturaDto.cliente.tipoRes, "Cliente tipo responsable");
			Assert.AreEqual(clienteCuitFijo, Convert.ToInt64(facturaDto.cliente.cuit.Replace("-", "")), "Cliente cuit");
		}
		
		[Test]
		public void FromDtoFactura()
		{
			Factura facturaTemp = TestBase.CrearFactura();
			facturaTemp.CondicionVenta.Condicion = CondicionPago.Contado;
			facturaTemp.Cliente = this.ClienteSinBonificacion;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(facturaTemp);
			facturaTemp.Create();
			
			this.FlushAndRecreateScope();
			
			facturaTemp.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateScope();
			
			this.MarcarComoNoPersistido(ref facturaTemp);
			FacturaDto facturaDto = ConvertidorDto.ToDto(facturaTemp);
			
			Factura factura = ConvertidorDto.FromDto<FacturaDto,Factura>(facturaDto);
			
			Assert.AreEqual(facturaDto.id, factura.Id, "Id");
			Assert.AreEqual(facturaDto.fechaTicks, factura.Fecha.Ticks, "Fecha");
			Assert.AreEqual(facturaDto.codigo, factura.Codigo);
			Assert.AreEqual(facturaDto.numero, factura.Numero);
			Assert.IsTrue(factura.Valida, "Valida");
			
			Assert.AreEqual(facturaDto.anulada, factura.Anulada, "Anulada");
			Assert.AreEqual(facturaDto.facturaManual, factura.FacturaManual, "Factura manual");
			Assert.AreEqual(facturaDto.historica, factura.Historica, "Solo lectura");
			
			// Cliente
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual(facturaDto.cliente.codigo, factura.Cliente.Codigo, "Cliente código");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(facturaDtoItems.Length, factura.Items.Count, "Items count");
			Assert.AreEqual(facturaDtoItems[0].productoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(facturaDtoItems[0].productoCodigo, factura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].descripcion, factura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(facturaDtoItems[0].precioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].porcentajeBonificacion, factura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(facturaDtoItems[0].cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(facturaDtoItems[0].importe, factura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(facturaDto.importeBruto, factura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(facturaDto.porcentajeBonificacion, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(facturaDto.importeBonificacion, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(facturaDto.importeNeto, factura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(facturaDto.importeFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(facturaDto.importeTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(facturaDto.importeIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(facturaDto.condVenta.condPago, factura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(facturaDto.condVenta.plazo, factura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(facturaDto.tipo, factura.Tipo, "Tipo de factura");
			Assert.AreEqual(facturaDto.remitos, factura.Remitos, "Remitos");
			Assert.AreEqual(facturaDto.observaciones, factura.Observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoFactura_FacturaExistente()
		{
			Factura facturaTemp = TestBase.CrearFactura();
			facturaTemp.Cliente = this.ClienteSinBonificacion;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(facturaTemp);
			facturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(facturaTemp);
			
			Factura factura = ConvertidorDto.FromDto<FacturaDto,Factura>(facturaDto);
			
			Assert.AreEqual(facturaDto.id, factura.Id, "Id");
			Assert.AreEqual(facturaDto.fechaTicks, factura.Fecha.Ticks, "Fecha");
			Assert.AreEqual(facturaDto.codigo, factura.Codigo);
			Assert.AreEqual(facturaDto.numero, factura.Numero);
			Assert.IsTrue(factura.Valida, "Valida");
			
			Assert.AreEqual(facturaDto.anulada, factura.Anulada);
			Assert.AreEqual(facturaDto.facturaManual, factura.FacturaManual);
			
			// Cliente
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual(facturaDto.cliente.codigo, factura.Cliente.Codigo, "Cliente código");
			Assert.AreEqual(facturaDto.cliente.razonSocial, factura.ClienteRazonSocial, "Cliente razón social");
			Assert.AreEqual(facturaDto.cliente.tipoRes, factura.ClienteTipoResponsable, "Cliente tipo responsable");
			Assert.AreEqual(Convert.ToInt64(facturaDto.cliente.cuit.Replace("-", "")), factura.ClienteCuit, "Cliente cuit");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(facturaDtoItems.Length, factura.Items.Count, "Items count");
			Assert.AreEqual(facturaDtoItems[0].productoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(facturaDtoItems[0].productoCodigo, factura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].descripcion, factura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(facturaDtoItems[0].precioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].porcentajeBonificacion, factura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(facturaDtoItems[0].cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(facturaDtoItems[0].importe, factura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(facturaDto.importeBruto, factura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(facturaDto.porcentajeBonificacion, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(facturaDto.importeBonificacion, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(facturaDto.importeNeto, factura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(facturaDto.importeFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(facturaDto.importeTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(facturaDto.importeIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(facturaDto.condVenta.condPago, factura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(facturaDto.condVenta.plazo, factura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(facturaDto.tipo, factura.Tipo, "Tipo de factura");
			Assert.AreEqual(facturaDto.remitos, factura.Remitos, "Remitos");
			Assert.AreEqual(facturaDto.observaciones, factura.Observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoFactura_ConProductoAbstracto()
		{
			Factura facturaTemp = TestBase.CrearFactura();
			facturaTemp.Cliente = this.ClienteSinBonificacion;
			facturaTemp.Items.Clear();
			
			ItemFactura itemTemp = TestBase.CrearItemFactura(this.ProductoAbstracto, 10);
			itemTemp.Descripcion += " agregado";
			itemTemp.PrecioUnitario = 1.00m;
			itemTemp.Importe = 1.00m;
			
			facturaTemp.Items.Add(itemTemp);
			
			TestBase.ProcesarItemsYCalcularImportesFactura(facturaTemp);
			facturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(facturaTemp);
			
			Factura factura = ConvertidorDto.FromDto<FacturaDto,Factura>(facturaDto);
			
			Assert.AreEqual(facturaDto.id, factura.Id, "Id");
			Assert.AreEqual(facturaDto.fechaTicks, factura.Fecha.Ticks, "Fecha");
			Assert.AreEqual(facturaDto.codigo, factura.Codigo);
			Assert.AreEqual(facturaDto.numero, factura.Numero);
			Assert.IsTrue(factura.Valida, "Valida");
			
			Assert.AreEqual(facturaDto.anulada, factura.Anulada);
			Assert.AreEqual(facturaDto.facturaManual, factura.FacturaManual);
			
			// Cliente
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual(facturaDto.cliente.razonSocial, factura.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(facturaDtoItems.Length, factura.Items.Count, "Items count");
			Assert.AreEqual(facturaDtoItems[0].productoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(facturaDtoItems[0].productoCodigo, factura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].descripcion, factura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(facturaDtoItems[0].precioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].porcentajeBonificacion, factura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(facturaDtoItems[0].cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(facturaDtoItems[0].importe, factura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(facturaDto.importeBruto, factura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(facturaDto.porcentajeBonificacion, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(facturaDto.importeBonificacion, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(facturaDto.importeNeto, factura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(facturaDto.importeFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(facturaDto.importeTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(facturaDto.importeIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(facturaDto.condVenta.condPago, factura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(facturaDto.condVenta.plazo, factura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(facturaDto.tipo, factura.Tipo, "Tipo de factura");
			Assert.AreEqual(facturaDto.remitos, factura.Remitos, "Remitos");
			Assert.AreEqual(facturaDto.observaciones, factura.Observaciones, "Observaciones");
		}
		
		[Test]
		public void ToFactura_From_PreFactura()
		{
			PreFactura preFacturaTemp = TestBase.CrearPreFactura();
			preFacturaTemp.Cliente = this.ClienteSinBonificacion;
			preFacturaTemp.Items.Clear();
			preFacturaTemp.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTemp);
			preFacturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			Factura factura = ConvertidorDto.ToFactura(preFacturaTemp);
			
			Assert.AreEqual(preFacturaTemp.Fecha, factura.Fecha, "Fecha");
			Assert.AreEqual(0, factura.Codigo);
			Assert.AreEqual(0, factura.Numero);
			Assert.IsTrue(factura.Valida, "Valida");
			
			Assert.AreEqual(false, factura.Anulada);
			
			// Cliente
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual(preFacturaTemp.Cliente.Codigo, factura.Cliente.Codigo, "Cliente código");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items nulo");
			Assert.AreEqual(preFacturaTemp.Items.Count, factura.Items.Count, "Items count");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoCodigo, factura.Items[0].ProductoCodigo, "Item producto codigo");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoDescripcion, factura.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoUnidad, factura.Items[0].ProductoUnidad, "Item producto unidad");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoImporteSinIva, factura.Items[0].ProductoImporteSinIva, "Item producto precio sin iva");
			Assert.AreEqual(preFacturaTemp.Items[0].ProductoImporteVenta, factura.Items[0].ProductoImporteVenta, "Item producto precio con iva");
			Assert.AreEqual(preFacturaTemp.Items[0].Cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preFacturaTemp.Items[0].Importe, factura.Items[0].Importe, "Item importe");
			Assert.AreEqual(preFacturaTemp.Items[0].PrecioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
				
			// Importes
			Assert.AreEqual(preFacturaTemp.ImporteBruto, factura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(preFacturaTemp.PorcentajeBonificacion, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preFacturaTemp.ImporteBonificacion, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(preFacturaTemp.ImporteNeto, factura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(preFacturaTemp.ImporteFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(preFacturaTemp.ImporteTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(preFacturaTemp.ImporteIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preFacturaTemp.CondicionVenta.Condicion, factura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(preFacturaTemp.CondicionVenta.Plazo, factura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preFacturaTemp.Tipo, factura.Tipo, "Tipo de factura");
			Assert.AreEqual(preFacturaTemp.Remitos, factura.Remitos, "Remitos");
			Assert.AreEqual(preFacturaTemp.Observaciones, factura.Observaciones, "Observaciones");
			
			// Vuelvo a crear una SessionScope de escritura para guardar la factura (rima y todo).
			this.FlushAndRecreateScope();
			
			// La Factura además debería poder guardarse
			try {
				factura.Codigo = 1;
				factura.Numero = 1;
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			preFacturaTemp = PreFactura.Find(preFacturaTemp.Id);
			factura = Factura.Find(factura.Id);
			
			// Una vez guardada, debería tener los datos fijos del cliente igual a los de
			// la pre factura.
			Assert.AreEqual(preFacturaTemp.Cliente.RazonSocial, factura.ClienteRazonSocial, "Cliente razón social");
			Assert.AreEqual(preFacturaTemp.Cliente.TipoResponsable, factura.ClienteTipoResponsable, "Cliente tipo responsable");
			Assert.AreEqual(preFacturaTemp.Cliente.Cuit.ValorNumerico.Value, factura.ClienteCuit, "Cliente cuit");
		}
		
		[Test]
		public void ToPreFactura_From_Factura()
		{
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			
			PreFactura preFactura = ConvertidorDto.ToPreFactura(factura);
			
			Assert.AreEqual(factura.Fecha, preFactura.Fecha, "Fecha");
			
			// Cliente
			Assert.IsNotNull(preFactura.Cliente, "Cliente no nulo");
			Assert.AreEqual(factura.Cliente.Codigo, preFactura.Cliente.Codigo, "Cliente codigo");
			
			// Items
			Assert.IsNotNull(preFactura.Items, "Items nulo");
			Assert.AreEqual(factura.Items.Count, preFactura.Items.Count, "Items count");
			Assert.AreEqual(factura.Items[0].ProductoId, preFactura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(factura.Items[0].ProductoCodigo, preFactura.Items[0].ProductoCodigo, "Item producto codigo");
			Assert.AreEqual(factura.Items[0].ProductoDescripcion, preFactura.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(factura.Items[0].ProductoUnidad, preFactura.Items[0].ProductoUnidad, "Item producto unidad");
			Assert.AreEqual(factura.Items[0].ProductoImporteSinIva, preFactura.Items[0].ProductoImporteSinIva, "Item producto precio sin iva");
			Assert.AreEqual(factura.Items[0].ProductoImporteVenta, preFactura.Items[0].ProductoImporteVenta, "Item producto precio con iva");
			Assert.AreEqual(factura.Items[0].Cantidad, preFactura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(factura.Items[0].Importe, preFactura.Items[0].Importe, "Item importe");
			Assert.AreEqual(factura.Items[0].PrecioUnitario, preFactura.Items[0].PrecioUnitario, "Item precio unitario");
				
			// Importes
			Assert.AreEqual(factura.ImporteBruto, preFactura.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(factura.PorcentajeBonificacion, preFactura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(factura.ImporteBonificacion, preFactura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(factura.ImporteNeto, preFactura.ImporteNeto, "Importe Neto");
			Assert.AreEqual(factura.ImporteFinal, preFactura.ImporteFinal, "Importe Final");
			Assert.AreEqual(factura.ImporteTotal, preFactura.ImporteTotal, "Importe Total");
			Assert.AreEqual(factura.ImporteIva, preFactura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(factura.CondicionVenta.Condicion, preFactura.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(factura.CondicionVenta.Plazo, preFactura.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(factura.Tipo, preFactura.Tipo, "Tipo de factura");
			Assert.AreEqual(factura.Remitos, preFactura.Remitos, "Remitos");
			Assert.AreEqual(factura.Observaciones, preFactura.Observaciones, "Observaciones");
			
			// Vuelvo a crear una SessionScope de escritura para guardar la factura (rima y todo).
			this.FlushAndRecreateScope();
			
			// La PreFactura además debería poder guardarse
			try {
				preFactura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
		}
		#endregion
		
		#region NotaCredito
		[Test]
		public void ToDtoNotaCredito()
		{
			// Creo primero la factura asociada a la nota de crédito
			Factura facturaTemp = TestBase.CrearFactura();
			facturaTemp.Cliente = this.ClienteSinBonificacion;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(facturaTemp);
			facturaTemp.Create();
			
			this.FlushAndRecreateScope();
			
			// Ahora creo la nota de crédito
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Codigo = facturaTemp.Codigo;
			notaCredito.Numero = facturaTemp.Numero + 1;
			notaCredito.FacturaAsociada = Factura.Find(facturaTemp.Id);
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			NotaCreditoDto notaCreditoDto = ConvertidorDto.ToDto<NotaCredito,NotaCreditoDto>(notaCredito);
			
			Assert.AreEqual(notaCredito.Id, notaCreditoDto.id, "Id");
			Assert.AreEqual(notaCredito.Fecha.Ticks, notaCreditoDto.fechaTicks, "Fecha");
			Assert.AreEqual(notaCredito.Codigo, notaCreditoDto.codigo);
			Assert.AreEqual(notaCredito.Numero, notaCreditoDto.numero);
			
			Assert.IsNotNull(notaCreditoDto.facturaAsociada, "Factura asociada no nula");
			Assert.AreEqual(notaCredito.FacturaAsociada.Id, notaCreditoDto.facturaAsociada.id, "Factura asociada id");
			
			Assert.AreEqual(notaCredito.Anulada, notaCreditoDto.anulada);
			Assert.AreEqual(notaCredito.Valida, notaCreditoDto.valida);
			Assert.AreEqual(notaCredito.FacturaManual, notaCreditoDto.facturaManual);
			
			// Cliente
			Assert.IsNotNull(notaCreditoDto.cliente, "Cliente no nulo");
			Assert.AreEqual(notaCredito.Cliente.RazonSocial, notaCreditoDto.cliente.razonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(notaCreditoDto.items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] notaCreditoDtoItems = notaCreditoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(notaCredito.Items.Count, notaCreditoDtoItems.Length, "Items count");
			Assert.AreEqual(notaCredito.Items[0].Id, notaCreditoDtoItems[0].id, "Item id");
			Assert.AreEqual(notaCredito.Items[0].ProductoId, notaCreditoDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(notaCredito.Items[0].ProductoCodigo, notaCreditoDtoItems[0].productoCodigo, "Item producto código");
			Assert.AreEqual(notaCredito.Items[0].Descripcion, notaCreditoDtoItems[0].descripcion, "Item producto descripción");
			Assert.AreEqual(notaCredito.Items[0].PrecioUnitario, notaCreditoDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(notaCredito.Items[0].PorcentajeBonificacion, notaCreditoDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificación");
			Assert.AreEqual(notaCredito.Items[0].Cantidad, notaCreditoDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(notaCredito.Items[0].Importe, notaCreditoDtoItems[0].importe, "Item importe");
				
			// Importes
			Assert.AreEqual(notaCredito.ImporteBruto, notaCreditoDto.importeBruto, "Importe Bruto");
			Assert.AreEqual(notaCredito.PorcentajeBonificacion, notaCreditoDto.porcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(notaCredito.ImporteBonificacion, notaCreditoDto.importeBonificacion, "Importe bonificación");
			Assert.AreEqual(notaCredito.ImporteNeto, notaCreditoDto.importeNeto, "Importe Neto");
			Assert.AreEqual(notaCredito.ImporteFinal, notaCreditoDto.importeFinal, "Importe Final");
			Assert.AreEqual(notaCredito.ImporteTotal, notaCreditoDto.importeTotal, "Importe Total");
			Assert.AreEqual(notaCredito.ImporteIva, notaCreditoDto.importeIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(notaCredito.CondicionVenta.Condicion, notaCreditoDto.condVenta.condPago, "CondicionVenta - Condicion");
			Assert.AreEqual(notaCredito.CondicionVenta.Plazo, notaCreditoDto.condVenta.plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(notaCredito.Tipo, notaCreditoDto.tipo, "Tipo de factura");
			Assert.AreEqual(notaCredito.Remitos, notaCreditoDto.remitos, "Remitos");
			Assert.AreEqual(notaCredito.Observaciones, notaCreditoDto.observaciones, "Observaciones");
		}
		
		[Test]
		public void FromDtoNotaCredito()
		{
			// Creo primero la factura asociada a la nota de crédito
			Factura facturaTemp = TestBase.CrearFactura();
			facturaTemp.Cliente = this.ClienteSinBonificacion;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(facturaTemp);
			facturaTemp.Create();
			
			this.FlushAndRecreateScope();
			
			// Ahora creo la nota de crédito
			NotaCredito notaCreditoTemp = TestBase.CrearNotaCredito();
			notaCreditoTemp.Codigo = facturaTemp.Codigo;
			notaCreditoTemp.Numero = facturaTemp.Numero + 1;
			notaCreditoTemp.FacturaAsociada = Factura.Find(facturaTemp.Id);
			notaCreditoTemp.Cliente = this.ClienteSinBonificacion;
			notaCreditoTemp.Items.Clear();
			notaCreditoTemp.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCreditoTemp);
			notaCreditoTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			NotaCreditoDto notaCreditoDto = ConvertidorDto.ToDto<NotaCredito,NotaCreditoDto>(notaCreditoTemp);
			
			NotaCredito notaCredito = ConvertidorDto.FromDto<NotaCreditoDto,NotaCredito>(notaCreditoDto);
			
			Assert.AreEqual(notaCreditoDto.id, notaCredito.Id, "Id");
			Assert.AreEqual(notaCreditoDto.fechaTicks, notaCredito.Fecha.Ticks, "Fecha");
			Assert.AreEqual(notaCreditoDto.codigo, notaCredito.Codigo);
			Assert.AreEqual(notaCreditoDto.numero, notaCredito.Numero);
			Assert.IsTrue(notaCredito.Valida, "Valida");
			
			Assert.IsNotNull(notaCredito.FacturaAsociada);
			Assert.AreEqual(notaCredito.FacturaAsociada.Id, notaCreditoDto.facturaAsociada.id);
			
			Assert.AreEqual(notaCreditoDto.anulada, notaCredito.Anulada);
			Assert.AreEqual(notaCreditoDto.facturaManual, notaCredito.FacturaManual);
			
			// Cliente
			Assert.IsNotNull(notaCredito.Cliente, "Cliente no nulo");
			Assert.AreEqual(notaCreditoDto.cliente.razonSocial, notaCredito.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(notaCredito.Items, "Items nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] notaCreditoDtoItems = notaCreditoDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(notaCreditoDtoItems.Length, notaCredito.Items.Count, "Items count");
			Assert.AreEqual(notaCreditoDtoItems[0].productoId, notaCredito.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(notaCreditoDtoItems[0].productoCodigo, notaCredito.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(notaCreditoDtoItems[0].descripcion, notaCredito.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(notaCreditoDtoItems[0].precioUnitario, notaCredito.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(notaCreditoDtoItems[0].porcentajeBonificacion, notaCredito.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(notaCreditoDtoItems[0].cantidad, notaCredito.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(notaCreditoDtoItems[0].importe, notaCredito.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(notaCreditoDto.importeBruto, notaCredito.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(notaCreditoDto.porcentajeBonificacion, notaCredito.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(notaCreditoDto.importeBonificacion, notaCredito.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(notaCreditoDto.importeNeto, notaCredito.ImporteNeto, "Importe Neto");
			Assert.AreEqual(notaCreditoDto.importeFinal, notaCredito.ImporteFinal, "Importe Final");
			Assert.AreEqual(notaCreditoDto.importeTotal, notaCredito.ImporteTotal, "Importe Total");
			Assert.AreEqual(notaCreditoDto.importeIva, notaCredito.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(notaCreditoDto.condVenta.condPago, notaCredito.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(notaCreditoDto.condVenta.plazo, notaCredito.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(notaCreditoDto.tipo, notaCredito.Tipo, "Tipo de factura");
			Assert.AreEqual(notaCreditoDto.remitos, notaCredito.Remitos, "Remitos");
			Assert.AreEqual(notaCreditoDto.observaciones, notaCredito.Observaciones, "Observaciones");
		}
		
		[Test]
		public void ToNotaCredito_From_PreNotaCredito()
		{
			// Primero creo la factura que estará asociada con la nota de crédito
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo la nota de crédito
			PreNotaCredito preNotaCreditoTemp = TestBase.CrearPreNotaCredito();
			preNotaCreditoTemp.FacturaAsociada = Factura.Find(factura.Id);
			preNotaCreditoTemp.Cliente = this.ClienteSinBonificacion;
			preNotaCreditoTemp.Items.Clear();
			preNotaCreditoTemp.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			preNotaCreditoTemp.Items[0].Orden = 1;
			TestBase.ProcesarItemsYCalcularImportesFactura(preNotaCreditoTemp);
			preNotaCreditoTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			// Ahora convierto la PreNotaCredito a NotaCredito
			NotaCredito notaCredito = ConvertidorDto.ToNotaCredito(preNotaCreditoTemp);
			
			Assert.AreEqual(preNotaCreditoTemp.Fecha, notaCredito.Fecha, "Fecha");
			Assert.AreEqual(0, notaCredito.Codigo);
			Assert.AreEqual(0, notaCredito.Numero);
			Assert.IsTrue(notaCredito.Valida, "Valida");
			
			Assert.IsNotNull(notaCredito.FacturaAsociada);
			Assert.AreEqual(notaCredito.FacturaAsociada.Id, preNotaCreditoTemp.FacturaAsociada.Id);
			
			Assert.AreEqual(false, notaCredito.Anulada);
			
			// Cliente
			Assert.IsNotNull(notaCredito.Cliente, "Cliente no nulo");
			Assert.AreEqual(preNotaCreditoTemp.Cliente.RazonSocial, notaCredito.Cliente.RazonSocial, "Cliente razón social");
			
			// Items
			Assert.IsNotNull(notaCredito.Items, "Items nulo");
			Assert.AreEqual(preNotaCreditoTemp.Items.Count, notaCredito.Items.Count, "Items count");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoId, notaCredito.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoCodigo, notaCredito.Items[0].ProductoCodigo, "Item producto codigo");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoDescripcion, notaCredito.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoUnidad, notaCredito.Items[0].ProductoUnidad, "Item producto unidad");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoImporteSinIva, notaCredito.Items[0].ProductoImporteSinIva, "Item producto precio sin iva");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].ProductoImporteVenta, notaCredito.Items[0].ProductoImporteVenta, "Item producto precio con iva");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].Cantidad, notaCredito.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].Importe, notaCredito.Items[0].Importe, "Item importe");
			Assert.AreEqual(preNotaCreditoTemp.Items[0].PrecioUnitario, notaCredito.Items[0].PrecioUnitario, "Item precio unitario");
				
			// Importes
			Assert.AreEqual(preNotaCreditoTemp.ImporteBruto, notaCredito.ImporteBruto, "Importe Bruto");
			Assert.AreEqual(preNotaCreditoTemp.PorcentajeBonificacion, notaCredito.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(preNotaCreditoTemp.ImporteBonificacion, notaCredito.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(preNotaCreditoTemp.ImporteNeto, notaCredito.ImporteNeto, "Importe Neto");
			Assert.AreEqual(preNotaCreditoTemp.ImporteFinal, notaCredito.ImporteFinal, "Importe Final");
			Assert.AreEqual(preNotaCreditoTemp.ImporteTotal, notaCredito.ImporteTotal, "Importe Total");
			Assert.AreEqual(preNotaCreditoTemp.ImporteIva, notaCredito.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(preNotaCreditoTemp.CondicionVenta.Condicion, notaCredito.CondicionVenta.Condicion, "CondicionVenta - Condicion");
			Assert.AreEqual(preNotaCreditoTemp.CondicionVenta.Plazo, notaCredito.CondicionVenta.Plazo, "CondicionVenta - Plazo");
				
			Assert.AreEqual(preNotaCreditoTemp.Tipo, notaCredito.Tipo, "Tipo de factura");
			Assert.AreEqual(preNotaCreditoTemp.Remitos, notaCredito.Remitos, "Remitos");
			Assert.AreEqual(preNotaCreditoTemp.Observaciones, notaCredito.Observaciones, "Observaciones");
			
			// Vuelvo a crear una SessionScope para guardar la nota de crédito
			this.FlushAndRecreateScope();
			
			// La notaCredito además debería poder guardarse
			try {
				notaCredito.Codigo = factura.Codigo;
				notaCredito.Numero = factura.Numero + 1;
				notaCredito.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
		}
		#endregion
		
		#region FacturaCompra
		[Test]
		public void ToDtoFacturaCompra()
		{
			FacturaCompra factura = TestBase.CrearFacturaCompra();
			factura.Proveedor = this.ProveedorFerrum;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFacturaCompra(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(factura, true);
			factura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaCompraDto facturaDto = ConvertidorDto.ToDto(factura);
			
			Assert.AreEqual(factura.Id, facturaDto.id, "Id");
			
			// Fechas
			Assert.AreEqual(factura.Fecha.Ticks, facturaDto.fechaTicks, "Fecha");
			Assert.AreEqual(factura.FechaImputacion.Ticks, facturaDto.fechaImputacionTicks, "Fecha Imputacion");
			
			// Cabecera
			Assert.AreEqual(factura.Tipo, facturaDto.tipo, "Tipo");
			Assert.AreEqual(factura.Codigo, facturaDto.codigo, "Código");
			Assert.AreEqual(factura.Numero, facturaDto.numero, "Número");

			// Proveedor
			Assert.IsNotNull(facturaDto.proveedor, "Proveedor no nulo");
			Assert.AreEqual(factura.Proveedor.Codigo, facturaDto.proveedor.codigo, "Proveedor código");
			Assert.AreEqual(factura.ProveedorRazonSocial, facturaDto.proveedor.razonSocial, "Proveedor razón social");
			Assert.AreEqual(factura.ProveedorTipoResponsable, facturaDto.proveedor.tipoRes, "Proveedor tipo responsable");
			Assert.AreEqual(factura.ProveedorCuit, Convert.ToInt64(facturaDto.proveedor.cuit.Replace("-", "")), "Proveedor cuit");
			
			// Items
			Assert.IsNotNull(facturaDto.items, "Items no nulo");
			
			// Conviero los items de FacturaCompraDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(factura.Items.Count, facturaDtoItems.Length, "Items count");
			Assert.AreEqual(factura.Items[0].ProductoId, facturaDtoItems[0].productoId, "Item producto id");
			Assert.AreEqual(factura.Items[0].ProductoCodigo, facturaDtoItems[0].productoCodigo, "Item precio unitario");
			Assert.AreEqual(factura.Items[0].Descripcion, facturaDtoItems[0].descripcion, "Item descripción");
			Assert.AreEqual(factura.Items[0].PrecioUnitario, facturaDtoItems[0].precioUnitario, "Item precio unitario");
			Assert.AreEqual(factura.Items[0].PorcentajeBonificacion, facturaDtoItems[0].porcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(factura.Items[0].Cantidad, facturaDtoItems[0].cantidad, "Item cantidad");
			Assert.AreEqual(factura.Items[0].Importe, facturaDtoItems[0].importe, "Item importe");
				
			// Importes
			Assert.AreEqual(factura.ImporteGravado, facturaDto.importeGravado, "Importe Gravado");
			Assert.AreEqual(factura.ImporteExento, facturaDto.importeExento, "Exento");
			Assert.AreEqual(factura.ImporteIva, facturaDto.importeIva, "Importe IVA");
			Assert.AreEqual(factura.PercepcionIB, facturaDto.percepcionIB, "Percepcion IB");
			Assert.AreEqual(factura.PercepcionIva, facturaDto.percepcionIva, "Percepción IVA");
			Assert.AreEqual(factura.ImpuestoCombustible, facturaDto.impuestoCombustible, "Imp Combustible");
			Assert.AreEqual(factura.ImporteFinal, facturaDto.importeFinal, "Importe Final");
			Assert.AreEqual(factura.ImporteTotal, facturaDto.importeTotal, "Importe Total");
			
			// Otros
			Assert.AreEqual(factura.Observaciones, facturaDto.observaciones, "Observaciones");
			Assert.AreEqual(factura.Historica, facturaDto.historica, "Historica");
		}
		
		[Test]
		public void ToDtoFacturaCompra_SinProveedorExistenteAsignado()
		{
			FacturaCompra factura = TestBase.CrearFacturaCompra();
			factura.Proveedor = this.ProveedorFerrum; // hago que que se copien los datos fijos del proveedor
			factura.Proveedor = null; // quito la asociación al proveedor
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFacturaCompra(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(factura, true);
			factura.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaCompraDto facturaDto = ConvertidorDto.ToDto(factura);
			
			Assert.AreEqual(factura.Id, facturaDto.id, "Id");
			
			// Fechas
			Assert.AreEqual(factura.Fecha.Ticks, facturaDto.fechaTicks, "Fecha");
			Assert.AreEqual(factura.FechaImputacion.Ticks, facturaDto.fechaImputacionTicks, "Fecha Imputacion");
			
			// Cabecera
			Assert.AreEqual(factura.Tipo, facturaDto.tipo, "Tipo");
			Assert.AreEqual(factura.Codigo, facturaDto.codigo, "Código");
			Assert.AreEqual(factura.Numero, facturaDto.numero, "Número");

			// Proveedor
			Assert.IsNotNull(facturaDto.proveedor, "Proveedor no nulo");
			Assert.AreEqual(factura.ProveedorRazonSocial, facturaDto.proveedor.razonSocial, "Proveedor razón social");
			Assert.AreEqual(factura.ProveedorTipoResponsable, facturaDto.proveedor.tipoRes, "Proveedor tipo responsable");
			Assert.AreEqual(factura.ProveedorCuit, Convert.ToInt64(facturaDto.proveedor.cuit.Replace("-", "")), "Proveedor cuit");
			
			Assert.IsNotNull(facturaDto.proveedor.domicilio, "Domicilio no nulo");
			Assert.AreEqual(ProvinciaDto.NoEspecificada, facturaDto.proveedor.domicilio.provincia, "Domicilio provincia no especificada");
			
			// No hace falta validar los otros datos...
		}
		
		[Test]
		public void FromDtoFacturaCompra_FacturaCompraExistente()
		{
			FacturaCompra facturaTemp = TestBase.CrearFacturaCompra();
			facturaTemp.Proveedor = this.ProveedorFerrum;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFacturaCompra(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(facturaTemp, true);
			facturaTemp.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaCompraDto facturaDto = ConvertidorDto.ToDto(facturaTemp);
			
			FacturaCompra factura = ConvertidorDto.FromDto<FacturaCompraDto,FacturaCompra>(facturaDto);
			
			Assert.AreEqual(facturaDto.id, factura.Id, "Id");
			
			// Fechas
			Assert.AreEqual(facturaDto.fechaTicks, factura.Fecha.Ticks, "Fecha");
			Assert.AreEqual(facturaDto.fechaImputacionTicks, factura.FechaImputacion.Ticks, "Fecha imputación");
			
			// Cabecera
			Assert.AreEqual(facturaDto.tipo, factura.Tipo, "Tipo");
			Assert.AreEqual(facturaDto.codigo, factura.Codigo, "Código");
			Assert.AreEqual(facturaDto.numero, factura.Numero, "Número");

			// Proveedor
			Assert.IsNotNull(factura.Proveedor, "Proveedor no nulo");
			Assert.AreEqual(facturaDto.proveedor.codigo, factura.Proveedor.Codigo, "Proveedor código");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items no nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(facturaDtoItems.Length, factura.Items.Count, "Items count");
			Assert.AreEqual(facturaDtoItems[0].productoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(facturaDtoItems[0].productoCodigo, factura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].descripcion, factura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(facturaDtoItems[0].precioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].porcentajeBonificacion, factura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(facturaDtoItems[0].cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(facturaDtoItems[0].importe, factura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(facturaDto.importeGravado, factura.ImporteGravado, "Gravado");
			Assert.AreEqual(facturaDto.importeExento, factura.ImporteExento, "Exento");
			Assert.AreEqual(facturaDto.percepcionIB, factura.PercepcionIB, "Percepcion IB");
			Assert.AreEqual(facturaDto.percepcionIva, factura.PercepcionIva, "Percepción IVA");
			Assert.AreEqual(facturaDto.impuestoCombustible, factura.ImpuestoCombustible, "Imp Combustible");
			Assert.AreEqual(facturaDto.importeFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(facturaDto.importeTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(facturaDto.importeIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(facturaDto.observaciones, factura.Observaciones, "Observaciones");
			Assert.AreEqual(facturaDto.historica, factura.Historica, "Historica");
		}
		
		[Test]
		public void FromDtoFacturaCompra()
		{
			FacturaCompra facturaTemp = TestBase.CrearFacturaCompra();
			facturaTemp.Proveedor = this.ProveedorFerrum;
			facturaTemp.Items.Clear();
			facturaTemp.Items.Add(TestBase.CrearItemFacturaCompra(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(facturaTemp, true);
			
			try {
				facturaTemp.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			this.FlushAndRecreateScope();
			
			facturaTemp.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			this.MarcarComoNoPersistido(ref facturaTemp);
			FacturaCompraDto facturaDto = ConvertidorDto.ToDto(facturaTemp);
			
			FacturaCompra factura = ConvertidorDto.FromDto<FacturaCompraDto,FacturaCompra>(facturaDto);
			
			Assert.AreEqual(facturaDto.id, factura.Id, "Id");
			
			// Fechas
			Assert.AreEqual(facturaDto.fechaTicks, factura.Fecha.Ticks, "Fecha");
			Assert.AreEqual(facturaDto.fechaImputacionTicks, factura.FechaImputacion.Ticks, "Fecha imputación");
			
			// Cabecera
			Assert.AreEqual(facturaDto.tipo, factura.Tipo, "Tipo");
			Assert.AreEqual(facturaDto.codigo, factura.Codigo, "Código");
			Assert.AreEqual(facturaDto.numero, factura.Numero, "Número");

			// Proveedor
			Assert.IsNotNull(factura.Proveedor, "Proveedor no nulo");
			Assert.AreEqual(facturaDto.proveedor.codigo, factura.Proveedor.Codigo, "Proveedor código");
			
			// Items
			Assert.IsNotNull(factura.Items, "Items no nulo");
			
			// Conviero los items de PresupuestoDto, que son de tipo ItemGenericoDto, a ItemProductoDto.
			ItemProductoDto[] facturaDtoItems = facturaDto.items.Cast<ItemProductoDto>().ToArray();
			
			Assert.AreEqual(facturaDtoItems.Length, factura.Items.Count, "Items count");
			Assert.AreEqual(facturaDtoItems[0].productoId, factura.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(facturaDtoItems[0].productoCodigo, factura.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].descripcion, factura.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(facturaDtoItems[0].precioUnitario, factura.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(facturaDtoItems[0].porcentajeBonificacion, factura.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(facturaDtoItems[0].cantidad, factura.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(facturaDtoItems[0].importe, factura.Items[0].Importe, "Item importe");
				
			// Importes
			Assert.AreEqual(facturaDto.importeGravado, factura.ImporteGravado, "Gravado");
			Assert.AreEqual(facturaDto.importeExento, factura.ImporteExento, "Exento");
			Assert.AreEqual(facturaDto.percepcionIB, factura.PercepcionIB, "Percepcion IB");
			Assert.AreEqual(facturaDto.percepcionIva, factura.PercepcionIva, "Percepción IVA");
			Assert.AreEqual(facturaDto.impuestoCombustible, factura.ImpuestoCombustible, "Imp Combustible");
			Assert.AreEqual(facturaDto.importeFinal, factura.ImporteFinal, "Importe Final");
			Assert.AreEqual(facturaDto.importeTotal, factura.ImporteTotal, "Importe Total");
			Assert.AreEqual(facturaDto.importeIva, factura.ImporteIva, "Importe IVA");
			
			// Otros
			Assert.AreEqual(facturaDto.observaciones, factura.Observaciones, "Observaciones");
			Assert.AreEqual(facturaDto.historica, factura.Historica, "Historica");
		}
		
		[Test]
		public void FromDtoFacturaCompra_SinProveedorExistenteAsignado()
		{
			FacturaCompra factura = TestBase.CrearFacturaCompra();
			factura.Proveedor = this.ProveedorFerrum; // hago que que se copien los datos fijos del proveedor
			factura.Proveedor = null; // quito la asociación al proveedor
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFacturaCompra(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(factura, true);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			factura.Delete();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaCompraDto facturaDto = ConvertidorDto.ToDto(factura);
			
			facturaDto.proveedor = new ProveedorDto();
			facturaDto.proveedor.razonSocial = "Pepe Biondi";
			facturaDto.proveedor.tipoRes = TipoResponsable.Monotributo;
			facturaDto.proveedor.cuit = "20-31486846-4";
			
			factura = ConvertidorDto.FromDto<FacturaCompraDto,FacturaCompra>(facturaDto);
			
			Assert.AreEqual(factura.Id, facturaDto.id, "Id");

			// Proveedor
			Assert.IsNull(factura.Proveedor, "Proveedor nulo");
			Assert.AreEqual(facturaDto.proveedor.razonSocial, factura.ProveedorRazonSocial, "Proveedor razón social");
			Assert.AreEqual(facturaDto.proveedor.tipoRes, factura.ProveedorTipoResponsable, "Proveedor tipo responsable");
			Assert.AreEqual(Convert.ToInt64(facturaDto.proveedor.cuit.Replace("-", "")), factura.ProveedorCuit, "Proveedor cuit");
			
			// No hace falta validar los otros datos...
		}
		#endregion
		
		#region Cheque
		[Test]
		public void ToDtoCheque()
		{
			// Agrego un cheque
			Cheque cheque = new Cheque();
			cheque.Banco = "El banco";
			cheque.Sucursal = "Avellaneda";
			cheque.FechaEmision = DateTime.Today.Date;
			cheque.FechaVencimiento = DateTime.Today + TimeSpan.FromDays(10);
			cheque.Importe = 150.75m;
			cheque.Numero = 123456;
			cheque.Observacion = "Observación";
			
			cheque.Create();
			
			this.FlushAndRecreateScope();
			
			ChequeDto chequeDto = ConvertidorDto.ToDto(cheque);
			
			Assert.AreEqual(cheque.Id, chequeDto.id, "Cheque id");
			Assert.AreEqual(cheque.Banco, chequeDto.banco, "Cheque banco");
			Assert.AreEqual(cheque.Sucursal, chequeDto.sucursal, "Cheque sucursal");
			Assert.AreEqual(cheque.FechaEmision.Ticks, chequeDto.fechaEmisionTicks, "Cheque fecha emision");
			Assert.AreEqual(cheque.FechaVencimiento.Ticks, chequeDto.fechaVencimientoTicks, "Cheque fecha vencimiento");
			Assert.AreEqual(cheque.Importe, chequeDto.importe, "Cheque importe");
			Assert.AreEqual(cheque.Observacion, chequeDto.observacion, "Cheque observacion");
			Assert.AreEqual(cheque.Numero, chequeDto.numero, "Cheque número");
		}
		
		[Test]
		public void FromDtoCheque()
		{
			// Agrego un cheque
			Cheque chequeTemp = new Cheque();
			chequeTemp.Banco = "El banco";
			chequeTemp.Sucursal = "Avellaneda";
			chequeTemp.FechaEmision = DateTime.Today.Date;
			chequeTemp.FechaVencimiento = DateTime.Today + TimeSpan.FromDays(10);
			chequeTemp.Importe = 150.75m;
			chequeTemp.Numero = 123456;
			chequeTemp.Observacion = "Observación";
			
			chequeTemp.Create();
			
			this.FlushAndRecreateScope();
			
			ChequeDto chequeDto = ConvertidorDto.ToDto(chequeTemp);
			Cheque cheque = ConvertidorDto.FromDto(chequeDto);
			
			Assert.AreEqual(chequeDto.id, cheque.Id, "Cheque id");
			Assert.AreEqual(chequeDto.banco, cheque.Banco, "Cheque banco");
			Assert.AreEqual(chequeDto.sucursal, cheque.Sucursal, "Cheque sucursal");
			Assert.AreEqual(chequeDto.fechaEmisionTicks, cheque.FechaEmision.Ticks, "Cheque fecha emision");
			Assert.AreEqual(chequeDto.fechaVencimientoTicks, cheque.FechaVencimiento.Ticks, "Cheque fecha vencimiento");
			Assert.AreEqual(chequeDto.importe, cheque.Importe, "Cheque importe");
			Assert.AreEqual(chequeDto.observacion, cheque.Observacion, "Cheque observacion");
			Assert.AreEqual(chequeDto.numero, cheque.Numero, "Cheque número");
		}
		#endregion
		
		#region Recibos
		[Test]
		public void ToDtoRecibo_DesdeReciboExistente()
		{
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 100));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			
			// Agrego un cheque
			Cheque cheque = new Cheque();
			cheque.Banco = "El banco";
			cheque.Sucursal = "Avellaneda";
			cheque.FechaEmision = DateTime.Today.Date;
			cheque.FechaVencimiento = DateTime.Today + TimeSpan.FromDays(10);
			cheque.Importe = 150.75m;
			cheque.Numero = 123456;
			cheque.Observacion = "Observación";
			
			recibo.Items.Add(new ItemRecibo(cheque));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
//			recibo.AgregarFacturaAsociada(Factura.Find(factura.Id));
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			recibo = Recibo.Find(recibo.Id);
			
			ReciboDto reciboDto = ConvertidorDto.ToDto<Recibo,ReciboDto>(recibo);
			
			Assert.AreEqual(recibo.Id, reciboDto.id, "Id");
			Assert.AreEqual(recibo.Fecha.Ticks, reciboDto.fechaTicks, "Fecha");
			Assert.AreEqual(recibo.Numero, reciboDto.numero);
			
			Assert.IsNotNull(reciboDto.cliente, "Recibo cliente no nulo");
			Assert.AreEqual(recibo.Cliente.Codigo, reciboDto.cliente.codigo, "Cliente código");
			
			Assert.AreEqual(recibo.Concepto, reciboDto.concepto, "Recibo concepto");
			Assert.AreEqual(recibo.Observaciones, reciboDto.observaciones, "Recibo observaciones");
			
			// Items
			Assert.IsNotNull(reciboDto.items, "Items no nulos");
			
			ItemReciboDto[] reciboDtoItems = reciboDto.items.Cast<ItemReciboDto>().ToArray();
			
			Assert.AreEqual(recibo.Items.Count, reciboDtoItems.Length, "Items count");
			Assert.AreEqual(recibo.Items[0].Id, reciboDtoItems[0].id, "Item id");
			Assert.AreEqual(recibo.Items[0].MedioPago, reciboDtoItems[0].medioPag, "Item Medio pago");
			
			Assert.IsNotNull(reciboDtoItems[0].cheque, "Item cheque no nulo");
			Assert.AreEqual(recibo.Items[0].Cheque.Id, reciboDtoItems[0].cheque.id, "Item cheque id");
			
			Assert.AreEqual(recibo.Items[0].Orden, reciboDtoItems[0].orden, "Item orden");
			Assert.AreEqual(recibo.Items[0].Importe, reciboDtoItems[0].importe, "Item Importe");
			
			Assert.AreEqual(recibo.Importe, reciboDto.importe, "Recibo importe");
		}
		
		[Test]
		public void ToDtoRecibo_DesdeRecibo_Que_NoExiste()
		{
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 100));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			
			// Agrego un cheque
			Cheque cheque = new Cheque();
			cheque.Banco = "El banco";
			cheque.Sucursal = "Avellaneda";
			cheque.FechaEmision = DateTime.Today.Date;
			cheque.FechaVencimiento = DateTime.Today + TimeSpan.FromDays(10);
			cheque.Importe = 150.75m;
			cheque.Numero = 123456;
			cheque.Observacion = "Observación";
			
			recibo.Items.Add(new ItemRecibo(cheque));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
//			recibo.AgregarFacturaAsociada(Factura.Find(factura.Id));
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Ahora mapeo el recibo a ReciboDto y lo borro luego
			Recibo reciboTemp = Recibo.Find(recibo.Id);
			
			this.MarcarComoNoPersistido(ref recibo);
			ReciboDto reciboDto = ConvertidorDto.ToDto<Recibo,ReciboDto>(recibo);
			
			reciboTemp.Delete();
			reciboTemp.Items[0].Cheque.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(recibo.Id, reciboDto.id, "Id");
			Assert.AreEqual(recibo.Fecha.Ticks, reciboDto.fechaTicks, "Fecha");
			Assert.AreEqual(recibo.Numero, reciboDto.numero);
			
			Assert.IsNotNull(reciboDto.cliente, "Recibo cliente no nulo");
			Assert.AreEqual(recibo.Cliente.Codigo, reciboDto.cliente.codigo, "Cliente código");
			
			Assert.AreEqual(recibo.Concepto, reciboDto.concepto, "Recibo concepto");
			Assert.AreEqual(recibo.Observaciones, reciboDto.observaciones, "Recibo observaciones");
			
			// Items
			Assert.IsNotNull(reciboDto.items, "Items no nulos");
			
			ItemReciboDto[] reciboDtoItems = reciboDto.items.Cast<ItemReciboDto>().ToArray();
			
			Assert.AreEqual(recibo.Items.Count, reciboDtoItems.Length, "Items count");
			Assert.AreEqual(recibo.Items[0].Id, reciboDtoItems[0].id, "Item id");
			Assert.AreEqual(recibo.Items[0].MedioPago, reciboDtoItems[0].medioPag, "Item Medio pago");
			
			Assert.IsNotNull(reciboDtoItems[0].cheque, "Item cheque no nulo");
			Assert.AreEqual(recibo.Items[0].Cheque.Id, reciboDtoItems[0].cheque.id, "Item cheque id");
			
			Assert.AreEqual(recibo.Items[0].Orden, reciboDtoItems[0].orden, "Item orden");
			Assert.AreEqual(recibo.Items[0].Importe, reciboDtoItems[0].importe, "Item Importe");
			
			Assert.AreEqual(recibo.Importe, reciboDto.importe, "Recibo importe");
		}
		
		[Test]
		public void FromDtoRecibo_ItemCheque()
		{
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo reciboTemp = TestBase.CrearRecibo();
			reciboTemp.Cliente = this.ClienteSinBonificacion;
			reciboTemp.Items.Clear();
			
			// Agrego un cheque
			Cheque cheque = new Cheque();
			cheque.Banco = "El banco";
			cheque.Sucursal = "Avellaneda";
			cheque.FechaEmision = DateTime.Today.Date;
			cheque.FechaVencimiento = DateTime.Today + TimeSpan.FromDays(10);
			cheque.Importe = 50.75m;
			cheque.Numero = 123456;
			cheque.Observacion = "Observación";
			
			reciboTemp.Items.Add(new ItemRecibo(cheque));
			TestBase.ProcesarItemsRecibo(reciboTemp);
			
//			reciboTemp.AgregarFacturaAsociada(Factura.Find(factura.Id));
			
			reciboTemp.Concepto = "Concepto del recibo";
			
			reciboTemp.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo reciboPorEliminar = Recibo.Find(reciboTemp.Id);
			
			this.MarcarComoNoPersistido(ref reciboTemp);
			ReciboDto reciboDto = ConvertidorDto.ToDto<Recibo,ReciboDto>(reciboTemp);
			
			reciboPorEliminar.Delete();
			reciboPorEliminar.Items[0].Cheque.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			Recibo recibo = ConvertidorDto.FromDto<ReciboDto,Recibo>(reciboDto);
			
			Assert.AreEqual(reciboDto.id, recibo.Id, "Id");
			Assert.AreEqual(reciboDto.fechaTicks, recibo.Fecha.Ticks, "Fecha");
			Assert.AreEqual(reciboDto.numero, recibo.Numero, "Número recibo");
			
			Assert.IsNotNull(recibo.Cliente, "Recibo cliente no nulo");
			Assert.AreEqual(reciboDto.cliente.codigo, recibo.Cliente.Codigo, "Cliente código");
			
			Assert.AreEqual(reciboDto.concepto, recibo.Concepto, "Recibo concepto");
			Assert.AreEqual(reciboDto.observaciones, recibo.Observaciones, "Recibo observaciones");
			
			// Items
			Assert.IsNotNull(recibo.Items, "Items no nulos");
			
			ItemReciboDto[] reciboDtoItems = reciboDto.items.Cast<ItemReciboDto>().ToArray();
			
			Assert.AreEqual(reciboDtoItems.Length, recibo.Items.Count, "Items count");
			Assert.AreEqual(reciboDtoItems[0].id, recibo.Items[0].Id, "Item id");
			Assert.AreEqual(reciboDtoItems[0].medioPag, recibo.Items[0].MedioPago, "Item Medio pago");
			
			Assert.IsNotNull(recibo.Items[0].Cheque, "Item cheque no nulo");
			Assert.AreEqual(reciboDtoItems[0].cheque.id, recibo.Items[0].Cheque.Id, "Item cheque id");
			
			Assert.AreEqual(reciboDtoItems[0].orden, recibo.Items[0].Orden, "Item orden");
			Assert.AreEqual(reciboDtoItems[0].importe, recibo.Items[0].Importe, "Item Importe");
			
			Assert.AreEqual(reciboDto.importe, recibo.Importe, "Recibo importe");
		}
		
		[Test]
		public void FromDtoRecibo_ItemEfectivo()
		{
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo reciboTemp = TestBase.CrearRecibo();
			reciboTemp.Cliente = this.ClienteSinBonificacion;
			
			reciboTemp.Items.Clear();
			reciboTemp.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(reciboTemp);
			
//			reciboTemp.AgregarFacturaAsociada(Factura.Find(factura.Id));
			
			reciboTemp.Concepto = "Concepto del recibo";
			
			reciboTemp.Create();
			
			this.FlushAndRecreateScope();
			
			Recibo reciboPorEliminar = Recibo.Find(reciboTemp.Id);
			
			this.MarcarComoNoPersistido(ref reciboTemp);
			ReciboDto reciboDto = ConvertidorDto.ToDto<Recibo,ReciboDto>(reciboTemp);
			
			reciboPorEliminar.Delete();
			
			// Creo un SessionScope de solo lectura porque solo pienso levantar datos ahora,
			// no guardarlos.
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			
			/* Intencionalmente agrego un cheque para simular lo que hace ZeroC Ice: no pasar
			 * valor nules, sino un objeto vacío. */
			(reciboDto.items[0] as ItemReciboDto).cheque = new ChequeDto();
			
			Recibo recibo = ConvertidorDto.FromDto<ReciboDto,Recibo>(reciboDto);
			
			Assert.AreEqual(reciboDto.id, recibo.Id, "Id");
			Assert.AreEqual(reciboDto.fechaTicks, recibo.Fecha.Ticks, "Fecha");
			Assert.AreEqual(reciboDto.numero, recibo.Numero, "Número recibo");
			
			Assert.IsNotNull(recibo.Cliente, "Recibo cliente no nulo");
			Assert.AreEqual(reciboDto.cliente.codigo, recibo.Cliente.Codigo, "Cliente código");
			
			Assert.AreEqual(reciboDto.concepto, recibo.Concepto, "Recibo concepto");
			Assert.AreEqual(reciboDto.observaciones, recibo.Observaciones, "Recibo observaciones");
			
			// Items
			Assert.IsNotNull(recibo.Items, "Items no nulos");
			
			ItemReciboDto[] reciboDtoItems = reciboDto.items.Cast<ItemReciboDto>().ToArray();
			
			Assert.AreEqual(reciboDtoItems.Length, recibo.Items.Count, "Items count");
			Assert.AreEqual(reciboDtoItems[0].id, recibo.Items[0].Id, "Item id");
			Assert.AreEqual(reciboDtoItems[0].medioPag, recibo.Items[0].MedioPago, "Item Medio pago");
			
			Assert.IsNull(recibo.Items[0].Cheque, "Item cheque nulo");
			
			Assert.AreEqual(reciboDtoItems[0].orden, recibo.Items[0].Orden, "Item orden");
			Assert.AreEqual(reciboDtoItems[0].importe, recibo.Items[0].Importe, "Item Importe");
			
			Assert.AreEqual(reciboDto.importe, recibo.Importe, "Recibo importe");
		}
		#endregion
		
		#region DocumentoCredito
		[Test]
		public void ToDtoDocumentoCredito_ConRecibo()
		{
			// Primero creo un recibo para que se cree un DocumentoCredito
			Recibo reciboTemp = TestBase.CrearRecibo();
			reciboTemp.Cliente = this.ClienteSinBonificacion;
			reciboTemp.Items.Clear();
			
			reciboTemp.Items.Add(new ItemRecibo(939.33m));
			TestBase.ProcesarItemsRecibo(reciboTemp);
			
			reciboTemp.Concepto = "Concepto del recibo";
			
			reciboTemp.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Obtengo el DocumentoCredito
			Assert.AreEqual(1, DocumentoCredito.FindAll().Length, "Cantidad documentos de crédito");
			DocumentoCredito documentoCredito = DocumentoCredito.FindAll()[0];
			
			DocumentoCreditoDto documentoCreditoDto =
				ConvertidorDto.ToDto<DocumentoCredito,DocumentoCreditoDto>(documentoCredito);
			
			// Verifico el mapeo
			Assert.AreEqual(documentoCredito.Id, documentoCreditoDto.id, "Id");
			
			Assert.IsNotNull(documentoCreditoDto.cliente.codigo, "Cliente no nulo");
			Assert.AreEqual(documentoCredito.Cliente.Codigo, documentoCreditoDto.cliente.codigo, "Cliente código");
			
			Assert.AreEqual(TipoDocumento.Recibo, documentoCreditoDto.tipoDoc, "Tipo documento");
			Assert.AreEqual(documentoCredito.IdDocumento, documentoCreditoDto.idDocumento, "Id documento");
			Assert.AreEqual(0, documentoCreditoDto.codigoNotaCredito, "Codigo nota crédito documento");
			Assert.AreEqual(0, documentoCreditoDto.numeroNotaCredito, "Numero nota crédito documento");
			
			Assert.AreEqual(documentoCredito.ImporteTotal, documentoCreditoDto.importeTotal, "Importe total");
			Assert.AreEqual(documentoCredito.ImportePendiente, documentoCreditoDto.importePendiente, "Importe pendiente");
		}
		
		[Test]
		public void ToDtoDocumentoCredito_ConNotaCredito()
		{
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = factura;
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Obtengo el DocumentoCredito
			Assert.AreEqual(1, DocumentoCredito.FindAll().Length, "Cantidad documentos de crédito");
			DocumentoCredito documentoCredito = DocumentoCredito.FindAll()[0];
			
			DocumentoCreditoDto documentoCreditoDto =
				ConvertidorDto.ToDto<DocumentoCredito,DocumentoCreditoDto>(documentoCredito);
			
			// Verifico el mapeo
			Assert.AreEqual(documentoCredito.Id, documentoCreditoDto.id, "Id");
			
			Assert.IsNotNull(documentoCreditoDto.cliente.codigo, "Cliente no nulo");
			Assert.AreEqual(documentoCredito.Cliente.Codigo, documentoCreditoDto.cliente.codigo, "Cliente código");
			
			Assert.AreEqual(TipoDocumento.NotaCredito, documentoCreditoDto.tipoDoc, "Tipo documento");
			Assert.AreEqual(documentoCredito.IdDocumento, documentoCreditoDto.idDocumento, "Id documento");
			Assert.AreEqual(notaCredito.Codigo, documentoCreditoDto.codigoNotaCredito, "Codigo nota crédito documento");
			Assert.AreEqual(notaCredito.Numero, documentoCreditoDto.numeroNotaCredito, "Id documento");
			
			Assert.AreEqual(documentoCredito.ImporteTotal, documentoCreditoDto.importeTotal, "Importe total");
			Assert.AreEqual(documentoCredito.ImportePendiente, documentoCreditoDto.importePendiente, "Importe pendiente");
		}
		#endregion
		
		#region DocumentoDebito
		[Test]
		public void ToDtoDocumentoDebito()
		{
			// Creo la factura para que se cree un DocumentoDebito
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.CondicionVenta.Plazo = 10; // Plazo de diez días
			factura.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Obtengo el DocumentoDebito
			Assert.AreEqual(1, DocumentoDebito.FindAll().Length, "Cantidad documentos de débito");
			DocumentoDebito documentoDebito = DocumentoDebito.FindAll()[0];
			
			DocumentoDebitoDto documentoDebitoDto =
				ConvertidorDto.ToDto<DocumentoDebito,DocumentoDebitoDto>(documentoDebito);
			
			// Verifico el mapeo
			Assert.AreEqual(documentoDebito.Id, documentoDebitoDto.id, "Id");
			
			Assert.IsNotNull(documentoDebitoDto.cliente.codigo, "Cliente no nulo");
			Assert.AreEqual(documentoDebito.Cliente.Codigo, documentoDebitoDto.cliente.codigo, "Cliente código");
			
			Assert.AreEqual(TipoDocumento.Factura, documentoDebitoDto.tipoDoc, "Tipo documento");
			Assert.AreEqual(documentoDebito.IdDocumento, documentoDebitoDto.idDocumento, "Id documento");
			Assert.AreEqual(documentoDebito.CodigoFactura, documentoDebitoDto.codigoFactura, "Codigo factura");
			Assert.AreEqual(documentoDebito.NumeroFactura, documentoDebitoDto.numeroFactura, "Numero factura");
			Assert.AreEqual(documentoDebito.FechaVencimiento.Ticks, documentoDebitoDto.fechaVencimientoTicks, "Fecha vencimiento");
			
			Assert.AreEqual(documentoDebito.ImporteTotal, documentoDebitoDto.importeTotal, "Importe total");
			Assert.AreEqual(documentoDebito.ImportePendiente, documentoDebitoDto.importePendiente, "Importe pendiente");
		}
		#endregion
	}
}
