// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.
using System;

using NUnit.Framework;
using Castle.ActiveRecord;

using Stratos.Tests;
using Stratos.ModeloObjetos;
using Stratos.ModeloObjetos.MetaCampos;
using Stratos.ModeloDto;

namespace Stratos.ModeloObjetos.Tests
{
	[TestFixture()]
	public class MetaCampoClienteTests : TestBase
	{
		#region Campos
		private MetaCampoCliente metaCampoVacio;
		private MetaCampoCliente metaCampoCompleto;
		private Cliente clienteCompleto;
		#endregion

		[SetUp]
		public override void TestInit ()
		{
			base.TestInit();
			
			this.clienteCompleto = TestBase.CrearClienteCesar();
			this.clienteCompleto.CreateAndFlush();
			
			this.metaCampoVacio = new MetaCampoCliente();
			
			this.metaCampoCompleto = new MetaCampoCliente {
				Clave = "ProductoPreferido",
				Valor = "Mesada",
				Cliente = this.clienteCompleto
			};
		}
				
		[Test()]
		public void Creacion()
		{
			try {
				this.metaCampoCompleto.Create();
			}
			catch(ActiveRecordValidationException ex) {
				Assert.Fail(ex.ValidationErrorMessages[0]);
			}
			catch(Exception e) {
				Assert.Fail(e.Message);
			}
			
			base.FlushAndRecreateScope();
		
			MetaCampoCliente[] metaCampos = MetaCampoCliente.FindAll();
			
			Assert.AreEqual(1, metaCampos.Length, "Cantidad de metacampos insertados");
			
			MetaCampoCliente mc = metaCampos[0];
			Assert.AreEqual("ProductoPreferido", mc.Clave, "Clave");
			Assert.AreEqual("Mesada", mc.Valor, "Valor");
			Assert.AreEqual("César M. Sandrigo", mc.Cliente.RazonSocial, "Cliente razón social");			
		}
		
		[Test()]
		public void Actualizacion()
		{
			try {
				this.metaCampoCompleto.Create();
			}
			catch(ActiveRecordValidationException ex) {
				Assert.Fail(ex.ValidationErrorMessages[0]);
			}
			catch(Exception e) {
				Assert.Fail(e.Message);
			}
			
			base.FlushAndRecreateScope();
		
			MetaCampoCliente[] metaCampos = MetaCampoCliente.FindAll();
			
			Assert.AreEqual(1, metaCampos.Length, "Cantidad de metacampos insertados");
			
			MetaCampoCliente mc = metaCampos[0];
			Assert.AreEqual("ProductoPreferido", mc.Clave, "Clave");
			Assert.AreEqual("Mesada", mc.Valor, "Valor");
			Assert.AreEqual("César M. Sandrigo", mc.Cliente.RazonSocial, "Cliente razón social");
			
			mc.Valor = "Mosaico";
			
			try {
				mc.Save();
			}
			catch(ActiveRecordValidationException ex) {
				Assert.Fail(ex.ValidationErrorMessages[0]);
			}
			catch(Exception e) {
				Assert.Fail(e.Message);
			}
			
			metaCampos = MetaCampoCliente.FindAll();
			
			Assert.AreEqual(1, metaCampos.Length, "Cantidad de metacampos recuperados");
			
			mc = metaCampos[0];
			Assert.AreEqual("ProductoPreferido", mc.Clave, "Clave metacampo actualizado");
			Assert.AreEqual("Mosaico", mc.Valor, "Valor metacampo actualizado");
			Assert.AreEqual("César M. Sandrigo", mc.Cliente.RazonSocial, "Cliente razón social metacampo actualizado");
		}
		
		[Test()]
		[Ignore("No funciona correctamente el tests")]
		public void SinDatos()
		{
			bool excepcionLanzada = false;
			
			try {
				this.metaCampoVacio.Create();
			}
			catch (ActiveRecordValidationException) {
				excepcionLanzada = true;
			}
			catch (Exception e)
			{
				Assert.Fail(e.Message + ": " + e.InnerException.Message);
			}
			
			Assert.IsTrue(excepcionLanzada, "No se lanzó ninguna excepción");
		}
		
		[Test()]
		[Ignore("No funciona correctamente el tests")]
		public void SinClave()
		{
			bool excepcionLanzada = false;
			
			try {
				this.metaCampoCompleto.Clave = "";
				
				this.metaCampoCompleto.Create();
			}
			catch (ActiveRecordValidationException) {
				excepcionLanzada = true;
			}
			catch (Exception e)
			{
				Assert.Fail(e.Message + ": " + e.InnerException.Message);
			}
			
			Assert.IsTrue(excepcionLanzada, "No se lanzó ninguna excepción");
		}
		
		[Test()]
		[Ignore("No funciona correctamente el tests")]
		public void SinCliente()
		{
			bool excepcionLanzada = false;
			
			try {
				this.metaCampoCompleto.Cliente = null;
				this.metaCampoCompleto.Create();
			}
			catch (ActiveRecordValidationException) {
				excepcionLanzada = true;
			}
			catch (Exception e)
			{
				Assert.Fail(e.Message + ": " + e.InnerException.Message);
			}
			
			Assert.IsTrue(excepcionLanzada, "No se lanzó ninguna excepción");
		}
	}
}

