// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections.Generic;

using NUnit.Framework;

using Stratos.ModeloDto;
using Stratos.ModeloObjetos;

namespace Stratos.Tests
{
	[TestFixture()]
	public class TestBaseTests : TestBase
	{
		[Test]
		public void CrearDomicilioMiltonTest()
		{
			Domicilio domicilio = TestBase.CrearDomicilioMilton();
			
			Assert.AreEqual("Bv. Galvez 956", domicilio.Direccion, "Dirección");
			Assert.AreEqual("Santa Fe de la Veracruz", domicilio.Localidad, "Localidad");
			Assert.AreEqual("3000", domicilio.CodigoPostal, "Código postal");
			Assert.AreEqual(ProvinciaDto.SantaFe, domicilio.Provincia, "Provincia");
		}
		
		[Test]
		public void CrearClienteMiltonTest()
		{
			Cliente cliente = TestBase.CrearClienteMilton();
			
			Assert.AreEqual(5, cliente.Codigo, "Código");
			Assert.AreEqual("Milton Pividori", cliente.RazonSocial, "Razón social");
			
			Assert.IsNotNull(cliente.Domicilio, "Domicilio no nulo");
			Assert.AreEqual("Bv. Galvez 956", cliente.Domicilio.Direccion, "Domicilio dirección");
			Assert.AreEqual("Santa Fe de la Veracruz", cliente.Domicilio.Localidad, "Domicilio localidad");
			Assert.AreEqual("3000", cliente.Domicilio.CodigoPostal, "Domicilio código postal");
			Assert.AreEqual(ProvinciaDto.SantaFe, cliente.Domicilio.Provincia, "Domicilio provincia");
			
			Assert.AreEqual("0342 156654321", cliente.Telefono, "Teléfono");
			Assert.AreEqual("miltondp@gmail.com", cliente.CorreoElectronico, "Correo electrónico");
			Assert.AreEqual(TipoResponsable.Monotributo, cliente.TipoResponsable, "Tipo responsable");
			
			Assert.IsNotNull(cliente.Cuit, "Cuit no nulo");
			Assert.AreEqual(30674393683, cliente.Cuit.ValorNumerico, "Cuit valor numérico");
			
			Assert.AreEqual(15m, cliente.PorcentajeBonificacion, "Bonificación");
			Assert.AreEqual("Estudiante de ISI", cliente.Notas, "Notas");
		}
		
		[Test]
		public void CrearRubroGriferiaTest()
		{
			Rubro rubro = TestBase.CrearRubroGriferia();
			
			Assert.AreEqual(20, rubro.Codigo, "Codigo");
			Assert.AreEqual("Grifería", rubro.Nombre, "Nombre");
			Assert.AreEqual("Griferías", rubro.Descripcion, "Descripcion");
		}
		
		[Test]
		public void CrearProductoGriferiaTest()
		{
			Producto producto = TestBase.CrearProductoGriferia();
			
			Assert.AreEqual("7", producto.Codigo, "Codigo");
			Assert.AreEqual("Griferia XYZ Modelo 8", producto.Descripcion, "Descripcion");
			Assert.AreEqual(60.00m, producto.ImporteSinIva, "ImporteSinIva");
			Assert.AreEqual(21m, producto.PorcentajeIva, "PorcentajeIva");
			Assert.AreEqual(54.22m, producto.PrecioCosto, "PrecioCosto");
			
			Assert.IsNotNull(producto.SubRubro, "SubRubro no nulo");
			Assert.AreEqual(producto.SubRubro.Descripcion, producto.SubRubro.Descripcion, "SubRubro descripcion");
			
			Assert.AreEqual(UnidadProducto.Unidad, producto.Unidad, "Unidad");
			Assert.AreEqual("Un producto griferia", producto.Notas, "Notas");
		}
		
		[Test]
		public void CrearItemFacturaTest()
		{
			ItemFactura item = TestBase.CrearItemFactura(TestBase.CrearProductoGriferia(), 5);
			
			Assert.IsTrue(item.ProductoCodigo != null && item.ProductoCodigo.Trim() != String.Empty, "Producto no nulo");
			Assert.IsNull(item.ProductoDescripcion, "Producto descripción");
			Assert.AreEqual("Griferia XYZ Modelo 8", item.Descripcion, "Item descripción");
			Assert.AreEqual(5, item.Cantidad, "Item cantidad");
		}
		
		[Test]
		public void CrearItemFacturaTest_ConProductoAbstracto()
		{
			ItemFactura item = TestBase.CrearItemFactura(TestBase.CrearProductoAbstracto(), 5);
			
			Assert.IsTrue(item.ProductoCodigo != null && item.ProductoCodigo.Trim() != String.Empty, "Producto no nulo");
			Assert.AreEqual("Producto de industria", item.ProductoDescripcion, "Producto descripción");
			Assert.AreEqual("Producto de industria", item.Descripcion, "Item descripción");
			Assert.AreEqual(5, item.Cantidad, "Item cantidad");
		}
		
		[Test]
		public void CrearFacturaTest()
		{
			Factura factura = TestBase.CrearFactura();
			
			Assert.AreEqual(DateTime.Today, factura.Fecha, "Fecha");
			Assert.AreEqual(FacturaTipoVenta.FacturaA, factura.Tipo, "Tipo");
			Assert.AreEqual(4, factura.Codigo, "Codigo");
			Assert.AreEqual(23423, factura.Numero, "Numero");
			
			Assert.IsNotNull(factura.CondicionVenta, "Condicion venta");
			Assert.AreEqual(CondicionPago.Contado, factura.CondicionVenta.Condicion, "Condicion venta condicion pago");
			
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual("César M. Sandrigo", factura.Cliente.RazonSocial, "Cliente razon social");
			
			Assert.AreEqual("12256666", factura.Remitos, "Remitos");
			Assert.AreEqual(60.00m, factura.ImporteBruto, "Importe bruto");
			Assert.AreEqual(0.00m, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(0.00m, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(60.00m, factura.ImporteNeto, "Importe neto");
			Assert.AreEqual(12.60m, factura.ImporteIva, "Importe Iva");
			Assert.AreEqual(72.60m, factura.ImporteTotal, "Importe total");
			Assert.AreEqual(72.60m, factura.ImporteFinal, "Importe final");
			Assert.AreEqual("Estas son las observaciones", factura.Observaciones, "Observaciones");
			
			Assert.IsNotNull(factura.Items, "Items");
			
			List<ItemFactura> items = factura.Items.Cast<ItemFactura>().ToList();
			
			Assert.AreEqual(1, items.Count, "Items cantidad");
			Assert.AreEqual("Griferia XYZ Modelo 8", items[0].Descripcion, "Item descripción");
		}
		
		[Test]
		public void CrearItemPreFacturaTest()
		{
			ItemPreFactura item = TestBase.CrearItemPreFactura(TestBase.CrearProductoGriferia(), 5);
			
			Assert.IsTrue(item.ProductoCodigo != null && item.ProductoCodigo.Trim() != String.Empty, "Producto no nulo");
			Assert.IsNull(item.ProductoDescripcion, "Producto descripción");
			Assert.AreEqual("Griferia XYZ Modelo 8", item.Descripcion, "Item descripción");
			Assert.AreEqual(5, item.Cantidad, "Item cantidad");
		}
		
		[Test]
		public void CrearPreFacturaTest()
		{
			PreFactura factura = TestBase.CrearPreFactura();
			
			Assert.AreEqual(DateTime.Today, factura.Fecha, "Fecha");
			Assert.AreEqual(FacturaTipoVenta.FacturaA, factura.Tipo, "Tipo");
			
			Assert.IsNotNull(factura.CondicionVenta, "Condicion venta");
			Assert.AreEqual(CondicionPago.Contado, factura.CondicionVenta.Condicion, "Condicion venta condicion pago");
			
			Assert.IsNotNull(factura.Cliente, "Cliente no nulo");
			Assert.AreEqual("Milton Pividori", factura.Cliente.RazonSocial, "Cliente razon social");
			
			Assert.AreEqual("12256666,2343423", factura.Remitos, "Remitos");
			Assert.AreEqual(60.00m, factura.ImporteBruto, "Importe bruto");
			Assert.AreEqual(0.00m, factura.PorcentajeBonificacion, "Porcentaje bonificación");
			Assert.AreEqual(0.00m, factura.ImporteBonificacion, "Importe bonificación");
			Assert.AreEqual(60.00m, factura.ImporteNeto, "Importe neto");
			Assert.AreEqual(12.60m, factura.ImporteIva, "Importe Iva");
			Assert.AreEqual(72.60m, factura.ImporteTotal, "Importe total");
			Assert.AreEqual(72.60m, factura.ImporteFinal, "Importe final");
			Assert.AreEqual("Estas son las observaciones", factura.Observaciones, "Observaciones");
			
			// Items
			List<ItemPreFactura> items = factura.Items.Cast<ItemPreFactura>().ToList();
			
			Assert.IsNotNull(items, "Items");
			Assert.AreEqual(1, items.Count, "Items cantidad");
			Assert.AreEqual("Griferia XYZ Modelo 8", items[0].Descripcion, "Item descripción");
		}
	}
}
