// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Interfaces;
using Rhino.Mocks.Constraints;
using Castle.ActiveRecord;
using NHibernate.Criterion;

using Stratos.Common;
using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Controladores;
using StratosClient.Tests;
using Stratos.Tests;
using StratosClient.Widgets.Level2;

namespace StratosClient.Clientes.CuentaCorriente.Tests
{
	[TestFixture]
	public class CuentaCorrientePresenterTests : StratosClientTestBase
	{
		#region Campos
		protected ICuentaCorrienteVista cuentaCorrienteVista;
		protected ControladorClientesPrx controladorClientes;
		
		protected Factura factura1;
		protected Factura factura2;
		protected Factura factura3;
		protected Factura factura4;
		
		protected Recibo recibo1;
		protected Recibo recibo2;
		#endregion
		
		#region Métodos de inicialización
		[TestFixtureSetUp]
		public override void FixtureInit ()
		{
			base.FixtureInit ();
			
			/* Obtengo los proxies a los controladores accediendo a la instancia del servidor
			 * Stratos que tengo. */
			this.controladorClientes = ControladorClientesPrxHelper.checkedCast(
				this.controladorTerminales
					.ObtenerProxyControlador("ControladorClientes"));
		}
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			#region Facturas
			// Creo unas facturas
			this.factura1 = TestBase.CrearFactura();
			this.factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			this.factura1.Codigo = 1;
			this.factura1.Numero = 1;
			this.factura1.Cliente = this.ClienteSinBonificacion;
			this.factura1.Items.Clear();
			this.factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura1);
			
			this.factura2 = TestBase.CrearFactura();
			this.factura2.CondicionVenta.Condicion = CondicionPago.Contado;
			this.factura2.Codigo = 1;
			this.factura2.Numero = 2;
			this.factura2.Cliente = this.ClienteSinBonificacion;
			this.factura2.Items.Clear();
			this.factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura2);
			
			this.factura3 = TestBase.CrearFactura();
			this.factura3.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			this.factura3.Codigo = 1;
			this.factura3.Numero = 3;
			this.factura3.Cliente = this.ClienteSinBonificacion;
			this.factura3.Items.Clear();
			this.factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura3);
			
			this.factura4 = TestBase.CrearFactura();
			this.factura4.CondicionVenta.Condicion = CondicionPago.Contado;
			this.factura4.Codigo = 1;
			this.factura4.Numero = 4;
			this.factura4.Cliente = this.ClienteSinBonificacion;
			this.factura4.Items.Clear();
			this.factura4.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 4));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura4);
			#endregion
			
			#region Recibos
			// Creo recibo1
			this.recibo1 = TestBase.CrearRecibo();
			this.recibo1.Cliente = this.factura1.Cliente;
			
			this.recibo1.Items.Clear();
			this.recibo1.Items.Add(new ItemRecibo(this.factura1.ImporteTotal));
			this.recibo1.Items.Add(new ItemRecibo(this.factura2.ImporteTotal / 2.0m));
			
			TestBase.ProcesarItemsRecibo(this.recibo1);
			
			this.recibo1.Concepto = "Concepto del recibo";
			
			// Creo recibo2
			this.recibo2 = TestBase.CrearRecibo();
			this.recibo2.Cliente = this.factura3.Cliente;
			
			this.recibo2.Items.Clear();
			this.recibo2.Items.Add(new ItemRecibo(this.factura3.ImporteTotal));
			this.recibo2.Items.Add(new ItemRecibo(this.factura4.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(this.recibo2);
			
			this.recibo2.Concepto = "Concepto del recibo";
			#endregion
		}
		
		[Test]
		public void RecibosValidos()
		{
			Assert.IsTrue(this.recibo1.IsValid(), "recibo1 válido");
			Assert.IsTrue(this.recibo2.IsValid(), "recibo2 válido");
		}
		#endregion
		
		#region Métodos auxiliares
		protected DocumentoDebito ObtenerDocumentoDebito(Factura factura)
		{
			DocumentoDebito[] resultado =
				DocumentoDebito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Factura),
				                        Restrictions.Eq("IdDocumento", factura.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoDebito: más de un documento débito devuelto");
			
			return resultado[0];
		}
		
		protected DocumentoCredito ObtenerDocumentoCredito(Recibo recibo)
		{
			DocumentoCredito[] resultado =
				DocumentoCredito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Recibo),
				                         Restrictions.Eq("IdDocumento", recibo.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoCredito: más de un documento de crédito devuelto");
			
			return resultado[0];
		}
		#endregion
		
		#region Cálculo de importes
		[Test]
		public void ImporteFavor_ImporteContra_SaldoFinal_Calculo()
		{
			// Creo las facturas
			this.factura1.Create();
			this.FlushAndRecreateScope();
			
			this.factura2.Create();
			this.FlushAndRecreateScope();
			
			this.factura3.Create();
			this.FlushAndRecreateScope();
			
			this.factura4.Create();
			this.FlushAndRecreateScope();
			
			// Creo los recibos
			this.recibo1.Create();
			this.recibo2.Create();
			this.FlushAndRecreateReadOnlyScope();
			
			// Vuelvo el MockRepository al modo Record
			this.mocks.BackToRecordAll();
			
			// El cliente que será devuelto por IRazonSocialVista
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.recibo1.Cliente);
			
			// Mock para la razón social
			IRazonSocialVista<ClienteDto> razonSocial = this.mocks.Stub<IRazonSocialVista<ClienteDto>>();
			razonSocial.Persona = clienteDto;
			razonSocial.PersonaCargada += null; // Necesario para obtener el IEventRaiser
			IEventRaiser razonSocial_ClienteCargado = LastCall.GetEventRaiser();
			
			// Mock para IDeudasVista
			this.cuentaCorrienteVista = this.mocks.Stub<ICuentaCorrienteVista>();
			
			SetupResult.For(this.cuentaCorrienteVista.RazonSocialVista)
				.Return(razonSocial);
			SetupResult.For(this.cuentaCorrienteVista.DocumentosCredito)
				.Return(this.mocks.Stub<IDocumentoCreditoVista>());
			SetupResult.For(this.cuentaCorrienteVista.DocumentosDebito)
				.Return(this.mocks.Stub<IDocumentoDebitoVista>());
			
			this.mocks.ReplayAll();
			
			// Código a testear
			CuentaCorrientePresenter cuentaCorrientePresenter =
				new CuentaCorrientePresenter(this.cuentaCorrienteVista,
				                             this.controladorClientes);
			
			razonSocial_ClienteCargado.Raise(clienteDto);
			
			// Verifico los importes
			Assert.AreEqual(this.recibo1.Importe + this.recibo2.Importe,
			                this.cuentaCorrienteVista.ImporteFavor,
			                "Recibo importe a favor");
			
			Assert.AreEqual(this.factura1.ImporteTotal + this.factura3.ImporteTotal,
			                this.cuentaCorrienteVista.ImporteContra,
			                "Facturas por pagar - Importe en contra");
			
			Assert.AreEqual(StratosUtil.RoundDouble(this.cuentaCorrienteVista.ImporteFavor - this.cuentaCorrienteVista.ImporteContra, 2),
			                this.cuentaCorrienteVista.SaldoTotal,
			                "Saldo total");
			
			this.mocks.VerifyAll();
		}
		#endregion
	}
}
