// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Interfaces;
using Rhino.Mocks.Constraints;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Queries;
using NHibernate.Criterion;

using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Controladores;
using StratosClient.Tests;
using Stratos.Tests;
using StratosClient.Widgets.Level2;

namespace StratosClient.Clientes.CuentaCorriente.Tests
{
	[TestFixture]
	public class ImputacionesPresenterTests : StratosClientTestBase
	{
		#region Campos
		protected IImputacionesVista imputacionesVista;
		protected ControladorClientesPrx controladorClientes;
		
		protected Factura factura1;
		protected Factura factura2;
		protected Factura factura3;
		protected Factura factura4;
		
		protected NotaCredito notaCredito1;
		protected NotaCredito notaCredito2;
		
		protected Recibo recibo1;
		protected Recibo recibo2;
		#endregion
		
		#region Métodos de inicialización
		[TestFixtureSetUp]
		public override void FixtureInit ()
		{
			base.FixtureInit ();
			
			/* Obtengo los proxies a los controladores accediendo a la instancia del servidor
			 * Stratos que tengo. */
			this.controladorClientes = ControladorClientesPrxHelper.checkedCast(
				this.controladorTerminales
					.ObtenerProxyControlador("ControladorClientes"));
		}
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			#region Facturas
			// Creo unas facturas
			this.factura1 = TestBase.CrearFactura();
			this.factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			this.factura1.Codigo = 1;
			this.factura1.Numero = 1;
			this.factura1.Cliente = this.ClienteSinBonificacion;
			this.factura1.Items.Clear();
			this.factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura1);
			
			this.factura2 = TestBase.CrearFactura();
			this.factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			this.factura2.Codigo = 1;
			this.factura2.Numero = 2;
			this.factura2.Cliente = this.ClienteSinBonificacion;
			this.factura2.Items.Clear();
			this.factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura2);
			
			this.factura3 = TestBase.CrearFactura();
			this.factura3.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			this.factura3.Codigo = 1;
			this.factura3.Numero = 3;
			this.factura3.Cliente = this.ClienteSinBonificacion;
			this.factura3.Items.Clear();
			this.factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura3);
			
			this.factura4 = TestBase.CrearFactura();
			this.factura4.Tipo = FacturaTipoVenta.FacturaB;
			this.factura4.CondicionVenta.Condicion = CondicionPago.Contado;
			this.factura4.Codigo = 1;
			this.factura4.Numero = 4;
			this.factura4.Cliente = this.ClienteSinBonificacion;
			this.factura4.Items.Clear();
			this.factura4.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura4);
			#endregion
			
			#region Recibos
			// recibo1
			this.recibo1 = TestBase.CrearRecibo();
			this.recibo1.Cliente = this.factura1.Cliente;
			
			this.recibo1.Items.Clear();
			this.recibo1.Items.Add(new ItemRecibo(this.factura1.ImporteTotal));
			this.recibo1.Items.Add(new ItemRecibo(this.factura2.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(this.recibo1);
			
			this.recibo1.Concepto = "Concepto del recibo";
			
			// recibo2
			this.recibo2 = TestBase.CrearRecibo();
			this.recibo2.Cliente = this.factura3.Cliente;
			
			this.recibo2.Items.Clear();
			this.recibo2.Items.Add(new ItemRecibo(this.factura3.ImporteTotal));
			this.recibo2.Items.Add(new ItemRecibo(this.factura4.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(this.recibo2);
			
			this.recibo2.Concepto = "Concepto del recibo";
			#endregion
		}
		#endregion
		
		#region Métodos auxiliares
		protected DocumentoDebito ObtenerDocumentoDebito(Factura factura)
		{
			DocumentoDebito[] resultado =
				DocumentoDebito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Factura),
				                        Restrictions.Eq("IdDocumento", factura.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoDebito: más de un documento débito devuelto");
			
			return resultado[0];
		}
		
		protected DocumentoCredito ObtenerDocumentoCredito(Recibo recibo)
		{
			DocumentoCredito[] resultado =
				DocumentoCredito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Recibo),
				                         Restrictions.Eq("IdDocumento", recibo.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoCredito: más de un documento de crédito devuelto");
			
			return resultado[0];
		}
		
		protected DocumentoCredito ObtenerDocumentoCredito(NotaCredito notaCredito)
		{
			DocumentoCredito[] resultado =
				DocumentoCredito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.NotaCredito),
				                         Restrictions.Eq("IdDocumento", notaCredito.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoCredito: más de un documento de crédito devuelto");
			
			return resultado[0];
		}
		
		public long SiguienteNumeroFactura(FacturaTipoVenta tipo, int codigo) {
			ScalarQuery<Nullable<long>> consultaUltimoNumero =
				new ScalarQuery<Nullable<long>>(typeof(Factura),
				                          @"select max(f.Numero)
											from Factura f
											where f.Tipo = :tipo
												and f.Codigo = :codigo");
			
			consultaUltimoNumero.SetParameter("tipo", tipo);
			consultaUltimoNumero.SetParameter("codigo", codigo);
			
			long? ultimoNumero = consultaUltimoNumero.Execute();
			
			return ultimoNumero.Value + 1;
		}
		
		protected void PrepararEscenarioImputacion(double[] importesDocumentosCredito,
		                                           double[] importesDocumentosDebito)
		{
			Factura factura;
			Recibo recibo;
			
			foreach (double importeCredito in importesDocumentosCredito) {
			}
			
			foreach (double importeDebito in importesDocumentosDebito) {
				factura = TestBase.CrearFactura();
				factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
				factura.Codigo = 1;
				factura.Numero = this.SiguienteNumeroFactura(factura.Tipo, 1);
				factura.Cliente = this.ClienteSinBonificacion;
				factura.Items.Clear();
				factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
				TestBase.ProcesarItemsYCalcularImportesFactura(factura);
				factura.Create();
			}
		}
		#endregion
		
		#region Recibos - Eventos al cargar/descargar cliente
		[Test]
		public void DocumentosExistentes_SeCargan_AlCargarCliente()
		{
			#region Creación de documentos
			this.factura1.Create();
			this.FlushAndRecreateScope();
			DocumentoDebito documentoDebitoFactura1 = this.ObtenerDocumentoDebito(this.factura1);
			
			this.factura2.Create();
			this.FlushAndRecreateScope();
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(this.factura2);
			
			this.recibo1.Create();
			this.recibo2.Create();
			this.FlushAndRecreateScope();
			
			DocumentoCredito documentoCreditoRecibo1 = this.ObtenerDocumentoCredito(this.recibo1);
			DocumentoCredito documentoCreditoRecibo2 = this.ObtenerDocumentoCredito(this.recibo2);
			#endregion
			
			// Vuelvo el MockRepository al modo Record
			this.mocks.BackToRecordAll();
			
			// El cliente que será devuelto por IRazonSocialVista
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.recibo1.Cliente);
			
			// Mock para IRazonSocialVista
			IRazonSocialVista<ClienteDto> razonSocial = this.mocks.Stub<IRazonSocialVista<ClienteDto>>();
			razonSocial.Persona = clienteDto;
			razonSocial.PersonaCargada += null; // Necesario para obtener el IEventRaiser
			IEventRaiser razonSocial_ClienteCargado = LastCall.GetEventRaiser();
			
			#region Mock para IDocumentoCreditoVista.
			IDocumentoCreditoVista listaDocumentosCredito =
				this.mocks.DynamicMock<IDocumentoCreditoVista>();
			
			/* Me aseguro que se ejecuten el método Vaciar y AgregarElementos, en ese orden.
			 * Para el método AgregarElementos, me aseguro de que la colección pasada tenga
			 * exactamente los dos recibos con importe disponible. */
			
			// Este es el array de IDs de las facturas que tienen que pasarse como argumento
			long[] idsCredito = new long[] { documentoCreditoRecibo1.Id, documentoCreditoRecibo2.Id };
			
			// Método AgregarElementos: agrego algunas restricciones a los parámetros que
			// espero se le pasen.
			Expect.Call(delegate {
				listaDocumentosCredito.AgregarElementos(
					Arg<DocumentoCreditoDto[]>
						.Matches<DocumentoCreditoDto[]>(x => x.Length == 2 &&
				                              x.All(dc => idsCredito.Contains(dc.id)))
				);
			})
				.Message("DocumentosCredito: AgregarElementos con dos argumentos y con los IDs de documentos de crédito correspondientes");
			#endregion
			
			#region Mock para IDocumentoDebitoVista
			IDocumentoDebitoVista listaDocumentosDebito =
				this.mocks.DynamicMock<IDocumentoDebitoVista>();
			
			// Este es el array de IDs de las facturas que tienen que pasarse como argumento
			long[] idsDebito = new long[] { documentoDebitoFactura1.Id, documentoDebitoFactura2.Id };
			
			// Método AgregarElementos: agrego algunas restricciones a los parámetros que
			// espero se le pasen.
			Expect.Call(delegate {
				listaDocumentosDebito.AgregarElementos(
					Arg<DocumentoDebitoDto[]>
						.Matches<DocumentoDebitoDto[]>(x => x.Length == 2 &&
				                              x.All(dd => idsDebito.Contains(dd.id)))
				);
			})
				.Message("DocumentosDebito: AgregarElementos con dos argumentos y con los IDs de documentos de débito correspondientes");
			#endregion
			
			// Mock para IImputacionesVista
			this.imputacionesVista = this.mocks.Stub<IImputacionesVista>();
			
			SetupResult.For(this.imputacionesVista.RazonSocialVista)
				.Return(razonSocial);
			SetupResult.For(this.imputacionesVista.DocumentosCredito)
				.Return(listaDocumentosCredito);
			SetupResult.For(this.imputacionesVista.DocumentosDebito)
				.Return(listaDocumentosDebito);
			
			this.mocks.ReplayAll();
			
			// Código a testear
			ImputacionesPresenter imputacionesPresenter =
				new ImputacionesPresenter(this.imputacionesVista,
				                          this.controladorClientes);
			
			razonSocial_ClienteCargado.Raise(clienteDto);
			
			this.mocks.VerifyAll();
		}
		#endregion
		
		#region Selección de facturas
		/// <summary>
		/// Esta función ejecuta los tests que comprueban el funcionamiento del Presenter al
		/// seleccionar una factura en la lista de facturas. Verifica que, en los distintos casos,
		/// se haya permitido o no realizar la operación.
		/// </summary>
		/// <param name="importeFacturaAnterior">
		/// A <see cref="System.Double"/>
		/// </param>
		/// <param name="importePorPagarFacturaSeleccionada">
		/// A <see cref="System.Double"/>
		/// </param>
		/// <param name="importeRecibo">
		/// A <see cref="System.Double"/>
		/// </param>
		/// <param name="sePermiteLaSeleccion">
		/// A <see cref="System.Boolean"/>
		/// </param>
		protected void TestDocumentoDebito(double[] importesDocumentosCreditoAnteriores,
		                                   double[] importesDocumentosDebitoAnteriores,
		                                   double importePendienteDocumentoDebitoSeleccionado,
		                                   bool sePermiteLaSeleccion)
		{
			// Vuelvo el MockRepository al modo Record
			this.mocks.BackToRecordAll();
			
			System.Func<DocumentoDebitoDto,bool> permitirSeleccionDebito = null;
			System.Func<DocumentoCreditoDto,bool> permitirSeleccionCredito = null;
			
			IDocumentoCreditoVista listaDocumentosCredito = this.mocks.DynamicMock<IDocumentoCreditoVista>();
			IDocumentoDebitoVista listaDocumentosDebito = this.mocks.DynamicMock<IDocumentoDebitoVista>();
			
			/* Cuando se asigna la propiedad PermitirSeleccion (que es un tipo delegate)
			 * obtengo el objeto asignado (el delegate) para luego llamarlo más abajo y verificar
			 * que funciona. */
			listaDocumentosDebito.Expect(lista => lista.PermitirSeleccion).SetPropertyAndIgnoreArgument()
				.WhenCalled(x => permitirSeleccionDebito = (System.Func<DocumentoDebitoDto,bool>) x.Arguments[0]);
			
			listaDocumentosCredito.Expect(lista => lista.PermitirSeleccion).SetPropertyAndIgnoreArgument()
				.WhenCalled(x => permitirSeleccionCredito = (System.Func<DocumentoCreditoDto,bool>) x.Arguments[0]);
			
			// Creo los documentos necesario para preparar los documentos de débito
			// y crédito
			DocumentoCreditoDto[] documentosCredito = new DocumentoCreditoDto[importesDocumentosCreditoAnteriores.Length];
			DocumentoDebitoDto[] documentosDebito = new DocumentoDebitoDto[importesDocumentosDebitoAnteriores.Length];
			
			for (int i=0; i<documentosCredito.Length; i++) {
				documentosCredito[i] = new DocumentoCreditoDto();
				documentosCredito[i].importeTotal = importesDocumentosCreditoAnteriores[i];
				documentosCredito[i].importePendiente = importesDocumentosCreditoAnteriores[i];
			}
			
			listaDocumentosCredito.Stub(lista => lista.ElementosSeleccionados)
				.Return(documentosCredito.ToList());
			
			for (int i=0; i<documentosDebito.Length; i++) {
				documentosDebito[i] = new DocumentoDebitoDto();
				documentosDebito[i].importeTotal = importesDocumentosDebitoAnteriores[i];
				documentosDebito[i].importePendiente = importesDocumentosDebitoAnteriores[i];
			}
			
			listaDocumentosDebito.Stub(lista => lista.ElementosSeleccionados)
				.Return(documentosDebito.ToList());
			
			// Creo un fake del documento débito que se acaba de seleccionar.
			DocumentoDebitoDto ultimoDocumentoDebitoSeleccionado = new DocumentoDebitoDto();
			ultimoDocumentoDebitoSeleccionado.importePendiente = importePendienteDocumentoDebitoSeleccionado;
			
			// Mock para IReciboVista
			this.imputacionesVista = this.mocks.Stub<IImputacionesVista>();
			
			SetupResult.For(this.imputacionesVista.DocumentosCredito)
				.Return(listaDocumentosCredito);
			SetupResult.For(this.imputacionesVista.DocumentosDebito)
				.Return(listaDocumentosDebito);
			SetupResult.For(this.imputacionesVista.RazonSocialVista)
				.Return(this.mocks.Stub<IRazonSocialVista<ClienteDto>>());
			
			this.mocks.ReplayAll();
			
			// Creo el Presenter con los mocks y stubs creados antes
			ImputacionesPresenter imputacionesPresenter =
				new ImputacionesPresenter(this.imputacionesVista,
				                          this.controladorClientes);
			
			Assert.IsNotNull(permitirSeleccionDebito, "PermitirSeleccion no nulo");
			Assert.AreEqual(sePermiteLaSeleccion,
			                permitirSeleccionDebito(ultimoDocumentoDebitoSeleccionado),
			                "PermitirSeleccino retorno");
			
			// Verifico las expectations
			this.mocks.VerifyAll();
		}
		
		[Test]
		public void DocumentoDebitoSeleccionado_NoHayOtrosDebitosSeleccionados_SePuedeSeleccionar_ImporteCreditoSobrante()
		{
			this.TestDocumentoDebito(new double[] { 150.0 }, // Importe documento crédito anterior (ya seleccionado)
			                         new double[] { }, // importe documento débito anterior (ya seleccionada)
			                         50.0, // importe documento débito seleccionado último
			                         true // Se debe permitir la selección
			                         );
		}
		
		[Test]
		public void DocumentoDebitoSeleccionado_NoHayOtrosDebitosSeleccionados_SePuedeSeleccionar_ImporteJusto()
		{
			this.TestDocumentoDebito(new double[] { 150.0 }, // Importe documento crédito anterior (ya seleccionado)
			                         new double[] { }, // importe documento débito anterior (ya seleccionada)
			                         150.0, // importe documento débito seleccionado último
			                         true // Se debe permitir la selección
			                         );
		}
		
		[Test]
		public void DocumentoDebitoSeleccionado__NoHayDocumentosCreditoSeleccionados_NoPuedeSeleccionar()
		{
			this.TestDocumentoDebito(new double[] { }, // Importe documento crédito anterior (ya seleccionado)
			                         new double[] { }, // importe documento débito anterior (ya seleccionada)
			                         150.0, // importe documento débito seleccionado último
			                         false // Se debe permitir la selección
			                         );
		}
		
		[Test]
		public void DocumentoDebitoSeleccionado__HayDocumentosSeleccionados_NoPuedeSeleccionar()
		{
			this.TestDocumentoDebito(new double[] { 175.65 }, // Importe documento crédito anterior (ya seleccionado)
			                         new double[] { 100.0, 75.65 }, // importe documento débito anterior (ya seleccionada)
			                         150.0, // importe documento débito seleccionado último
			                         false // Se debe permitir la selección
			                         );
		}
		#endregion
	}
}
