// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Interfaces;
using Rhino.Mocks.Constraints;

using StratosClient.Tests;
using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Controladores;
using Stratos.Tests;
using StratosClient.Widgets.Level2;

namespace StratosClient.Compras.Tests
{
	[TestFixture()]
	public class BusquedasComprasPresenterTest : StratosClientTestBase
	{
		#region Campos
		protected IBusquedasComprasVista busquedaVista;
		protected ControladorComprasPrx controladorCompras;
		
		protected FacturaCompra factura1;
		protected FacturaCompra factura2;
		#endregion

		#region Métodos de inicialización
		[TestFixtureSetUp]
		public override void FixtureInit ()
		{
			base.FixtureInit ();
		}

		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			/* Obtengo el proxy del controlador de compras accediendo a
			 * la instancia del servidor Stratos que tengo. */
			this.controladorCompras = ControladorComprasPrxHelper.checkedCast(
				this.controladorTerminales.ObtenerProxyControlador("ControladorCompras"));
			
			// Creo unas facturas
			this.factura1 = TestBase.CrearFacturaCompra();
			this.factura1.Proveedor = TestBase.CrearProveedorFerrum();
			this.factura1.Items.Clear();
			this.factura1.Items.Add(CrearItemFacturaCompra(TestBase.CrearProductoGriferia(), 1));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura1, true);
			factura1.Create();
			
			this.factura2 = TestBase.CrearFacturaCompra();
			this.factura2.Proveedor = TestBase.CrearProveedorFV();
			this.factura2.Items.Clear();
			this.factura2.Items.Add(CrearItemFacturaCompra(TestBase.CrearProductoMosaico(), 2));
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura2, true);
			factura2.Create();
			
			this.FlushAndRecreateScope();
		}
		#endregion
		
		[Ignore("Test no terminado")]
		public void BusquedaProveedorFerrum ()
		{
			// Vuelvo el MockRepository al modo Record
			this.mocks.BackToRecordAll();
			
			ClienteDto clienteDto = ConvertidorDto.ToDto(this.ClienteSinBonificacion);
			
			// Mock para IRazonSocialVista
			IBusquedasComprasVista busquedaVista = this.mocks.Stub<IBusquedasComprasVista>();
			
			// Mock para IListaFacturasAsociadasVista.
			IColeccionElementosVista listaFacturas =
				this.mocks.DynamicMock<IListaSeleccionable<FacturaCompraDto>>();
		}
	}
}
