// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

using NUnit.Framework;
using Rhino.Mocks;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Framework;
using NHibernate.Criterion;

using Stratos.Tests;
using StratosServer.Tests;
using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Clientes;
using Stratos.Common.Excepciones;

namespace StratosServer.Clientes.Tests
{
	[TestFixture]
	public class ControladorClientesITests : StratosServerDocumentoGenericoTestBase
	{
		#region Campos
		private ControladorClientesI controlador;
		#endregion
		
		#region Métodos de inicialización
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			this.controlador =
				new ControladorClientesI();
			
			ControladorClientesI.CANTIDAD_MAXIMA_RESULTADOS_MOVIMIENTOS = 200;
		}
		#endregion
		
		[Test]
		[Ignore("Test no implementado aún")]
		public void BuscarHQLTest()
		{
		}
		
		[Test]
		[Ignore("Test no implementado aún")]
		public void BuscarQueryObjectsTest()
		{
		}
		
		[Test]
		public void ObtenerClienteConsumidorFinalGenericoTest()
		{
			string consulta = "from PreFactura";
			
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado = this.controlador.ObtenerClienteConsumidorFinalGenerico();
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad retornado");
			Assert.AreEqual(ControladorClientesI.CONSUMIDOR_FINAL_RAZON_SOCIAL,
			                resultado[0].razonSocial);
			Assert.AreEqual(TipoResponsable.ConsumidorFinal, resultado[0].tipoRes);
			Assert.AreEqual(0.00, resultado[0].porcentajeBonificacion);
		}
		
		[Test]
		[Ignore("Ver por qué tarda tanto")]
		public void AgregarClienteTest()
		{
			// Creo un cliente nuevo, que no esté ingresado en la base de datos
			ClienteDto clienteElvio = new ClienteDto();
			clienteElvio.razonSocial = "Elvio L. Sandrigo";
			
			clienteElvio.domicilio = new DomicilioDto();
			clienteElvio.domicilio.direccion = "Zona urbana";
			clienteElvio.domicilio.localidad = "Guadalupe Norte";
			clienteElvio.domicilio.provincia = ProvinciaDto.SantaFe;
			
			clienteElvio.tipoRes = TipoResponsable.ResponsableInscripto;
			clienteElvio.cuit = "20-11926039-7";
			
			clienteElvio.porcentajeBonificacion = 10.00d;
			
			this.RemoveTransaction();
			
			// Llamo a método AgregarCliente del controlador
			ClienteDto clienteElvioAgregado = (ClienteDto)
				this.controlador.Guardar(clienteElvio);
			
			// Chequeo que el cliente que devuelve en el parametro out tenga la misma razón social
			Assert.AreEqual(clienteElvio.razonSocial, clienteElvioAgregado.razonSocial, "Razón social cliente agregado");
			
			this.FlushAndRecreateReadOnlyScope();
			
			Cliente clienteRecuperado = null;
			try {
				// Ahora veo si el cliente efectivamente se dió de alta en la base de datos
				clienteRecuperado = Cliente.Find(clienteElvioAgregado.codigo);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			// Chequeo que el cliente que está en la base de datos tenga los mismos datos que el agregado
			Assert.AreEqual(clienteElvioAgregado.codigo, clienteRecuperado.Codigo, "Código cliente recuperado");
			Assert.AreEqual(clienteElvioAgregado.razonSocial, clienteRecuperado.RazonSocial, "Razón social cliente recuperado");
		}
		
		[Test]
		public void BusquedaExactaPorCodigoTest()
		{
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado = (ClienteDto[])
					this.controlador
						.BusquedaExactaPorCodigo(this.ClienteConBonificacion.Codigo);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de clientes encontrados");
			Assert.AreEqual(this.ClienteConBonificacion.Codigo, resultado[0].codigo, "Código");
			Assert.AreEqual(this.ClienteConBonificacion.RazonSocial, resultado[0].razonSocial, "Razón social");
		}
		
		[Test]
		public void BusquedaExactaPorRazonSocialTest()
		{
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado = (ClienteDto[])
					this.controlador
						.BusquedaExactaPorRazonSocial(this.ClienteConBonificacion.RazonSocial);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de clientes encontrados");
			Assert.AreEqual(this.ClienteConBonificacion.Codigo, resultado[0].codigo, "Código");
			Assert.AreEqual(this.ClienteConBonificacion.RazonSocial, resultado[0].razonSocial, "Razón social");
		}
		
		[Test]
		public void BusquedaExactaPorRazonSocial_MenosDeTresCaracteresDeBusqueda()
		{
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado = (ClienteDto[])
					this.controlador
						.BusquedaExactaPorRazonSocial("a");
				
				Assert.Fail("Se pudo ejecutar la consulta");
			}
			catch (ParametrosInvalidosException ex) {
			}
		}
		
		[Test]
		public void ConsultaPorRazonSocialTest()
		{
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado =
					this.controlador
						.ConsultaPorRazonSocial("Milt").Cast<ClienteDto>().ToArray();
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de clientes encontrados");
			Assert.AreEqual(this.ClienteConBonificacion.Codigo, resultado[0].codigo, "Código");
			Assert.AreEqual(this.ClienteConBonificacion.RazonSocial, resultado[0].razonSocial, "Razón social");
		}
		
		[Test]
		public void ConsultaPorRazonSocial_MenosDeTresCaracteresDeBusqueda()
		{
			ClienteDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				resultado =
					this.controlador
						.ConsultaPorRazonSocial("i").Cast<ClienteDto>().ToArray();
				
				Assert.Fail("Se pudo realizar la consulta");
			}
			catch (ParametrosInvalidosException ex) {
			}
		}
		
		[Test]
		public void ObtenerClienteInvalido_ConsumidorFinal()
		{
			this.RemoveTransaction();
			
			Cliente clienteInvalido =
				this.controlador.ObtenerClienteInvalido(TipoResponsable.ConsumidorFinal);
			
			Assert.AreEqual(ControladorClientesI.CLIENTE_INVALIDO_RAZON_SOCIAL,
			                clienteInvalido.RazonSocial,
			                "Razón social");
			Assert.AreEqual(TipoResponsable.ConsumidorFinal,
			                clienteInvalido.TipoResponsable,
			                "Tipo responsable");
		}
		
		[Test]
		public void ObtenerClienteInvalido_ResponsableInscripto()
		{
			this.RemoveTransaction();
			
			Cliente clienteInvalido =
				this.controlador.ObtenerClienteInvalido(TipoResponsable.ResponsableInscripto);
			
			Assert.AreEqual(ControladorClientesI.CLIENTE_INVALIDO_RAZON_SOCIAL,
			                clienteInvalido.RazonSocial,
			                "Razón social");
			Assert.AreEqual(TipoResponsable.ResponsableInscripto,
			                clienteInvalido.TipoResponsable,
			                "Tipo responsable");
		}
		
		[Test]
		public void ObtenerClienteInvalido_ConsultaDoble()
		{
			/* Me aseguro que tras una segunda llamada no se cree otro cliente inválido,
			 * sino que se devuelva el mismo. */
			
			this.RemoveTransaction();
			
			Cliente clienteInvalido =
				this.controlador.ObtenerClienteInvalido(TipoResponsable.ConsumidorFinal);
			
			clienteInvalido =
				this.controlador.ObtenerClienteInvalido(TipoResponsable.ConsumidorFinal);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Cliente[] resultado =
				Cliente.FindAllByProperty("RazonSocial",
				                          ControladorClientesI.CLIENTE_INVALIDO_RAZON_SOCIAL);
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de clientes inválidos devueltos");
		}
		
		#region ObtenerMovimientos - Tests
		[Test]
		public void ObtenerMovimientos_ClienteNoExiste()
		{
			bool resultadosRecortados;
			
			try {
				this.controlador.ObtenerMovimientos(123123123, 1, 2, out resultadosRecortados);
				Assert.Fail("Se pudo realizar la consulta");
			}
			catch (ObjetoInexistenteException) {
			}
		}
		
		[Test]
		public void ObtenerMovimientos_HayFacturas_Y_Recibos_EnFechaEspecificada()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today.AddDays(-15);
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Fecha = DateTime.Today.AddDays(-15);
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(recibo);
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo la consulta
			DocumentoGenericoDto[] movimientos = null;
			
			this.RemoveTransaction();
			
			bool resultadosRecortados = false;
			
			try {
				movimientos =
					this.controlador
						.ObtenerMovimientos(this.ClienteSinBonificacion.Codigo,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    out resultadosRecortados);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción al realizar consulta: " + ex.mensaje);
			}
			
			Assert.IsNotNull(movimientos, "Movimientos no nulo");
			Assert.AreEqual(2, movimientos.Length, "Movimientos cantidad");
			Assert.IsFalse(resultadosRecortados, "Resultados recortados");
			
			// Hay una factura en el resultado, y es la que espero
			FacturaDto[] facturasRetornadas =
				(from doc in movimientos
				 where doc is FacturaDto
				 select doc as FacturaDto).ToArray();
			
			Assert.AreEqual(1, facturasRetornadas.Length, "Cantidad facturas retornadas");
			Assert.AreEqual(factura.Id, facturasRetornadas[0].id, "Id factura retornada");
			Assert.AreEqual(this.ClienteSinBonificacion.Codigo, facturasRetornadas[0].cliente.codigo, "Cliente factura retornada");
			
			// Hay un recibo en el resultado y es el que espero
			ReciboDto[] recibosRetornados =
				(from doc in movimientos
				 where doc is ReciboDto
				 select doc as ReciboDto).ToArray();
			
			Assert.AreEqual(1, recibosRetornados.Length, "Cantidad recibos retornadas");
			Assert.AreEqual(recibo.Id, recibosRetornados[0].id, "Id recibo retornada");
			Assert.AreEqual(this.ClienteSinBonificacion.Codigo, recibosRetornados[0].cliente.codigo, "Cliente recibo retornado");
		}
		
		[Test]
		public void ObtenerMovimientos_HayFacturas_Y_Recibos_EnFechaEspecificada_TambienHay_FacturasYRecibos_De_OtroCliente()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today.AddDays(-15);
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura de otro cliente
			Factura otraFactura = TestBase.CrearFactura();
			otraFactura.Tipo = FacturaTipoVenta.FacturaB;
			otraFactura.Codigo = 1;
			otraFactura.Numero = 1;
			otraFactura.Fecha = DateTime.Today.AddDays(-10);
			otraFactura.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;
			otraFactura.Items.Clear();
			otraFactura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(otraFactura);
			otraFactura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Fecha = DateTime.Today.AddDays(-15);
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(recibo);
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo de otro cliente
			Recibo otroRecibo = TestBase.CrearRecibo();
			otroRecibo.Fecha = DateTime.Today.AddDays(-20);
			otroRecibo.Cliente = this.ClienteConBonificacion;
			otroRecibo.Items.Clear();
			otroRecibo.Items.Add(new ItemRecibo(otraFactura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(otroRecibo);
			otroRecibo.Concepto = "Concepto del recibo";
			otroRecibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo la consulta
			DocumentoGenericoDto[] movimientos = null;
			
			this.RemoveTransaction();
			
			bool resultadosRecortados = false;
			
			try {
				movimientos =
					this.controlador
						.ObtenerMovimientos(this.ClienteSinBonificacion.Codigo,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    out resultadosRecortados);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción al realizar consulta: " + ex.mensaje);
			}
			
			Assert.IsNotNull(movimientos, "Movimientos no nulo");
			Assert.AreEqual(2, movimientos.Length, "Movimientos cantidad");
			Assert.IsFalse(resultadosRecortados, "Resultados recortados");
			
			// Hay una factura en el resultado, y es la que espero
			FacturaDto[] facturasRetornadas =
				(from doc in movimientos
				 where doc is FacturaDto
				 select doc as FacturaDto).ToArray();
			
			Assert.AreEqual(1, facturasRetornadas.Length, "Cantidad facturas retornadas");
			Assert.AreEqual(factura.Id, facturasRetornadas[0].id, "Id factura retornada");
			Assert.AreEqual(this.ClienteSinBonificacion.Codigo, facturasRetornadas[0].cliente.codigo, "Cliente factura retornada");
			
			// Hay un recibo en el resultado y es el que espero
			ReciboDto[] recibosRetornados =
				(from doc in movimientos
				 where doc is ReciboDto
				 select doc as ReciboDto).ToArray();
			
			Assert.AreEqual(1, recibosRetornados.Length, "Cantidad recibos retornadas");
			Assert.AreEqual(recibo.Id, recibosRetornados[0].id, "Id recibo retornada");
			Assert.AreEqual(this.ClienteSinBonificacion.Codigo, recibosRetornados[0].cliente.codigo, "Cliente recibo retornado");
		}
		
		[Test]
		public void ObtenerMovimientos_HayFacturas_Y_Recibos_EnFechaEspecificada_TambienHay_Facturas_EnOtraFecha()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today.AddDays(-20);
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura en otra fecha
			Factura otraFactura = TestBase.CrearFactura();
			otraFactura.Codigo = 1;
			otraFactura.Numero = 2;
			otraFactura.Fecha = DateTime.Today.AddDays(-15);
			otraFactura.Cliente = this.ClienteSinBonificacion;
			otraFactura.Items.Clear();
			otraFactura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(otraFactura);
			otraFactura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Fecha = DateTime.Today.AddDays(-25);
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(recibo);
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo de otro cliente
			Recibo otroRecibo = TestBase.CrearRecibo();
			otroRecibo.Fecha = DateTime.Today.AddDays(-28);
			otroRecibo.Cliente = this.ClienteSinBonificacion;
			otroRecibo.Items.Clear();
			otroRecibo.Items.Add(new ItemRecibo(otraFactura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(otroRecibo);
			otroRecibo.Concepto = "Concepto del recibo";
			otroRecibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo la consulta
			DocumentoGenericoDto[] movimientos = null;
			
			this.RemoveTransaction();
			
			bool resultadosRecortados = false;
			
			try {
				movimientos =
					this.controlador
						.ObtenerMovimientos(this.ClienteSinBonificacion.Codigo,
						                    (DateTime.Today.AddDays(-20)).Ticks,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    out resultadosRecortados);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción al realizar consulta: " + ex.mensaje);
			}
			
			Assert.IsNotNull(movimientos, "Movimientos no nulo");
			Assert.AreEqual(2, movimientos.Length, "Movimientos cantidad");
			Assert.IsFalse(resultadosRecortados, "Resultados recortados");
			
			// Hay una factura en el resultado, y es la que espero
			FacturaDto[] facturasRetornadas =
				(from doc in movimientos
				 where doc is FacturaDto
				 select doc as FacturaDto).ToArray();
			
			Assert.AreEqual(2, facturasRetornadas.Length, "Cantidad facturas retornadas");
			Assert.IsTrue(facturasRetornadas.Select(fac => fac.id).Distinct().Count() == 2, "Todos los ids de factura distintos");
			Assert.IsTrue(facturasRetornadas.All(fac => new long[] { factura.Id, otraFactura.Id }.Contains(fac.id)), "Ids facturas");
			
			// Hay un recibo en el resultado y es el que espero
			ReciboDto[] recibosRetornados =
				(from doc in movimientos
				 where doc is ReciboDto
				 select doc as ReciboDto).ToArray();
			
			Assert.AreEqual(0, recibosRetornados.Length, "Cantidad recibos retornadas");
		}
		
		[Test]
		public void ObtenerMovimientos_Retorno_EnOrden_Fecha_Ascendente()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today.AddDays(-20);
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura en otra fecha
			Factura otraFactura = TestBase.CrearFactura();
			otraFactura.Codigo = 1;
			otraFactura.Numero = 2;
			otraFactura.Fecha = DateTime.Today.AddDays(-15);
			otraFactura.Cliente = this.ClienteSinBonificacion;
			otraFactura.Items.Clear();
			otraFactura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(otraFactura);
			otraFactura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Fecha = DateTime.Today.AddDays(-18);
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(recibo);
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo de otro cliente
			Recibo otroRecibo = TestBase.CrearRecibo();
			otroRecibo.Fecha = DateTime.Today.AddDays(-28);
			otroRecibo.Cliente = this.ClienteSinBonificacion;
			otroRecibo.Items.Clear();
			otroRecibo.Items.Add(new ItemRecibo(otraFactura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(otroRecibo);
			otroRecibo.Concepto = "Concepto del recibo";
			otroRecibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo la consulta
			DocumentoGenericoDto[] movimientos = null;
			
			this.RemoveTransaction();
			
			bool resultadosRecortados = false;
			
			try {
				movimientos =
					this.controlador
						.ObtenerMovimientos(this.ClienteSinBonificacion.Codigo,
						                    (DateTime.Today.AddDays(-20)).Ticks,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    out resultadosRecortados);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción al realizar consulta: " + ex.mensaje);
			}
			
			Assert.IsNotNull(movimientos, "Movimientos no nulo");
			Assert.AreEqual(3, movimientos.Length, "Movimientos cantidad");
			Assert.IsFalse(resultadosRecortados, "Resultados recortados");
			
			// Verifico que los resultados estén ordenados
			Assert.AreEqual(factura.Id, movimientos[0].id, "Primer elemento");
			Assert.AreEqual(recibo.Id, movimientos[1].id, "Segundo elemento");
			Assert.AreEqual(otraFactura.Id, movimientos[2].id, "Tercer elemento");
		}
		
		[Test]
		public void ObtenerMovimientos_Resultados_SeRecortan()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today.AddDays(-20);
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura en otra fecha
			Factura otraFactura = TestBase.CrearFactura();
			otraFactura.Codigo = 1;
			otraFactura.Numero = 2;
			otraFactura.Fecha = DateTime.Today.AddDays(-16);
			otraFactura.Cliente = this.ClienteSinBonificacion;
			otraFactura.Items.Clear();
			otraFactura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(otraFactura);
			otraFactura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Fecha = DateTime.Today.AddDays(-18);
			recibo.Cliente = this.ClienteSinBonificacion;
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(recibo);
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo de otro cliente
			Recibo otroRecibo = TestBase.CrearRecibo();
			otroRecibo.Fecha = DateTime.Today.AddDays(-15);
			otroRecibo.Cliente = this.ClienteSinBonificacion;
			otroRecibo.Items.Clear();
			otroRecibo.Items.Add(new ItemRecibo(otraFactura.ImporteTotal));
			TestBase.ProcesarItemsRecibo(otroRecibo);
			otroRecibo.Concepto = "Concepto del recibo";
			otroRecibo.Create();
			
			this.FlushAndRecreateScope();
			
			/* En este punto tengo creados 4 documentos que deben ser retornados.
			 * Modifico el punto de recorte a 2, y verifico que además de retorar
			 * sólo ésta cantidad de resultados */
			ControladorClientesI.CANTIDAD_MAXIMA_RESULTADOS_MOVIMIENTOS = 2;
			
			// Realizo la consulta
			DocumentoGenericoDto[] movimientos = null;
			
			this.RemoveTransaction();
			
			bool resultadosRecortados = false;
			
			try {
				movimientos =
					this.controlador
						.ObtenerMovimientos(this.ClienteSinBonificacion.Codigo,
						                    (DateTime.Today.AddDays(-20)).Ticks,
						                    (DateTime.Today.AddDays(-15)).Ticks,
						                    out resultadosRecortados);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción al realizar consulta: " + ex.mensaje);
			}
			
			Assert.IsNotNull(movimientos, "Movimientos no nulo");
			Assert.AreEqual(2, movimientos.Length, "Movimientos cantidad");
			Assert.IsTrue(resultadosRecortados, "Resultados recortados");
			
			// Debe haber una factura en el resultado (la otra se recorta)
			FacturaDto[] facturasRetornadas =
				(from doc in movimientos
				 where doc is FacturaDto
				 select doc as FacturaDto).ToArray();
			
			Assert.AreEqual(1, facturasRetornadas.Length, "Cantidad facturas retornadas");
			Assert.AreEqual(factura.Id, facturasRetornadas[0].id, "Id facturas retornadas");
			
			// Hay dos recibos en el resultado
			ReciboDto[] recibosRetornados =
				(from doc in movimientos
				 where doc is ReciboDto
				 select doc as ReciboDto).ToArray();
			
			Assert.AreEqual(1, recibosRetornados.Length, "Cantidad recibos retornadas");
			Assert.AreEqual(recibo.Id, recibosRetornados[0].id, "Id recibo retornado");
		}
		#endregion
		
		#region Cuenta corriente
		#region ObtenerDocumentosCredito
		[Test]
		public void ObtenerDocumentosCredito_NoHayDocumentosCredito()
		{
			DocumentoCreditoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(0, resultado.Length, "Cantidad de resultados");
		}
		
		[Test]
		public void ObtenerDocumentosCredito_HayUnRecibo()
		{
			// Creo un documento de crédito (Recibo)
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(552.88m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			DocumentoCreditoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultados");
			
			Assert.AreEqual(TipoDocumento.Recibo, resultado[0].tipoDoc, "Tipo documento");
			Assert.AreEqual(recibo.Id, resultado[0].idDocumento, "Id documento");
		}
		
		[Test]
		public void ObtenerDocumentosCredito_HayUnaNotaCredito()
		{
			// Creo una nota de crédito a partir de una factura
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = Factura.Find(factura.Id);
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.RemoveTransaction();
			
			DocumentoCreditoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultados");
			
			Assert.AreEqual(TipoDocumento.NotaCredito, resultado[0].tipoDoc, "Tipo documento");
			Assert.AreEqual(notaCredito.Id, resultado[0].idDocumento, "Id documento");
		}
		
		[Test]
		public void ObtenerDocumentosCredito_HayUnRecibo_DeOtroCliente()
		{
			// Creo un documento de crédito (Recibo) con otro cliente
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteConBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(552.88m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			DocumentoCreditoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			// No debe haber resultados porque consulté por otro cliente
			Assert.AreEqual(0, resultado.Length, "Cantidad de resultados");
		}
		#endregion
		
		#region ObtenerDocumentosDebito
		[Test]
		public void ObtenerDocumentosDebito_NoHayDocumentosDebito()
		{
			DocumentoDebitoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(0, resultado.Length, "Cantidad de resultados");
		}
		
		[Test]
		public void ObtenerDocumentosDebito_HayUnaFactura()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.RemoveTransaction();
			
			DocumentoDebitoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultados");
			
			Assert.AreEqual(TipoDocumento.Factura, resultado[0].tipoDoc, "Tipo documento");
			Assert.AreEqual(factura.Id, resultado[0].idDocumento, "Id documento");
		}
		
		[Test]
		public void ObtenerDocumentosDebito_HayUnaFactura_DeOtroCliente()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.RemoveTransaction();
			
			DocumentoDebitoDto[] resultado = null;
			
			try {
				resultado = this.controlador.ObtenerDocumentosDebito(this.ClienteConBonificacion.Codigo);
			}
			catch(StratosException exception) {
				Assert.Fail("StratosException: " + exception.mensaje);
			}
			
			Assert.AreEqual(0, resultado.Length, "Cantidad de resultados");
		}
		#endregion
		
		#region Imputar
		#region Métodos auxiliares
		protected DocumentoDebito ObtenerDocumentoDebito(Factura factura)
		{
			DocumentoDebito[] resultado =
				DocumentoDebito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Factura),
				                        Restrictions.Eq("IdDocumento", factura.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoDebito: más de un documento débito devuelto");
			
			return resultado[0];
		}
		
		protected DocumentoCredito ObtenerDocumentoCredito(Recibo recibo)
		{
			DocumentoCredito[] resultado =
				DocumentoCredito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.Recibo),
				                         Restrictions.Eq("IdDocumento", recibo.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoCredito: más de un documento de crédito devuelto");
			
			return resultado[0];
		}
		
		protected DocumentoCredito ObtenerDocumentoCredito(NotaCredito notaCredito)
		{
			DocumentoCredito[] resultado =
				DocumentoCredito.FindAll(Restrictions.Eq("TipoDocumento", TipoDocumento.NotaCredito),
				                         Restrictions.Eq("IdDocumento", notaCredito.Id));
			
			if (resultado.Length == 0)
				return null;
			else if (resultado.Length > 1)
				throw new Exception("DocumentoCredito: más de un documento de crédito devuelto");
			
			return resultado[0];
		}
		#endregion
		
		[Test]
		public void Imputar_SinDocumentos()
		{
			try {
				this.controlador.Imputar(new DocumentoCreditoDto[0],
				                         new DocumentoDebitoDto[0]);
			}
			catch (ParametrosInvalidosException) {
			}
		}
		
		[Test]
		[Ignore("Se podría escribir un test que verifique esta validaciión en el servidor")]
		public void Imputar_DocumentosDebito_TienenClienteDistinto()
		{
		}
		
		[Test]
		public void Imputar_ImporteDebito_MayorImporte_Que_ImporteCredito()
		{
			// Creo facturas con clientes distintos
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal / 2.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// El documento de débito de la factura, tiene que tener un importe
			// pendiente igual al total menos el importe del recibo.
			DocumentoDebito documentoDebitoFactura = this.ObtenerDocumentoDebito(factura);
			Assert.IsNotNull(documentoDebitoFactura, "Documento débito de factura no nulo");
			Assert.AreEqual(factura.ImporteTotal, documentoDebitoFactura.ImporteTotal, "Documento débito importe total");
			Assert.AreEqual(factura.ImporteTotal - recibo.Importe, documentoDebitoFactura.ImportePendiente, "Documento débito importe pendiente");
			
			// El documento de crédito correspondiente al recibo se debió
			// haber eliminado
			Assert.IsNull(this.ObtenerDocumentoCredito(recibo), "Documentos de crédito existentes");
		}
		
		[Test]
		public void Imputar_ImporteCredito_MayorImporte_Que_ImporteDebito()
		{
			// Creo facturas con clientes distintos
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal * 2.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// El documento de crédito del recibo, tiene que tener un importe
			// pendiente igual al total menos el importe de la factura.
			DocumentoCredito documentoCreditoRecibo = this.ObtenerDocumentoCredito(recibo);
			Assert.IsNotNull(documentoCreditoRecibo, "Documento crédito de recibo no nulo");
			Assert.AreEqual(recibo.Importe, documentoCreditoRecibo.ImporteTotal, "Documento crédito importe total");
			Assert.AreEqual(recibo.Importe - factura.ImporteTotal, documentoCreditoRecibo.ImportePendiente, "Documento crédito importe pendiente");
			
			// El documento de débito de la factura no debe existir más,
			// porque ya fue cubierto por el recibo
			Assert.IsNull(this.ObtenerDocumentoDebito(factura), "Documentos de débito existentes");
		}
		
		[Test]
		public void Imputar_ImporteCredito_Igual_Que_ImporteDebito()
		{
			// Creo facturas con clientes distintos
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// No debe haber documentos de crédito existentes, ya que se
			// imputaron completamente.
			Assert.IsNull(this.ObtenerDocumentoCredito(recibo), "Documentos de crédito existentes");
			
			// No debe haber documentos de débito existentes, ya que se
			// imputaron completamente.
			Assert.IsNull(this.ObtenerDocumentoDebito(factura), "Documentos de débito existentes");
		}
		
		#region Imputar - Múltiples documentos de crédito
		[Test]
		public void Imputar_MultiplesDocumentosCredito_ImporteDebito_MayorImporte_Que_ImporteCredito()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una nota de crédito asociada a la factura anterior
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = Factura.Find(factura.Id);
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(this.ProductoGriferia.ImporteVenta));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			notaCredito = NotaCredito.Find(notaCredito.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// El documento de débito de la factura debe existir, y tener un importe
			// pendiente igual al importe total de la factura menos el importe total
			// de la nota de crédito y el del recibo.
			DocumentoDebito documentoDebitoFactura = this.ObtenerDocumentoDebito(factura);
			Assert.IsNotNull(documentoDebitoFactura, "Documento débito de factura no nulo");
			Assert.AreEqual(factura.ImporteTotal, documentoDebitoFactura.ImporteTotal, "Documento débito importe total");
			Assert.AreEqual(factura.ImporteTotal - notaCredito.ImporteTotal - recibo.Importe,
			                documentoDebitoFactura.ImportePendiente, "Documento débito importe pendiente");
			
			// Como el importe total de débito es mayor al importe total
			// de crédito imputado, no debe haber documentos de crédito
			// existentes.
			Assert.IsNull(this.ObtenerDocumentoCredito(recibo), "Documentos de crédito del recibo");
			Assert.IsNull(this.ObtenerDocumentoCredito(notaCredito), "Documentos de crédito de la nota de crédito");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosCredito_ImporteCredito_MayorImporte_Que_ImporteDebito()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una nota de crédito asociada a la factura anterior
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = Factura.Find(factura.Id);
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 8));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(this.ProductoGriferia.ImporteVenta * 5.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			notaCredito = NotaCredito.Find(notaCredito.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// No debe quedar el documento de débito de la factura
			Assert.IsNull(this.ObtenerDocumentoDebito(factura), "Documento débito de factura nulo");
			
			// El documento de crédito de la nota de crédito debió haberse eliminado,
			// y debió haber quedado el del recibo con el importe pendiente correspondiente.
			Assert.IsNull(this.ObtenerDocumentoCredito(notaCredito), "Documentos de crédito de la nota de crédito");
			
			DocumentoCredito documentoCreditoRecibo = this.ObtenerDocumentoCredito(recibo);
			Assert.IsNotNull(documentoCreditoRecibo, "Documentos de crédito del recibo");
			Assert.AreEqual(recibo.Importe, documentoCreditoRecibo.ImporteTotal, "Documento crédito importe total");
			Assert.AreEqual(recibo.Importe - (factura.ImporteTotal - notaCredito.ImporteTotal),
			                documentoCreditoRecibo.ImportePendiente, "Documento crédito importe pendiente");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosCredito_ImporteCredito_Igual_Que_ImporteDebito()
		{
			// Creo una factura
			Factura factura = TestBase.CrearFactura();
			factura.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 12));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una nota de crédito asociada a la factura anterior
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = Factura.Find(factura.Id);
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 8));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(this.ProductoGriferia.ImporteVenta * 4.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			notaCredito = NotaCredito.Find(notaCredito.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// No debe quedar el documento de débito de la factura
			Assert.IsNull(this.ObtenerDocumentoDebito(factura), "Documento débito de factura nulo");
			
			// El documento de crédito de la nota de crédito debió haberse eliminado,
			// y debió haber quedado el del recibo con el importe pendiente correspondiente.
			Assert.IsNull(this.ObtenerDocumentoCredito(notaCredito), "Documentos de crédito de la nota de crédito");
			Assert.IsNull(this.ObtenerDocumentoCredito(recibo), "Documentos de crédito del recibo");
		}
		#endregion
		
		#region Imputar - Múltiples documentos de débito
		[Test]
		public void Imputar_MultiplesDocumentosDebito_ImporteDebito_MayorImporte_Que_ImporteCredito()
		{
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(this.ProductoGriferia.ImporteVenta * 11.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura1 = Factura.Find(factura1.Id);
			factura2 = Factura.Find(factura2.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura1 = this.ObtenerDocumentoDebito(factura1);
			Assert.IsNull(documentoDebitoFactura1, "Documento débito de factura nulo");
			
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(factura2);
			Assert.IsNotNull(documentoDebitoFactura2, "Documento débito de factura no nulo");
			Assert.AreEqual(factura2.ImporteTotal, documentoDebitoFactura2.ImporteTotal, "Documento débito importe total");
			Assert.AreEqual(factura2.ImporteTotal - (recibo.Importe - factura1.ImporteTotal),
			                documentoDebitoFactura2.ImportePendiente, "Documento débito importe pendiente");
			
			// Como el importe total de débito es mayor al importe total
			// de crédito imputado, no debe haber documentos de crédito
			// existentes.
			Assert.IsNull(this.ObtenerDocumentoCredito(recibo), "Documentos de crédito del recibo");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosDebito_ImporteCredito_MayorImporte_Que_ImporteDebito()
		{
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura1.ImporteTotal));
			recibo.Items.Add(new ItemRecibo(factura2.ImporteTotal + 100.0m));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura1 = Factura.Find(factura1.Id);
			factura2 = Factura.Find(factura2.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura1 = this.ObtenerDocumentoDebito(factura1);
			Assert.IsNull(documentoDebitoFactura1, "Documento débito de factura nulo");
			
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(factura2);
			Assert.IsNull(documentoDebitoFactura2, "Documento débito de factura nulo");
			
			// Como el importe total de débito es mayor al importe total
			// de crédito imputado, no debe haber documentos de crédito
			// existentes.
			DocumentoCredito documentoCreditoRecibo = this.ObtenerDocumentoCredito(recibo);
			Assert.IsNotNull(documentoCreditoRecibo, "Documento crédito de factura no nulo");
			Assert.AreEqual(recibo.Importe, documentoCreditoRecibo.ImporteTotal, "Documento crédito importe total");
			Assert.AreEqual(recibo.Importe - factura1.ImporteTotal - factura2.ImporteTotal,
			                documentoCreditoRecibo.ImportePendiente, "Documento crédito importe pendiente");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosDebito_ImporteCredito_Igual_Que_ImporteDebito()
		{
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo = TestBase.CrearRecibo();
			recibo.Cliente = this.ClienteSinBonificacion;
			
			recibo.Items.Clear();
			recibo.Items.Add(new ItemRecibo(factura1.ImporteTotal));
			recibo.Items.Add(new ItemRecibo(factura2.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(recibo);
			
			recibo.Concepto = "Concepto del recibo";
			recibo.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura1 = Factura.Find(factura1.Id);
			factura2 = Factura.Find(factura2.Id);
			recibo = Recibo.Find(recibo.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura1 = this.ObtenerDocumentoDebito(factura1);
			Assert.IsNull(documentoDebitoFactura1, "Documento débito de factura1 nulo");
			
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(factura2);
			Assert.IsNull(documentoDebitoFactura2, "Documento débito de factura2 nulo");
			
			// Documentos de crédito
			DocumentoCredito documentoCreditoRecibo = this.ObtenerDocumentoCredito(recibo);
			Assert.IsNull(documentoCreditoRecibo, "Documento crédito de recibo nulo");
		}
		#endregion
		
		#region Imputar - Multiples documentos de ambos
		[Test]
		public void Imputar_MultiplesDocumentosAmbos_ImporteCredito_MayorQue_ImporteDebito()
		{
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo1 = TestBase.CrearRecibo();
			recibo1.Cliente = this.ClienteSinBonificacion;
			
			recibo1.Items.Clear();
			recibo1.Items.Add(new ItemRecibo(factura1.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(recibo1);
			
			recibo1.Concepto = "Concepto del recibo";
			recibo1.Create();
			
			// Creo un recibo
			Recibo recibo2 = TestBase.CrearRecibo();
			recibo2.Cliente = this.ClienteSinBonificacion;
			
			recibo2.Items.Clear();
			recibo2.Items.Add(new ItemRecibo(factura2.ImporteTotal + 100.0m));
			
			TestBase.ProcesarItemsRecibo(recibo2);
			
			recibo2.Concepto = "Concepto del recibo";
			recibo2.Create();
			
			this.RemoveTransaction();
			
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosCredito.Length, "Cantidad documentos crédito");
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(2, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura1 = Factura.Find(factura1.Id);
			factura2 = Factura.Find(factura2.Id);
			recibo1 = Recibo.Find(recibo1.Id);
			recibo2 = Recibo.Find(recibo2.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura1 = this.ObtenerDocumentoDebito(factura1);
			Assert.IsNull(documentoDebitoFactura1, "Documento débito de factura1 nulo");
			
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(factura2);
			Assert.IsNull(documentoDebitoFactura2, "Documento débito de factura2 nulo");
			
			// Documentos de crédito
			DocumentoCredito documentoCreditoRecibo1 = this.ObtenerDocumentoCredito(recibo1);
			Assert.IsNull(documentoCreditoRecibo1, "Documento crédito de recibo1 nulo");
			
			DocumentoCredito documentoCreditoRecibo2 = this.ObtenerDocumentoCredito(recibo2);
			Assert.IsNotNull(documentoCreditoRecibo2, "Documento crédito de recibo2 nulo");
			Assert.AreEqual(recibo2.Importe, documentoCreditoRecibo2.ImporteTotal, "Documento crédito importe total");
			Assert.AreEqual(100.0m, documentoCreditoRecibo2.ImportePendiente, "Documento crédito importe pendiente");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosAmbos_DocumentoCredito_YaImputadoAntes()
		{
			#region PreEscenario - 2 documentos débito y 2 documentos crédito
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo1 = TestBase.CrearRecibo();
			recibo1.Cliente = this.ClienteSinBonificacion;
			
			recibo1.Items.Clear();
			recibo1.Items.Add(new ItemRecibo(factura1.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(recibo1);
			
			recibo1.Concepto = "Concepto del recibo";
			recibo1.Create();
			
			// Creo un recibo
			Recibo recibo2 = TestBase.CrearRecibo();
			recibo2.Cliente = this.ClienteSinBonificacion;
			
			recibo2.Items.Clear();
			recibo2.Items.Add(new ItemRecibo(factura2.ImporteTotal + 100.0m));
			
			TestBase.ProcesarItemsRecibo(recibo2);
			
			recibo2.Concepto = "Concepto del recibo";
			recibo2.Create();
			
			this.RemoveTransaction();
			
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			#endregion
			
			
			this.FlushAndRecreateScope();
			
			// En este punto tengo un documento de crédito con importe disponible
			// Creo una factura y los imputo
			Factura factura3 = TestBase.CrearFactura();
			factura3.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura3.Codigo = 1;
			factura3.Numero = 3;
			factura3.Cliente = this.ClienteSinBonificacion;
			factura3.Items.Clear();
			factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura3);
			factura3.Create();
			
			// Me aseguro que el importe final de la factura sea mayor a 100.0
			Assert.IsTrue(factura3.ImporteTotal > 100.0m, "Importe factura mayor a importe pendiente recibo");
			
			this.RemoveTransaction();
			
			// Me aseguro que haya la cantidad de documentos correspondiente
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura3 = Factura.Find(factura3.Id);
			recibo2 = Recibo.Find(recibo2.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura3 = this.ObtenerDocumentoDebito(factura3);
			Assert.IsNotNull(documentoDebitoFactura3, "Documento débito de factura3 nulo");
			Assert.AreEqual(factura3.ImporteTotal, documentoDebitoFactura3.ImporteTotal, "Documento débito de factura3 importe total");
			Assert.AreEqual(factura3.ImporteTotal - 100.0m, documentoDebitoFactura3.ImportePendiente, "Documento débito de factura3 importe pendiente");
			
			// Documentos de crédito
			DocumentoCredito documentoCreditoRecibo2 = this.ObtenerDocumentoCredito(recibo2);
			Assert.IsNull(documentoCreditoRecibo2, "Documento crédito de recibo2 nulo");
		}
		
		[Test]
		public void Imputar_MultiplesDocumentosAmbos_DocumentoDebito_YaImputadoAntes()
		{
			#region PreEscenario - 2 documentos débito y 2 documentos crédito
			// Creo una factura
			Factura factura1 = TestBase.CrearFactura();
			factura1.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura1.Codigo = 1;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura
			Factura factura2 = TestBase.CrearFactura();
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 3));
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 7));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo un recibo
			Recibo recibo1 = TestBase.CrearRecibo();
			recibo1.Cliente = this.ClienteSinBonificacion;
			
			recibo1.Items.Clear();
			recibo1.Items.Add(new ItemRecibo(factura1.ImporteTotal));
			
			TestBase.ProcesarItemsRecibo(recibo1);
			
			recibo1.Concepto = "Concepto del recibo";
			recibo1.Create();
			
			// Creo un recibo
			Recibo recibo2 = TestBase.CrearRecibo();
			recibo2.Cliente = this.ClienteSinBonificacion;
			
			recibo2.Items.Clear();
			recibo2.Items.Add(new ItemRecibo(factura2.ImporteTotal - 100.0m));
			
			TestBase.ProcesarItemsRecibo(recibo2);
			
			recibo2.Concepto = "Concepto del recibo";
			recibo2.Create();
			
			this.RemoveTransaction();
			
			DocumentoCreditoDto[] documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			DocumentoDebitoDto[] documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			#endregion
			
			
			this.FlushAndRecreateScope();
			
			// En este punto tengo un documento débito con importe pendiente (factura2)
			// Creo un recibo
			Recibo recibo3 = TestBase.CrearRecibo();
			recibo3.Cliente = this.ClienteSinBonificacion;
			
			recibo3.Items.Clear();
			recibo3.Items.Add(new ItemRecibo(350.0m));
			
			TestBase.ProcesarItemsRecibo(recibo3);
			
			recibo3.Concepto = "Concepto del recibo";
			recibo3.Create();
			
			this.RemoveTransaction();
			
			// Me aseguro que haya la cantidad de documentos correspondiente
			// Obtengo los documentos de crédito y débito del cliente que estoy
			// probando.
			documentosCredito =
				this.controlador
					.ObtenerDocumentosCredito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosCredito.Length, "Cantidad documentos crédito");
			
			documentosDebito =
				this.controlador
					.ObtenerDocumentosDebito(this.ClienteSinBonificacion.Codigo);
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			try {
				this.controlador.Imputar(documentosCredito, documentosDebito);
			}
			catch (StratosException ex) {
				Assert.Fail("No se pudo imputar: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura2 = Factura.Find(factura2.Id);
			recibo3 = Recibo.Find(recibo3.Id);
			
			// Me aseguro que los documentos de crédito y débito
			// tengan los datos correctos.
			
			// Documento de débito
			DocumentoDebito documentoDebitoFactura2 = this.ObtenerDocumentoDebito(factura2);
			Assert.IsNull(documentoDebitoFactura2, "Documento débito de factura3 nulo");
			
			// Documentos de crédito
			DocumentoCredito documentoCreditoRecibo3 = this.ObtenerDocumentoCredito(recibo3);
			Assert.IsNotNull(documentoCreditoRecibo3, "Documento crédito de recibo3 nulo");
			Assert.AreEqual(recibo3.Importe, documentoCreditoRecibo3.ImporteTotal, "Documento crédito de recibo3 importe total");
			Assert.AreEqual(250.0m, documentoCreditoRecibo3.ImportePendiente, "Documento crédito de recibo3 importe pendiente");
		}
		#endregion
		#endregion
		
		#region Pasar facturas a cuenta corriente
		[Test]
		public void PasarFacturasACuentaCorriente_FacturaContado()
		{
			// Creo una factura al contado
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today;
			factura.CondicionVenta.Condicion = CondicionPago.Contado;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Me aseguro que no haya documentos de débito
			Assert.AreEqual(0, DocumentoDebito.FindAll().Length, "Cantidad documentos débito");
			
			factura = Factura.Find(factura.Id);
			
			this.RemoveTransaction();
			
			// Paso la factura a la cuenta corriente del cliente
			try {
				long[] idsFacturas = new long[] { factura.Id };
				
				this.controlador.PasarFacturasACuentaCorriente(idsFacturas);
			}
			catch (System.Exception ex) {
				Assert.Fail(ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			
			// Me aseguro que exista un documento de débito creado asociado
			// a la factura pasada.
			DocumentoDebito[] documentosDebito = DocumentoDebito.FindAll();
			
			Assert.AreEqual(1, documentosDebito.Length, "Cantidad documentos débito");
			
			DocumentoDebito docDebito = documentosDebito[0];
			
			Assert.AreEqual(factura.Id, docDebito.IdDocumento, "Documento débito - Id documento");
			Assert.AreEqual(factura.Cliente.Codigo, docDebito.Cliente.Codigo, "Documento débito - Cliente código");
			Assert.AreEqual(factura.ImporteTotal, docDebito.ImportePendiente, "Documento débito - Importe pendiente");
		}
		
		[Test]
		public void PasarFacturasACuentaCorriente_VariasFacturasContado()
		{
			// Creo una factura al contado
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today;
			factura.CondicionVenta.Condicion = CondicionPago.Contado;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Factura factura2 = TestBase.CrearFactura();
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Fecha = DateTime.Today;
			factura2.CondicionVenta.Condicion = CondicionPago.Contado;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Me aseguro que no haya documentos de débito
			Assert.AreEqual(0, DocumentoDebito.FindAll().Length, "Cantidad documentos débito");
			
			factura = Factura.Find(factura.Id);
			factura2 = Factura.Find(factura2.Id);
			
			this.RemoveTransaction();
			
			// Paso la factura a la cuenta corriente del cliente
			try {
				long[] idsFacturas = new long[] { factura.Id, factura2.Id };
				
				this.controlador.PasarFacturasACuentaCorriente(idsFacturas);
			}
			catch (System.Exception ex) {
				Assert.Fail(ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			factura2 = Factura.Find(factura2.Id);
			
			// Me aseguro que exista un documento de débito creado asociado
			// a la factura pasada.
			DocumentoDebito[] documentosDebito = DocumentoDebito.FindAll(Order.Asc("Id"));
			
			Assert.AreEqual(2, documentosDebito.Length, "Cantidad documentos débito");
			
			DocumentoDebito docDebito = documentosDebito.Where(docDeb => docDeb.IdDocumento == factura.Id).Single();
			DocumentoDebito docDebito2 = documentosDebito.Where(docDeb => docDeb.IdDocumento == factura2.Id).Single();
			
			Assert.AreEqual(factura.Id, docDebito.IdDocumento, "Documento débito - Id documento");
			Assert.AreEqual(factura.Cliente.Codigo, docDebito.Cliente.Codigo, "Documento débito - Cliente código");
			Assert.AreEqual(factura.ImporteTotal, docDebito.ImportePendiente, "Documento débito - Importe pendiente");
			
			Assert.AreEqual(factura2.Id, docDebito2.IdDocumento, "Documento débito - Id documento 2");
			Assert.AreEqual(factura2.Cliente.Codigo, docDebito2.Cliente.Codigo, "Documento débito - Cliente código 2");
			Assert.AreEqual(factura2.ImporteTotal, docDebito2.ImportePendiente, "Documento débito - Importe pendiente 2");
		}
		
		[Test]
		public void PasarFacturasACuentaCorriente_FacturaYaEstaEnCuentaCorriente()
		{
			// Creo una factura al contado
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today;
			factura.CondicionVenta.Condicion = CondicionPago.Contado;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Factura factura2 = TestBase.CrearFactura();
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Fecha = DateTime.Today;
			factura2.CondicionVenta.Condicion = CondicionPago.CuentaCorriente;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(factura.Id);
			factura2 = Factura.Find(factura2.Id);
			
			// Me aseguro que haya un documento de débito correspondiente a la factura2
			Assert.AreEqual(1, DocumentoDebito.FindAll().Length, "Cantidad documentos débito");
			Assert.AreEqual(factura2.Id, DocumentoDebito.FindAll()[0].IdDocumento, "Id documento");
			
			this.RemoveTransaction();
			
			// Paso la factura a la cuenta corriente del cliente
			try {
				long[] idsFacturas = new long[] { factura.Id, factura2.Id };
				
				this.controlador.PasarFacturasACuentaCorriente(idsFacturas);
				
				Assert.Fail("Se pudo pasar a cuenta corriente");
			}
			catch (ParametrosInvalidosException ex) {
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Me aseguro que no haya habido cambios en los documentos de débito
			Assert.AreEqual(1, DocumentoDebito.FindAll().Length, "Cantidad documentos débito");
			Assert.AreEqual(factura2.Id, DocumentoDebito.FindAll()[0].IdDocumento, "Id documento factura");
		}
		
		[Test]
		public void PasarFacturasACuentaCorriente_VariasFacturasYNotaDeCredito()
		{
			Factura factura = TestBase.CrearFactura();
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Fecha = DateTime.Today;
			factura.CondicionVenta.Condicion = CondicionPago.Contado;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura);
			factura.Create();
			
			this.FlushAndRecreateScope();
			
			Factura factura2 = TestBase.CrearFactura();
			factura2.Codigo = 1;
			factura2.Numero = 2;
			factura2.Fecha = DateTime.Today;
			factura2.CondicionVenta.Condicion = CondicionPago.Contado;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			this.FlushAndRecreateScope();
			
			NotaCredito nc = TestBase.CrearNotaCredito();
			nc.Codigo = 1;
			nc.Numero = 3;
			nc.Fecha = DateTime.Today;
			nc.CondicionVenta.Condicion = CondicionPago.Contado;
			nc.FacturaAsociada = factura2;
			nc.Cliente = this.ClienteSinBonificacion;
			nc.Items.Clear();
			nc.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(nc);
			nc.Create();
			
			this.RemoveTransaction();
			
			try {
				this.controlador.PasarFacturasACuentaCorriente(new long[]{
					factura.Id,
					factura2.Id,
					nc.Id
				});
				
				Assert.Fail("Se pudo pasar a cuenta corriente");
			}
			catch(ParametrosInvalidosException)
			{
			}
		}
		#endregion
		#endregion
		
		#region Recuperar cliente eliminado
		[Test]
		public void RecuperarClienteEliminado()
		{
			ClienteConBonificacion.Eliminado = true;
			ClienteSinBonificacion.SaveAndFlush();
			
			this.FlushAndRecreateScope();
			
			// Recupero el cliente
			ClienteDto clienteRecuperado = this.controlador.RecuperarClienteEliminado(ClienteConBonificacion.Codigo);
			
			this.FlushAndRecreateScope();
			
			// Obtengo el cliente recién recuperado desde la base de datos
			Cliente cliente = Cliente.FindAllByProperty("Codigo", clienteRecuperado.codigo)[0];
			
			Assert.AreEqual(cliente.Codigo, clienteRecuperado.codigo, "Código del cliente recuperado");
			Assert.IsFalse(cliente.Eliminado, "Propiedad Eliminado del cliente recien recuperado");
		}
		
		[Test]
		public void RecuperarClienteNoEliminado()
		{
			try {
				// Intento recuperar un cliente que no está eliminado
				ClienteDto clienteRecuperado = this.controlador.RecuperarClienteEliminado(ClienteConBonificacion.Codigo);
				
				Assert.Fail("No se lanzó ninguna excepción");
			}
			catch(StratosException)
			{}
		}
		
		[Test]
		public void RecuperarClienteNoExistente()
		{
			try {
				// Intento recuperar un cliente que no existe
				ClienteDto clienteRecuperado = this.controlador.RecuperarClienteEliminado(999999);
				
				Assert.Fail("No se lanzó ninguna excepción");
			}
			catch(StratosException)
			{
			}
		}
		
		#endregion
	}
}
