// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

using NUnit.Framework;
using Rhino.Mocks;

using StratosServer.Core;
using StratosServer.Controladores;
using Stratos.Servicios;
using Stratos.ModeloDto;
using StratosServer.Tests;

namespace StratosServer.Core.Tests
{
	[TestFixture()]
	public class ControladorTerminalesITests : StratosServerTestBase
	{
		#region Métodos inicializadores
//		[TestFixtureSetUp]
//		public override void FixtureInit ()
//		{
//			/* Con este método intento anular el inicio de una instancia de ServidorStratos
//			 * que se lleva a cabo en StratosServerTestBase.FixtureInit(). */
//		}
		#endregion
		
		#region Método AgregarControlador
		[Test]
		public void AgregarControladorTest_AgregarNuevo()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			controladorTerminales.AgregarControlador(this.controladorBase1);
			
			Assert.AreEqual(1, controladorTerminales.ControladoresDisponibles.Count,
			                "Count");
			Assert.IsTrue(controladorTerminales
			              	.ControladoresDisponibles.ContainsKey("IdentidadControlador"),
			              "Existe el controlador");
			
			Ice.ObjectPrx contr =
				controladorTerminales.ControladoresDisponibles["IdentidadControlador"];
			
			Assert.IsNotNull(contr, "Controlador no nulo");
			Assert.AreEqual("IdentidadControlador",
			                contr.ice_getIdentity().name, "Identidad controlador");
		}
		
		[Test]
		public void AgregarControladorTest_AgregarRepetido()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			controladorTerminales.AgregarControlador(this.controladorBase1);
			controladorTerminales.AgregarControlador(this.controladorBase2);
			
			Assert.AreEqual(2, controladorTerminales.ControladoresDisponibles.Count,
			                "Count");
			Assert.IsTrue(controladorTerminales
			              	.ControladoresDisponibles.ContainsKey("IdentidadControlador"),
			              "Existe controlador 1");
			Assert.IsTrue(controladorTerminales
			              	.ControladoresDisponibles.ContainsKey("OtraIdentidadControlador"),
			              "Existe controlador 2");
			
			Ice.ObjectPrx contr =
				controladorTerminales.ControladoresDisponibles["OtraIdentidadControlador"];
			
			Assert.IsNotNull(contr, "Controlador 2 no nulo");
			Assert.AreEqual("OtraIdentidadControlador",
			                contr.ice_getIdentity().name, "Identidad controlador 2");
		}
		#endregion
		
		#region Método ObtenerDireccion
		[Test]
		public void ObtenerDireccionRemotaTest()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			string iceConnection = "local address = 127.0.0.1:2384\n" +
				"remote address = 127.0.0.1:3161";
			
			Assert.AreEqual("127.0.0.1",
			                controladorTerminales.ObtenerDireccionRemota(iceConnection),
			                "Dirección remota");
		}
		#endregion
		
		#region Método GenerarIdentificadorTerminal
		[Test]
		public void GenerarIdentificadorTerminalTest()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			Assert.AreEqual("IdentificadorTerminal1 - 127.0.0.1",
			                controladorTerminales
			                	.GenerarIdentificadorTerminal(terminal1.identificador,
			                                              this.iceCurrentIp1Puerto1),
			                "Identificador generado");
		}
		#endregion
		
		#region Terminales conectadas al iniciar
		[Test]
		public void SinTerminalesConectadasTest()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			Assert.AreEqual(0, controladorTerminales.TerminalesConectadas.Count);
		}
		#endregion
		
		#region Método Conectar
		[Test]
		public void ConectarTest_UnaTerminal()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			
			string identificadorTerminal =
				controladorTerminales
					.GenerarIdentificadorTerminal(terminal1.identificador, this.iceCurrentIp1Puerto1);
			
			Assert.AreEqual(1, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
			Assert.AreEqual("IdentificadorTerminal1",
			                controladorTerminales.TerminalesConectadas[identificadorTerminal]
			                	.identificador,
			                "Identificador de terminal");
		}
		
		[Test]
		[ExpectedException(typeof(Stratos.Common.Excepciones.ConexionException))]
		public void ConectarTest_UnaTerminalSinIdentificador()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
		}
		
		[Test]
		public void ConectarTest_DosTerminalesDistintasIps()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			DatosTerminal terminal2 = new DatosTerminal {
				identificador= "IdentificadorTerminal2",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			controladorTerminales.Conectar(terminal2, this.iceCurrentIp2Puerto1);
			
			string identificadorTerminal2 =
				controladorTerminales
					.GenerarIdentificadorTerminal(terminal2.identificador, this.iceCurrentIp2Puerto1);
			
			Assert.AreEqual(2, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
			Assert.AreEqual("IdentificadorTerminal2",
			                controladorTerminales.TerminalesConectadas[identificadorTerminal2]
			                	.identificador,
			                "Identificador de terminal");
		}
		
		[Test]
		public void ConectarTest_DosTerminalesMismaIpsDistintosPuertos()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			DatosTerminal terminal2 = new DatosTerminal {
				identificador= "IdentificadorTerminal2",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			controladorTerminales.Conectar(terminal2, this.iceCurrentIp1Puerto2);
			
			string identificadorTerminal2 =
				controladorTerminales
					.GenerarIdentificadorTerminal(terminal2.identificador, this.iceCurrentIp1Puerto2);
			
			Assert.AreEqual(2, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
			Assert.AreEqual("IdentificadorTerminal2",
			                controladorTerminales.TerminalesConectadas[identificadorTerminal2]
			                	.identificador,
			                "Identificador de terminal");
		}
		
		[Test]
		[ExpectedException(typeof(Stratos.Common.Excepciones.ConexionException))]
		public void ConectarTest_SeConectaLaMismaTerminalDosVeces()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
		}
		#endregion
		
		#region Método Desconectar
		[Test]
		public void DesconectarTest_UnaTerminalConectada()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			
			// Desconecto la terminal
			controladorTerminales.Desconectar(terminal1, this.iceCurrentIp1Puerto1);
			
			Assert.AreEqual(0, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
		}
		
		[Test]
		public void DesconectarTest_DosTerminalesConectadas()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			DatosTerminal terminal2 = new DatosTerminal {
				identificador= "IdentificadorTerminal2",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			controladorTerminales.Conectar(terminal2, this.iceCurrentIp2Puerto1);
			
			// Desconecto una terminal
			controladorTerminales.Desconectar(terminal2, this.iceCurrentIp2Puerto1);
			
			Assert.AreEqual(1, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
		}
		
		[Test]
		[ExpectedException(typeof(Stratos.Common.Excepciones.ConexionException))]
		public void DesconectarTest_UnaTerminalNoConectada()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			// Desconecto la terminal
			controladorTerminales.Desconectar(terminal1, this.iceCurrentIp1Puerto1);
		}
		#endregion
		
		#region Método ActualizarDatosTerminal
		[Test]
		public void ActualizarDatosTerminalTest_DosTerminalesConectadas()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = null
			};
			
			DatosTerminal terminal2 = new DatosTerminal {
				identificador= "IdentificadorTerminal2",
				datosTermImpresora = null
			};
			
			controladorTerminales.Conectar(terminal1, this.iceCurrentIp1Puerto1);
			controladorTerminales.Conectar(terminal2, this.iceCurrentIp2Puerto1);
			
			string identificadorTerminal1 =
				controladorTerminales
					.GenerarIdentificadorTerminal(terminal1.identificador, this.iceCurrentIp1Puerto1);
			
			string identificadorTerminal2 =
				controladorTerminales
					.GenerarIdentificadorTerminal(terminal2.identificador, this.iceCurrentIp2Puerto1);
			
			// Actualizo los datos de una terminal
			DatosTerminal terminal1_nuevo = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = new DatosTerminalImpresora {
				}
			};
			terminal1_nuevo.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoA };
			
			controladorTerminales.ActualizarDatosTerminal(terminal1_nuevo,
			                                              this.iceCurrentIp1Puerto1);
			
			// Verifico
			Assert.AreEqual(2, controladorTerminales.TerminalesConectadas.Count,
			                "Cantidad terminales conectadas");
			
			Assert.AreEqual("IdentificadorTerminal1",
			                controladorTerminales.TerminalesConectadas[identificadorTerminal1]
			                	.identificador,
			                "Identificador de terminal 1");
			Assert.AreEqual(1,
			                controladorTerminales
			                 	.TerminalesConectadas[identificadorTerminal1]
			                 		.datosTermImpresora.tiposDocumento.Length,
			                 "Terminal 1 - Cantidad de tipos de documentos");
			
			Assert.AreEqual("IdentificadorTerminal2",
			                controladorTerminales.TerminalesConectadas[identificadorTerminal2]
			                	.identificador,
			                "Identificador de terminal 2");
			Assert.IsNull(controladorTerminales
			              	.TerminalesConectadas[identificadorTerminal2]
			              		.datosTermImpresora,
			              "Terminal 2 - datosTermImpresora nulo");
		}
		
		[Test]
		[ExpectedException(typeof(Stratos.Common.Excepciones.ConexionException))]
		public void ActualizarDatosTerminalTest_UnaTerminalNoConectada()
		{
			ControladorTerminalesI controladorTerminales = ControladorTerminalesI.Instancia;
			
			DatosTerminal terminal1_nuevo = new DatosTerminal {
				identificador= "",
				datosTermImpresora = new DatosTerminalImpresora {
				}
			};
			terminal1_nuevo.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoA };
			
			controladorTerminales.ActualizarDatosTerminal(terminal1_nuevo,
			                                              this.iceCurrentIp1Puerto1);
		}
		#endregion
	}
}
