// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;

using NUnit.Framework;
using Castle.ActiveRecord;

using Stratos.Tests;
using StratosServer.Tests;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;
using StratosServer.Controladores;

namespace StratosServer.Reportes.Tests
{
	[TestFixture()]
	public class ControladorReportesITestsCompras : StratosServerDocumentoGenericoTestBase
	{
		protected ControladorReportesI controlador;
		
		protected FacturaCompra factura1;
		protected FacturaCompra factura2;
		protected FacturaCompra factura3;
		protected FacturaCompra factura4;
		protected FacturaCompra factura5;
		protected FacturaCompra factura6;
		protected FacturaCompra factura7;
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			string directorioActual =
				System.IO.Directory.GetCurrentDirectory() + System.IO.Path.DirectorySeparatorChar;
			
			this.controlador =
				new ControladorReportesI(directorioActual + "templateReporteIvaVentas.vm",
				                         directorioActual + "templateReporteIvaCompras.vm");
			
			// Creo facturas y las agrego a la base de datos
			FacturaCompra factura;
			
			// ### Facturas fuera de fecha ###
			
			this.factura1 = CrearFacturaCompra();
			this.factura1.FechaImputacion = DateTime.Today - TimeSpan.FromDays(29);
			this.factura1.Fecha = this.factura1.FechaImputacion;
			this.factura1.Proveedor = this.ProveedorFerrum;
			this.factura1.Codigo = 1;
			this.factura1.Numero = 1;
			this.factura1.Items.Clear();
			this.factura1.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura1.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 7));
			this.factura1.ImporteGravado = 167.12m;
			this.factura1.ImporteExento = 33.12m;
			this.factura1.ImporteIva = 12.12m;
			this.factura1.ImpuestoCombustible = 11.22m;
			this.factura1.PercepcionIB = 1.22m;
			this.factura1.PercepcionIva = 4.21m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura1, false);
			this.factura1.Create();
			
			this.factura2 = CrearFacturaCompra();
			this.factura2.FechaImputacion = DateTime.Today - TimeSpan.FromDays(20);
			this.factura2.Fecha = this.factura2.FechaImputacion;
			this.factura2.Proveedor = this.ProveedorFV;
			this.factura2.Codigo = 2;
			this.factura2.Numero = 1;
			this.factura2.Items.Clear();
			this.factura2.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura2.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 17));
			this.factura2.ImporteGravado = 367.32m;
			this.factura2.ImporteExento = 11.12m;
			this.factura2.ImporteIva = 53.33m;
			this.factura2.ImpuestoCombustible = 4.98m;
			this.factura2.PercepcionIB = 0.00m;
			this.factura2.PercepcionIva = 0.00m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura2, false);
			this.factura2.Create();
			
			this.factura3 = CrearFacturaCompra();
			this.factura3.FechaImputacion = DateTime.Today - TimeSpan.FromDays(20);
			this.factura3.Fecha = this.factura3.FechaImputacion;
			this.factura3.Proveedor = this.ProveedorFV;
			this.factura3.Codigo = 2;
			this.factura3.Numero = 2;
			this.factura3.Items.Clear();
			this.factura3.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura3.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 17));
			this.factura3.ImporteGravado = 1467.77m;
			this.factura3.ImporteExento = 0.00m;
			this.factura3.ImporteIva = 2.12m;
			this.factura3.ImpuestoCombustible = 0.00m;
			this.factura3.PercepcionIB = 0.66m;
			this.factura3.PercepcionIva = 44.10m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura3, false);
			this.factura3.Create();
			
			this.factura4 = CrearFacturaCompra();
			this.factura4.FechaImputacion = DateTime.Today - TimeSpan.FromDays(13);
			this.factura4.Fecha = this.factura4.FechaImputacion;
			this.factura4.Proveedor = this.ProveedorFerrum;
			this.factura4.Codigo = 1;
			this.factura4.Numero = 2;
			this.factura4.Items.Clear();
			this.factura4.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura4.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 1));
			this.factura4.ImporteGravado = 97.33m;
			this.factura4.ImporteExento = 23.22m;
			this.factura4.ImporteIva = 17.13m;
			this.factura4.ImpuestoCombustible = 0.00m;
			this.factura4.PercepcionIB = 1.22m;
			this.factura4.PercepcionIva = 4.21m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura4, false);
			this.factura4.Create();
			
			this.factura5 = CrearFacturaCompra();
			this.factura5.FechaImputacion = DateTime.Today - TimeSpan.FromDays(13);
			this.factura5.Fecha = this.factura5.FechaImputacion;
			this.factura5.Proveedor = this.ProveedorFerrum;
			this.factura5.Codigo = 1;
			this.factura5.Numero = 3;
			this.factura5.Items.Clear();
			this.factura5.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura5.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 1));
			this.factura5.ImporteGravado = 62881.32m;
			this.factura5.ImporteExento = 5523.38m;
			this.factura5.ImporteIva = 10333.55m;
			this.factura5.ImpuestoCombustible = 0.00m;
			this.factura5.PercepcionIB = 51.22m;
			this.factura5.PercepcionIva = 0.00m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura5, false);
			this.factura5.Create();
			
			this.factura6 = CrearFacturaCompra();
			this.factura6.FechaImputacion = DateTime.Today - TimeSpan.FromDays(10);
			this.factura6.Fecha = this.factura6.FechaImputacion;
			this.factura6.Proveedor = this.ProveedorFV;
			this.factura6.Codigo = 2;
			this.factura6.Numero = 3;
			this.factura6.Items.Clear();
			this.factura6.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura6.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 17));
			this.factura6.ImporteGravado = 781.22m;
			this.factura6.ImporteExento = 125.12m;
			this.factura6.ImporteIva = 187.32m;
			this.factura6.ImpuestoCombustible = 51.82m;
			this.factura6.PercepcionIB = 0.00m;
			this.factura6.PercepcionIva = 0.00m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura6, false);
			this.factura6.Create();
			
			this.factura7 = CrearFacturaCompra();
			this.factura7.FechaImputacion = DateTime.Today - TimeSpan.FromDays(5);
			this.factura7.Fecha = this.factura7.FechaImputacion;
			this.factura7.Proveedor = this.ProveedorFV;
			this.factura7.Codigo = 2;
			this.factura7.Numero = 4;
			this.factura7.Items.Clear();
			this.factura7.Items.Add(CrearItemFacturaCompra(this.ProductoGriferia, 5));
			this.factura7.Items.Add(CrearItemFacturaCompra(this.ProductoMosaico, 17));
			this.factura7.ImporteGravado = 367.32m;
			this.factura7.ImporteExento = 11.12m;
			this.factura7.ImporteIva = 2.12m;
			this.factura7.ImpuestoCombustible = 0.00m;
			this.factura7.PercepcionIB = 1.22m;
			this.factura7.PercepcionIva = 0.00m;
			TestBase.ProcesarItemsYCalcularImportesFacturaCompra(this.factura7, false);
			this.factura7.Create();
			
			this.FlushAndRecreateScope();
		}
		
		[Test()]
		public void ConsultarFacturasCompra ()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-20);
			DateTime fechaFinal = DateTime.Today.AddDays(-10);
			
			this.RemoveTransaction();
			
			FacturaCompra[] facturasRecuperadas =
				this.controlador.ConsultarFacturasCompras(fechaInicial, fechaFinal);
			
			// Total de facturas recuperadas
			Assert.AreEqual(5, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			long[] idsFacturasCorrectas =
				new long[] {this.factura2.Id, this.factura3.Id, this.factura4.Id, this.factura5.Id, this.factura6.Id};
			
			// Las facturas deben venir ordenadas por fecha, por lo tanto chequeo que
			// estén en el orden correcto. Veo el identificador de cada una
			facturasRecuperadas.All(facturaCompra => idsFacturasCorrectas.Contains(facturaCompra.Id));
		}
		
		[Test()]
		public void ProcesarFacturasComprasTest ()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-20);
			DateTime fechaFinal = DateTime.Today.AddDays(-10);
			
			this.RemoveTransaction();
			
			// Recupero las facturas
			FacturaCompra[] facturasRecuperadas =
				this.controlador.ConsultarFacturasCompras(fechaInicial, fechaFinal);
			
			Assert.AreEqual(5, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			decimal[] totales;
			
			// Proceso las facturas
			FacturaIvaComprasWrapper[] facturasProcesadas =
				controlador.ProcesarFacturasCompra(facturasRecuperadas, out totales);
			
			// Veo que la cantidad de facturas procesadas sea la misma que las recuperadas
			Assert.AreEqual(facturasRecuperadas.Length, facturasProcesadas.Length, "Cantidad de facturas procesadas");
			
			// Acá no hago tests para el calculo individual de cada factura, ya que eso es
			// responsabilidad de la clase FacturaIvaComprasWrapper
			
			Assert.AreEqual(7, totales.Length, "Cantidad de totales");
			
			// Testeo que los totales sean los correctos
			Assert.AreEqual(65594.96m, StratosUtil.RoundDecimal(totales[0], 2), "Total Gravado");
			Assert.AreEqual(5682.84m, StratosUtil.RoundDecimal(totales[1], 2), "Total Exento");
			Assert.AreEqual(10593.45m, StratosUtil.RoundDecimal(totales[2], 2), "Total IVA");
			Assert.AreEqual(53.10m, StratosUtil.RoundDecimal(totales[3], 2), "Total Perc IB");
			Assert.AreEqual(48.31m, StratosUtil.RoundDecimal(totales[4], 2), "Total Perc IVA");
			Assert.AreEqual(56.80m, StratosUtil.RoundDecimal(totales[5], 2), "Total Perc Comb");
			Assert.AreEqual(82029.46m, StratosUtil.RoundDecimal(totales[6], 2), "Total Total");
		}
		
		[Test()]
		public void MarcarComoHistoricaTest()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-20);
			DateTime fechaFinal = DateTime.Today.AddDays(-10);
			
			this.RemoveTransaction();
			
			this.controlador.MarcarComoHistoricaFacturaCompra(fechaInicial.Ticks, fechaFinal.Ticks);
			
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaCompra[] facturas = this.controlador.ConsultarFacturasCompras(fechaInicial, fechaFinal);
			
			Assert.AreEqual(5, facturas.Length, "Cantidad de facturas");
			
			foreach (FacturaCompra f in facturas)
				Assert.IsTrue(f.Historica);
		}
		
		[Test()]
		[Ignore("Este método ya se testea en las pruebas de reportes de ventas")]
		public void ValidarFechasTest_FechasInvalidas()
		{
		}
		
		[Test()]
		[Ignore("Este método ya se testea en las pruebas de reportes de ventas")]
		public void ValidarFechasTest_FechasValidas()
		{
		}
		
		[Test]
		[Ignore("Por alguna razón NVelocity no toma el path del archivo template. Por ahora ignoro este test")]
		public void GenerarIvaComprasTest()
		{
			this.RemoveTransaction();
			
			DateTime fechaInicial = DateTime.Today.AddDays(-20);
			DateTime fechaFinal = DateTime.Today.AddDays(-10);
			
			string ivaCompras = null;
			
			try {
				ivaCompras =
					this.controlador
						.GenerarReporteIvaCompras(fechaInicial.Ticks,
						                          fechaFinal.Ticks,
						                          Environment.OSVersion.VersionString);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción StratosException: " + ex.mensaje);
			}
			catch (Ice.Exception ex) {
				Assert.Fail("Excepción Ice: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Me aseguro que el string no sea vacio
			Assert.IsNotNull(ivaCompras, "Iva compras no nulo");
			Assert.AreNotEqual(String.Empty, ivaCompras, "Iva compras no vacio");
			
			// Me aseguro que en el string generado estén los datos de las facturas
			
			// Levanto las facturas procesadas que tienen que estar en el iva compras
			FacturaCompra[] facturasProcesadas =
				this.controlador.ConsultarFacturasCompras(fechaInicial, fechaFinal);
			
			var facturasWrapper =
				from factura in facturasProcesadas
				select new FacturaIvaComprasWrapper(factura);
			
			Assert.AreEqual(5, facturasWrapper.Count(), "Cantidad facturas wrapper");
			
			foreach (FacturaIvaComprasWrapper fac in facturasWrapper) {
				Assert.That(ivaCompras.Contains(fac.Fecha), "Factura " + fac.ResumenFactura + " - Fecha");
				Assert.That(ivaCompras.Contains(fac.ResumenFactura), "Factura " + fac.ResumenFactura + " - Resumen");
				Assert.That(ivaCompras.Contains(fac.RazonSocial), "Factura " + fac.ResumenFactura + " - Razon social");
				Assert.That(ivaCompras.Contains(fac.Cuit), "Factura " + fac.ResumenFactura + " - Cuit");
				Assert.That(ivaCompras.Contains(((int)fac.ImporteGravado).ToString()), "Factura " + fac.ResumenFactura + " - Importe gravado");
			}
		}
	}
}
