// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using NUnit.Framework;
using Castle.ActiveRecord;

using Stratos.Tests;
using StratosServer.Tests;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;
using Stratos.Common;
using Stratos.Common.Excepciones;

namespace StratosServer.Reportes.Tests
{
	[TestFixture()]
	public class ControladorReportesITestsVentas : StratosServerDocumentoGenericoTestBase
	{
		protected ControladorReportesI controlador;
		
		[SetUp]
		public override void TestInit()
		{
			this.controlador = new ControladorReportesI();
			
			base.TestInit();
			
			// Creo facturas y las agrego a la base de datos
			Factura factura;
			
			// ### Facturas fuera de fecha ###
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-18);
			factura.Tipo = FacturaTipoVenta.FacturaA;
			factura.Codigo = 4;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today;
			factura.Tipo = FacturaTipoVenta.FacturaA;
			factura.Codigo = 1;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			
			// ### Facturas dentro de la fecha, válidas ###
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-17);
			factura.Tipo = FacturaTipoVenta.FacturaA;
			factura.Codigo = 1;
			factura.Numero = 2;
			factura.Cliente = this.ClienteSinBonificacion;
			
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			// Nota de crédito asociada a la factura anterior
			NotaCredito nc = TestBase.CrearNotaCredito();
			nc.Fecha = DateTime.Today.AddDays(-16);
			nc.Tipo = factura.Tipo;
			nc.Codigo = factura.Codigo;
			nc.Numero = factura.Numero + 1;
			nc.Cliente = factura.Cliente;
			nc.FacturaAsociada = factura;
			
			nc.Items.Clear();
			nc.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(nc);
			
			try {
				nc.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			// Anulada
			factura = TestBase.CrearFactura();
			factura.Anulada = true;
			factura.Fecha = DateTime.Today.AddDays(-10);
			factura.Tipo = FacturaTipoVenta.FacturaA;
			factura.Codigo = 1;
			factura.Numero = 4;
			factura.Cliente = this.ClienteSinBonificacion;
			
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-5);
			factura.Tipo = FacturaTipoVenta.FacturaB;
			factura.Codigo = 9;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;			
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 2));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 125.3m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			
			// ### Facturas para octubre de 2008, inválidas
			factura = TestBase.CrearFactura();
			factura.Valida = false;
			factura.Fecha = DateTime.Today.AddDays(-3);
			factura.Tipo = FacturaTipoVenta.FacturaA;
			factura.Codigo = 4;
			factura.Numero = 2;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			this.FlushAndRecreateScope();
		}
		
		[Test()]
		public void ConsultarFacturasVentaTest ()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-17);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			Factura[] facturasRecuperadas =
				this.controlador.ConsultarFacturasVentas(fechaInicial, fechaFinal);
			
			// Total de facturas recuperadas
			Assert.AreEqual(4, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			// Las facturas deben venir ordenadas por fecha, por lo tanto chequeo que
			// estén en el orden correcto. Veo el identificador de cada una
			
			Assert.AreEqual(FacturaTipoVenta.FacturaA, facturasRecuperadas[0].Tipo, "Tipo primer factura");
			Assert.AreEqual(1, facturasRecuperadas[0].Codigo, "Código primer factura");
			Assert.AreEqual(2, facturasRecuperadas[0].Numero, "Numero primer factura");
			
			// Nota de crédito
			Assert.AreEqual(FacturaTipoVenta.FacturaA, facturasRecuperadas[1].Tipo, "Tipo segunda factura");
			Assert.AreEqual(1, facturasRecuperadas[1].Codigo, "Código segunda factura");
			Assert.AreEqual(3, facturasRecuperadas[1].Numero, "Numero segunda factura");
			Assert.IsTrue((facturasRecuperadas[1] is NotaCredito), "Nota de credito");
			
			// La factura Anulada también tiene que estar incluida
			Assert.AreEqual(FacturaTipoVenta.FacturaA, facturasRecuperadas[2].Tipo, "Tipo tercer factura");
			Assert.AreEqual(1, facturasRecuperadas[2].Codigo, "Código tercer factura");
			Assert.AreEqual(4, facturasRecuperadas[2].Numero, "Numero tercer factura");
			Assert.IsTrue(facturasRecuperadas[2].Anulada, "Anulada segunda factura");
			
			Assert.AreEqual(FacturaTipoVenta.FacturaB, facturasRecuperadas[3].Tipo, "Tipo cuarta factura");
			Assert.AreEqual(9, facturasRecuperadas[3].Codigo, "Código cuarta factura");
			Assert.AreEqual(1, facturasRecuperadas[3].Numero, "Numero cuarta factura");
		}
		
		[Test()]
		public void ProcesarFacturasVentasTest ()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-17);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			// Recupero las facturas
			Factura[] facturasRecuperadas =
				this.controlador.ConsultarFacturasVentas(fechaInicial, fechaFinal);
			
			Assert.AreEqual(4, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			decimal[] totales;
			
			// Proceso las facturas
			FacturaIvaVentasWrapper[] facturasProcesadas =
				controlador.ProcesarFacturasVenta(facturasRecuperadas, out totales);
			
			// Veo que la cantidad de facturas procesadas sea la misma que las recuperadas
			Assert.AreEqual(facturasRecuperadas.Length, facturasProcesadas.Length, "Cantidad de facturas procesadas");
			
			// Acá no hago tests para el calculo individual de cada factura, ya que eso es
			// responsabilidad de la clase FacturaIvaVentasWrapper
			
			// Testeo que los totales sean los correctos
			Assert.AreEqual(0.00m, StratosUtil.RoundDecimal(totales[0], 2), "Total Exento");
			Assert.AreEqual(2788.58m, StratosUtil.RoundDecimal(totales[1], 2), "Total Gravado");
			Assert.AreEqual(585.60m, StratosUtil.RoundDecimal(totales[2], 2), "Total Iva Insc");
			Assert.AreEqual(3374.18m, StratosUtil.RoundDecimal(totales[3], 2), "Total Total");
		}
		
		[Test()]
		public void RealizarConsultaResumenIvaVentasTest ()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-17);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			Factura[] facturasRecuperadas =
				this.controlador.RealizarConsultaResumenIvaVentas(fechaInicial, fechaFinal);
			
			// Total de facturas recuperadas. La factura anulada no debe levantarse.
			Assert.AreEqual(3, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			Assert.AreEqual(FacturaTipoVenta.FacturaA, facturasRecuperadas[0].Tipo, "Tipo primer factura");
			Assert.AreEqual(1, facturasRecuperadas[0].Codigo, "Código primer factura");
			Assert.AreEqual(2, facturasRecuperadas[0].Numero, "Numero primer factura");
			
			Assert.AreEqual(FacturaTipoVenta.FacturaA, facturasRecuperadas[1].Tipo, "Tipo segunda factura");
			Assert.AreEqual(1, facturasRecuperadas[1].Codigo, "Código segunda factura");
			Assert.AreEqual(3, facturasRecuperadas[1].Numero, "Numero segunda factura");
			Assert.IsTrue((facturasRecuperadas[1] is NotaCredito), "Nota de credito");
			
			Assert.AreEqual(FacturaTipoVenta.FacturaB, facturasRecuperadas[2].Tipo, "Tipo cuarta factura");
			Assert.AreEqual(9, facturasRecuperadas[2].Codigo, "Código cuarta factura");
			Assert.AreEqual(1, facturasRecuperadas[2].Numero, "Numero cuarta factura");
		}
		
		
		[Test()]
		public void ProcesarResumenIvaVentasTest ()
		{
			// Para el resumen IVA Ventas creo otras facturas más aparte
			
			Factura factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-2);
			factura.Tipo = FacturaTipoVenta.FacturaB;
			factura.Codigo = 1;
			factura.Numero = 5;
			factura.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1.2m));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 2.57m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex), "Primer factura");
			}
			
			this.FlushAndRecreateScope();
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-2);
			factura.Tipo = FacturaTipoVenta.FacturaA;
			// Coloco un porcentaje de bonificación
			factura.PorcentajeBonificacion = 10.0m;
			factura.Codigo = 4;
			factura.Numero = 3;
			factura.Cliente = this.ClienteSinBonificacion;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 13));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex), "Primer factura");
			}
			
			this.FlushAndRecreateScope();
			
			factura = TestBase.CrearFactura();
			factura.Fecha = DateTime.Today.AddDays(-2);
			factura.Tipo = FacturaTipoVenta.FacturaB;
			// Coloco un porcentaje de bonificación
			factura.PorcentajeBonificacion = 19.0m;
			factura.Codigo = 4;
			factura.Numero = 1;
			factura.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;
			factura.Items.Clear();
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 5));
			factura.Items.Add(TestBase.CrearItemFactura(this.ProductoMosaico, 7));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(factura);
			try {
				factura.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex), "Segunda factura");
			}
			
			DateTime fechaInicial = DateTime.Today.AddDays(-17);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			// Recupero las facturas
			Factura[] facturasRecuperadas =
				this.controlador.RealizarConsultaResumenIvaVentas(fechaInicial, fechaFinal);
			
			Assert.AreEqual(6, facturasRecuperadas.Length, "Cantidad de facturas recuperadas");
			
			decimal[] totales;
			
			this.RemoveTransaction();
			
			// Proceso las facturas
			ResumenSucursal[] resumenSucursales =
				controlador.ProcesarResumenIvaVentas(facturasRecuperadas, out totales);
			
			// Veo que la cantidad de facturas procesadas sea la misma que las recuperadas
			Assert.AreEqual(3, resumenSucursales.Length, "Cantidad resumenes de sucursales procesadas");
			
			/* Acá no hago tests para el calculo individual de cada tipo de facturas porque es
			 * responsabilidad de la clase ResumenFacturaTipo. Tampoco hago tests para los calculos
			 * referidos al resumen de una sucursal ya que esto es responsabilidad de la clase
			 * ResumenSucursal. Si hago chequeo los totales globables y chequeo que se hayan repartido
			 * bien las facturas correspondientes a cada sucursal */
			
			// Chequeo los totales de los distintos resumenes de sucursales
			
			// Sucursal Avellaneda: Hay una factura y una nota de crédito cuyo saldo final
			// debería ser 0.00, todo esto en el rubro comercio.
			Assert.AreEqual(72.00m + 54.00m + 243.0m, resumenSucursales[0].TotalComercio, "Avellaneda - Total comercio");
			Assert.AreEqual(54.74m + 249.21m + 120.75m, resumenSucursales[0].TotalIndustria, "Avellaneda - Total industria"); // La suma de la segunda factura daría 149.10, pero hay error de redondeo
			Assert.AreEqual(0.00m, resumenSucursales[0].TotalNoDefinido, "Avellaneda - Total no definido");
			Assert.AreEqual(166.67m, resumenSucursales[0].TotalIva, "Avellaneda - Total iva");
			Assert.AreEqual(960.37m, resumenSucursales[0].TotalTotal, "Avellaneda - Total total");
			
			// Sucursal Transporte: no hay facturas
			Assert.AreEqual(0.00m, resumenSucursales[1].TotalComercio, "Transporte - Total comercio");
			Assert.AreEqual(0.00m, resumenSucursales[1].TotalIndustria, "Transporte - Total industria");
			Assert.AreEqual(0.00m, resumenSucursales[1].TotalNoDefinido, "Transporte - Total no definido");
			Assert.AreEqual(0.00m, resumenSucursales[1].TotalIva, "Transporte - Total iva");
			Assert.AreEqual(0.00m, resumenSucursales[1].TotalTotal, "Transporte - Total total");
			
			// Sucursal Malabrigo: una factura con un articulo de comercio y otro de industria
			Assert.AreEqual(120.00m, resumenSucursales[2].TotalComercio, "Malabrigo - Total comercio");
			Assert.AreEqual(2668.58m, resumenSucursales[2].TotalIndustria, "Malabrigo - Total industria");
			Assert.AreEqual(0.00m, resumenSucursales[2].TotalNoDefinido, "Malabrigo - Total no definido");
			Assert.AreEqual(585.60m, resumenSucursales[2].TotalIva, "Malabrigo - Total iva");
			Assert.AreEqual(3374.18m, resumenSucursales[2].TotalTotal, "Malabrigo - Total total");
			
			// Testeo que los totales globales sean los correctos
			Assert.AreEqual(369.00m + 120.00m, totales[0], "Global - Total comercio");
			Assert.AreEqual(424.70m + 2668.58m, totales[1], "Global - Total industria");
			Assert.AreEqual(0.00m, totales[2], "Global - Total no definido");
			Assert.AreEqual(166.67m + 585.60m, totales[3], "Global - Total iva");
			Assert.AreEqual(960.37m + 3374.18m, totales[4], "Global - Total total");
		}
		
		[Test()]
		public void MarcarComoHistoricaTest()
		{
			DateTime fechaInicial = DateTime.Today.AddDays(-17);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			this.controlador.MarcarComoHistoricaFacturaVenta(fechaInicial.Ticks, fechaFinal.Ticks);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] facturas = this.controlador.ConsultarFacturasVentas(fechaInicial, fechaFinal);
			
			Assert.AreEqual(4, facturas.Length, "Cantidad de facturas");
			
			foreach (Factura f in facturas)
				Assert.IsTrue(f.Historica);
		}
		
		[Test()]
		public void ValidarFechasTest_FechasInvalidas()
		{
			// Fecha final posterior a fecha inicial
			bool excepcionLanzada = false;
			
			DateTime fechaIncial = DateTime.Today.AddDays(-5);
			DateTime fechaFinal = DateTime.Today.AddDays(-6);
			
			this.RemoveTransaction();
			
			try {
				this.controlador.ValidarFechas(fechaIncial, fechaFinal);
			} catch (ObjetoInvalidoException ex) {
				excepcionLanzada = true;
			}
			
			Assert.IsTrue(excepcionLanzada, "Fecha inicial mayor que fecha final");
			
			// Fecha final mayor a fecha actual
			excepcionLanzada = false;
			
			fechaFinal = DateTime.Today.AddDays(2);
			
			this.RemoveTransaction();
			
			try {
				this.controlador.ValidarFechas(fechaIncial, fechaFinal);
			} catch (ObjetoInvalidoException ex) {
				excepcionLanzada = true;
			}
			
			Assert.IsTrue(excepcionLanzada, "Fecha final mayor a fecha actual");
			
			// Rango de fechas posterior a la fecha actual
			excepcionLanzada = false;
			
			fechaIncial = DateTime.Today.AddDays(1);
			fechaFinal = DateTime.Today.AddDays(6);
			
			this.RemoveTransaction();
			
			try {
				this.controlador.ValidarFechas(fechaIncial, fechaFinal);
			} catch (ObjetoInvalidoException ex) {
				excepcionLanzada = true;
			}
			
			Assert.IsTrue(excepcionLanzada, "Rango de fechas posterior a la fecha actual");
		}
		
		[Test()]
		public void ValidarFechasTest_FechasValidas()
		{
			DateTime fechaIncial = DateTime.Today.AddDays(-30);
			DateTime fechaFinal = DateTime.Today.AddDays(-2);
			
			this.RemoveTransaction();
			
			try {
				this.controlador.ValidarFechas(fechaIncial, fechaFinal);
			} catch (ObjetoInvalidoException ex) {
				Assert.Fail("Rango de fechas valido, invalidado");
			}
		}
	}
}
