// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using NUnit.Framework;

using Stratos.Tests;
using Stratos.ModeloObjetos;
using StratosServer.Reportes;
using Stratos.ModeloDto;
using Stratos.Common;
using StratosServer.Tests;

namespace StratosServer.Reportes.Tests
{
	[TestFixture()]
	public class ResumenFacturaTipoTests : StratosServerDocumentoGenericoTestBase
	{
		protected Factura facturaA_comercio;
		protected Factura facturaA_industria;
		protected Factura facturaB_comercio;
		protected Factura facturaB_industria;
		
		protected Factura notaCredito_comercio;
		
		protected Factura facturaA_varios_items;
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			// Creo facturas
			this.facturaA_comercio = TestBase.CrearFactura();
			this.facturaA_comercio.Tipo = FacturaTipoVenta.FacturaA;
			this.facturaA_comercio.Cliente = this.ClienteSinBonificacion;
			this.facturaA_comercio.Items.Clear();
			this.facturaA_comercio.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 2));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaA_comercio);
			
			
			this.facturaA_industria = TestBase.CrearFactura();
			this.facturaA_industria.Tipo = FacturaTipoVenta.FacturaA;
			this.facturaA_industria.Cliente = this.ClienteSinBonificacion;
			this.facturaA_industria.Items.Clear();
			this.facturaA_industria.Items.Add(TestBase.CrearItemFactura(this.productoMosaico, 125.3m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaA_industria);
			
			
			this.facturaB_comercio = TestBase.CrearFactura();
			this.facturaB_comercio.Tipo = FacturaTipoVenta.FacturaB;
			this.facturaB_comercio.Cliente = this.ClienteSinBonificacion;
			this.facturaB_comercio.Items.Clear();
			this.facturaB_comercio.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaB_comercio);
			
			
			this.facturaB_industria = TestBase.CrearFactura();
			this.facturaB_industria.Tipo = FacturaTipoVenta.FacturaB;
			this.facturaB_industria.Cliente = this.ClienteSinBonificacion;
			this.facturaB_industria.Items.Clear();
			this.facturaB_industria.Items.Add(TestBase.CrearItemFactura(this.productoMosaico, 125m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaB_industria);
			
			
			this.notaCredito_comercio = TestBase.CrearNotaCredito();
			this.notaCredito_comercio.Tipo = this.facturaA_comercio.Tipo;
			this.notaCredito_comercio.Cliente = this.facturaA_comercio.Cliente;
			this.notaCredito_comercio.Codigo = facturaA_comercio.Codigo;
			this.notaCredito_comercio.Numero = facturaA_comercio.Numero + 1;
			((NotaCredito) this.notaCredito_comercio).FacturaAsociada = this.facturaA_comercio;
			this.notaCredito_comercio.Items.Clear();
			this.notaCredito_comercio.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.notaCredito_comercio);
			
			
			this.facturaA_varios_items = TestBase.CrearFactura();
			this.facturaA_varios_items.Tipo = FacturaTipoVenta.FacturaA;
			this.facturaA_varios_items.Cliente = this.ClienteSinBonificacion;
			this.facturaA_varios_items.Items.Clear();
			this.facturaA_varios_items.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			this.facturaA_varios_items.Items.Add(TestBase.CrearItemFactura(this.productoMosaico, 125m));
			this.facturaA_varios_items.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaA_varios_items);
		}
		
		[Test()]
		public void FacturaA_Comercio ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("A");
			rf.Imputar(this.facturaA_comercio);
			
			Assert.AreEqual("A", rf.TipoFactura, "Tipo factura");
			Assert.AreEqual(120.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(0.00m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(25.20m, rf.Iva, "Iva");
			Assert.AreEqual(145.20m, rf.Total, "Total");
		}
		
		[Test()]
		public void FacturaA_Industria ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("A");
			rf.Imputar(this.facturaA_industria);
			
			Assert.AreEqual(0.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(2668.89m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(560.47m, rf.Iva, "Iva");
			Assert.AreEqual(3229.36m, rf.Total, "Total");
		}
		
		[Test()]
		public void FacturaB_Comercio ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("B");
			rf.Imputar(this.facturaB_comercio);
			
			Assert.AreEqual(72.60m, this.facturaB_comercio.Items[0].Importe, "Importe item");
			
			Assert.AreEqual("B", rf.TipoFactura, "Tipo factura");
			Assert.AreEqual(60.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(0.00m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(12.60m, rf.Iva, "Iva");
			Assert.AreEqual(72.60m, rf.Total, "Total");
		}
		
		[Test()]
		public void FacturaB_Industria ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("B");
			rf.Imputar(this.facturaB_industria);
			
			Assert.AreEqual(3221.25m, this.facturaB_industria.Items[0].Importe, "Importe item");
			Assert.AreEqual(3221.25m, this.facturaB_industria.ImporteTotal, "Importe total");
			
			Assert.AreEqual(0.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(2662.19m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(559.06m, rf.Iva, "Iva");
			Assert.AreEqual(3221.25m, rf.Total, "Total");
		}
		
		[Test()]
		public void FacturaA_NotaCreditoA_Comercio ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("A");
			
			// Imputo la factura
			rf.Imputar(this.facturaA_comercio);
			
			Assert.AreEqual(120.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(0.00m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(25.20m, rf.Iva, "Iva");
			Assert.AreEqual(145.20m, rf.Total, "Total");
			
			// Imputo la nota de crédito
			rf.Imputar(this.notaCredito_comercio);
			
			Assert.AreEqual(60.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(0.00m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(12.60m, rf.Iva, "Iva");
			Assert.AreEqual(72.60m, rf.Total, "Total");
		}
		
		[Test()]
		public void FacturaA_VariosItems ()
		{
			ResumenFacturaTipo rf = new ResumenFacturaTipo("A");
			rf.Imputar(this.facturaA_varios_items);
			
			Assert.AreEqual(120.00m, rf.RubroComercio, "Rubro comercio");
			Assert.AreEqual(2662.50m, rf.RubroIndustria, "Rubro industria");
			Assert.AreEqual(0.00m, rf.RubroNoDefinido, "Rubro no definido");
			Assert.AreEqual(584.33m, rf.Iva, "Iva");
			Assert.AreEqual(3366.83m, rf.Total, "Total");
		}
	}
}
