// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;

using NUnit.Framework;

using Stratos.Tests;
using Stratos.ModeloObjetos;
using StratosServer.Reportes;
using Stratos.ModeloDto;
using Stratos.Common;
using StratosServer.Tests;

namespace StratosServer.Reportes.Tests
{
	[TestFixture()]
	public class ResumenSucursalTests : StratosServerDocumentoGenericoTestBase
	{	
		protected Factura facturaA_comercio;
		protected Factura facturaA_industria;
		protected Factura facturaB_comercio;
		protected Factura facturaB_industria;
		
		protected Factura notaCredito_comercio;
		
		protected Factura facturaA_varios_items;
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			// Creo facturas
			this.facturaA_comercio = TestBase.CrearFactura();
			this.facturaA_comercio.Tipo = FacturaTipoVenta.FacturaA;
			this.facturaA_comercio.Cliente = this.ClienteSinBonificacion;
			this.facturaA_comercio.Items.Clear();
			this.facturaA_comercio.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 2));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaA_comercio);
			
			
			this.facturaA_industria = TestBase.CrearFactura();
			this.facturaA_industria.Tipo = FacturaTipoVenta.FacturaA;
			this.facturaA_industria.Cliente = this.ClienteSinBonificacion;
			this.facturaA_industria.Items.Clear();
			this.facturaA_industria.Items.Add(TestBase.CrearItemFactura(this.productoMosaico, 125.3m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaA_industria);
			
			
			this.facturaB_comercio = TestBase.CrearFactura();
			this.facturaB_comercio.Tipo = FacturaTipoVenta.FacturaB;
			this.facturaB_comercio.Cliente = this.ClienteSinBonificacion;
			this.facturaB_comercio.Items.Clear();
			this.facturaB_comercio.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaB_comercio);
			
			
			this.facturaB_industria = TestBase.CrearFactura();
			this.facturaB_industria.Tipo = FacturaTipoVenta.FacturaB;
			this.facturaB_industria.Cliente = this.ClienteSinBonificacion;
			this.facturaB_industria.Items.Clear();
			this.facturaB_industria.Items.Add(TestBase.CrearItemFactura(this.productoMosaico, 125m));
			FacturaIvaVentasWrapperTests.ProcesarItemsYCalcularImportesFactura(this.facturaB_industria);
		}
		
		[Test()]
		public void PropiedadesComunes ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.Descripcion = "Sucursal Avellaneda";
			
			Assert.AreEqual("Sucursal Avellaneda", rs.Descripcion, "Descripción");
			Assert.AreEqual(2, rs.ResumenesFacturas.Length, "Cantidad de resumenes de faturas");
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Tipo primer resumen factura");
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Tipo segundo resumen factura");
		}
		
		[Test()]
		public void FacturaA_Comercio ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaA_comercio);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(120.00m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(25.20m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(145.20m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(120.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(0.00m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(25.20m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(145.20m, rs.TotalTotal, "Total - Total");
		}
		
		[Test()]
		public void FacturaA_Industria ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaA_industria);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(00.0m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(2668.89m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(560.47m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(3229.36m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(0.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(2668.89m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(560.47m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(3229.36m, rs.TotalTotal, "Total - Total");
		}
		
		[Test()]
		public void FacturaB_Comercio ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaB_comercio);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(60.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(12.60m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(72.60m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(60.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(0.00m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(12.60m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(72.60m, rs.TotalTotal, "Total - Total");
		}
		
		[Test()]
		public void FacturaB_Industria ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaB_industria);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(2662.19m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(559.06m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(3221.25m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(0.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(2662.19m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(559.06m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(3221.25m, rs.TotalTotal, "Total - Total");
		}
		
		[Test()]
		public void FacturaA_FacturaB_Comercio ()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaA_comercio);
			rs.AgregarFactura(this.facturaB_comercio);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(120.00m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(25.20m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(145.20m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(60.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(12.60m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(72.60m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(180.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(0.00m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(37.80m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(217.80m, rs.TotalTotal, "Total - Total");
		}
		
		[Test()]
		public void FacturaA_FacturaB_Industria()
		{
			ResumenSucursal rs = new ResumenSucursal();
			rs.AgregarFactura(this.facturaA_industria);
			rs.AgregarFactura(this.facturaB_industria);
			
			// Chequeo que se hayan imputado bien las facturas A y las facturas B segun corresponda
			Assert.AreEqual("A", rs.ResumenesFacturas[0].TipoFactura, "Facturas A - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroComercio, "Facturas A - Rubro comercio");
			Assert.AreEqual(2668.89m, rs.ResumenesFacturas[0].RubroIndustria, "Facturas A - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[0].RubroNoDefinido, "Facturas A - Rubro no definido");
			Assert.AreEqual(560.47m, rs.ResumenesFacturas[0].Iva, "Facturas A - Iva");
			Assert.AreEqual(3229.36m, rs.ResumenesFacturas[0].Total, "Facturas A - Total");
			
			Assert.AreEqual("B", rs.ResumenesFacturas[1].TipoFactura, "Facturas B - Tipo factura");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroComercio, "Facturas B - Rubro comercio");
			Assert.AreEqual(2662.19m, rs.ResumenesFacturas[1].RubroIndustria, "Facturas B - Rubro industria");
			Assert.AreEqual(0.00m, rs.ResumenesFacturas[1].RubroNoDefinido, "Facturas B - Rubro no definido");
			Assert.AreEqual(559.06m, rs.ResumenesFacturas[1].Iva, "Facturas B - Iva");
			Assert.AreEqual(3221.25m, rs.ResumenesFacturas[1].Total, "Facturas B - Total");
			
			// Chequeo los totales calculados
			Assert.AreEqual(0.00m, rs.TotalComercio, "Total - Rubro comercio");
			Assert.AreEqual(5331.08m, rs.TotalIndustria, "Total - Rubro industria");
			Assert.AreEqual(0.00m, rs.TotalNoDefinido, "Total - Rubro no definido");
			Assert.AreEqual(1119.53m, rs.TotalIva, "Total - Iva");
			Assert.AreEqual(6450.61m, rs.TotalTotal, "Total - Total");
		}
	}
}
