// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

using NUnit.Framework;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Framework;
using NHibernate.Criterion;

using Stratos.Tests;
using Stratos.ModeloObjetos;
using Stratos.ModeloDto;

namespace StratosServer.Tests
{
	/// <summary>
	/// Esta clase brinda facilidades para escribir tests de clases que heredan
	/// de DocumentoGenérico, como Factura, PreFactura y Presupuesto.
	/// </summary>
	public abstract class StratosServerDocumentoGenericoTestBase : StratosServerTestBase
	{
		#region Campos
		protected Cliente clienteSinBonificacion;
		protected Cliente clienteSinBonificacion_ConsumidorFinal;
		protected Cliente clienteConBonificacion;
		
		protected Rubro rubroComercio;
		protected Rubro rubroIndustria;
		protected Rubro rubroMosaico;
		protected Rubro rubroGriferia;
		
		protected Producto productoMosaico;
		protected Producto productoGriferia;
		
		private Proveedor proveedorFerrum;
		private Proveedor proveedorFV;
		#endregion
		
		/// <summary>
		/// Cliente Monotributo
		/// </summary>
		public Cliente ClienteConBonificacion {
			get {
				return Cliente.Find(clienteConBonificacion.Codigo);
			}
		}
		
		/// <summary>
		/// Cliente Responsable Inscripto.
		/// </summary>
		public Cliente ClienteSinBonificacion {
			get {
				return Cliente.Find(clienteSinBonificacion.Codigo);
			}
		}

		public Cliente ClienteSinBonificacion_ConsumidorFinal {
			get {
				return Cliente.Find(clienteSinBonificacion_ConsumidorFinal.Codigo);
			}
		}

		public Producto ProductoGriferia {
			get {
				return Producto.Find(productoGriferia.Id);
			}
		}

		public Producto ProductoMosaico {
			get {
				return Producto.Find(productoMosaico.Id);
			}
		}
		
		public Rubro RubroPadreComercio {
			get {
				return Rubro.Find(this.rubroComercio.Codigo);
			}
		}

		public Rubro RubroPadreIndustria {
			get {
				return Rubro.Find(this.rubroIndustria.Codigo);
			}
		}
		
		public Rubro RubroGriferia {
			get {
				return Rubro.Find(rubroGriferia.Codigo);
			}
		}

		public Rubro RubroMosaico {
			get {
				return Rubro.Find(rubroMosaico.Codigo);
			}
		}
		
		public Proveedor ProveedorFerrum {
			get {
				return Proveedor.Find(this.proveedorFerrum.Codigo);
			}
		}
		
		public Proveedor ProveedorFV {
			get {
				return Proveedor.Find(this.proveedorFV.Codigo);
			}
		}
		
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			this.clienteConBonificacion = TestBase.CrearClienteMilton();
			this.clienteConBonificacion.PorcentajeBonificacion = 10m;
			this.clienteConBonificacion.Create();
			
			this.clienteSinBonificacion = TestBase.CrearClienteCesar();
			this.clienteSinBonificacion.PorcentajeBonificacion = 0.00m;
			this.clienteSinBonificacion.Create();
			
			this.clienteSinBonificacion_ConsumidorFinal = TestBase.CrearClienteCesar();
			this.clienteSinBonificacion_ConsumidorFinal.RazonSocial += " Consumidor Final";
			this.clienteSinBonificacion_ConsumidorFinal.Codigo += 145;
			this.clienteSinBonificacion_ConsumidorFinal.TipoResponsable = TipoResponsable.ConsumidorFinal;
			this.clienteSinBonificacion_ConsumidorFinal.Cuit = new CUIT();
			this.clienteSinBonificacion_ConsumidorFinal.PorcentajeBonificacion = 0.00m;
			this.clienteSinBonificacion_ConsumidorFinal.Create();
			
			// Rubro comercio
			this.rubroComercio = new Rubro {
				Codigo = 1,
				Nombre = "Comercio",
				Descripcion = "Comercio",
				RubroPadre = null
			};
			this.rubroComercio.CreateAndFlush();
			
			// Rubro industria
			this.rubroIndustria = new Rubro {
				Codigo = 2,
				Nombre = "Industria",
				Descripcion = "Industria",
				RubroPadre = null
			};
			this.rubroIndustria.CreateAndFlush();
			
			this.FlushAndRecreateScope();
			
			this.rubroGriferia = TestBase.CrearRubroGriferia();
			this.rubroGriferia.RubroPadre = this.RubroPadreComercio;
			this.rubroGriferia.Create();
			
			this.rubroMosaico = TestBase.CrearRubroMosaico();
			this.rubroMosaico.RubroPadre = this.RubroPadreIndustria;
			this.rubroMosaico.Create();
			
			this.productoGriferia = TestBase.CrearProductoGriferia();
			this.productoGriferia.SubRubro = this.RubroGriferia;
			this.productoGriferia.Create();
			
			this.productoMosaico = TestBase.CrearProductoMosaico();
			this.productoMosaico.SubRubro = this.RubroMosaico;
			this.productoMosaico.Create();
			
			this.proveedorFerrum = TestBase.CrearProveedorFerrum();
			this.proveedorFerrum.Create();
			
			this.proveedorFV = TestBase.CrearProveedorFV();
			this.proveedorFV.Create();
			
			this.FlushAndRecreateScope();
		}
	}
}
