// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

using NUnit.Framework;

using StratosServer.Tests;
using Stratos.Servicios;
using Stratos.ModeloDto;
using StratosServer.Ventas.Facturacion;

namespace StratosServer.Ventas.Facturacion.Tests
{
	[TestFixture]
	public class BuscadorTerminalesImpresorasTests : StratosServerTestBase
	{
		private Dictionary<string,DatosTerminal> terminalesImpresoras;
		private BuscadorTerminalesImpresoras buscador;
		
		#region Métodos inicializadores
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			this.terminalesImpresoras = new Dictionary<string, DatosTerminal>();
			this.buscador = new BuscadorTerminalesImpresoras(this.terminalesImpresoras);
		}
		#endregion
		
		#region Método BuscarTerminalesImpresora
		[Test]
		public void BuscarTerminalesImpresora_UnaTerminalImpresoraConectadaFacturaA()
		{
			// Ejecuto el método por testear
			this.terminalImpresora1.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoA };
			
			this.terminalesImpresoras.Add(this.terminalImpresora1.identificador,
			                              this.terminalImpresora1);
			
			List<DatosTerminal> terminalesImpresoras;
			
			// FacturaTipoA
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA);
			
			Assert.AreEqual(1,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoA");
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalesImpresoras[0].identificador,
			                "Identificador terminal conectada");
			
			// FacturaTipoB
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoB");
			
			// Presupuesto
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.Presupuesto);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - Presupuesto");
		}
		
		[Test]
		public void BuscarTerminalesImpresora_UnaTerminalImpresoraConectadaFacturaB()
		{
			// Ejecuto el método por testear
			this.terminalImpresora1.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoB };
			
			this.terminalesImpresoras.Add(this.terminalImpresora1.identificador,
			                              this.terminalImpresora1);
			
			List<DatosTerminal> terminalesImpresoras;
			
			// FacturaTipoA
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoA");
			
			// FacturaTipoB
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB);
			
			Assert.AreEqual(1,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoB");
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalesImpresoras[0].identificador,
			                "Identificador terminal conectada");
			
			// Presupuesto
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.Presupuesto);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - Presupuesto");
		}
		
		[Test]
		public void BuscarTerminalesImpresora_UnaTerminalImpresoraConectadaPresupuesto()
		{
			// Ejecuto el método por testear
			this.terminalImpresora1.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.Presupuesto };
			
			this.terminalesImpresoras.Add(this.terminalImpresora1.identificador,
			                              this.terminalImpresora1);
			
			List<DatosTerminal> terminalesImpresoras;
			
			// FacturaTipoA
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoA");
			
			// FacturaTipoB
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB);
			
			Assert.AreEqual(0,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoB");
			
			// Presupuesto
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.Presupuesto);
			
			Assert.AreEqual(1,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - Presupuesto");
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalesImpresoras[0].identificador,
			                "Identificador terminal conectada");
		}
		
		[Test]
		public void BuscarTerminalesImpresora_DosTerminalesImpresorasConectadas()
		{
			this.terminalImpresora1.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoA, TipoDocumento.FacturaTipoB };
			
			this.terminalImpresora2.datosTermImpresora.tiposDocumento =
				new TipoDocumento[] { TipoDocumento.FacturaTipoA, TipoDocumento.Presupuesto };
			
			this.terminalesImpresoras.Add(this.terminalImpresora1.identificador,
			                              this.terminalImpresora1);
			this.terminalesImpresoras.Add(this.terminalImpresora2.identificador,
			                              this.terminalImpresora2);
			
			List<DatosTerminal> terminalesImpresoras;
			
			// FacturaTipoA
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA);
			
			Assert.AreEqual(2,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoA");
			
			// FacturaTipoB
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB);
			
			Assert.AreEqual(1,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - FacturaTipoB");
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalesImpresoras[0].identificador,
			                "Identificador terminal conectada");
			
			// Presupuesto
			terminalesImpresoras =
				this.buscador.BuscarTerminalesImpresora(TipoDocumento.Presupuesto);
			
			Assert.AreEqual(1,
			                terminalesImpresoras.Count,
			                "Cantidad terminales impresoras - Presupuesto");
			Assert.AreEqual("IdentificadorTerminal2",
			                terminalesImpresoras[0].identificador,
			                "Identificador terminal conectada");
		}
		#endregion
	}
}
