// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

using NUnit.Framework;

using StratosServer.Tests;
using Stratos.Servicios;
using Stratos.ModeloDto;
using StratosServer.Ventas.Facturacion;

namespace StratosServer.Ventas.Facturacion.Tests
{
	[TestFixture]
	public class ConexionTerminalListenerTests : StratosServerTestBase
	{
		#region Campos
		private ConexionTerminalListener conexionListener;
		private IBuscadorTerminalesImpresoras buscadorImpresoras;
		#endregion
		
		#region Métodos de inicialización
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			// Seteo la fuente de terminales y creo una instancia del listener
			ConexionTerminalListener.TerminalesImpresoras =
				new Dictionary<string, DatosTerminal>();
			
			this.conexionListener = new ConexionTerminalListener();
		}
		#endregion
		
		[Test]
		public void SinTerminalesConectadas()
		{
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		#region Método TerminalConectada
		[Test]
		public void TerminalConectada_UnaTerminalImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(1,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
			
			DatosTerminal terminalConectada =
				ConexionTerminalListener
					.TerminalesImpresoras[this.terminalImpresora1.identificador];
			
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalConectada.identificador,
			                "Identificador terminal conectada");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.tiposDocumento.Length,
			                "Cantidad tipos documentos");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.servFacturacion.Count,
			                "Cantidad servicio facturacion");
		}
		
		[Test]
		public void TerminalConectada_UnaTerminalNoImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalNoImpresora1.identificador,
			                                        this.terminalNoImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		[Test]
		public void TerminalConectada_DosTerminalesImpresorasConectadas()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			this.conexionListener.TerminalConectada(this.terminalImpresora2.identificador,
			                                        this.terminalImpresora2);
			
			// Verifico los valores
			Assert.AreEqual(2,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
			
			DatosTerminal terminalConectada =
				ConexionTerminalListener
					.TerminalesImpresoras[this.terminalImpresora2.identificador];
			
			Assert.AreEqual("IdentificadorTerminal2",
			                terminalConectada.identificador,
			                "Identificador terminal conectada");
			Assert.AreEqual(2,
			                terminalConectada.datosTermImpresora.tiposDocumento.Length,
			                "Cantidad tipos documentos");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.servFacturacion.Count,
			                "Cantidad servicio facturacion");
			                
		}
		#endregion
		
		#region Método TerminalDesconectada
		[Test]
		public void TerminalDesconectada_UnaTerminalImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			this.conexionListener.TerminalDesconectada(this.terminalImpresora1.identificador,
			                                           this.terminalImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		[Test]
		public void TerminalDesconectada_UnaTerminalNoImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalNoImpresora1.identificador,
			                                        this.terminalNoImpresora1);
			this.conexionListener.TerminalDesconectada(this.terminalNoImpresora1.identificador,
			                                           this.terminalNoImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		[Test]
		public void TerminalDesconectada_UnaTerminalNoImpresoraDesconectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			this.conexionListener.TerminalDesconectada(this.terminalNoImpresora1.identificador,
			                                           this.terminalNoImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(1,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		[Test]
		public void TerminalDesconectada_DosTerminalesImpresorasConectadas()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			this.conexionListener.TerminalConectada(this.terminalImpresora2.identificador,
			                                        this.terminalImpresora2);
			
			this.conexionListener.TerminalDesconectada(this.terminalImpresora1.identificador,
			                                           this.terminalImpresora1);
			this.conexionListener.TerminalDesconectada(this.terminalImpresora2.identificador,
			                                           this.terminalImpresora2);
			
			// Verifico los valores
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		#endregion
		
		#region Método DatosTerminalActualizados
		[Test]
		public void DatosTerminalActualizados_UnaTerminalImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			
			DatosTerminal nuevosDatosTerminal = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = new DatosTerminalImpresora {
					tiposDocumento = new TipoDocumento[] {
						TipoDocumento.FacturaTipoA,
						TipoDocumento.FacturaTipoB,
						TipoDocumento.Presupuesto
					},
					servFacturacion = new List<ServicioFacturacionPrx>() {
						this.servicioFacturacion1
					}
				}
			};
			
			this.conexionListener.DatosTerminalActualizados(nuevosDatosTerminal.identificador,
			                                                nuevosDatosTerminal);
			
			// Verifico los valores
			Assert.AreEqual(1,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
			
			DatosTerminal terminalConectada =
				ConexionTerminalListener
					.TerminalesImpresoras[nuevosDatosTerminal.identificador];
			
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalConectada.identificador,
			                "Identificador terminal conectada");
			Assert.AreEqual(3,
			                terminalConectada.datosTermImpresora.tiposDocumento.Length,
			                "Cantidad tipos documentos");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.servFacturacion.Count,
			                "Cantidad servicio facturacion");
		}
		
		[Test]
		public void DatosTerminalActualizados_UnaTerminalNoImpresoraConectada()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalNoImpresora1.identificador,
			                                        this.terminalNoImpresora1);
			this.conexionListener
				.DatosTerminalActualizados(this.terminalNoImpresora1.identificador,
				                           this.terminalNoImpresora1);
			
			// Verifico los valores
			Assert.AreEqual(0,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
		}
		
		[Test]
		public void DatosTerminalActualizados_UnaTerminalNoImpresoraAhoraEsImpresora()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalNoImpresora1.identificador,
			                                        this.terminalNoImpresora1);
			
			DatosTerminal nuevosDatosTerminal = new DatosTerminal {
				identificador= "IdentificadorTerminalNoImpresora1",
				datosTermImpresora = new DatosTerminalImpresora {
					tiposDocumento = new TipoDocumento[] {
						TipoDocumento.FacturaTipoA,
						TipoDocumento.FacturaTipoB,
						TipoDocumento.Presupuesto
					},
					servFacturacion = new List<ServicioFacturacionPrx>() {
						this.servicioFacturacion1
					}
				}
			};
			
			this.conexionListener.DatosTerminalActualizados(nuevosDatosTerminal.identificador,
			                                                nuevosDatosTerminal);
			
			// Verifico los valores
			Assert.AreEqual(1,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
			
			DatosTerminal terminalConectada =
				ConexionTerminalListener
					.TerminalesImpresoras[nuevosDatosTerminal.identificador];
			
			Assert.AreEqual("IdentificadorTerminalNoImpresora1",
			                terminalConectada.identificador,
			                "Identificador terminal conectada");
			Assert.AreEqual(3,
			                terminalConectada.datosTermImpresora.tiposDocumento.Length,
			                "Cantidad tipos documentos");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.servFacturacion.Count,
			                "Cantidad servicio facturacion");
		}
		
		[Test]
		public void DatosTerminalActualizados_DosTerminalesImpresorasConectadas()
		{
			// Ejecuto el método por testear
			this.conexionListener.TerminalConectada(this.terminalImpresora1.identificador,
			                                        this.terminalImpresora1);
			this.conexionListener.TerminalConectada(this.terminalImpresora2.identificador,
			                                        this.terminalImpresora2);
			
			DatosTerminal nuevosDatosTerminal1 = new DatosTerminal {
				identificador= "IdentificadorTerminal1",
				datosTermImpresora = new DatosTerminalImpresora {
					tiposDocumento = new TipoDocumento[] {
						TipoDocumento.FacturaTipoA,
						TipoDocumento.FacturaTipoB,
						TipoDocumento.Presupuesto
					},
					servFacturacion = new List<ServicioFacturacionPrx>() {
						this.servicioFacturacion1
					}
				}
			};
			
			this.conexionListener
				.DatosTerminalActualizados(nuevosDatosTerminal1.identificador,
				                           nuevosDatosTerminal1);
			
			// Verifico los valores
			Assert.AreEqual(2,
			                ConexionTerminalListener.TerminalesImpresoras.Count,
			                "Cantidad terminales impresoras");
			
			DatosTerminal terminalConectada =
				ConexionTerminalListener
					.TerminalesImpresoras[nuevosDatosTerminal1.identificador];
			
			Assert.AreEqual("IdentificadorTerminal1",
			                terminalConectada.identificador,
			                "Identificador terminal conectada");
			Assert.AreEqual(3,
			                terminalConectada.datosTermImpresora.tiposDocumento.Length,
			                "Cantidad tipos documentos");
			Assert.AreEqual(1,
			                terminalConectada.datosTermImpresora.servFacturacion.Count,
			                "Cantidad servicio facturacion");
		}
		#endregion
	}
}
