// Copyright (C) 2009, 2011 Milton Pividori, César Sandrigo
// 
// This file is part of Stratos.
// 
// Stratos is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Stratos is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Stratos.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Threading;

using NUnit.Framework;
using NUnit.Framework.Constraints;
using Rhino.Mocks;
using NHibernate.Criterion;
using Castle.ActiveRecord;
using Castle.ActiveRecord.Queries;

using Stratos.Tests;
using StratosServer.Tests;
using Stratos.Servicios;
using Stratos.ModeloDto;
using Stratos.ModeloObjetos;
using StratosServer.Ventas.Facturacion;
using Stratos.Common.Excepciones;
using StratosServer.Core;
using Stratos.QueryObjects;

namespace StratosServer.Ventas.Facturacion.Tests
{
	[TestFixture]
	public class ControladorFacturacionITests : StratosServerDocumentoGenericoTestBase
	{
		#region Campos
		private ControladorFacturacionI controladorFacturacion;
		
		// Facturas
		private Factura factura1;
		private Factura factura2;
		private Factura factura3;
		
		private Factura facturaB1;
		
		// PreFacturas
		private PreFactura preFactura1;
		private PreFactura preFactura2;
		private PreFactura preFactura3;
		
		private PreFactura preFacturaB1;
		
		// NotaCredito
		private NotaCredito notaCredito1;
		
		// PreNotaCredito
		private PreNotaCredito preNotaCredito1;
		#endregion
		
		#region Métodos de inicialización
		[SetUp]
		public override void TestInit()
		{
			base.TestInit();
			
			BuscadorTerminalesImpresoras buscador =
				new BuscadorTerminalesImpresoras(new Dictionary<string,DatosTerminal>());
			
			ControladorTerminalesI controladorTerminales =
				ControladorTerminalesI.Instancia;
			
			this.controladorFacturacion =
				new ControladorFacturacionI(this.servidorStratos, buscador, controladorTerminales);
			
			// Creo las facturas
			this.factura1 = TestBase.CrearFactura();
			this.factura1.Codigo = 1;
			this.factura1.Numero = 1;
			this.factura1.Cliente = this.ClienteSinBonificacion;
			this.factura1.Items.Clear();
			this.factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura1);
			this.factura1.Create();
			
			this.factura2 = TestBase.CrearFactura();
			this.factura2.Codigo = this.factura1.Codigo;
			this.factura2.Numero = this.factura1.Numero + 1;
			this.factura2.Cliente = this.ClienteSinBonificacion;
			this.factura2.Items.Clear();
			this.factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura2);
			this.factura2.Create();
			
			this.factura3 = TestBase.CrearFactura();
			this.factura3.Codigo = this.factura2.Codigo;
			this.factura3.Numero = this.factura2.Numero + 1;
			this.factura3.Cliente = this.ClienteSinBonificacion;
			this.factura3.Items.Clear();
			this.factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.factura3);
			this.factura3.Create();
			
			this.facturaB1 = TestBase.CrearFactura();
			this.facturaB1.Tipo = FacturaTipoVenta.FacturaB;
			this.facturaB1.Codigo = 1;
			this.facturaB1.Numero = 1;
			this.facturaB1.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;
			this.facturaB1.Items.Clear();
			this.facturaB1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.facturaB1);
			this.facturaB1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo las prefacturas
			this.preFactura1 = TestBase.CrearPreFactura();
			this.preFactura1.Tipo = FacturaTipoVenta.FacturaA;
			this.preFactura1.Cliente = this.ClienteSinBonificacion;
			this.preFactura1.Items.Clear();
			this.preFactura1.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preFactura1);
			this.preFactura1.Create();
			
			this.preFactura2 = TestBase.CrearPreFactura();
			this.preFactura2.Tipo = FacturaTipoVenta.FacturaA;
			this.preFactura2.Cliente = this.ClienteSinBonificacion;
			this.preFactura2.Items.Clear();
			this.preFactura2.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preFactura2);
			this.preFactura2.Create();
			
			this.preFactura3 = TestBase.CrearPreFactura();
			this.preFactura3.Tipo = FacturaTipoVenta.FacturaA;
			this.preFactura3.Cliente = this.ClienteSinBonificacion;
			this.preFactura3.Items.Clear();
			this.preFactura3.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preFactura3);
			this.preFactura3.Create();
			
			this.preFacturaB1 = TestBase.CrearPreFactura();
			this.preFacturaB1.Tipo = FacturaTipoVenta.FacturaB;
			this.preFacturaB1.Cliente = this.ClienteSinBonificacion_ConsumidorFinal;
			this.preFacturaB1.Items.Clear();
			this.preFacturaB1.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preFacturaB1);
			this.preFacturaB1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo las notas de crédito
			this.notaCredito1 = TestBase.CrearNotaCredito();
			this.notaCredito1.Tipo = this.factura3.Tipo;
			this.notaCredito1.Codigo = this.factura3.Codigo;
			this.notaCredito1.Numero = this.factura3.Numero + 1;
			this.notaCredito1.FacturaAsociada = Factura.Find(this.factura3.Id);
			this.notaCredito1.Cliente = this.ClienteSinBonificacion;
			this.notaCredito1.Items.Clear();
			this.notaCredito1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.notaCredito1);
			this.notaCredito1.Create();
			
			// Creo las pre notas de crédito
			this.preNotaCredito1 = TestBase.CrearPreNotaCredito();
			this.preNotaCredito1.Tipo = this.factura1.Tipo;
			this.preNotaCredito1.FacturaAsociada = Factura.Find(this.factura1.Id);
			this.preNotaCredito1.Cliente = this.ClienteSinBonificacion;
			this.preNotaCredito1.Items.Clear();
			this.preNotaCredito1.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preNotaCredito1);
			this.preNotaCredito1.Create();
			
			this.FlushAndRecreateScope();
		}
		#endregion
		
		#region Métodos de soporte
		public long SiguienteNumeroFactura(FacturaTipoVenta tipo, int codigo) {
			ScalarQuery<Nullable<long>> consultaUltimoNumero =
				new ScalarQuery<Nullable<long>>(typeof(Factura),
				                          @"select max(f.Numero)
											from Factura f
											where f.Tipo = :tipo
												and f.Codigo = :codigo");
			
			consultaUltimoNumero.SetParameter("tipo", tipo);
			consultaUltimoNumero.SetParameter("codigo", codigo);
			
			long? ultimoNumero = consultaUltimoNumero.Execute();
			
			return ultimoNumero.Value + 1;
		}
		#endregion
		
		[Test]
		public void BuscarPreFacturaHQLTest()
		{
			string consulta = "from PreFactura f where f.class = PreFactura";
			
			PreFacturaDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				// FIXME: Testear la consulta paginable
				resultado = this.controladorFacturacion.BuscarPreFacturaHQL(consulta,
				                                                            null);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(4, resultado.Length, "Cantidad retornado");
		}
		
		[Test]
		public void BuscarFacturaHQLTest()
		{
			// Borro todas las PreNotaCredito y Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			try {
				this.factura1.Create();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex), "Creación de factura 1");
			}
			
			this.MarcarComoNoPersistido(ref this.factura2);
			this.factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo la prueba
			string consulta = "from Factura";
			
			FacturaDto[] resultado = null;
			
			this.RemoveTransaction();
			
			try {
				// FIXME: Testear la consulta paginable
				resultado = this.controladorFacturacion.BuscarFacturaHQL(consulta, null);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(2, resultado.Length, "Cantidad retornado");
		}
		
		[Test]
		public void ObtenerPreFacturasEncoladasParaImprimir_FacturaA()
		{
			// Primero borro todas las NotaCredito y PreNotaCredito,
			// ya que son Facturas y estorban aquí.
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			ActiveRecordValidationBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IControladorTerminales controladorTerminales =
				this.mocks.StrictMock<IControladorTerminales>();
			
			Expect.Call(controladorTerminales.ObtenerTerminal("identificador", this.iceCurrentIp1Puerto1))
				.Return(
					new DatosTerminal {
						identificador = "identificador",
						datosTermImpresora = new DatosTerminalImpresora {
							tiposDocumento = new TipoDocumento[] { TipoDocumento.FacturaTipoA }
						}
					}
				);
			
			this.mocks.ReplayAll();
			
			this.controladorFacturacion.ControladorTerminales = controladorTerminales;
			
			this.RemoveTransaction();
			
			PreFacturaDto[] resultado =
				this.controladorFacturacion
					.ObtenerPreFacturasEncoladasParaImprimir("identificador", null,
					                                         this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(3, resultado.Length, "Cantidad facturas");
			Assert.IsTrue(resultado.Cast<PreFacturaDto>()
			              	.All<PreFacturaDto>(f => f.tipo == FacturaTipoVenta.FacturaA),
			              "Tipos de las facturas");
		}
		
		[Test]
		public void ObtenerPreFacturasEncoladasParaImprimir_FacturaB_SinResultados()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IControladorTerminales controladorTerminales =
				this.mocks.StrictMock<IControladorTerminales>();
			
			Expect.Call(controladorTerminales.ObtenerTerminal("identificador", this.iceCurrentIp1Puerto1))
				.Return(
					new DatosTerminal {
						identificador = "identificador",
						datosTermImpresora = new DatosTerminalImpresora {
							tiposDocumento = new TipoDocumento[] { TipoDocumento.FacturaTipoB }
						}
					}
				);
			
			this.mocks.ReplayAll();
			
			this.controladorFacturacion.ControladorTerminales = controladorTerminales;
			
			this.RemoveTransaction();
			
			PreFacturaDto[] resultado =
				this.controladorFacturacion
					.ObtenerPreFacturasEncoladasParaImprimir("identificador", null,
					                                         this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(0, resultado.Length, "Cantidad facturas");
		}
		
		[Test]
		public void ObtenerPreFacturasEncoladasParaImprimir_FacturaB()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IControladorTerminales controladorTerminales =
				this.mocks.StrictMock<IControladorTerminales>();
			
			Expect.Call(controladorTerminales.ObtenerTerminal("identificador", this.iceCurrentIp1Puerto1))
				.Return(
					new DatosTerminal {
						identificador = "identificador",
						datosTermImpresora = new DatosTerminalImpresora {
							tiposDocumento = new TipoDocumento[] { TipoDocumento.FacturaTipoB }
						}
					}
				);
			
			this.mocks.ReplayAll();
			
			this.controladorFacturacion.ControladorTerminales = controladorTerminales;
			
			// Creo una factura B
			this.ClienteSinBonificacion.TipoResponsable =
				TipoResponsable.ConsumidorFinal;
			this.ClienteSinBonificacion.Cuit = new CUIT();
			this.ClienteSinBonificacion.Update();
			
			PreFactura preFacturaTipoB = TestBase.CrearPreFacturaTipoB();
			preFacturaTipoB.Cliente = this.ClienteSinBonificacion;
			preFacturaTipoB.PorcentajeBonificacion =
				this.ClienteSinBonificacion.PorcentajeBonificacion;
			preFacturaTipoB.Items.Clear();
			preFacturaTipoB.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTipoB);
			preFacturaTipoB.Create();
			
			this.RemoveTransaction();
			
			PreFacturaDto[] resultado =
				this.controladorFacturacion
					.ObtenerPreFacturasEncoladasParaImprimir("identificador", null,
					                                         this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(1, resultado.Length, "Cantidad facturas");
			Assert.IsTrue(resultado.Cast<PreFacturaDto>()
			              	.All<PreFacturaDto>(f => f.tipo == FacturaTipoVenta.FacturaB),
			              "Tipos de las facturas");
		}
		
		[Test]
		[ExpectedException(typeof(StratosException))]
		public void BuscarQueryObjects_QueryObjectTipoInvalido()
		{
			// Preparo la consulta al servidor
			ClienteDtoQuery consulta = new ClienteDtoQuery();
			
			this.RemoveTransaction();
			
			Ice.Object[] resultados =
				this.controladorFacturacion
					.BuscarQueryObjects(consulta, null, this.iceCurrentIp1Puerto1);
		}
		
		[Test]
		public void BuscarQueryObjects_FacturaDto()
		{
			// Preparo la consulta al servidor
			FacturaDtoQuery consulta = new FacturaDtoQuery();
			
			this.RemoveTransaction();
			
			Ice.Object[] resultados =
				this.controladorFacturacion
					.BuscarQueryObjects(consulta, null, this.iceCurrentIp1Puerto1);
		}
		
		[Test]
		public void BuscarQueryObjects_PreFacturaDto()
		{
			// Preparo la consulta al servidor
			PreFacturaDtoQuery consulta = new PreFacturaDtoQuery();
			
			this.RemoveTransaction();
			
			Ice.Object[] resultados =
				this.controladorFacturacion
					.BuscarQueryObjects(consulta, null, this.iceCurrentIp1Puerto1);
		}
		
		[Test]
		public void BuscarQueryObjects_CriterioOr()
		{
			// Borro todas las Facturas
			// Borro todas las PreNotaCredito y Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Create();
			
			this.MarcarComoNoPersistido(ref this.factura2);
			this.factura2.Create();
			
			this.FlushAndRecreateScope();
			
			// Preparo la consulta al servidor
			FacturaDtoQuery consultaFacturas = new FacturaDtoQuery();
			consultaFacturas.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioOr conjuncion = new CriterioOr();
			conjuncion.criterios = new List<CriterioGenerico>();
			
			// Criterio de número
			CriterioSimple criterioCodigo = new CriterioSimple();
			criterioCodigo.operador = TipoOperador.Igual;
			criterioCodigo.propiedad = "Numero";
			criterioCodigo.valor = this.factura1.Numero.ToString() + "L";
			
			conjuncion.criterios.Add(criterioCodigo);
			
			criterioCodigo = new CriterioSimple();
			criterioCodigo.operador = TipoOperador.Igual;
			criterioCodigo.propiedad = "Numero";
			criterioCodigo.valor = this.factura2.Numero.ToString() + "L";
			
			conjuncion.criterios.Add(criterioCodigo);
			
			// Agrego el criterio AND a la consulta
			consultaFacturas.listaCriteriosComplejos.Add(conjuncion);
			
			this.RemoveTransaction();
			
			Ice.Object[] resultados = null;
			try {
				resultados =
					this.controladorFacturacion
						.BuscarQueryObjects(consultaFacturas, null, this.iceCurrentIp1Puerto1);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			catch (System.Exception ex) {
				Assert.Fail(ex.Message);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(2, resultados.Length, "Cantidad de resultados");
			CollectionAssert.AllItemsAreInstancesOfType(resultados, typeof(FacturaDto), "Tipo Factura");
		}
		
		[Test]
		[Ignore("Este test no parece funcionar muy bien.")]
		public void BuscarQueryObjects_CriterioAnd()
		{
			// Preparo la consulta al servidor
			FacturaDtoQuery consultaFacturas = new FacturaDtoQuery();
			consultaFacturas.listaCriteriosComplejos = new List<CriterioComplejo>();
			
			CriterioAnd conjuncion = new CriterioAnd();
			conjuncion.criterios = new List<CriterioGenerico>();
			
			// Criterio de código
			CriterioSimple criterioCodigo = new CriterioSimple();
			criterioCodigo.operador = TipoOperador.Igual;
			criterioCodigo.propiedad = "Codigo";
			criterioCodigo.valor = this.factura1.Codigo.ToString();
			
			conjuncion.criterios.Add(criterioCodigo);
			
			// Criterio numero
			CriterioSimple criterioNumero = new CriterioSimple();
			criterioNumero.operador = TipoOperador.Igual;
			criterioNumero.propiedad = "Numero";
			criterioNumero.valor = this.factura1.Numero.ToString() + "L";
			
			conjuncion.criterios.Add(criterioNumero);
			
			// Criterio tipo
			CriterioSimple criterioTipo = new CriterioSimple();
			criterioTipo.operador = TipoOperador.Igual;
			criterioTipo.propiedad = "Tipo";
			criterioTipo.tipoValor = "FacturaTipo";
			criterioTipo.valor = this.factura1.Tipo.ToString();
			
			conjuncion.criterios.Add(criterioTipo);
			
			// Agrego el criterio AND a la consulta
			consultaFacturas.listaCriteriosComplejos.Add(conjuncion);
			
			this.RemoveTransaction();
			
			Ice.Object[] resultados = null;
			try {
				resultados =
					this.controladorFacturacion
						.BuscarQueryObjects(consultaFacturas, null, this.iceCurrentIp1Puerto1);
			}
			catch (StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			catch (System.Exception ex) {
				Assert.Fail(ex.Message);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(1, resultados.Length, "Cantidad de resultados");
			CollectionAssert.AllItemsAreInstancesOfType(resultados, typeof(FacturaDto), "Tipo Factura");
			
			FacturaDto[] resultadosFacturas = resultados.Cast<FacturaDto>().ToArray<FacturaDto>(); //Array.ConvertAll<Ice.Object,FacturaDto>(resultados);//.Cast<FacturaDto>();
			Assert.AreEqual(this.factura1.Codigo, resultadosFacturas[0].codigo, "Codigo factura");
			Assert.AreEqual(this.factura1.Numero, resultadosFacturas[0].numero, "Número factura");
			Assert.AreEqual(this.factura1.Tipo, resultadosFacturas[0].tipo, "Tipo factura");
		}
		
		[Test]
		public void RefrescarPreFactura()
		{
			PreFacturaDto preFacturaDto1 = ConvertidorDto.ToDto(this.preFactura1);
			
			this.preFactura1.Fecha = DateTime.Today - TimeSpan.FromDays(3);
			this.preFactura1.CondicionVenta = new CondicionVentaFacturaGenerica(CondicionPago.Tarjeta, 0);
			this.preFactura1.Update();
			
			this.RemoveTransaction();
			
			PreFacturaDto preFacturaDto1Refrescada = (PreFacturaDto)
				this.controladorFacturacion.Refrescar(preFacturaDto1, this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			this.preFactura1 = PreFactura.Find(this.preFactura1.Id);
			
			Assert.AreEqual(this.preFactura1.Fecha.Ticks, preFacturaDto1Refrescada.fechaTicks, "Fecha");
			Assert.AreEqual(this.preFactura1.CondicionVenta.Condicion,
			                preFacturaDto1Refrescada.condVenta.condPago, "Condicion de venta");
			Assert.AreEqual(this.preFactura1.Version, preFacturaDto1Refrescada.version, "Version");
			
			// Items
			Assert.IsNotNull(preFacturaDto1Refrescada.items, "Items prefactura refrescada no son nulos");
			Assert.IsTrue(preFacturaDto1Refrescada.items.Count > 0, "Items prefactura refrescada mayor a cero");
			Assert.AreEqual(this.preFactura1.Items.Count, preFacturaDto1Refrescada.items.Count, "Cantidad items");
		}
		
		[Test]
		public void RefrescarPreFacturaEliminada()
		{
			PreFacturaDto preFacturaDto1 = ConvertidorDto.ToDto(this.preFactura1);
			
			this.preFactura1.Delete();
			
			this.RemoveTransaction();
			
			try {
				PreFacturaDto preFacturaDto1Refrescada = (PreFacturaDto)
					this.controladorFacturacion.Refrescar(preFacturaDto1, this.iceCurrentIp1Puerto1);
				Assert.Fail("No se debio poder refrescar la prefactura");
			}
			catch(ObjetoInexistenteException) {
			}
		}
		
		[Test]
		public void GuardarPreFactura_ConClienteExistente_SinTerminalesImpresoras()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			// Para asegurarme que tome la factura como nueva, le asigno un id igual a cero
			this.MarcarComoNoPersistido(ref this.preFactura1);
			
			// Creo la prefactura a convertir
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(this.preFactura1);
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturaLeida = PreFactura.FindAll();
				
				Assert.AreEqual(4, preFacturaLeida.Length, "Cantidad de PreFacturas");
				Assert.IsFalse(retorno, "Retorno falso");
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo imprimir la PreFactura");
			}
		}
		
		[Test]
		public void GuardarPreFactura_ConClienteExistente_SinTerminalesImpresoras_PreFacturaExistente()
		{
			// Borro todas las facturas existentes
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la prefactura a convertir
			this.MarcarComoNoPersistido(ref this.preFactura1);
			
			this.preFactura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Ahora que guardé la prefactura, agrego un nuevo item
			this.preFactura1.Items.Add(TestBase.CrearItemPreFactura(this.ProductoMosaico, 10));
			TestBase.ProcesarItemsYCalcularImportesFactura(this.preFactura1);
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(this.preFactura1);
			
			// Realizo unas modificaciones a la versión DTO
			CondicionVentaDto nuevaCondicionVenta =
				new CondicionVentaDto(CondicionPago.Tarjeta, 0);
			DateTime nuevaFecha = DateTime.Today - TimeSpan.FromDays(2);
			
			preFacturaDto.condVenta = nuevaCondicionVenta;
			preFacturaDto.fechaTicks = nuevaFecha.Ticks;
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturaLeida = PreFactura.FindAll();
				
				Assert.AreEqual(1, preFacturaLeida.Length, "Cantidad de PreFacturas");
				Assert.AreEqual(nuevaCondicionVenta.condPago,
				                preFacturaLeida[0].CondicionVenta.Condicion, "Condicion venta");
				Assert.AreEqual(nuevaCondicionVenta.plazo,
				                preFacturaLeida[0].CondicionVenta.Plazo, "Condicion venta plazo");
				Assert.AreEqual(nuevaFecha,
				                preFacturaLeida[0].Fecha, "Fecha factura");
				
				// Me aseguro que haya solamente los items de esta prefactura
				ItemPreFactura[] items = ItemPreFactura.FindAll();
				
				Assert.AreEqual(2, items.Length, "Cantidad de items guardados");
			}
			catch (StratosException exception) {
				Assert.Fail("Hubo un error: " + exception.mensaje);
			}
		}
		
		[Test]
		public void GuardarPreFactura_VersionIncorrecta()
		{
			// Borro todas las facturas existentes
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la prefactura a convertir
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(this.preFactura1);
			
			// Ahora modifico la prefactura aumentándo su versión y dejando el objeto
			// DTO inválido
			this.preFactura1.Fecha = this.preFactura1.Fecha - TimeSpan.FromDays(1);
			this.preFactura1.UpdateAndFlush();
			
			// Realizo unas modificaciones a la versión DTO
			CondicionVentaDto nuevaCondicionVenta =
				new CondicionVentaDto(CondicionPago.Tarjeta, 0);
			DateTime nuevaFecha = DateTime.Today - TimeSpan.FromDays(2);
			
			preFacturaDto.condVenta = nuevaCondicionVenta;
			preFacturaDto.fechaTicks = nuevaFecha.Ticks;
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				Assert.Fail("Se pudo guardar una prefactura con versión incorrecta");
			}
			catch (ObjetoInexistenteException exception) {
			}
		}
		
		[Test]
		public void GuardarPreFactura_ConClienteExistente_ConTerminalesImpresoras()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			// Para asegurarme que tome la factura como nueva, le asigno un id igual a cero
			this.MarcarComoNoPersistido(ref this.preFactura1);
			
			// Creo la prefactura a convertir
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(this.preFactura1);
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			Expect.Call(delegate { this.servicioFacturacion1.ImprimirPreFactura(null); })
				.IgnoreArguments();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal> {
					new DatosTerminal {
						identificador = "id",
						datosTermImpresora = new DatosTerminalImpresora {
							servFacturacion = new List<ServicioFacturacionPrx> {
								this.servicioFacturacion1
							},
							tiposDocumento = new TipoDocumento[] { TipoDocumento.FacturaTipoA }
						}
					}
				});
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturaLeida = PreFactura.FindAll();
				
				Assert.AreEqual(4, preFacturaLeida.Length, "Cantidad de PreFacturas");
				Assert.IsTrue(retorno, "Retorno verdadero");
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo guardar la PreFactura: " + exception.mensaje);
			}
		}
		
		[Test]
		public void GuardarPreFactura_ConClienteNoExistente_SinTerminalesImpresoras()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			// TODO: Verificar que tenga la propiedad AltaAutomatica en TRUE.
			
			// Para asegurarme que tome la factura como nueva, le asigno un id igual a cero
			this.MarcarComoNoPersistido(ref this.preFactura1);
			
			// Creo una factura B
			Cliente cliente = new Cliente {
				Codigo = -1,
				RazonSocial = "Un cliente nuevo",
				TipoResponsable = TipoResponsable.ConsumidorFinal
			};
			
			PreFactura preFacturaTipoB = TestBase.CrearPreFacturaTipoB();
			preFacturaTipoB.Cliente = cliente;
			preFacturaTipoB.PorcentajeBonificacion =
				cliente.PorcentajeBonificacion;
			preFacturaTipoB.Items.Clear();
			preFacturaTipoB.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTipoB);
			
			// Creo la prefactura a guardar y le quito el cliente para que se de alta automáticamente
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(preFacturaTipoB);
			preFacturaDto.cliente.cuit = String.Empty;
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturaLeida = PreFactura.FindAll();
				
				Assert.AreEqual(4, preFacturaLeida.Length, "Cantidad de PreFacturas");
				Assert.IsFalse(retorno, "Retorno falso");
				
				PreFactura preFacturaNueva =
					(from f in preFacturaLeida
					 where f.Cliente.RazonSocial == "Un cliente nuevo"
					 select f).First();
				
				Assert.IsNotNull(preFacturaNueva, "PreFactura no nula");
				Assert.IsTrue(preFacturaNueva.Cliente.AltaAutomatica, "Cliente alta automática");
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo imprimir la PreFactura");
			}
		}
		
		[Test]
		public void GuardarPreFactura_ConClienteNoExistente_ConTerminalesImpresoras()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			// TODO: Verificar que tenga la propiedad AltaAutomatica en TRUE.
			
			// Para asegurarme que tome la factura como nueva, le asigno un id igual a cero
			this.MarcarComoNoPersistido(ref this.preFactura1);
			
			// Creo una factura B
			Cliente cliente = new Cliente {
				Codigo = -1,
				RazonSocial = "Un cliente nuevo",
				TipoResponsable = TipoResponsable.ConsumidorFinal
			};
			
			PreFactura preFacturaTipoB = TestBase.CrearPreFacturaTipoB();
			preFacturaTipoB.Cliente = cliente;
			preFacturaTipoB.PorcentajeBonificacion =
				cliente.PorcentajeBonificacion;
			preFacturaTipoB.Items.Clear();
			preFacturaTipoB.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(preFacturaTipoB);
			
			// Creo la prefactura a guardar y le quito el cliente para que se de alta automáticamente
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto<PreFactura, PreFacturaDto>(preFacturaTipoB);
			preFacturaDto.cliente.cuit = String.Empty;
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			Expect.Call(delegate { this.servicioFacturacion1.ImprimirPreFactura(null); })
				.IgnoreArguments();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoB))
				.Return(new List<DatosTerminal> {
					new DatosTerminal {
						identificador = "id",
						datosTermImpresora = new DatosTerminalImpresora {
							servFacturacion = new List<ServicioFacturacionPrx> {
								this.servicioFacturacion1
							},
							tiposDocumento = new TipoDocumento[] { TipoDocumento.FacturaTipoB }
						}
					}
				});
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preFacturaDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturaLeida = PreFactura.FindAll();
				
				Assert.AreEqual(4, preFacturaLeida.Length, "Cantidad de PreFacturas");
				Assert.IsTrue(retorno, "Retorno verdadero");
				
				PreFactura preFacturaNueva =
					(from f in preFacturaLeida
					 where f.Cliente.RazonSocial == "Un cliente nuevo"
					 select f).First();
				
				Assert.IsNotNull(preFacturaNueva, "PreFactura no nula");
				Assert.IsTrue(preFacturaNueva.Cliente.AltaAutomatica, "Cliente alta automática");
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo imprimir la PreFactura: " + exception.mensaje);
			}
		}
		
		[Test]
		public void GuardarPreFactura_PreNotaCreditoDto()
		{
			// Borro todas las PreNotaCredito
			ActiveRecordValidationBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la pre nota credito 1
			this.MarcarComoNoPersistido(ref this.preNotaCredito1);
			this.preNotaCredito1.Create();
			
			this.FlushAndRecreateScope();
			
			// Para asegurarme que tome la nota de crédito como nueva, le asigno un id igual a cero
			this.MarcarComoNoPersistido(ref this.preNotaCredito1);
			
			// Creo la prefactura a convertir
			PreNotaCreditoDto preNotaCreditoDto =
				ConvertidorDto.ToDto<PreNotaCredito, PreNotaCreditoDto>(this.preNotaCredito1);
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			string mensajeError;
			bool retorno = true;
			
			this.RemoveTransaction();
			
			try {
				retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preNotaCreditoDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo guardar la nota de crédito: " + exception.mensaje);
			}
			
			PreNotaCredito[] preNotaCreditoLeida = PreNotaCredito.FindAll();
			PreNotaCredito preNotaCreditoRecienIngresada =
				(from nc in preNotaCreditoLeida
				 where nc.FacturaAsociada != null &&
				 	nc.FacturaAsociada.Id == this.preNotaCredito1.FacturaAsociada.Id
				 select nc).First();
			
			Assert.AreEqual(2, preNotaCreditoLeida.Length, "Cantidad de PreNotaCreditos");
			Assert.IsFalse(retorno, "Retorno falso");
			
			Assert.IsNotNull(preNotaCreditoRecienIngresada.FacturaAsociada, "Factura asociada no nula");
			Assert.AreEqual(this.preNotaCredito1.FacturaAsociada.Id,
			                preNotaCreditoRecienIngresada.FacturaAsociada.Id, "Id factura asociada");
		}
		
		[Test]
		public void GuardarPreFactura_PreNotaCreditoDto_SobreFacturaConPreciosViejosDeProductos()
		{
			// Borro todas las PreNotaCredito
			ActiveRecordValidationBase<PreNotaCredito>.DeleteAll();
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			this.FlushAndRecreateScope();
			
			this.factura1.Codigo = 4;
			this.factura1.Update();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaAsociada = ConvertidorDto.ToDto(this.factura1);
			
			// Creo la nota de crédito Dto de la factura
			PreNotaCreditoDto preNotaCreditoDto = new PreNotaCreditoDto();
			preNotaCreditoDto.codigo = facturaAsociada.codigo;
			preNotaCreditoDto.cliente = facturaAsociada.cliente;
			preNotaCreditoDto.condVenta = new CondicionVentaDto(CondicionPago.Contado, 0);
			preNotaCreditoDto.facturaAsociada = facturaAsociada;
			preNotaCreditoDto.fechaTicks = DateTime.Today.Ticks;
			preNotaCreditoDto.importeBonificacion = 0.00;
			preNotaCreditoDto.importeBruto = facturaAsociada.importeBruto;
			preNotaCreditoDto.importeFinal = facturaAsociada.importeFinal;
			preNotaCreditoDto.importeIva = facturaAsociada.importeIva;
			preNotaCreditoDto.importeNeto = facturaAsociada.importeNeto;
			preNotaCreditoDto.importeTotal = facturaAsociada.importeTotal;
			
			preNotaCreditoDto.items = new List<ItemGenericoDto>();
			ItemProductoDto itemNotaCredito = new ItemProductoDto();
			itemNotaCredito.cantidad = (facturaAsociada.items[0] as ItemProductoDto).cantidad;
			itemNotaCredito.descripcion = (facturaAsociada.items[0] as ItemProductoDto).descripcion;
			itemNotaCredito.importe = (facturaAsociada.items[0] as ItemProductoDto).importe;
			itemNotaCredito.orden = (facturaAsociada.items[0] as ItemProductoDto).orden;
			itemNotaCredito.porcentajeBonificacion = (facturaAsociada.items[0] as ItemProductoDto).porcentajeBonificacion;
			itemNotaCredito.precioUnitario = (facturaAsociada.items[0] as ItemProductoDto).precioUnitario;
			itemNotaCredito.productoAbstracto = (facturaAsociada.items[0] as ItemProductoDto).productoAbstracto;
			itemNotaCredito.productoCodigo = (facturaAsociada.items[0] as ItemProductoDto).productoCodigo;
			itemNotaCredito.productoDescripcion = (facturaAsociada.items[0] as ItemProductoDto).productoDescripcion;
			itemNotaCredito.productoId = (facturaAsociada.items[0] as ItemProductoDto).productoId;
			itemNotaCredito.productoImporteSinIva = (facturaAsociada.items[0] as ItemProductoDto).productoImporteSinIva;
			itemNotaCredito.productoImporteVenta = (facturaAsociada.items[0] as ItemProductoDto).productoImporteVenta;
			itemNotaCredito.productoPrecioCosto = (facturaAsociada.items[0] as ItemProductoDto).productoPrecioCosto;
			itemNotaCredito.productoRubroId = (facturaAsociada.items[0] as ItemProductoDto).productoRubroId;
			itemNotaCredito.productoUnidad = (facturaAsociada.items[0] as ItemProductoDto).productoUnidad;
			
			preNotaCreditoDto.items.Add(itemNotaCredito);
			
			preNotaCreditoDto.tipo = facturaAsociada.tipo;
			
			// Ahora modifico el precio del producto asociado a la factura de la nota de crédito
			Producto productoAsociado = this.ProductoGriferia;
			productoAsociado.ImporteSinIva += 10.0m;
			productoAsociado.Update();
			
			this.FlushAndRecreateScope();
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			string mensajeError;
			bool retorno = true;
			
			this.RemoveTransaction();
			
			try {
				retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preNotaCreditoDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo guardar la nota de crédito: " + exception.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			PreNotaCredito[] resultado = PreNotaCredito.FindAll();
			Assert.AreEqual(1, resultado.Length, "Cantidad de PreNotaCreditos");
			
			PreNotaCredito preNotaCreditoRecienIngresada = resultado[0];
			
			Assert.IsNotNull(preNotaCreditoRecienIngresada.FacturaAsociada, "Factura asociada no nula");
			Assert.AreEqual(this.factura1.Id, preNotaCreditoRecienIngresada.FacturaAsociada.Id, "Id factura asociada");
			
			// Me aseguro que los items estén bien guardados
			this.factura1 = Factura.Find(this.factura1.Id);
			
			ItemPreFactura[] preNotaCreditoItems = preNotaCreditoRecienIngresada.Items.ToArray();
			
			Assert.AreEqual(this.factura1.Items.Count, preNotaCreditoItems.Length, "Items count");
			Assert.AreEqual(this.factura1.Items[0].ProductoId, preNotaCreditoRecienIngresada.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(this.factura1.Items[0].ProductoCodigo, preNotaCreditoRecienIngresada.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(this.factura1.Items[0].Descripcion, preNotaCreditoRecienIngresada.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(this.factura1.Items[0].PrecioUnitario, preNotaCreditoRecienIngresada.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(this.factura1.Items[0].PorcentajeBonificacion, preNotaCreditoRecienIngresada.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(this.factura1.Items[0].Cantidad, preNotaCreditoRecienIngresada.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(this.factura1.Items[0].ProductoAbstracto, preNotaCreditoRecienIngresada.Items[0].ProductoAbstracto, "Item producto abstracto");
			Assert.AreEqual(this.factura1.Items[0].ProductoDescripcion, preNotaCreditoRecienIngresada.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(this.factura1.Items[0].ProductoImporteSinIva, preNotaCreditoRecienIngresada.Items[0].ProductoImporteSinIva, "Item producto importe sin iva");
			Assert.AreEqual(this.factura1.Items[0].ProductoImporteVenta, preNotaCreditoRecienIngresada.Items[0].ProductoImporteVenta, "Item producto importe venta");
			Assert.AreEqual(this.factura1.Items[0].ProductoPrecioCosto, preNotaCreditoRecienIngresada.Items[0].ProductoPrecioCosto, "Item producto precio costo");
			Assert.AreEqual(this.factura1.Items[0].Importe, preNotaCreditoRecienIngresada.Items[0].Importe, "Item importe");
			
			// Además valido que al imprimirse la PreNotaCredito, la NotaCredito
			// correspondiente se cree correctamente.
			
			long siguienteNumero =
				this.SiguienteNumeroFactura(preNotaCreditoRecienIngresada.Tipo,
				                            this.factura1.Codigo);
			
			preNotaCreditoDto = ConvertidorDto.ToDto<PreNotaCredito, PreNotaCreditoDto>(preNotaCreditoRecienIngresada);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.ImpresionPreFacturaFinalizadaCorrectamente(preNotaCreditoDto,
					                                            siguienteNumero,
					                                            this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("No se pudo imprimir la pre nota crédito: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Assert.AreEqual(1, NotaCredito.FindAll().Length, "Cantidad notas de crédito");
			NotaCredito notaCredito = NotaCredito.FindAll()[0];
			
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items.Count, notaCredito.Items.Count, "Items count");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoId, notaCredito.Items[0].ProductoId, "Item producto id");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoCodigo, notaCredito.Items[0].ProductoCodigo, "Item precio unitario");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].Descripcion, notaCredito.Items[0].Descripcion, "Item descripción");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].PrecioUnitario, notaCredito.Items[0].PrecioUnitario, "Item precio unitario");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].PorcentajeBonificacion, notaCredito.Items[0].PorcentajeBonificacion, "Item porcentaje bonificacion");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].Cantidad, notaCredito.Items[0].Cantidad, "Item cantidad");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoAbstracto, notaCredito.Items[0].ProductoAbstracto, "Item producto abstracto");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoDescripcion, notaCredito.Items[0].ProductoDescripcion, "Item producto descripción");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoImporteSinIva, notaCredito.Items[0].ProductoImporteSinIva, "Item producto importe sin iva");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoImporteVenta, notaCredito.Items[0].ProductoImporteVenta, "Item producto importe venta");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].ProductoPrecioCosto, notaCredito.Items[0].ProductoPrecioCosto, "Item producto precio costo");
			Assert.AreEqual(preNotaCreditoRecienIngresada.Items[0].Importe, notaCredito.Items[0].Importe, "Item importe");
		}
		
		[Test]
		public void GuardarPreFactura_PreNotaCreditoDto_Existente()
		{
			// Borro todas las PreNotaCredito
			ActiveRecordValidationBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la pre nota credito 1
			this.MarcarComoNoPersistido(ref this.preNotaCredito1);
			this.preNotaCredito1.Create();
			
			this.FlushAndRecreateScope();
			
			// Creo la prefactura a convertir
			PreNotaCreditoDto preNotaCreditoDto =
				ConvertidorDto.ToDto<PreNotaCredito, PreNotaCreditoDto>(this.preNotaCredito1);
			
			// Realizo unas modificaciones a la versión DTO
			CondicionVentaDto nuevaCondicionVenta =
				new CondicionVentaDto(CondicionPago.Tarjeta, 0);
			DateTime nuevaFecha = DateTime.Today - TimeSpan.FromDays(2);
			
			preNotaCreditoDto.condVenta = nuevaCondicionVenta;
			preNotaCreditoDto.fechaTicks = nuevaFecha.Ticks;
			
			// Seteo los mocks objects
			this.mocks.BackToRecordAll();
			
			IBuscadorTerminalesImpresoras buscador =
				this.mocks.StrictMock<IBuscadorTerminalesImpresoras>();
			
			Expect.Call(buscador.BuscarTerminalesImpresora(TipoDocumento.FacturaTipoA))
				.Return(new List<DatosTerminal>());
			
			this.mocks.ReplayAll();
			
			// Creo el Controlador de Facturacion con el buscador simulado
			this.controladorFacturacion.BuscadorTerminalesImpresoras = buscador;
			
			this.RemoveTransaction();
			
			try {
				string mensajeError;
				
				bool retorno =
					this.controladorFacturacion
						.GuardarPreFactura(preNotaCreditoDto,
						                   out mensajeError,
						                   this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreNotaCredito[] preNotaCreditoLeida = PreNotaCredito.FindAll();
				
				Assert.AreEqual(1, preNotaCreditoLeida.Length, "Cantidad de PreNotaCreditos");
				Assert.IsFalse(retorno, "Retorno falso");
				
				Assert.AreEqual(nuevaCondicionVenta.condPago,
				                preNotaCreditoLeida[0].CondicionVenta.Condicion, "Condicion venta");
				Assert.AreEqual(nuevaCondicionVenta.plazo,
				                preNotaCreditoLeida[0].CondicionVenta.Plazo, "Condicion venta plazo");
				Assert.AreEqual(nuevaFecha,
				                preNotaCreditoLeida[0].Fecha, "Fecha factura");
			}
			catch (StratosException exception) {
				Assert.Fail("No se pudo guardar la nota de crédito: " + exception.mensaje);
			}
		}
		
		[Test]
		public void PrepararPreFacturaParaImprimir_FacturaExistente()
		{
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.UpdateAndFlush();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto(this.preFactura1);
			
			long siguienteNumeroFactura;
			
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.PrepararPreFacturaParaImprimir(preFacturaDto,
				                                out siguienteNumeroFactura,
				                                this.iceCurrentIp1Puerto1);
			
			Assert.AreEqual(2, siguienteNumeroFactura, "Siguiente número factura");
		}
		
		[Test]
		public void PrepararPreFacturaParaImprimir_FacturaNoExistente()
		{
			// Borro todas las PreNotaCredito y Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto(this.preFactura1);
			
			long siguienteNumeroFactura;
			
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.PrepararPreFacturaParaImprimir(preFacturaDto,
				                                out siguienteNumeroFactura,
				                                this.iceCurrentIp1Puerto1);
			
			Assert.AreEqual(1, siguienteNumeroFactura, "Siguiente número factura");
		}
		
		[Test]
		public void PrepararPreFacturaParaImprimir_PreFacturaFechaInvalida()
		{
			// Primero borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Asigno a la PreFactura una fecha inválida
			this.preFactura1.Fecha = DateTime.Today - TimeSpan.FromDays(1);
			this.preFactura1.Update();
			
			// Ahora creo una Factura con fecha posterior a la PreFactura, dejando
			// a esta última con una fecha inválida
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.Fecha = DateTime.Today;
			this.factura1.Create();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto(this.preFactura1);
			
			long siguienteNumeroFactura;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.PrepararPreFacturaParaImprimir(preFacturaDto,
					                                out siguienteNumeroFactura,
					                                this.iceCurrentIp1Puerto1);
				
				Assert.Fail("No fue arrojada una excepción");
			}
			catch (ImpresionFacturaException) {
			}
		}
		
		[Test]
		public void ImpresionPreFacturaFinalizadaCorrectamente_FacturaExistente_NumeroCorrecto()
		{
			// Primero borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Asigno a la PreFactura una fecha inválida
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto(this.preFactura1);
			
			// Ahora creo una Factura con fecha posterior a la PreFactura, dejando
			// a esta última con una fecha inválida
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.Fecha = DateTime.Today;
			this.factura1.Create();
			
			long siguienteNumeroFactura = 2;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.ImpresionPreFacturaFinalizadaCorrectamente(preFacturaDto,
					                                            siguienteNumeroFactura,
					                                            this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturas = PreFactura.FindAll();
				Factura[] facturas = Factura.FindAll();
				
				Assert.AreEqual(0, preFacturas.Length, "Cantidad de prefacturas");
				Assert.AreEqual(2, facturas.Length, "Cantidad de facturas");
				
				Factura facturaIngresada =
					(from f in facturas
					 where f.Codigo == 4 && f.Numero == siguienteNumeroFactura
					 select f).First();
				
				Assert.IsNotNull(facturaIngresada);
				Assert.AreEqual(4, facturaIngresada.Codigo, "Codigo factura");
				Assert.AreEqual(siguienteNumeroFactura, facturaIngresada.Numero, "Numero factura");
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.Message);
			}
		}
		
		[Test]
		public void ImpresionPreFacturaFinalizadaCorrectamente_FacturaExistente_NumeroIncorrecto()
		{
			// Primero borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Asigno a la PreFactura una fecha inválida
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			PreFacturaDto preFacturaDto =
				ConvertidorDto.ToDto(this.preFactura1);
			
			// Ahora creo una Factura con fecha posterior a la PreFactura, dejando
			// a esta última con una fecha inválida
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.Fecha = DateTime.Today;
			this.factura1.Create();
			
			this.FlushAndRecreateScope();
			
			// Pongo un siguiente número incorrecto
			long siguienteNumeroFactura = 3;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.ImpresionPreFacturaFinalizadaCorrectamente(preFacturaDto,
					                                            siguienteNumeroFactura,
					                                            this.iceCurrentIp1Puerto1);
				
				Assert.Fail("No debio haberse podido ejecutar el método correctamente");
			}
			catch (ObjetoInvalidoException ex) {
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] preFacturas = PreFactura.FindAll();
				Factura[] facturas = Factura.FindAll();
				
				Assert.AreEqual(1, preFacturas.Length, "Cantidad de prefacturas");
				Assert.AreEqual(1, facturas.Length, "Cantidad de facturas");
			}
		}
		
		[Test]
		public void ImpresionPreFacturaFinalizadaCorrectamente_PreNotaCredito_NumeroCorrecto()
		{
			// Primero borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura, que será la fatura asignada a la nota de crédito
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.Fecha = DateTime.Today;
			this.factura1.Create();
			
			// Asigno a la PreFactura una fecha inválida
			this.MarcarComoNoPersistido(ref this.preNotaCredito1);
			this.preNotaCredito1.FacturaAsociada = this.factura1;
			this.preNotaCredito1.Create();
			
			// Paso la PreNotaCredito a PreNotaCreditoDto
			PreNotaCreditoDto preNotaCreditoDto =
				ConvertidorDto.ToDto<PreNotaCredito, PreNotaCreditoDto>(this.preNotaCredito1);
			
			// Dejo en la base de datos un Nota de Credito ya existente
			this.MarcarComoNoPersistido(ref this.factura2);
			this.factura2.Codigo = 4;
			this.factura2.Numero = 2;
			this.factura2.Fecha = DateTime.Today;
			this.factura2.Create();
			
			this.MarcarComoNoPersistido(ref this.notaCredito1);
			this.notaCredito1.FacturaAsociada = this.factura2;
			this.notaCredito1.Codigo = 4;
			this.notaCredito1.Numero = 3;
			this.notaCredito1.Fecha = DateTime.Today;
			this.notaCredito1.Create();
			
			this.FlushAndRecreateScope();
			
			long siguienteNumero = 4;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.ImpresionPreFacturaFinalizadaCorrectamente(preNotaCreditoDto,
					                                            siguienteNumero,
					                                            this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreNotaCredito[] preNotasCredito = PreNotaCredito.FindAll();
				NotaCredito[] notasCredito = NotaCredito.FindAll();
				Factura[] facturas = Factura.FindAll();
				
				Assert.AreEqual(0, preNotasCredito.Length, "Cantidad de PreNotaCredito");
				Assert.AreEqual(2, notasCredito.Length, "Cantidad de NotaCredito");
				Assert.AreEqual(4, facturas.Length, "Cantidad de Facturas");
				
				NotaCredito notaCreditoIngresada =
					(from f in notasCredito
					 where f.Codigo == 4 && f.Numero == siguienteNumero
					 select f).First();
				
				Assert.IsNotNull(notaCreditoIngresada);
				Assert.AreEqual(4, notaCreditoIngresada.Codigo, "Codigo NotaCredito");
				Assert.AreEqual(siguienteNumero, notaCreditoIngresada.Numero, "Numero NotaCredito");
				Assert.IsNotNull(notaCreditoIngresada.FacturaAsociada, "Factura asociada no nula");
				Assert.AreEqual(this.factura1.Id, notaCreditoIngresada.FacturaAsociada.Id, "Id Factura asociada");
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.Message);
			}
		}
		
		[Test]
		public void RegistrarFacturaManualTest()
		{
			this.factura1.Codigo = 3;
			this.factura1.Numero = 1;
			this.factura1.Update();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto =
				ConvertidorDto.ToDto(this.factura1);
			
			// Borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.RegistrarFacturaManual(facturaDto,
					                        this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(1, facturas.Length, "Cantidad facturas");
			
			Assert.AreEqual(3, facturas[0].Codigo, "Factura codigo");
			Assert.IsTrue(facturas[0].FacturaManual, "Factura manual");
			Assert.AreEqual(1, facturas[0].Items.Count, "Cantidad items");
		}
		
		#region GuardarFacturaManual
		protected Factura PrepararEscenario_GuardarFacturaManual()
		{
			#region Preparación del test
			this.factura1.Codigo = 3;
			this.factura1.Numero = 1;
			this.factura1.Update();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto =
				ConvertidorDto.ToDto(this.factura1);
			
			// Borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.RegistrarFacturaManual(facturaDto,
					                        this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(1, facturas.Length, "Cantidad facturas");
			
			Assert.AreEqual(3, facturas[0].Codigo, "Factura codigo");
			Assert.IsTrue(facturas[0].FacturaManual, "Factura manual");
			Assert.AreEqual(1, facturas[0].Items.Count, "Cantidad items");
			#endregion
			
			this.FlushAndRecreateScope();
			
			Factura factura = Factura.Find(facturas[0].Id);
			NHibernate.NHibernateUtil.Initialize(factura.Items);
			
			return factura;
		}
		
		[Test]
		public void GuardarFacturaManual_FacturaManualValida()
		{
			Factura factura = this.PrepararEscenario_GuardarFacturaManual();
			
			// Realizo unos cambios en la factura manual e intento
			// guardarlos
			FacturaDto facturaManualDto = ConvertidorDto.ToDto(factura);
			Assert.IsTrue(facturaManualDto.facturaManual, "Dto es factura manual");
			
			DateTime nuevaFecha = new DateTime(facturaManualDto.fechaTicks);
			nuevaFecha.AddDays(-15);
			facturaManualDto.fechaTicks = nuevaFecha.Ticks;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.GuardarFacturaManual(facturaManualDto,
					                      this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Verifico que se hayan realizado los cambios
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(1, facturas.Length, "Cantidad facturas 2");
			
			Assert.IsTrue(facturas[0].FacturaManual, "Factura manual 2");
			Assert.AreEqual(nuevaFecha.Ticks, facturas[0].Fecha.Ticks, "Fecha factura manual");
		}
		
		[Test]
		public void GuardarFacturaManual_ConNotaCreditoManual()
		{
			Factura factura = this.PrepararEscenario_GuardarFacturaManual();
			
			// Creo una nota de crédito sobre esta factura manual
			NotaCredito notaCredito = TestBase.CrearNotaCredito();
			notaCredito.FacturaManual = true;
			notaCredito.Tipo = factura.Tipo;
			notaCredito.Codigo = factura.Codigo;
			notaCredito.Numero = factura.Numero + 1;
			notaCredito.FacturaAsociada = factura;
			notaCredito.Cliente = this.ClienteSinBonificacion;
			notaCredito.Items.Clear();
			notaCredito.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(notaCredito);
			notaCredito.Create();
			
			this.FlushAndRecreateScope();
			
			// Realizo unos cambios en la nota de crédito manual e intento
			// guardarlos
			NotaCreditoDto facturaManualDto = ConvertidorDto.ToDto(notaCredito);
			Assert.IsTrue(facturaManualDto.facturaManual, "Dto es factura manual");
			
			DateTime nuevaFecha = new DateTime(facturaManualDto.fechaTicks);
			nuevaFecha.AddDays(-15);
			facturaManualDto.fechaTicks = nuevaFecha.Ticks;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.GuardarFacturaManual(facturaManualDto,
					                      this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			notaCredito = NotaCredito.Find(notaCredito.Id);
			
			// Verifico que se hayan realizado los cambios
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(2, facturas.Length, "Cantidad facturas 2");
			
			NotaCredito[] notasCredito = NotaCredito.FindAll();
			Assert.AreEqual(1, notasCredito.Length, "Cantidad notas crédito 2");
			
			Assert.AreEqual(notaCredito.Id, notasCredito[0].Id, "Nota credito id");
			Assert.IsTrue(notasCredito[0].FacturaManual, "Factura manual 2");
			Assert.AreEqual(nuevaFecha.Ticks, notasCredito[0].Fecha.Ticks, "Fecha factura manual");
		}
		
		[Test]
		public void GuardarFacturaManual_FacturaNoEsManual()
		{
			// Borro todas las facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Numero = 1;
			this.factura1.FacturaManual = false;
			this.factura1.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Realizo unos cambios en la factura que no es manual e intento
			// guardarlos
			FacturaDto facturaManualDto = ConvertidorDto.ToDto(this.factura1);
			Assert.IsFalse(facturaManualDto.facturaManual, "Dto no es factura manual");
			
			DateTime nuevaFecha = new DateTime(facturaManualDto.fechaTicks);
			nuevaFecha.AddDays(-15);
			facturaManualDto.fechaTicks = nuevaFecha.Ticks;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.GuardarFacturaManual(facturaManualDto,
					                      this.iceCurrentIp1Puerto1);
				
				Assert.Fail("Se pudo guardar una factura que no es manual");
			}
			catch (ObjetoInvalidoException ex) {
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			this.factura1 = Factura.Find(this.factura1.Id);
			
			// Verifico que se hayan realizado los cambios
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(1, facturas.Length, "Cantidad facturas");
			
			Assert.IsFalse(facturas[0].FacturaManual, "Factura manual");
			Assert.AreEqual(this.factura1.Fecha, facturas[0].Fecha, "Fecha factura manual");
		}
		#endregion
		
		[Test]
		public void RegistrarFacturaManualTest_NotaCredito()
		{
			// Borro todas las facturas
			ActiveRecordBase<NotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			this.MarcarComoNoPersistido(ref this.notaCredito1);
			
			NotaCreditoDto notaCreditoDto =
				ConvertidorDto.ToDto<NotaCredito,NotaCreditoDto>(this.notaCredito1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.RegistrarFacturaManual(notaCreditoDto,
					                        this.iceCurrentIp1Puerto1);
			}
			catch (System.Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.Message);
			}
			
			this.FlushAndRecreateScope();
			
			NotaCredito[] resultado = NotaCredito.FindAll();
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultados");
			Assert.AreEqual(this.notaCredito1.Codigo, resultado[0].Codigo, "Codigo");
			Assert.AreEqual(this.notaCredito1.Numero, resultado[0].Numero, "Codigo");
			Assert.AreEqual(this.notaCredito1.Tipo, resultado[0].Tipo, "Codigo");
			Assert.AreEqual(this.notaCredito1.Codigo, resultado[0].Codigo, "Codigo");
		}
		
		[Test]
		public void EliminarFactura_UltimaFactura()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Create();
			
			this.MarcarComoNoPersistido(ref this.factura2);
			this.factura2.Codigo = 4;
			this.factura2.Create();
			
			this.MarcarComoNoPersistido(ref this.factura3);
			this.factura3.Codigo = 4;
			this.factura3.Create();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura3);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaDto, this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				Factura[] facturas = Factura.FindAll();
				Assert.AreEqual(2, facturas.Length, "Cantidad de facturas");
			}
			catch (Exception ex) {
				Assert.Fail("Excepcion arrojada: " + ex.ToString());
			}
		}
		
		[Test]
		public void EliminarFactura_NoUltimaFactura()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Create();
			
			this.MarcarComoNoPersistido(ref this.factura2);
			this.factura2.Codigo = 4;
			this.factura2.Create();
			
			this.MarcarComoNoPersistido(ref this.factura3);
			this.factura3.Codigo = 4;
			this.factura3.Create();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaDto, this.iceCurrentIp1Puerto1);
				Assert.Fail("Se pudo eliminar una factura que no es la última");
			}
			catch (FacturaImposibleDeEliminarException ex) {
				this.FlushAndRecreateReadOnlyScope();
				
				Factura[] facturas = Factura.FindAll();
				
				Assert.AreEqual(3, facturas.Length, "Cantidad de facturas");
			}
		}
		
		[Test]
		public void EliminarFactura_FacturaHistorica()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Historica = true;
			this.factura1.Create();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaDto, this.iceCurrentIp1Puerto1);
				Assert.Fail("Se pudo eliminar una factura historica");
			}
			catch (FacturaImposibleDeEliminarException ex) {
				this.FlushAndRecreateReadOnlyScope();
				
				Factura[] facturas = Factura.FindAll();
				
				Assert.AreEqual(1, facturas.Length, "Cantidad de facturas");
			}
		}
		
		[Test]
		public void EliminarFactura_UltimaFactura_SeCreaPreFacturaCorrespondiente()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 4;
			this.factura1.Create();
			
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaDto, this.iceCurrentIp1Puerto1);
			}
			catch (Exception ex) {
				Assert.Fail("Excepcion arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// No debe haber más facturas
			Factura[] facturas = Factura.FindAll();
			Assert.AreEqual(0, facturas.Length, "Cantidad de facturas");
			
			// Debe haber una PreFactura correspondiente a la Factura eliminada
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(1, preFacturas.Length, "Cantidad pre facturas");
			
			// La PreFactura debe ser igual a la Factura
			PreFactura preFactura = preFacturas[0];
			
			Assert.IsNotNull(preFactura.Cliente, "cliente no nulo");
			Assert.AreEqual(this.factura1.Cliente.Codigo, preFactura.Cliente.Codigo, "Prefactura cliente código");
			Assert.AreEqual(this.factura1.CondicionVenta.Condicion, preFactura.CondicionVenta.Condicion, "Prefactura condición pago");
			Assert.AreEqual(this.factura1.CondicionVenta.Plazo, preFactura.CondicionVenta.Plazo, "Prefactura plazo");
			Assert.AreEqual(this.factura1.Items.Count, preFactura.Items.Count, "Prefactura cantidad items");
			Assert.AreEqual(this.factura1.Fecha, preFactura.Fecha, "Prefactura fecha");
			Assert.AreEqual(this.factura1.ImporteBonificacion, preFactura.ImporteBonificacion, "Prefactura importe bonif");
			Assert.AreEqual(this.factura1.ImporteBruto, preFactura.ImporteBruto, "Prefactura importe bruto");
			Assert.AreEqual(this.factura1.ImporteFinal, preFactura.ImporteFinal, "Prefactura importe final");
			Assert.AreEqual(this.factura1.ImporteIva, preFactura.ImporteIva, "Prefactura importe iva");
			Assert.AreEqual(this.factura1.ImporteNeto, preFactura.ImporteNeto, "Prefactura importe neto");
			Assert.AreEqual(this.factura1.ImporteTotal, preFactura.ImporteTotal, "Prefactura importe total");
			
			Assert.AreEqual(this.factura1.PorcentajeBonificacion, preFactura.PorcentajeBonificacion, "Prefactura porcentaje bonif");
			Assert.AreEqual(this.factura1.Remitos, preFactura.Remitos, "Prefactura remitos");
			
			Assert.AreEqual(this.factura1.Tipo, preFactura.Tipo, "Prefactura tipo");
		}
		
		[Test]
		public void EliminarFactura_FacturaInvalida_NoSeCreaPreFacturaCorrespondiente()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			// Ingreso una factura inválida
			this.controladorFacturacion
				.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 4, 1,
				                                this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la factura inválida a Dto
			Factura facturaInvalida = Factura.FindAll()[0];
			Assert.IsTrue(!facturaInvalida.Valida, "Factura inválida");
			
			FacturaDto facturaInvalidaDto = ConvertidorDto.ToDto(facturaInvalida);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaInvalidaDto,
				                                            this.iceCurrentIp1Puerto1);
			}
			catch (Exception ex) {
				Assert.Fail("Excepcion arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// No debe haber PreFacturas creadas
			// Debe haber una PreFactura correspondiente a la Factura eliminada
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(0, preFacturas.Length, "Cantidad pre facturas");
		}
		
		[Test]
		public void EliminarFactura_FacturaManual_NoSeCreaPreFacturaCorrespondiente()
		{
			// Borro todas las Facturas
			ActiveRecordBase<Factura>.DeleteAll();
			ActiveRecordBase<PreFactura>.DeleteAll();
			ActiveRecordBase<NotaCredito>.DeleteAll();
			ActiveRecordBase<PreNotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo una factura manual
			this.MarcarComoNoPersistido(ref this.factura1);
			this.factura1.Codigo = 9;
			this.factura1.Numero = 1;
			this.factura1.FacturaManual = true;
			this.factura1.Create();
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Paso la factura inválida a Dto
			this.factura1 = Factura.Find(this.factura1.Id);
			Assert.IsTrue(this.factura1.FacturaManual, "Factura manual");
			
			FacturaDto facturaManualDto = ConvertidorDto.ToDto(this.factura1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaManualDto,
				                                            this.iceCurrentIp1Puerto1);
			}
			catch (Exception ex) {
				Assert.Fail("Excepcion arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// No debe haber PreFacturas creadas
			// Debe haber una PreFactura correspondiente a la Factura eliminada
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(0, preFacturas.Length, "Cantidad pre facturas");
		}
		
		[Test]
		public void EliminarFactura_ExistePreNotaCredito_AsociadaAFacturaPorEliminar()
		{
			NotaCredito.DeleteAll();
			PreNotaCredito.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo una PreNotaCredito con esta factura asociada
			PreNotaCredito preNotaCredito = TestBase.CrearPreNotaCredito();
			preNotaCredito.FacturaAsociada = this.factura3;
			preNotaCredito.Cliente = this.factura3.Cliente;
			preNotaCredito.Items.Clear();
			preNotaCredito.Items.Add(TestBase.CrearItemPreFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(preNotaCredito);
			
			preNotaCredito.Create();
			
			this.FlushAndRecreateScope();
			
			// Ahora intento eliminar la factura
			this.factura1 = Factura.Find(this.factura3.Id);
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura3);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarFactura(facturaDto);
			}
			catch (Exception ex) {
				Assert.Fail("Excepción arrojada: " + ex.ToString());
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			// Se debió haber eliminado la Factura y la PreFactura correspondiente
			Assert.IsTrue(Factura.TryFind(this.factura3.Id) == null, "La factura fue eliminada");
//			Assert.IsTrue(PreNotaCredito.TryFind(preNotaCredito.Id) == null, "La PreNotaCredito fue eliminada");
		}
		
		[Test]
		public void EliminarPreFactura()
		{
			// Borro todas las Facturas
			ActiveRecordBase<PreFactura>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			// Creo la factura1 y factura2
			this.MarcarComoNoPersistido(ref this.preFactura1);
			this.preFactura1.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura2);
			this.preFactura2.Create();
			
			this.MarcarComoNoPersistido(ref this.preFactura3);
			this.preFactura3.Create();
			
			this.FlushAndRecreateScope();
			
			PreFacturaDto preFacturaDto = ConvertidorDto.ToDto(this.preFactura1);
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.EliminarPreFactura(preFacturaDto,
				                                               this.iceCurrentIp1Puerto1);
				
				this.FlushAndRecreateReadOnlyScope();
				
				PreFactura[] facturas = PreFactura.FindAll();
				
				Assert.AreEqual(2, facturas.Length, "Cantidad de facturas");
				
				// Items
				SimpleQuery<ItemPreFactura> consultaItems =
					new SimpleQuery<ItemPreFactura>("from ItemPreFactura where PreFacturaId = :preFacturaId");
				consultaItems.SetParameter("preFacturaId", preFacturaDto.id);
				
				ItemPreFactura[] items = consultaItems.Execute();
				
				Assert.AreEqual(0, items.Length, "Cantidad de items");
			}
			catch (Exception ex) {
				throw;
			}
		}
		
		#region IngresarFacturaDeSincronizacion
		[Test]
		public void IngresarFacturaDeSincronizacion_FacturaA_NumeroCorrecto_Concecutivo()
		{
			// Primero borro todas las NotaCredito, ya que son Facturas y estorban aquí.
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 1, 4,
					                                this.iceCurrentIp1Puerto1);
			}
			catch (StratosException ex) {
				Assert.Fail("Excepción: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaA),
				                Expression.Eq("Codigo", 1),
				                Expression.Eq("Numero", 4L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_FacturaA_NumeroCorrecto_NoConcecutivo()
		{
			// Primero borro todas las NotaCredito, ya que son Facturas y estorban aquí.
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 1, 100,
				                                this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaA),
				                Expression.Eq("Codigo", 1),
				                Expression.Eq("Numero", 100L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_FacturaB_NumeroCorrecto_Concecutivo()
		{
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaB, 1, 2,
				                                this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaB),
				                Expression.Eq("Codigo", 1),
				                Expression.Eq("Numero", 2L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_FacturaB_NumeroCorrecto_NoConcecutivo()
		{
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaB, 1, 200,
				                                this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaB),
				                Expression.Eq("Codigo", 1),
				                Expression.Eq("Numero", 200L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_NumeroCorrecto_ExisteNumeroMayorDeOtroCodigo()
		{
			// Primero borro todas las NotaCredito, ya que son Facturas y estorban aquí.
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			
			this.FlushAndRecreateScope();
			
			/* Creo facturas de otro código hasta tener un número de factura mayor que el código
			 * que estoy testeando. De esta forma me aseguro que busque la Factura de mayor número
			 * pero del mismo código. */
			Factura factura1 = TestBase.CrearFactura();
			factura1.Codigo = 2;
			factura1.Numero = 1;
			factura1.Cliente = this.ClienteSinBonificacion;
			factura1.Items.Clear();
			factura1.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura1);
			factura1.Create();
			
			Factura factura2 = TestBase.CrearFactura();
			factura2.Codigo = 2;
			factura2.Numero = 2;
			factura2.Cliente = this.ClienteSinBonificacion;
			factura2.Items.Clear();
			factura2.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura2);
			factura2.Create();
			
			Factura factura3 = TestBase.CrearFactura();
			factura3.Codigo = 2;
			factura3.Numero = 3;
			factura3.Cliente = this.ClienteSinBonificacion;
			factura3.Items.Clear();
			factura3.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura3);
			factura3.Create();
			
			Factura factura4 = TestBase.CrearFactura();
			factura4.Codigo = 2;
			factura4.Numero = 4;
			factura4.Cliente = this.ClienteSinBonificacion;
			factura4.Items.Clear();
			factura4.Items.Add(TestBase.CrearItemFactura(this.ProductoGriferia, 1));
			TestBase.ProcesarItemsYCalcularImportesFactura(factura4);
			factura4.Create();
			
			this.RemoveTransaction();
			
			this.controladorFacturacion
				.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 1, 4,
				                                this.iceCurrentIp1Puerto1);
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaA),
				                Expression.Eq("Codigo", 1),
				                Expression.Eq("Numero", 4L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_NumeroIncorrecto_MenorAlUltimo()
		{
			// Primero borro todas las NotaCredito, ya que son Facturas y estorban aquí.
			ActiveRecordValidationBase<NotaCredito>.DeleteAll();
			
			this.RemoveTransaction();
			
			try {
				// Intento ingresar una factura de sincronización con un número menor
				// al último numero de ese tipo de factura con ese código.
				this.controladorFacturacion
					.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 1, 3,
					                                this.iceCurrentIp1Puerto1);
				
				Assert.Fail("No debió haberse podido crear la factura inválida");
			}
			catch (ParametrosInvalidosException) {
			}
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_NumeroCorrecto_PrimeraIngresada()
		{
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 9, 1,
					                                this.iceCurrentIp1Puerto1);
			}
			catch(StratosException ex) {
				Assert.Fail(ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura[] resultado =
				Factura.FindAll(Expression.Eq("Tipo", FacturaTipoVenta.FacturaA),
				                Expression.Eq("Codigo", 9),
				                Expression.Eq("Numero", 1L));
			
			Assert.AreEqual(1, resultado.Length, "Cantidad de resultado");
			Assert.IsFalse(resultado[0].Anulada, "Anulada");
			Assert.IsFalse(resultado[0].Valida, "Valida");
		}
		
		[Test]
		public void IngresarFacturaDeSincronizacion_NumeroIncorrecto_PrimeraIngresada()
		{
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion
					.IngresarFacturaDeSincronizacion(FacturaTipoVenta.FacturaA, 20, 5,
					                                this.iceCurrentIp1Puerto1);
				
				Assert.Fail("No debió haberse podido crear la factura inválida");
			}
			catch (StratosException) {
			}
		}
		#endregion
		
		[Test]
		public void AnularFactura()
		{
			// TODO: Testear la eliminación de la última factura
			// TODO: Testear la eliminación de una factura que no sea la última (debe fallar).
			
			ActiveRecordBase<PreFactura>.DeleteAll();
			this.FlushAndRecreateReadOnlyScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsFalse(facturaDto.anulada, "FacturaDto no anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.AnularFactura(facturaDto, false, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al anular la factura: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura factura = Factura.Find(facturaDto.id);
			
			Assert.IsTrue(factura.Anulada, "Factura anulada");
			Assert.IsTrue(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(factura.Id, nuevaFacturaDto.id, "Nueva factura id");
			
			// No debe haber PreFacturas creadas
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(0, preFacturas.Length, "Cantidad pre facturas");
		}
		
		[Test]
		public void AnularFactura_CrearPreFacturaDesdeFacturaAnulada()
		{
			// TODO: Testear la eliminación de la última factura
			// TODO: Testear la eliminación de una factura que no sea la última (debe fallar).
			
			ActiveRecordBase<PreFactura>.DeleteAll();
			this.FlushAndRecreateScope();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsFalse(facturaDto.anulada, "FacturaDto no anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.AnularFactura(facturaDto, true, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al anular la factura: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura factura = Factura.Find(facturaDto.id);
			
			Assert.IsTrue(factura.Anulada, "Factura anulada");
			Assert.IsTrue(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(factura.Id, nuevaFacturaDto.id, "Nueva factura id");
			
			// PreFactura
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(1, preFacturas.Length, "Cantidad pre facturas");
			
			// La PreFactura debe ser igual a la Factura
			PreFactura preFactura = preFacturas[0];
			
			Assert.IsNotNull(preFactura.Cliente, "cliente no nulo");
			Assert.AreEqual(factura.Cliente.Codigo, preFactura.Cliente.Codigo, "Prefactura cliente código");
			Assert.AreEqual(factura.CondicionVenta.Condicion, preFactura.CondicionVenta.Condicion, "Prefactura condición pago");
			Assert.AreEqual(factura.CondicionVenta.Plazo, preFactura.CondicionVenta.Plazo, "Prefactura plazo");
			Assert.AreEqual(factura.Items.Count, preFactura.Items.Count, "Prefactura cantidad items");
			Assert.AreEqual(factura.Fecha, preFactura.Fecha, "Prefactura fecha");
			Assert.AreEqual(factura.ImporteBonificacion, preFactura.ImporteBonificacion, "Prefactura importe bonif");
			Assert.AreEqual(factura.ImporteBruto, preFactura.ImporteBruto, "Prefactura importe bruto");
			Assert.AreEqual(factura.ImporteFinal, preFactura.ImporteFinal, "Prefactura importe final");
			Assert.AreEqual(factura.ImporteIva, preFactura.ImporteIva, "Prefactura importe iva");
			Assert.AreEqual(factura.ImporteNeto, preFactura.ImporteNeto, "Prefactura importe neto");
			Assert.AreEqual(factura.ImporteTotal, preFactura.ImporteTotal, "Prefactura importe total");
			
			Assert.AreEqual(factura.PorcentajeBonificacion, preFactura.PorcentajeBonificacion, "Prefactura porcentaje bonif");
			Assert.AreEqual(factura.Remitos, preFactura.Remitos, "Prefactura remitos");
			
			Assert.AreEqual(factura.Tipo, preFactura.Tipo, "Prefactura tipo");
		}
		
		[Test]
		public void AnularFactura_NoLaUltimaFactura()
		{
			/* Este test verifica que no salte un problema al validar la fecha
			 * de la Factura anulada (ya que se valida que la fecha de la factura
			 * no sea menor a la última factura creada del mismo tipo y código). */
			
			// Primero hago que la primer factura tenga una fecha anterior a las demás
			this.factura1.Fecha = this.factura1.Fecha - TimeSpan.FromDays(2);
			this.factura1.UpdateAndFlush();
			
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsFalse(facturaDto.anulada, "FacturaDto no anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.AnularFactura(facturaDto, false, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al anular la factura: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			Factura factura = Factura.Find(facturaDto.id);
			
			Assert.IsTrue(factura.Anulada, "Factura anulada");
			Assert.IsTrue(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(factura.Id, nuevaFacturaDto.id, "Nueva factura id");
		}
		
		[Test]
		public void AnularFactura_FacturaHistorica()
		{
			/* Este test verifica que no salte un problema al validar la fecha
			 * de la Factura anulada (ya que se valida que la fecha de la factura
			 * no sea menor a la última factura creada del mismo tipo y código). */
			
			this.factura1.Historica = true;
			
			try {
				this.factura1.UpdateAndFlush();
			}
			catch (ActiveRecordValidationException ex) {
				Assert.Fail(this.ObtenerMensaje(ex));
			}
			
			this.FlushAndRecreateScope();
			
			this.factura1 = Factura.Find(this.factura1.Id);
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			FacturaDto nuevaFacturaDto;
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.AnularFactura(facturaDto, false, out nuevaFacturaDto);
				Assert.Fail("Se pudo anular una factura histórica");
			}
			catch (ObjetoInvalidoException ex) {
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			this.factura1 = Factura.Find(this.factura1.Id);
			
			Assert.IsTrue(this.factura1.Historica, "Factura histórica");
			Assert.IsFalse(this.factura1.Anulada, "Factura anulada");
		}
		
		[Test]
		public void AnularNotaCredito()
		{
			// TODO: Testear la eliminación de la última factura
			// TODO: Testear la eliminación de una factura que no sea la última (debe fallar).
			
			ActiveRecordBase<PreFactura>.DeleteAll();
			this.FlushAndRecreateScope();
			
			NotaCreditoDto notaCreditoDto = ConvertidorDto.ToDto(this.notaCredito1);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsFalse(notaCreditoDto.anulada, "NotaCreditoDto no anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.AnularFactura(notaCreditoDto, false, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al anular la nota de credito: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			NotaCredito notaCredito = NotaCredito.Find(notaCreditoDto.id);
			
			Assert.IsTrue(notaCredito.Anulada, "NotaCredito anulada");
			Assert.IsTrue(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(notaCredito.Id, nuevaFacturaDto.id, "Nueva factura id");
			
			// No debe haber PreFacturas creadas
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(0, preFacturas.Length, "Cantidad pre facturas");
		}
		
		[Test]
		public void AnularNotaCredito_NuncaCreaPreNotaCredito()
		{
			// TODO: Testear la eliminación de la última factura
			// TODO: Testear la eliminación de una factura que no sea la última (debe fallar).
			
			ActiveRecordBase<PreFactura>.DeleteAll();
			this.FlushAndRecreateScope();
			
			NotaCreditoDto notaCreditoDto = ConvertidorDto.ToDto(this.notaCredito1);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsFalse(notaCreditoDto.anulada, "NotaCreditoDto no anulada");
			
			this.RemoveTransaction();
			
			try {
				// Aunque marque true no debe crear la PreNotaCredito
				this.controladorFacturacion.AnularFactura(notaCreditoDto, true, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al anular la nota de credito: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			NotaCredito notaCredito = NotaCredito.Find(notaCreditoDto.id);
			
			Assert.IsTrue(notaCredito.Anulada, "NotaCredito anulada");
			Assert.IsTrue(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(notaCredito.Id, nuevaFacturaDto.id, "Nueva factura id");
			
			// No debe haber PreFacturas creadas
			PreFactura[] preFacturas = PreFactura.FindAll();
			Assert.AreEqual(0, preFacturas.Length, "Cantidad pre facturas");
		}
		
		[Test]
		public void DesanularNotaCredito()
		{
			// Primero anulo la factura
			NotaCreditoDto notaCreditoDto = ConvertidorDto.ToDto(this.notaCredito1);
			FacturaDto temp;
			this.controladorFacturacion.AnularFactura(notaCreditoDto, false, out temp);
			
			// La vuelvo a traer de la base de datos
			NotaCredito notaCredito = NotaCredito.Find(notaCreditoDto.id);
			notaCreditoDto = ConvertidorDto.ToDto(notaCredito);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsTrue(notaCreditoDto.anulada, "NotaCreditoDto anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.DesanularFactura(notaCreditoDto, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al desanular la nota de credito: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			notaCredito = NotaCredito.Find(notaCreditoDto.id);
			
			Assert.IsFalse(notaCredito.Anulada, "NotaCredito desanulada");
			Assert.IsFalse(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(notaCredito.Id, nuevaFacturaDto.id, "Nueva factura id");
		}
		
		[Test]
		public void DesanularFactura()
		{
			// Primero anulo la factura
			FacturaDto facturaDto = ConvertidorDto.ToDto(this.factura1);
			FacturaDto temp;
			this.controladorFacturacion.AnularFactura(facturaDto, false, out temp);
			
			// La vuelvo a traer de la base de datos
			Factura factura = Factura.Find(facturaDto.id);
			facturaDto = ConvertidorDto.ToDto(factura);
			FacturaDto nuevaFacturaDto = null;
			
			Assert.IsTrue(facturaDto.anulada, "FacturaDto anulada");
			
			this.RemoveTransaction();
			
			try {
				this.controladorFacturacion.DesanularFactura(facturaDto, out nuevaFacturaDto);
			}
			catch (StratosException ex) {
				Assert.Fail("Se produjo una excepción al desanular la factura: " + ex.mensaje);
			}
			
			this.FlushAndRecreateReadOnlyScope();
			
			factura = Factura.Find(facturaDto.id);
			
			Assert.IsFalse(factura.Anulada, "Factura desanulada");
			Assert.IsFalse(nuevaFacturaDto.anulada, "Nueva factura anulada");
			Assert.AreEqual(factura.Id, nuevaFacturaDto.id, "Nueva factura id");
		}
	}
}
